/* 2010-08-25: File added by Sony Corporation */
/*
 *  File Name		: linux/arch/arm/mach-mp200/cpu.c
 *  Function		: CPU Frequency control
 *  Release Version : Ver 1.00
 *  Release Date	: 2007/12/07
 *
 *  Copyright (C) NEC Electronics Corporation 2007
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/cpufreq.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/err.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/system.h>

#include <asm/arch/smu.h>

#include "cpu.h"

static unsigned int mp200_cur_speed, mp200_max_speed;
static struct cpufreq_frequency_table freq_table[3];

int mp200_verify_speed(struct cpufreq_policy *policy)
{
	return cpufreq_frequency_table_verify(policy, freq_table);
}

unsigned int mp200_getspeed(unsigned int cpu)
{
	unsigned int speed, pll1, mode;

	if (cpu) return 0;

	/* HACK HACK: the clk fwk can sleep */
	if (in_atomic() || irqs_disabled()) {
		return mp200_cur_speed;
	}

	pll1 = (inl(SMU_PLL1CTRL0) + 1) * 125 * 32768;
	speed = pll1/ 1000000;

	mode = (inl(SMU_CLK_MODE_SEL) & 0x00000F00) >> 8;
	switch (mode) {
 	case NORMAL_B: speed /= NORMAL_B_DIV; break;
	}

	return speed;
}

static int mp200_target(struct cpufreq_policy *policy,
		       unsigned int target_freq,
		       unsigned int relation)
{
	struct cpufreq_freqs freqs;
	int ret = 0, normal_div;

	/* Ensure desired rate is within the table range */
	if (target_freq >= policy->cpuinfo.max_freq) {
		target_freq = policy->cpuinfo.max_freq;
		normal_div = NORMAL_A;
	} else {
		target_freq = freq_table[1].frequency;
		normal_div = NORMAL_B;
	}

	freqs.old = mp200_getspeed(0);
	freqs.new = target_freq;
	freqs.cpu = 0;

	if (freqs.old == freqs.new)
		return ret;

	cpufreq_notify_transition(&freqs, CPUFREQ_PRECHANGE);

	mp200_change_normalx(normal_div);
	mp200_cur_speed = target_freq;

	cpufreq_notify_transition(&freqs, CPUFREQ_POSTCHANGE);

#ifdef CONFIG_CPU_FREQ_DEBUG
	printk("cpufreq-mp200: transition: %u --> %u (lpj=%d)\n ",
			freqs.old, freqs.new, loops_per_jiffy);
#endif

	return ret;
}

static int __init mp200_cpu_init(struct cpufreq_policy *policy)
{
	int result = 0;

	if (policy->cpu != 0)
		return -EINVAL;

	policy->governor = CPUFREQ_DEFAULT_GOVERNOR;
	policy->cur = mp200_cur_speed = mp200_max_speed = mp200_getspeed(0);;

	outl(0x00244222, SMU_NORMALB_DIV);

	freq_table[0].index = 0;
	freq_table[0].frequency = mp200_cur_speed;
	freq_table[1].index = 1;
	freq_table[1].frequency = mp200_cur_speed / NORMAL_B_DIV;
	freq_table[2].index = 2;
	freq_table[2].frequency = CPUFREQ_TABLE_END;

	result = cpufreq_frequency_table_cpuinfo(policy, freq_table);
	if (!result)
		cpufreq_frequency_table_get_attr(freq_table, policy->cpu);

	/* If CPUFREQ_ETERNAL is used, then governers cannot be changed */
	policy->cpuinfo.transition_latency = 10 * 1000 * 1000; /* 10 msec */

	return result;
}

static struct freq_attr *mp200_cpufreq_attr[] = {
	&cpufreq_freq_attr_scaling_available_freqs,
	NULL,
};

static struct cpufreq_driver mp200_driver = {
	.flags		= CPUFREQ_STICKY,
	.verify		= mp200_verify_speed,
	.target		= mp200_target,
	.get		= mp200_getspeed,
	.init		= mp200_cpu_init,
	.name		= "mp200",
	.attr		= mp200_cpufreq_attr,
};

static int __init mp200_cpufreq_init(void)
{
	return cpufreq_register_driver(&mp200_driver);
}

arch_initcall(mp200_cpufreq_init);

