/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name		: linux/arch/arm/mach-mp200/dma.c
 *  Function		: dmac
 *  Release Version 	: Ver 1.01
 *  Release Date	: 2009/02/19
 *
 *  Copyright (C) NEC Electronics Corporation 2009
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <asm/irq.h>
#include <asm/dma.h>

#include <linux/platform_device.h>
#include <linux/delay.h>
#include <asm/arch/pmu.h>

#include "dma.h"

/****  Prototype definition  ****/

static int setup_M2M(dma_regs_t *regs, dma_addr_t src_ptr, u_int size, dma_addr_t dst_ptr, u_int32_t mode);
static int setup_M2P(dma_regs_t *regs, dma_addr_t src_ptr, u_int size, dma_addr_t dst_ptr, u_int32_t mode);
static int setup_P2M(dma_regs_t *regs, dma_addr_t src_ptr, u_int size, dma_addr_t dst_ptr, u_int32_t mode);

/****  Global variables (within this file)  ****/
static int dma_tclk_users = 0;	/* Number of TCLK users. */

/****  Structured data definition  ****/

/* for physical channel #0 */
mp200_dmal_t dma_pch0_lch[MP200_DMAC_P0_MAX_L_CHANNELS] = {
	{
		.name    = "ARM M->M0",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "ARM M->M1",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "ARM M->M2",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "ARM M->M3",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
};

/* for physical channel #2 */
mp200_dmal_t dma_pch2_lch[MP200_DMAC_P2_MAX_L_CHANNELS] = {
	{
		.name    = "M->P0",
		.defmode = MP200_DMAC_DEFMODE_8BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "M->P1",
		.defmode = MP200_DMAC_DEFMODE_8BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "M->P2",
		.defmode = MP200_DMAC_DEFMODE_8BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "M->P3",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "M->P4",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = 0,
	},
	{
		.name    = "M->P5",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = 0,
	},
	{ /* dummy */
		.name    = "M->P6",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P7",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = SPEC_TIMEOUT,
	},
	{ /* dummy */
		.name    = "M->P8",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P9",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P10",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P11",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P12",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P13",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "M->P14",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
};
/* for physical channel #3 */
mp200_dmal_t dma_pch3_lch[MP200_DMAC_P3_MAX_L_CHANNELS] = {
	{
		.name    = "P->M0",
		.defmode = MP200_DMAC_DEFMODE_8BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "P->M1",
		.defmode = MP200_DMAC_DEFMODE_8BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "P->M2",
		.defmode = MP200_DMAC_DEFMODE_8BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "P->M3",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = SPEC_TIMEOUT,
	},
	{
		.name    = "P->M4",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = 0,
	},
	{
		.name    = "P->M5",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = 0,
	},
	{ /* dummy */
		.name    = "P->M6",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = 0,
	},
	{
		.name    = "P->M7",
		.defmode = MP200_DMAC_DEFMODE_16BIT,
		.special = SPEC_TIMEOUT,
	},
	{ /* dummy */
		.name    = "M->P8",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "P->M9",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "P->M10",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "P->M11",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "P->M12",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "P->M13",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
	{
		.name    = "P->M14",
		.defmode = MP200_DMAC_DEFMODE_32BIT,
		.special = 0,
	},
};

/* for every physical channels (4ch) */
mp200_dmap_t dma_pch[MP200_DMAC_MAX_P_CHANNELS] = {
	{ /* PCH #0 */
		.control   = (cntsts_t *)IO_ADDRESS(MP200_DMAC_BASE),
		.intstat   = (intparm_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x0100),
		.intsel    = (intdest_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x0800),
		.parbase   = (dma_regs_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x1000),
		.lch       = dma_pch0_lch,
		.lchno     = MP200_DMAC_P0_MAX_L_CHANNELS,
		.not_int   = ~(MP200_DMAC_INT_LENG_RD | MP200_DMAC_INT_BLOCK_RD),
		.setup_dma = setup_M2M,
	},
	{ /* PCH #1 dummy */
		.lchno     = MP200_DMAC_P1_MAX_L_CHANNELS,
	},
	{ /* PCH #2 */
		.control   = (cntsts_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x4000),
		.intstat   = (intparm_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x4100),
		.intsel    = (intdest_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x4800),
		.parbase   = (dma_regs_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x5000),
		.lch       = dma_pch2_lch,
		.lchno     = MP200_DMAC_P2_MAX_L_CHANNELS,
		.not_int   = ~(MP200_DMAC_INT_LENG_RD),
		.setup_dma = setup_M2P,
	},
	{ /* PCH #3 */
		.control   = (cntsts_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x6000),
		.intstat   = (intparm_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x6100),
		.intsel    = (intdest_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x6800),
		.parbase   = (dma_regs_t *)IO_ADDRESS(MP200_DMAC_BASE + 0x7000),
		.lch       = dma_pch3_lch,
		.lchno     = MP200_DMAC_P3_MAX_L_CHANNELS,
		.not_int   = ~(MP200_DMAC_INT_LENG_RD),
		.setup_dma = setup_P2M,
	},
};


/*
 * Check channel number
 */
static inline int check_validity_channel(int channel)
{
	unsigned int pchno;
	unsigned int lchno;

	pchno = MP200_DMAC_PCHNO(channel);
	lchno = MP200_DMAC_LCHNO(channel);

	if ((channel & ~(MP200_DMAC_MASK_CHNO | MP200_DMAC_MASK_OWNER)) == 0) {
		if ( ((pchno == 0) && (lchno < MP200_DMAC_P0_MAX_L_CHANNELS))
		  || ((pchno == 2) && (lchno < MP200_DMAC_P2_MAX_L_CHANNELS))
		  || ((pchno == 3) && (lchno < MP200_DMAC_P3_MAX_L_CHANNELS)) ) {
			return 0;
		}
	}

	return 1;
}

/*
 * Functions for default settings.
 *
 * Each function sets DMA up to appropriate default setting when
 * `size' is not 0. Skip parameter settings except source and destination
 * addresses when `size' is 0 so that client driver can use special mode
 * like repeat transfer, timeout, etc.
 */

/* PCH #0 */
static int setup_M2M(dma_regs_t *regs, dma_addr_t src_ptr, u_int size,
		dma_addr_t dst_ptr, u_int32_t mode)
{
	u_int blocksize, offset;

	if (size == 0) {	/* short cut for special transfer mode */
		regs->aadd = src_ptr;
		regs->badd = dst_ptr;
		return 0;
	}

	/*
	 * size != 0
	 * Handle just two contiguous memory region, source and destination.
	 * Simple transfer, but these may be overlapped.
	 */
	if ((size & ~MP200_DMAC_MASK_LENGTH) != 0) {
		return 1;
	}

	/*
	 * Determine which mode to use.
	 */
	offset = 0 | MP200_DMAC_OFFSET_ADD; /* default is forward transfer */

	if (dst_ptr <= src_ptr && src_ptr < dst_ptr + size) { /* XXX ignore wrap-around case */
		if (src_ptr < dst_ptr + MP200_DMAC_SIZE_BLKSIZES) {
			/* Difference between src_ptr and dst_ptr must be
			   greater than MP200_DMAC_SIZE_BLKSIZES. */
			return 1;
		}
		/*
		 * overlap, copy forward
		 * It may not need to use multi-block transfer.
		 */
		if (src_ptr < dst_ptr + MP200_DMAC_SIZE_BLKSIZEL) {
			blocksize = MP200_DMAC_SIZE_BLKSIZES;
		} else {
			blocksize = MP200_DMAC_SIZE_BLKSIZEL;
		}
	} else if (src_ptr < dst_ptr && dst_ptr < src_ptr + size) { /* XXX ignore wrap-around case */
		u_int sharesize, isolatesize;

		isolatesize = dst_ptr - src_ptr;
		if (isolatesize < MP200_DMAC_SIZE_BLKSIZES) {
			/* Difference between src_ptr and dst_ptr must be
			   greater than MP200_DMAC_SIZE_BLKSIZES. */
			return 1;
		}
		sharesize = size - isolatesize;
		/*
		 * overlap, copy backward
		 */
		if (size % isolatesize == 0) {
			/*
			 * It's most likely that backward copy is used for video
			 * console back scroll.
			 * In that case, offset between source and destination is equal
			 * to one line, and size, most likely, can be divided by it.
			 * (Assume one line, does not fit for scrolling multiple lines.)
			 */
			 blocksize = isolatesize;
		} else {
			/*
			 * Copy with block size MP200_DMAC_SIZE_BLKSIZEMIN.
			 * This will cause inefficiency around bus traffic.
			 * It may be better that this case does not exist and
			 * handle as error.
			 */
			blocksize = MP200_DMAC_SIZE_BLKSIZEMIN;
		}
		src_ptr += size - blocksize;
		dst_ptr += size - blocksize;
		offset = blocksize * 2 | MP200_DMAC_OFFSET_SUB;
	} else if ((size & ~MP200_DMAC_MASK_BLKSIZE) == 0) {
		/*
		 * does not overlap, small enough to use single block transfer
		 */
		blocksize = size;
	} else {
		/*
		 * does not overlap, greater than 65534, use multiple block transfer
		 */
		blocksize = MP200_DMAC_SIZE_BLKSIZEL;
	}
	/*
	 * setup DMAC registers
	 */
	regs->aadd = src_ptr;
	regs->badd = dst_ptr;
	regs->aoff = offset;
	regs->boff = offset;
	regs->asize_count = 0;
	regs->bsize_count = 0;
	regs->leng = size;
	regs->size = blocksize;
	regs->mode = mode;

	return 0;
}

/* PCH #2 */
static int setup_M2P(dma_regs_t *regs, dma_addr_t src_ptr, u_int size,
		   dma_addr_t dst_ptr, u_int32_t mode)
{
	u_int blocksize;

	if (size == 0) {	/* short cut for special transfer mode */
		regs->aadd = src_ptr;
		regs->badd = dst_ptr;
		return 0;
	}

	if ((size & ~MP200_DMAC_MASK_LENGTH) != 0) {
		return 1;
	}
	if ((size & ~MP200_DMAC_MASK_BLKSIZE) == 0) {
		blocksize = size;			/* use single block transfer */
	} else {
		blocksize = MP200_DMAC_SIZE_BLKSIZEL;	/* use multiple block transfer */
	}
	regs->aadd = src_ptr;
	regs->badd = dst_ptr;
	regs->aoff = 0;
	regs->asize_count = 0;
	regs->asize = blocksize;
	regs->leng = size;
	regs->mode = mode;

	return 0;
}

/* PCH #3 */
static int setup_P2M(dma_regs_t *regs, dma_addr_t src_ptr, u_int size,
		   dma_addr_t dst_ptr, u_int32_t mode)
{
	u_int blocksize;

	if (size == 0) {	/* short cut for special transfer mode */
		regs->aadd = src_ptr;
		regs->badd = dst_ptr;
		return 0;
	}

	if ((size & ~MP200_DMAC_MASK_LENGTH) != 0) {
		return 1;
	}
	if ((size & ~MP200_DMAC_MASK_BLKSIZE) == 0) {
		blocksize = size;			/* use single block transfer */
	} else {
		blocksize = MP200_DMAC_SIZE_BLKSIZEL;	/* use multiple block transfer */
	}
	regs->aadd = src_ptr;
	regs->badd = dst_ptr;
	regs->boff = 0;
	regs->bsize_count = 0;
	regs->bsize = blocksize;
	regs->leng = size;
	regs->mode = mode;

	return 0;
}

/*
 * Interrupt Handler
 */
static irqreturn_t dma_irq_handler(int irq, void *dev_id)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t   *regs = dev_id;
	int           chno;
	int           stshift;
	intparm_t    *intcnt;
	unsigned int  sts;
	unsigned int  rawsts;
	int           err;

	/*
	 * We can compute channel number from address of control registers
	 */
	chno = MP200_DMAC_PARBASE2CHNO(regs);
	err = check_validity_channel(chno);
	if (err != 0) {
		return IRQ_NONE;
	}
	pch = &dma_pch[MP200_DMAC_PCHNO(chno)];

	/*
	 * Check interrupt status.	If it's not for this channel then return.
	 */
	intcnt = pch->intstat + MP200_DMAC_LCHNO(chno) / 4;

	stshift = (MP200_DMAC_LCHNO(chno) % 4) * 8;
	sts = (intcnt->stat >> stshift) & 0xff;
	if ((sts & pch->not_int) == 0) { /* Some bits are not cause of interrupt. */
		return IRQ_NONE; /* there is no interrupt */
	}

	rawsts = (intcnt->raw_stat >> stshift) & 0xff;

	/* clear interrupt status */
	intcnt->clear = sts << stshift;

	/* call client driver's interrupt handling routine */
	lch = pch->lch + MP200_DMAC_LCHNO(chno);
	lch->callback(lch->data, sts, rawsts);

	return IRQ_HANDLED;
}


/****  External functions  ****/

/*
 * Acquire DMA channel
 *   mp200_request_dma()
 */
int mp200_request_dma(int channel, const char *device_id,
				 dma_callback_t callback, void *data, dma_regs_t **dma_regs)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t   *regs;
	unsigned int  intdest;
	int           err;
	int           owner;

	err = check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_NO_USE) {
		return -EBUSY;
	}
	regs = pch->parbase + MP200_DMAC_LCHNO(channel);
	owner = MP200_DMAC_OWNER(channel);

	/*
	 * Stop desired DMA channel to make it sure before
	 * interrupt related settings.
	 */
#if 0 /* Because of errata, can't stop DMA here. */
	mp200_clear_dma(channel);
#endif

	if (owner == MP200_DMAC_OWNER(MP200_DMAC_OWNER_ACPU)) {
		err = request_irq(INT_DMAPE0, dma_irq_handler, (IRQF_SHARED | IRQF_DISABLED), device_id, regs);
		if (err != 0) {
			printk(KERN_INFO "%s(): unable to request IRQ %d for DMA channel (%s)\n",
			  __FUNCTION__, INT_DMAPE0, device_id);
			return err;
		}
	}

	/*
	 * Set interrupt output line to the PE which executing this routine.
	 * XXX needs mutual exclusion between PE's
	 */
	intdest = pch->intsel->intdest;
	intdest &= ~(3 << (MP200_DMAC_LCHNO(channel) * 2));
	intdest |= owner << (MP200_DMAC_LCHNO(channel) * 2);
	pch->intsel->intdest = intdest;

	/*
	 * Supply TCLK if needed.
	 */
	if (lch->special & SPEC_TIMEOUT) {
		if (dma_tclk_users == 0)
			mp200_pmu_open_clockgate(MP200_CLOCKGATE_DMA_TCLK);
		dma_tclk_users++;
	}

	/*
	 * RESET does not initialize some DMAC registers.
	 * Set DMA channel to known state (for single block transfer) here
	 * so that stabilizing phenomena caused by client code
	 * lacks register setting (bug).
	 * Use cold start address where most likely boot ROM exists.
	 */
	(void)pch->setup_dma(regs, 0x0, 0x1000, 0x0, lch->defmode);

	/*
	 * Hold information for further DMAC operations.
	 */
	lch->in_use = DMA_USE;
	lch->owner = owner;
	lch->callback = callback;
	lch->data = data;

	/*
	 * return pointer to control registers.
	 */
	if (dma_regs != NULL) {
		*dma_regs = regs;
	}

	return 0;
}

/*
 * Release DMA channel
 */
void mp200_free_dma(int channel)
{
	mp200_dmap_t  *pch;
	mp200_dmal_t  *lch;
	dma_regs_t    *regs;
	int            err;

	err = check_validity_channel(channel);
	if (err != 0) {
		return;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return;
	}
	if (lch->owner != MP200_DMAC_OWNER(channel)) {
		return;
	}

	/*
	 * Stop DMA
	 */
	mp200_clear_dma(channel);

	/*
	 * Stop TCLK if possible.
	 */
	if (lch->special & SPEC_TIMEOUT) {
		dma_tclk_users--;
		if (dma_tclk_users == 0)
			mp200_pmu_close_clockgate(MP200_CLOCKGATE_DMA_TCLK);
	}

	/*
	 * Release interrupt
	 */
	if (lch->owner == MP200_DMAC_OWNER(MP200_DMAC_OWNER_ACPU)) {
		regs = pch->parbase + MP200_DMAC_LCHNO(channel);
		free_irq(INT_DMAPE0, regs);
	}

	lch->in_use = DMA_NO_USE;
	lch->callback = NULL;
	lch->data = NULL;

	return;
}

/*
 * Setup and enable DMA.
 */
int mp200_start_dma(int channel, dma_addr_t src_ptr, u_int size,
			   dma_addr_t dst_ptr, int intmask)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t   *regs;
	int           stshift;
	intparm_t    *intcnt;
	cntsts_t     *base;
	int           err;

	err = check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}
	if ((intmask & ~MP200_DMAC_MASK_INT) != 0) {
		return -EINVAL;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return -ENXIO;
	}
	if (lch->owner != MP200_DMAC_OWNER(channel)) {
		return -ENODEV;
	}
	if ((intmask != 0) && (lch->callback == NULL)) {
		return -EINVAL;
	}

	/*
	 * Set DMA channel up to appropriate default setting.
	 */
	regs = pch->parbase + MP200_DMAC_LCHNO(channel);
	err = pch->setup_dma(regs, src_ptr, size, dst_ptr, lch->defmode);
	if (err != 0) {
		return -EINVAL;
	}

	/*
	 * When size > 65534 or overlapped source and destination, we need to
	 * use multiple block transfer.
	 * But we don't need block interrupt, so mask it when size != 0.
	 */
	if (size != 0) {
		intmask &= ~MP200_DMAC_INT_BLOCK_EN;
	}
	/*
	 * Enable interrupt.
	 * At first, disable all interrupt, then enable desired interrupt.
	 */
	intcnt = pch->intstat + MP200_DMAC_LCHNO(channel) / 4;

	stshift = (MP200_DMAC_LCHNO(channel) % 4) * 8;
	intcnt->disable = 0xff << stshift;
	intcnt->clear = 0xff << stshift;
	intcnt->enable = (unsigned char)intmask << stshift;

	/*
	 * Enable DMA.
	 */
	base = pch->control;
	base->start = 1 << MP200_DMAC_LCHNO(channel);

	return 0;
}

/*
 * Acquire DMA channel status.
 */
int mp200_dma_status(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	int           stat;
	cntsts_t     *base;
	int           err;

	err = check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	/* does not check `in_use'. can read status of any DMA channel */

	/*
	 * Read status register, return corresponding bit.
	 */
	base = pch->control;
	stat = (base->status >> MP200_DMAC_LCHNO(channel)) & 1;

	return stat;
}

/*
 * Acquire DMA channel status (register writable flag).
 */
int mp200_dma_writable_flag(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	int           stat;
	cntsts_t     *base;
	int           err;

	err = check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}
	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	/* does not check `in_use'. can read status of any DMA channel */

	/*
	 * Read status register, return corresponding bit.
	 */
	base = pch->control;
	stat = (base->status >> (16 + MP200_DMAC_LCHNO(channel))) & 1;

	return stat;
}

/*
 * Acquire all DMA channel status.
 * check all DMA channels,
 * return 1 if there is enabled channel.
 * Don't start DMA channel while calling this function.
 */
int mp200_dma_busy(void)
{
	int p;
	int l;
	cntsts_t     *base;
	mp200_dmal_t *lch;

	for (p = 0 ; p < MP200_DMAC_MAX_P_CHANNELS ; p++) {
		for (l = 0 ; l < dma_pch[p].lchno ; l++) {
		   lch = dma_pch[p].lch + l;
		   if (lch->in_use == DMA_USE) {
				base = dma_pch[p].control;
				if (((base->status >> l) & 1) != 0) {
				   return 1;
				}
		   }
		}
	}

	return 0;
}

/*
 * Stop DMA channel.
 */
void mp200_clear_dma(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	cntsts_t     *base;
	intparm_t    *intcnt;
	int stshift;
	int err;
	int stat;

	err = check_validity_channel(channel);
	if (err != 0) {
		return;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return;
	}
	if (lch->owner != MP200_DMAC_OWNER(channel)) {
		return;
	}

	/*
	 * disable all interrupt
	 */
	intcnt = pch->intstat + MP200_DMAC_LCHNO(channel) / 4;

	stshift = (MP200_DMAC_LCHNO(channel) % 4) * 8;
	intcnt->disable = 0xff << stshift;
	intcnt->clear = 0xff << stshift;

	/*
	 * stop DMA
	 */
	base = pch->control;
	base->stop = 1 << MP200_DMAC_LCHNO(channel);
	/* Until DMA stops, polling */
	for(stat = 1;stat != 0;) {
		stat = (base->status >> MP200_DMAC_LCHNO(channel)) & 1 ;
	}
}

/*
 * Stop DMA channel too.
 */
void mp200_reset_dma(int channel)
{
	mp200_clear_dma(channel);
}

/*
 * Retrieve current DMA position.
 */
dma_addr_t mp200_get_dma_pos(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t   *regs;
	int err;

	err = check_validity_channel(channel);
	if (err != 0) {
		return 0;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return 0;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	/* does not check `in_use'. can read position of any DMA channel */
	regs = pch->parbase + MP200_DMAC_LCHNO(channel);

	switch (MP200_DMAC_PCHNO(channel)) {
	case 0:
	case 1:
	case 2:
		return regs->aadp;
		break;
	case 3:
		return regs->badp;
		break;
	default:
		break;
	}

	return 0;
}

/*
 * Stop DMA.
 * We'd like to suspend DMA instead of stopping, if we can.
 * Unfortunately, DMAC on MP200 can't suspend, we stop DMA at this moment.
 * (can't resume)
 */
void mp200_stop_dma(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	cntsts_t     *base;
	intparm_t    *intcnt;
	int stshift;
	int err;
	int stat;

	err = check_validity_channel(channel);
	if (err != 0) {
		return;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return;
	}
	if (lch->owner != MP200_DMAC_OWNER(channel)) {
		return;
	}

	/*
	 * disable all interrupt
	 */
	intcnt = pch->intstat + MP200_DMAC_LCHNO(channel) / 4;

	stshift = (MP200_DMAC_LCHNO(channel) % 4) * 8;
	intcnt->disable = 0xff << stshift;
	intcnt->clear = 0xff << stshift;

	/*
	 * stop DMA
	 */
	base = pch->control;
	base->stop = 1 << MP200_DMAC_LCHNO(channel);
	/* Until DMA stops, polling */
	for(stat = 1;stat != 0;) {
		stat = (base->status >> MP200_DMAC_LCHNO(channel)) & 1 ;
	}
}

static int dma_suspend(struct platform_device *dev, pm_message_t state)
{
	switch (state.event) {
	case DEV_SUSPEND_IDLE_1:
	case PM_EVENT_SUSPEND:
		if(mp200_dma_busy())
			return -EBUSY;
		break;
	default:
		break;
	}
	return 0;
}

static int dma_resume(struct platform_device *dev)
{
	return 0;
}

static int dma_probe(struct platform_device *dev)
{
	int i;

	/* open all clock gate for DMAC (except TCLK) */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_DMA_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_DMA_PCH0_CLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_DMA_PCH2_CLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_DMA_PCH3_CLK);
	/* disable automatic clock supply control (for reset). */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_DMAPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_DMAPCH0);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_DMAPCH2);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_DMAPCH3);
	/* release reset */
	mp200_pmu_unreset_device(MP200_RESETDEVICE_DMA);
	/* mp200_pmu_unreset_device() waits apropriate period.
	   Now, we can enable automatic clock control. */
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_DMAPCLK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_DMAPCH0);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_DMAPCH2);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_DMAPCH3);

	for (i = 0 ; i < MP200_DMAC_P0_MAX_L_CHANNELS ; i++) {
		dma_pch0_lch[i].callback = NULL;
		dma_pch0_lch[i].data = NULL;
		dma_pch0_lch[i].in_use = DMA_NO_USE;
	}
	for (i = 0 ; i < MP200_DMAC_P2_MAX_L_CHANNELS ; i++) {
		dma_pch2_lch[i].callback = NULL;
		dma_pch2_lch[i].data = NULL;
		dma_pch2_lch[i].in_use = DMA_NO_USE;
	}
	for (i = 0 ; i < MP200_DMAC_P3_MAX_L_CHANNELS ; i++) {
		dma_pch3_lch[i].callback = NULL;
		dma_pch3_lch[i].data = NULL;
		dma_pch3_lch[i].in_use = DMA_NO_USE;
	}

	return 0;
}

static struct platform_driver dma_driver = {
	.probe   = dma_probe,
	.suspend = dma_suspend,
	.resume  = dma_resume,
	.driver  = {
		.name  = "dma",
		.owner = THIS_MODULE,
	},
};


/*
 * Initialize
 */
static int __init mp200_dma_init(void)
{
	return platform_driver_register(&dma_driver);
}

arch_initcall(mp200_dma_init);

EXPORT_SYMBOL(mp200_request_dma);
EXPORT_SYMBOL(mp200_free_dma);
EXPORT_SYMBOL(mp200_start_dma);
EXPORT_SYMBOL(mp200_dma_status);
EXPORT_SYMBOL(mp200_dma_busy);
EXPORT_SYMBOL(mp200_clear_dma);
EXPORT_SYMBOL(mp200_reset_dma);
EXPORT_SYMBOL(mp200_get_dma_pos);
EXPORT_SYMBOL(mp200_stop_dma);
EXPORT_SYMBOL(mp200_dma_writable_flag);
