/* 2010-08-25: File added by Sony Corporation */
/*
 *  File Name	    : linux/arch/arm/mach-mp200/dma.h
 *  Function	    : dmac
 *  Release Version : Ver 1.00
 *  Release Date    : 2006/07/11
 *
 *  Copyright (C) NEC Electronics Corporation 2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __ARCH_ARM_MACH_MP200_DMA_H
#define __ARCH_ARM_MACH_MP200_DMA_H


/****  Macro definition  ****/

#define MP200_DMAC_MASK_CHNO		0x000000ef
#define	MP200_DMAC_MASK_OWNER		0x03000000
#define MP200_DMAC_MASK_BLKSIZE 	0x0000ffff
#define MP200_DMAC_MASK_LENGTH		0x00ffffff
#define MP200_DMAC_MASK_INT		( MP200_DMAC_INT_TIME_EN \
					| MP200_DMAC_INT_ERROR_EN \
					| MP200_DMAC_INT_BLOCK_EN \
					| MP200_DMAC_INT_LENG_EN )
#define	MP200_DMAC_SIZE_BLKSIZEL	0x00001000
#define	MP200_DMAC_SIZE_BLKSIZES	0x00000080
#define	MP200_DMAC_SIZE_BLKSIZEMIN	0x00000001


/****  Macro function  ****/

#define MP200_DMAC_PCHNO(x)        (((x) & 0xe0) >> 5)
#define MP200_DMAC_LCHNO(x)        ((x) & 0x0f)
#define MP200_DMAC_PARBASE2CHNO(x) ((((long)(x)) >> 8) & MP200_DMAC_MASK_CHNO)
#define MP200_DMAC_OWNER(x)        (((x) & MP200_DMAC_MASK_OWNER) >> 24)


/****  Structure definition  ****/

/*
 * MP200 DMAC register arrangement
 */

/* control registers */
typedef struct {
    volatile u_int32_t	start;	/* +0x00 DMA start control */
    volatile u_int32_t	status; /* +0x04 DMA status */
    volatile u_int32_t	stop;	/* +0x08 DMA stop control */
} cntsts_t;

/*
 * interrupt related parameter registers
 */
typedef struct {
    volatile u_int32_t	stat;		/* +0x00 interrupt status */
    volatile u_int32_t	raw_stat;	/* +0x04 interrupt raw status */
    volatile u_int32_t	enable; 	/* +0x08 interrupt enable */
    volatile u_int32_t	disable;	/* +0x0c interrupt disable */
    volatile u_int32_t	clear;		/* +0x10 interrupt clear */
    volatile u_int32_t	rsv[3];		/* +0x14,0x18,0x1c Reserved */
} intparm_t;

/*
 * interrupt destination setting register
 */
typedef struct {
    volatile u_int32_t	intdest;	/* +0x00 interrupt destination */
} intdest_t;

/*
 * Data for managing logical DMA channels
 */
typedef struct {
    const char      *name;		/* irq name of L-channel. */
    dma_callback_t  callback;	/* call-back function from interrupt. */
    void            *data;		/* argument of call-back function. */
    u_int32_t       defmode;	/* default transfer mode. */
    u_int8_t        special; 	/* Special ability for this LCH. */
#define	SPEC_TIMEOUT	1	/* This LCH has timeout function. */
    u_int8_t        in_use; 	/* DMA in use flag. */
    u_int8_t        owner; 	/* Holder of this LCH. (ACPU/CCPU/DSP) */
} mp200_dmal_t;

/* for `in_use' setting */
#define DMA_NO_USE	0
#define DMA_USE 	1

/*
 * Data for managing physical DMA channels
 */
typedef struct {
    cntsts_t     *control;	/* control/status regs. */
    intparm_t    *intstat;	/* interrupt control regs. */
    intdest_t    *intsel;	/* interrupt destination setting reg. */
    dma_regs_t   *parbase;	/* base of parameter registers array. */
    mp200_dmal_t *lch;		/* control structures for logical CH. */
    u_int32_t    lchno;		/* No. of logical channel. */
    u_int32_t    not_int;	/* Interrupt status register contains
				   some bits that are not cause of
				   interrupt, just indicate status.
				   We must mask out them. How stupid! */
    int          (*setup_dma)(dma_regs_t *, dma_addr_t, u_int, dma_addr_t, u_int32_t);
} mp200_dmap_t;

#endif /* __ARCH_ARM_MACH_MP200_DMA_H */
