/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name       : linux/arch/arm/mach-mp200/generic.c
 *  Function        : generic
 *  Release Version : Ver 1.30
 *  Release Date    : 2009/03/18
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2009
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/serial_8250.h>
#include <linux/serial_reg.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/mach/map.h>
#include <asm/hardware/cache-l2x0.h>

#include <asm/arch/uart.h>
#include <asm/arch/hardware.h>

#include <asm/arch/pmu.h>
#include <asm/arch/smu.h>

#include "generic.h"

/*
 * Common MP200 I/O Mapping
 *
 * Logical      Physical
 * e4000000	40000000	APB0
 * e5000000	50000000	ASYNC1
 * e6000000	60000000	AHB Slave
 * e8000000	80000000	Local
 * ec000000	c0000000	APB1
 */

static struct map_desc standard_io_desc[] __initdata = {
	/* BANK 2 */
	MP200_MAP_DESC(MP200_AB0_BASE, MP200_AB0_SIZE),

	/* BANK 4 */
	MP200_MAP_DESC(MP200_ROTATOR_BASE,   MP200_ROTATOR_SIZE),
	MP200_MAP_DESC(MP200_DMAC_BASE,      MP200_DMAC_SIZE),
	MP200_MAP_DESC(MP200_IMAGEPROC_BASE, MP200_IMAGEPROC_SIZE),
	MP200_MAP_DESC(MP200_CAMERA_BASE,    MP200_CAMERA_SIZE),
	MP200_MAP_DESC(MP200_PCM0_BASE,      MP200_PCM0_SIZE),
	MP200_MAP_DESC(MP200_PCM1_BASE,      MP200_PCM1_SIZE),
	MP200_MAP_DESC(MP200_PWM_BASE,       MP200_PWM_SIZE),
	MP200_MAP_DESC(MP200_SP2_BASE,       MP200_SP2_SIZE),
	MP200_MAP_DESC(MP200_DTV_BASE,       MP200_DTV_SIZE),
	MP200_MAP_DESC(MP200_DBG_BASE,       MP200_DBG_SIZE),
	MP200_MAP_DESC(MP200_NTSC_BASE,      MP200_NTSC_SIZE),
	MP200_MAP_DESC(MP200_NAND_BASE,      MP200_NAND_SIZE),
	MP200_MAP_DESC(MP200_IPUDMA_BASE,    MP200_IPUDMA_SIZE),
	MP200_MAP_DESC(MP200_IMC_BASE,       MP200_IMC_SIZE),
	MP200_MAP_DESC(MP200_LCD_BASE,       MP200_LCD_SIZE),

	/* BANK5 */
	MP200_MAP_DESC(MP200_UART0_BASE,  MP200_UART0_SIZE),
	MP200_MAP_DESC(MP200_UART1_BASE,  MP200_UART1_SIZE),
	MP200_MAP_DESC(MP200_UART2_BASE,  MP200_UART2_SIZE),
	MP200_MAP_DESC(MP200_IIC2_BASE,   MP200_IIC2_SIZE),
	MP200_MAP_DESC(MP200_IIC_BASE,    MP200_IIC_SIZE),
	MP200_MAP_DESC(MP200_SDIA_BASE,   MP200_SDIA_SIZE),
	MP200_MAP_DESC(MP200_SDIB_BASE,   MP200_SDIB_SIZE),
	MP200_MAP_DESC(MP200_SDIC_BASE,   MP200_SDIC_SIZE),

	/* BANK 6 */
	MP200_MAP_DESC(MP200_USB_BASE, MP200_USB_SIZE),
	MP200_MAP_DESC(MP200_AVC_BASE, MP200_AVC_SIZE),
	MP200_MAP_DESC(MP200_MSP_BASE, MP200_MSP_SIZE),
	MP200_MAP_DESC(MP200_MMM_BASE, MP200_MMM_SIZE),

	/* BANK 8 */
	MP200_MAP_DESC(MP200_L220_BASE, MP200_L220_SIZE),

	/* BANK 10 */
	MP200_MAP_DESC(MP200_SSRAM_BASE, MP200_SSRAM_SIZE),

	/* BANK 12 */
	MP200_MAP_DESC(MP200_TIMER_BASE,   MP200_TIMER_SIZE),
	MP200_MAP_DESC(MP200_PCM0_BASE,    MP200_PCM0_SIZE),
	MP200_MAP_DESC(MP200_INTC_BASE,    MP200_INTC_SIZE),
	MP200_MAP_DESC(MP200_GPIO_BASE,    MP200_GPIO_SIZE),
	MP200_MAP_DESC(MP200_PDMA_BASE,    MP200_PDMA_SIZE),
	MP200_MAP_DESC(MP200_MEMC_BASE,    MP200_MEMC_SIZE),
	MP200_MAP_DESC(MP200_DCV_BASE,       MP200_DCV_SIZE),

	MP200_MAP_DESC(MP200_PMU_BASE,     MP200_PMU_SIZE),
	MP200_MAP_DESC(MP200_SMU_BASE,     MP200_SMU_SIZE),
	MP200_MAP_DESC(MP200_SP0_BASE,     MP200_SP0_SIZE),
	MP200_MAP_DESC(MP200_SP1_BASE,     MP200_SP1_SIZE),
	MP200_MAP_DESC(MP200_CHG_BASE,     MP200_CHG_SIZE),
	MP200_MAP_DESC(MP200_MWI_BASE,     MP200_MWI_SIZE),
	MP200_MAP_DESC(MP200_SMU_S2_BASE,  MP200_SMU_S2_SIZE),
	MP200_MAP_DESC(MP200_INTC_S2_BASE, MP200_INTC_S2_SIZE),
};

static struct plat_serial8250_port serial_platform_data[] = {
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART0_BASE),
	 .mapbase = (unsigned long)MP200_UART0_BASE,
	 .irq = INT_UART0,
	 .flags = UPF_BOOT_AUTOCONF,
	 .iotype = UPIO_MP200,
	 .regshift = 2,
	 .uartclk = MP200_BASE_BAUD,
	 },
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART1_BASE),
	 .mapbase = (unsigned long)MP200_UART1_BASE,
	 .irq = INT_UART1,
	 .flags = UPF_BOOT_AUTOCONF,
	 .iotype = UPIO_MP200,
	 .regshift = 2,
	 .uartclk = MP200_BASE_BAUD,
	 },
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART2_BASE),
	 .mapbase = (unsigned long)MP200_UART2_BASE,
	 .irq = INT_UART2,
	 .flags = UPF_BOOT_AUTOCONF,
	 .iotype = UPIO_MP200,
	 .regshift = 2,
	 .uartclk = MP200_BASE_BAUD,
	 },
	{
	 /* terminate */
	 },
};

static struct platform_device dma_device = {
	.name = "dma",
	.id = -1,
};

static struct platform_device spi0_device = {
	.name = "spi0",
	.id = 0,
};

static struct platform_device spi1_device = {
	.name = "spi",
	.id = 1,
};

static struct platform_device spi2_device = {
	.name = "spi",
	.id = 2,
};


#ifdef CONFIG_I2C_MP200_ENABLE_CH2
static struct platform_device i2c1_device = {
	.name = "i2c",
	.id = 0,
};

static struct platform_device i2c2_device = {
	.name = "i2c",
	.id = 1,
};
#else
static struct platform_device i2c_device = {
	.name = "i2c",
	.id = 0,
};
#endif

static struct platform_device serial8250_device = {
	.name = "serial8250",
	.id = 0,
	.dev = {
		.platform_data = serial_platform_data,
		},
};

static struct platform_device pcm_device = {
	.name = "pcm",
	.id = -1,
};

static struct platform_device mp200_fb_device = {
	.name = "mp200_fb",
	.id = -1,
};

static struct platform_device mp200_image_device = {
	.name = "mp200_image",
	.id = -1,
};

static struct platform_device mp200_dsp_device = {
	.name = "mp200_dsp",
	.id = -1,
};

#ifndef CONFIG_SNSC_MP200_PLATFORM
static struct platform_device mp200_udc_device = {
	.name = "mp200_udc",
	.id = -1,
};

static struct platform_device mp200_ehci_device = {
	.name = "mp200-ehci-driver",
	.id = -1,
};
#endif /* !CONFIG_SNSC_MP200_PLATFORM */

static struct platform_device mp200_mmc_device = {
	.name = "mp200_mmc",
	.id = -1,
};

#ifndef CONFIG_SNSC_MP200_PLATFORM
static struct platform_device mp200_mmc_sd_device = {
	.name = "mp200_mmc_sd",
	.id = -1,
};
#endif /* !CONFIG_SNSC_MP200_PLATFORM */

static struct platform_device mp200_dtv_device = {
	.name = "mp200_dtv",
	.id = -1,
};

static struct platform_device *platform_devs[] __initdata = {
	&dma_device,
	&spi0_device,
	&spi1_device,
	&spi2_device,
#ifdef CONFIG_I2C_MP200_ENABLE_CH2
	&i2c1_device,
	&i2c2_device,
#else
	&i2c_device,
#endif
	&serial8250_device,
	&pcm_device,
	&mp200_fb_device,
	&mp200_image_device,
	&mp200_dsp_device,
#ifndef CONFIG_SNSC_MP200_PLATFORM
	&mp200_udc_device,
	&mp200_ehci_device,
#endif /* !CONFIG_SNSC_MP200_PLATFORM */
	&mp200_mmc_device,
#ifndef CONFIG_SNSC_MP200_PLATFORM
	&mp200_mmc_sd_device,
#endif /* !CONFIG_SNSC_MP200_PLATFORM */
	&mp200_dtv_device,
};

#ifdef CONFIG_DEBUG_LL
static void __init mp200_fixup_ll_uart_divisor(
				struct plat_serial8250_port *port)
{
	unsigned int lcr;
	unsigned int baud_div;
	unsigned long base;

	base     = (unsigned long)port->membase;
	baud_div = (port->uartclk + (MP200_LL_UART_BAUD * 8)) /
						(MP200_LL_UART_BAUD * 16);

	/* wait for transmission to be complete */
	while (!(inl(base + MP200_UART_LSR) & UART_LSR_TEMT))
		/* busy wait */;

	/* save LCR */
	lcr = inl(base + MP200_UART_LCR);

	/* set DLL/DLM */
	outl(UART_LCR_DLAB         , base + MP200_UART_LCR);
	outl((baud_div >> 0) & 0xff, base + MP200_UART_DLL);
	outl((baud_div >> 8) & 0xff, base + MP200_UART_DLM);

	/* restore LCR */
	outl(lcr, base + MP200_UART_LCR);
}
#endif /* CONFIG_DEBUG_LL */

void __init mp200_map_io(void)
{
	iotable_init(standard_io_desc, ARRAY_SIZE(standard_io_desc));
}

void __init mp200_serial_init(int *ports)
{
	int i;

#ifdef CONFIG_DEBUG_LL
	if (ports[MP200_LL_UART_PORT] != 0)
		mp200_fixup_ll_uart_divisor(
			serial_platform_data + MP200_LL_UART_PORT);
#endif

	mp200_pmu_open_clockgate(MP200_CLOCKGATE_U70_CLK);
	for (i = 0; i < 3; i++) {
		if (ports[i] == 0) {
			serial_platform_data[i].flags = 0;
			continue;
		}
		/* Set U7x DIV clock */
		outl( SMU_DIV4, SMU_DIVU70SCLK + i*4);
		/* Unreset U7x */
		mp200_pmu_open_clockgate(MP200_CLOCKGATE_U70_SCLK + i);
		mp200_pmu_unreset_device(MP200_RESETDEVICE_U70 + i);
	}
}

void __init mp200_platform_init(void)
{
#ifdef CONFIG_MP200_L220
	unsigned int l2_pte, l2_page;
	unsigned long l2_addr = IO_ADDRESS(MP200_L220_BASE);
	pgd_t *pgd;
	pmd_t *pmd;
#endif

#ifdef CONFIG_MP200_QR
	outl(0x00020402, SMU_QR_CLKDIV);
	outl(0x00530103, SMU_QR_ENA);
#endif

#ifdef CONFIG_MP200_L220
#if 0
	/* Change L220 register mapping (Non-shared device). */
	l2_pte = 0x80010003;
	__asm__ __volatile__("mcr p15, 0, %0, c15, c2, 4"::"r"(l2_pte));
#else
	pgd = pgd_offset_k(l2_addr);
	pmd = pmd_offset(pgd, l2_addr);
	l2_pte = (unsigned int)pte_offset_kernel(pmd, l2_addr);
	l2_pte -= 2048;
	l2_page = inl(l2_pte);
	l2_page &= ~0x0c;
	l2_page |= 0x80;
	outl(l2_page, l2_pte);
	/* flush pte */
	__asm__ __volatile__("mcr p15, 0, %0, c7, c14, 1"::"r"(l2_pte));
	__asm__ __volatile__("mcr p15, 0, %0, c8, c6, 1"::"r"(l2_pte));	// Inv D TLB
	l2_pte = 0;
	__asm__ __volatile__("mcr p15, 0, %0, c7, c10, 4"::"r"(l2_pte));	// Sync
#endif

#ifdef CONFIG_MP200_L220_WT
	/* Force L2 write through */
	outl(0x2, l2_addr + 0xf40);
#endif

	/* 1MB (128KB/way), 8-way associativity, evmon/parity/share enabled
	 * Bits:  .... ...0 0111 1001 0000 .... .... .... */
//	l2x0_init((void __iomem *)l2_addr, 0x0003092d, 0xfe000000);
	l2x0_init((void __iomem *)l2_addr, 0x00020000, 0xfe000000);
#endif
}

static int __init mp200_init(void)
{
	return platform_add_devices(platform_devs, ARRAY_SIZE(platform_devs));
}

arch_initcall(mp200_init);

#ifdef CONFIG_MEMORY_HOTPLUG
unsigned int target_get_mem_type(unsigned long phys)
{
	return MT_HOTPLUG;
}
#endif
