/* 2009-10-02: File added and changed by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This file is based on linux/arch/arm/mach-mp200/mp200_board.c
 */
/*
 *  File Name       : linux/arch/arm/mach-mp200/mp200_board.c
 *  Function        : mp200_board
 *  Release Version : Ver 1.00
 *  Release Date    : 2006/07/11
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>

#include "../generic.h"
#include "../time.h"

#include "../../../../drivers/net/smsc911x.h"

static int __initdata mp200_serial_ports[] = { 1, 1, 1, 0 };

static struct resource smsc911x_resources[] = {
	[0] = {
		.start	= GROVER_LAN_BASE,
		.end	= GROVER_LAN_BASE + SZ_64K - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= GROVER_INT_LAN,
		.end	= GROVER_INT_LAN,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct smsc911x_platform_config smsc911x_config = {
	.irq_polarity	= 0, /* active low */
	.irq_type	= 0, /* open-drain */
};

static struct platform_device smsc911x_device = {
	.name		= "smsc911x",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(smsc911x_resources),
	.resource	= smsc911x_resources,
	.dev = {
		.platform_data = &smsc911x_config,
	},
};

static void __init grover_lan_init(void)
{
	if (GROVER_BOARD_ID() <= GROVER_BOARD_ID_GCP02)
		platform_device_register(&smsc911x_device);
}

static void __init grover_board_map_io(void)
{
	mp200_map_io();
}

static void __init grover_board_init(void)
{
	mp200_serial_init(mp200_serial_ports);

	mp200_platform_init();

	grover_lan_init();
}

MACHINE_START(GROVER, "GROVER")
	.phys_io      = MP200_LL_UART_BASE & 0xfff00000,
	.io_pg_offst  = (IO_ADDRESS(MP200_LL_UART_BASE) >> 18) & 0xfffc,
	.boot_params  = PHYS_OFFSET+0x100,
	.map_io       = grover_board_map_io,
	.init_irq     = mp200_init_irq,
	.init_machine = grover_board_init,
	.timer        = &mp200_timer,
MACHINE_END
