/* 2009-07-11: File added and changed by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>
#include <linux/mmc/mmc.h>
#include <linux/mmc/host.h>
#include <linux/mmc/sd.h>
#include <linux/mmc/core.h>
#include <linux/mmc/card.h>

#include <asm/io.h>
#include <asm/mach/irq.h>
#include <asm/arch/gpio.h>
#include <asm/arch/mmc_sd.h>
#include "../../../../drivers/mmc/host/mp200_mmc.h"

#define GROVER_MMC_SD_DEVICE0_ID	0
#define GROVER_MMC_SD_DEVICE0_INS_IRQ	INT_GPIO_72
#define GROVER_MMC_SD_DEVICE0_INS_GPIO	GPIO_P72
#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
#define GROVER_MMC_SD_DEVICE0_SDI	NULL
#else  /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
#define GROVER_MMC_SD_DEVICE0_SDI	(&mp200_mmc_sdia)
#define GROVER_MMC_SD_DEVICE1_ID	1
#define GROVER_MMC_SD_DEVICE1_INS_IRQ	INT_GPIO_82
#define GROVER_MMC_SD_DEVICE1_INS_GPIO	GPIO_P82
#define GROVER_MMC_SD_DEVICE1_SDI	(&mp200_mmc_sdib)
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */

static irqreturn_t
grover_mp200_mmc_detect_irq(int irq, void *dev_id)
{
	struct platform_device *pdev;
	struct mp200_mmc_host *host;

	host = dev_id;
	pdev = to_platform_device(host->mmc->parent);

	switch (pdev->id) {
	case GROVER_MMC_SD_DEVICE0_ID:
		if (irq != GROVER_MMC_SD_DEVICE0_INS_IRQ)
			return (IRQ_NONE);
		break;
#ifdef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
	case GROVER_MMC_SD_DEVICE1_ID:
		if (irq != GROVER_MMC_SD_DEVICE1_INS_IRQ)
			return (IRQ_NONE);
		break;
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
	default :
		return (IRQ_NONE);
	}

	mmc_detect_change(host->mmc, (HZ * 50) / 1000);

	return IRQ_HANDLED;
}

static int
grover_mmc_sd_probe(struct platform_device *pdev, struct mmc_host *mmc)
{
	struct mp200_mmc_host *host;
	int ret;

	host = mmc_priv(mmc);

	switch (pdev->id) {
	case GROVER_MMC_SD_DEVICE0_ID :
		host->detect_irq = GROVER_MMC_SD_DEVICE0_INS_IRQ;
		gpio_direction_input(GROVER_MMC_SD_DEVICE0_INS_GPIO);
		break;
#ifdef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
	case GROVER_MMC_SD_DEVICE1_ID :
		host->detect_irq = GROVER_MMC_SD_DEVICE1_INS_IRQ;
		gpio_direction_input(GROVER_MMC_SD_DEVICE1_INS_GPIO);
		break;
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
	default :
		printk("%s: invalid device id(%d)", __FUNCTION__, pdev->id);
		return (-EINVAL);
	}

	set_irq_type(host->detect_irq, IRQ_TYPE_EDGE_BOTH);
	ret = request_irq(host->detect_irq, grover_mp200_mmc_detect_irq,
			  IRQF_DISABLED, pdev->name, host);

	return (ret);
}

static void
grover_mmc_sd_remove(struct platform_device *pdev)
{
	struct mp200_mmc_host *host;

	host = platform_get_drvdata(pdev);

	free_irq(host->detect_irq, host);
}

static void
grover_mp200_mmc_sd_release(struct device *dev)
{
	/* nothing to do */
}

struct mp200_mmc_sd_platform_data mp200_mmc_sd_data0 = {
	.mmc_sdi        = GROVER_MMC_SD_DEVICE0_SDI,
	.mmc_sd_get_ro  = NULL,
	.mmc_sd_probe   = grover_mmc_sd_probe,
	.mmc_sd_remove  = grover_mmc_sd_remove,
	.mmc_sd_suspend = NULL,
	.mmc_sd_resume  = NULL,
};

static struct platform_device mp200_mmc_sd_device0 = {
	.name = "mp200_mmc_sd",
	.id   = GROVER_MMC_SD_DEVICE0_ID,
	.dev  = {
		.release       = grover_mp200_mmc_sd_release,
		.platform_data = &mp200_mmc_sd_data0,
	},
};

#ifdef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
struct mp200_mmc_sd_platform_data mp200_mmc_sd_data1 = {
	.mmc_sdi        = GROVER_MMC_SD_DEVICE1_SDI,
	.mmc_sd_get_ro  = NULL,
	.mmc_sd_probe   = grover_mmc_sd_probe,
	.mmc_sd_remove  = grover_mmc_sd_remove,
	.mmc_sd_suspend = NULL,
	.mmc_sd_resume  = NULL,
};

static struct platform_device mp200_mmc_sd_device1 = {
	.name = "mp200_mmc_sd",
	.id   = GROVER_MMC_SD_DEVICE1_ID,
	.dev  = {
		.release       = grover_mp200_mmc_sd_release,
		.platform_data = &mp200_mmc_sd_data1,
	},
};
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */

static struct platform_device *devs[] = {
 	&mp200_mmc_sd_device0,
#ifdef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
 	&mp200_mmc_sd_device1,
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
};

static int __init grover_mp200_mmc_sd_init(void)
{

	return platform_add_devices(devs, ARRAY_SIZE(devs));
}

static void __exit grover_mp200_mmc_sd_exit(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(devs); i++)
		platform_device_unregister(devs[i]);
}

arch_initcall(grover_mp200_mmc_sd_init);
module_exit(grover_mp200_mmc_sd_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Sony Corporation");
