/* 2009-04-22: File added and changed by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>

#include <asm/arch/udc.h>
#include <asm/arch/gpio.h>

#include <asm/io.h>
#include <linux/usb.h>
#include <linux/usb/gadget.h>
#include "../../../../drivers/usb/gadget/mp200_udc.h"

static void grover_udc_enable_phy(struct mp200_udc *udc)
{
	gpio_set_value(GPIO_P79, 1);
}

static void grover_udc_disable_phy(struct mp200_udc *udc)
{
	gpio_set_value(GPIO_P79, 0);
}

static void grover_udc_check_vbus(struct mp200_udc *udc)
{
	udc->vbus_active = 1;
}

static int grover_udc_is_ulpi_active(struct mp200_udc *udc)
{
	int status = gpio_get_value(GPIO_P79);

	switch (status){
	case 0:
		return MP200_UDC_ULPI_INACTIVE;
	case 1:
		return MP200_UDC_ULPI_ACTIVE;
	default:
		return status;
	}
}

static struct mp200_udc_platform_data mp200_udc_config = {
	.udc_enable_phy = &grover_udc_enable_phy,
	.udc_disable_phy = &grover_udc_disable_phy,
	.udc_enable_power_save_mode = NULL,
	.udc_disable_power_save_mode = NULL,
	.udc_probe = NULL,
	.udc_remove = NULL,
	.udc_check_vbus = &grover_udc_check_vbus,
	.udc_enable_vbus_int = NULL,
	.udc_disable_vbus_int = NULL,
	.udc_is_ulpi_active = &grover_udc_is_ulpi_active,
};

static struct platform_device mp200_udc_device = {
	.name = "mp200_udc",
	.id = -1,
	.dev = {
		.platform_data = &mp200_udc_config,
	},
};

static struct platform_device *devs[] __initdata = {
	&mp200_udc_device,
};

static int __init grover_mp200_udc_init(void)
{
	return platform_add_devices(devs, ARRAY_SIZE(devs));
}

arch_initcall(grover_mp200_udc_init);
