/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name	    : linux/arch/arm/mach-mp200/irq.c
 *  Function	    : irq
 *  Release Version : Ver 1.00
 *  Release Date    : 2008/07/16
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/interrupt.h>

#include <asm/io.h>
#include <asm/mach/irq.h>

static void mp200_mask_irq(unsigned int irq)
{
	unsigned long ids;
	unsigned long mask = 0;

	ids = VA_INTC + MP200_INTC_IT0_IDS0;

	if (irq < 32) {
		mask = 1 << irq;
	} else  if (irq < 64){
		ids += 4;
		mask = 1 << (irq - 32);
	} else {
		ids += (MP200_INTC_IT0_IDS2 - MP200_INTC_IT0_IDS0);
		mask = 1 << (irq - 64);
	}

	__raw_writel(mask, ids);	/* disable */
}

static void mp200_unmask_irq(unsigned int irq)
{
	unsigned long ien;
	unsigned long mask = 0;

	ien = VA_INTC + MP200_INTC_IT0_IEN0;

	if (irq < 32) {
		mask = 1 << irq;
	} else if (irq < 64) {
		ien += 4;
		mask = 1 << (irq - 32);
	} else {
		ien += (MP200_INTC_IT0_IEN2 - MP200_INTC_IT0_IEN0);
		mask = 1 << (irq - 64);
	}

	__raw_writel(mask, ien);	/* enable */
}

static void mp200_ack_irq(unsigned int irq)
{
	__raw_writel((1 << (irq - 32)), (VA_INTC + MP200_INTC_IT0_IIR));
}

static struct irq_chip mp200_irq_chip_ack = {
	.name   = "INTC-edge",
	.ack    = mp200_ack_irq,
	.mask   = mp200_mask_irq,
	.unmask = mp200_unmask_irq,
	.disable= mp200_mask_irq,
};

static struct irq_chip mp200_irq_chip = {
	.name   = "INTC-level",
	.ack    = mp200_mask_irq,
	.mask   = mp200_mask_irq,
	.unmask = mp200_unmask_irq,
	.disable= mp200_mask_irq,
};


extern int clk_init(void);

void __init mp200_init_irq(void)
{
	unsigned int i;

	clk_init();

	/* Disable all interrupts */
	__raw_writel(~0U, VA_INTC + MP200_INTC_IT0_IDS0);
	__raw_writel(~0U, VA_INTC + MP200_INTC_IT0_IDS1);
	__raw_writel(~0U, VA_INTC + MP200_INTC_IT0_IDS2);
	__raw_writel(1, VA_INTC + MP200_INTC_IT0_FID);

	for (i = 0; i <= INT_LAST; i++) {
		if ((i == INT_IIC) || (i == INT_IIC2) || (i == INT_USBWAK)
		    || (INT_TIMER0 <= i && i <= INT_WDT3)
		    || (INT_TG0 <= i && i <= INT_TG2)
		    || (INT_TG3 <= i && i <= INT_TG5)) {
			set_irq_chip(i, &mp200_irq_chip_ack);
			set_irq_handler(i, handle_edge_irq);
		} else {
			set_irq_chip(i, &mp200_irq_chip);
			set_irq_handler(i, handle_level_irq);
		}
		set_irq_flags(i, IRQF_VALID);
	}
}
