/* 2009-03-05: File added and changed by Sony Corporation */
/*
 *  File Name       : linux/arch/arm/mach-mp200/mp200_board.c
 *  Function        : mp200_board
 *  Release Version : Ver 1.00
 *  Release Date    : 2006/07/11
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/device.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/types.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <linux/platform_device.h>
#include <linux/mm.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <asm/arch/smu.h>
#include <asm/gpio.h>

#include "generic.h"
#include "time.h"

static int __initdata mp200_serial_ports[] = { 1, 1, 1, 0 };

static struct resource smc91x_resources[] = {
	[0] = {
	       .start = MP200_ETHER_BASE+0x300,
	       .end = MP200_ETHER_BASE + 0x400,
	       .flags = IORESOURCE_MEM,
	       },
	[1] = {
	       .start = GPIO_ETH + INT_GPIO_BASE,
	       .end = GPIO_ETH + INT_GPIO_BASE,
	       .flags = IORESOURCE_IRQ,
	       },
};
static struct platform_device smc91x_device = {
	.name = "smc91x",
	.id = -1,
	.num_resources = ARRAY_SIZE(smc91x_resources),
	.resource = smc91x_resources,
};

static struct resource mp200_rtc_resources[] = {
	{
		.start = INT_PWC_RTCINT,
		.end   = INT_PWC_RTCINT,
		.flags = IORESOURCE_IRQ,
	},
};
static struct platform_device mp200_rtc_device = {
	.name = "mp200-rtc",
	.id = -1,
	.num_resources = ARRAY_SIZE(mp200_rtc_resources),
	.resource = mp200_rtc_resources,
};

static struct mtd_partition mp200_flash0_partitions[] = {
	{
	 .name   = "boot",
	 .offset = 0,
	 .size   = 0x00080000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	},
	{
	 .name   = "kernel",
	 .offset = MTDPART_OFS_APPEND,
	 .size   = 0x00380000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	},
	{
	 .name   = "root",
	 .offset = MTDPART_OFS_APPEND,
	 .size   = 0x01C00000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	},
};
static struct flash_platform_data mp200_flash0_data = {
	.map_name = "cfi_probe",
	.width    = 2,
	.parts    = mp200_flash0_partitions,
	.nr_parts = ARRAY_SIZE(mp200_flash0_partitions),
};
static struct resource mp200_flash0_resource = {
	.start = MP200_FLASH_BASE,
	.end   = MP200_FLASH_BASE + SZ_128M - 1,
	.flags = IORESOURCE_MEM,
};
static struct platform_device mp200_flash0_device = {
	.name = "mp200_flash",
	.id   = 0,
	.dev  = {
		.platform_data = &mp200_flash0_data,
		},
	.num_resources = 1,
	.resource = &mp200_flash0_resource,
};

#if 0
static struct mtd_partition mp200_nand_partition[] = {
	{
	 .name   = "nand device",
	 .offset = 0,
	 .size   = SZ_128M
	 },
};
static struct platform_nand_chip mp200_nand_data = {
	.nr_chips      = 1,
	.chip_delay    = 15,
	.options       = NAND_BUSWIDTH_16,
	.partitions    = mp200_nand_partition,
	.nr_partitions = ARRAY_SIZE(mp200_nand_partition),
};
static struct resource mp200_nand_resource[] = {
	{
		.start = MP200_NAND_DATA_BASE,
		.end   = MP200_NAND_DATA_BASE + 4 -1,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = MP200_NAND_COMMAND_BASE,
		.end   = MP200_NAND_COMMAND_BASE + 4 -1 ,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = MP200_NAND_ADDRESS_BASE,
		.end   = MP200_NAND_ADDRESS_BASE + 4 -1,
		.flags = IORESOURCE_MEM,
	},
};
static struct platform_device mp200_nand_device = {
	.name = "mp200_nand",
	.id   = -1,
	.dev  = {
		.platform_data = &mp200_nand_data,
		},
	.num_resources = ARRAY_SIZE(mp200_nand_resource),
	.resource = mp200_nand_resource,
};
#endif

static struct platform_device *devs[] __initdata = {
	&smc91x_device,
	&mp200_rtc_device,
	&mp200_flash0_device,
#if 0
	&mp200_nand_device,
#endif
};

static void __init mp200_board_map_io(void)
{
	mp200_map_io();
}

static void __init mp200_board_init(void)
{
	mp200_serial_init(mp200_serial_ports);

	mp200_platform_init();

	platform_add_devices(devs, ARRAY_SIZE(devs));
}

MACHINE_START(MP200, "MP200")
	.phys_io      = MP200_LL_UART_BASE & 0xfff00000,
	.io_pg_offst  = (IO_ADDRESS(MP200_LL_UART_BASE) >> 18) & 0xfffc,
	.boot_params  = PHYS_OFFSET+0x100,
	.map_io       = mp200_board_map_io,
	.init_irq     = mp200_init_irq,
	.init_machine = mp200_board_init,
	.timer        = &mp200_timer,
MACHINE_END
