/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name	    : linux/arch/arm/mach-mp200/pmu.c
 *  Function	    : pmu
 *  Release Version : Ver 1.02
 *  Release Date    : 2009/02/24
 *
 *  Copyright (C) NEC Electronics Corporation 2009
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/delay.h>

#include <asm/io.h>

#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>

static spinlock_t lock_clockgate = SPIN_LOCK_UNLOCKED;
static spinlock_t lock_reset0 = SPIN_LOCK_UNLOCKED;
static spinlock_t lock_reset1 = SPIN_LOCK_UNLOCKED;
static spinlock_t lock_reset2 = SPIN_LOCK_UNLOCKED;
static spinlock_t lock_reset3 = SPIN_LOCK_UNLOCKED;
static spinlock_t lock_clkctrl = SPIN_LOCK_UNLOCKED;


#define MLD_RESET_MASK	(1 << (MP200_RESETDEVICE_MLD & SMU_REG_BIT_MASK))
#define MLD_RCLK_MASK	(1 << (MP200_CLOCKGATE_MLD_RCLK & SMU_REG_BIT_MASK))


static inline void pmu_parent_clock(unsigned int clkbit, int enable)
{

	switch (clkbit) {
	default:
		break;
	}
}


int mp200_pmu_open_clockgate(unsigned int clkbit)
{
	unsigned long cpu_flags;
	unsigned int mask;
	int bit;
	int ret = 0, delay_usec = 0;

	bit = clkbit & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	pmu_parent_clock(clkbit, 1);

	spin_lock_irqsave(&lock_clockgate, cpu_flags);

	switch (clkbit & SMU_REG_GROUP_MASK) {
	case SMU_GCLKCTRL0_GROUP:
		outl(mask, SMU_GCLKCTRL0ENA);
		outl(mask, SMU_GCLKCTRL0);
		outl(0, SMU_GCLKCTRL0ENA);
		break;
	case SMU_GCLKCTRL1_GROUP:
		outl(mask, SMU_GCLKCTRL1ENA);
		outl(mask, SMU_GCLKCTRL1);
		outl(0, SMU_GCLKCTRL1ENA);
		break;
	case SMU_GCLKCTRL2_GROUP:
		outl(mask, SMU_GCLKCTRL2ENA);
		outl(mask, SMU_GCLKCTRL2);
		outl(0, SMU_GCLKCTRL2ENA);
		break;
	case SMU_GCLKCTRL3_GROUP:
		outl(mask, SMU_GCLKCTRL3ENA);
		outl(mask, SMU_GCLKCTRL3);
		outl(0, SMU_GCLKCTRL3ENA);
		break;
	case SMU_GCLKCTRL4_GROUP:
		outl(mask, SMU_GCLKCTRL4ENA);
		outl(mask, SMU_GCLKCTRL4);
		outl(0, SMU_GCLKCTRL4ENA);
		break;
	default :
		ret = -EINVAL;
		break;
	}

	spin_unlock_irqrestore(&lock_clockgate, cpu_flags);

	if (delay_usec != 0)
		udelay(delay_usec);

	return ret;
}

int mp200_pmu_close_clockgate(unsigned int clkbit)
{
	unsigned long cpu_flags;
	unsigned int mask;
	int bit;
	int ret = 0;
	int close_skip = 0;

	bit = clkbit & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	spin_lock_irqsave(&lock_clockgate, cpu_flags);

	switch (clkbit) {
	case MP200_CLOCKGATE_U70_CLK:
		if (
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_U70_SCLK) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_U71_SCLK) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_U72_SCLK)) {
		    close_skip = 1;
		}
		break;
	case MP200_CLOCKGATE_ATIM_PCLK:
		if (
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TI0_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TI1_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TI2_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TI3_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TW0_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TW1_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TW2_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TW3_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TG0_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TG1_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TG2_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TG3_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TG4_TIN) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_TG5_TIN)
		   ) {
		    close_skip = 1;
		}
		break;
	case MP200_CLOCKGATE_PM0_PCLK:
		if (
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_PM0_SCLK) ||
		    mp200_pmu_get_clockgate(MP200_CLOCKGATE_PM1_SCLK)
		   ) {
		    close_skip = 1;
		}
		break;
	default:
		break;
	}

	if (close_skip == 0) {
		switch (clkbit & SMU_REG_GROUP_MASK) {
		case SMU_GCLKCTRL0_GROUP:
			outl(mask, SMU_GCLKCTRL0ENA);
			outl(0, SMU_GCLKCTRL0);
			outl(0, SMU_GCLKCTRL0ENA);
			break;
		case SMU_GCLKCTRL1_GROUP:
			outl(mask, SMU_GCLKCTRL1ENA);
			outl(0, SMU_GCLKCTRL1);
			outl(0, SMU_GCLKCTRL1ENA);
			break;
		case SMU_GCLKCTRL2_GROUP:
			outl(mask, SMU_GCLKCTRL2ENA);
			outl(0, SMU_GCLKCTRL2);
			outl(0, SMU_GCLKCTRL2ENA);
			break;
		case SMU_GCLKCTRL3_GROUP:
			outl(mask, SMU_GCLKCTRL3ENA);
			outl(0, SMU_GCLKCTRL3);
			outl(0, SMU_GCLKCTRL3ENA);
			break;
		case SMU_GCLKCTRL4_GROUP:
			outl(mask, SMU_GCLKCTRL4ENA);
			outl(0, SMU_GCLKCTRL4);
			outl(0, SMU_GCLKCTRL4ENA);
			break;
		default :
			ret = -EINVAL;
			break;
		}
	}

	spin_unlock_irqrestore(&lock_clockgate, cpu_flags);

	pmu_parent_clock(clkbit, 0);

	return ret;
}

int mp200_pmu_get_clockgate(unsigned int clkbit)
{
	unsigned int regval;
	unsigned int mask;
	int bit;
	int ret;

	bit = clkbit & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	switch (clkbit & SMU_REG_GROUP_MASK) {
	case SMU_GCLKCTRL0_GROUP:
		regval = inl(SMU_GCLKCTRL0);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_GCLKCTRL1_GROUP:
		regval = inl(SMU_GCLKCTRL1);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_GCLKCTRL2_GROUP:
		regval = inl(SMU_GCLKCTRL2);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_GCLKCTRL3_GROUP:
		regval = inl(SMU_GCLKCTRL3);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_GCLKCTRL4_GROUP:
		regval = inl(SMU_GCLKCTRL4);
		ret = (regval & mask) ? 1 : 0;
		break;
	default:
		ret = -EINVAL;
		break;
	}

	return ret;
}

#define RST_WAIT_MAX_CLKCTRL	3
static inline void clkctrl_off_wait(unsigned int clkctrl[], unsigned int clkctrl_num, unsigned int delay_num)
{
	int i;
	unsigned int val[RST_WAIT_MAX_CLKCTRL];

	for (i = 0; i<clkctrl_num; i++) {
		val[i] = mp200_pmu_get_clkctrl_status(clkctrl[i]);
		if (val[i] == 1) {
			mp200_pmu_clkctrl_off(clkctrl[i]);
		}
	}

	udelay(delay_num);

	for (i = 0; i<clkctrl_num; i++) {
		if (val[i] == 1) {
			mp200_pmu_clkctrl_on(clkctrl[i]);
		}
	}
}

static inline void pmu_reset_wait(unsigned int rstdev)
{
	unsigned int clkctrl[RST_WAIT_MAX_CLKCTRL];

	switch (rstdev) {
	case MP200_RESETDEVICE_SP2:
		clkctrl[0] = MP200_CLKCTRL_SP2PCLK;
		clkctrl[1] = MP200_CLKCTRL_SP2SCLK;
		clkctrl_off_wait(clkctrl, 2, 100);
		break;
	case MP200_RESETDEVICE_SP1:
		clkctrl[0] = MP200_CLKCTRL_SP1PCLK;
		clkctrl[1] = MP200_CLKCTRL_SP1SCLK;
		clkctrl_off_wait(clkctrl, 2, 100);
		break;
	case MP200_RESETDEVICE_SP0:
		clkctrl[0] = MP200_CLKCTRL_SP0PCLK;
		clkctrl[1] = MP200_CLKCTRL_SP0SCLK;
		clkctrl_off_wait(clkctrl, 2, 100);
		break;
	case MP200_RESETDEVICE_TG5:
	case MP200_RESETDEVICE_TG4:
	case MP200_RESETDEVICE_TG3:
	case MP200_RESETDEVICE_TG2:
	case MP200_RESETDEVICE_TG1:
	case MP200_RESETDEVICE_TG0:
	case MP200_RESETDEVICE_TW3:
	case MP200_RESETDEVICE_TW2:
	case MP200_RESETDEVICE_TW1:
	case MP200_RESETDEVICE_TW0:
	case MP200_RESETDEVICE_TI3:
	case MP200_RESETDEVICE_TI2:
	case MP200_RESETDEVICE_TI1:
	case MP200_RESETDEVICE_TI0:
		clkctrl[0] = MP200_CLKCTRL_TIMPCLK;
		clkctrl_off_wait(clkctrl, 1, 100);
		break;
	case MP200_RESETDEVICE_IIC:
	case MP200_RESETDEVICE_IIC2:
	case MP200_RESETDEVICE_MWI:
	case MP200_RESETDEVICE_NAND:
	case MP200_RESETDEVICE_PM1:
	case MP200_RESETDEVICE_PM0:
	case MP200_RESETDEVICE_U72:
	case MP200_RESETDEVICE_U71:
	case MP200_RESETDEVICE_U70:
		udelay(100);
		break;
	case MP200_RESETDEVICE_LCD:
		clkctrl[0] = MP200_CLKCTRL_LCDPCLK;
		clkctrl[1] = MP200_CLKCTRL_LCD;
		clkctrl[2] = MP200_CLKCTRL_LCDCCK;
		clkctrl_off_wait(clkctrl, 3, 1);
		break;
	default:
		break;
	}
}

int mp200_pmu_reset_device(unsigned int rstdev)
{
	unsigned long cpu_flags;
	unsigned int mask;
	int bit;
	int ret = 0;

	bit = rstdev & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	switch (rstdev & SMU_REG_GROUP_MASK) {
	case SMU_RESETREQ0_GROUP:
		spin_lock_irqsave(&lock_reset0, cpu_flags);
		outl(mask, SMU_RESETREQ0ENA);
		if (bit < 8) {
			outb(0, SMU_RESETREQ0);
		} else if (bit < 16) {
			outb(0, SMU_RESETREQ0 + 1);
		} else if (bit < 24) {
			outb(0, SMU_RESETREQ0 + 2);
		} else {
			outb(0, SMU_RESETREQ0 + 3);
		}
		outl(0, SMU_RESETREQ0ENA);
		spin_unlock_irqrestore(&lock_reset0, cpu_flags);
		break;
	case SMU_RESETREQ1_GROUP:
		spin_lock_irqsave(&lock_reset1, cpu_flags);
		outl(mask, SMU_RESETREQ1ENA);
		outl(0, SMU_RESETREQ1);
		outl(0, SMU_RESETREQ1ENA);
		spin_unlock_irqrestore(&lock_reset1, cpu_flags);
		break;
	case SMU_RESETREQ2_GROUP:
		spin_lock_irqsave(&lock_reset2, cpu_flags);
		outl(mask, SMU_RESETREQ2ENA);
		outl(0, SMU_RESETREQ2);
		outl(0, SMU_RESETREQ2ENA);
		spin_unlock_irqrestore(&lock_reset2, cpu_flags);
		break;
	case SMU_RESETREQ3_GROUP:
		spin_lock_irqsave(&lock_reset3, cpu_flags);
		outl(mask, SMU_RESETREQ3ENA);
		outl(0, SMU_RESETREQ3);
		outl(0, SMU_RESETREQ3ENA);
		spin_unlock_irqrestore(&lock_reset3, cpu_flags);
		break;
	default :
		ret = -EINVAL;
		break;
	}

	pmu_reset_wait(rstdev);

	return ret;
}

int mp200_pmu_unreset_device(unsigned int rstdev)
{
	unsigned long cpu_flags;
	unsigned int mask;
	int bit;
	int ret = 0;

	bit = rstdev & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;


	switch (rstdev & SMU_REG_GROUP_MASK) {
	case SMU_RESETREQ0_GROUP:
		spin_lock_irqsave(&lock_reset0, cpu_flags);
		outl(mask, SMU_RESETREQ0ENA);
		if (bit < 8) {
			outb(mask, SMU_RESETREQ0);
		} else if (bit < 16) {
			outb(mask >>  8, SMU_RESETREQ0 + 1);
		} else if (bit < 24) {
			outb(mask >> 16, SMU_RESETREQ0 + 2);
		} else {
			outb(mask >> 24, SMU_RESETREQ0 + 3);
		}
		outl(0, SMU_RESETREQ0ENA);
		spin_unlock_irqrestore(&lock_reset0, cpu_flags);
		break;
	case SMU_RESETREQ1_GROUP:
		spin_lock_irqsave(&lock_reset1, cpu_flags);
		outl(mask, SMU_RESETREQ1ENA);
		outl(mask, SMU_RESETREQ1);
		outl(0, SMU_RESETREQ1ENA);
		spin_unlock_irqrestore(&lock_reset1, cpu_flags);
		break;
	case SMU_RESETREQ2_GROUP:
		spin_lock_irqsave(&lock_reset2, cpu_flags);
		outl(mask, SMU_RESETREQ2ENA);
		outl(mask, SMU_RESETREQ2);
		outl(0, SMU_RESETREQ2ENA);
		spin_unlock_irqrestore(&lock_reset2, cpu_flags);
		break;
	case SMU_RESETREQ3_GROUP:
		spin_lock_irqsave(&lock_reset3, cpu_flags);
		outl(mask, SMU_RESETREQ3ENA);
		outl(mask, SMU_RESETREQ3);
		outl(0, SMU_RESETREQ3ENA);
		spin_unlock_irqrestore(&lock_reset3, cpu_flags);
		break;
	default:
		ret = -EINVAL;
		break;
	}

	pmu_reset_wait(rstdev);

	return ret;
}

int mp200_pmu_get_reset_status(unsigned int rstdev)
{
	unsigned int regval;
	unsigned int mask;
	int bit;
	int ret;

	bit = rstdev & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	switch (rstdev & SMU_REG_GROUP_MASK) {
	case SMU_RESETREQ0_GROUP:
		regval = inl(SMU_RESETREQ0);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_RESETREQ1_GROUP:
		regval = inl(SMU_RESETREQ1);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_RESETREQ2_GROUP:
		regval = inl(SMU_RESETREQ2);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_RESETREQ3_GROUP:
		regval = inl(SMU_RESETREQ3);
		ret = (regval & mask) ? 1 : 0;
		break;
	default:
		ret = -EINVAL;
		break;
	}

	return ret;
}

int mp200_pmu_clkctrl_on(unsigned int dev)
{
	unsigned long cpu_flags;
	unsigned int regval;
	unsigned int mask;
	int bit;
	int ret = 0;

	bit = dev & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	spin_lock_irqsave(&lock_clkctrl, cpu_flags);

	switch (dev & SMU_REG_GROUP_MASK) {
	case SMU_AHBCLKCTRL0_GROUP:
		regval = inl(SMU_AHBCLKCTRL0);
		regval |= mask;
		outl(regval, SMU_AHBCLKCTRL0);
		break;
	case SMU_AHBCLKCTRL1_GROUP:
		regval = inl(SMU_AHBCLKCTRL1);
		regval |= mask;
		outl(regval, SMU_AHBCLKCTRL1);
		break;
	case SMU_APBCLKCTRL0_GROUP:
		regval = inl(SMU_APBCLKCTRL0);
		regval |= mask;
		outl(regval, SMU_APBCLKCTRL0);
		break;
	case SMU_APBCLKCTRL1_GROUP:
		regval = inl(SMU_APBCLKCTRL1);
		regval |= mask;
		outl(regval, SMU_APBCLKCTRL1);
		break;
	case SMU_CLKCTRL_GROUP:
		regval = inl(SMU_CLKCTRL);
		regval |= mask;
		outl(regval, SMU_CLKCTRL);
		break;
	case SMU_APBCLKCTRL2_GROUP:
		regval = inl(SMU_APBCLKCTRL2);
		regval |= mask;
		outl(regval, SMU_APBCLKCTRL2);
		break;
	default:
		ret = -EINVAL;
		break;
	}

	spin_unlock_irqrestore(&lock_clkctrl, cpu_flags);

	return ret;
}

int mp200_pmu_clkctrl_off(unsigned int dev)
{
	unsigned long cpu_flags;
	unsigned int regval;
	unsigned int mask;
	int bit;
	int ret = 0;

	bit = dev & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	spin_lock_irqsave(&lock_clkctrl, cpu_flags);

	switch (dev & SMU_REG_GROUP_MASK) {
	case SMU_AHBCLKCTRL0_GROUP:
		regval = inl(SMU_AHBCLKCTRL0);
		regval &= ~mask;
		outl(regval, SMU_AHBCLKCTRL0);
		break;
	case SMU_AHBCLKCTRL1_GROUP:
		regval = inl(SMU_AHBCLKCTRL1);
		regval &= ~mask;
		outl(regval, SMU_AHBCLKCTRL1);
		break;
	case SMU_APBCLKCTRL0_GROUP:
		regval = inl(SMU_APBCLKCTRL0);
		regval &= ~mask;
		outl(regval, SMU_APBCLKCTRL0);
		break;
	case SMU_APBCLKCTRL1_GROUP:
		regval = inl(SMU_APBCLKCTRL1);
		regval &= ~mask;
		outl(regval, SMU_APBCLKCTRL1);
		break;
	case SMU_CLKCTRL_GROUP:
		regval = inl(SMU_CLKCTRL);
		regval &= ~mask;
		outl(regval, SMU_CLKCTRL);
		break;
	case SMU_APBCLKCTRL2_GROUP:
		regval = inl(SMU_APBCLKCTRL2);
		regval &= ~mask;
		outl(regval, SMU_APBCLKCTRL2);
		break;
	default:
		ret = -EINVAL;
		break;
	}

	spin_unlock_irqrestore(&lock_clkctrl, cpu_flags);

	return ret;
}

int mp200_pmu_get_clkctrl_status(unsigned int dev)
{
	unsigned int regval;
	unsigned int mask;
	int bit;
	int ret;

	bit = dev & SMU_REG_BIT_MASK;
	if (bit >= SMU_REG_MAX_BIT_LEN) {
		return -EINVAL;
	}
	mask = 1 << bit;

	switch (dev & SMU_REG_GROUP_MASK) {
	case SMU_AHBCLKCTRL0_GROUP:
		regval = inl(SMU_AHBCLKCTRL0);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_AHBCLKCTRL1_GROUP:
		regval = inl(SMU_AHBCLKCTRL1);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_APBCLKCTRL0_GROUP:
		regval = inl(SMU_APBCLKCTRL0);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_APBCLKCTRL1_GROUP:
		regval = inl(SMU_APBCLKCTRL1);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_CLKCTRL_GROUP:
		regval = inl(SMU_CLKCTRL);
		ret = (regval & mask) ? 1 : 0;
		break;
	case SMU_APBCLKCTRL2_GROUP:
		regval = inl(SMU_APBCLKCTRL2);
		ret = (regval & mask) ? 1 : 0;
		break;
	default:
		ret = -EINVAL;
		break;
	}

	return ret;
}


EXPORT_SYMBOL(mp200_pmu_open_clockgate);
EXPORT_SYMBOL(mp200_pmu_close_clockgate);
EXPORT_SYMBOL(mp200_pmu_get_clockgate);

EXPORT_SYMBOL(mp200_pmu_reset_device);
EXPORT_SYMBOL(mp200_pmu_unreset_device);
EXPORT_SYMBOL(mp200_pmu_get_reset_status);

EXPORT_SYMBOL(mp200_pmu_clkctrl_on);
EXPORT_SYMBOL(mp200_pmu_clkctrl_off);
EXPORT_SYMBOL(mp200_pmu_get_clkctrl_status);

