/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name       : linux/arch/arm/mach-mp200/time.c
 *  Function        : time
 *  Release Version : Ver 1.04
 *  Release Date    : 2007/08/08
 *
 *  Copyright (C) NEC Electronics Corporation 2007
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/clocksource.h>
#include <linux/clockchips.h>

#include <asm/io.h>
#include <asm/div64.h>
#include <asm/mach/time.h>

#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/arch/timer.h>
#include <asm/cnt32_to_63.h>

#include "time.h"

static spinlock_t timer_spinlock = SPIN_LOCK_UNLOCKED;

static struct tm_param_t tm_param[] = {
	/* TIMER_TGn */
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	/* TIMER_TIn */
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	/* TIMER_TWn */
	{.usecs = TW_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
};

static const struct tm_reg_t tm_reg[] = {
	/* TIMER_TGn */
	{
	 .irq    = INT_TG0,
	 .tm_op  = IO_ADDRESS(MP200_TG0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG0_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TG0_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TG0_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TG0_TIN,
	 .rstdev = MP200_RESETDEVICE_TG0,
	 },
	{
	 .irq    = INT_TG1,
	 .tm_op  = IO_ADDRESS(MP200_TG1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG1_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TG1_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TG1_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TG1_TIN,
	 .rstdev = MP200_RESETDEVICE_TG1,
	 },
	{
	 .irq    = INT_TG2,
	 .tm_op  = IO_ADDRESS(MP200_TG2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG2_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TG2_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TG2_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TG2_TIN,
	 .rstdev = MP200_RESETDEVICE_TG2,
	 },
	{
	 .irq    = INT_TG3,
	 .tm_op  = IO_ADDRESS(MP200_TG3_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG3_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG3_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TG3_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TG3_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TG3_TIN,
	 .rstdev = MP200_RESETDEVICE_TG3,
	 },
	{
	 .irq    = INT_TG4,
	 .tm_op  = IO_ADDRESS(MP200_TG4_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG4_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG4_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TG4_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TG4_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TG4_TIN,
	 .rstdev = MP200_RESETDEVICE_TG4,
	 },
	{
	 .irq    = INT_TG5,
	 .tm_op  = IO_ADDRESS(MP200_TG5_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG5_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG5_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TG5_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TG5_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TG5_TIN,
	 .rstdev = MP200_RESETDEVICE_TG5,
	 },
	/* TIMER_TIn */
	{
	 .irq    = INT_TIMER0, /* 54 */
	 .tm_op  = IO_ADDRESS(MP200_TIMER0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER0_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TIMER0_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TIMER0_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TI0_TIN,
	 .rstdev = MP200_RESETDEVICE_TI0,
	 },
	{
	 .irq    = INT_TIMER1, /* 55 */
	 .tm_op  = IO_ADDRESS(MP200_TIMER1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER1_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TIMER1_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TIMER1_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI1,
	 },
	{
	 .irq    = INT_TIMER2, /* 56 */
	 .tm_op  = IO_ADDRESS(MP200_TIMER2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER2_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TIMER2_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TIMER2_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TI2_TIN,
	 .rstdev = MP200_RESETDEVICE_TI2,
	 },
	{
	 .irq    = INT_TIMER3, /* 57 */
	 .tm_op  = IO_ADDRESS(MP200_TIMER3_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER3_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER3_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_TIMER3_BASE) + 0x000c,
	 .tm_sclr= IO_ADDRESS(MP200_TIMER3_BASE) + 0x0014,
	 .clkdev = MP200_CLOCKGATE_TI3_TIN,
	 .rstdev = MP200_RESETDEVICE_TI3,
	 },
	/* TIMER_TWn */
	{
	 .irq    = INT_WDT0,
	 .tm_op  = IO_ADDRESS(MP200_WDT0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT0_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_WDT0_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TW0_TIN,
	 .rstdev = MP200_RESETDEVICE_TW0,
	 .tin_sel= SMU_TI0TIN_SEL,
	 },
	{
	 .irq    = INT_WDT1, /* 59 */
	 .tm_op  = IO_ADDRESS(MP200_WDT1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT1_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_WDT1_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TW1_TIN,
	 .rstdev = MP200_RESETDEVICE_TW1,
	 .tin_sel= SMU_TI1TIN_SEL,
	 },
	{
	 .irq    = INT_WDT2, /* 60 */
	 .tm_op  = IO_ADDRESS(MP200_WDT2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT2_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_WDT2_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TW2_TIN,
	 .rstdev = MP200_RESETDEVICE_TW2,
	 .tin_sel= SMU_TI2TIN_SEL,
	 },
	{
	 .irq    = INT_WDT3, /* 61 */
	 .tm_op  = IO_ADDRESS(MP200_WDT3_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT3_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT3_BASE) + 0x0008,
	 .rcr    = IO_ADDRESS(MP200_WDT3_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TW3_TIN,
	 .rstdev = MP200_RESETDEVICE_TW3,
	 .tin_sel= SMU_TI3TIN_SEL,
	 },
};

static const unsigned int intc_raw1 = VA_INTC + MP200_INTC_IT0_RAW1;

/* inline function */
inline int check_validity_channel(unsigned int timer)
{
    if (((TIMER_MIN <= timer) && (timer <= TIMER_MAX)) 
	|| (timer == DAC_HANG_TG) || (timer == DAC_HANG_TG_INTERDSP)){
		return 0;
	} else {
		return -ENODEV;
	}
}

inline unsigned int get_count_value(unsigned timer, unsigned int usecs)
{

	uint64_t counts;

	if (usecs == TIMER_MAX_COUNT) {
	  return TIMER_MAX_COUNT;
	}

	if (!(inl(SMU_TGNTIN_SEL) & (0x3 << (timer * 4)))) {
	  /* 12MHz */
	  counts = (uint64_t)usecs * TIMER_CLOCK_TICK_RATE_PLL3;
        } else {
	  /* 32KHz */
	  counts = (uint64_t)usecs * TIMER_CLOCK_TICK_RATE_32K;
        }
	do_div(counts,1000000);
	counts--;

	return (unsigned int)counts;

}
static inline unsigned int
get_tw_count_value(unsigned timer, unsigned int usecs)
{

	uint64_t counts;
	if ((inl(tm_reg[timer].tin_sel) & MASK_TWNTIN_SEL) ==
	    TWNTIN_SEL_32K) {
	  /* 32KHz */
	  counts = (uint64_t)usecs * TIMER_CLOCK_TICK_RATE_32K;
	} else {
	  /* 12MHz */
	  counts = (uint64_t)usecs * TIMER_CLOCK_TICK_RATE_PLL3;
	}

	do_div(counts,1000000);
	counts--;

	return (unsigned int)counts;

}


inline unsigned int get_usec_value(unsigned timer, unsigned int counts)
{

	uint64_t usecs;

	usecs = (uint64_t)counts * 1000000;

        if (!(inl(SMU_TGNTIN_SEL) & (0x3 << (timer * 4)))) {
	  /* 12MHz */
	  do_div(usecs,TIMER_CLOCK_TICK_RATE_PLL3);
        } else {
	  /* 32KHz */
	  do_div(usecs,TIMER_CLOCK_TICK_RATE_32K);
        }

	return (unsigned int)usecs;

}

unsigned long mp200_system_timer_read(int nr)
{

	return inl(tm_reg[nr].rcr);

}

void mp200_system_timer_start(int nr, unsigned int mode)
{
	outl((TSTART | TM_EN | mode), tm_reg[nr].tm_op);
}

static unsigned long timer_interval = TIMER_INTERVAL_PLL3;

unsigned long mp200_timer_ticks_to_usecs(unsigned long nr_ticks)
{
	uint64_t usecs;

	usecs = (uint64_t)nr_ticks * 1000000;

	if (timer_interval == TIMER_INTERVAL_32K) {
	  /* 32KHz */
	  do_div(usecs,TIMER_CLOCK_TICK_RATE_32K);

        } else {
	  /* 12MHz */
	  do_div(usecs,TIMER_CLOCK_TICK_RATE_PLL3);
        }

	return (unsigned int)usecs;

}

/*
 * Returns number of ms since last clock interrupt.
 * Note that interrupts will have been disabled by do_gettimeoffset().
 */
static unsigned long mp200_gettimeoffset(void)
{
	unsigned long ticks;
	unsigned long status1;
	unsigned long status2;

        /* This function will be called with interrupt disabled.
         * Check interrupt RAW register before and after reading counter.
         * If counter is saturated and interrupt is not occurred,
         * wait a few PCLK after reading counter to ensure
         * whether interrupt handling is or is not done.
         * (Determine we are called whether before or after interrupt
         *  handler execution.)
         */

	status1 = inl(intc_raw1) & (1 << (tm_reg[TIMER_SYSTEM].irq - 32));

	ticks = mp200_system_timer_read(TIMER_SYSTEM);


        /* counter saturated, but not interrupt */
        if ((ticks == 0) && (status1 == 0)) {
	  /* wait for about 3 PCLK */
	  __asm__ __volatile__("nop");
	  __asm__ __volatile__("nop");
	  __asm__ __volatile__("nop");
	  __asm__ __volatile__("nop");
	  __asm__ __volatile__("nop");
	  __asm__ __volatile__("nop");
        }

        status2 = inl(intc_raw1) & (1 << (tm_reg[TIMER_SYSTEM].irq - 32));

        /* If interrupt is occurred, we need to add 1 tick because
         * jiffies is not updated yet.
         */
        if (status2 != 0) {
	  if (ticks < (timer_interval / 2)) {
	    ticks += (timer_interval + 1);
	  }
        }

	return mp200_timer_ticks_to_usecs(ticks);

}

/*
 * IRQ handler for the timer
 */
static irqreturn_t mp200_timer_interrupt(int irq, void *dev_id)
{
	timer_tick();

	return IRQ_HANDLED;
}

static struct irqaction mp200_tick_count_timer_irq = {
	.name    = "mp200_timer",
	.flags   = IRQF_DISABLED | IRQF_TIMER,
	.handler = mp200_timer_interrupt
};


/*
 * Set up timer interrupt, and return the current time in seconds.
 */
static void __init mp200_init_timer(void)
{

        timer_set_clock_parm(TIMER_FULLSPEED);
        timer_set_clock(TIMER_INIT);

	setup_irq(tm_reg[TIMER_SYSTEM].irq, &mp200_tick_count_timer_irq);
	mp200_timer.offset = mp200_gettimeoffset;

	mp200_system_timer_start(TIMER_SYSTEM, TO_EN);
#ifdef CONFIG_SNSC_MP200_ACCURATE_SCHED_CLOCK
	/* Set the sched_clock() timer to run freely. */
	outl(0xffffffff, tm_reg[TIMER_SCHED_CLOCK].tm_set);
	mp200_timer_start(TIMER_SCHED_CLOCK);
	/* allow calls to sched_clock now */
	safe_to_call_sched_clock = 1;
#endif
}

struct sys_timer mp200_timer = {
	.init = mp200_init_timer,
	.offset = NULL,		/* Initialized later */
};

/*
 * Change internal timer parameter
 */
void timer_set_clock_parm(unsigned int mode)
{
#if 0
	switch (mode) {
	case TIMER_RESUME:
	case TIMER_FULLSPEED:
	  timer_interval = TIMER_INTERVAL_PLL3;
	  break;
	case TIMER_SUSPEND:
	case TIMER_ECONOMY:
	  timer_interval = TIMER_INTERVAL_32K;
	  break;
	default:
	  break;
	}
#endif
}

static unsigned int tg_tm_op[] = { 0, 0, 0, 0, 0, 0 };

void timer_set_clock(unsigned int mode)
{
	int i;
	unsigned int count;
	unsigned int tm_delay=0;

	/* delay value set */
	tm_delay = TIMER_DELAY(TIMER_CLOCK_TICK_RATE_32K);

	switch (mode) {
	case TIMER_SUSPEND:
	  /* close clockgate */
	  mp200_pmu_close_clockgate(tm_reg[TIMER_SYSTEM].clkdev);
	  mp200_pmu_close_clockgate(tm_reg[TIMER_WDT].clkdev);
	  mp200_pmu_close_clockgate(tm_reg[TIMER_DSP].clkdev);
	  for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
		if (tg_tm_op[i] == 1) {
		    mp200_pmu_close_clockgate(tm_reg[TIMER_TG0 + i].clkdev);
		}
	  }
	  break;

	case TIMER_SUSPEND_PDMA:
	  /* close clockgate */
	  mp200_pmu_close_clockgate(tm_reg[TIMER_SYSTEM].clkdev);
	  mp200_pmu_close_clockgate(tm_reg[TIMER_WDT].clkdev);
	  mp200_pmu_close_clockgate(tm_reg[TIMER_DSP].clkdev);
	  for (i = 0; i <= TIMER_MAX; i++) {
		if (tg_tm_op[i] == 1) {
		    mp200_pmu_close_clockgate(tm_reg[TIMER_TG0 + i].clkdev);
		}
	  }
	  break;

	case TIMER_RESUME:
	  mp200_pmu_open_clockgate(tm_reg[TIMER_SYSTEM].clkdev);
	  mp200_pmu_open_clockgate(tm_reg[TIMER_WDT].clkdev);
	  mp200_pmu_open_clockgate(tm_reg[TIMER_DSP].clkdev);

	  for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
		if (tg_tm_op[i] == 1) {
		    mp200_pmu_open_clockgate(tm_reg[TIMER_TG0 + i].clkdev);
		}
	  }
	  break;

	case TIMER_RESUME_PDMA:
	  mp200_pmu_open_clockgate(tm_reg[TIMER_SYSTEM].clkdev);
	  mp200_pmu_open_clockgate(tm_reg[TIMER_WDT].clkdev);
	  mp200_pmu_open_clockgate(tm_reg[TIMER_DSP].clkdev);

	  for (i = 0; i <= TIMER_MAX ; i++) {
		if (tg_tm_op[i] == 1) {
		    mp200_pmu_open_clockgate(tm_reg[TIMER_TG0 + i].clkdev);
		}
	  }
	  break;

	case TIMER_INIT:
	  mp200_pmu_open_clockgate(MP200_CLOCKGATE_ATIM_PCLK);

	  mp200_pmu_open_clockgate(tm_reg[TIMER_SYSTEM].clkdev);
	  mp200_pmu_unreset_device(tm_reg[TIMER_SYSTEM].rstdev);
	  mp200_pmu_unreset_device(tm_reg[TIMER_TI2].rstdev);

	  for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
	    mp200_pmu_unreset_device(tm_reg[TIMER_TG0 + i].rstdev);
	  }

	  outl(TSTOP, tm_reg[TIMER_SYSTEM].tm_op);
	  outl(TSTOP, tm_reg[TIMER_WDT].tm_op);
	  outl(TSTOP, tm_reg[TIMER_DSP].tm_op);
	  outl(TCR_CLR, tm_reg[TIMER_SYSTEM].tm_clr);
	  outl(TCR_CLR, tm_reg[TIMER_WDT].tm_clr);
	  outl(TCR_CLR, tm_reg[TIMER_DSP].tm_clr);

	  outl(TSTOP, tm_reg[TIMER_TI2].tm_op);
	  outl(0xffffffff, tm_reg[TIMER_TI2].tm_set);

	  /* select TIN clock */
	  outl(TINTIN_SEL_PLL3 | TWNTIN_SEL_32K, SMU_TI0TIN_SEL);
	  outl(TINTIN_SEL_PLL3 | TWNTIN_SEL_PLL3, SMU_TI1TIN_SEL);
	  outl(TINTIN_SEL_PLL3 | TWNTIN_SEL_PLL3, SMU_TI2TIN_SEL);
	  outl(TINTIN_SEL_32K | TWNTIN_SEL_PLL3, SMU_TI3TIN_SEL);
	  outl(TGNTIN_SEL_PLL3, SMU_TGNTIN_SEL);

	  /* DIVTIMTIN Register set */
	  outl(SMU_DIV40, SMU_DIVTIMTIN);

	  /* set counter */
	  outl(COUNTER_MAXVAL, tm_reg[TIMER_SYSTEM].tm_set);
	  count = get_tw_count_value(TIMER_WDT, tm_param[TIMER_WDT].usecs);
	  outl(count, tm_reg[TIMER_WDT].tm_set);
	  outl(TIMER_INTERVAL_DSP, tm_reg[TIMER_DSP].tm_set);

	  for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
	    count = get_count_value(i, tm_param[TIMER_TG0 + i].usecs);
	    outl(count, tm_reg[TIMER_TG0 + i].tm_set);
	  }

	  /* Auto clock ON */
	  mp200_pmu_clkctrl_on(MP200_CLKCTRL_TIMPCLK);
	  break;
      default:
		  printk(KERN_INFO "%s(): set clock error mode = %d.\n",
		 __FUNCTION__, mode);
	  break;
	}
}


int mp200_timer_start(unsigned int timer)
{
	int ret=0;
	unsigned long flags;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}

	spin_lock_irqsave(&timer_spinlock,flags);

	mp200_pmu_open_clockgate(tm_reg[TIMER_TG0 + timer].clkdev);
	tg_tm_op[timer] = 1;

	/* Timer start */
	outl((TSTART | TM_EN | TO_EN), tm_reg[TIMER_TG0 + timer].tm_op);

	spin_unlock_irqrestore(&timer_spinlock,flags);

	return 0;
}

int mp200_timer_stop(unsigned int timer)
{
	int ret=0;
	unsigned int tm_delay=0;
	unsigned long flags;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}
	/* delay value set */
	tm_delay = TIMER_DELAY(TIMER_CLOCK_TICK_RATE_PLL3);

	spin_lock_irqsave(&timer_spinlock,flags);

	/* Timer stop */
	outl(TSTOP, tm_reg[TIMER_TG0 + timer].tm_op);

	udelay(tm_delay);

	mp200_pmu_close_clockgate(tm_reg[TIMER_TG0 + timer].clkdev);
	tg_tm_op[timer] = 0;

	spin_unlock_irqrestore(&timer_spinlock,flags);

	return 0;
}

int mp200_timer_set_period(unsigned int timer, unsigned int usecs)
{
	int ret=0, regval=0;
	unsigned long	flags;
	unsigned int tm_delay=0;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}

	if ((usecs < TIMER_MIN_USECS) || (TIMER_MAX_USECS < usecs)){
		return -EINVAL;
	}

	/* 4clk x2 wait value */
	tm_delay = TIMER_DELAY(TIMER_CLOCK_TICK_RATE_PLL3) * 2;

	spin_lock_irqsave(&timer_spinlock,flags);

	if (inl(tm_reg[timer].tm_op) & TSTART){
		spin_unlock_irqrestore(&timer_spinlock,flags);
		return -EBUSY;
	}

	mp200_pmu_open_clockgate(tm_reg[TIMER_TG0 + timer].clkdev);
	udelay(tm_delay);

	tm_param[TIMER_TG0 + timer].usecs = usecs;

	/* set counter */
	regval = get_count_value(timer, usecs);
	outl(regval, tm_reg[TIMER_TG0 + timer].tm_set);

	mp200_pmu_close_clockgate(tm_reg[TIMER_TG0 + timer].clkdev);
	spin_unlock_irqrestore(&timer_spinlock,flags);

	return 0;
}

int mp200_timer_get_period(unsigned int timer, unsigned int *usecs)
{
	int ret=0;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}

	if (usecs == NULL) {
		return -EINVAL;
	}

	*usecs = tm_param[TIMER_TG0 + timer].usecs;

	return 0;
}

int mp200_timer_get_usecs(unsigned int timer, unsigned int *usecs)
{
	unsigned int regval=0;
	int ret=0;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}

	if (usecs == NULL) {
		return -EINVAL;
	}

	regval = inl(tm_reg[TIMER_TG0 + timer].rcr);
	*usecs = get_usec_value(timer, regval);

	return 0;
}
int mp200_timer_register_cb(unsigned int timer, char *devname,
			irqreturn_t(*cb) (int, void *), void *dev_id)
{
	unsigned int irq=0;
	int ret=0;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}

	if (inl(tm_reg[timer].tm_op) & TSTART) {
		return -EBUSY;
	}

	irq = tm_reg[TIMER_TG0 + timer].irq;
	ret = request_irq(irq, cb, IRQF_SHARED | IRQF_DISABLED, devname, dev_id);
	if (ret != 0) {
		return ret;
	}

	return 0;
}

int mp200_timer_unregister_cb(unsigned int timer, void *dev_id)
{
	unsigned int irq=0;
	int ret=0;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}
	if (timer == DAC_HANG_TG_INTERDSP) {
	    timer = TIMER_TG4;
	}
	if(timer == DAC_HANG_TG){
	    timer = TIMER_TG5;
	}

	if (inl(tm_reg[timer].tm_op) & TSTART) {
		return -EBUSY;
	}

	irq = tm_reg[TIMER_TG0 + timer].irq;
	free_irq(irq, dev_id);

	return 0;
}

static spinlock_t wdt_spinlock = SPIN_LOCK_UNLOCKED;
/*
 * WatchDog Timer Function
 */
int mp200_wdt_set_timeout(unsigned int usecs)
{
	int ret = 0, val_tm_op, regval;
	unsigned int tm_delay=0;

	if ((usecs < TW_MIN_USECS) || (TW_MAX_USECS < usecs)) {
		return -EINVAL;
	}
	tm_delay = TIMER_DELAY(TW_CLOCK_TICK_RATE) *2;

	spin_lock(&wdt_spinlock);

	/* get tm_op */
	val_tm_op = inl(tm_reg[TIMER_WDT].tm_op);

	/* timer stop */
	outl(TSTOP, tm_reg[TIMER_WDT].tm_op);

	/* set counter */
	regval = get_tw_count_value(TIMER_WDT, usecs);

	udelay(tm_delay);
	outl(regval, tm_reg[TIMER_WDT].tm_set);

	/* timer start (restore tm_op) */
	outl(val_tm_op, tm_reg[TIMER_WDT].tm_op);

	spin_unlock(&wdt_spinlock);

	return ret;
}

/* Clear watchdog timer. */
void mp200_wdt_ping(void)
{
	unsigned int tm_delay = TIMER_DELAY(TW_CLOCK_TICK_RATE);

	spin_lock(&wdt_spinlock);
	outl(TCR_CLR, tm_reg[TIMER_WDT].tm_clr);
	udelay(tm_delay);
	spin_unlock(&wdt_spinlock);
}

/* Start watchdog timer counting. */
void mp200_wdt_enable(void)
{
	spin_lock(&wdt_spinlock);
	outl(TM_EN | TSTART | TO_EN, tm_reg[TIMER_WDT].tm_op);
	spin_unlock(&wdt_spinlock);
}

/* Stop watchdog timer counting. */
void mp200_wdt_disable(void)
{
	unsigned int tm_delay = TIMER_DELAY(TW_CLOCK_TICK_RATE);

	spin_lock(&wdt_spinlock);
	outl(TSTOP, tm_reg[TIMER_WDT].tm_op);
	udelay(tm_delay);
	spin_unlock(&wdt_spinlock);
}

/* Setup watchdog timer. */
void mp200_wdt_setup(void)
{
	unsigned int regval;
	unsigned int tm_delay=0;

	tm_delay = TIMER_DELAY(TW_CLOCK_TICK_RATE) *2;
	spin_lock(&wdt_spinlock);

	outl(inl(SMU_WDT_INT_RESET) | 0x00000101, SMU_WDT_INT_RESET);

	regval = inl(SMU_TI0TIN_SEL) & ~MASK_TWNTIN_SEL;
	outl(regval | TWNTIN_SEL_32K, SMU_TI0TIN_SEL);

	/* Auto clock OFF */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_ATIM_PCLK);
	mp200_pmu_open_clockgate(tm_reg[TIMER_WDT].clkdev);
	mp200_pmu_unreset_device(tm_reg[TIMER_WDT].rstdev);
	/* Auto clock ON */
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_TIMPCLK);

	/* stop timer for sure. */
	outl(TSTOP, tm_reg[TIMER_WDT].tm_op);

	/* set counter */
	regval = get_tw_count_value(TIMER_WDT, tm_param[TIMER_WDT].usecs);
	udelay(tm_delay);
	outl(regval, tm_reg[TIMER_WDT].tm_set);

	spin_unlock(&wdt_spinlock);

}

#ifdef CONFIG_SNSC_MP200_ACCURATE_SCHED_CLOCK
#define CYC2NS_SCALE_FACTOR 10

/*
 * To reduce error, better to make sure
 * TIMER_CLOCK_TICK_RATE_PLL3 % TICK_RATE_SCALE_FACTOR == 0
 */
#define TICK_RATE_SCALE_FACTOR 10240

/*
 * cycles to nsec conversions taken from arch/arm/mach-omap1/time.c,
 * convert from cycles(64bits) => nanoseconds (64bits)
 */
#define CYC2NS_SCALE (((NSEC_PER_SEC / TICK_RATE_SCALE_FACTOR) << CYC2NS_SCALE_FACTOR) \
                      / (TIMER_CLOCK_TICK_RATE_PLL3 / TICK_RATE_SCALE_FACTOR))

static inline unsigned long long notrace cycles_2_ns(unsigned long long cyc)
{
#if (CYC2NS_SCALE & 0x1)
	return (cyc * CYC2NS_SCALE << 1) >> (CYC2NS_SCALE_FACTOR + 1);
#else
	return (cyc * CYC2NS_SCALE) >> CYC2NS_SCALE_FACTOR;
#endif
}

unsigned long long notrace sched_clock(void)
{
	unsigned long long cyc;
	cyc = cnt32_to_63(mp200_system_timer_read(TIMER_SCHED_CLOCK));
	return cycles_2_ns(cyc);
}
#endif

EXPORT_SYMBOL(mp200_timer_start);
EXPORT_SYMBOL(mp200_timer_stop);
EXPORT_SYMBOL(mp200_timer_set_period);
EXPORT_SYMBOL(mp200_timer_get_period);
EXPORT_SYMBOL(mp200_timer_get_usecs);
EXPORT_SYMBOL(mp200_timer_register_cb);
EXPORT_SYMBOL(mp200_timer_unregister_cb);
EXPORT_SYMBOL(mp200_wdt_set_timeout);
EXPORT_SYMBOL(mp200_wdt_ping);
EXPORT_SYMBOL(mp200_wdt_enable);
EXPORT_SYMBOL(mp200_wdt_disable);
EXPORT_SYMBOL(mp200_wdt_setup);
