/* 2010-08-25: File added by Sony Corporation */
/*
 *  File Name       : linux/arch/arm/mach-mp200/uwire.c
 *  Function        : uwire
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/01/31
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/module.h>
#include <asm/hardware.h>
#include <asm/arch/uwire.h>
#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/io.h>
#include "uwire.h"

/*
 * Solution Board
 */
static int uwire_cs0_active_level = 1;
static int uwire_cs1_active_level = 0;

static spinlock_t lck = SPIN_LOCK_UNLOCKED;

static int uwire_init_flag = 1;

/* PLL3 229.376MHz : 237.568MHz */
static unsigned int mwi_plldiv_table[] = {
  SMU_DIV160,   /* 1500KHz(1434KHz : 1485KHz) */
  SMU_DIV80,    /* 3MHz   (2.87MHz : 2.97MHz) */
  SMU_DIV40,    /* 6MHz   (5.73MHz : 5.94MHz) */
  SMU_DIV20,    /* 12MHz  (11.47MHz: 11.88MHz) */
  SMU_DIV10,    /* 24MHz  (22.94MHz: 23.76MHz) */
  SMU_DIV6,     /* 48MHz  (38.23MHz: 39.59MHz) */
};

/*
 * SMU Control
 */

static inline void uwire_pclock_start(void)
{
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_PCLK);
}

static inline void uwire_pclock_stop(void)
{
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_PCLK);
}

static inline void uwire_sclock_start(void)
{
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_sclock_stop(void)
{
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_clock_start(void)
{
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_clock_stop(void)
{
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_dev_unreset(void)
{
	mp200_pmu_unreset_device(MP200_RESETDEVICE_MWI);
}
static inline void uwire_clkctrl_on(void)
{
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_MWIPCLK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_MWISCLK);
}

static inline void uwire_clkctrl_off(void)
{
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_MWIPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_MWISCLK);
}


/*
   setting frequency devider function
   argument is positive integer 1-256
*/

static inline void uwire_set_divisor(u_int sclk)
{

	switch(sclk){
	case UWIRE_SCLOCK_1500KHz:
	case UWIRE_SCLOCK_3MHz:
	case UWIRE_SCLOCK_6MHz:
	case UWIRE_SCLOCK_12MHz:
	case UWIRE_SCLOCK_24MHz:
	case UWIRE_SCLOCK_48MHz:
	  break;
	default:
	  return;
	}

	outl(mwi_plldiv_table[sclk], SMU_DIVMWISCLK);

}

/*
 * cs control
 */
static inline u_int sl_board_cs_fixup(int cs)
{
	if (cs == UWIRE_CS0) {
		return ((CS0_EN | (uwire_cs0_active_level ? CS0 : 0))
			| (CS1_EN | (uwire_cs1_active_level ? 0 : CS1)));
	}
	if (cs == UWIRE_CS1) {
		return ((CS0_EN | (uwire_cs0_active_level ? 0 : CS0))
			| (CS1_EN | (uwire_cs1_active_level ? CS1 : 0)));
	}
	return ((CS0_EN | (uwire_cs0_active_level ? 0 : CS0))
		| (CS1_EN | (uwire_cs1_active_level ? 0 : CS1)));
}

int uwire_set_active_level(int cs, int high)
{
	int org_level;

	/* uwire initialize error check */
	if (uwire_init_flag) {
		printk(KERN_INFO "uwire_set_active_level(): "
		       "uwire initialize error\n");
		return -EPERM;
	}

	if (cs == UWIRE_CS0) {
		org_level = uwire_cs0_active_level;
		uwire_cs0_active_level = high;
	} else if (cs == UWIRE_CS1) {
		org_level = uwire_cs1_active_level;
		uwire_cs1_active_level = high;
	} else {
		return -EINVAL;
	}
	return org_level;
}


/*
 * uWire I/O
 */

int uwire_vl_read(int cs, int sclk, u_char addr, u_int len, u_long * data)
{
	u_int uwire_tick;
	u_long cpu_flags;
	u_long mask;
	u_int org_sclk = ~0U;

	/* uwire initialize error check */
	if (uwire_init_flag) {
		printk(KERN_INFO "uwire_vl_read(): "
		       "uwire initialize error\n");
		return -EPERM;
	}
#ifdef UWIRE_DEBUG
	printk("uwire_vl_read(): CS%d read.\n", cs);
	printk("uwire_vl_read(): addr=0x%x\n", addr);
#endif

	/* Check CS Number */
	if ((UWIRE_CS0 != cs) && (UWIRE_CS1 != cs)) {
		printk(KERN_INFO "uwire_vl_read(): #CS(%d) is invalid number\n",
		       cs);
		return -EINVAL;
	}

	if ((len < 1) || (UWIRE_MAX_BIT_LEN < len)) {
		printk(KERN_INFO "uwire_vl_read(): data length invalid(%u)\n",
		       len);
		return -EINVAL;
	} else if (len == UWIRE_MAX_BIT_LEN) {
		mask = ~0UL;
	} else {
		mask = (1UL << len) - 1;
	}

	spin_lock_irqsave(&lck, cpu_flags);

	if (sclk != -1) {
		org_sclk = inl(SMU_DIVMWISCLK);
		uwire_set_divisor(sclk);
	}

	uwire_sclock_start();	/* SCLK start */

	/* clear */
	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	outl(sl_board_cs_fixup(-1), MWI_CONT);

	/* start transmit&receive */
	if (UWIRE_CS0 == cs) {
		outl(MWI_EDGE, MWI_CS0);
	} else {
		outl(MWI_EDGE, MWI_CS1);
	}

	outl(MWI_SEQUENTIAL_MODE | ((len - 1) << UWIRE_BIT_RX_SHIFT) |
	     sl_board_cs_fixup(cs), MWI_CONT);

	outl((addr << 24), MWI_TXQA);	/* MSB shift (31-24 <-- 7-0) */
	outl(MWI_START_FLAG, MWI_START);

	/* wait end transmit */
	uwire_tick = 0;
	while ((inl(MWI_INTRAW) & MWI_RAW) == 0) {
		if (++uwire_tick > UWIRE_MAX_WAIT) {
#ifdef UWIRE_DEBUG
			printk("uwire_vl_read(): raw=%x\n", inl(MWI_INTRAW));
#endif
			uwire_sclock_stop();
			outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */

			/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
			outl(sl_board_cs_fixup(-1), MWI_CONT);

			if (org_sclk != ~0U) {
				outl(org_sclk, SMU_DIVMWISCLK);
			}

			spin_unlock_irqrestore(&lck, cpu_flags);
			printk(KERN_INFO "uwire_vl_read(): busy\n");

			return -EBUSY;
		}
	}

	*data = inl(MWI_RXQ) & mask;

	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */
	uwire_sclock_stop();	/* SCLK stop */

	if (org_sclk != ~0U) {
	  outl(org_sclk, SMU_DIVMWISCLK);
	}

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
	outl(sl_board_cs_fixup(-1), MWI_CONT);

	spin_unlock_irqrestore(&lck, cpu_flags);

#ifdef UWIRE_DEBUG
	printk("uwire_vl_read(): CS%d read data=%x\n", cs, *data);
#endif
	return 0;
}

int uwire_read(int cs, u_char addr, u_char * data)
{
	u_long r_data;
	int ret;

	ret = uwire_vl_read(cs, UWIRE_SCLOCK_ANY, addr, 8, &r_data);
	if (ret == 0) {
		*data = (u_char) r_data;
	}
	return ret;
}

int uwire_vl_write(int cs, int sclk, u_char addr, u_int len, u_long data)
{
	u_int uwire_tick;
	u_long cpu_flags;
	u_int org_sclk = ~0U;

	/* uwire initialize error check */
	if (uwire_init_flag) {
		printk(KERN_INFO "uwire_vl_write(): "
		       "uwire initialize error\n");
		return -EPERM;
	}
#ifdef UWIRE_DEBUG
	printk("uwire_vl_write(): CS%d write.\n", cs);
	printk("uwire_vl_write(): addr=0x%x\n", addr);
#endif

	/* Check CS Number */
	if ((UWIRE_CS0 != cs) && (UWIRE_CS1 != cs)) {
		printk(KERN_INFO
		       "uwire_vl_write(): #CS(%d) is invalid number\n", cs);
		return -EINVAL;
	}

	if ((len < 1) || (UWIRE_MAX_BIT_LEN < len)) {
		printk(KERN_INFO "uwire_vl_write(): data length invalid(%u)\n",
		       len);
		return -EINVAL;
	}

	spin_lock_irqsave(&lck, cpu_flags);

	if (sclk != -1) {
		org_sclk = inl(SMU_DIVMWISCLK);
		uwire_set_divisor(sclk);
	}

	uwire_sclock_start();	/* SCLK start */

	/* clear */
	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	outl(sl_board_cs_fixup(-1), MWI_CONT);

	/* start transmit */
	if (UWIRE_CS0 == cs) {
		outl(MWI_EDGE, MWI_CS0);
	} else {
		outl(MWI_EDGE, MWI_CS1);
	}
	outl(MWI_PACKAGE_MODE | ((len - 1) << UWIRE_BIT_DTX_SHIFT) |
	     sl_board_cs_fixup(cs), MWI_CONT);
	outl((addr << 24), MWI_TXQA);	/* MSB shift (31-24 <-- 7-0) */
	outl((data << (32 - len)), MWI_TXQ);	/* MSB shift (31-24 <-- 7-0) */
	outl(MWI_START_FLAG, MWI_START);

	/* wait end transmit */
	uwire_tick = 0;
	while ((inl(MWI_INTRAW) & MWI_RAW) == 0) {
		if (++uwire_tick > UWIRE_MAX_WAIT) {
#ifdef UWIRE_DEBUG
			printk("uwire_vl_write(): raw=%x\n", inl(MWI_INTRAW));
#endif
			outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */
			uwire_sclock_stop();

			/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
			outl(sl_board_cs_fixup(-1), MWI_CONT);

			if (org_sclk != ~0U) {
				outl(org_sclk, SMU_DIVMWISCLK);
			}

			spin_unlock_irqrestore(&lck, cpu_flags);
			printk(KERN_INFO "uwire_vl_write(): busy\n");

			return -EBUSY;
		}
	}

	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */
	uwire_sclock_stop();	/* SCLK stop */

	if (org_sclk != ~0U) {
	  outl(org_sclk, SMU_DIVMWISCLK);
	}

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
	outl(sl_board_cs_fixup(-1), MWI_CONT);

	spin_unlock_irqrestore(&lck, cpu_flags);

	return 0;
}

int uwire_write(int cs, u_char addr, u_char data)
{
	int ret;

	ret = uwire_vl_write(cs, UWIRE_SCLOCK_ANY, addr, 8, data);

	return ret;
}

static int __init cs0_setup(char *sel)
{
	if (*sel == '0') {
		uwire_cs0_active_level = 0;
	} else {
		uwire_cs0_active_level = 1;
	}
	return 0;
}

static int __init cs1_setup(char *sel)
{
	if (*sel == '0') {
		uwire_cs1_active_level = 0;
	} else {
		uwire_cs1_active_level = 1;
	}
	return 0;
}

__setup("uwire_cs0=", cs0_setup);
__setup("uwire_cs1=", cs1_setup);

static int __init uwire_init(void)
{
	u_long flags;

	printk(KERN_INFO "Starting uwire.\n");

	spin_lock_irqsave(&lck, flags);
	uwire_clock_stop();	/* PCLK/SCLK stop */

/*
    determin clock and set a devisor at valid value
*/

	uwire_set_divisor(UWIRE_SCLOCK_3MHz);


	/* PCLK stop is possible, but we get going that just in case */
	uwire_clock_start();	/* CLK start */
	uwire_clkctrl_off();	/* AUTO CLK OFF */
	uwire_dev_unreset();	/* UNRESET */
	uwire_clkctrl_on();	/* AUTO CLK ON */
	spin_unlock_irqrestore(&lck, flags);

	uwire_init_flag = 0;

#ifdef UWIRE_DEBUG
	{
		u_int cs0, cs1, non;
		cs0 = sl_board_cs_fixup(UWIRE_CS0);
		cs1 = sl_board_cs_fixup(UWIRE_CS1);
		non = sl_board_cs_fixup(-1);
		printk(KERN_INFO "uWire: CS0=0x%02x, CS1=0x%02x, INI=0x%02x\n",
		       cs0, cs1, non);
	}
#endif

	return 0;
}


static void __exit uwire_exit(void)
{
	u_long flags;

	spin_lock_irqsave(&lck, flags);
	uwire_clock_stop();
	spin_unlock_irqrestore(&lck, flags);

	uwire_init_flag = 1;

	return;
}

arch_initcall(uwire_init);
module_exit(uwire_exit);

EXPORT_SYMBOL(uwire_read);
EXPORT_SYMBOL(uwire_write);
EXPORT_SYMBOL(uwire_set_active_level);
EXPORT_SYMBOL(uwire_vl_read);
EXPORT_SYMBOL(uwire_vl_write);
