/* 2010-08-25: File added by Sony Corporation */
/*
 * arch/arm/plat-omap/brd_dsi.c
 *
 * Copyright (C) 2008 Texas Instruments.
 *
 * This file is licensed under the terms of the GNU General Public License 
 * version 2. This program is licensed "as is" without any warranty of any 
 * kind, whether express or implied
 *   
 * This file contains platform dependent low level functions for OMAP DSI 
 * interface on reference board
 *  
 */


/*=======INCLUDES======================================================*/

// #include "math.h"
// #include "graphics.h"
#include "dsi.h"
#include "brd_dsi.h"
// #include "triton2_utils.h"
// #include "padconfig.h"
#include <linux/module.h>
#include <linux/delay.h>
//#define NB_DATALANES_1
#define QVGA_IMAGE
//#define RGB18_IMAGE

/*==== DECLARATION CONTROL ==================================================*/
//static S32 enable_dsi_power(void);
//static S32 disable_dsi_power(void);
//static void dsi_pad_config(void);
void update_timing_parameters(U32 ddr_clk,U32 num_data_lanes,U32 bits_per_pixel);

/* DISPC timings */
U8 dispc_hfp;
U8 dispc_hsa;
U8 dispc_hbp;

/* DSI Video Mode timings */
U8 vid_hfp;
U8 vid_hsa;
U8 vid_hbp;
U8 vid_vfp;
U8 vid_vsa;
U8 vid_vbp;
U8 vid_hs;
U8 vid_lp_clk_div;
U8 window_sync;

U16 vid_wc;
U16 vid_tl;
U16 vid_vact;
U16 enter_hs_mode_latency;
U16 exit_hs_mode_latency;

/* D-PHY timings */
U16 ttag0;
U16 ttasure;
U16 ttaget;
U16 tlpxdiv2;
U16 tclktrail;
U16 tclkzero;
U16 thsp;
U16 thspz;
U16 thse;
U16 tclkp;
U16 tlpx;
U16 tclk_pre;
U16 tclk_post;
U16 gddr_clk_pre;
U16 gddr_clk_post;
U16 thst;

#define DIVROUNDUP(A,B) (((A) % (B) == 0) ? ((A) / (B)) : (((A) / (B)) + 1))

#define MARGE 3
#define FOR_EDISCO_ES1 32


/*============ FUNCTIONS =====================================================*/

/*-----------------------------------------------------------------------------
| Function    : S32 dsi_platform_init(T_DSI_DIS *dsi_conf) 
+------------------------------------------------------------------------------
| Description : This function does the platform specific initialization required
|               on the SDP platform
|
| Parameters  : dsi_conf - pointer to the DSI interface information structure
|
| Returns     : Status (CSST_DAL_SUCCESS or Error)
+-----------------------------------------------------------------------------*/
S32 dsi_platform_init(T_DSI_DIS *dsi_conf)
{	
	S32 ret=OMAP_DAL_SUCCESS;

#if 0
	/* Enable power to the DSI interface of OMAP(VDDS_DSI) */
	ret = enable_dsi_power();
	if(ret != OMAP_DAL_SUCCESS)
	{
		return ret;	
	}
	/* mux setup for the DSI interface */	
	dsi_pad_config();
	/* mux setup for LCD's backlight */
	mux_setup_lcd_bk_vdd();

	ret = enable_lcd_power();
	if(ret != OMAP_DAL_SUCCESS)
	{
    	dbg_print(DBG_LEVEL_CRITICAL,OMAP_DRV_ID,"LCD power enable failed \n\r");
		return ret;
	}

	ret = enable_lcd_backlight();
	if(ret != OMAP_DAL_SUCCESS)
	{
    	dbg_print(DBG_LEVEL_CRITICAL,OMAP_DRV_ID,"LCD backlight enable failed \n\r");
		return ret;
	}
#endif
	return ret;
}

/*-----------------------------------------------------------------------------
| Function    : S32 dsi_platform_deinit(T_DSI_DIS *dsi_conf) 
+------------------------------------------------------------------------------
| Description : This function does the platform specific deinit required
|               on the SDP platform
|
| Parameters  : dsi_conf - pointer to the DSI interface information structure
|
| Returns     : Status (OMAP_DAL_SUCCESS or Error)
+-----------------------------------------------------------------------------*/
S32 dsi_platform_deinit(T_DSI_DIS *dsi_conf)
{	
	S32 ret = OMAP_DAL_SUCCESS;

#if 0
	/* Disable the power to the DSI interface of OMAP */
	ret = disable_dsi_power();
	if(ret != OMAP_DAL_SUCCESS)
	{
		return ret;	
	}
	ret = disable_lcd_backlight();
	if(ret != OMAP_DAL_SUCCESS)
	{
    	dbg_print(DBG_LEVEL_CRITICAL,OMAP_DRV_ID,"LCD backlight disable failed \n\r");
		return ret;
	}

	ret = disable_lcd_power();
	if(ret != OMAP_DAL_SUCCESS)
	{
	   	dbg_print(DBG_LEVEL_CRITICAL,OMAP_DRV_ID,"LCD power disable failed \n\r");
		return ret;
	}
#endif	
	return ret;
}

/*-----------------------------------------------------------------------------
| Function    : S32 enable_dsi_power() 
+------------------------------------------------------------------------------
| Description : This function enable the power to DSI module of OMAP
|
| Parameters  : None
|
| Returns     : Status (OMAP_DAL_SUCCESS or Error)
+-----------------------------------------------------------------------------*/
/*S32 enable_dsi_power(void)
{
	S32 ret = OMAP_DAL_SUCCESS;

#if 0
#ifdef SDP3430V1
	return ret;
#else
	
	ret = enable_triton2_vpll2(VPLL2_V1P8);
	return ret;
#endif
#endif
	return ret;
} */

/*-----------------------------------------------------------------------------
| Function    : S32 disable_dsi_power() 
+------------------------------------------------------------------------------
| Description : This function disable the power to DSI module of OMAP
|
| Parameters  : None
|
| Returns     : Status (OMAP_DAL_SUCCESS or Error)
+-----------------------------------------------------------------------------*/
/*S32 disable_dsi_power(void)
{
	S32 ret = OMAP_DAL_SUCCESS;
#if 0
#ifdef SDP3430V1
	return ret; 
#else
	ret = disable_triton2_vpll2();
	return ret;
#endif
#endif
	return ret;
}*/

/*-----------------------------------------------------------------------------
| Function    : S32 dsi_pad_config() 
+------------------------------------------------------------------------------
| Description : This function does the PAD configuration of DSI interface 
|
| Parameters  : None
|
| Returns     : None
+-----------------------------------------------------------------------------*/
/*void dsi_pad_config(void)
{
#if 0
		// Configure the DSI interface PAD.
	  //Put all the other DSS interface PAD to safe mode 
	enable_dsi_pinmux();
#endif
} */

/*---------------------------------------------------------------------------------------------------------------------
| Function    : void get_dphy_config(T_DPHY_CONFIG *complexio_config, U8 num_data_lanes, U16 pixel_clk, U16 pixel_format) 
+-----------------------------------------------------------------------------------------------------------------------
| Description : This function return the DPHY configuration required on SDP3430 platform 
|
| Parameters  : complexio_config - pointer to the ComplexIO config structure
|		num_data_lanes	 - number of data lanes
|		pixel_clk	 - pixel clock frequency
|		pixel_format	 - pixel format
| Returns     : None
+--------------------------------------------------------------------------------------------------------------------*/
void get_dphy_config(T_DPHY_CONFIG *complexio_config, U8 num_data_lanes, U16 pixel_clk, U16 pixel_format)
{
	U16 data_rate,ddr_clk ;
	F32 ddr_clk_period;
	U8 bits_per_pixel = 0;

	/* Number of bits in each format, can be a macro */
	if(pixel_format == RGB565)
	{
		bits_per_pixel = 16;
	}
	else if (pixel_format == RGB666)
	{
		bits_per_pixel = 18;
	}
	else if (pixel_format == RGB888)
	{
		bits_per_pixel = 24;
	}

	/* Calculate DDR_CLK frequency */
	ddr_clk = get_ddr_clk_val(pixel_clk,
				  bits_per_pixel,
				  num_data_lanes);

	update_timing_parameters(ddr_clk,num_data_lanes,bits_per_pixel);
	/* Findout the DDR_CLK period */
	ddr_clk_period = (1000/ddr_clk);
	data_rate = pixel_clk * bits_per_pixel;

	complexio_config->data_rate = data_rate;
	
	/* Configure the Lane position and Polariy */
	/*TODO: cross check the lane position values */
	complexio_config->clk_lane.pos = LANE_POS_1;
	complexio_config->clk_lane.pol = LANE_POL_XP_YN;
	complexio_config->data_lane[0].pos = LANE_POS_2;
	complexio_config->data_lane[0].pol = LANE_POL_XP_YN;

	if(num_data_lanes == 2)
	{
		complexio_config->data_lane[1].pos = LANE_POS_3;
		complexio_config->data_lane[1].pol = LANE_POL_XP_YN;
	}
	else if (num_data_lanes == 1)
	{
		/* in this case, LANE2 configuration is initialized to zero */
		complexio_config->data_lane[1].pos = 0;
		complexio_config->data_lane[1].pol = 0;
	}

	/* Fill DSI protocol timing parameters */
	if(ddr_clk != 150)
	{
//		complexio_config->ths_prepare = ceil(59/ddr_clk_period) + 2; 
//		complexio_config->ths_prepare_ths_zero = ceil(137/ddr_clk_period) + 5; 
//		complexio_config->ths_trail = ceil(60/ddr_clk_period) + 2 + 70; // 30
//		complexio_config->ths_exit = ceil(100/ddr_clk_period); 
//		complexio_config->tlpx_half = 3; //ceil(25/ddr_clk_period); 
//		complexio_config->tclk_trail = ceil(60/ddr_clk_period); 
//		complexio_config->tclk_zero = ceil(292/ddr_clk_period);
//		complexio_config->tclk_prepare = ceil(65/ddr_clk_period);

		complexio_config->ths_prepare = 8; 
		complexio_config->ths_prepare_ths_zero = 25; 
		complexio_config->ths_trail = 35; // 30
		complexio_config->ths_exit = 11; 
		complexio_config->tlpx_half = 3; //ceil(25/ddr_clk_period); 
		complexio_config->tclk_trail = 7; 
		complexio_config->tclk_zero = 23;
		complexio_config->tclk_prepare = 7;
		tclk_pre = 9;
		tclk_post = 59;
		gddr_clk_pre = (int)(tclk_pre + complexio_config->tlpx_half*2 + complexio_config->tclk_zero + complexio_config->tclk_prepare )/4;
		gddr_clk_post = (int)(tclk_post + complexio_config->tclk_trail)/4;
		enter_hs_mode_latency = 1 + DIVROUNDUP (2 * complexio_config->tlpx_half, 4) + DIVROUNDUP (complexio_config->ths_prepare ,4) + DIVROUNDUP (complexio_config->ths_prepare_ths_zero - complexio_config->ths_prepare + 3, 4);
		exit_hs_mode_latency = ((complexio_config->ths_trail + complexio_config->ths_exit)/4) + 1;

	}
	else
	{
		complexio_config->ths_prepare = 11; 
		complexio_config->ths_prepare_ths_zero = 27; 
		complexio_config->ths_trail = 38; // 30
		complexio_config->ths_exit = 16; 
		complexio_config->tlpx_half = 4; //ceil(25/ddr_clk_period); 
		complexio_config->tclk_trail = 10; 
		complexio_config->tclk_zero = 35;
		complexio_config->tclk_prepare = 10;
		vid_lp_clk_div = 8; 
	
	}
	/* Calculation for eDISCO */
//	complexio_config->ths_prepare = ceil(59/ddr_clk_period) + 5;
//	complexio_config->ths_prepare_ths_zero = ceil(137/ddr_clk_period) + 10;
//	complexio_config->ths_trail = ceil(60/ddr_clk_period) + 4 + FOR_EDISCO_ES1;
//	complexio_config->ths_exit = ceil(100/ddr_clk_period)+ MARGE;
//	complexio_config->tlpx_half = ceil(25/ddr_clk_period); 
//	complexio_config->tclk_trail = ceil(60/ddr_clk_period)+ MARGE;
//	complexio_config->tclk_zero = ceil(390/ddr_clk_period)+ MARGE;
//	complexio_config->tclk_prepare = ceil(67/ddr_clk_period);

//	enter_hs_mode_latency = 2 * complexio_config->tlpx_half + complexio_config->ths_prepare;
//	exit_hs_mode_latency = complexio_config->tclk_trail + complexio_config->ths_exit;
//	enter_hs_mode_latency = 1 + DIVROUNDUP (2 * complexio_config->tlpx_half, 4) + DIVROUNDUP (complexio_config->ths_prepare ,4) + DIVROUNDUP (complexio_config->ths_prepare_ths_zero - complexio_config->ths_prepare + 3, 4);
//	exit_hs_mode_latency = DIVROUNDUP((complexio_config->ths_trail + complexio_config->ths_exit) ,4) + 1;


}

/* Following settings taken from SilVal team */

void update_timing_parameters(U32 ddr_clk,U32 num_data_lanes,U32 bits_per_pixel)
{
//	U8 marge = 3;
	//U16 mthst1, mthst2, mthst;
#if 1
		dispc_hfp = 20;
	dispc_hsa = 10;
	dispc_hbp = 20;
#else
	dispc_hfp = 20;
	dispc_hsa = 1;
	dispc_hbp = 27;
#endif
			
#ifdef QVGA_IMAGE

#ifdef NB_DATALANES_1
	vid_hfp = 38; 
	vid_hsa = 0; 
	vid_hbp = 52; 
	vid_tl = 580; 
#else //NB_DATALANES_1

#ifdef RGB18_IMAGE
	vid_hfp = 22; 
	vid_hsa = 0; 
	vid_hbp = 59; 
	vid_tl = 326; 
#else //RGB18_IMAGE
	vid_hfp = 19; 
	vid_hsa = 0; 
	vid_hbp = 26; 
	vid_tl = 290; 
#endif //RGB18_IMAGE

#endif // NB_DATALANES_1

#else // QVGA_IMAGE

#ifdef NB_DATALANES_1
	vid_hfp = 38; 
	vid_hsa = 0; 
	vid_hbp = 52; 
	vid_tl = 1060; 
#else // NB_DATALANES_1
	vid_hfp = 19; 
	vid_hsa = 0; 
	vid_hbp = 26; 
	vid_tl = 530; 
#endif // NB_DATALANES_1


#endif // QVGA_IMAGE

	vid_vfp = 1;
	vid_vsa = 1;
	vid_vbp = 1;

	vid_hs = 4/num_data_lanes;
	vid_lp_clk_div = 8; 
	window_sync = 4;

	vid_wc = 480 * 2;
//	vid_tl = vid_hs + dispc_hbp + ((vid_wc + 6) / num_data_lanes) + vid_hfp;
//	vid_tl = vid_hs + vid_hsa + vid_hs + vid_hbp + ((vid_wc + 6) / num_data_lanes) + vid_hfp;
//	vid_tl = 530; //1060 
	vid_vact = 640;




#if 0
	thsp = DIVROUNDUP ((((40+4*(500/ddr_clk))+(85+6*(500/ddr_clk)))/2),1000/ddr_clk) + marge;  
	thspz = DIVROUNDUP ((145+10*(500/ddr_clk)),1000/ddr_clk) + marge;
	thse = DIVROUNDUP (100,1000/ddr_clk) + marge;
	tclkp = DIVROUNDUP (60,1000/ddr_clk) + marge;
	tlpx = DIVROUNDUP (50,1000/ddr_clk) + marge;
//	tclk_pre = DIVROUNDUP (8,1000/ddr_clk) + marge;
	tclk_pre = DIVROUNDUP ((8*(500/ddr_clk)),1000/ddr_clk) + marge;
	tclk_post = DIVROUNDUP ((60 + (52*(500/ddr_clk))),1000/ddr_clk) + marge;
	tclktrail = DIVROUNDUP (60,1000/ddr_clk) +  marge;
//	tclkzero = DIVROUNDUP (300,1000/ddr_clk) - thsp;
	tclkzero = DIVROUNDUP (150,1000/ddr_clk);

	gddr_clk_pre = (int)(tclk_pre + tlpx + tclkzero + tclkp )/4;
	gddr_clk_post = (int)(tclk_post + tclktrail)/4;
	
	mthst1     = DIVROUNDUP ((1*8*(500/ddr_clk)),1000/ddr_clk) ;	   
	mthst2     = DIVROUNDUP ((60+ 4*(500/ddr_clk)),1000/ddr_clk) ;	  

	mthst        = mthst2;          

	if (mthst1 > mthst2)
	{
	    mthst     = mthst1;
	}

	thst = mthst + marge + 70; //mthst + marge;

	ttag0 = 2;
	ttasure = 0;
	ttaget = 2;
	tlpxdiv2 = DIVROUNDUP (50,1000/ddr_clk)/2 + marge ;
#endif
//	enter_hs_mode_latency = 1 + DIVROUNDUP (2 * tlpxdiv2, 4) + DIVROUNDUP (thsp ,4) + DIVROUNDUP (thspz - thsp + 3, 4);
//	exit_hs_mode_latency = DIVROUNDUP((thst + thse + marge) ,4) ;
}
