/* 2010-08-25: File added by Sony Corporation */
/*!
 * @file
 * @brief functions to control HW-IP
 * @author NIS
 * @date 2007/9/14
 */

#include <linux/string.h> // memset
#include <linux/stddef.h> // NULL,offsetof
#include <asm/delay.h> // udelay

#include "avc_lib.h"
#include "avc_info.h"

#define MAX_WORK_NUM_BIT 0x001F
#define MEM_NUM_WR_BIT   0x1F00

/* read decoded parameters --------------------------------------------------*/

static int get_header_size(void)
{
    return readw(&avc_info->regs->head_PRreg14);
}

#define TRY_GET_HEADER_DATA int _exception = 0
// <->CATCH_GET_HEADER_DATA:

#define get_header_data(_hsize) \
 _get_header_data(_hsize,&_exception); \
 if (_exception) goto CATCH_GET_HEADER_DATA

#define get_header_data32(_hsize) \
 _get_header_data32(_hsize,&_exception); \
 if (_exception) goto CATCH_GET_HEADER_DATA

static unsigned short _get_header_data(int *hsize, int *ex)
{
    if (*hsize > 0) {
        *hsize -= 1;
        return readw(&avc_info->regs->head_PRreg15);
    }

    *ex = 1;
    return 0;
}

static unsigned int _get_header_data32(int *hsize, int *ex)
{
    unsigned int ret;

    if (*hsize >= 2) {
        *hsize -= 2;
        ret  = readw(&avc_info->regs->head_PRreg15) << 16;
        ret |= readw(&avc_info->regs->head_PRreg15);
        return ret;
    }

    *hsize = 0;
    *ex = 1;
    return 0;
}

#define debug_level_check (debug_level < 2)

#define _param_log(_st,_name) \
 debug2_np(" " #_name " 0x%X %u",_st->_name,_st->_name)

#define _param_array_log(_st,_name,_i) \
 debug2_np(" " #_name "[%d] 0x%X %u",_i,_st->_name[_i],_st->_name[_i])

/*
 * HRD (Hypothetical Reference Decoder)
 */

#define hrd_log(_name) _param_log(hrd,_name)
#define hrd_array_log(_name,_i) _param_array_log(hrd,_name,_i)

void show_hrd(struct hrd_parameters_t *hrd)
{
    int i;
    if (debug_level_check) return;
    debug2("HRD (Hypothetical Reference Decoder)");
    hrd_log(cpb_cnt_minus1);
    hrd_log(bit_rate_scale);
    hrd_log(cpb_size_scale);
    for (i=0; i<hrd->cpb_cnt_minus1 && i<max_cpb_cnt; i++) {
        hrd_array_log(bit_rate_value_minus1,i);
        hrd_array_log(cpb_size_value_minus1,i);
        hrd_array_log(cbr_flag,i);
    }
    hrd_log(initial_cpb_removal_delay_length_minus1);
    hrd_log(cpb_removal_delay_length_minus1);
    hrd_log(dpb_output_delay_length_minus1);
    hrd_log(time_offset_length);
}

void read_hrd(struct hrd_parameters_t *hrd, int *hsize)
{
    int i;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    hrd->cpb_cnt_minus1 = get_header_data(hsize);
    if (hrd->cpb_cnt_minus1 >= max_cpb_cnt) {
        // it cannot be
        debug0("error cpb_cnt_minus1 %u >= max_cpb_cnt %u",
               hrd->cpb_cnt_minus1, max_cpb_cnt);
        return;
    }

    v = get_header_data(hsize);
    hrd->bit_rate_scale = bits(v,7,4);
    hrd->cpb_size_scale = bits(v,3,0);

    for (i=0; i<=hrd->cpb_cnt_minus1; i++) {
        hrd->bit_rate_value_minus1[i] = get_header_data32(hsize);
        hrd->cpb_size_value_minus1[i] = get_header_data32(hsize);
        hrd->cbr_flag[i] = get_header_data(hsize);
    }

    hrd->initial_cpb_removal_delay_length_minus1 = get_header_data(hsize);
    v = get_header_data(hsize);
    hrd->cpb_removal_delay_length_minus1 = bits(v,14,10);
    hrd->dpb_output_delay_length_minus1  = bits(v,9,5);
    hrd->time_offset_length = bits(v,4,0);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * VUI (Video Usability Information)
 */

#define vui_log(_name) _param_log(vui,_name)

void show_vui(struct vui_parameters_t *vui)
{
    if (debug_level_check) return;
    debug2("VUI (Video Usability Information)");
    vui_log(aspect_ratio_info_present_flag);
    vui_log(aspect_ratio_idc);
    vui_log(sar_width);
    vui_log(sar_height);
    vui_log(overscan_info_present_flag);
    vui_log(overscan_appropriate_flag);
    vui_log(video_signal_type_present_flag);
    vui_log(video_format);
    vui_log(video_full_range_flag);
    vui_log(colour_description_present_flag);
    vui_log(colour_primaries);
    vui_log(transfer_characteristics);
    vui_log(matrix_coefficients);
    vui_log(chroma_loc_info_present_flag);
    vui_log(chroma_sample_loc_type_top_field);
    vui_log(chroma_sample_loc_type_bottom_field);
    vui_log(timing_info_present_flag);
    vui_log(num_units_in_tick);
    vui_log(time_scale);
    vui_log(fixed_frame_rate_flag);
    vui_log(low_delay_hrd_flag);
    vui_log(pic_struct_present_flag);
    vui_log(bitstream_restriction_flag);
    vui_log(motion_vectors_over_pic_boundaries_flag);
    vui_log(max_bytes_per_pic_denom);
    vui_log(max_bits_per_mb_denom);
    vui_log(log2_max_mv_length_horizontal);
    vui_log(log2_max_mv_length_vertical);
    vui_log(num_reorder_frames);
    vui_log(max_dec_frame_buffering);

    vui_log(nal_hrd_parameters_present_flag);
    vui_log(vcl_hrd_parameters_present_flag);

    if (vui->nal_hrd_parameters_present_flag) {
        debug2("HRD NAL");
        show_hrd(&vui->nal_hrd_parameters);
    }
    if (vui->vcl_hrd_parameters_present_flag) {
        debug2("HRD VCL");
        show_hrd(&vui->vcl_hrd_parameters);
    }
}

void read_vui(struct vui_parameters_t *vui, int *hsize)
{
    unsigned short v;
    TRY_GET_HEADER_DATA;

    vui->aspect_ratio_info_present_flag = get_header_data(hsize);
    if (vui->aspect_ratio_info_present_flag) {
        vui->aspect_ratio_idc = get_header_data(hsize);
        if (vui->aspect_ratio_idc == Extended_SAR) {
            vui->sar_width  = get_header_data(hsize);
            vui->sar_height = get_header_data(hsize);
        }
    }

    vui->overscan_info_present_flag = get_header_data(hsize);
    if (vui->overscan_info_present_flag) {
        vui->overscan_appropriate_flag = get_header_data(hsize);
    }

    vui->video_signal_type_present_flag = get_header_data(hsize);
    if (vui->video_signal_type_present_flag) {
        v = get_header_data(hsize);
        vui->video_format = bits(v,4,2);
        vui->video_full_range_flag = bits(v,1,1);
        vui->colour_description_present_flag = bits(v,0,0);
        if (vui->colour_description_present_flag) {
            vui->colour_primaries = get_header_data(hsize);
            v = get_header_data(hsize);
            vui->transfer_characteristics = bits(v,15,8);
            vui->matrix_coefficients      = bits(v, 7,0);
        }
    }

    vui->chroma_loc_info_present_flag = get_header_data(hsize);
    if (vui->chroma_loc_info_present_flag) {
        vui->chroma_sample_loc_type_top_field = get_header_data(hsize);
        vui->chroma_sample_loc_type_bottom_field = get_header_data(hsize);
    }

    vui->timing_info_present_flag = get_header_data(hsize);
    if (vui->timing_info_present_flag) {
        vui->num_units_in_tick = get_header_data32(hsize);
        vui->time_scale        = get_header_data32(hsize);
        vui->fixed_frame_rate_flag = get_header_data(hsize);
    }

    vui->nal_hrd_parameters_present_flag = get_header_data(hsize);
    if (vui->nal_hrd_parameters_present_flag) {
        read_hrd(&vui->nal_hrd_parameters, hsize);
    }

    vui->vcl_hrd_parameters_present_flag = get_header_data(hsize);
    if (vui->vcl_hrd_parameters_present_flag) {
        read_hrd(&vui->vcl_hrd_parameters, hsize);
    }

    if (vui->nal_hrd_parameters_present_flag ||
        vui->vcl_hrd_parameters_present_flag) {
        vui->low_delay_hrd_flag = get_header_data(hsize);
    }

    v = get_header_data(hsize);
    vui->pic_struct_present_flag    = bits(v,1,1);
    vui->bitstream_restriction_flag = bits(v,0,0);

    if (vui->bitstream_restriction_flag) {
        vui->motion_vectors_over_pic_boundaries_flag = get_header_data(hsize);
        vui->max_bytes_per_pic_denom = get_header_data(hsize);
        vui->max_bits_per_mb_denom = get_header_data(hsize);
        vui->log2_max_mv_length_horizontal = get_header_data(hsize);
        vui->log2_max_mv_length_vertical = get_header_data(hsize);
        vui->num_reorder_frames = get_header_data(hsize);
        vui->max_dec_frame_buffering = get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SPS (Sequence Parameter Set)
 */

#define sps_log(_name) _param_log(sps,_name)
#define sps_array_log(_name,_i) _param_array_log(sps,_name,_i)

void show_sps(struct seq_parameter_set_rbsp_t *sps)
{
    int i;
    int max;

    if (debug_level_check) return;
    debug2("SPS (Sequence Parameter Set)");
    sps_log(profile_idc);
    switch (sps->profile_idc) {
    case 66: debug2_np("  Baseline profile"); break;
    case 77: debug2_np("  Main profile"); break;
    case 88: debug2_np("  Extended profile"); break;
    default: debug2_np("  error!!"); break;
    }
    sps_log(constraint_set0_flag);
    sps_log(constraint_set1_flag);
    sps_log(constraint_set2_flag);
    //sps_log(reserved_zero_5bits);
    sps_log(level_idc);
    sps_log(seq_parameter_set_id);
    sps_log(log2_max_frame_num_minus4);
    sps_log(pic_order_cnt_type);
    sps_log(log2_max_pic_order_cnt_lsb_minus4);
    sps_log(delta_pic_order_always_zero_flag);
    sps_log(offset_for_non_ref_pic);
    sps_log(offset_for_top_to_bottom_field);
    sps_log(num_ref_frames_in_pic_order_nct_cycle);

    max = sps->num_ref_frames_in_pic_order_nct_cycle;
    if (max > max_num_ref_frames_in_pic_order_nct_cycle) {
        max = max_num_ref_frames_in_pic_order_nct_cycle;
    }
    for (i=0; i<max; i++) {
        sps_array_log(offset_for_ref_frame,i);
    }

    sps_log(num_ref_frames);
    sps_log(gaps_in_frame_num_value_allowed_flag);
    sps_log(pic_width_in_mbs_minus1);
    sps_log(pic_height_in_maps_units_minus1);
    sps_log(frame_mbs_only_flag);
    sps_log(mb_adaptive_frame_field_flag);
    sps_log(direct_8x8_inference_flag);
    sps_log(frame_cropping_flag);
    sps_log(frame_crop_left_offset);
    sps_log(frame_crop_right_offset);
    sps_log(frame_crop_top_offset);
    sps_log(frame_crop_bottom_offset);
    sps_log(vui_parameters_present_flag);
    if (sps->vui_parameters_present_flag) {
        show_vui(&sps->vui_parameters);
    }
}

void read_sps(struct seq_parameter_set_rbsp_t *sps, int *hsize)
{
    int i;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sps,0,sizeof(*sps));

    sps->profile_idc = get_header_data(hsize);

    v = get_header_data(hsize);
    sps->constraint_set0_flag = bits(v,15,15);
    sps->constraint_set1_flag = bits(v,14,14);
    sps->constraint_set2_flag = bits(v,13,13);
    sps->level_idc = bits(v,7,0);

    sps->seq_parameter_set_id = get_header_data(hsize);
    sps->log2_max_frame_num_minus4 = get_header_data(hsize);
    sps->pic_order_cnt_type = get_header_data(hsize);

    if (sps->pic_order_cnt_type == 0) {
        sps->log2_max_pic_order_cnt_lsb_minus4 = get_header_data(hsize);
    } else if (sps->pic_order_cnt_type == 1) {
        sps->delta_pic_order_always_zero_flag = get_header_data(hsize);
        sps->offset_for_non_ref_pic = get_header_data32(hsize);
        sps->offset_for_top_to_bottom_field = get_header_data32(hsize);
        sps->num_ref_frames_in_pic_order_nct_cycle = get_header_data(hsize);
        if (sps->num_ref_frames_in_pic_order_nct_cycle >=
            max_num_ref_frames_in_pic_order_nct_cycle) {
            // it cannot be
            debug0("error num_ref_frames_in_pic_order_nct_cycle %u >= max %u",
                   sps->num_ref_frames_in_pic_order_nct_cycle,
                   max_num_ref_frames_in_pic_order_nct_cycle);
            return;
        }

        for (i=0;i<sps->num_ref_frames_in_pic_order_nct_cycle;i++) {
            sps->offset_for_ref_frame[i] = get_header_data32(hsize);
        }
    }

    sps->num_ref_frames = get_header_data(hsize);
    sps->gaps_in_frame_num_value_allowed_flag = get_header_data(hsize);
    sps->pic_width_in_mbs_minus1 = get_header_data(hsize);
    sps->pic_height_in_maps_units_minus1 = get_header_data(hsize);
    sps->frame_mbs_only_flag = get_header_data(hsize);

    if (!sps->frame_mbs_only_flag) {
        sps->mb_adaptive_frame_field_flag = get_header_data(hsize);
    }

    v = get_header_data(hsize);
    sps->direct_8x8_inference_flag = bits(v,1,1);
    sps->frame_cropping_flag = bits(v,0,0);

    if (sps->frame_cropping_flag) {
        sps->frame_crop_left_offset  = get_header_data(hsize);
        sps->frame_crop_right_offset = get_header_data(hsize);
        sps->frame_crop_top_offset   = get_header_data(hsize);
        sps->frame_crop_bottom_offset= get_header_data(hsize);
    }

    sps->vui_parameters_present_flag = get_header_data(hsize);
    if (sps->vui_parameters_present_flag) {
        read_vui(&sps->vui_parameters, hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * PPS (Picture Parameter Set)
 */

#define pps_log(_name) _param_log(pps,_name)
#define pps_array_log(_name,_i) _param_array_log(pps,_name,_i)

void show_pps(struct pic_parameter_set_rbsp_t *pps)
{
    int i;
    int max;

    if (debug_level_check) return;
    debug2("PPS (Picture Parameter Set)");
    pps_log(pic_parameter_set_id);
    pps_log(seq_parameter_set_id);
    pps_log(entropy_coding_mode_flag);
    pps_log(pic_order_present_flag);
    pps_log(num_slice_groups_minus1);
    pps_log(slice_group_map_type);

    max = minimum(pps->slice_group_map_type, max_num_slice_groups-1);
    for (i=0;i<=max;i++) pps_array_log(run_length_minus1,i);

    max = minimum(pps->slice_group_map_type, max_num_slice_groups);
    for (i=0;i<max;i++) {
        pps_array_log(top_left,i);
        pps_array_log(bottom_right,i);
    }

    pps_log(slice_group_change_direction_flag);
    pps_log(slice_group_change_rate_minus1);
    pps_log(pic_size_in_map_units_minus1);

    max = minimum(pps->pic_size_in_map_units_minus1,
                  max_pic_size_in_map_units-1);
    for (i=0;i<=pps->pic_size_in_map_units_minus1;i++) {
        pps_array_log(slice_group_id,i);
    }

    pps_log(num_ref_idx_l0_active_minus1);
    pps_log(num_ref_idx_l1_active_minus1);

    pps_log(weighted_pred_flag);
    pps_log(weighted_bipred_idc);

    pps_log(pic_init_qp_minus26);
    pps_log(pic_init_qs_minus26);
    pps_log(chroma_qp_index_offset);
    pps_log(deblocking_filter_control_present_flag);
    pps_log(constrained_intra_flag);
    pps_log(redundant_pic_cnt_present_flag);
}

void read_pps(struct pic_parameter_set_rbsp_t *pps, int *hsize)
{
    int i;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(pps,0,sizeof(*pps));

    pps->pic_parameter_set_id = get_header_data(hsize);
    pps->seq_parameter_set_id = get_header_data(hsize);

    v = get_header_data(hsize);
    pps->entropy_coding_mode_flag = bits(v,1,1);
    pps->pic_order_present_flag   = bits(v,0,0);

    pps->num_slice_groups_minus1 = get_header_data(hsize);
    if (pps->num_slice_groups_minus1 >= max_num_slice_groups) {
        // it cannot be
        debug0("error num_slice_groups_minus1 %u >= max_num_slice_groups %u",
               pps->num_slice_groups_minus1, max_num_slice_groups);
        return;
    }

    if (pps->num_slice_groups_minus1 > 0) {
        pps->slice_group_map_type = get_header_data(hsize);
        if (pps->slice_group_map_type == 0) {
            for (i=0; i<=pps->num_slice_groups_minus1; i++) {
                pps->run_length_minus1[i] = get_header_data(hsize);
            }
        } else if (pps->slice_group_map_type == 2) {
            for (i=0; i<pps->num_slice_groups_minus1; i++) {
                pps->top_left[i]     = get_header_data(hsize);
                pps->bottom_right[i] = get_header_data(hsize);
            }
        } else if (pps->slice_group_map_type == 3 ||
                   pps->slice_group_map_type == 4 ||
                   pps->slice_group_map_type == 5) {
            pps->slice_group_change_direction_flag = get_header_data(hsize);
            pps->slice_group_change_rate_minus1 = get_header_data(hsize);
        } else if (pps->slice_group_map_type == 6) {
            pps->pic_size_in_map_units_minus1 = get_header_data(hsize);
            if (pps->pic_size_in_map_units_minus1>=max_pic_size_in_map_units){
                // it cannot be
                debug0("error pic_size_in_map_units_minus1 %u >= "
                       "max_pic_size_in_map_units %u",
                       pps->pic_size_in_map_units_minus1,
                       max_pic_size_in_map_units);
                return;
            }
            for (i=0; i<=pps->pic_size_in_map_units_minus1; i++) {
                pps->slice_group_id[i] = get_header_data(hsize);
            }
        }
    }

    pps->num_ref_idx_l0_active_minus1 = get_header_data(hsize);
    pps->num_ref_idx_l1_active_minus1 = get_header_data(hsize);

    v = get_header_data(hsize);
    pps->weighted_pred_flag  = bits(v,2,2);
    pps->weighted_bipred_idc = bits(v,1,0);

    pps->pic_init_qp_minus26 = get_header_data(hsize);
    pps->pic_init_qs_minus26 = get_header_data(hsize);
    pps->chroma_qp_index_offset = get_header_data(hsize);

    v = get_header_data(hsize);
    pps->deblocking_filter_control_present_flag = bits(v,2,2);
    pps->constrained_intra_flag = bits(v,1,1);
    pps->redundant_pic_cnt_present_flag = bits(v,0,0);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SH (Slice Header)
 */

/* ref_pic_list_reordering */

#define rplr_log(_name) _param_log(rplr,_name)
#define rplr_rop_log(_name,_i) \
 debug2_np(" rop[%d]." #_name " 0x%X %u", \
  _i,rplr->rop[_i]._name,rplr->rop[_i]._name)

void show_ref_pic_list_reordering(struct ref_pic_list_reordering_t *rplr)
{
    int i;
    if (debug_level_check) return;
    debug2("ref_pic_list_reordering");
    rplr_log(sh_slice_type);
    rplr_log(ref_pic_list_reordering_flag_l0);

    if (rplr->ref_pic_list_reordering_flag_l0) {
        for (i=0; i<=max_reordering_of_pic; i++) {
            unsigned short idc = rplr->rop[i].reordering_of_pic_nums_idc;
            rplr_rop_log(reordering_of_pic_nums_idc,i);
            if (idc == 0 || idc == 1) {
                rplr_rop_log(u.abs_diff_pic_num_minus1,i);
            } else if (idc == 2) {
                rplr_rop_log(u.long_term_pic_num,i);
            } else {
                break;
	    }
        }
    }
}

void read_ref_pic_list_reordering(struct ref_pic_list_reordering_t *rplr,
                                  int *hsize,
                                  unsigned short slice_type,
				  unsigned short num_ref_idx_l0_active)
{
    unsigned short idc;
    int i;
    TRY_GET_HEADER_DATA;

    if (num_ref_idx_l0_active > max_reordering_of_pic) {
      debug0("error num_ref_idx_l0_active %u > max_reordering_of_pic %u",
	     num_ref_idx_l0_active, max_reordering_of_pic);
      return;
    }

    rplr->sh_slice_type = slice_type;

    if (!is_slice_type_I(slice_type) && !is_slice_type_SI(slice_type)) {
        rplr->ref_pic_list_reordering_flag_l0 = get_header_data(hsize);
        if (rplr->ref_pic_list_reordering_flag_l0) {
	  for (i = 0; i <= num_ref_idx_l0_active; i++) {
	    idc = get_header_data(hsize);
	    rplr->rop[i].reordering_of_pic_nums_idc = idc;
	    if (idc == 0 || idc == 1) {
                    rplr->rop[i].u.abs_diff_pic_num_minus1 =
		      get_header_data(hsize);
	    } else if (idc == 2) {
                    rplr->rop[i].u.long_term_pic_num =
		      get_header_data(hsize);
	    } else {
	      break;    // if idc == 3 then end loop
	    }
	  }
	  if (i > num_ref_idx_l0_active) {
	    debug0("error over num_ref_idx_l0_active %u",
		   num_ref_idx_l0_active);
	    return;
	  }
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/* dec_ref_pic_marking */

#define drpm_log(_name) _param_log(drpm,_name)

#define drpm_mmco_log(_name,_i) \
 debug2_np(" mmco[%d]." #_name " 0x%X %u", \
  _i,drpm->mmco[_i]._name,drpm->mmco[_i]._name)

void show_dec_ref_pic_marking(struct dec_ref_pic_marking_t *drpm)
{
    int i;
    if (debug_level_check) return;
    debug2("dec_ref_pic_marking");
    drpm_log(nal_unit_type);
    drpm_log(no_output_of_prior_pics_flag);
    drpm_log(long_term_reference_flag);
    drpm_log(adaptive_ref_pic_marking_mode_flag);

    if (drpm->adaptive_ref_pic_marking_mode_flag) {
        for (i=0; i<=max_memory_management_control_operation; i++) {
            drpm_mmco_log(memory_management_control_operation,i);
            drpm_mmco_log(difference_of_pic_nums_minus1,i);
            drpm_mmco_log(long_term_pic_num,i);
            drpm_mmco_log(long_term_frame_idx,i);
            drpm_mmco_log(max_long_term_frame_idx_plus1,i);
            if (drpm->mmco[i].memory_management_control_operation == 0) {
               break;
            }
        }
    }
}

void read_dec_ref_pic_marking(struct dec_ref_pic_marking_t *dec_ref,int *hsize)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short v;
    unsigned short op;
    int i;
    TRY_GET_HEADER_DATA;

    v = readw(&regs->dec_PARreg8);
    dec_ref->nal_unit_type = bits(v,4,0);

    if (dec_ref->nal_unit_type == 5) {
        v = get_header_data(hsize);
        dec_ref->no_output_of_prior_pics_flag = bits(v,1,1);
        dec_ref->long_term_reference_flag     = bits(v,0,0);
    } else {
        dec_ref->adaptive_ref_pic_marking_mode_flag = get_header_data(hsize);
        if (dec_ref->adaptive_ref_pic_marking_mode_flag) {
         for (i=0; i<=max_memory_management_control_operation; i++) {
	    op = get_header_data(hsize);
	    dec_ref->mmco[i].memory_management_control_operation = op;
	    if (op == 1 || op == 3) {
	      dec_ref->mmco[i].difference_of_pic_nums_minus1 =
		get_header_data(hsize);
	    }
	    if (op == 2) {
	      dec_ref->mmco[i].long_term_pic_num =
		get_header_data(hsize);
	    }
	    if (op == 3 || op == 6) {
	      dec_ref->mmco[i].long_term_frame_idx =
		get_header_data(hsize);
	    }
	    if (op == 4) {
	      dec_ref->mmco[i].max_long_term_frame_idx_plus1 =
		get_header_data(hsize);
	    }
	    if (op == 0) {
	      break;
	    }
	  }
	  if (i > max_memory_management_control_operation) {
	    debug0("error over max_memory_management_control_operation %u",
		   max_memory_management_control_operation);
	    return;
          }
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

#define sh_log(_name) _param_log(sh,_name)

void show_sh(struct slice_header_t *sh)
{
    if (debug_level_check) return;
    debug2("SH (Slice Header)");

    sh_log(nal_ref_idc);
    sh_log(nal_unit_type);
    sh_log(sps_pic_order_cnt_type);
    sh_log(sps_delta_pic_order_always_zero_flag);
    sh_log(sps_frame_mbs_only_flag);
    sh_log(pps_pic_order_present_flag);
    sh_log(pps_deblocking_filter_control_present_flag);

    sh_log(slice_type);
    sh_log(pic_parameter_set_id);
    sh_log(frame_num);
    sh_log(field_pic_flag);
    sh_log(bottom_field_flag);
    sh_log(idr_pic_id);
    sh_log(pic_order_cnt_lsb);

    sh_log(delta_pic_order_cnt_bottom);
    sh_log(delta_pic_order_cnt[0]);
    sh_log(delta_pic_order_cnt[1]);

    sh_log(num_ref_idx_active_override_flag);
    sh_log(num_ref_idx_l0_active_minus1);

    sh_log(slice_qp_delta);
    sh_log(slice_qs_delta);
    sh_log(disable_deblocking_filter_idc);
    sh_log(slice_alpha_c0_offset_div2);
    sh_log(slice_beta_offset_div2);

    show_ref_pic_list_reordering(&sh->ref_pic_list_reordering);
    show_dec_ref_pic_marking(&sh->dec_ref_pic_marking);
}

void read_sh(struct slice_header_t *sh, int *hsize)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct seq_parameter_set_rbsp_t *sps = &avc_info->last_sps;
    struct pic_parameter_set_rbsp_t *pps = &avc_info->last_pps;
    unsigned short num_ref_idx_l0_active_minus1;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sh,0,sizeof(*sh));

    v = readw(&regs->dec_PARreg8);
    sh->nal_ref_idc   = bits(v,6,5);
    sh->nal_unit_type = bits(v,4,0);
    sh->sps_pic_order_cnt_type = sps->pic_order_cnt_type;
    sh->sps_delta_pic_order_always_zero_flag =
        sps->delta_pic_order_always_zero_flag;
    sh->sps_frame_mbs_only_flag = sps->frame_mbs_only_flag;
    sh->pps_pic_order_present_flag = pps->pic_order_present_flag;
    sh->pps_deblocking_filter_control_present_flag =
        pps->deblocking_filter_control_present_flag;

    v = get_header_data(hsize); // dummy
    sh->slice_type = get_header_data(hsize);
    sh->pic_parameter_set_id = get_header_data(hsize);
    sh->frame_num = get_header_data(hsize);
    if (!sh->sps_frame_mbs_only_flag) {
        sh->field_pic_flag = get_header_data(hsize);
        if (sh->field_pic_flag) {
            sh->bottom_field_flag = get_header_data(hsize);
        }
    }

    if (sh->nal_unit_type == 5) {
        sh->idr_pic_id = get_header_data(hsize);
    }

    if (sh->sps_pic_order_cnt_type == 0) {
        sh->pic_order_cnt_lsb = get_header_data(hsize);
        if (sh->pps_pic_order_present_flag && !sh->field_pic_flag) {
            sh->delta_pic_order_cnt_bottom = get_header_data32(hsize);
        }
    }

    if (sh->sps_pic_order_cnt_type == 1 &&
        !sh->sps_delta_pic_order_always_zero_flag) {
        sh->delta_pic_order_cnt[0] = get_header_data32(hsize);
        if (sh->pps_pic_order_present_flag && !sh->field_pic_flag) {
            sh->delta_pic_order_cnt[1] = get_header_data32(hsize);
        }
    }

    num_ref_idx_l0_active_minus1 = pps->num_ref_idx_l0_active_minus1;
    if (is_slice_type_P (sh->slice_type) ||
        is_slice_type_SP(sh->slice_type) ||
        is_slice_type_B (sh->slice_type)
        ) {
        sh->num_ref_idx_active_override_flag = get_header_data(hsize);
        if (sh->num_ref_idx_active_override_flag) {
            sh->num_ref_idx_l0_active_minus1 = get_header_data(hsize);
	    num_ref_idx_l0_active_minus1 = sh->num_ref_idx_l0_active_minus1;
        }
    }

    read_ref_pic_list_reordering(&sh->ref_pic_list_reordering,hsize,
                                 sh->slice_type,
				 num_ref_idx_l0_active_minus1+1);

    if (sh->nal_ref_idc != 0) {
        read_dec_ref_pic_marking(&sh->dec_ref_pic_marking,hsize);
    }

    sh->slice_qp_delta = get_header_data(hsize);
    if (is_slice_type_SP(sh->slice_type) ||
        is_slice_type_SI(sh->slice_type)) {
        sh->slice_qs_delta = get_header_data(hsize);
    }

    if (sh->pps_deblocking_filter_control_present_flag) {
        sh->disable_deblocking_filter_idc = get_header_data(hsize);
        if (sh->disable_deblocking_filter_idc != 1) {
            sh->slice_alpha_c0_offset_div2 = get_header_data(hsize);
            sh->slice_beta_offset_div2 = get_header_data(hsize);
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * AUD (Access Unit Delimiter)
 */

void show_aud(struct aud_parameters_t *aud)
{
    if (debug_level_check) return;
    debug2("AUD (Access Unit Delimiter)");
    debug2(" primary_pic_type %u",aud->primary_pic_type);
    switch (aud->primary_pic_type) {
    case 0: debug2("  I");           break;
    case 1: debug2("  I,P");         break;
    case 2: debug2("  I,B,P");       break;
    case 3: debug2("  SI");          break;
    case 4: debug2("  SI,SP");       break;
    case 5: debug2("  I,SI");        break;
    case 6: debug2("  I,SI,P,SP");   break;
    case 7: debug2("  I,SI,P,SP,B"); break;
    default:debug2("  error");       break;
    }
}

/*
 * SEI 0: buffering_period
 */

#define sei_log(_name) _param_log(sei,_name)
#define sei_array_log(_name,_i) _param_array_log(sei,_name,_i)
#define sei_st_array_log(_st,_name,_i) \
 debug2_np("  " #_name " 0x%X %u", sei->_st[_i]._name, sei->_st[_i]._name)

void show_buffering_period(void *vp)
{
    struct buffering_period_t *sei = (struct buffering_period_t *)vp;
    int i;

    debug2("SEI 0: buffering_period");
    sei_log(NalHrdBpPresentFlag);
    sei_log(VclHrdBpPresentFlag);
    sei_log(nal_hrd_cpb_cnt_minus1);
    sei_log(vcl_hrd_cpb_cnt_minus1);
    sei_log(seq_parameter_set_id);

    if (sei->NalHrdBpPresentFlag) {
        for (i=0; i<=sei->nal_hrd_cpb_cnt_minus1 && i<max_cpb_cnt; i++) {
            sei_array_log(nal_initial_cpb_removal_delay,i);
            sei_array_log(nal_initial_cpb_removal_delay_offset,i);
        }
    }

    if (sei->VclHrdBpPresentFlag) {
        for (i=0; i<=sei->vcl_hrd_cpb_cnt_minus1 && i<max_cpb_cnt; i++) {
            sei_array_log(vcl_initial_cpb_removal_delay,i);
            sei_array_log(vcl_initial_cpb_removal_delay_offset,i);
        }
    }
}

void read_buffering_period(void *vp, int *hsize)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct seq_parameter_set_rbsp_t *sps = &avc_info->last_sps;
    struct buffering_period_t *sei = (struct buffering_period_t *)vp;
    int i;
    unsigned short v;
    unsigned short nal_hrd_cpb_cnt_minus1;
    unsigned short vcl_hrd_cpb_cnt_minus1;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    v = readw(&regs->head_PRreg18);
    sei->NalHrdBpPresentFlag = bits(v,3,3);
    sei->VclHrdBpPresentFlag = bits(v,2,2);

    sei->seq_parameter_set_id = get_header_data(hsize);

    if (sei->NalHrdBpPresentFlag) {
        nal_hrd_cpb_cnt_minus1 =
            sps->vui_parameters.nal_hrd_parameters.cpb_cnt_minus1;
        if (nal_hrd_cpb_cnt_minus1 >= max_cpb_cnt) {
            // it cannot be
            debug0("error nal_hrd_cpb_cnt_minus1 %u >= max_cpb_cnt %u",
                   nal_hrd_cpb_cnt_minus1, max_cpb_cnt);
            return;
        }
        sei->nal_hrd_cpb_cnt_minus1 = nal_hrd_cpb_cnt_minus1;

        for (i=0; i<=sei->nal_hrd_cpb_cnt_minus1; i++) {
            sei->nal_initial_cpb_removal_delay[i] = get_header_data32(hsize);
            sei->nal_initial_cpb_removal_delay_offset[i] =
                get_header_data32(hsize);
        }
    }

    if (sei->VclHrdBpPresentFlag) {
        vcl_hrd_cpb_cnt_minus1 =
            sps->vui_parameters.vcl_hrd_parameters.cpb_cnt_minus1;
        if (vcl_hrd_cpb_cnt_minus1 >= max_cpb_cnt) {
            // it cannot be
            debug0("error vcl_hrd_cpb_cnt_minus1 %u >= max_cpb_cnt %u",
                   vcl_hrd_cpb_cnt_minus1, max_cpb_cnt);
            return;
        }
        sei->vcl_hrd_cpb_cnt_minus1 = vcl_hrd_cpb_cnt_minus1;

        for (i=0; i<=sei->vcl_hrd_cpb_cnt_minus1; i++) {
            sei->vcl_initial_cpb_removal_delay[i] = get_header_data32(hsize);
            sei->vcl_initial_cpb_removal_delay_offset[i] =
                get_header_data32(hsize);
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 1: pic_timing
 */

void show_pic_timing(void *vp)
{
    struct pic_timing_t *sei = (struct pic_timing_t *)vp;
    int i;

    if (debug_level_check) return;
    debug2("SEI 1: pic_timing");
    sei_log(CpbDpbDelaysPresentFlag);
    sei_log(vui_pic_struct_present_flag);
    sei_log(cpb_removal_delay);
    sei_log(dpb_output_delay);
    sei_log(pic_struct);

    for (i=0; i<NumClockTS; i++) {
        debug2_np(" ct[%d]",i);
        sei_st_array_log(ct,clock_timestamp_flag,i);
        if (sei->ct[i].clock_timestamp_flag == 0) continue;
        sei_st_array_log(ct,ct_type,i);
        sei_st_array_log(ct,nuit_field_based_flag,i);
        sei_st_array_log(ct,counting_type,i);
        sei_st_array_log(ct,full_timestamp_flag,i);
        sei_st_array_log(ct,discontinuity_flag,i);
        sei_st_array_log(ct,cnt_dropped_flag,i);
        sei_st_array_log(ct,n_frames,i);
        sei_st_array_log(ct,seconds_value,i);
        sei_st_array_log(ct,minutes_value,i);
        sei_st_array_log(ct,hours_value,i);
        sei_st_array_log(ct,seconds_flag,i);
        sei_st_array_log(ct,minutes_flag,i);
        sei_st_array_log(ct,hours_flag,i);
        sei_st_array_log(ct,time_offset,i);
    }
}

void read_pic_timing(void *vp, int *hsize)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct seq_parameter_set_rbsp_t *sps = &avc_info->last_sps;
    struct pic_timing_t *sei = (struct pic_timing_t *)vp;
    unsigned short time_offset_length = 0;
    unsigned short numclock = 0;
    int i;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    v = readw(&regs->head_PRreg18);
    sei->CpbDpbDelaysPresentFlag = bits(v,1,1);
    sei->vui_pic_struct_present_flag =
        sps->vui_parameters.pic_struct_present_flag;

    if (sei->CpbDpbDelaysPresentFlag) {
        sei->cpb_removal_delay = get_header_data32(hsize);
        sei->dpb_output_delay  = get_header_data32(hsize);
    }

    if (sei->vui_pic_struct_present_flag) {

        struct vui_parameters_t *vui = &sps->vui_parameters;
        time_offset_length = vui->nal_hrd_parameters.time_offset_length;
        if (time_offset_length == 0) {
            time_offset_length = vui->vcl_hrd_parameters.time_offset_length;
        }

        sei->pic_struct = get_header_data(hsize);
        switch (sei->pic_struct) {
        case 0:
        case 1:
        case 2:
            numclock = 1;
            break;
        case 3:
        case 4:
        case 7:
            numclock = 2;
            break;
        case 5:
        case 6:
        case 8:
            numclock = 3;
            break;
        default:
            break;
        }

        for (i=0; i<numclock; i++) {
            sei->ct[i].clock_timestamp_flag = get_header_data(hsize);
            if (sei->ct[i].clock_timestamp_flag) {
                v = get_header_data(hsize);
                sei->ct[i].ct_type               = bits(v,7,6);
                sei->ct[i].nuit_field_based_flag = bits(v,5,5);
                sei->ct[i].counting_type         = bits(v,4,0);
                v = get_header_data(hsize);
                sei->ct[i].full_timestamp_flag = bits(v,10,10);
                sei->ct[i].discontinuity_flag  = bits(v,9,9);
                sei->ct[i].cnt_dropped_flag    = bits(v,8,8);
                sei->ct[i].n_frames            = bits(v,7,0);
                if (sei->ct[i].full_timestamp_flag) {
                    sei->ct[i].seconds_value = get_header_data(hsize);
                    v = get_header_data(hsize);
                    sei->ct[i].minutes_value = bits(v,13,8);
                    sei->ct[i].hours_value   = bits(v, 5,0);
                } else {
                    sei->ct[i].seconds_flag = get_header_data(hsize);
                    if (sei->ct[i].seconds_flag) {
                        v = get_header_data(hsize);
                        sei->ct[i].seconds_value = bits(v,6,1);
                        sei->ct[i].minutes_flag  = bits(v,0,0);
                        if (sei->ct[i].minutes_flag) {
                            v = get_header_data(hsize);
                            sei->ct[i].minutes_value = bits(v,6,1);
                            sei->ct[i].hours_flag    = bits(v,0,0);
                            if (sei->ct[i].hours_flag) {
                                sei->ct[i].hours_value= get_header_data(hsize);
                            }
                        }
                    }
                }
                if (time_offset_length > 0) {
                    sei->ct[i].time_offset = get_header_data32(hsize);
                }
            }
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 2: pan_scan_rect
 */

void show_pan_scan_rect(void *vp)
{
    struct pan_scan_rect_t *sei = (struct pan_scan_rect_t *)vp;
    int i;

    if (debug_level_check) return;
    debug2("SEI 2: pan_scan_rect");
    sei_log(pan_scan_rect_id);
    sei_log(pan_scan_rect_cancel_flag);
    sei_log(pan_scan_cnt_minus1);

    if (!sei->pan_scan_rect_cancel_flag) {
        for (i=0; i<=sei->pan_scan_cnt_minus1 && i<max_pan_scan_cnt; i++) {
            debug2_np(" psr[%d]",i);
            sei_st_array_log(psr,left_offset,i);
            sei_st_array_log(psr,right_offset,i);
            sei_st_array_log(psr,top_offset,i);
            sei_st_array_log(psr,bottom_offset,i);
        }
    }

    sei_log(pan_scan_rect_repetition_period);
}

void read_pan_scan_rect(void *vp, int *hsize)
{
    struct pan_scan_rect_t *sei = (struct pan_scan_rect_t *)vp;
    int i;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->pan_scan_rect_id = get_header_data32(hsize);
    sei->pan_scan_rect_cancel_flag = get_header_data(hsize);

    if (!sei->pan_scan_rect_cancel_flag) {
        sei->pan_scan_cnt_minus1 = get_header_data(hsize);
        if (sei->pan_scan_cnt_minus1 >= max_pan_scan_cnt) {
            // it cannot be
            debug0("error pan_scan_cnt_minus1 %u >= max_pan_scan_cnt %u",
                   sei->pan_scan_cnt_minus1, max_pan_scan_cnt);
            return;
        }
        for (i=0; i<=sei->pan_scan_cnt_minus1; i++) {
            sei->psr[i].left_offset   = get_header_data32(hsize);
            sei->psr[i].right_offset  = get_header_data32(hsize);
            sei->psr[i].top_offset    = get_header_data32(hsize);
            sei->psr[i].bottom_offset = get_header_data32(hsize);
        }
        sei->pan_scan_rect_repetition_period = get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 3: filler_payload
 */

void show_filler_payload(void *vp)
{
    struct filler_payload_t *sei = (struct filler_payload_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 3: filler_payload");
    sei_log(payloadSize);
}

void read_filler_payload(void *vp, int *hsize)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct filler_payload_t *sei = (struct filler_payload_t *)vp;

    memset(sei,0,sizeof(*sei));

    sei->payloadSize = readw(&regs->head_PRreg17);
}

/*
 * SEI 4: user_data_registered_itu_t_t35
 */

void show_user_data_registered_itu_t_t35(void *vp)
{
    struct user_data_registered_itu_t_t35_t *sei =
        (struct user_data_registered_itu_t_t35_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 4: user_data_registered_itu_t_t35");
    sei_log(itu_t_t35_country_code);
    sei_log(itu_t_t35_payload_byte);
    sei_log(itu_t_t35_country_code_extension_byte);
}

void read_user_data_registered_itu_t_t35(void *vp, int *hsize)
{
    struct user_data_registered_itu_t_t35_t *sei =
        (struct user_data_registered_itu_t_t35_t *)vp;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    v = get_header_data(hsize);
    sei->itu_t_t35_country_code = bits(v,15,8);
    if (sei->itu_t_t35_country_code != 0xFF) {
        sei->itu_t_t35_payload_byte = bits(v,7,0);
    } else {
        sei->itu_t_t35_country_code_extension_byte = bits(v,7,0);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 5: user_data_unregistered
 */

void show_user_data_unregistered(void *vp)
{
    struct user_data_unregistered_t *sei =
        (struct user_data_unregistered_t *)vp;
    int i;

    if (debug_level_check) return;
    debug2("SEI 5: user_data_unregistered");
    for (i=0; i<8; i++) {
        sei_array_log(uuid_iso_iec_11578,i);
    }
}

void read_user_data_unregistered(void *vp, int *hsize)
{
    struct user_data_unregistered_t *sei =
        (struct user_data_unregistered_t *)vp;
    int i;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    for (i=7; i>=0; i--) {
        sei->uuid_iso_iec_11578[i] = get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 6: recovery_point
 */

void show_recovery_point(void *vp)
{
    struct recovery_point_t *sei =
        (struct recovery_point_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 6: recovery_point");
    sei_log(recovery_frame_cnt);
    sei_log(exact_match_flag);
    sei_log(broken_link_flag);
    sei_log(changing_slice_group_idc);
}

void read_recovery_point(void *vp, int *hsize)
{
    struct recovery_point_t *sei =
        (struct recovery_point_t *)vp;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->recovery_frame_cnt = get_header_data(hsize);
    v = get_header_data(hsize);
    sei->exact_match_flag = bits(v,3,3);
    sei->broken_link_flag = bits(v,2,2);
    sei->changing_slice_group_idc = bits(v,1,0);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 7: dec_ref_pic_marking_repetition
 */

void show_dec_ref_pic_marking_repetition(void *vp)
{
    struct dec_ref_pic_marking_repetition_t *sei =
        (struct dec_ref_pic_marking_repetition_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 7: dec_ref_pic_marking_repetition");
    sei_log(sps_frame_mbs_only_flag);
    sei_log(original_idr_flag);
    sei_log(original_frame_num);
    sei_log(original_field_pic_flag);
    sei_log(original_bottom_field_flag);

    show_dec_ref_pic_marking(&sei->dec_ref_pic_marking);
}

void read_dec_ref_pic_marking_repetition(void *vp, int *hsize)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct dec_ref_pic_marking_repetition_t *sei =
        (struct dec_ref_pic_marking_repetition_t *)vp;
    //struct avc_last_sps_t *sps = &avc_info->last_sps;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    v = readw(&regs->dec_PARreg8);
    sei->sps_frame_mbs_only_flag = bits(v,15,15);
    //sei->sps_frame_mbs_only_flag = sps->frame_mbs_only_flag;

    sei->original_idr_flag = get_header_data(hsize);
    sei->original_frame_num = get_header_data(hsize);
    if (!sei->sps_frame_mbs_only_flag) {
        sei->original_field_pic_flag = get_header_data(hsize);
        if (sei->original_field_pic_flag) {
            sei->original_bottom_field_flag = get_header_data(hsize);
        }
    }
    read_dec_ref_pic_marking(&sei->dec_ref_pic_marking,hsize);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 8: spare_pic
 */

#define sp_u_log(_name,_i,_j) \
 debug2_np("  " #_name "[%d] 0x%X %u", \
  _j,sei->spare_pics[_i].u._name[_j],sei->spare_pics[_i].u._name[_j])

void show_spare_pic(void *vp)
{
    struct spare_pic_t *sei = (struct spare_pic_t *)vp;
    int i,j;
    int max1,max2;

    if (debug_level_check) return;
    debug2("SEI 8: spare_pic");
    sei_log(pps_pic_size_in_map_units);
    sei_log(target_frame_num);
    sei_log(spare_field_flag);
    sei_log(target_bottom_field_flag);
    sei_log(num_spare_pics_minus1);

    max1 = minimum(sei->num_spare_pics_minus1+1,max_num_spare_pics);
    for (i=0; i<max1; i++) {
        debug2_np(" spare_pics[%d]",i);
        sei_st_array_log(spare_pics,delta_spare_frame_num,i);
        sei_st_array_log(spare_pics,spare_bottom_field_flag,i);
        sei_st_array_log(spare_pics,spare_area_idc,i);
        max2 = minimum(sei->pps_pic_size_in_map_units,
                       max_pic_size_in_map_units);
        if (sei->spare_pics[i].spare_area_idc == 1) {
            for (j=0; j<max2; j++) {
                sp_u_log(spare_unit_flag,i,j);
            }
        } else if (sei->spare_pics[i].spare_area_idc == 2) {
            unsigned short mapUnitCnt = 0;
            for (j=0; mapUnitCnt<max2; j++) {
                sp_u_log(zero_run_length,i,j);
                mapUnitCnt += sei->spare_pics[i].u.zero_run_length[j] + 1;
            }
        }
    }
}

void read_spare_pic(void *vp, int *hsize)
{
    struct spare_pic_t *sei = (struct spare_pic_t *)vp;
    struct pic_parameter_set_rbsp_t *pps = &avc_info->last_pps;
    int i,j;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->pps_pic_size_in_map_units = pps->pic_size_in_map_units_minus1 + 1;

    if (sei->pps_pic_size_in_map_units > max_pic_size_in_map_units) {
        // it cannot be
        debug0("error pps_pic_size_in_map_units %u >"
               "max_pic_size_in_map_units %u",
               sei->pps_pic_size_in_map_units, max_pic_size_in_map_units);
        return;
    }

    sei->target_frame_num = get_header_data(hsize);
    sei->spare_field_flag = get_header_data(hsize);
    if (sei->spare_field_flag) {
        sei->target_bottom_field_flag = get_header_data(hsize);
    }
    sei->num_spare_pics_minus1 = get_header_data(hsize);

    if (sei->num_spare_pics_minus1+1 >= max_num_spare_pics) {
        // it cannot be
        debug0("error num_spare_pics_minus1+1 %u >= max_num_spare_pics %u",
               sei->num_spare_pics_minus1, max_num_spare_pics);
        return;
    }

    for (i=0; i<sei->num_spare_pics_minus1+1; i++) {
        sei->spare_pics[i].delta_spare_frame_num = get_header_data(hsize);
        if (sei->spare_field_flag) {
            sei->spare_pics[i].spare_bottom_field_flag= get_header_data(hsize);
        }
        sei->spare_pics[i].spare_area_idc = get_header_data(hsize);
        if (sei->spare_pics[i].spare_area_idc == 1) {
            for (j=0; j<sei->pps_pic_size_in_map_units; j++) {
                sei->spare_pics[i].u.spare_unit_flag[j]=get_header_data(hsize);
            }
        } else if (sei->spare_pics[i].spare_area_idc == 2) {
            unsigned short mapUnitCnt = 0;
            for (j=0; mapUnitCnt<sei->pps_pic_size_in_map_units; j++) {
                sei->spare_pics[i].u.zero_run_length[j]=get_header_data(hsize);
                mapUnitCnt += sei->spare_pics[i].u.zero_run_length[j] + 1;
            }
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 9: scene_info
 */

void show_scene_info(void *vp)
{
    struct scene_info_t *sei = (struct scene_info_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 9: scene_info");
    sei_log(scene_info_present_flag);
    sei_log(scene_id);
    sei_log(scene_transition_type);
    sei_log(second_scene_id);
}

void read_scene_info(void *vp, int *hsize)
{
    struct scene_info_t *sei = (struct scene_info_t *)vp;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->scene_info_present_flag = get_header_data(hsize);
    if (sei->scene_info_present_flag) {
        sei->scene_id = get_header_data32(hsize);
        sei->scene_transition_type = get_header_data(hsize);
        if (sei->scene_transition_type > 3) {
            sei->second_scene_id = get_header_data32(hsize);
        }
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 10: sub_seq_info
 */

void show_sub_seq_info(void *vp)
{
    struct sub_seq_info_t *sei = (struct sub_seq_info_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 10: sub_seq_info");
    sei_log(sub_seq_layer_num);
    sei_log(sub_seq_id);
    sei_log(first_ref_pic_flag);
    sei_log(leading_non_ref_pic_flag);
    sei_log(last_pic_flag);
    sei_log(sub_seq_frame_num_flag);
    sei_log(sub_seq_frame_num);
}

void read_sub_seq_info(void *vp, int *hsize)
{
    struct sub_seq_info_t *sei = (struct sub_seq_info_t *)vp;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->sub_seq_layer_num = get_header_data(hsize);
    sei->sub_seq_id = get_header_data(hsize);
    v = get_header_data(hsize);
    sei->first_ref_pic_flag = bits(v,3,3);
    sei->leading_non_ref_pic_flag = bits(v,2,2);
    sei->last_pic_flag = bits(v,1,1);
    sei->sub_seq_frame_num_flag = bits(v,0,0);
    if (sei->sub_seq_frame_num_flag) {
        sei->sub_seq_frame_num = get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 11: sub_seq_layer_characteristics
 */

void show_sub_seq_layer_characteristics(void *vp)
{
    struct sub_seq_layer_characteristics_t *sei =
        (struct sub_seq_layer_characteristics_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 11: sub_seq_layer_characteristics");
    sei_log(num_sub_seq_layers_minus1);
    sei_log(accurate_statistics_flag);
    sei_log(average_bit_rate);
    sei_log(average_frame_rate);
}

void read_sub_seq_layer_characteristics(void *vp, int *hsize)
{
    struct sub_seq_layer_characteristics_t *sei =
        (struct sub_seq_layer_characteristics_t *)vp;
    int i;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->num_sub_seq_layers_minus1 = get_header_data(hsize);
    for (i=0; i<=sei->num_sub_seq_layers_minus1; i++) {
        sei->accurate_statistics_flag = get_header_data(hsize);
        sei->average_bit_rate = get_header_data(hsize);
        sei->average_frame_rate = get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}


/*
 * SEI 12: sub_seq_characteristics
 */

void show_sub_seq_characteristics(void *vp)
{
    struct sub_seq_characteristics_t *sei =
        (struct sub_seq_characteristics_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 12: sub_seq_characteristics");
    sei_log(sub_seq_layer_num);
    sei_log(sub_seq_id);
    sei_log(duration_flag);
    sei_log(sub_seq_duration);
    sei_log(average_rate_flag);
    sei_log(accurate_statistics_flag);
    sei_log(average_bit_rate);
    sei_log(average_frame_rate);
    sei_log(num_referenced_subseqs);
    sei_log(ref_sub_seq_layer_num);
    sei_log(ref_sub_seq_id);
    sei_log(ref_sub_seq_direction);
}

void read_sub_seq_characteristics(void *vp, int *hsize)
{
    struct sub_seq_characteristics_t *sei =
        (struct sub_seq_characteristics_t *)vp;
    unsigned short v;
    int i;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->sub_seq_layer_num = get_header_data(hsize);
    sei->sub_seq_id = get_header_data(hsize);
    sei->duration_flag = get_header_data(hsize);
    if (sei->duration_flag) {
        sei->sub_seq_duration = get_header_data32(hsize);
    }

    v = get_header_data(hsize);
    sei->average_rate_flag = bits(v,0,0);
    if (sei->average_rate_flag) {
        sei->accurate_statistics_flag = bits(v,1,1);
        sei->average_bit_rate = get_header_data(hsize);
        sei->average_frame_rate = get_header_data(hsize);
    }

    sei->num_referenced_subseqs = get_header_data(hsize);
    for (i=0; i<sei->num_referenced_subseqs; i++) {
        sei->ref_sub_seq_layer_num = get_header_data(hsize);
        sei->ref_sub_seq_id = get_header_data(hsize);
        sei->ref_sub_seq_direction = get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 13: full_frame_freeze
 */

void show_full_frame_freeze(void *vp)
{
    struct full_frame_freeze_t *sei = (struct full_frame_freeze_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 13: full_frame_freeze");
    sei_log(full_frame_freeze_repetition_period);
}

void read_full_frame_freeze(void *vp, int *hsize)
{
    struct full_frame_freeze_t *sei = (struct full_frame_freeze_t *)vp;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->full_frame_freeze_repetition_period = get_header_data(hsize);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 15: full_frame_snapshot
 */

void show_full_frame_snapshot(void *vp)
{
    struct full_frame_snapshot_t *sei = (struct full_frame_snapshot_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 15: full_frame_snapshot");
    sei_log(snapshot_id);
}

void read_full_frame_snapshot(void *vp, int *hsize)
{
    struct full_frame_snapshot_t *sei = (struct full_frame_snapshot_t *)vp;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->snapshot_id = get_header_data32(hsize);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 16: progressive_refinement_segment_start
 */

void show_progressive_refinement_segment_start(void *vp)
{
    struct progressive_refinement_segment_start_t *sei =
        (struct progressive_refinement_segment_start_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 16: progressive_refinement_segment_start");
    sei_log(progressive_refinement_id);
    sei_log(num_refinement_steps_minus1);
}

void read_progressive_refinement_segment_start(void *vp, int *hsize)
{
    struct progressive_refinement_segment_start_t *sei =
        (struct progressive_refinement_segment_start_t *)vp;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->progressive_refinement_id = get_header_data32(hsize);
    sei->num_refinement_steps_minus1 = get_header_data(hsize);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 17: progressive_refinement_segment_end
 */

void show_progressive_refinement_segment_end(void *vp)
{
    struct progressive_refinement_segment_end_t *sei =
        (struct progressive_refinement_segment_end_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 17: progressive_refinement_segment_end");
    sei_log(progressive_refinement_id);
}

void read_progressive_refinement_segment_end(void *vp, int *hsize)
{
    struct progressive_refinement_segment_end_t *sei =
        (struct progressive_refinement_segment_end_t *)vp;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->progressive_refinement_id = get_header_data32(hsize);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 18: motion_constrained_slice_group_set
 */

void show_motion_constrained_slice_group_set(void *vp)
{
    struct motion_constrained_slice_group_set_t *sei =
        (struct motion_constrained_slice_group_set_t *)vp;
    int i;
    int max;

    if (debug_level_check) return;
    debug2("SEI 18: motion_constrained_slice_group_set");
    sei_log(num_slice_groups_in_set_minus1);
    max = minimum(sei->num_slice_groups_in_set_minus1,
                  max_num_slice_groups_in_set-1);
    for (i=0; i<=max; i++) {
        sei_array_log(slice_group_id,i);
    }
    sei_log(exact_sample_value_match_flag);
    sei_log(pan_scan_rect_flag);
    sei_log(pan_scan_rect_id);
}

void read_motion_constrained_slice_group_set(void *vp, int *hsize)
{
    struct motion_constrained_slice_group_set_t *sei =
        (struct motion_constrained_slice_group_set_t *)vp;
    unsigned short v;
    int i;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->num_slice_groups_in_set_minus1 = get_header_data(hsize);
    if (sei->num_slice_groups_in_set_minus1 >= max_num_slice_groups_in_set) {
        // it cannot be
        debug0("error num_slice_groups_in_set_minus1 %u >= "
               "max_num_slice_groups_in_set %u",
               sei->num_slice_groups_in_set_minus1,
               max_num_slice_groups_in_set);
        return;
    }

    for (i=0; i<=sei->num_slice_groups_in_set_minus1; i++) {
        sei->slice_group_id[i] = get_header_data(hsize);
    }
    v = get_header_data(hsize);
    sei->exact_sample_value_match_flag = bits(v,1,1);
    sei->pan_scan_rect_flag = bits(v,0,0);
    if (sei->pan_scan_rect_flag) {
        sei->pan_scan_rect_id = get_header_data32(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 19: film_grain_characteristics
 */

#define sei_array2_log(_name,_i,_j) \
 debug2_np(" " #_name "[%d][%d] 0x%X %u", \
  _i,_j,sei->_name[_i][_j],sei->_name[_i][_j])

#define sei_array3_log(_name,_i,_j,_k) \
 debug2_np(" " #_name "[%d][%d][%d] 0x%X %u", \
  _i,_j,_k,sei->_name[_i][_j][_k],sei->_name[_i][_j][_k])

void show_film_grain_characteristics(void *vp)
{
    struct film_grain_characteristics_t *sei =
        (struct film_grain_characteristics_t *)vp;
    int c,i,j;
    int max1,max2;

    if (debug_level_check) return;
    debug2("SEI 19: film_grain_characteristics");
    sei_log(film_grain_characteristics_cancel_flag);
    sei_log(model_id);
    sei_log(separate_colour_description_present_flag);
    sei_log(film_grain_bit_depth_luma_minus8);
    sei_log(film_grain_bit_depth_chroma_minus8);
    sei_log(film_grain_full_range_flag);
    sei_log(film_grain_colour_primaries);
    sei_log(film_grain_transfer_characteristics);
    sei_log(film_grain_matrix_coefficients);
    sei_log(blending_mode_id);
    sei_log(log2_scale_factor);
    for (c=0; c<3; c++) {
        sei_array_log(comp_model_present_flag,c);
        if (sei->comp_model_present_flag[c]) {
            sei_array_log(num_intensity_intervals_minus1,c);
            sei_array_log(num_model_values_minus1,c);
            max1 = minimum(sei->num_intensity_intervals_minus1[c],
                           max_num_intensity_intervals-1);
            max2 = minimum(sei->num_model_values_minus1[c],
                           max_num_model_values-1);
            for (i=0; i<=max1; i++) {
                sei_array2_log(intensity_interval_lower_bound,c,i);
                sei_array2_log(intensity_interval_upper_bound,c,i);
                for (j=0; j<=max2; j++) {
                    sei_array3_log(comp_model_value,c,i,j);
                }
            }
        }
    }
    sei_log(film_grain_characteristics_repetition_period);
}

void read_film_grain_characteristics(void *vp, int *hsize)
{
    struct film_grain_characteristics_t *sei =
        (struct film_grain_characteristics_t *)vp;
    unsigned short v;
    int c,i,j;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->film_grain_characteristics_cancel_flag = get_header_data(hsize);
    if (!sei->film_grain_characteristics_cancel_flag) {
        v = get_header_data(hsize);
        sei->model_id = bits(v,2,1);
        sei->separate_colour_description_present_flag = bits(v,0,0);
        if (sei->separate_colour_description_present_flag) {
            v = get_header_data(hsize);
            sei->film_grain_bit_depth_luma_minus8   = bits(v,14,12);
            sei->film_grain_bit_depth_chroma_minus8 = bits(v,11,9);
            sei->film_grain_full_range_flag  = bits(v,8,8);
            sei->film_grain_colour_primaries = bits(v,7,0);
            v = get_header_data(hsize);
            sei->film_grain_transfer_characteristics = bits(v,15,8);
            sei->film_grain_matrix_coefficients = bits(v,7,0);
        }
        v = get_header_data(hsize);
        sei->blending_mode_id  = bits(v,8,7);
        sei->log2_scale_factor = bits(v,6,3);
        for (c=0; c<3; c++) {
            sei->comp_model_present_flag[c] = bits(v,c,c);
        }
        for (c=0; c<3; c++) {
            if (sei->comp_model_present_flag[c]) {
                v = get_header_data(hsize);
                sei->num_intensity_intervals_minus1[c] = bits(v,10,3);
                sei->num_model_values_minus1[c] = bits(v,2,0);

                if (sei->num_intensity_intervals_minus1[c] >=
                    max_num_intensity_intervals) {
                    // it cannot be
                    debug0("error num_intensity_intervals_minus1[%d] %u >= "
                           "max_num_intensity_intervals %u",
                           c,sei->num_intensity_intervals_minus1[c],
                           max_num_intensity_intervals);
                    return;
                }
                if (sei->num_model_values_minus1[c] >=
                    max_num_model_values) {
                    // it cannot be
                    debug0("error num_model_values_minus1[%d] %u >= "
                           "max_num_model_values %u",
                           c,sei->num_model_values_minus1[c],
                           max_num_model_values);
                    return;
                }

                for (i=0; i<=sei->num_intensity_intervals_minus1[c]; i++) {
                    v = get_header_data(hsize);
                    sei->intensity_interval_lower_bound[c][i] = bits(v,15,8);
                    sei->intensity_interval_upper_bound[c][i] = bits(v,7,0);
                    for (j=0; j<=sei->num_model_values_minus1[c]; j++) {
                        sei->comp_model_value[c][i][j]= get_header_data(hsize);
                    }
                }
            }
        }
        sei->film_grain_characteristics_repetition_period =
            get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 20: deblocking_filter_display_preference
 */

void show_deblocking_filter_display_preference(void *vp)
{
    struct deblocking_filter_display_preference_t *sei =
        (struct deblocking_filter_display_preference_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 20: deblocking_filter_display_preference");
    sei_log(deblocking_display_preference_cancel_flag);
    sei_log(display_prior_to_deblocking_preferred_flag);
    sei_log(dec_frame_buffering_constraint_flag);
    sei_log(deblocking_display_preference_repetition_period);
}

void read_deblocking_filter_display_preference(void *vp, int *hsize)
{
    struct deblocking_filter_display_preference_t *sei =
        (struct deblocking_filter_display_preference_t *)vp;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->deblocking_display_preference_cancel_flag = get_header_data(hsize);
    if (!sei->deblocking_display_preference_cancel_flag) {
        v = get_header_data(hsize);
        sei->display_prior_to_deblocking_preferred_flag = bits(v,1,1);
        sei->dec_frame_buffering_constraint_flag = bits(v,0,0);
        sei->deblocking_display_preference_repetition_period =
            get_header_data(hsize);
    }

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}

/*
 * SEI 21: stereo_video_info
 */

void show_stereo_video_info(void *vp)
{
    struct stereo_video_info_t *sei = (struct stereo_video_info_t *)vp;

    if (debug_level_check) return;
    debug2("SEI 21: stereo_video_info");
    sei_log(field_views_flag);
    sei_log(top_field_is_left_view_flag);
    sei_log(current_frame_is_left_view_flag);
    sei_log(next_frame_is_second_view_flag);
    sei_log(left_view_self_contained_flag);
    sei_log(right_view_self_contained_flag);
}

void read_stereo_video_info(void *vp, int *hsize)
{
    struct stereo_video_info_t *sei = (struct stereo_video_info_t *)vp;
    unsigned short v;
    TRY_GET_HEADER_DATA;

    memset(sei,0,sizeof(*sei));

    sei->field_views_flag = get_header_data(hsize);
    if (sei->field_views_flag) {
        sei->top_field_is_left_view_flag = get_header_data(hsize);
    } else {
        v = get_header_data(hsize);
        sei->current_frame_is_left_view_flag = bits(v,1,1);
        sei->next_frame_is_second_view_flag  = bits(v,0,0);
    }
    v = get_header_data(hsize);
    sei->left_view_self_contained_flag  = bits(v,1,1);
    sei->right_view_self_contained_flag = bits(v,0,0);

    return;

 CATCH_GET_HEADER_DATA:
    debug1("get_header_data error");
}


/*
 * SEI (Supplemental Enhancement Information)
 */

#define SEI_ENTRY(_name) \
 { #_name, offsetof(struct avc_common_info_t, _name), \
   read_##_name, show_##_name }

struct read_sei_table_t {
    char *name;
    int offset;
    void (*read_func)(void *vp, int *hsize);
    void (*show_func)(void *vp);
};

static struct read_sei_table_t read_sei_table[] = {
    SEI_ENTRY(buffering_period),
    SEI_ENTRY(pic_timing),
    SEI_ENTRY(pan_scan_rect),
    SEI_ENTRY(filler_payload),
    SEI_ENTRY(user_data_registered_itu_t_t35),
    SEI_ENTRY(user_data_unregistered),
    SEI_ENTRY(recovery_point),
    SEI_ENTRY(dec_ref_pic_marking_repetition),
    SEI_ENTRY(spare_pic),
    SEI_ENTRY(scene_info),
    SEI_ENTRY(sub_seq_info),
    SEI_ENTRY(sub_seq_layer_characteristics),
    SEI_ENTRY(sub_seq_characteristics),
    SEI_ENTRY(full_frame_freeze),
    SEI_ENTRY(full_frame_freeze), // SEI 14: dummy
    SEI_ENTRY(full_frame_snapshot),
    SEI_ENTRY(progressive_refinement_segment_start),
    SEI_ENTRY(progressive_refinement_segment_end),
    SEI_ENTRY(motion_constrained_slice_group_set),
    SEI_ENTRY(film_grain_characteristics),
    SEI_ENTRY(deblocking_filter_display_preference),
    SEI_ENTRY(stereo_video_info),
};

static struct read_sei_table_t *get_sei_entry(int payloadtype, char **fieldp)
{
    struct read_sei_table_t *entry;

    if (payloadtype < 0 ||
        payloadtype > stereo_video_info ||
        payloadtype == 14)
    {
        return NULL;
    }

    entry = &read_sei_table[payloadtype];
    *fieldp = (char *)avc_info->common_info + entry->offset;
    return entry;
}

void read_sei(int payloadtype, int *hsize)
{
    struct read_sei_table_t *entry;
    char *fieldp;

    entry = get_sei_entry(payloadtype, &fieldp);
    if (entry == NULL) {
        debug1("unsupported payloadtype %d",payloadtype);
        return;
    }

    debug2("type %d, %s, offset %d",payloadtype,entry->name,entry->offset);
    entry->read_func(fieldp, hsize);
}

void show_sei(int payloadtype)
{
    struct read_sei_table_t *entry;
    char *fieldp;

    entry = get_sei_entry(payloadtype, &fieldp);
    if (entry) {
        entry->show_func(fieldp);
    }
}

void read_status_reg(struct avc_interrupt_t *interrupt)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short action_state;
    unsigned short pic_type;
    unsigned short v;

    v = readw(&regs->STATreg);
    debug2("STATreg 0x%04X",v);

    // action_state
    action_state = bits(v,3,2);
    interrupt->status.action_state = action_state;
    switch (action_state) {
    case 0: debug2(" action_state: standby"); break;
    case 1: debug2(" action_state: coding");  break;
    case 2: debug2(" action_state: decoding"); break;
    default:debug2(" action_state: reserved"); break;
    }

    // pic_type
    pic_type = bits(v,1,0);
    interrupt->status.pic_type = pic_type;
    switch (pic_type) {
    case 0: debug2(" pic_type: I-slice"); break;
    case 1: debug2(" pic_type: P-slice"); break;
    default:debug2(" pic_type: reserved");break;
    }
}

void read_dec_parreg(struct decode_info_t *decode_info)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short v;

    decode_info->dec_width = readw(&regs->dec_PARreg1);
    decode_info->dec_height= readw(&regs->dec_PARreg2);

    v = readw(&regs->dec_PARreg4);
    decode_info->frm_crop_flag = bits(v,15,15);
    decode_info->frm_crop_rect_lft_oft = bits(v,10,0);
    decode_info->frm_crop_rect_rgt_oft = readw(&regs->dec_PARreg5);
    decode_info->frm_crop_rect_top_oft = readw(&regs->dec_PARreg6);
    decode_info->frm_crop_rect_btm_oft = readw(&regs->dec_PARreg7);

    debug2("dec_width  %u",decode_info->dec_width);
    debug2("dec_height %u",decode_info->dec_height);
    debug2("frm_crop_flag  %u",decode_info->frm_crop_flag);
    debug2("frm_crop_lft_oft,rgt,top,btm %u, %u, %u, %u",
           decode_info->frm_crop_rect_lft_oft,
           decode_info->frm_crop_rect_rgt_oft,
           decode_info->frm_crop_rect_top_oft,
           decode_info->frm_crop_rect_btm_oft
           );
}

void set_sei_decode_flags(void)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct sei_decode_flags_t *sei_decode_flags =
        &avc_info->common_info->sei_decode_flags;
    unsigned short v = 0;

#if 0
    struct vui_parameters_t *vui;

    vui = &avc_info->last_sps.vui_parameters;

    if (vui->nal_hrd_parameters_present_flag == 0 &&
        sei_decode_flags->NalHrdBpPresentFlag) {
        v = wbits(1,3,3);
    }

    if (vui->vcl_hrd_parameters_present_flag == 0 &&
        sei_decode_flags->VclHrdBpPresentFlag) {
        v |= wbits(1,2,2);
    }

    if (vui->nal_hrd_parameters_present_flag == 0 &&
        sei_decode_flags->CpbDpbDelaysPresentFlag) {
        v |= wbits(1,1,1);
    }
#endif

    v  = wbits(sei_decode_flags->NalHrdBpPresentFlag,3,3);
    v |= wbits(sei_decode_flags->VclHrdBpPresentFlag,2,2);
    v |= wbits(sei_decode_flags->CpbDpbDelaysPresentFlag,1,1);
    avc_writew(v, &regs->head_PRreg18);
}

void decode_header(struct avc_interrupt_t *interrupt)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short header_end_flag;
    int hsize;

    header_end_flag = readw(&regs->head_PRreg12);
    interrupt->decode_header.l |= header_end_flag;
    debug2("header_end_flag 0x%04X",header_end_flag);

    hsize = get_header_size();
    debug2("header_size %d",hsize);

    if (header_end_flag & (1<<8)) { // SPS
        struct seq_parameter_set_rbsp_t *sps =
            &avc_info->common_info->seq_parameter_set_rbsp;
        debug2("SPS (Sequence Parameter Set)");
        read_sps(sps, &hsize);
        show_sps(sps);
        avc_info->last_sps = *sps;
        read_dec_parreg(&avc_info->common_info->decode_info);
    }

    if (header_end_flag & (1<<7)) { // PPS
        struct pic_parameter_set_rbsp_t *pps =
            &avc_info->common_info->pic_parameter_set_rbsp;
        debug2("PPS (Picture Parameter Set)");
        read_pps(pps, &hsize);
        show_pps(pps);
        avc_info->last_pps = *pps;
    }

    if (header_end_flag & (1<<6)) { // SEI
        unsigned short payload_type = readw(&regs->head_PRreg16);
        debug2("SEI (Supplemental Enhancement Information)");
        read_sei(payload_type, &hsize);
        show_sei(payload_type);
        interrupt->payload_type = payload_type;
    }

    if (header_end_flag & (1<<5)) { // AUD
        struct aud_parameters_t *aud =
            &avc_info->common_info->aud_parameters;
        debug2("AUD (Access Unit Delimiter)");
        aud->primary_pic_type = readw(&regs->head_PRreg29) & 0x7;
        show_aud(aud);
    }

    if (header_end_flag & (1<<4)) { // EOSEQ
        debug2("End of sequence (EOSEQ)");
    }

    if (header_end_flag & (1<<3)) { // SH
        struct slice_header_t *sh = &avc_info->common_info->slice_header;
        debug2("SH (Slice Header)");
        read_sh(sh, &hsize);
        show_sh(sh);
    }

    if (header_end_flag & (1<<2)) { // EOS
        debug2("End of stream (EOS)");
    }

    if (header_end_flag & (1<<1)) { // FD
        debug2("Filler data (FD)");
    }

    if (hsize > 0) {
        debug2("header_size %d remains !!",hsize);
    }
}

void decode_error(struct avc_interrupt_t *interrupt)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short reg;

    debug2("!!");

    // NAL error
    reg = readw(&regs->head_PRreg1);
    interrupt->decode_error.err_nal.l |= reg;
    if (reg) {
        debug2("NAL error: head_PRreg1 0x%04X",reg);
        if (reg & (1<<3)) debug2(" err_nal_end");
        if (reg & (1<<2)) debug2(" err_forbidden_zero_bit");
        if (reg & (1<<1)) debug2(" err_ref_idc");
        if (reg & (1<<0)) debug2(" err_nal_unit_type");
    }

    // SPS error
    reg = readw(&regs->head_PRreg2);
    interrupt->decode_error.err_sps.l = reg;
    if (reg) {
        debug2("SPS error: head_PRreg2 0x%04X",reg);
        if (reg & (1<<13)) debug2(" HRD error");
        if (reg & (1<<12)) debug2(" VUI error");
        if (reg & (1<<11)) debug2(" SPS other error");
        if (reg & (1<<10)) debug2(" err_direct_8x8_inference_flag");
        if (reg & (1<<9))  debug2(" err_mb_adaptive_frame_field_flag");
        if (reg & (1<<8))  debug2(" err_profile");
        if (reg & (1<<7))  debug2(" err_level_idc");
        if (reg & (1<<6))  debug2(" err_seq_parameter_set_id");
        if (reg & (1<<5))  debug2(" err_log2_max_frame_num_minus4");
        if (reg & (1<<4))  debug2(" err_pic_order_cnt_type");
        if (reg & (1<<3))  debug2(" err_log2_max_pic_order_cnt_lsb_minus4");
        if (reg & (1<<2))  debug2(" err_num_ref_frames");
        if (reg & (1<<1))  debug2(" err_pic_width_in_mbs_minus1");
        if (reg & (1<<0))  debug2(" err_pic_height_in_map_units_minus1");
    }

    // PPS error
    reg = readw(&regs->head_PRreg3);
    interrupt->decode_error.err_pps.l = reg;
    if (reg) {
        debug2("PPS error: head_PRreg3 0x%04X",reg);
        if (reg & (1<<9)) debug2(" err_pps_pps_id");
        if (reg & (1<<8)) debug2(" err_pps_sps_id");
        if (reg & (1<<7)) debug2(" err_pps_entropy_flag");
        if (reg & (1<<6)) debug2(" err_pps_num_slice");
        if (reg & (1<<5)) debug2(" err_pps_num_ref_idx");
        if (reg & (1<<4)) debug2(" err_pps_weighted_flag");
        if (reg & (1<<3)) debug2(" err_pps_weighted_idc");
        if (reg & (1<<2)) debug2(" err_pps_pic_init_q");
        if (reg & (1<<1)) debug2(" err_pps_cqp_idx_ofs");
        if (reg & (1<<0)) debug2(" err_pps_redundant_flag");
    }

    // SH error
    reg = readw(&regs->head_PRreg4);
    interrupt->decode_error.err_sh.l = reg;
    if (reg) {
        debug2("SH error: head_PRreg4 0x%04X",reg);
        if (reg & (1<<9)) debug2(" err_sh_end");
        if (reg & (1<<8)) debug2(" err_sh_slice_type");
        if (reg & (1<<7)) debug2(" err_sh_pps_id");
        if (reg & (1<<6)) debug2(" err_sh_idr_pic_id");
        if (reg & (1<<5)) debug2(" err_sh_num_ref_idx_l0");
        if (reg & (1<<4)) debug2(" err_sh_dis_dbf_idc");
        if (reg & (1<<3)) debug2(" err_sh_slc_a_c0_ofs");
        if (reg & (1<<2)) debug2(" err_sh_slc_beta_ofs");
        if (reg & (1<<1)) debug2(" err_sh_ropn_idc");
        if (reg & (1<<0)) debug2(" err_sh_mmco");
    }

    // SD error
    reg = readw(&regs->head_PRreg5);
    interrupt->decode_error.err_sd.l = reg;
    if (reg) {
        debug2("SD error: head_PRreg5 0x%04X",reg);
        if (reg & (1<<11)) debug2(" err_1st_mb_in_slice");
        if (reg & (1<<10)) debug2(" err_sd_end");
        if (reg & (1<<9))  debug2(" err_sd_mb_type");
        if (reg & (1<<8))  debug2(" err_sd_cbp");
        if (reg & (1<<7))  debug2(" err_sd_mbp_ref_idx");
        if (reg & (1<<6))  debug2(" err_sd_mbp_icpm");
        if (reg & (1<<5))  debug2(" err_sd_smb_ref_idx");
        if (reg & (1<<4))  debug2(" err_zsl_lt_run");
        if (reg & (1<<3))  debug2(" err_cv_coeft");
        if (reg & (1<<2))  debug2(" err_cv_lv_prefix");
        if (reg & (1<<1))  debug2(" err_cv_tzeros");
        if (reg & (1<<0))  debug2(" err_cv_run_before");
    }

    // macro block number
    reg = readw(&regs->head_PRreg6);
    interrupt->decode_error.macro_block_number = reg;
    debug2("macro block number: head_PRreg6 0x%04X",reg);
}

/* set_mem_plane */

void set_mem_plane(struct avc_user_set_parameter_t *usp)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short max_work_num;

    max_work_num = readw(&regs->head_PRreg19);
    avc_writew((max_work_num & MAX_WORK_NUM_BIT)
	       | ((usp->set_mem_num_wr << 8) & MEM_NUM_WR_BIT),
		  &regs->head_PRreg19);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx0 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx1),
	       &regs->head_PRreg20);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx2 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx3),
	       &regs->head_PRreg21);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx4 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx5),
	       &regs->head_PRreg22);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx6 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx7),
	       &regs->head_PRreg23);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx8 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx9),
	       &regs->head_PRreg24);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx10 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx11),
	       &regs->head_PRreg25);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx12 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx13),
	       &regs->head_PRreg26);

    avc_writew((usp->mem_num_ref_idx.set_ref_idx14 << 8)
	       | (usp->mem_num_ref_idx.set_ref_idx15),
	       &regs->head_PRreg27);
}

/* command and interruption -------------------------------------------------*/

/*!
 * Set HW-IP work address
 * @param[in] *wa struct avc_work_address_t
 * @return void
 */
void set_work_address(struct avc_work_address_t *wa)
{
    struct avc_regs_t *regs = avc_info->regs;

    avc_writel(wa->work_yuv01, &regs->bar1);
    avc_writel(wa->work_yuv02, &regs->bar2);
    avc_writel(wa->work_yuv03, &regs->bar3);
    avc_writel(wa->work_yuv04, &regs->bar4);
    avc_writel(wa->work_yuv05, &regs->bar5);
    avc_writel(wa->work_yuv06, &regs->bar6);
    avc_writel(wa->work_yuv07, &regs->bar7);
    avc_writel(wa->work_yuv08, &regs->bar8);
    avc_writel(wa->work_yuv09, &regs->bar9);
    avc_writel(wa->work_yuv10, &regs->bar10);
    avc_writel(wa->work_yuv11, &regs->bar11);
    avc_writel(wa->work_yuv12, &regs->bar12);
    avc_writel(wa->work_yuv13, &regs->bar13);
    avc_writel(wa->work_yuv14, &regs->bar14);
    avc_writel(wa->work_yuv15, &regs->bar15);
    avc_writel(wa->work_yuv16, &regs->bar16);
    avc_writel(wa->work_yuv17, &regs->bar17);
    avc_writel(wa->work_yuv18, &regs->bar18);
    avc_writew(wa->max_work_num, &regs->head_PRreg19);
}

/*!
 * Set stream address
 * @param[in] *ba struct avc_buffer_address_t
 * @return void
 */
void set_stream_address(struct avc_buffer_address_t *ba)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short sel;

    // set stream0 to BAR19
    avc_writel(ba->stream0_base, &regs->bar19);
    avc_writel(ba->stream0_size, &regs->buf_size_reg19);

    // set stream1 to BAR20
    avc_writel(ba->stream1_base, &regs->bar20);
    avc_writel(ba->stream1_size, &regs->buf_size_reg20);

    // stream bar19/bar20
    sel = (ba->stream_select) ? 1 : 0;
    avc_writel((0x2|sel), &regs->stream_buf_sel);
}

/*!
 * Set raster address
 * @param[in] *ba struct avc_buffer_address_t
 * @return void
 */
void set_raster_address(struct avc_buffer_address_t *ba)
{
    struct avc_regs_t *regs = avc_info->regs;

    // set raster0 to raster YUV #21
    avc_writel(ba->raster0_Y, &regs->bar21y);
    avc_writel(ba->raster0_U, &regs->bar21u);
    avc_writel(ba->raster0_V, &regs->bar21v);

    // set raster1 to raster YUV #22
    avc_writel(ba->raster1_Y, &regs->bar22y);
    avc_writel(ba->raster1_U, &regs->bar22u);
    avc_writel(ba->raster1_V, &regs->bar22v);

    // raster_work bar21/bar22
    avc_writel((ba->raster_select)?1:0, &regs->raster1);
}

/*!
 * Set interruption registers
 * @return void
 */
void set_interruption(struct avc_user_set_parameter_t *usp)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short v;

    // clear
    v = readw(&regs->head_PRreg1);  // err_nal
    v = readw(&regs->head_PRreg2);  // err_sps
    v = readw(&regs->head_PRreg3);  // err_pps
    v = readw(&regs->head_PRreg4);  // err_sh
    v = readw(&regs->head_PRreg5);  // err_sd
    v = readw(&regs->head_PRreg12); // header_end

    // enable int
    avc_writew(usp->set_int_en, &regs->INT3reg);

    // header_end_en all
    avc_writew(usp->set_header_end_en, &regs->head_PRreg13); // header_end_en

    // header error enable all
    avc_writew(usp->set_err_nal_en, &regs->head_PRreg7);  //err_nal
    avc_writew(usp->set_err_sps_en, &regs->head_PRreg8);  //err_sps
    avc_writew(usp->set_err_pps_en, &regs->head_PRreg9);  //err_pps
    avc_writew(usp->set_err_sh_en, &regs->head_PRreg10); //err_sh
    avc_writew(usp->set_err_sd_en, &regs->head_PRreg11); //err_sd

    // int_buf_clr
    avc_writel(1, &regs->BINT5);

    // int_buf_en : stream buffer full
    avc_writel(1, &regs->BINT3);
}

/*!
 * Reset HW-IP
 * @return void
 */
void avc_hw_reset(void)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short command = (1<<11); // software_reset
    avc_writew(command, &regs->CMDreg);
    udelay(10); // >100cycle=1.2us
}

/*!
 * Set command
 * @param[in] command
 * @return void
 */
void avc_hw_command(unsigned short command)
{
    struct avc_regs_t *regs = avc_info->regs;
    avc_writew(command, &regs->CMDreg);
}

static int reverse_stream(void *virt, unsigned int length);

/*!
 * Stream change command
 * @return void
 */
void stream_change(void)
{
    struct avc_buffer_address_t *ba =
        &avc_info->common_info->avc_buffer_address;
    debug1("");

//    set_stream_address(ba);

    if (avc_info->state == state_decoding ||
        avc_info->state == state_decode_suspended) {
        reverse_stream(avc_info->streambuf_virt, avc_info->streambuf_size);
    }

    set_stream_address(ba);
}

static unsigned int byte_order(unsigned int x) {
    unsigned int y =
        ((x & 0x000000FF) << 24) |
        ((x & 0x0000FF00) <<  8) |
        ((x & 0x00FF0000) >>  8) |
        ((x & 0xFF000000) >> 24);
    return y;
}

int reverse_stream(void *virt, unsigned int length)
{
    unsigned int *va = (unsigned int *)virt;
    int i;

    if (avc_info->common_info->avc_buffer_address.change_endian == 0) {
        return 0;
    }
    if (length == 0) return 0;
#if 0
    va = (unsigned int *)ioremap((unsigned long)pa, length);
    if (va == NULL) {
        debug0("ioremap failed !! pa 0x%08X, length %d",pa,length);
        return -1;
    }
#endif
    length /= sizeof(int);
    for (i=0; i<length; i++) {
        unsigned int *p = (unsigned int *)(va+i);
        unsigned int  v = byte_order(*p);
        *p = v;
    }
#if 0
    iounmap(va);
#endif
    return 0;
}

/*!
 * Decode start command
 * @param[in] last_frame last frame or not
 * @return void
 */
void decode_start(int last_frame)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct avc_work_address_t *wa = &avc_info->common_info->avc_work_address;
    struct avc_buffer_address_t *ba =
        &avc_info->common_info->avc_buffer_address;
    unsigned short command;
    struct avc_user_set_parameter_t *usp = &avc_info->common_info->avc_user_set_parameter;

    debug1("");

    avc_hw_reset();

    set_interruption(usp);
    set_work_address(wa);
    set_raster_address(ba);
    reverse_stream(avc_info->streambuf_virt, avc_info->streambuf_size);
    set_stream_address(ba);

    // ref_hw_ctrl, dec_error_conceal
    avc_writew((usp->set_ref_hw_ctrl << 2) |
	       (usp->set_dec_error_conceal << 0),
	       &regs->dec_PARreg3);

    /* mem num */
    set_mem_plane(usp);

    // flags for SEI
    set_sei_decode_flags();

    // sei_raw_output : SEI payload type 0-21
    avc_writew((usp->set_sei_raw_output & 0xFFFF),
		&regs->head_PRreg31);
    avc_writew((usp->set_sei_raw_output >> 16) & 0x003F ,
	       &regs->head_PRreg32);

    /* set planar sel */
    avc_writel(usp->set_planar_sel, &regs->raster2);

    /* set bytelane*/
    avc_writel(usp->set_bytelane, &regs->raster3);

    // command
    command = (1<<14); // sys_dec_start
    if (last_frame) command |= (1<<12); // sys_dec_stop
    avc_hw_command(command);
}

/*!
 * Decode reactivate command
 * @param[in] last_frame last frame or not
 * @return void
 */
void decode_reactivate(int last_frame)
{
    unsigned short command;
    struct avc_user_set_parameter_t *usp = &avc_info->common_info->avc_user_set_parameter;
    debug1("");

    set_raster_address(&avc_info->common_info->avc_buffer_address);
    set_sei_decode_flags();

    set_mem_plane(usp);

    command = (1<<8); // dec_reactivate
    if (last_frame) command |= (1<<12); // sys_dec_stop
    avc_hw_command(command);
}

#define ENCPARAM_ENTRY(_name, _min, _max, _def) \
 {#_name, offsetof(struct encode_info_t, _name), _min, _max, _def}

struct encode_params_t {
    const char *name;
    int offset;
    unsigned short min;
    unsigned short max;
    unsigned short def;
};

static struct encode_params_t encode_params[] = {
    ENCPARAM_ENTRY(enc_width, 80, 720, 720),
    ENCPARAM_ENTRY(enc_height, 80, 576, 480),
    ENCPARAM_ENTRY(enc_rc_rate, 16, 8000, 2000),
    ENCPARAM_ENTRY(enc_rc_frame_rate, 1, 62, 30),
    ENCPARAM_ENTRY(enc_ifrm_rate, 1, 1023, 15),
    ENCPARAM_ENTRY(sps_rate, 0, 1023, 0),
    ENCPARAM_ENTRY(pps_rate, 0, 1023, 0),
    ENCPARAM_ENTRY(enc_max_rate, 0, 12000, 3000),
};

/*!
 * Check encode parameters
 * @param[in] *encode_info struct encode_info_t
 * @retval  0 OK
 * @retval -1 NG
 */
int encode_info_check(struct encode_info_t *encode_info)
{
    struct encode_params_t *param;
    int i;

    for (i=0, param=encode_params;
         i<sizeof(encode_params)/sizeof(struct encode_params_t);
         i++, param++)
    {
        char *fieldp = (char *)encode_info + param->offset;
        unsigned short value = *(unsigned short *)fieldp;
        debug2("%d, name %s, offset %d, min %u, max %u, value %u",
               i,param->name,param->offset,param->min,param->max,value);

        if (value < param->min || value > param->max) {
            debug1("error %s %u",param->name,value);
            return -1;
        }
    }

    return 0;
}

#define write_uint_ereg(_v,_r1,_r2) \
do { \
 avc_writew( _v     &0xFFFF,&regs->enc_PARreg##_r1); \
 avc_writew((_v>>16)&0xFFFF,&regs->enc_PARreg##_r2); \
} while(0)

/*!
 * Set encode parameters
 * @param[in] *ei struct encode_info_t
 * @return void
 */
static void set_encode_param(struct encode_info_t *ei)
{
    struct avc_regs_t *regs = avc_info->regs;
    unsigned short v;

    v = (1<<14) | ei->enc_width; // VBR, width
    avc_writew(v, &regs->enc_PARreg1);
    avc_writew(ei->enc_height, &regs->enc_PARreg2);
    avc_writew(ei->enc_rc_rate, &regs->enc_PARreg3);
    v  = wbits(ei->enc_rc_frame_rate,15,10);
    v |= wbits(ei->enc_ifrm_rate,9,0);
    avc_writew(v, &regs->enc_PARreg4);
    avc_writew(ei->sps_rate, &regs->enc_PARreg5);
    avc_writew(ei->pps_rate, &regs->enc_PARreg6);

    if (ei->frm_crop_flag) {
        v  = wbits(1,15,15);
        v |= wbits(ei->frm_crop_rect_lft_oft,9,0);
        avc_writew(v, &regs->enc_PARreg7);
        avc_writew(ei->frm_crop_rect_rgt_oft, &regs->enc_PARreg8);
        avc_writew(ei->frm_crop_rect_top_oft, &regs->enc_PARreg9);
        avc_writew(ei->frm_crop_rect_btm_oft, &regs->enc_PARreg10);
    } else {
        avc_writew(0, &regs->enc_PARreg7);
    }

    v  = wbits(ei->const_intra_pred_flg,15,15);
    v |= wbits(ei->chroma_qp_idx_oft,4,0);
    avc_writew(v, &regs->enc_PARreg11);

    v  = wbits(ei->dis_deblk_filter_idc_i,9,8);
    v |= wbits(ei->slice_alpha_oft_div2_i,7,4);
    v |= wbits(ei->slice_beta_oft_div2_i, 3,0);
    avc_writew(v, &regs->enc_PARreg12);

    v  = wbits(ei->dis_deblk_filter_idc_p,9,8);
    v |= wbits(ei->slice_alpha_oft_div2_p,7,4);
    v |= wbits(ei->slice_beta_oft_div2_p, 3,0);
    avc_writew(v, &regs->enc_PARreg13);

    v = ei->enc_cpb_size;
    if ((v == 0) || (v > 511)) {
        v = 0x70; /* Set default value */
    }
    avc_writew(v, &regs->enc_PARreg14);

    // enc_max_rate: enc_rc_rate * 1.5 recommended [16..12000]
    v = ei->enc_max_rate;
    if (v == 0) {
        v = ei->enc_rc_rate * 3 / 2;
    } else if (v < ei->enc_rc_rate) {
        v = ei->enc_rc_rate;
    }
    avc_writew(v, &regs->enc_PARreg15);

    // enc_PARreg16: Maximum QP
    v = ei->enc_image_value;
    if ((v < 1) || (v > 0x33)) {
        v = 0x33; // Set default value (MaxQP = 51)
    }
    avc_writew(v, &regs->enc_PARreg16);

    avc_writew(ei->level_idc, &regs->enc_PARreg19);

    // enc_PARreg22: Initial QP
    v = ei->enc_initial_image_value;
    if ((v < 0x10) || (v > 0x30)) {
        v = 0x19; // Set default value (InitQP = 25)
    }
    avc_writew(v, &regs->enc_PARreg22);

    // enc_PARreg23: Decrement Q for IDR
    v = ei->enc_islice_image_mode;
    if (v > 0x08) {
        v = 0x08; // Set default value (DecQ = 8)
    }
    avc_writew(v, &regs->enc_PARreg23);

    /*
      enc_RCreg54 (2E0h: test register): Minimum QP
      [15:12] reserved
      [11: 6] Minimum QP for I-pic (default:1)
      [ 5: 0] Minimum QP for P-pic (default:1)
    */
    {
        unsigned long  enc_RCreg54_addr;
        unsigned short max_qp;
        unsigned short min_qp_i;
        unsigned short min_qp_p;

        // enc_RCreg54 offset addr = 0x02E0
        enc_RCreg54_addr = (unsigned long)regs + 0x02E0;

        max_qp = readw(&regs->enc_PARreg16);

        min_qp_i = ei->enc_min_qp_i;
        if ((min_qp_i < 1) || (min_qp_i > max_qp)) {
            min_qp_i = 1;
        }

        min_qp_p = ei->enc_min_qp_p;
        if ((min_qp_p < 1) || (min_qp_p > max_qp)) {
            min_qp_p = 1;
        }

        v = (min_qp_i << 6) | min_qp_p;
        avc_writew(v, enc_RCreg54_addr);
    }

    // slice_mb_num
    v = ei->slice_mb_num;
    if (v > 0) {
        unsigned short max_v;

        /* Saturate slice_mb_num to max MB num */
        max_v = ei->enc_width * ei->enc_height / 256;
        if (v > max_v) {
            v = max_v;
        }

        /* Enable multi slice */
        v |= (1<<15);
    }
    avc_writew(v, &regs->enc_PARreg28);

    if (ei->enc_vui_present_flag) {
        struct vui_parameters_t *vui =
            &avc_info->common_info->seq_parameter_set_rbsp.vui_parameters;
        struct hrd_parameters_t *nal_hrd = &vui->nal_hrd_parameters;
        struct hrd_parameters_t *vcl_hrd = &vui->vcl_hrd_parameters;

        // VUI 33-46
        avc_writew(1, &regs->enc_PARreg33); //enc_vui_present_flag
        v  = wbits(vui->aspect_ratio_info_present_flag,8,8);
        v |= wbits(vui->aspect_ratio_idc,7,0);
        avc_writew(v, &regs->enc_PARreg34);
        avc_writew(vui->sar_width,  &regs->enc_PARreg35);
        avc_writew(vui->sar_height, &regs->enc_PARreg36);
        v  = wbits(vui->overscan_info_present_flag,15,15);
        v |= wbits(vui->overscan_appropriate_flag,14,14);
        v |= wbits(vui->video_signal_type_present_flag,13,13);
        v |= wbits(vui->video_format,12,10);
        v |= wbits(vui->video_full_range_flag,9,9);
        v |= wbits(vui->colour_description_present_flag,8,8);
        v |= wbits(vui->colour_primaries,7,0);
        avc_writew(v, &regs->enc_PARreg37);
        v  = wbits(vui->transfer_characteristics,15,8);
        v |= wbits(vui->matrix_coefficients,7,0);
        avc_writew(v, &regs->enc_PARreg38);
        v  = wbits(vui->chroma_loc_info_present_flag,8,8);
        v |= wbits(vui->chroma_sample_loc_type_top_field,7,5);
        v |= wbits(vui->chroma_sample_loc_type_bottom_field,4,2);
        v |= wbits(vui->timing_info_present_flag,1,1);
        v |= wbits(vui->fixed_frame_rate_flag,0,0);
        avc_writew(v, &regs->enc_PARreg39);
        write_uint_ereg(vui->num_units_in_tick,40,41);
        write_uint_ereg(vui->time_scale,42,43);
        v  = wbits(vui->nal_hrd_parameters_present_flag,15,15);
        v |= wbits(vui->vcl_hrd_parameters_present_flag,14,14);
        v |= wbits(vui->low_delay_hrd_flag,13,13);
        v |= wbits(vui->pic_struct_present_flag,12,12);
        v |= wbits(vui->bitstream_restriction_flag,11,11);
        v |= wbits(vui->motion_vectors_over_pic_boundaries_flag,10,10);
        v |= wbits(vui->max_bytes_per_pic_denom,9,5);
        v |= wbits(vui->max_bits_per_mb_denom,4,0);
        avc_writew(v, &regs->enc_PARreg44);
        v  = wbits(vui->log2_max_mv_length_horizontal,14,10);
        v |= wbits(vui->log2_max_mv_length_vertical,9,5);
        v |= wbits(vui->num_reorder_frames,4,0);
        avc_writew(v, &regs->enc_PARreg45);
        v  = wbits(vui->max_dec_frame_buffering,4,0);
        avc_writew(v, &regs->enc_PARreg46);

        // HRD(NAL) 47-51
        v  = wbits(nal_hrd->cbr_flag[0],13,13);
        v |= wbits(nal_hrd->cpb_cnt_minus1&0x1,12,8); // 00-01h
        v |= wbits(nal_hrd->bit_rate_scale,7,4);
        v |= wbits(nal_hrd->cpb_size_scale,3,0);
        avc_writew(v, &regs->enc_PARreg47);
        write_uint_ereg(nal_hrd->bit_rate_value_minus1[0],48,49);
        write_uint_ereg(nal_hrd->cpb_size_value_minus1[0],50,51);

        // HRD(VCL) 52-56
        v  = wbits(vcl_hrd->cbr_flag[0],13,13);
        v |= wbits(vcl_hrd->cpb_cnt_minus1 & 0x1,12,8); // 00-01h
        v |= wbits(vcl_hrd->bit_rate_scale,7,4);
        v |= wbits(vcl_hrd->cpb_size_scale,3,0);
        avc_writew(v, &regs->enc_PARreg52);
        write_uint_ereg(vcl_hrd->bit_rate_value_minus1[0],53,54);
        write_uint_ereg(vcl_hrd->cpb_size_value_minus1[0],55,56);

        // HRD 57-58
        v  = wbits(vcl_hrd->initial_cpb_removal_delay_length_minus1,9,5);
        v |= wbits(vcl_hrd->cpb_removal_delay_length_minus1,4,0);
        avc_writew(v, &regs->enc_PARreg57);
        v  = wbits(vcl_hrd->dpb_output_delay_length_minus1,9,5);
        v |= wbits(vcl_hrd->time_offset_length,4,0);
        avc_writew(v, &regs->enc_PARreg58);
    } else {
        avc_writew(0, &regs->enc_PARreg33); //enc_vui_present_flag
    }

    // SEI enable 59
    v  = wbits(ei->enc_buffering_period_en,3,3);
    v |= wbits(ei->enc_pic_timing_en,2,2);
    v |= wbits(ei->enc_pan_scan_rect_en,1,1);
    v |= wbits(ei->enc_sei_filler_en,0,0);
    avc_writew(v, &regs->enc_PARreg59);

    // SEI 0: buffering_period 60-67
    if (ei->enc_buffering_period_en) {
        struct buffering_period_t *sei =
            &avc_info->common_info->buffering_period;
        write_uint_ereg(sei->nal_initial_cpb_removal_delay[0],60,61);
        write_uint_ereg(sei->nal_initial_cpb_removal_delay_offset[0],62,63);
        write_uint_ereg(sei->vcl_initial_cpb_removal_delay[0],64,65);
        write_uint_ereg(sei->vcl_initial_cpb_removal_delay_offset[0],66,67);
    }

    // SEI 1: pic_timing 68-77
    if (ei->enc_pic_timing_en) {
        struct pic_timing_t *sei =
            &avc_info->common_info->pic_timing;
        write_uint_ereg(sei->cpb_removal_delay,68,69);
        write_uint_ereg(sei->dpb_output_delay, 70,71);
        v  = wbits(sei->pic_struct,15,12);
        v |= wbits(sei->ct[0].clock_timestamp_flag,11,9);
        v |= wbits(sei->ct[0].ct_type,8,7);
        v |= wbits(sei->ct[0].nuit_field_based_flag,6,6);
        v |= wbits(sei->ct[0].counting_type,5,1);
        v |= wbits(sei->ct[0].full_timestamp_flag,0,0);
        avc_writew(v, &regs->enc_PARreg72);
        v  = wbits(sei->ct[0].discontinuity_flag,9,9);
        v |= wbits(sei->ct[0].cnt_dropped_flag,8,8);
        v |= wbits(sei->ct[0].n_frames,7,0);
        avc_writew(v, &regs->enc_PARreg73);
        v  = wbits(sei->ct[0].seconds_value,11,6);
        v |= wbits(sei->ct[0].minutes_value,5,0);
        avc_writew(v, &regs->enc_PARreg74);
        v  = wbits(sei->ct[0].hours_value,7,3);
        v |= wbits(sei->ct[0].seconds_flag,2,2);
        v |= wbits(sei->ct[0].minutes_flag,1,1);
        v |= wbits(sei->ct[0].hours_flag,0,0);
        avc_writew(v, &regs->enc_PARreg75);
        write_uint_ereg(sei->ct[0].time_offset,76,77);
    }

    // SEI 2: pan_scan_rect 78-105
    if (ei->enc_pan_scan_rect_en) {
        struct pan_scan_rect_t *sei =
            &avc_info->common_info->pan_scan_rect;
        write_uint_ereg(sei->pan_scan_rect_id,78,79);
        v  = wbits(sei->pan_scan_rect_cancel_flag,2,2);
        v |= wbits(sei->pan_scan_cnt_minus1,1,0);
        avc_writew(v, &regs->enc_PARreg80);
        write_uint_ereg(sei->psr[0].left_offset,81,82);
        write_uint_ereg(sei->psr[1].left_offset,83,84);
        write_uint_ereg(sei->psr[2].left_offset,85,86);
        write_uint_ereg(sei->psr[0].right_offset,87,88);
        write_uint_ereg(sei->psr[1].right_offset,89,90);
        write_uint_ereg(sei->psr[2].right_offset,91,92);
        write_uint_ereg(sei->psr[0].top_offset,93,94);
        write_uint_ereg(sei->psr[1].top_offset,95,96);
        write_uint_ereg(sei->psr[2].top_offset,97,98);
        write_uint_ereg(sei->psr[0].bottom_offset,99,100);
        write_uint_ereg(sei->psr[1].bottom_offset,101,102);
        write_uint_ereg(sei->psr[2].bottom_offset,103,104);
        avc_writew(sei->pan_scan_rect_repetition_period, &regs->enc_PARreg105);
    }

    // SEI 3: filler 106-107
    // enc_sei_filler_en/enc_fil_on
    {
        struct filler_payload_t *sei =
            &avc_info->common_info->filler_payload;
        write_uint_ereg(sei->payloadSize,106,107);
    }

    // AUD,FD insertion 108
    v  = wbits(ei->enc_aud_on,1,1);
    v |= wbits(ei->enc_fil_on,0,0);
    avc_writew(v, &regs->enc_PARreg108);
}

/*!
 * Encode start command
 * @param[in] last_frame last frame or not
 * @return void
 */
void encode_start(int last_frame)
{
    struct avc_work_address_t *wa = &avc_info->common_info->avc_work_address;
    struct avc_buffer_address_t *ba =
        &avc_info->common_info->avc_buffer_address;
    struct encode_info_t *ei = &avc_info->common_info->encode_info;
    unsigned short command;
    struct avc_user_set_parameter_t *usp = &avc_info->common_info->avc_user_set_parameter;
    debug1("");

    avc_hw_reset();

    set_interruption(usp);
    set_work_address(wa);
    set_raster_address(ba);
    set_stream_address(ba);
    set_encode_param(ei);

    // command
    command = (1<<15); // sys_enc_start
    if (last_frame) command |= (1<<13); // sys_enc_end
    avc_hw_command(command);
}

/*!
 * Encode reactivate command
 * @param[in] last_frame last frame or not
 * @return void
 */
void encode_reactivate(int last_frame)
{
    struct avc_buffer_address_t *ba =
        &avc_info->common_info->avc_buffer_address;
    struct encode_info_t *ei = &avc_info->common_info->encode_info;
    unsigned short command;
    debug1("");

    set_raster_address(ba);
    set_encode_param(ei);

    // command
    command = (1<<15); // sys_enc_start
    if (last_frame) command |= (1<<13); // sys_enc_end
    avc_hw_command(command);
}

static const char *avc_command_string[] = {
    "avc_command_encode_start",
    "avc_command_encode_reactivate",
    "avc_command_decode_start",
    "avc_command_decode_reactivate",
    "avc_command_stream_change",
    "avc_command_reset",
};

static const char *avc_state_string[] = {
    "state_initial",
    "state_idle",
    "state_encoding",
    "state_encode_suspended",
    "state_decoding",
    "state_decode_suspended",
};

void *avc_ioremap(unsigned long offset, size_t size)
{
    void *ret;

    if (size == 0) return NULL;

    ret = ioremap(offset, size);
    if (ret == NULL) {
        debug0("ioremap failed !! offset 0x%08X, size %d",offset,size);
    }

    return ret;
}

void avc_iounmap(void *addr)
{
    iounmap(addr);
}

/*!
 * User command handler
 * @param[in] command
 * @param[in] last_frame last frame or not
 * @retval  0 OK
 * @retval -1 NG
 */
int command_start(unsigned short command, unsigned short last_frame)
{
    enum avc_state_t state = avc_info->state;
    debug2("command %u %s, last_frame %u",
           command, (command <= avc_command_reset) ?
           avc_command_string[command] : "undefined !!", last_frame);
    debug2("state %d %s", state, (state <= avc_command_reset) ?
           avc_state_string[state] : "undefined !!");

    /* ioremap stream buffer */
    if (avc_info->common_info->avc_buffer_address.change_endian &&
        (command == avc_command_encode_start ||
         command == avc_command_decode_start ||
         command == avc_command_stream_change)
        ) {
        struct avc_buffer_address_t *ba =
            &avc_info->common_info->avc_buffer_address;
        void *virt;
        unsigned int phys;
        unsigned int len;
        if (avc_info->streambuf_virt) {
            avc_iounmap(avc_info->streambuf_virt);
            avc_info->streambuf_virt = NULL;
        }
        if (ba->stream_select == 0) {
            phys = ba->stream0_base;
            len  = ba->stream0_size;
        } else {
            phys = ba->stream1_base;
            len  = ba->stream1_size;
        }
        virt = avc_ioremap((unsigned long)phys, len);
        if (virt == NULL) {
            debug0("avc_ioremap failed !!: phys 0x%08X, len %d",phys,len);
            return -1;
        }
        avc_info->streambuf_virt = virt;
        avc_info->streambuf_size = len;
    }

    if (command == avc_command_encode_start) {
        struct encode_info_t *ei = &avc_info->common_info->encode_info;
        if (state != state_idle) {
            debug1("encode_start: %u != state_idle",state);
            return -1;
        }
        if (encode_info_check(ei) < 0) {
            debug1("encode_info_check failed");
            return -1;
        }
        avc_info->state = state_encoding;
        avc_info->encbuf_order_changed = 0;
        encode_start(last_frame);
    }
    else if (command == avc_command_encode_reactivate) {
        struct encode_info_t *ei = &avc_info->common_info->encode_info;
        if (state != state_encode_suspended) {
            debug1("encode_reactivate: %u != state_encode_suspended",state);
            return -1;
        }
        if (encode_info_check(ei) < 0) {
            debug1("encode_info_check failed");
            return -1;
        }
        avc_info->state = state_encoding;
        encode_reactivate(last_frame);
    }
    else if (command == avc_command_decode_start) {
        if (state != state_idle) {
            debug1("decode_start: %u != state_idle",state);
            return -1;
        }
        avc_info->state = state_decoding;
        decode_start(last_frame);
    }
    else if (command == avc_command_decode_reactivate) {
        if (state != state_decode_suspended) {
            debug1("decode_reactivate: %u != state_decode_suspended",state);
            return -1;
        }
        avc_info->state = state_decoding;
        decode_reactivate(last_frame);
    }
    else if (command == avc_command_stream_change) {
        avc_info->encbuf_order_changed = 0;
        stream_change();
    }
    else if (command == avc_command_reset) {
        avc_info->state = state_idle;
        avc_hw_reset();
        if (avc_info->streambuf_virt) {
            avc_iounmap(avc_info->streambuf_virt);
            avc_info->streambuf_virt = NULL;
        }
    }
    else {
        return -1;
    }

    return 0;
}

/*!
 * Interruption handler
 * @param[in] factor interruption factor
 * @param[in] int_a  int_buf_full
 * @retval 0
 */
int avc_interruption_handler(unsigned int factor, unsigned int int_a)
{
    struct avc_regs_t *regs = avc_info->regs;
    struct avc_buffer_address_t *ba =
        &avc_info->common_info->avc_buffer_address;
    struct decode_info_t *di = &avc_info->common_info->decode_info;
    unsigned int stream_buf_adr;
    unsigned short v;

    debug2("factor 0x%04X",factor);
    if (int_a) debug2(" int_buf_full");
    if (factor & (1<<7)) debug2(" int_enc_end");
    if (factor & (1<<6)) debug2(" int_dec_end");
    if (factor & (1<<5)) debug2(" int_dec_err");
    if (factor & (1<<4)) debug2(" int_enc_pic_end");
    if (factor & (1<<3)) debug2(" int_dec_pic_end");
    if (factor & (1<<2)) debug2(" int_head_dec_end");
    if (factor & (1<<1)) debug2(" int_ref_pic_manual");

    if (factor & (1<<5)) { // int_dec_err
        decode_error(&avc_info->interrupt);
    }

    if (factor & (1<<3)) { // int_dec_pic_end
        avc_info->state = state_decode_suspended;
        read_dec_parreg(di);
    }

    if (factor & (1<<4)) { // int_enc_pic_end
        unsigned int bit_cnt;
        avc_info->state = state_encode_suspended;
        /* enc_bit_count */
        bit_cnt  = readw(&regs->enc_INFOreg3) << 16;
        bit_cnt |= readw(&regs->enc_INFOreg4);
        ba->enc_bit_cnt = bit_cnt;
        debug2("enc_bit_cnt 0x%X %u",bit_cnt,bit_cnt);
    }

    if (factor & (1<<2)) { // int_head_dec_end
        avc_info->state = state_decode_suspended;
        decode_header(&avc_info->interrupt);
    }

    if (factor & (1<<6)) { // int_dec_end
        avc_info->state = state_idle;
    }

    if (factor & (1<<7)) { // int_enc_end
        avc_info->state = state_idle;
    }

    /* read action_state and pic_type */
    read_status_reg(&avc_info->interrupt);

    /* read stream_buf_adr */
    stream_buf_adr = readl(&regs->stream_buf_adr);
    ba->stream_buf_adr = stream_buf_adr;
    debug2("stream_buf_adr 0x%08X %u",stream_buf_adr,stream_buf_adr);

    /* read nal_unit_type */
    v = readw(&regs->dec_PARreg8);
    di->nal_ref_idc   = bits(v,6,5);
    di->nal_unit_type = bits(v,4,0);
    debug2("nal_ref_idc   %u", di->nal_ref_idc);
    debug2("nal_unit_type %u", di->nal_unit_type);

    /* change byte order in encoded stream buffer */
    if (avc_info->state == state_encoding ||
        avc_info->state == state_encode_suspended ||
        (factor & (1<<7)) ) { // int_enc_end
        unsigned int base;
        base = (ba->stream_select == 0) ? ba->stream0_base : ba->stream1_base;
        reverse_stream((char *)avc_info->streambuf_virt +
                       avc_info->encbuf_order_changed,
                       stream_buf_adr - avc_info->encbuf_order_changed);
        avc_info->encbuf_order_changed = stream_buf_adr;
    }

    return 0;
}
