/* 2010-08-25: File added by Sony Corporation */
#include <linux/kernel.h> //printk
#include <stdarg.h>

/* debug functions --------------------------------------------------------- */

int debug_level = 0;
static const char debug_prefix[] = "";
#define debug_printf printk
//#define debug_printf __debug_printf

static int debug_nowstring(char *buf, int len)
{
#if 1
    buf[0]='\0'; return 0;
#else
    struct timeval now;
    do_gettimeofday(&now);
    return snprintf(buf,len,"%03ld.%06ld: ",now.tv_sec%1000,now.tv_usec);
#endif
}

#if 0
#define DEBUG_MAX_LENGTH 128
#define DEBUG_MAX_LINE   100
static char debug_logs[DEBUG_MAX_LINE][DEBUG_MAX_LENGTH];
static int  debug_logs_index = 0;

void debug_out(void)
{
    int i;
    for (i=0;i<debug_logs_index;i++) {
        printk("%s",debug_logs[i]);
    }
    debug_logs_index = 0;
}

void __debug_printf(const char *format, ...)
{
    va_list args;
    if (debug_logs_index >= DEBUG_MAX_LINE) return;
    va_start(args, format);
    vsnprintf(debug_logs[debug_logs_index++], DEBUG_MAX_LENGTH, format, args);
    va_end(args);
    if (in_interrupt() == 0 && debug_logs_index > 50) debug_out();
}
#endif

void __debug_np(int level, const char *format, ...)
{
    char sbuf[128];
    va_list args;
    if (debug_level < level) return;

    va_start(args, format);
    vsnprintf(sbuf, sizeof(sbuf), format, args);
    va_end(args);

    debug_printf("%s%s\n", debug_prefix, sbuf);
}

void __debug(int level, const char *function, const char *format, ...)
{
    char sbuf[128];
    char times[20];
    va_list args;
    if (debug_level < level) return;

    va_start(args, format);
    vsnprintf(sbuf, sizeof(sbuf), format, args);
    va_end(args);

    debug_nowstring(times, sizeof(times));
    debug_printf("%s%s%-16s: %s\n", debug_prefix, times, function, sbuf);
}

void __dump(int level, const char *function, void *ptr, int len) {
    unsigned char *p = (unsigned char *)ptr;
    char sbuf[128];
    int i;
    if (debug_level < level) return;

    __debug(level, function, "length=%d", len);
    if (len>32) len=32;

    for (i=0; i<len;) {
        int nl=16;
        char *sbufp = sbuf + sprintf(sbuf, " %04X: ", (unsigned int)(i));
        for (; i<len && nl>0; i++,nl--) {
            sbufp += sprintf(sbufp, "%02X ", p[i]);
        }
        debug_printf("%s\n", sbuf);
    }
}
