/* 2008-02-22: File added and changed by Sony Corporation */
/*
 *  UMA block device driver
 */
/* With non GPL files, use following license */
/*
 * Copyright 2007-2009 Sony Corporation.
 *
 *  Baseed on sbull/sbull.c from code in Linux Device Drivers, Third Edition.
 *
 * Copyright 2005 O'Reilly Media, Inc.
 * "Linux Device Drivers, Third Edition, by Jonathan Corbet, Alessandro
 * Rubini, and Greg Kroah-Hartman."
 *
 *  The	 source code distributed  with this book
 *  can	  be  redistributed in source	 or  binary form   so  long  as an
 *  acknowledgment  appears in derived source	files. The citation should
 *  list that the code comes from BOOK	by AUTHOR, published by O'Reilly &
 *  Associates.	 This  code is under copyright and  cannot be  included in
 *  any other book, publication, or educational product without permission
 *  from O'Reilly &  Associates. No warranty  is attached; we  cannot take
 *  responsibility for errors or fitness for use.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Otherwise with GPL files, use following license */
/*
 * Copyright 2007-2009 Sony Corporation.
 *
 *  Baseed on sbull/sbull.c from code in Linux Device Drivers, Third Edition.
 *
 * Copyright 2005 O'Reilly Media, Inc.
 * "Linux Device Drivers, Third Edition, by Jonathan Corbet, Alessandro
 * Rubini, and Greg Kroah-Hartman."
 *
 *  The	 source code distributed  with this book
 *  can	  be  redistributed in source	 or  binary form   so  long  as an
 *  acknowledgment  appears in derived source	files. The citation should
 *  list that the code comes from BOOK	by AUTHOR, published by O'Reilly &
 *  Associates.	 This  code is under copyright and  cannot be  included in
 *  any other book, publication, or educational product without permission
 *  from O'Reilly &  Associates. No warranty  is attached; we  cannot take
 *  responsibility for errors or fitness for use.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>

#include <linux/sched.h>
#include <linux/kernel.h>	/* printk() */
#include <linux/slab.h>		/* kmalloc() */
#include <linux/fs.h>		/* everything... */
#include <linux/errno.h>	/* error codes */
#include <linux/timer.h>
#include <linux/types.h>	/* size_t */
#include <linux/fcntl.h>	/* O_ACCMODE */
#include <linux/hdreg.h>	/* HDIO_GETGEO */
#include <linux/kdev_t.h>
#include <linux/vmalloc.h>
#include <linux/genhd.h>
#include <linux/blkdev.h>
#include <linux/buffer_head.h>	/* invalidate_bdev */
#include <linux/bio.h>
#include <asm/div64.h>

#include <asm/snsc/emma3_uma.h>

MODULE_LICENSE("Dual BSD/GPL");

#define EMMA3_UMABLK_MAJOR_NUMBER 120

#define EMMA3_UMABLK_MEM0	  0
#define EMMA3_UMABLK_MEM1	  1

static int emma3_umablk_major = EMMA3_UMABLK_MAJOR_NUMBER;
module_param(emma3_umablk_major, int, 0);
static unsigned int hardsect = CONFIG_SNSC_BLK_DEV_UMABLK_HARDSECT;
module_param(hardsect, uint, 0);
static unsigned int mem0_size = CONFIG_SNSC_BLK_DEV_UMABLK_MEM0_SIZE;
module_param(mem0_size, uint, 0);
static unsigned int mem1_size = CONFIG_SNSC_BLK_DEV_UMABLK_MEM1_SIZE;
module_param(mem1_size, uint, 0);

#ifdef CONFIG_SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G
static unsigned int size32 = CONFIG_SNSC_BLK_DEV_UMABLK_MEM0_SIZE;
module_param(size32, uint, 0);
static unsigned int size16 = CONFIG_SNSC_BLK_DEV_UMABLK_MEM1_SIZE;
module_param(size16, uint, 0);
#endif /* CONFIG_SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G */

/*
 * Minor number and partition management.
 */
#define EMMA3_UMABLK_MINORS    16
#define MINOR_SHIFT		4
#define DEVNUM(kdevnum) (MINOR(kdev_t_to_nr(kdevnum)) >> MINOR_SHIFT)

/*
 * We can tweak our hardware sector size, but the kernel talks to us
 * in terms of small sectors, always.
 */
#define KERNEL_SECTOR_SIZE	512

/*
 * The internal representation of our device.
 */
typedef struct emma3_umablk_dev {
	sector_t mem1_size;		/* Device size for VR MEM1 */
	sector_t mem0_size;		/* Device size for VR MEM0 */
	u8 *mem1_data;			/* The data array for VR MEM1 */
	u8 *mem0_data;			/* The data array for VR MEM0 */
	short users;			/* How many users */
	spinlock_t lock;		/* For mutual exclusion */
	struct request_queue *queue;	/* The device request queue */
	struct gendisk *gd;		/* The gendisk structure */
} emma3_umablk_dev_t;

static emma3_umablk_dev_t *Devices = NULL;

/*
 * Handle an I/O request.
 */
static void emma3_umablk_transfer(struct emma3_umablk_dev *dev,
				  sector_t sector,
				  sector_t nsect, char *buffer, int write)
{
	sector_t offset = sector * KERNEL_SECTOR_SIZE;
	sector_t nbytes = nsect * KERNEL_SECTOR_SIZE;

#ifdef UMA_DEBUG
	printk(KERN_ERR "emma3_umablk: %s offset=0x%08llX, nbyte=0x%08llX "
	       "buffer=%p size(MEM0:%llu, MEM1:%llu)\n",
	       (write == READ ? "READ" : "WRITE"), offset, nbytes, buffer,
	       dev->mem0_size, dev->mem1_size);
#endif /* UMA_DEBUG */

	if ((offset + nbytes) > (dev->mem0_size + dev->mem1_size)) {
		printk (KERN_NOTICE "emma3_umablk: Beyond-end write "
			"(%llu %llu)\n", offset, nbytes);
		return;
	}

	if (write) {
		if (offset < dev->mem0_size) {
			if ((offset + nbytes) < dev->mem0_size ) {
				memcpy(dev->mem0_data + offset, buffer, nbytes);
			}
			else {
				memcpy(dev->mem0_data + offset, buffer,
				       dev->mem0_size - offset);
				memcpy(dev->mem1_data,
				       buffer + (dev->mem0_size - offset),
				       (offset + nbytes) - dev->mem0_size);
			}
		}
		else {
			memcpy(dev->mem1_data + (offset - dev->mem0_size),
			       buffer, nbytes);
		}
		/* for virtual alias */
		dma_cache_inv((unsigned long)buffer, nbytes);
	}
	else {
		if (offset < dev->mem0_size) {
			if ((offset + nbytes) < dev->mem0_size ) {
				memcpy(buffer, dev->mem0_data + offset, nbytes);
			}
			else {
				memcpy(buffer, dev->mem0_data + offset,
				       dev->mem0_size - offset);
				memcpy(buffer + (dev->mem0_size - offset),
				       dev->mem1_data,
				       (offset + nbytes) - dev->mem0_size);
			}
		}
		else {
			memcpy(buffer,
			       dev->mem1_data + (offset - dev->mem0_size),
			       nbytes);
		}
		/* for virtual alias and wback cache */
		dma_cache_wback_inv((unsigned long)buffer, nbytes);
	}
}

/*
 * The simple form of the request function.
 */
static void emma3_umablk_request(struct request_queue *q)
{
	struct request *req;

	while ((req = elv_next_request(q)) != NULL) {
		struct emma3_umablk_dev *dev = req->rq_disk->private_data;
		if (! blk_fs_request(req)) {
			printk (KERN_NOTICE "emma3_umablk: Skip non-fs "
				"request\n");
			end_request(req, 0);
			continue;
		}
		emma3_umablk_transfer(dev, req->sector,
				      req->current_nr_sectors,
				      req->buffer, rq_data_dir(req));
		end_request(req, 1);
	}
}


/*
 * Open and close.
 */

static int emma3_umablk_open(struct inode *inode, struct file *filp)
{
	struct emma3_umablk_dev *dev = inode->i_bdev->bd_disk->private_data;

	filp->private_data = dev;
	spin_lock(&dev->lock);
	dev->users++;
	spin_unlock(&dev->lock);
	return 0;
}

static int emma3_umablk_release(struct inode *inode, struct file *filp)
{
	struct emma3_umablk_dev *dev = inode->i_bdev->bd_disk->private_data;

	spin_lock(&dev->lock);
	dev->users--;
	spin_unlock(&dev->lock);

	return 0;
}


/*
 * The getgeo implementation
 */

int emma3_umablk_getgeo(struct block_device *bdev, struct hd_geometry *geo)
{
	struct emma3_umablk_dev *dev = bdev->bd_disk->private_data;
	sector_t size;

	if (dev == NULL) {
		return -ENODEV;
	}

	/*
	 * Get geometry: since we are a virtual device, we have to make
	 * up something plausible.  So we claim 16 sectors, four heads,
	 * and calculate the corresponding number of cylinders. We set the
	 * start of data at sector zero.
	 */
	size = dev->mem0_size + dev->mem1_size;
	geo->cylinders = (unsigned short)((unsigned int)(size & ~0x3f) >> 6);
	geo->heads = 4;
	geo->sectors = 16;
	geo->start = 0;

	return 0;

}


/*
 * The device operations structure.
 */
static struct block_device_operations emma3_umablk_ops = {
	.owner		 = THIS_MODULE,
	.open		 = emma3_umablk_open,
	.release	 = emma3_umablk_release,
	.getgeo		 = emma3_umablk_getgeo
};

static void *
get_memory(sector_t size, int bit)
{
	unsigned long *mem = NULL;
	unsigned long vaddr, umaddr;
	int ret;

	if (bit == EMMA3_UMABLK_MEM0) {
		ret = emma3_uma_request(size >> PAGE_SHIFT,
#if defined (CONFIG_SNSC_UMA_EMMA3_BDRE_4G_6GBIT) || \
    defined (CONFIG_SNSC_UMA_EMMA3_BDRE_5G_5GBIT) || \
    defined (CONFIG_SNSC_UMA_EMMA3_BDRE_5G_6GBIT) || \
    defined (CONFIG_SNSC_UMA_EMMA3_BDRE_6G_5GBIT) || \
    defined (CONFIG_SNSC_UMA_EMMA3_BDRE_7G_5GBIT) || \
    defined (CONFIG_SNSC_UMA_EMMA3_BDRE_7G_6GBIT)
					UMA_HIMEM | UMA_MEM0 | UMA_VR,
#elif defined (CONFIG_SNSC_UMA_EMMA3_BDP_3G_4GBIT) || \
      defined (CONFIG_SNSC_UMA_EMMA3_BDP_4G_4GBIT)
					UMA_LOMEM | UMA_MEM0 | UMA_VR,
#endif
					"umablk_MEM0", &vaddr, &umaddr);
	}
	else {
		ret = emma3_uma_request(size >> PAGE_SHIFT,
					UMA_HIMEM | UMA_MEM1 | UMA_VR,
					"umablk_MEM1", &vaddr, &umaddr);
	}
	if (ret < 0) {
		printk("emma3_umablk: emma3_uma_request() failed: %d\n", ret);
	} else {
		mem = (unsigned long *)vaddr;
		printk(KERN_INFO "emma3_umablk: (%s) vaddr=0x%08lx, "
		       "umaddr=0x%08lx size=0x%08llx\n",
		       (bit == EMMA3_UMABLK_MEM0 ? "MEM0" : "MEM1"),
		       vaddr, umaddr, size);
	}

	if (mem == NULL)
		printk("emma3_umablk: Not enough memory "
		       "(%llu bytes requested)\n", size);
	return mem;
}

#ifdef CONFIG_SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G
static void
check_valid_param(void)
{
	/* check value of mem0, mem1 */
	if ((mem0_size != CONFIG_SNSC_BLK_DEV_UMABLK_MEM0_SIZE) ||
	    (mem1_size != CONFIG_SNSC_BLK_DEV_UMABLK_MEM1_SIZE)) {
		return;
	}

	/* check value of size32, size16 */
	if ((size32 != CONFIG_SNSC_BLK_DEV_UMABLK_MEM0_SIZE) ||
	    (size16 != CONFIG_SNSC_BLK_DEV_UMABLK_MEM1_SIZE)) {
		mem0_size = size32;
		mem1_size = size16;
	}
}
#endif /* CONFIG_SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G */

/*
 * Set up our internal device.
 */
static int setup_device(struct emma3_umablk_dev *dev)
{
	int retVal = 0;
	sector_t mem0_mod, mem1_mod, div;

#ifdef CONFIG_SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G
	check_valid_param();
#endif /* CONFIG_SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G */

	/*
	 * Get some memory.
	 */
	memset (dev, 0, sizeof (struct emma3_umablk_dev));
	dev->mem0_size = mem0_size;
	dev->mem1_size = mem1_size;
	if ((hardsect == 0) || ((hardsect & (KERNEL_SECTOR_SIZE - 1)) != 0)) {
		printk (KERN_ERR "emma3_umablk: hardsect(0x%x) must not be 0 "
			"and multiple of 512\n", hardsect);
		return -EINVAL;
	}
	mem0_mod = (dev->mem0_size) & (hardsect - 1);
	mem1_mod = (dev->mem1_size) & (hardsect - 1);
	if ((dev->mem0_size == 0 && dev->mem1_size == 0) || (mem0_mod != 0) ||
	    (mem1_mod != 0)) {
		printk (KERN_ERR "emma3_umablk: mem1_size(0x%llx) or "
			"mem0_size(0x%llx) must be larger than 0 and multiple "
			"of hardsect size(0x%x) \n", dev->mem1_size,
			dev->mem0_size,
			hardsect);

		return -EINVAL;
	}
	if (dev->mem0_size != 0) {
		dev->mem0_data = get_memory(dev->mem0_size, EMMA3_UMABLK_MEM0);
		if (dev->mem0_data == NULL) {
			return -ENOMEM;
		}
	}
	if (dev->mem1_size != 0) {
		dev->mem1_data = get_memory(dev->mem1_size, EMMA3_UMABLK_MEM1);
		if (dev->mem1_data == NULL) {
			return -ENOMEM;
		}
	}
	spin_lock_init(&dev->lock);

	dev->queue = blk_init_queue(emma3_umablk_request, &dev->lock);
	if (dev->queue == NULL) {
		return -ENOMEM;
	}
	blk_queue_hardsect_size(dev->queue, hardsect);
	dev->queue->queuedata = dev;
	/*
	 * And the gendisk structure.
	 */
	dev->gd = alloc_disk(EMMA3_UMABLK_MINORS);
	if (dev->gd == NULL) {
		printk (KERN_ERR "alloc_disk failure\n");
		retVal = -ENOMEM;
		goto err_setup;
	}

	dev->gd->major = emma3_umablk_major;
	dev->gd->first_minor = 0;
	dev->gd->fops = &emma3_umablk_ops;
	dev->gd->queue = dev->queue;
	dev->gd->private_data = dev;
	snprintf (dev->gd->disk_name, 32, "umablk%c", 'a');
	div = dev->mem0_size + dev->mem1_size;
	(void)do_div(div, KERNEL_SECTOR_SIZE);
	set_capacity(dev->gd, div);
	add_disk(dev->gd);

	return 0;

 err_setup:
	if (dev->queue) {
		blk_cleanup_queue(dev->queue);
	}

	return retVal;
}

/*
 * Delete our internal device.
 */
static void delete_device(void)
{
	struct emma3_umablk_dev *dev = Devices;

	if (dev->gd) {
		del_gendisk(dev->gd);
		put_disk(dev->gd);
	}
	if (dev->queue) {
		blk_cleanup_queue(dev->queue);
	}
}

static int __init emma3_umablk_init(void)
{
	int retVal = 0;

	/*
	 * Get registered.
	 */
	retVal = register_blkdev(emma3_umablk_major, "emma3_umablk");
	if (retVal != 0) {
		printk(KERN_WARNING "emma3_umablk: unable to get major "
		       "number\n");
		return -EBUSY;
	}
	/*
	 * Allocate the device array, and initialize each one.
	 */
	Devices = kmalloc(sizeof (struct emma3_umablk_dev),
			  GFP_KERNEL);
	if (Devices == NULL) {
		retVal = -ENOMEM;
		goto out_unregister;
	}

	retVal = setup_device(Devices);
	if (retVal < 0) {
		printk(KERN_ERR "emma3_umablk: can't set up "
		       "device\n");
		delete_device();
		goto out_unregister;
	}

	return 0;

  out_unregister:
	if (Devices) {
		kfree(Devices);
		Devices = NULL;
	}
	unregister_blkdev(emma3_umablk_major, "emma3_umablk");
	return retVal;
}

static void emma3_umablk_exit(void)
{
	delete_device();
	unregister_blkdev(emma3_umablk_major, "emma3_umablk");
	if (Devices) {
		kfree(Devices);
		Devices = NULL;
	}
}

module_init(emma3_umablk_init);
module_exit(emma3_umablk_exit);

/* options - nonmodular */
#ifndef MODULE
static int __init umablk_hardsect(char *str)
{
	hardsect = simple_strtol(str, NULL, 0);
	return 1;
}
static int __init umablk_mem1_size(char *str)
{
	mem1_size = simple_strtol(str, NULL, 0);
	return 1;
}
static int __init umablk_mem0_size(char *str)
{
	mem0_size = simple_strtol(str, NULL, 0);
	return 1;
}
#ifdef SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G
static int __init umablk_size16(char *str)
{
	size16 = simple_strtol(str, NULL, 0);
	return 1;
}
static int __init umablk_size32(char *str)
{
	size32 = simple_strtol(str, NULL, 0);
	return 1;
}
#endif /* SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G */

__setup("umablk_hardsect=", umablk_hardsect);
__setup("umablk_mem1_size=", umablk_mem1_size);
__setup("umablk_mem0_size=", umablk_mem0_size);
#ifdef SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G
__setup("umablk_size16=", umablk_size16);
__setup("umablk_size32=", umablk_size32);
#endif /* SNSC_BLK_DEV_UMABLK_COMPAT_BDRE_4G */

#endif /* MODULE */

MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("emma3 umablk driver");
