/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name       : linux/arch/arm/mach-mp200/pwc.c
 *  Function        : pwc
 *  Release Version : Ver 1.04
 *  Release Date    : 2009/04/02
 *
 *  Copyright (C) NEC Electronics Corporation 2009
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>

#include <asm/delay.h>
#include <asm/mach/irq.h>
#include <asm/gpio.h>
#include <asm/arch/pwc.h>
#include <asm/arch/spi.h>

#ifdef PWC_DEBUG
#define DPRINT(FMT,ARGS...) printk(KERN_INFO "%s(): " FMT, __FUNCTION__,##ARGS)
#else
#define DPRINT(FMT,ARGS...)
#endif

spinlock_t pwc_spinlock = SPIN_LOCK_UNLOCKED;
spinlock_t pwc_write_spinlock = SPIN_LOCK_UNLOCKED;

#define PWC_GIO_OUTPORT1	0xB0
#define PWC_GIO_OUTPORT2	0x03
#define MASK_PWC_IOSEL1		0xFF
#define MASK_PWC_IOSEL2		0xFF
#define PWC_TIMCNTR_INIT_VAL (0x01 << 1)

static SPI_CONFIG device0_config_read = {
	.dev	= SPI_DEV_SP0,
	.cs_sel	= SPI_CS_SEL_CS0,
	.m_s	= SPI_M_S_MASTER,
	.dma	= SPI_DMA_OFF,
	.pol	= SPI_CSW_8CLK | SPI_CK_DLY_ON | SPI_CK_POL_POS | SPI_CS_POL_POS,
 	.tiecs	= SPI_TIECS_NORMAL,
	.nbw	= SPI_NB_8BIT,
	.nbr	= SPI_NB_8BIT,
	.sclk	= SPI_SCLK_12MHZ,
};

static SPI_CONFIG device0_config_write = {
	.dev	= SPI_DEV_SP0,
	.cs_sel	= SPI_CS_SEL_CS0,
	.m_s	= SPI_M_S_MASTER,
	.dma	= SPI_DMA_OFF,
	.pol	= SPI_CSW_8CLK | SPI_CK_DLY_ON | SPI_CK_POL_POS | SPI_CS_POL_POS,
 	.tiecs	= SPI_TIECS_NORMAL,
	.nbw	= SPI_NB_16BIT,
	.nbr	= 0,
	.sclk	= SPI_SCLK_24MHZ,
};

/* PWC register infomation */
typedef struct __pwc_reginfo {
  unsigned char data;
  unsigned char flag;
} pwc_reginfo;

#define PWC_REG_RESERVED       0x00
#define PWC_REG_WRITEONLY      0x01
#define PWC_REG_READONLY       0x02
#define PWC_REG_WRITENOMODIFY  0x04

#define REG_MAX		0xA0


pwc_reginfo cs_table[REG_MAX] =
{
    { 0x00, PWC_REG_RESERVED },/* 00 h */
    { 0x00, PWC_REG_RESERVED },/* 01 h */
    { 0x03, PWC_REG_WRITEONLY},/* 02 h */
    { 0x00, PWC_REG_RESERVED },/* 03 h */
    { 0x00, PWC_REG_WRITEONLY},/* 04 h */
    { 0x00, PWC_REG_RESERVED },/* 05 h */
    { 0x02, PWC_REG_WRITEONLY},/* 06 h *//* change by bootloader. default 0x00 */
    { 0x00, PWC_REG_RESERVED },/* 07 h */
    { 0x00, PWC_REG_WRITEONLY},/* 08 h */
    { 0x00, PWC_REG_RESERVED },/* 09 h */
    { 0xAC, PWC_REG_WRITEONLY},/* 0A h */
    { 0x00, PWC_REG_RESERVED },/* 0B h */
    { 0xA0, PWC_REG_WRITEONLY},/* 0C h */
    { 0x00, PWC_REG_RESERVED },/* 0D h */
    { 0x00, PWC_REG_WRITEONLY},/* 0E h */
    { 0x00, PWC_REG_RESERVED },/* 0F h */
    { 0x80, PWC_REG_WRITEONLY},/* 10 h *//* change by bootloader. default 0x00 */
    { 0x00, PWC_REG_RESERVED },/* 11 h */
    { 0x00, PWC_REG_WRITEONLY},/* 12 h */
    { 0x00, PWC_REG_RESERVED },/* 13 h */
    { 0x00, PWC_REG_WRITEONLY},/* 14 h */
    { 0x00, PWC_REG_READONLY },/* 15 h */
    { 0x00, PWC_REG_WRITEONLY},/* 16 h */
    { 0x00, PWC_REG_READONLY },/* 17 h */
    { 0x00, PWC_REG_WRITEONLY},/* 18 h */
    { 0x00, PWC_REG_RESERVED },/* 19 h */
    { 0x00, PWC_REG_WRITEONLY},/* 1A h */
    { 0x00, PWC_REG_RESERVED },/* 1B h */
    { 0x55, PWC_REG_WRITEONLY},/* 1C h */
    { 0x00, PWC_REG_RESERVED },/* 1D h */
    { 0x55, PWC_REG_WRITEONLY},/* 1E h */
    { 0x00, PWC_REG_RESERVED },/* 1F h */
    { 0x00, PWC_REG_WRITEONLY},/* 20 h */
    { 0x00, PWC_REG_READONLY },/* 21 h */
    { 0x00, PWC_REG_WRITEONLY},/* 22 h */
    { 0x00, PWC_REG_READONLY },/* 23 h */
    { 0x00, PWC_REG_WRITEONLY},/* 24 h */
    { 0x00, PWC_REG_READONLY },/* 25 h */
    { 0x00, PWC_REG_WRITEONLY},/* 26 h */
    { 0x00, PWC_REG_RESERVED },/* 27 h */
    { 0x00, PWC_REG_WRITEONLY},/* 28 h */
    { 0x00, PWC_REG_RESERVED },/* 29 h */
    { 0x00, PWC_REG_WRITEONLY},/* 2A h */
    { 0x00, PWC_REG_RESERVED },/* 2B h */
    { 0x00, PWC_REG_WRITENOMODIFY },/* 2C h */
    { 0x00, PWC_REG_RESERVED },/* 2D h */
    { 0x00, PWC_REG_WRITENOMODIFY },/* 2E h */
    { 0x00, PWC_REG_RESERVED },/* 2F h */
    { 0x00, PWC_REG_WRITENOMODIFY },/* 30 h */
    { 0x00, PWC_REG_RESERVED },/* 31 h */
    { 0x00, PWC_REG_WRITEONLY},/* 32 h */
    { 0x00, PWC_REG_RESERVED },/* 33 h */
    { 0x00, PWC_REG_WRITEONLY},/* 34 h */
    { 0x00, PWC_REG_RESERVED },/* 35 h */
    { 0x00, PWC_REG_WRITEONLY},/* 36 h */
    { 0x00, PWC_REG_RESERVED },/* 37 h */
    { 0x00, PWC_REG_WRITEONLY},/* 38 h */
    { 0x00, PWC_REG_RESERVED },/* 39 h */
    { 0x00, PWC_REG_WRITEONLY},/* 3A h */
    { 0x00, PWC_REG_RESERVED },/* 3B h */
    { 0x00, PWC_REG_WRITEONLY},/* 3C h */
    { 0x00, PWC_REG_RESERVED },/* 3D h */
    { 0x00, PWC_REG_WRITEONLY},/* 3E h */
    { 0x00, PWC_REG_RESERVED },/* 3F h */
    { 0x00, PWC_REG_WRITEONLY},/* 40 h */
    { 0x00, PWC_REG_RESERVED },/* 41 h */
    { 0x00, PWC_REG_WRITEONLY},/* 42 h */
    { 0x00, PWC_REG_RESERVED },/* 43 h */
    { 0x00, PWC_REG_RESERVED },/* 44 h */
    { 0x00, PWC_REG_RESERVED },/* 45 h */
    { 0x00, PWC_REG_RESERVED },/* 46 h */
    { 0x00, PWC_REG_RESERVED },/* 47 h */
    { 0x00, PWC_REG_RESERVED },/* 48 h */
    { 0x00, PWC_REG_RESERVED },/* 49 h */
    { 0x00, PWC_REG_RESERVED },/* 4A h */
    { 0x00, PWC_REG_RESERVED },/* 4B h */
    { 0x00, PWC_REG_WRITEONLY},/* 4C h */
    { 0x00, PWC_REG_READONLY },/* 4D h */
    { 0x00, PWC_REG_WRITEONLY},/* 4E h */
    { 0x00, PWC_REG_READONLY },/* 4F h */
    { 0x00, PWC_REG_WRITEONLY},/* 50 h */
    { 0x00, PWC_REG_READONLY },/* 51 h */
    { 0x00, PWC_REG_WRITEONLY},/* 52 h */
    { 0x00, PWC_REG_READONLY },/* 53 h */
    { 0x00, PWC_REG_RESERVED },/* 54 h */
    { 0x00, PWC_REG_READONLY },/* 55 h */
    { 0x00, PWC_REG_RESERVED },/* 56 h */
    { 0x00, PWC_REG_READONLY },/* 57 h */
    { 0x00, PWC_REG_RESERVED },/* 58 h */
    { 0x00, PWC_REG_READONLY },/* 59 h */
    { 0x00, PWC_REG_RESERVED },/* 5A h */
    { 0x00, PWC_REG_READONLY },/* 5B h */
    { 0x03, PWC_REG_WRITEONLY},/* 5C h */
    { 0x00, PWC_REG_READONLY },/* 5D h */
    { 0x00, PWC_REG_WRITENOMODIFY },/* 5E h */
    { 0x00, PWC_REG_READONLY },/* 5F h */
    { 0x02, PWC_REG_WRITEONLY},/* 60 h */
    { 0x00, PWC_REG_RESERVED },/* 61 h */
    { 0x00, PWC_REG_WRITEONLY},/* 62 h */
    { 0x00, PWC_REG_RESERVED },/* 63 h */
    { 0x00, PWC_REG_WRITEONLY},/* 64 h */
    { 0x00, PWC_REG_READONLY },/* 65 h */
    { 0x00, PWC_REG_WRITENOMODIFY },/* 66 h */
    { 0x00, PWC_REG_RESERVED },/* 67 h */
    { 0x00, PWC_REG_WRITEONLY},/* 68 h */
    { 0x00, PWC_REG_RESERVED },/* 69 h */
    { 0x00, PWC_REG_WRITEONLY},/* 6A h */
    { 0x00, PWC_REG_RESERVED },/* 6B h */
    { 0x00, PWC_REG_WRITEONLY},/* 6C h */
    { 0x00, PWC_REG_READONLY },/* 6D h */
    { 0x00, PWC_REG_WRITENOMODIFY },/* 6E h */
    { 0x06, PWC_REG_READONLY },/* 6F h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 70 h */
    { 0x00, PWC_REG_READONLY },/* 71 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 72 h */
    { 0x00, PWC_REG_READONLY },/* 73 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 74 h */
    { 0x00, PWC_REG_READONLY },/* 75 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 76 h */
    { 0x00, PWC_REG_READONLY },/* 77 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 78 h */
    { 0x00, PWC_REG_READONLY },/* 79 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 7A h */
    { 0x00, PWC_REG_READONLY },/* 7B h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 7C h */
    { 0x00, PWC_REG_READONLY },/* 7D h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 7E h */
    { 0x00, PWC_REG_READONLY },/* 7F h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 80 h */
    { 0x00, PWC_REG_READONLY },/* 81 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 82 h */
    { 0x00, PWC_REG_READONLY },/* 83 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 84 h */
    { 0x00, PWC_REG_READONLY },/* 85 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 86 h */
    { 0x00, PWC_REG_READONLY },/* 87 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 88 h */
    { 0x00, PWC_REG_READONLY },/* 89 h */
    { 0x00, PWC_REG_WRITEONLY},/* 8A h */
    { 0x00, PWC_REG_READONLY },/* 8B h */
    { 0x00, PWC_REG_WRITEONLY},/* 8C h */
    { 0x00, PWC_REG_READONLY },/* 8D h */
    { 0x00, PWC_REG_RESERVED },/* 8E h */
    { 0x00, PWC_REG_RESERVED },/* 8F h */
    { 0x07, PWC_REG_WRITEONLY},/* 90 h */
    { 0x00, PWC_REG_RESERVED },/* 91 h */
    { 0x00, PWC_REG_WRITEONLY},/* 92 h */
    { 0x00, PWC_REG_RESERVED },/* 93 h */
    { 0x00, PWC_REG_WRITENOMODIFY},/* 94 h */
    { 0x00, PWC_REG_RESERVED },/* 95 h */
    { 0x00, PWC_REG_RESERVED },/* 96 h */
    { 0x00, PWC_REG_RESERVED },/* 97 h */
    { 0x00, PWC_REG_RESERVED },/* 98 h */
    { 0x00, PWC_REG_RESERVED },/* 99 h */
    { 0x00, PWC_REG_RESERVED },/* 9A h */
    { 0x00, PWC_REG_RESERVED },/* 9B h */
    { 0x00, PWC_REG_RESERVED },/* 9C h */
    { 0x00, PWC_REG_RESERVED },/* 9D h */
    { 0x00, PWC_REG_RESERVED },/* 9E h */
    { 0x00, PWC_REG_RESERVED },/* 9F h */
};

/* PWC access functions */
inline int pwc_reg_read(unsigned char addr, unsigned char *data)
{
	int ret = 0;

	if (addr >= REG_MAX) {
		return -EINVAL;
	}

	if (cs_table[addr].flag & PWC_REG_READONLY) {
		ret = spi_cmd_read(&device0_config_read, &addr, data, 0);
		if (ret < 0) {
			printk(KERN_INFO "pwc_reg_read(): spi_read error\n");
			return ret;
		}
	} else {
		*data = cs_table[addr].data;
	}

	return 0;
}

inline int pwc_reg_write(unsigned char addr, unsigned char data)
{
	int ret = 0;
	char buf[2];

	if (addr >= REG_MAX) {
		return -EINVAL;
	}

	if (cs_table[addr].flag & (PWC_REG_WRITEONLY | PWC_REG_WRITENOMODIFY)) {
		buf[0] = (char)data;
		buf[1] = (char)addr;
		ret = spi_write(&device0_config_write, buf, 0, 2, 0);
		if (ret < 0) {
			printk(KERN_INFO "pwc_reg_write(): spi_write error\n");
			return ret;
		}
		cs_table[addr].data = data;
	} else {
		printk(KERN_INFO "pwc_reg_write(): Read Only error\n");
		return -EINVAL;
	}

	return 0;
}

int pwc_read(unsigned short offset, unsigned int *data)
{
	*data = 0;
	return pwc_reg_read(offset, (unsigned char *)data);
}

int pwc_write(unsigned short offset, unsigned int data, unsigned int mask)
{
	int ret = 0;
	unsigned long flags;
	unsigned char tmp_data;

	spin_lock_irqsave(&pwc_write_spinlock, flags);
	ret = pwc_reg_read(offset, &tmp_data);
	if (ret != 0) {
		spin_unlock_irqrestore(&pwc_spinlock, flags);
		return ret;
	}
	if (cs_table[offset].flag & PWC_REG_WRITEONLY) {
		if( offset == PWC_TIMCNTR ) {
			tmp_data |= PWC_TIMCNTR_INIT_VAL;
		}
		tmp_data = (tmp_data & (~mask)) | (data & mask);
	} else if (cs_table[offset].flag & PWC_REG_WRITENOMODIFY) {
		tmp_data = data & mask;
	}
	ret = pwc_reg_write(offset, tmp_data);
	spin_unlock_irqrestore(&pwc_write_spinlock, flags);

	return ret;
}


int pwc_set_direction(unsigned gpio, int is_input)
{
	int ret = -EINVAL;
	unsigned int mask;
	unsigned char addr;

	if ((GPIO_EXT1_BASE <= gpio) && (gpio <= GPIO_EXT1_LAST)) {
		if (gpio <= GPIO_EXT1_P7) {
			mask = 1U << (gpio - GPIO_EXT1_P0);
			addr = PWC_IOSEL1;
		} else {
			mask = 1U << (gpio - GPIO_EXT1_P8);
			addr = PWC_IOSEL2;
		}

		if (is_input) {	/* input */
			ret = pwc_write(addr, 0, mask);
		} else {	/* output */
			ret = pwc_write(addr, mask, mask);
		}
	}

	return ret;
}

int pwc_get_value(unsigned int gpio)
{
	int ret = -EINVAL;
	unsigned char addr, in_addr, out_addr, sel_addr;
	unsigned char data, mask = 0, direction;

	if ((GPIO_EXT1_BASE <= gpio) && (gpio <= GPIO_EXT1_LAST)) {
		if (gpio <= GPIO_EXT1_P7) {
			mask = 1U << (gpio - GPIO_EXT1_P0);
			sel_addr = PWC_IOSEL1;
			in_addr  = PWC_IOIN1;
			out_addr = PWC_IOOUT1;
		} else {
			mask = 1U << (gpio - GPIO_EXT1_P8);
			sel_addr = PWC_IOSEL2;
			in_addr  = PWC_IOIN2;
			out_addr = PWC_IOOUT2;
		}

		if (pwc_reg_read(sel_addr, &direction)) {
			return ret;
		}

		if (direction & mask) {
			addr = out_addr;
		} else {
			addr = in_addr;
		}

		ret = pwc_reg_read(addr, &data);
	}

	return ret ? ret : ((data & mask) ? 1 : 0);
}

void pwc_set_value(unsigned int gpio, int value)
{
	unsigned int mask;
	unsigned char direction;
	unsigned char sel_addr, out_addr;

	if ((GPIO_EXT1_BASE <= gpio) && (gpio <= GPIO_EXT1_LAST)) {
		if (gpio <= GPIO_EXT1_P7) {
			mask = 1U << (gpio - GPIO_EXT1_P0);
			sel_addr = PWC_IOSEL1;
			out_addr = PWC_IOOUT1;
		} else {
			mask = 1U << (gpio - GPIO_EXT1_P8);
			sel_addr = PWC_IOSEL2;
			out_addr = PWC_IOOUT2;
		}

		if (pwc_reg_read(sel_addr, &direction)) {
			return;
		}

		if (~direction & mask) {
			return;
		}

		if(value) {
			pwc_write(out_addr, mask, mask);
		} else {
			pwc_write(out_addr, 0, mask);
		}
	}
}


/* PWC interrupt functions */
static void mp200_pwc_mask_irq(unsigned int irq)
{
	unsigned int port = 0;

	if ((irq < INT_PWC_MASK1_BASE) || (INT_PWC_LAST < irq)) {
		return;
	}
	if (irq < INT_PWC_MASK2_BASE) {
		port = irq - INT_PWC_MASK1_BASE;
		DPRINT("MASK1 port = %d\n", port);
		/* mask FMASK1/BMASK1 */
		pwc_write(PWC_BMASK1, 0, (1 << port));
		pwc_write(PWC_FMASK1, 0, (1 << port));
	} else if (irq < INT_PWC_MASK3_BASE) {
		port = irq - INT_PWC_MASK2_BASE;
		DPRINT("MASK2 port = %d\n", port);
		/* mask FMASK2/BMASK2 */
		pwc_write(PWC_BMASK2, 0, (1 << port));
		pwc_write(PWC_FMASK2, 0, (1 << port));
	} else {
		port = irq - INT_PWC_MASK3_BASE;
		DPRINT("MASK3 port = %d\n", port);
		/* mask FMASK3/BMASK3 */
		pwc_write(PWC_BMASK3, 0, (1 << port));
		pwc_write(PWC_FMASK3, 0, (1 << port));
	}
}

static void mp200_pwc_unmask_irq(unsigned int irq)
{
	unsigned int port = 0;

	if ((irq < INT_PWC_MASK1_BASE) || (INT_PWC_LAST < irq)) {
		return;
	}

	if (irq < INT_PWC_MASK2_BASE) {
		port = irq - INT_PWC_MASK1_BASE;
		DPRINT("MASK1 port = %d\n", port);
		/* unmask FMASK1/BMASK1 */
		pwc_write(PWC_FMASK1, (1 << port), (1 << port));
		pwc_write(PWC_BMASK1, (1 << port), (1 << port));
	} else if (irq < INT_PWC_MASK3_BASE) {
		port = irq - INT_PWC_MASK2_BASE;
		DPRINT("MASK2 port = %d\n", port);
		/* unmask FMASK2/BMASK2 */
		pwc_write(PWC_FMASK2, (1 << port), (1 << port));
		pwc_write(PWC_BMASK2, (1 << port), (1 << port));
	} else {
		port = irq - INT_PWC_MASK3_BASE;
		DPRINT("MASK3 port = %d\n", port);
		/* unmask FMASK3/BMASK3 */
		pwc_write(PWC_FMASK3, (1 << port), (1 << port));
		pwc_write(PWC_BMASK3, (1 << port), (1 << port));
	}
}

static void mp200_pwc_ack_irq(unsigned int irq)
{
	unsigned int port = 0;

	/* PWC clear before interrupt handling */
	if (irq < INT_PWC_MASK2_BASE) {
		port = irq - INT_PWC_MASK1_BASE;
		pwc_write(PWC_CLRFACT1, (1 << port), (1 << port));
	} else if (irq < INT_PWC_MASK3_BASE) {
		port = irq - INT_PWC_MASK2_BASE;
		pwc_write(PWC_CLRFACT2, (1 << port), (1 << port));
	} else {
		port = irq - INT_PWC_MASK3_BASE;
		pwc_write(PWC_CLRFACT3, (1 << port), (1 << port));
	}
}
static void mp200_pwc_ack_unmask_irq(unsigned int irq)
{
	mp200_pwc_ack_irq(irq);
	mp200_pwc_unmask_irq(irq);
}

static void mp200_pwc_irq_handler(unsigned int irq, struct irq_desc *desc)
{
	int i;
	unsigned int pwc_factor1_stat = 0;
	unsigned int pwc_factor2_stat = 0;
	unsigned int pwc_factor3_stat = 0;
	unsigned int pwc_irq;
	struct irq_desc *d;

	DPRINT("entry\n");

	if (irq != INT_GPIO_INTOUT) {
		return;
	}

	desc->chip->ack(irq);

	pwc_read(PWC_FACTOR1, &pwc_factor1_stat);
	pwc_read(PWC_FACTOR2, &pwc_factor2_stat);
	pwc_read(PWC_FACTOR3, &pwc_factor3_stat);

	if (pwc_factor1_stat != 0) {
		DPRINT("PWC_FACTOR1 = 0x%02x\n", pwc_factor1_stat);
		/* IO interrupt */
		for (i = PWC_INT_B0; i <= PWC_INT_MAX; i++) {
			if ((pwc_factor1_stat & (1 << i)) != 0) {
				pwc_irq = INT_PWC_MASK1_BASE + i;
				d = irq_desc + pwc_irq;
				d->handle_irq(pwc_irq, d);
			}
		}
	}
	if (pwc_factor2_stat != 0) {
		DPRINT("PWC_FACTOR2 = 0x%02x\n", pwc_factor2_stat);
		/* IO interrupt */
		for (i = PWC_INT_B0; i <= PWC_INT_MAX; i++) {
			if ((pwc_factor2_stat & (1 << i)) != 0) {
				pwc_irq = INT_PWC_MASK2_BASE + i;
				d = irq_desc + pwc_irq;
				d->handle_irq(pwc_irq, d);
			}
		}
	}
	if (pwc_factor3_stat != 0) {
		DPRINT("PWC_FACTOR3 = 0x%02x\n", pwc_factor3_stat);
		/* IO interrupt */
		for (i = PWC_INT_B0; i <= PWC_INT_MAX; i++) {
			if ((pwc_factor3_stat & (1 << i)) != 0) {
				pwc_irq = INT_PWC_MASK3_BASE + i;
				d = irq_desc + pwc_irq;
				d->handle_irq(pwc_irq, d);
			}
		}
	}

	desc->chip->unmask(irq);
	DPRINT("exit\n");
}


static int mp200_pwc_set_irq_type(unsigned int irq, unsigned int type);

static struct irq_chip mp200_pwc_chip_ack = {
	.name    = "PWC",
	.ack = mp200_pwc_ack_irq,
	.mask = mp200_pwc_mask_irq,
	.unmask = mp200_pwc_unmask_irq,
	.disable = mp200_pwc_mask_irq,
	.set_type = mp200_pwc_set_irq_type,
};
static struct irq_chip mp200_pwc_chip = {
	.name    = "PWC",
	.ack = mp200_pwc_mask_irq,
	.mask = mp200_pwc_mask_irq,
	.unmask = mp200_pwc_ack_unmask_irq,
	.disable = mp200_pwc_mask_irq,
	.set_type = mp200_pwc_set_irq_type,
};


/* called from set_irq_type() */
static int mp200_pwc_set_irq_type(unsigned int irq, unsigned int type)
{
	unsigned int port;
	unsigned int mode = 0;
	unsigned long flags;
	unsigned int pwc_bmask;
	unsigned int pwc_fmask;
	unsigned int shift;

	if ((irq < INT_PWC_MASK1_BASE) || (INT_PWC_LAST < irq)) {
		return -EINVAL;
	}

	if (irq == INT_PWC_IO01) {
		switch (type) {
		case IRQ_TYPE_LEVEL_HIGH:
			mode = 0x00;
			break;
		case IRQ_TYPE_EDGE_BOTH:
			mode = 0x01;
			break;
		default:
			return -EINVAL;
		}
	} else {
		switch (type) {
		case IRQ_TYPE_LEVEL_HIGH:
			mode = 0x00;
			break;
		case IRQ_TYPE_EDGE_RISING:
			mode = 0x01;
			break;
		case IRQ_TYPE_EDGE_FALLING:
			mode = 0x02;
			break;
		default:
			return -EINVAL;
		}
	}

	if ((mode == 0x01) || (mode == 0x02)) {
		set_irq_chip(irq, &mp200_pwc_chip_ack);
		set_irq_handler(irq, handle_edge_irq);
	} else {
		set_irq_chip(irq, &mp200_pwc_chip);
		set_irq_handler(irq, handle_level_irq);
	}

	spin_lock_irqsave(&pwc_spinlock, flags);

	if (irq < INT_PWC_MASK2_BASE) {
		port = irq - INT_PWC_MASK1_BASE;
		shift = (port & 0x03) << 1;

		/* BMASK1/FMASK1 enable -> disable */
		pwc_read(PWC_BMASK1, &pwc_bmask);
		if ((pwc_bmask & (1 << port)) != 0) {
			pwc_write(PWC_BMASK1, 0, (1 << port));
		}
		pwc_read(PWC_FMASK1, &pwc_fmask);
		if ((pwc_fmask & (1 << port)) != 0) {
			pwc_write(PWC_FMASK1, 0, (1 << port));
		}

		/* set DETMOD1 */
		if (port < 4) {
			pwc_write(PWC_DETMOD1L, (mode << shift),
				  (0x03 << shift));
		} else {
			pwc_write(PWC_DETMOD1H, (mode << shift),
				  (0x03 << shift));
		}
		udelay(60);
		pwc_write(PWC_CLRFACT1, (1 << port), (1 << port));

		/* Restore if changed */
		if ((pwc_bmask & (1 << port)) != 0) {
			pwc_write(PWC_BMASK1, (1 << port), (1 << port));
		}
		if ((pwc_fmask & (1 << port)) != 0) {
			pwc_write(PWC_FMASK1, (1 << port), (1 << port));
		}
	} else if (irq < INT_PWC_MASK3_BASE) {
		port = irq - INT_PWC_MASK2_BASE;
		shift = (port & 0x03) << 1;

		/* BMASK2/FMASK2 enable -> disable */
		pwc_read(PWC_BMASK2, &pwc_bmask);
		if ((pwc_bmask & (1 << port)) != 0) {
			pwc_write(PWC_BMASK2, 0, (1 << port));
		}
		pwc_read(PWC_FMASK2, &pwc_fmask);
		if ((pwc_fmask & (1 << port)) != 0) {
			pwc_write(PWC_FMASK2, 0, (1 << port));
		}

		/* set DETMOD2 */
		if (port < 4) {
			pwc_write(PWC_DETMOD2L, (mode << shift),
				  (0x03 << shift));
		} else {
			pwc_write(PWC_DETMOD2H, (mode << shift),
				  (0x03 << shift));
		}
		udelay(60);
		pwc_write(PWC_CLRFACT2, (1 << port), (1 << port));

		/* Restore if changed */
		if ((pwc_bmask & (1 << port)) != 0) {
			pwc_write(PWC_BMASK2, (1 << port), (1 << port));
		}
		if ((pwc_fmask & (1 << port)) != 0) {
			pwc_write(PWC_FMASK2, (1 << port), (1 << port));
		}
	} else {
		port = irq - INT_PWC_MASK3_BASE;
		shift = (port & 0x03) << 1;

		/* BMASK3/FMASK3 enable -> disable */
		pwc_read(PWC_BMASK3, &pwc_bmask);
		if ((pwc_bmask & (1 << port)) != 0) {
			pwc_write(PWC_BMASK3, 0, (1 << port));
		}
		pwc_read(PWC_FMASK3, &pwc_fmask);
		if ((pwc_fmask & (1 << port)) != 0) {
			pwc_write(PWC_FMASK3, 0, (1 << port));
		}

		/* set DETMOD3 */
		if (port < 4) {
			pwc_write(PWC_DETMOD3L, (mode << shift),
				  (0x03 << shift));
		} else {
			pwc_write(PWC_DETMOD3H, (mode << shift),
				  (0x03 << shift));
		}
		udelay(60);
		pwc_write(PWC_CLRFACT3, (1 << port), (1 << port));

		/* Restore if changed */
		if ((pwc_bmask & (1 << port)) != 0) {
			pwc_write(PWC_BMASK3, (1 << port), (1 << port));
		}
		if ((pwc_fmask & (1 << port)) != 0) {
			pwc_write(PWC_FMASK3, (1 << port), (1 << port));
		}
	}

	spin_unlock_irqrestore(&pwc_spinlock, flags);

	return 0;
}

#ifdef CONFIG_SNSC_MP200_PWC_GET_REG_STATUS
static void __init mp200_pwc_get_reg_status(void)
{
	unsigned int chk;
	unsigned char *addr;
	int i;

	printk(KERN_INFO
	       "mp200_pwc: register status address = 0x%08lX\n",
	       (unsigned long)&cs_table);

	chk = __raw_readl(IO_ADDRESS(MP200_SSRAM_BASE));
	if(chk != 0x63495770){
		printk(KERN_INFO
		       "mp200_pwc: Can't get "
		       "bootloader's register status.\n");
		return ;
	}
	printk(KERN_INFO "mp200_pwc: get bootloader's "
	       "register status.\n");
	for(i=0; i < sizeof(cs_table) / sizeof(pwc_reginfo); i++){
		addr = (char *)IO_ADDRESS(MP200_SSRAM_BASE + i + 4);
		cs_table[i].data = __raw_readb(addr);
#ifdef CONFIG_SNSC_MP200_PWC_DEBUG
		printk(KERN_DEBUG
		       "%02X(%02X) <= 0x%08lX(0x%02X)\n",
		       i, (int)cs_table[i].data,
		       (unsigned long)addr, (int)__raw_readb(addr));
#endif /* CONFIG_SNSC_MP200_PWC_DEBUG */
	}

	return ;
}
#endif /* CONFIG_SNSC_MP200_PWC_GET_REG_STATUS */

static int __init mp200_pwc_init(void)
{
	int i;

	/* setup default PWC Interrupt modes */
	for (i = INT_PWC_BASE; i <= INT_PWC_LAST; i++) {
		set_irq_chip(i, &mp200_pwc_chip);
		set_irq_handler(i, handle_level_irq);
		set_irq_flags(i, IRQF_VALID);
	}

#ifdef CONFIG_SNSC_MP200_PWC_GET_REG_STATUS
	mp200_pwc_get_reg_status();
#endif /* CONFIG_SNSC_MP200_PWC_GET_REG_STATUS */

#ifdef CONFIG_MACH_MP200
	/* Initialize PWC registers */
	pwc_write(PWC_IOSEL1, PWC_GIO_OUTPORT1, MASK_PWC_IOSEL1);
	pwc_write(PWC_IOSEL2, PWC_GIO_OUTPORT2, MASK_PWC_IOSEL2);
#endif

	/* Initialize PWC INT */
	set_irq_type(INT_GPIO_INTOUT, IRQ_TYPE_LEVEL_HIGH);
	set_irq_chained_handler(INT_GPIO_INTOUT, mp200_pwc_irq_handler);

	return 0;
}

module_init(mp200_pwc_init);

EXPORT_SYMBOL(pwc_reg_read);
EXPORT_SYMBOL(pwc_reg_write);
EXPORT_SYMBOL(pwc_read);
EXPORT_SYMBOL(pwc_write);
EXPORT_SYMBOL(pwc_set_direction);
EXPORT_SYMBOL(pwc_get_value);
EXPORT_SYMBOL(pwc_set_value);
