/* 2008-12-06: File added by Sony Corporation */
/* -------------------------------------------------------------------- */
/* i2c-emma.h: NEC EMMA2RH global defines                               */
/* -------------------------------------------------------------------- */
/*
    Shinya Kuribayashi, NEC Electronics Corporation
    s-kuribayashi@ap.jp.nec.com
    Takamitsu Honda, NEC Electronics Corporation
    takamitsu.honda@necel.com

    Copyright (C) NEC Electronics Corporation 2002-2005

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/
/* --------------------------------------------------------------------	*/

#ifndef I2C_EMMA_H
#define I2C_EMMA_H

/*---------------------------------------------------------------------------*/
/* CNT - Control register (00H R/W)                                          */
/*---------------------------------------------------------------------------*/
#define SPT         0x00000001
#define STT         0x00000002
#define ACKE        0x00000004
#define WTIM        0x00000008
#define SPIE        0x00000010
#define WREL        0x00000020
#define LREL        0x00000040
#define IICE        0x00000080
#define CNT_RESERVED    0x000000ff /* reserved bit 0                        */

#define I2C_EMMA_START      (IICE | STT)
#define I2C_EMMA_STOP       (IICE | SPT)
#define I2C_EMMA_REPSTART   I2C_EMMA_START

/*---------------------------------------------------------------------------*/
/* STA - Status register (04H Read)                                          */
/*---------------------------------------------------------------------------*/
#define MSTS        0x00000080
#define ALD         0x00000040
#define EXC         0x00000020
#define COI         0x00000010
#define TRC         0x00000008
#define ACKD        0x00000004
#define STD         0x00000002
#define SPD         0x00000001

/*---------------------------------------------------------------------------*/
/* CSEL - Clock select register (08H R/W)                                    */
/*---------------------------------------------------------------------------*/
#define SLEN        0x00000080 /* for test use, set "0"                 */
#define OLSI        0x00000040 /* for test use, set "0"                 */
#define CLD         0x00000020
#define DAD         0x00000010
#define SMC         0x00000008
#define DFC         0x00000004
#define CL          0x00000003
#define CSEL_RESERVED   0x000000ff /* reserved bit 0                        */

#define FAST397     0x0000008b
#define FAST297     0x0000008a
#define FAST347     0x0000000b
#define FAST260     0x0000000a
#define FAST130     0x00000008
#define STANDARD108 0x00000083
#define STANDARD83  0x00000082
#define STANDARD95  0x00000003
#define STANDARD73  0x00000002
#define STANDARD36  0x00000001
#define STANDARD71  0x00000000

/*---------------------------------------------------------------------------*/
/* SVA - Slave address register (0CH R/W)                                    */
/*---------------------------------------------------------------------------*/
#define SVA         0x000000fe

/*---------------------------------------------------------------------------*/
/* SHR - Shift register (10H R/W)                                            */
/*---------------------------------------------------------------------------*/
#define SR          0x000000ff

/*---------------------------------------------------------------------------*/
/* FCNT - FIFO control register (20H R/W)                                    */
/*---------------------------------------------------------------------------*/
#define FRST        0x00000001
#define FENA        0x00000002
#define FSTOP       0x00000004
#define TCSEL       0x00000100
#define RXT_ENA     0x00000200
#define TXT_ENA     0x00000400
#define SVA_REN     0x00010000
#define ACKSTOP     0x00020000
#define REINC       0x00040000
#define FORCE       0x00080000
#define RESERVED20  0x00100000
#define RESERVED21  0x00200000
#define FCNT_RESERVED   0x003f0707 /* reserved bit 0                        */

/*---------------------------------------------------------------------------*/
/* FSTA - FIFO status register (24H Read)                                    */
/*---------------------------------------------------------------------------*/
#define WPOS        0x3f000000
#define RPOS        0x003f0000
#define RWSTA       0x00008000
#define WAIT        0x00004000
#define POOL        0x00003f00
#define INTTXTO     0x00000080
#define INTRXTO     0x00000040
#define INTTXLV     0x00000020
#define INTRXLV     0x00000010
#define EMPTY       0x00000004
#define FULL        0x00000001

/*---------------------------------------------------------------------------*/
/* FTH - FIFO threshold register (28H R/W)                                   */
/*---------------------------------------------------------------------------*/
#define TXLV        0x003f0000
#define RX_STATUS   0x00001000
#define RX_STOP_EN  0x00000100
#define RXLV        0x0000003f
#define FTH_RESERVED    0x003f113f /* reserved bit 0                        */

/*---------------------------------------------------------------------------*/
/* FRXTO - Rx FIFO timeout level register (2CH R/W)                          */
/*---------------------------------------------------------------------------*/
#define RXTO        0x03ffffff

/*---------------------------------------------------------------------------*/
/* FTXTO - Tx FIFO timeout level register (30H R/W)                          */
/*---------------------------------------------------------------------------*/
#define TXTO        0x03ffffff

/*---------------------------------------------------------------------------*/
/* FRDAT - FIFO read data register (34H Read)                                */
/*---------------------------------------------------------------------------*/
#define R_DATA      0x000000ff

/*---------------------------------------------------------------------------*/
/* FWDAT - FIFO write data register (38H Write)                              */
/*---------------------------------------------------------------------------*/
#define W_DATA      0x000000ff

/*---------------------------------------------------------------------------*/
/* FDBG - Timeout counter data (3CH R/W)                                     */
/*---------------------------------------------------------------------------*/
#define TCOUNT      0x03ffffff

/*---------------------------------------------------------------------------*/
/* INT - Interrupt register (40H R/W)                                        */
/* INTM - Interrupt mask register (44H R/W)                                  */
/*---------------------------------------------------------------------------*/
#define INT_WCLK    0x00200000
#define INT_WSVA    0x00100000
#define INT_WFDAT   0x00080000
#define INT_WSHIFT  0x00040000
#define INT_WDWAIT  0x00020000	/* ? */
#define INT_WCONT   0x00010000
#define INT_TXTO    0x00004000
#define INT_RXTO    0x00002000
#define INT_TXTH    0x00001000
#define INT_RXTH    0x00000800
#define INT_EMPTY   0x00000400
#define INT_FULL    0x00000200
#define INT_CHANGE  0x00000100
#define INT_TEST    0x00000040
#define INT_NOP     0x00000020
#define INT_ALD9    0x00000010
#define INT_ALD8    0x00000008
#define INT_WAIT9   0x00000004
#define INT_WAIT8   0x00000002
#define INT_STOP    0x00000001
#define INT_RESERVED    0x003f7f7f /* reserved bit 0                        */

#define INT_USUAL   (INT_NOP | INT_ALD9 | INT_ALD8 | INT_WAIT9 | INT_WAIT8 | INT_STOP)

/***********************************************************************
 * I2C registers
 ***********************************************************************
 */
#define I2C_EMMA_CNT            0x00
#define I2C_EMMA_STA            0x10
#define I2C_EMMA_CSEL           0x20
#define I2C_EMMA_SVA            0x30
#define I2C_EMMA_SHR            0x40
#define I2C_EMMA_INT            0x50
#define I2C_EMMA_INTM           0x60

#endif /* I2C_EMMA_H */
