/* 2010-08-25: File added by Sony Corporation */
/*
 *  File Name       : linux/drivers/i2c/busses/i2c-mp200.h
 *  Function        : i2c
 *  Release Version : Ver 1.21
 *  Release Date    : 2009/01/29
 *
 *  Copyright (C) NEC Electronics Corporation 2007,2008,2009
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __DRIVERS_I2C_BUSSES_I2C_MP200_H
#define __DRIVERS_I2C_BUSSES_I2C_MP200_H

/* INTC Reg Start */
#define MP200_I2C_INTC_IT0_RAW	(IO_ADDRESS (MP200_INTC_BASE) + MP200_INTC_IT0_RAW1)
#define MP200_I2C_INTC_IT0_IIR	(IO_ADDRESS (MP200_INTC_BASE) + MP200_INTC_IT0_IIR)

/* INTC Reg End */
#define MP200_I2C_INTC_MST	(1 << (33 - 32))
#define MP200_I2C2_INTC_MST	(1 << (39 - 32))

#define I2C_ADDR			IO_ADDRESS (MP200_IIC_BASE)
#define I2C2_ADDR			IO_ADDRESS (MP200_IIC2_BASE)

#define I2C_OFS_IIC0		0x00		/* IIC0 shift */
#define I2C_OFS_IICC0		0x08		/* IIC0 control */
#define I2C_OFS_SVA0		0x0c		/* slave address */
#define I2C_OFS_IICCL0		0x10		/* IIC0 clock select */
#define I2C_OFS_IICSE0		0x1c		/* For emulation */
#define I2C_OFS_IICF0		0x28		/* IIC flag */


/* I2C IICC0 Masks (Even address) */
#define I2C_BIT_IICE0		0x0080
#define I2C_BIT_LREL0		0x0040
#define I2C_BIT_WREL0		0x0020
#define I2C_BIT_SPIE0		0x0010
#define I2C_BIT_WTIM0		0x0008
#define I2C_BIT_ACKE0		0x0004
#define I2C_BIT_STT0		0x0002
#define I2C_BIT_SPT0		0x0001

/* I2C IICCL0 Masks (Even address) */
#define I2C_BIT_CLD0		0x0020
#define I2C_BIT_DAD0		0x0010
#define I2C_BIT_SMC0		0x0008
#define I2C_BIT_DFC0		0x0004
#define I2C_BIT_CLO1		0x0002
#define I2C_BIT_CLO0		0x0001

/* I2C IICSE0 Masks (Odd address) */
#define I2C_BIT_MSTS0		0x8000
#define I2C_BIT_ALD0		0x4000
#define I2C_BIT_EXC0		0x2000
#define I2C_BIT_COI0		0x1000
#define I2C_BIT_TRC0		0x0800
#define I2C_BIT_ACKD0		0x0400
#define I2C_BIT_STD0		0x0200
#define I2C_BIT_SPD0		0x0100

/* I2C IICF0 Masks (Even address) */
#define I2C_BIT_STCF		0x0080
#define I2C_BIT_IICBSY		0x0040
#define I2C_BIT_STCEN		0x0002
#define I2C_BIT_IICRSV		0x0001

/* For setting of sending and receiving */
#define I2C_DIR_SHIFT		1
#define I2C_DIR_R		0x01
#define I2C_DIR_W		0x00

typedef struct i2c_ctrl {
	unsigned char smc;	/* mode */
	unsigned char dfc;	/* digital filter */
} i2c_ctrl_t;

/* mode */
#define I2C_SMC_NORMAL_SPEED	0	/* nomal mode(maximum transfer rate 70kbit/s)(default) */
#define I2C_SMC_HIGH_SPEED		1	/* high-speed mode(maximum transfer rate 250kbit/s) */

/* digital filter */
#define I2C_DFC_OFF				0	/* digital filter OFF (default) */
#define I2C_DFC_ON				1	/* digital filter ON */

/* mp200 wait */
#define MP200_I2C_WAIT		1		/* wait 1us after STT0 */

/* IIC_SCLK ( HIGHSPEED : 4.19MHz-8.38MHz */
#define I2C_DIVIIC_VAL1		0x00000072	/* (229.376MHz or 237.568MHz)/32/24 */
#define I2C_DIVIIC_VAL2		0x00720000	/* (229.376MHz or 237.568MHz)/32/24 */


#define MP200_I2C_CH1	0
#define MP200_I2C_CH2	1


#ifdef CONFIG_I2C_MP200_ENABLE_CH2
#define I2C_NR	2
#else
#define I2C_NR	1
#endif


#endif				/* __DRIVERS_I2C_BUSSES_I2C_MP200_H */
