/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /drivers/ipu/mp200_img_convert.h
*  Function        : IPU Driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/14|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/


#ifndef _MP200_IMG_CONVERT_H_
#define _MP200_IMG_CONVERT_H_


#define _DEBUG_CONVERT 0  /* 0: no debug log
                           * 1: print ioctl(IMGIO_CONVERT) arg
                           * 2: print ioctl(IMGIO_CONVERT) set IMG register
                           * 3: print ioctl(IMGIO_CONVERT) arg & set IMG register
                           */


/****************************************************************************/
/* Macro                                                                    */
/****************************************************************************/
#define ERR_PRINT(num, data) printk( KERN_INFO " @ipu: IMGIO_CONVERT: Error! %s is incorrect. %s = 0x%lx\n", num, num, (long)data);

#define ImgRequest(DATA, REG)                    \
do{                                              \
    pReq->ipu_reg_info[index_req].reg  = (REG);  \
    pReq->ipu_reg_info[index_req].data = (DATA); \
    index_req++;                                 \
}while(0)


/****************************************************************************/
/* Flags                                                                    */
/****************************************************************************/
/* ------------------------------------------- */
/*   internal flags                            */
/* ------------------------------------------- */
static unsigned long img_mode;
static unsigned long ulSrcStarAddrYRGB, ulSrcStarAddrU, ulSrcStarAddrV_B,  ulSrcScreenWidth, src_x, src_y, src_width , src_height;
static unsigned long ulSrcStarAddrYRGB_F, ulSrcStarAddrU_F, ulSrcStarAddrV_F,  ulSrcScreenWidth_F, src_x_F, src_y_F, src_width_F, src_height_F;
static unsigned long ulDestStarAddrYRGB, ulDestStarAddrU, ulDestStarAddrV, ulDestScreenWidth, dest_x, dest_y, dest_width , dest_height;
static unsigned long offset_x, offset_y, step_h, step_v, fold_h, fold_v;
static unsigned long ulSrc, ulSrcU, ulSrcV, ulSrc_F, ulSrcU_F, ulSrcV_F, ulDest, ulDestU, ulDestV;
static unsigned long format, src_format_b, src_format_f, dst_format;
static unsigned long in_byte, out_byte, endian;
#if _IPU_COLCONV
static unsigned long matrix_flag, matrix_conf, matrix[9];
#endif
static int mask_color, mask_alpha;

static int src_yuv_front, src_yuv_back, dest_yuv;
static int img_resize, mix_front_image, img_maskcolor;
static int bits_par_pixel_F, bits_par_pixel_B, bits_par_pixel_D;


/* ------------------------------------------- */
/*   definetion                                */
/* ------------------------------------------- */
/* mix_front_image */
#define MIX_OFF           0
#define MIX_ON            1
/* img_maskcolor */
#define MASK_OFF          0
#define MASK_ON           1
/* img_resize */
#define RESIZE_OFF        0
#define RESIZE_ON_BACK    1
#define RESIZE_ON_FRONT   2
/* src_yuv_front, src_yuv_back, dest_yuv */
#define RGB_IMAGE         0
#define YUV_IMAGE         1

/* bits_par_pixel_F, bits_par_pixel_B, bits_par_pixel_D */
#define BPP_YUV420_422    8
#define BPP_YUV422_INTER 16  /* Interleave */
#define BPP_RGB565       16
#define BPP_RGB666       18
#define BPP_RGB888       24

#define MAX_SCREEN_WIDTH 0x7FFC

/* pitch of back image must be less than 0x2000 because of IPU Errata  */
#define MAX_SCREEN_WIDTH_BACK 0x1FFC

/****************************************************************************/
/* Prototype declarations                                                   */
/****************************************************************************/
/* ------------------------------------------- */
/*   Private functions                         */
/* ------------------------------------------- */
static int  mp200_img_convert(MP200_IPU_DEV_INFO *dev_info, _IMGIO_CONVERT *p);
static int  mp200_img_mkreq(struct mp200_ipu_req *pReq, int index);
static int  mp200_img_mkdata(_IMGIO_CONVERT *p);
static int  mp200_img_chkdata(_IMGIO_CONVERT *p);
static int  mp200_img_chkbytelane(_IMGIO_CONVERT *p);
static int  mp200_img_chkstatus(void);
static int  mp200_img_chksize(unsigned long back_size, unsigned long front_size, unsigned long dest_size, char *num_size);
static int  mp200_img_chksize_sub(unsigned long size, unsigned long mode_fmt_bit, unsigned long mode_fmt_sft, char *num_size);
static int  mp200_img_chkcoordinate(void);
static int  mp200_img_chkcoordinate_sub(unsigned long mode_fmt_bit, unsigned long mode_fmt_sft);
static int  mp200_img_chkcoordinate_x_sub(unsigned long pos, unsigned long mode_fmt_bit, unsigned long mode_fmt_sft);
static int  mp200_img_chkcoordinate_y_sub(unsigned long pos, unsigned long mode_fmt_bit, unsigned long mode_fmt_sft);
static int  mp200_img_chkaddr(void);
static int  mp200_img_setaddr(void);

#if (_DEBUG_CONVERT & 0x1)
static void print_log1(_IMGIO_CONVERT *p);
#endif /* _DEBUG_CONVERT */

#if (_DEBUG_CONVERT & 0x2)
static void print_log2(void);
#endif /* _DEBUG_CONVERT */


/*****************************************************************************
* MODULE   : mp200_img_convert
* FUNCTION : ioctl IMGIO_CONVERT sub function.
* RETURN   : IMG_COMPLETE(0) : Success
*            -EINVAL         : Fail (parameter error)
*            -EBUSY          : Fail (IMG busy)
*            -EIO            : Fail (IMG error, or convert is not completed)
*            -ERESTARTSYS    : Fail (device busy)
* NOTE     : none
* CREATE   : 2006/11/09 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_convert(MP200_IPU_DEV_INFO *dev_info, _IMGIO_CONVERT *p)
{
    struct mp200_ipu_req *pReq;
    int ret;

    /* make IMG data. */
    if(mp200_img_mkdata(p) == FALSE) return -EINVAL;

    if(dev_info->status != IPU_STATUS_NO_REQ)
    {
        return -EBUSY;
    }

    if(down_interruptible (&dev_info->sem))            /* get semafore */
    {
        return -ERESTARTSYS;
    }

    /* set IMG data to register. */
    pReq = &dev_info->req;
    mp200_img_mkreq(pReq, 0);

    /* set pointer to Device information into dev_info->req.data */
    pReq->data = (unsigned long)dev_info;

    /* set callback function into dev_info->req.callback */
    pReq->callback = (void *)mp200_ipu_callback_img;

    /* set hw_func */
    pReq->ipu_hw_func = IPU_HW_FUNC_IMG;

    /* update status of IPU request process */
    dev_info->status = IPU_STATUS_WAIT;                /* running */

    ret = mp200_ipu_regist(pReq);                      /* register request (reserve)*/
    if (ret != IPU_REQ_REGISTERED)
    {
        dev_info->status = IPU_STATUS_NO_REQ;          /* target request does not exist */
        up(&dev_info->sem);                            /* release semafore */
        return ret;
    }

wait_img:
    dev_info->sleep_mode = TRUE;
    wait_event_interruptible(dev_info->wait_que, (dev_info->status != IPU_STATUS_WAIT)); /* wait until IMG process is completed */
    dev_info->sleep_mode = FALSE;

    switch (dev_info->status)
    {
    case IPU_STATUS_DONE:                              /* wake up by IMG_STATUS_DONE */
        switch (dev_info->req.status)
        {
        case IPU_CALLBACK_SUCCESS:
            ret = IMG_COMPLETE;
            break;

        case IPU_CALLBACK_IPU_ERROR:
        case IPU_CALLBACK_IPU_STOP:
            printk( KERN_INFO " @ipu: IMGIO_CONVERT: IMG Error! callback flag = %d\n", dev_info->req.status);
            ret = -EIO;
            break;

        case IPU_CALLBACK_SET_PARAM:
        case IPU_CALLBACK_CANCEL:
        case IPU_CALLBACK_FAIL:
            default:
            printk( KERN_INFO " @ipu: IMGIO_CONVERT: convert is not completed! callback flag = %d\n", dev_info->req.status);
            ret = -EIO;
            break;
        }
        break;

    case IPU_STATUS_CANCELLED:                         /* wake up by IMG_STATUS_CANCELLED */
        printk( KERN_INFO " @ipu: IMGIO_CONVERT: convert process is cancelled!\n");
        ret = -EIO;
        break;

    case IPU_STATUS_WAIT:                              /* wake up by signal */
    default:
        printk( KERN_INFO " @ipu: IMGIO_CONVERT: wake up by signal. retry to wait until IMG process is completed\n");
        goto wait_img;
        break;
    }
    /* update status of IPU request process */
    dev_info->status = IPU_STATUS_NO_REQ;              /* target request does not exist */
    up(&dev_info->sem);                                /* release semafore */
    return ret;
}


/*****************************************************************************
* MODULE   : mp200_img_mkreq
* FUNCTION : make IMG request data
* IMPORT   :
* RETURN   :
* NOTE     : none
* CREATE   : 2006/11/09 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_mkreq(struct mp200_ipu_req *pReq, int index)
{
    int index_req = index;

#if (_DEBUG_CONVERT & 0x2)
    print_log2();
    index_req = index;
#endif /* _DEBUG_CONVERT */

    /*******************************
        set IMG register
    ********************************/
    /*==============================
        IMG_MODE
    ================================*/
    ImgRequest(img_mode,                 IMG_MODE);

    /*==============================
        source image (back)
    ================================*/
    ImgRequest(ulSrcStarAddrYRGB,        IMG_SRCYRGBADR_B);
    ImgRequest(ulSrcStarAddrU,           IMG_SRCUVADR_B);
    ImgRequest(ulSrcStarAddrV_B,         IMG_SRCVADR_B);
    ImgRequest(ulSrcScreenWidth,         IMG_SRCSIZE_B);
    ImgRequest(src_width,                IMG_SRCHSIZE_B);
    ImgRequest(src_height,               IMG_SRCVSIZE_B);

    /*==============================
        source image (front)
    ================================*/
    if(mix_front_image)
    {
        ImgRequest(ulSrcStarAddrYRGB_F, IMG_SRCYRGBADR_F);
        ImgRequest(ulSrcStarAddrU_F,    IMG_SRCUVADR_F);
        ImgRequest(ulSrcStarAddrV_F,    IMG_SRCVADR_F);
        ImgRequest(ulSrcScreenWidth_F,  IMG_SRCSIZE_F);
        ImgRequest(src_width_F,         IMG_SRCHSIZE_F);
        ImgRequest(src_height_F,        IMG_SRCVSIZE_F);
        /*------------------------------
           offset
        --------------------------------*/
        ImgRequest(offset_x,             IMG_OFFSETX );
        ImgRequest(offset_y,             IMG_OFFSETY );
        /*------------------------------
            mask color
        --------------------------------*/
        if(img_maskcolor)
        {
            ImgRequest(mask_color,       IMG_MASKCOLR );
            ImgRequest(mask_alpha,       IMG_ALPHA );
        }
    }

    /*==============================
        destination image
    ================================*/
    ImgRequest(ulDestStarAddrYRGB,       IMG_DSTYRGBADR);
    ImgRequest(ulDestStarAddrU,          IMG_DSTUVADR);
    ImgRequest(ulDestStarAddrV,          IMG_DSTVADR);
    ImgRequest(ulDestScreenWidth,        IMG_DSTSIZE);

    /*==============================
        resize
    ================================*/
    if(img_resize)
    {
        ImgRequest(dest_width,           IMG_DSTHSIZE);
        ImgRequest(dest_height,          IMG_DSTVSIZE);
        /*------------------------------
           H/VSTEP,H/VFOLD
        --------------------------------*/
        ImgRequest(step_h,               IMG_HSTEP);
        ImgRequest(step_v,               IMG_VSTEP);
        ImgRequest(fold_h,               IMG_HFOLD);
        ImgRequest(fold_v,               IMG_VFOLD);
    }

    /*==============================
        format
    ================================*/
    ImgRequest(format,                   IMG_FORMAT);

    /*==============================
        bytelane
    ================================*/
    ImgRequest(in_byte,                  IMG_INDATABYTE);
    ImgRequest(out_byte,                 IMG_OUTDATABYTE);
    ImgRequest(endian,                   IMG_PEL_ENDIAN);

#if _IPU_COLCONV
    /*==============================
        color-space conversion
    ================================*/
    if(matrix_flag)
    {
        int i;
        ImgRequest(matrix_conf,          matrix_flag);
        for(i = 0; i < 9; i++){
            ImgRequest(matrix[i],        matrix_flag+(i+1)*4);
        }
    }
#endif

    /*==============================
        Start IMG
    ================================*/
    ImgRequest(IMG_REQ_BIT,              IMG_REQ);

    /*==============================
        end mark
    ================================*/
    ImgRequest(IPU_REG_INFO_END,         -1);

    return index_req;
}


/*****************************************************************************
* MODULE   : mp200_img_mkdata
* FUNCTION : make IMG register data
* IMPORT   : _IMGIO_CONVERT
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/10/18 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_mkdata(_IMGIO_CONVERT *p)
{
#if (_DEBUG_CONVERT & 0x1)
    print_log1(p);
#endif /* _DEBUG_CONVERT */

    /*******************************
        check import data
    ********************************/
    if(mp200_img_chkdata(p) == FALSE) return FALSE;

    src_x        = p->src_b_image.x;
    src_y        = p->src_b_image.y;
    src_width    = p->src_b_image.hsize;
    src_height   = p->src_b_image.vsize;
    src_x_F      = p->src_f_image.x;
    src_y_F      = p->src_f_image.y;
    src_width_F  = p->src_f_image.hsize;
    src_height_F = p->src_f_image.vsize;
    dest_x       = p->dest_image.x;
    dest_y       = p->dest_image.y;
    dest_width   = p->dest_image.hsize;
    dest_height  = p->dest_image.vsize;

    offset_x     = p->offsetx;
    offset_y     = p->offsety;
    mask_color   = p->maskcolr;
    mask_alpha   = p->alpha;

    ulSrc        = p->src_b_image.yrgbadr;
    ulSrcU       = p->src_b_image.uvadr;
    ulSrcV       = p->src_b_image.vadr;
    ulSrc_F      = p->src_f_image.yrgbadr;
    ulSrcU_F     = p->src_f_image.uvadr;
    ulSrcV_F     = p->src_f_image.vadr;
    ulDest       = p->dest_image.yrgbadr;
    ulDestU      = p->dest_image.uvadr;
    ulDestV      = p->dest_image.vadr;
    ulSrcScreenWidth   = p->src_b_image.size;
    ulSrcScreenWidth_F = p->src_f_image.size;
    ulDestScreenWidth  = p->dest_image.size;

    src_format_b = p->img_mode.srcfmt_b;
    src_format_f = p->img_mode.srcfmt_f;
    dst_format   = p->img_mode.dstfmt;

    /*******************************
        make IMG_MODE
    ********************************/
    img_mode  = IMG_NEWEN_BIT;
    if ((p->img_mode.dstfmt == IMG_FORM_RGB666) || (p->img_mode.dstfmt == IMG_FORM_RGB565))
    {
        if (mix_front_image)
        {
            img_mode |= (p->img_mode.dither_f  << IMG_DITHER_F_SFT);
        }
        img_mode |= (p->img_mode.dither_b  << IMG_DITHER_B_SFT);
    }
    if(img_resize == RESIZE_ON_FRONT)
    {
        img_mode |= (p->img_mode.rfilter_f << IMG_RFILTER_F_SFT);
    }
    if(img_resize == RESIZE_ON_BACK)
    {
        img_mode |= (p->img_mode.rfilter_b << IMG_RFILTER_B_SFT);
    }
    img_mode |= (p->img_mode.op_mode   << IMG_OP_MODE_SFT);
    if((p->img_mode.op_mode == IMG_SET_OP_MODE_12) || (p->img_mode.op_mode == IMG_SET_OP_MODE_18))
    {
        img_mode |= (p->img_mode.msk_col   << IMG_MASK_COL_SFT);
    }
    img_mode |= (p->img_mode.udinv     << IMG_UDINV_SFT);
    img_mode |= (p->img_mode.rlinv     << IMG_RLINV_SFT);

    /*******************************
        check IMG_MODE
    ********************************/
    if(mp200_img_chkstatus() == FALSE) return FALSE;

    /*******************************
        set format
    ********************************/
    format  = (src_format_b << IMG_FORMAT_SRCFMT_B_SFT);
    format |= (dst_format   << IMG_FORMAT_DSTFMT_SFT);
    if(mix_front_image) format |= (src_format_f << IMG_FORMAT_SRCFMT_F_SFT);

    /*******************************
        check image size
    ********************************/
    if(mp200_img_chksize(src_width,  src_width_F,  dest_width,  "hsize") == FALSE) return FALSE;
    if(mp200_img_chksize(src_height, src_height_F, dest_height, "vsize") == FALSE) return FALSE;

    if(img_resize)
    {
        unsigned long sw, sh;
        if(img_resize == RESIZE_ON_BACK)
        {
            sw = src_width;
            sh = src_height;
        }
        else /* RESIZE_ON_FRONT */
        {
            sw = src_width_F;
            sh = src_height_F;
        }
        /* H/VSTEP */
        step_h = (sw * 256 / dest_width);
        step_v = (sh * 256 / dest_height);
        /* H/VFOLD */
        fold_h = (dest_width  * 64 / sw);
        fold_v = (dest_height * 64 / sh);
    }

    /*******************************
        check (x,y)coordinate
    ********************************/
    if(mp200_img_chkcoordinate() == FALSE) return FALSE;

    /*******************************
        check address
    ********************************/
    if(mp200_img_chkaddr() == FALSE) return FALSE;

    /*******************************
        set IMG address
    ********************************/
    if(mp200_img_setaddr() == FALSE) return FALSE;

    /*******************************
        check bytelane
    ********************************/
    if(mp200_img_chkbytelane(p) == FALSE) return FALSE;

    /*******************************
        set bytelane
    ********************************/
    in_byte  = p->in_byte;
    out_byte = p->out_byte;

    return TRUE;

}


/*****************************************************************************
* MODULE   : mp200_img_chkbytelane
* FUNCTION : check image bytelane
* IMPORT   : _IMGIO_CONVERT
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/10/18 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_chkbytelane(_IMGIO_CONVERT *p)
{
    /* check input bytelane */
    if ((p->in_byte & IMG_INDATABYTE_DATA0_BIT) == ((p->in_byte & IMG_INDATABYTE_DATA1_BIT) >> IMG_INDATABYTE_DATA1_SFT ) ||
        (p->in_byte & IMG_INDATABYTE_DATA0_BIT) == ((p->in_byte & IMG_INDATABYTE_DATA2_BIT) >> IMG_INDATABYTE_DATA2_SFT ) ||
        (p->in_byte & IMG_INDATABYTE_DATA0_BIT) == ((p->in_byte & IMG_INDATABYTE_DATA3_BIT) >> IMG_INDATABYTE_DATA3_SFT ) ||
        ((p->in_byte & IMG_INDATABYTE_DATA1_BIT)>> IMG_INDATABYTE_DATA1_SFT ) == ((p->in_byte & IMG_INDATABYTE_DATA2_BIT) >> IMG_INDATABYTE_DATA2_SFT ) ||
        ((p->in_byte & IMG_INDATABYTE_DATA1_BIT)>> IMG_INDATABYTE_DATA1_SFT ) == ((p->in_byte & IMG_INDATABYTE_DATA3_BIT) >> IMG_INDATABYTE_DATA3_SFT ) ||
        ((p->in_byte & IMG_INDATABYTE_DATA2_BIT)>> IMG_INDATABYTE_DATA2_SFT ) == ((p->in_byte & IMG_INDATABYTE_DATA3_BIT) >> IMG_INDATABYTE_DATA3_SFT ))
    {
        printk( KERN_INFO " @ipu: IMGIO_IMAGE: Error! input_byte is incorrect.\n");
        printk( KERN_INFO " in_byte IMG_INDATABYTE_DATA0_BIT = %ld\n", (p->in_byte & IMG_INDATABYTE_DATA0_BIT) >> IMG_INDATABYTE_DATA0_SFT );
        printk( KERN_INFO " in_byte IMG_INDATABYTE_DATA1_BIT = %ld\n", (p->in_byte & IMG_INDATABYTE_DATA1_BIT) >> IMG_INDATABYTE_DATA1_SFT );
        printk( KERN_INFO " in_byte IMG_INDATABYTE_DATA2_BIT = %ld\n", (p->in_byte & IMG_INDATABYTE_DATA2_BIT) >> IMG_INDATABYTE_DATA2_SFT );
        printk( KERN_INFO " in_byte IMG_INDATABYTE_DATA3_BIT = %ld\n", (p->in_byte & IMG_INDATABYTE_DATA3_BIT) >> IMG_INDATABYTE_DATA3_SFT );
        return FALSE;
    }
    /* check output bytelane */
    if ((p->out_byte & IMG_OUTDATABYTE_DATA0_BIT) == ((p->out_byte & IMG_OUTDATABYTE_DATA1_BIT)>> IMG_OUTDATABYTE_DATA1_SFT ) ||
        (p->out_byte & IMG_OUTDATABYTE_DATA0_BIT) == ((p->out_byte & IMG_OUTDATABYTE_DATA2_BIT)>> IMG_OUTDATABYTE_DATA2_SFT ) ||
        (p->out_byte & IMG_OUTDATABYTE_DATA0_BIT) == ((p->out_byte & IMG_OUTDATABYTE_DATA3_BIT)>> IMG_OUTDATABYTE_DATA3_SFT ) ||
        ((p->out_byte & IMG_OUTDATABYTE_DATA1_BIT) >> IMG_OUTDATABYTE_DATA1_SFT ) == ((p->out_byte & IMG_OUTDATABYTE_DATA2_BIT) >> IMG_OUTDATABYTE_DATA2_SFT ) ||
        ((p->out_byte & IMG_OUTDATABYTE_DATA1_BIT) >> IMG_OUTDATABYTE_DATA1_SFT ) == ((p->out_byte & IMG_OUTDATABYTE_DATA3_BIT) >> IMG_OUTDATABYTE_DATA3_SFT ) ||
        ((p->out_byte & IMG_OUTDATABYTE_DATA2_BIT) >> IMG_OUTDATABYTE_DATA2_SFT ) == ((p->out_byte & IMG_OUTDATABYTE_DATA3_BIT) >> IMG_OUTDATABYTE_DATA3_SFT))
    {
        printk( KERN_INFO " @ipu: IMGIO_IMAGE: Error! output_byte is incorrect.\n");
        printk( KERN_INFO " out_byte IMG_OUTDATABYTE_DATA0_BIT = %ld\n", (p->out_byte & IMG_OUTDATABYTE_DATA0_BIT) >> IMG_OUTDATABYTE_DATA0_SFT );
        printk( KERN_INFO " out_byte IMG_OUTDATABYTE_DATA1_BIT = %ld\n", (p->out_byte & IMG_OUTDATABYTE_DATA1_BIT) >> IMG_OUTDATABYTE_DATA1_SFT );
        printk( KERN_INFO " out_byte IMG_OUTDATABYTE_DATA2_BIT = %ld\n", (p->out_byte & IMG_OUTDATABYTE_DATA2_BIT) >> IMG_OUTDATABYTE_DATA2_SFT );
        printk( KERN_INFO " out_byte IMG_OUTDATABYTE_DATA3_BIT = %ld\n", (p->out_byte & IMG_OUTDATABYTE_DATA3_BIT) >> IMG_OUTDATABYTE_DATA3_SFT );
       return FALSE;
    }

    endian = IMG_PEL_ENDIAN_IN_ENDI_LITTLE | IMG_PEL_ENDIAN_OUT_ENDI_LITTLE;

    return TRUE;
}


/*****************************************************************************
* MODULE   : mp200_img_chkdata
* FUNCTION : check ioctl IMGIO_CONVERT arg
* IMPORT   : _IMGIO_CONVERT
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/10/18 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_chkdata(_IMGIO_CONVERT *p)
{
/*====================*/
/* Image Mode check */
/*====================*/
    /* check OP_MODE flag */
    if((p->img_mode.op_mode != IMG_SET_OP_MODE_00) && (p->img_mode.op_mode != IMG_SET_OP_MODE_01) && (p->img_mode.op_mode != IMG_SET_OP_MODE_02) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_03) && (p->img_mode.op_mode != IMG_SET_OP_MODE_04) && (p->img_mode.op_mode != IMG_SET_OP_MODE_05) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_06) && (p->img_mode.op_mode != IMG_SET_OP_MODE_07) && (p->img_mode.op_mode != IMG_SET_OP_MODE_08) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_09) && (p->img_mode.op_mode != IMG_SET_OP_MODE_10) && (p->img_mode.op_mode != IMG_SET_OP_MODE_11) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_12) && (p->img_mode.op_mode != IMG_SET_OP_MODE_13) && (p->img_mode.op_mode != IMG_SET_OP_MODE_14) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_15) && (p->img_mode.op_mode != IMG_SET_OP_MODE_16) && (p->img_mode.op_mode != IMG_SET_OP_MODE_17) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_18) && (p->img_mode.op_mode != IMG_SET_OP_MODE_19) && (p->img_mode.op_mode != IMG_SET_OP_MODE_20) &&
       (p->img_mode.op_mode != IMG_SET_OP_MODE_21) && (p->img_mode.op_mode != IMG_SET_OP_MODE_22) && (p->img_mode.op_mode != IMG_SET_OP_MODE_23))
    {
        ERR_PRINT("img_mode.op_mode", p->img_mode.op_mode);
        return FALSE;
    }

    /* check src image(FRONT) */
    if ((p->img_mode.op_mode << IMG_OP_MODE_SFT) & IMG_OVERLAY_BIT)
    {
        mix_front_image = MIX_ON;
    }
    else
    {
        mix_front_image = MIX_OFF;
    }

    /* chack resize */
    if((p->img_mode.op_mode == IMG_SET_OP_MODE_02) || (p->img_mode.op_mode == IMG_SET_OP_MODE_03) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_04) || (p->img_mode.op_mode == IMG_SET_OP_MODE_05) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_12) || (p->img_mode.op_mode == IMG_SET_OP_MODE_13) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_14) || (p->img_mode.op_mode == IMG_SET_OP_MODE_15) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_16) || (p->img_mode.op_mode == IMG_SET_OP_MODE_17) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_18) || (p->img_mode.op_mode == IMG_SET_OP_MODE_19))
    {
        /* back image resize on */
        img_resize = RESIZE_ON_BACK;
    }
    else if((p->img_mode.op_mode == IMG_SET_OP_MODE_08) || (p->img_mode.op_mode == IMG_SET_OP_MODE_09) ||
            (p->img_mode.op_mode == IMG_SET_OP_MODE_10) || (p->img_mode.op_mode == IMG_SET_OP_MODE_11) ||
            (p->img_mode.op_mode == IMG_SET_OP_MODE_20) || (p->img_mode.op_mode == IMG_SET_OP_MODE_21) ||
            (p->img_mode.op_mode == IMG_SET_OP_MODE_22) || (p->img_mode.op_mode == IMG_SET_OP_MODE_23))
    {
        /* front image resize on */
        img_resize = RESIZE_ON_FRONT;
    }
    else
    {
        /* no resize */
        img_resize = RESIZE_OFF;
    }

    /* check mask */
    if((p->img_mode.op_mode == IMG_SET_OP_MODE_12) || (p->img_mode.op_mode == IMG_SET_OP_MODE_18))
    {
        /* check mask color flag */
        if(p->img_mode.msk_col == IMG_SET_MASKCOLOR_ON)
        {
            if ((p->img_mode.dstfmt == IMG_FORM_RGB666) || (p->img_mode.dstfmt == IMG_FORM_RGB565))
            {
                img_maskcolor = MASK_ON;
            }
            else
            {
                img_maskcolor = MASK_OFF;
            }
        }
        else if(p->img_mode.msk_col == IMG_SET_MASKCOLOR_OFF)
        {
            img_maskcolor = MASK_OFF;
        }
        else
        {
            ERR_PRINT("img_mode.msk_col", p->img_mode.msk_col);
            return FALSE;
        }
    }

    if ((p->img_mode.dstfmt == IMG_FORM_RGB666) || (p->img_mode.dstfmt == IMG_FORM_RGB565))
    {
        if (mix_front_image)
        {
            /* check enable dither(FRONT) flag */
            if(p->img_mode.dither_f  > (char)IMG_SET_DITHER_ON)
            {
                ERR_PRINT("img_mode.dither_f", p->img_mode.dither_f);
                return FALSE;
            }
        }
        /* check enable dither(BACK) flag */
        if(p->img_mode.dither_b  > (char)IMG_SET_DITHER_ON)
        {
            ERR_PRINT("img_mode.dither_b", p->img_mode.dither_b);
            return FALSE;
        }
    }

    if(img_resize == RESIZE_ON_FRONT)
    {
        /* check filter(FRONT) flag */
        if(p->img_mode.rfilter_f > (char)IMG_SET_SMOOTH_FILTER)
        {
            ERR_PRINT("img_mode.rfilter_f", p->img_mode.rfilter_f);
            return FALSE;
        }
    }
    if(img_resize == RESIZE_ON_BACK)
    {
        /* check filter(BACK) flag */
        if(p->img_mode.rfilter_b > (char)IMG_SET_SMOOTH_FILTER)
        {
            ERR_PRINT("img_mode.rfilter_b", p->img_mode.rfilter_b);
            return FALSE;
        }
    }

    /* check up/down mirroring flag */
    if(p->img_mode.udinv > (char)IMG_SET_MIRROR_ON)
    {
        ERR_PRINT("img_mode.udinv", p->img_mode.udinv);
        return FALSE;
    }
    /* check right/left mirroring flag */
    if(p->img_mode.rlinv > (char)IMG_SET_MIRROR_ON)
    {
        ERR_PRINT("img_mode.rlinv", p->img_mode.rlinv);
        return FALSE;
    }

/*====================*/
/* Image Format check */
/*====================*/
    if (mix_front_image)
    {
        /* check src image(FRONT) */
        if((p->img_mode.op_mode == IMG_SET_OP_MODE_09) || (p->img_mode.op_mode == IMG_SET_OP_MODE_10) ||
           (p->img_mode.op_mode == IMG_SET_OP_MODE_13) || (p->img_mode.op_mode == IMG_SET_OP_MODE_14) ||
           (p->img_mode.op_mode == IMG_SET_OP_MODE_16) || (p->img_mode.op_mode == IMG_SET_OP_MODE_19) ||
           (p->img_mode.op_mode == IMG_SET_OP_MODE_20) || (p->img_mode.op_mode == IMG_SET_OP_MODE_23))
        {
            /* src image(FRONT) YUV422/420 */
            src_yuv_front = YUV_IMAGE;
            if ((p->img_mode.srcfmt_f != IMG_FORM_YUV420_PLANAR)      && (p->img_mode.srcfmt_f != IMG_FORM_YUV422_PLANAR) &&
                (p->img_mode.srcfmt_f != IMG_FORM_YUV420_SEMI_PLANAR) && (p->img_mode.srcfmt_f != IMG_FORM_YUV422_SEMI_PLANAR) &&
                (p->img_mode.srcfmt_f != IMG_FORM_YUV422_INTERLEAVE))
            {
                ERR_PRINT("img_mode.srcfmt_f", p->img_mode.srcfmt_f);
                return FALSE;
            }
        }
        else
        {
            /* src image(FRONT) RGB888/565 */
            src_yuv_front = RGB_IMAGE;
            if ((p->img_mode.srcfmt_f != IMG_FORM_RGB888) && (p->img_mode.srcfmt_f != IMG_FORM_RGB565))
            {
                ERR_PRINT("img_mode.srcfmt_f", p->img_mode.srcfmt_f);
                return FALSE;
            }
        }

        if((p->img_mode.srcfmt_b == IMG_FORM_YUV420_PLANAR) || (p->img_mode.srcfmt_b == IMG_FORM_YUV422_PLANAR) ||
           (p->img_mode.srcfmt_f == IMG_FORM_YUV420_PLANAR) || (p->img_mode.srcfmt_f == IMG_FORM_YUV422_PLANAR))
        {
            if (p->img_mode.srcfmt_f != p->img_mode.srcfmt_b)
            {
                printk( KERN_INFO " @ipu: IMGIO_IMAGE: Error! combination of img_mode.srcfmt_b and img_mode.srcfmt_f is incorrect.\n");
                return FALSE;
            }
        }
        else if(p->img_mode.srcfmt_b == IMG_FORM_YUV422_INTERLEAVE)
        {
            if ((p->img_mode.srcfmt_f != IMG_FORM_YUV422_INTERLEAVE) && (p->img_mode.srcfmt_f != IMG_FORM_RGB565))
            {
                printk( KERN_INFO " @ipu: IMGIO_IMAGE: Error! combination of img_mode.srcfmt_b and img_mode.srcfmt_f is incorrect.\n");
                return FALSE;
            }
        }
        else if(p->img_mode.srcfmt_f == IMG_FORM_YUV422_INTERLEAVE)
        {
            if ((p->img_mode.srcfmt_b != IMG_FORM_YUV422_INTERLEAVE) && (p->img_mode.srcfmt_b != IMG_FORM_RGB565))
            {
                printk( KERN_INFO " @ipu: IMGIO_IMAGE: Error! combination of img_mode.srcfmt_b and img_mode.srcfmt_f is incorrect.\n");
                return FALSE;
            }
        }
    }

    /* check src image(BACK) */
    if((p->img_mode.op_mode == IMG_SET_OP_MODE_01) || (p->img_mode.op_mode == IMG_SET_OP_MODE_03) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_05) || (p->img_mode.op_mode == IMG_SET_OP_MODE_07) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_10) || (p->img_mode.op_mode == IMG_SET_OP_MODE_11) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_14) || (p->img_mode.op_mode == IMG_SET_OP_MODE_15) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_18) || (p->img_mode.op_mode == IMG_SET_OP_MODE_19) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_22) || (p->img_mode.op_mode == IMG_SET_OP_MODE_23))
    {
        /* src image(BACK) YUV422/420 */
        src_yuv_back = YUV_IMAGE;
        if ((p->img_mode.srcfmt_b != IMG_FORM_YUV420_PLANAR)      && (p->img_mode.srcfmt_b != IMG_FORM_YUV422_PLANAR) &&
            (p->img_mode.srcfmt_b != IMG_FORM_YUV420_SEMI_PLANAR) && (p->img_mode.srcfmt_b != IMG_FORM_YUV422_SEMI_PLANAR) &&
            (p->img_mode.srcfmt_b != IMG_FORM_YUV422_INTERLEAVE))
        {
            ERR_PRINT("img_mode.srcfmt_b", p->img_mode.srcfmt_b);
            return FALSE;
        }
    }
    else
    {
        /* src image(BACK) RGB888/565 */
        src_yuv_back = RGB_IMAGE;
        if ((p->img_mode.srcfmt_b != IMG_FORM_RGB888) && (p->img_mode.srcfmt_b != IMG_FORM_RGB565))
        {
            ERR_PRINT("img_mode.srcfmt_b", p->img_mode.srcfmt_b);
            return FALSE;
        }
    }

    /* check dest image */
    if((p->img_mode.op_mode == IMG_SET_OP_MODE_01) || (p->img_mode.op_mode == IMG_SET_OP_MODE_03) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_04) || (p->img_mode.op_mode == IMG_SET_OP_MODE_06) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_10) || (p->img_mode.op_mode == IMG_SET_OP_MODE_11) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_14) || (p->img_mode.op_mode == IMG_SET_OP_MODE_15) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_16) || (p->img_mode.op_mode == IMG_SET_OP_MODE_17) ||
       (p->img_mode.op_mode == IMG_SET_OP_MODE_20) || (p->img_mode.op_mode == IMG_SET_OP_MODE_21))
    {
        /* dest image YUV422/420 */
        dest_yuv = YUV_IMAGE;
        if ((p->img_mode.dstfmt != IMG_FORM_YUV420_PLANAR)      && (p->img_mode.dstfmt != IMG_FORM_YUV422_PLANAR) &&
            (p->img_mode.dstfmt != IMG_FORM_YUV420_SEMI_PLANAR) && (p->img_mode.dstfmt != IMG_FORM_YUV422_SEMI_PLANAR) &&
            (p->img_mode.dstfmt != IMG_FORM_YUV422_INTERLEAVE))
        {
            ERR_PRINT("img_mode.dstfmt", p->img_mode.dstfmt);
            return FALSE;
        }
    }
    else
    {
        /* dest image RGB888/666/565 */
        dest_yuv = RGB_IMAGE;
        if ((p->img_mode.dstfmt != IMG_FORM_RGB888) && (p->img_mode.dstfmt != IMG_FORM_RGB666) &&
            (p->img_mode.dstfmt != IMG_FORM_RGB565))
        {
            ERR_PRINT("img_mode.dstfmt", p->img_mode.dstfmt);
            return FALSE;
        }
    }

#if _IPU_COLCONV
/*========================*/
/* color-space conversion */
/*========================*/
    if(src_yuv_back != dest_yuv){
        if(src_yuv_back == YUV_IMAGE){
            matrix_flag = IMG_YUVRGB_CONF;
        }else{
            matrix_flag = IMG_RGBYUV_CONF;
        }
    }
    else if(mix_front_image && src_yuv_front != dest_yuv){
        if(src_yuv_front == YUV_IMAGE){
            matrix_flag = IMG_YUVRGB_CONF;
        }else{
            matrix_flag = IMG_RGBYUV_CONF;
        }
    }
    else{
        matrix_flag = 0;
    }

    /* set_matrix == FALSE is IMG initial value */
    if(matrix_flag && !p->set_matrix){
        matrix_flag = 0;
    }

    if(matrix_flag){
        int i;
        matrix_conf = p->matrix_conf;
        for(i = 0; i < 9; i++){
            matrix[i] = p->matrix[i];
        }
    }
#endif

    return TRUE;
}


/*****************************************************************************
* MODULE   : mp200_img_chkstatus
* FUNCTION : check IMG_MODE to internal flag
* IMPORT   : none
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/10/18 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_img_chkstatus(void)
{
    if (img_maskcolor)
    {
        if(src_format_f == IMG_FORM_RGB888)            /* RGB888 */
        {
            if(mask_color > 0xFFFFFF)    goto fail_maskcolor;
        }
        else                                           /* RGB565 */
        {
            if(mask_color > 0xFFFF)      goto fail_maskcolor;
        }
        if(mask_alpha > IMG_ALPHA_BIT) goto fail_alpha;
    }

    if (img_resize == RESIZE_OFF)
    {
        /* no resize */
        dest_width  = src_width;
        dest_height = src_height;
    }

    /* pixel size */
    if(src_yuv_back)
    {
        if(src_format_b == IMG_FORM_YUV422_INTERLEAVE)
            bits_par_pixel_B = BPP_YUV422_INTER;
        else
            bits_par_pixel_B = BPP_YUV420_422;
    }
    else
    {
        if(src_format_b == IMG_FORM_RGB888)
            bits_par_pixel_B = BPP_RGB888;
        else
            bits_par_pixel_B = BPP_RGB565;

    }
    if(src_yuv_front)
    {
        if(src_format_f == IMG_FORM_YUV422_INTERLEAVE)
            bits_par_pixel_F = BPP_YUV422_INTER;
        else
            bits_par_pixel_F = BPP_YUV420_422;
    }
    else
    {
        if(src_format_f == IMG_FORM_RGB888)
            bits_par_pixel_F = BPP_RGB888;
        else
            bits_par_pixel_F = BPP_RGB565;
    }

    if(dest_yuv)
    {
        if(dst_format == IMG_FORM_YUV422_INTERLEAVE)
            bits_par_pixel_D = BPP_YUV422_INTER;
        else
            bits_par_pixel_D = BPP_YUV420_422;
    }
    else
    {
        if(dst_format == IMG_FORM_RGB888)
            bits_par_pixel_D = BPP_RGB888;
        else if(dst_format == IMG_FORM_RGB666)
            bits_par_pixel_D = BPP_RGB666;
        else
            bits_par_pixel_D = BPP_RGB565;
    }

    return TRUE;

fail_maskcolor:
    ERR_PRINT("maskcolr", mask_color);
    return FALSE;
fail_alpha:
    ERR_PRINT("alpha", mask_alpha);
    return FALSE;
}


/*****************************************************************************
* MODULE   : mp200_img_chksize
* FUNCTION : check image size
* IMPORT   : unsigned long back_size
*          : unsigned long front_size
*          : unsigned long dest_size
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/10/18 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_img_chksize(unsigned long back_size, unsigned long front_size, unsigned long dest_size, char *num_size)
{
    /* src image(BACK) size check */
    if(mp200_img_chksize_sub(back_size, IMG_FORMAT_SRCFMT_B_BIT, IMG_FORMAT_SRCFMT_B_SFT, num_size) == FALSE)      return FALSE;
    /* src image(FRONT) size check */
    if(mix_front_image)
    {
        if(mp200_img_chksize_sub(front_size, IMG_FORMAT_SRCFMT_F_BIT, IMG_FORMAT_SRCFMT_F_SFT, num_size) == FALSE) return FALSE;

        /* src image(FRONT) > src image(BACK) is incorrect. except (OP_MODE12 or OP_MODE18) */
        if(((img_mode & IMG_OP_MODE_BIT) != IMG_OP_MODE_12_BIT) && ((img_mode & IMG_OP_MODE_BIT) != IMG_OP_MODE_18_BIT))
        {
            if(img_resize == RESIZE_ON_BACK)           /* back image resize */
            {
                if(front_size > dest_size)
                {
                    printk( KERN_INFO " @ipu: IMGIO_CONVERT: Error! src_f_image.%s > dest_image.%s is incorrect. (%ld > %ld)\n",
                        num_size, num_size, front_size, dest_size);
                    return FALSE;
                }
            }
            else if(img_resize == RESIZE_ON_FRONT)     /* front image resize */
            {
                if(dest_size > back_size)
                {
                    printk( KERN_INFO " @ipu: IMGIO_CONVERT: Error! dest_image.%s > src_b_image.%s is incorrect. (%ld > %ld)\n",
                        num_size, num_size, dest_size, back_size);
                    return FALSE;
                }
            }
            else                                       /* no resize */
            {
                if(front_size > back_size)
                {
                    printk( KERN_INFO " @ipu: IMGIO_CONVERT: Error! src_f_image.%s > src_b_image.%s is incorrect. (%ld > %ld)\n",
                        num_size, num_size, front_size, back_size);
                    return FALSE;
                }
            }
        }
    }
    /* dest image size check */ /* dest_width is always checked, because it's necessary for checking pitch. */
                                /* dest_height is checked only at resize.                                   */
    if(strcmp(num_size, "hsize") == 0 || img_resize)
    {
        if(mp200_img_chksize_sub(dest_size, IMG_FORMAT_DSTFMT_BIT, IMG_FORMAT_DSTFMT_SFT, num_size) == FALSE)    return FALSE;
    }

    /* resize check */
    if(img_resize)
    {
        unsigned long calc_size, max_rng, min_rng;
        int  src_yuv;
        char *num;

        if(img_resize == RESIZE_ON_BACK)               /* back image */
        {
            num       = "src_b_image.";
            src_yuv   = src_yuv_back;
            calc_size = back_size;
        }
        else                                           /* front image */
        {
            num       = "src_f_image.";
            src_yuv   = src_yuv_front;
            calc_size = front_size;
        }

        if(strcmp(num_size, "hsize") == 0) min_rng = 40; /* width  min range x1/40 */
        else                               min_rng = 40; /* height min range x1/40 */

        max_rng = 4;                                   /* max range x4 */

        if(((min_rng * dest_size) < calc_size) || (dest_size > (max_rng * calc_size)))
        {
            printk(" @ipu: IMGIO_CONVERT: Error! dest_image.%s / %s%s(%ld/%ld) (1/%ld - *%ld)\n", num_size, num, num_size, dest_size, calc_size, min_rng, max_rng);
            return FALSE;
        }
    }

    return TRUE;
}


/*****************************************************************************
* MODULE   : mp200_img_chksize_sub
* FUNCTION : check image size
* IMPORT   : unsigned long size
*          : unsigned long mode_fmt_bit
*          : unsigned long mode_fmt_sft
*          : char         *num_size
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/11/07 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_img_chksize_sub(unsigned long size, unsigned long mode_fmt_bit, unsigned long mode_fmt_sft, char *num_size)
{
    int   yuv_flag;
    char  *num;
    unsigned int min_size;

    if(mode_fmt_bit == IMG_FORMAT_SRCFMT_B_BIT)        /* src back */
    {
        yuv_flag = src_yuv_back;
        min_size = 8;
        num      = "src_b_image.";
    }
    else if(mode_fmt_bit == IMG_FORMAT_SRCFMT_F_BIT)   /* src front */
    {
        yuv_flag = src_yuv_front;
        min_size = 8;
        num      = "src_f_image.";
    }
    else /* IMG_FORMAT_DSTFMT_BIT */
    {
        yuv_flag = dest_yuv;
        min_size = 16;
        num      = "dst_image.";
    }

    if(size < min_size || size > 8188)
    {
        goto fail_size;
    }

    if(yuv_flag)                                       /* YUV422/YUV420 */
    {
        if((((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV420_PLANAR) ||      /* YUV420-Planar */
           (((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV422_PLANAR))        /* YUV422-Planar */
        {
            if(size & 0x7)
            {
                goto fail_size;
            }
        }
        else if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV422_INTERLEAVE) /* YUV422-InterLeave */
        {
            if(size & 0x1)
            {
                goto fail_size;
            }
        }
        else                                           /* YUV422-SemiPlanar/YUV420-SemiPlanar */
        {
            if(size & 0x3)
            {
                goto fail_size;
            }
        }
    }
    else                                               /* RGB565/RGB666/RGB888 */
    {
        if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_RGB888)                 /* RGB888 */
        {
            if(size & 0x3)
            {
                goto fail_size;
            }
        }
        else if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_RGB666)            /* RGB666 */
        {
            if(size & 0xF)
            {
                goto fail_size;
            }
        }
        else                                           /* RGB565 */
        {
            if(size & 0x1)
            {
                goto fail_size;
            }
        }
    }

    return TRUE;

fail_size:
    printk( KERN_INFO " @ipu: IMGIO_CONVERT: Error! %s%s is incorrect. %s%s = 0x%lx\n", num, num_size, num, num_size, size);
    return FALSE;
}


/*****************************************************************************
* MODULE   : mp200_img_chkcoordinate
* FUNCTION : check image (x,y)chkcoordinate
* IMPORT   :
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/10/18 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_chkcoordinate(void)
{
    /* check pitch or x ,over width, by 4 pixel */
    /* input data(BACK) check */
    if(mp200_img_chkcoordinate_sub(IMG_FORMAT_SRCFMT_B_BIT, IMG_FORMAT_SRCFMT_B_SFT) == FALSE)
    {
        return FALSE;
    }
    /* input data(FRONT) check */
    if(mix_front_image)
    {
        if(mp200_img_chkcoordinate_sub(IMG_FORMAT_SRCFMT_F_BIT, IMG_FORMAT_SRCFMT_F_SFT) == FALSE)
        {
            return FALSE;
        }
    }
    /* output data check */
    if(mp200_img_chkcoordinate_sub(IMG_FORMAT_DSTFMT_BIT, IMG_FORMAT_DSTFMT_SFT) == FALSE)
    {
        return FALSE;
    }

    /* check x & check y by 2 pixel */
    /* input data(BACK) check */
    if(mp200_img_chkcoordinate_x_sub(src_x, IMG_FORMAT_SRCFMT_B_BIT, IMG_FORMAT_SRCFMT_B_SFT) == FALSE) return FALSE;
    if(mp200_img_chkcoordinate_y_sub(src_y, IMG_FORMAT_SRCFMT_B_BIT, IMG_FORMAT_SRCFMT_B_SFT) == FALSE) return FALSE;
    /* input data(FRONT) check */
    if(mix_front_image)
    {
        if(mp200_img_chkcoordinate_x_sub(src_x_F, IMG_FORMAT_SRCFMT_F_BIT, IMG_FORMAT_SRCFMT_F_SFT) == FALSE) return FALSE;
        if(mp200_img_chkcoordinate_y_sub(src_y_F, IMG_FORMAT_SRCFMT_F_BIT, IMG_FORMAT_SRCFMT_F_SFT) == FALSE) return FALSE;
    }
    /* output data check */
    if(mp200_img_chkcoordinate_x_sub(dest_x, IMG_FORMAT_DSTFMT_BIT, IMG_FORMAT_DSTFMT_SFT) == FALSE) return FALSE;
    if(mp200_img_chkcoordinate_y_sub(dest_y, IMG_FORMAT_DSTFMT_BIT, IMG_FORMAT_DSTFMT_SFT) == FALSE) return FALSE;

    /* check offset */
    if(mix_front_image)
    {
        unsigned long sw, sh;
        if(img_resize == RESIZE_ON_BACK)               /* back image resize */
        {
            sw = src_width_F;
            sh = src_height_F;
        }
        else
        {
            sw = src_width;
            sh = src_height;
        }

        if(dest_yuv)
        {
            if(offset_x & 0x1)
            {
                goto fail_offset_x;
            }
            if(offset_y & 0x1)
            {
                goto fail_offset_y;
            }
        }

        if((dest_width == sw) && (dest_height == sh))
        {
            /* no offset */
            if(offset_x != 0) goto fail_offset_x;
            if(offset_y != 0) goto fail_offset_y;
        }
        else if((dest_width >= sw) && (dest_height >= sh))
        {
            if(offset_x > (dest_width  - sw)) goto fail_offset_x;
            if(offset_y > (dest_height - sh)) goto fail_offset_y;
        }
        else if((dest_width <= sw) && (dest_height <= sh))
        {
            if(offset_x > (sw - dest_width))  goto fail_offset_x;
            if(offset_y > (sh - dest_height)) goto fail_offset_y;
        }
        else
        {
            goto fail_ops;
        }
    }

    return TRUE;

fail_offset_x:
    ERR_PRINT("offsetx", offset_x);
    return FALSE;
fail_offset_y:
    ERR_PRINT("offsety", offset_y);
    return FALSE;
fail_ops:
    printk(" @ipu: IMGIO_CONVERT: Error! This image position, doesn't included even to BACK and FRONT any.\n");
    return FALSE;
}

/*****************************************************************************
* MODULE   : mp200_img_chkcoordinate_sub
* FUNCTION : check memory size(pitch)
* IMPORT   : unsigned long mode_fmt_bit
*          : unsigned long mode_fmt_sft
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2007/03/19 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_img_chkcoordinate_sub(unsigned long mode_fmt_bit, unsigned long mode_fmt_sft)
{
    unsigned long size, x, w, max_size;
    int  bpp, pix_dist;
    char *num1, *num2;

    if(mode_fmt_bit == IMG_FORMAT_SRCFMT_B_BIT)
    {
        size     = ulSrcScreenWidth;
        x        = src_x;
        w        = src_width;
        max_size = MAX_SCREEN_WIDTH_BACK;
        bpp      = bits_par_pixel_B;
        num1     = "src_b_image.size";
        num2     = "src_b_image.x";
    }
    else if(mode_fmt_bit == IMG_FORMAT_SRCFMT_F_BIT)
    {
        size     = ulSrcScreenWidth_F;
        x        = src_x_F;
        w        = src_width_F;
        max_size = MAX_SCREEN_WIDTH;
        bpp      = bits_par_pixel_F;
        num1     = "src_f_image.size";
        num2     = "src_f_image.x";
    }
    else /* IMG_FORMAT_DSTFMT_BIT */
    {
        size     = ulDestScreenWidth;
        x        = dest_x;
        w        = dest_width;
        max_size = MAX_SCREEN_WIDTH;
        bpp      = bits_par_pixel_D;
        num1     = "dest_image.size";
        num2     = "dest_image.x";
    }

    if((((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV420_PLANAR) ||          /* YUV422-Planar     */
       (((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV422_PLANAR))            /* YUV422-Planar     */
    {
        pix_dist  = 8 * bpp / 8;
    }
    else if((((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV422_INTERLEAVE) || /* YUV422-Interleave */
            (((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_RGB565))              /* RGB565            */
    {
        pix_dist  = 2 * bpp / 8;
    }
    else if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_RGB666)                /* RGB666            */
    {
        pix_dist  = 16 * bpp / 8;
    }
    else                                                                                 /* YUV420-Semi-Planar / YUV422-Semi-Planar / RGB888 */
    {
        pix_dist  = 4 * bpp / 8;
    }
    max_size = max_size / pix_dist * pix_dist;

    /* check */
    if((size / (bpp / 8)) < (x + w))
    {
        printk( KERN_INFO " @ipu: IMGIO_CONVERT: Error! %s or %s is incorrect. size = %ld, x = %ld\n", num1, num2, size, x);
        return FALSE;
    }
    if(size > max_size)
    {
        ERR_PRINT(num1, size);
        return FALSE;
    }
    if(size % pix_dist){
        ERR_PRINT(num1, size);
        return FALSE;
    }

    return TRUE;
}

/*****************************************************************************
* MODULE   : mp200_img_chkcoordinate_x_sub
* FUNCTION : check image x coordinate
* IMPORT   : unsigned long pos
*          : unsigned long mode_fmt_bit
*          : unsigned long mode_fmt_sft
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/04/17 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_img_chkcoordinate_x_sub(unsigned long pos, unsigned long mode_fmt_bit, unsigned long mode_fmt_sft)
{
    int   yuv_flag;
    char *num;

    if(mode_fmt_bit == IMG_FORMAT_SRCFMT_B_BIT)
    {
        yuv_flag = src_yuv_back;
        num      = "src_b_image.x";
    }
    else if(mode_fmt_bit == IMG_FORMAT_SRCFMT_F_BIT)
    {
        yuv_flag = src_yuv_front;
        num      = "src_f_image.x";
    }
    else                                                                                 /* IMG_FORMAT_DSTFMT_BIT */
    {
        yuv_flag = dest_yuv;
        num      = "dest_image.x";
    }

    if(yuv_flag)                                                                         /* YUV422/YUV420 */
    {
        if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV420_PLANAR)          /* YUV420-Planar */
        {
            if(pos & 0x7)
            {
                goto fail_pos;
            }
        }
        else                                                                            /* YUV422-SemiPlanar/YUV422-InterLeave/YUV420-SemiPlanar */
        {
            if(pos & 0x3)
            {
                goto fail_pos;
            }
        }
    }
    else                                                                                 /* RGB565/RGB888/RGB666 */
    {
        if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_RGB888)                 /* RGB888 */
        {
            if(pos & 0x3)
            {
                goto fail_pos;
            }
        }
        else if(((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_RGB666)            /* RGB666 */
        {
            if(pos & 0xF)
            {
                goto fail_pos;
            }
        }
        else                                                                             /* RGB565 */
        {
            if(pos & 0x1)
            {
                goto fail_pos;
            }
        }
    }

    return TRUE;

fail_pos:
    ERR_PRINT(num, pos);
    return FALSE;
}


/*****************************************************************************
* MODULE   : mp200_img_chkcoordinate_y_sub
* FUNCTION : check image y coordinate
* IMPORT   : unsigned long pos
*          : unsigned long mode_fmt_bit
*          : unsigned long mode_fmt_sft
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/05/9 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_img_chkcoordinate_y_sub(unsigned long pos, unsigned long mode_fmt_bit, unsigned long mode_fmt_sft)
{
    int   yuv_flag;
    char *num;

    if(mode_fmt_bit == IMG_FORMAT_SRCFMT_B_BIT)
    {
        yuv_flag = src_yuv_back;
        num      = "src_b_image.y";
    }
    else if(mode_fmt_bit == IMG_FORMAT_SRCFMT_F_BIT)
    {
        yuv_flag = src_yuv_front;
        num      = "src_f_image.y";
    }
    else                                                                                 /* IMG_FORMAT_DSTFMT_BIT */
    {
        yuv_flag = dest_yuv;
        num      = "dest_image.y";
    }

    if((((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV420_PLANAR) ||          /* YUV420 */
       (((format & mode_fmt_bit) >> mode_fmt_sft) == IMG_FORM_YUV420_SEMI_PLANAR))
    {
        if(pos & 0x1)
        {
            goto fail_pos;
        }
    }

    return TRUE;

fail_pos:
    ERR_PRINT(num, pos);
    return FALSE;
}


/*****************************************************************************
* MODULE   : mp200_img_chkaddr
* FUNCTION : check image address
* IMPORT   : none
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/11/09 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static int mp200_img_chkaddr(void)
{
    /* check input(BACK) Y/RGB data start address by 4 byte */
    if(ulSrc == 0 || (ulSrc & 0x3) != 0)
    {
        ERR_PRINT("src_b_image.yrgbadr", ulSrc);
        return FALSE;
    }

    /* check input(BACK) UV data start address by 4 byte */
    if(src_yuv_back && (src_format_b != IMG_FORM_YUV422_INTERLEAVE))
    {
        if(ulSrcU == 0 || (ulSrcU & 0x3) != 0)
        {
            ERR_PRINT("src_b_image.uvadr", ulSrcU);
            return FALSE;
        }
    }

    /* check input(BACK) V data start address by 4 byte */
    if(src_format_b == IMG_FORM_YUV420_PLANAR || src_format_b == IMG_FORM_YUV422_PLANAR)
    {
        if(ulSrcV == 0 || (ulSrcV & 0x3) != 0)
        {
            ERR_PRINT("src_b_image.vadr", ulSrcV);
            return FALSE;
        }
    }

    /* check input(FRONT) Y/RGB data start address by 4 byte */
    if(mix_front_image)
    {
        if(ulSrc_F == 0 || (ulSrc_F & 0x3) != 0)
        {
            ERR_PRINT("src_f_image.yrgbadr", ulSrc_F);
            return FALSE;
        }

        /* check input(FRONT) UV data start address by 4 byte */
        if(src_yuv_front && (src_format_f != IMG_FORM_YUV422_INTERLEAVE))
        {
            if(ulSrcU_F == 0 || (ulSrcU_F & 0x3) != 0)
            {
                ERR_PRINT("src_f_image.uvadr", ulSrcU_F);
                return FALSE;
            }
        }

        /* check input(FRONT) V data start address by 4 byte */
        if(src_format_f == IMG_FORM_YUV420_PLANAR || src_format_f == IMG_FORM_YUV422_PLANAR)
        {
            if(ulSrcV_F == 0 || (ulSrcV_F & 0x3) != 0)
            {
                ERR_PRINT("src_f_image.vadr", ulSrcV_F);
                return FALSE;
            }
        }
    }

    /* check output Y/RGB data start address by 4 byte */
    if(ulDest == 0 || (ulDest & 0x3) != 0)
    {
        ERR_PRINT("dest_image.yrgbadr", ulDest);
        return FALSE;
    }

    /* check output UV data start address by 4 byte */
    if(dest_yuv && (dst_format != IMG_FORM_YUV422_INTERLEAVE))
    {
        if(ulDestU == 0 || (ulDestU & 0x3) != 0)
        {
            ERR_PRINT("dest_image.uvadr", ulDestU);
            return FALSE;
        }
    }

    /* check output V data start address by 4 byte */
    if(dst_format == IMG_FORM_YUV420_PLANAR || dst_format == IMG_FORM_YUV422_PLANAR)
    {
        if(ulDestV == 0 || (ulDestV & 0x3) != 0)
        {
            ERR_PRINT("dest_image.vadr", ulDestV);
            return FALSE;
        }
    }

    return TRUE;
}


/*****************************************************************************
* MODULE   : mp200_img_setaddr
* FUNCTION : set image address
* IMPORT   : none
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/11/09 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_img_setaddr(void)
{
    /* set src image address(BACK) */
    ulSrcStarAddrYRGB = ulSrc + src_x  * bits_par_pixel_B / 8 + src_y * ulSrcScreenWidth;
    switch(src_format_b)
    {
        /* YUV422 Semi-Planar */
        case IMG_FORM_YUV422_SEMI_PLANAR:
            ulSrcStarAddrU   = ulSrcU + src_x + src_y * ulSrcScreenWidth;
            ulSrcStarAddrV_B = 0L;
            break;

        /* YUV420 Semi-Planar */
        case IMG_FORM_YUV420_SEMI_PLANAR:
            ulSrcStarAddrU   = ulSrcU + src_x + src_y / 2 * ulSrcScreenWidth;
            ulSrcStarAddrV_B = 0L;
            break;

        /* YUV422 Planar */
        case IMG_FORM_YUV422_PLANAR:
            ulSrcStarAddrU   = ulSrcU + (src_x / 2) + src_y * ulSrcScreenWidth;
            ulSrcStarAddrV_B = ulSrcV + (src_x / 2) + src_y * ulSrcScreenWidth;
            break;

        /* YUV420 Planar */
        case IMG_FORM_YUV420_PLANAR:
            ulSrcStarAddrU   = ulSrcU + (src_x / 2) + src_y / 2 * ulSrcScreenWidth;
            ulSrcStarAddrV_B = ulSrcV + (src_x / 2) + src_y / 2 * ulSrcScreenWidth;
            break;

        /* YUV422 Interleave */
        case IMG_FORM_YUV422_INTERLEAVE:
        /* RGB565/888 */
        case IMG_FORM_RGB888:
        case IMG_FORM_RGB565:
            ulSrcStarAddrU   = 0L;
            ulSrcStarAddrV_B = 0L;
            break;
    }

    /* set src image address(FRONT) */
    if(mix_front_image)
    {
        ulSrcStarAddrYRGB_F  = ulSrc_F + src_x_F * bits_par_pixel_F / 8 + src_y_F * ulSrcScreenWidth_F;;
        switch(src_format_f)
        {
            /* YUV422 Semi-Planar */
            case IMG_FORM_YUV422_SEMI_PLANAR:
                ulSrcStarAddrU_F = ulSrcU_F + src_x_F + src_y_F * ulSrcScreenWidth_F;
                ulSrcStarAddrV_F = 0L;
                break;

            /* YUV420 Semi-Planar */
            case IMG_FORM_YUV420_SEMI_PLANAR:
                ulSrcStarAddrU_F = ulSrcU_F + src_x_F + src_y_F / 2 * ulSrcScreenWidth_F;
                ulSrcStarAddrV_F = 0L;
                break;

            /* YUV422 Planar */
            case IMG_FORM_YUV422_PLANAR:
                ulSrcStarAddrU_F = ulSrcU_F + (src_x_F / 2) + src_y_F * ulSrcScreenWidth_F;
                ulSrcStarAddrV_F = ulSrcV_F + (src_x_F / 2) + src_y_F * ulSrcScreenWidth_F;
                break;

            /* YUV420 Planar */
            case IMG_FORM_YUV420_PLANAR:
                ulSrcStarAddrU_F = ulSrcU_F + (src_x_F / 2) + src_y_F / 2 * ulSrcScreenWidth_F;
                ulSrcStarAddrV_F = ulSrcV_F + (src_x_F / 2) + src_y_F / 2 * ulSrcScreenWidth_F;
                break;

            /* YUV422 Interleave */
            case IMG_FORM_YUV422_INTERLEAVE:
            /* RGB565/888 */
            case IMG_FORM_RGB888:
            case IMG_FORM_RGB565:
                ulSrcStarAddrU_F = 0L;
                ulSrcStarAddrV_F = 0L;
                break;
        }
    }

    /* set dest image address */
    ulDestStarAddrYRGB = ulDest + dest_x * bits_par_pixel_D / 8 + dest_y * ulDestScreenWidth;
    switch(dst_format)
    {
        /* YUV422 Semi-Planar */
        case IMG_FORM_YUV422_SEMI_PLANAR:
            ulDestStarAddrU = ulDestU + dest_x + dest_y * ulDestScreenWidth;
            ulDestStarAddrV = 0L;
            break;

        /* YUV420 Semi-Planar */
        case IMG_FORM_YUV420_SEMI_PLANAR:
            ulDestStarAddrU = ulDestU + dest_x + dest_y / 2 * ulDestScreenWidth;
            ulDestStarAddrV = 0L;
            break;

        /* YUV422 Planar */
        case IMG_FORM_YUV422_PLANAR:
            ulDestStarAddrU = ulDestU + (dest_x / 2) + dest_y * ulDestScreenWidth;
            ulDestStarAddrV = ulDestV + (dest_x / 2) + dest_y * ulDestScreenWidth;
            break;

        /* YUV420 Planar */
        case IMG_FORM_YUV420_PLANAR:
            ulDestStarAddrU = ulDestU + (dest_x / 2) + dest_y / 2 * ulDestScreenWidth;
            ulDestStarAddrV = ulDestV + (dest_x / 2) + dest_y / 2 * ulDestScreenWidth;
            break;

        /* YUV422 Interleave */
        case IMG_FORM_YUV422_INTERLEAVE:
        /* RGB565/666/888 */
        case IMG_FORM_RGB888:
        case IMG_FORM_RGB666:
        case IMG_FORM_RGB565:
            ulDestStarAddrU     = 0L;
            ulDestStarAddrV     = 0L;
            break;
    }

    return TRUE;
}


#if (_DEBUG_CONVERT & 0x1)
/*****************************************************************************
* MODULE   : print_log1
* FUNCTION : show debug log
* IMPORT   : none
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/11/09 M.Sasaki
* UPDATE   :
******************************************************************************/
static void print_log1(_IMGIO_CONVERT *p)
{
    dbg_printk((_DEBUG_CONVERT & 0x1), "***********************\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "****** tagIMGIO_CONVERT\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "*********************** IMG_MODE info\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .srcfmt_f  = %x\n",      p->img_mode.srcfmt_f);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .srcfmt_b  = %x\n",      p->img_mode.srcfmt_b);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .dstfmt    = %x\n",      p->img_mode.dstfmt);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .dither_f  = %x\n",      p->img_mode.dither_f);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .dither_b  = %x\n",      p->img_mode.dither_b);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .rfilter_f = %x\n",      p->img_mode.rfilter_f);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .rfilter_b = %x\n",      p->img_mode.rfilter_b);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .op_mode   = 0x%03x\n",  p->img_mode.op_mode);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .udinv     = %x\n",      p->img_mode.udinv);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .rlinv     = %x\n",      p->img_mode.rlinv);
    dbg_printk((_DEBUG_CONVERT & 0x1), "img_mode   .msk_col   = %x\n",      p->img_mode.msk_col);
    dbg_printk((_DEBUG_CONVERT & 0x1), "*********************** src image info(BACK)\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.size      = %3d\n",     p->src_b_image.size);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.yrgbadr   = 0x%08lx\n", p->src_b_image.yrgbadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.uvadr     = 0x%08lx\n", p->src_b_image.uvadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.vadr      = 0x%08lx\n", p->src_b_image.vadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.x         = %3d\n",     p->src_b_image.x);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.y         = %3d\n",     p->src_b_image.y);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.hsize     = %3d\n",     p->src_b_image.hsize);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_b_image.vsize     = %3d\n",     p->src_b_image.vsize);
    dbg_printk((_DEBUG_CONVERT & 0x1), "*********************** src image info(FRONT)\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.size      = %3d\n",     p->src_f_image.size);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.yrgbadr   = 0x%08lx\n", p->src_f_image.yrgbadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.uvadr     = 0x%08lx\n", p->src_f_image.uvadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.vadr      = 0x%08lx\n", p->src_f_image.vadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.x         = %3d\n",     p->src_f_image.x);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.y         = %3d\n",     p->src_f_image.y);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.hsize     = %3d\n",     p->src_f_image.hsize);
    dbg_printk((_DEBUG_CONVERT & 0x1), "src_f_image.vsize     = %3d\n",     p->src_f_image.vsize);
    dbg_printk((_DEBUG_CONVERT & 0x1), "*********************** dest image info\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .size      = %3d\n",     p->dest_image.size);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .yrgbadr   = 0x%08lx\n", p->dest_image.yrgbadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .uvadr     = 0x%08lx\n", p->dest_image.uvadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .vadr      = 0x%08lx\n", p->dest_image.vadr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .x         = %3d\n",     p->dest_image.x);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .y         = %3d\n",     p->dest_image.y);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .hsize     = %3d\n",     p->dest_image.hsize);
    dbg_printk((_DEBUG_CONVERT & 0x1), "dest_image .vsize     = %3d\n",     p->dest_image.vsize);
    dbg_printk((_DEBUG_CONVERT & 0x1), "*********************** etc info\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "offsetx               = %3d\n",     p->offsetx);
    dbg_printk((_DEBUG_CONVERT & 0x1), "offsety               = %3d\n",     p->offsety);
    dbg_printk((_DEBUG_CONVERT & 0x1), "maskcolr              = 0x%04x\n",  p->maskcolr);
    dbg_printk((_DEBUG_CONVERT & 0x1), "alpha                 = 0x%02x\n",  p->alpha);
    dbg_printk((_DEBUG_CONVERT & 0x1), "in_byte               = %lx\n",     p->in_byte);
    dbg_printk((_DEBUG_CONVERT & 0x1), "out_byte              = %lx\n",     p->out_byte);
    dbg_printk((_DEBUG_CONVERT & 0x1), "***********************\n");
    dbg_printk((_DEBUG_CONVERT & 0x1), "\n");
}
#endif /* _DEBUG_CONVERT */


#if (_DEBUG_CONVERT & 0x2)
/*****************************************************************************
* MODULE   : print_log2
* FUNCTION : show debug log
* IMPORT   :
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/11/19 M.Sasaki
* UPDATE   : 2008/03/14 M.Sasaki
******************************************************************************/
static void print_log2(void)
{
    int index_req = 0;

    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), "=== input data =======================================================\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_MODE         > 0x%08lx\n", img_mode);                index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_FORMAT       > 0x%08lx\n", format);                  index_req++;
    switch(src_format_b)
    {
        case IMG_FORM_YUV422_INTERLEAVE:   dbg_printk((_DEBUG_CONVERT & 0x2), "  Back  > YUV422_Interleave ");  break;
        case IMG_FORM_YUV422_SEMI_PLANAR:  dbg_printk((_DEBUG_CONVERT & 0x2), "  Back  > YUV422_Semi-Planar "); break;
        case IMG_FORM_YUV420_SEMI_PLANAR:  dbg_printk((_DEBUG_CONVERT & 0x2), "  Back  > YUV420_Semi-Planar "); break;
        case IMG_FORM_RGB888:              dbg_printk((_DEBUG_CONVERT & 0x2), "  Back  > RGB888 ");             break;
        case IMG_FORM_RGB565:              dbg_printk((_DEBUG_CONVERT & 0x2), "  Back  > RGB565 ");             break;
    }

    if(img_resize == RESIZE_ON_BACK) dbg_printk((_DEBUG_CONVERT & 0x2), ": ZOOM ");
    if(img_mode & IMG_DITHER_B_BIT)  dbg_printk((_DEBUG_CONVERT & 0x2), ": dither ");
    if(img_mode & IMG_RFILTER_B_BIT) dbg_printk((_DEBUG_CONVERT & 0x2), ": smooth filter ");
    else                             dbg_printk((_DEBUG_CONVERT & 0x2), ": radical filter ");
    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");

    if(mix_front_image)
    {
        switch(src_format_f)
        {
            case IMG_FORM_YUV422_INTERLEAVE:  dbg_printk((_DEBUG_CONVERT & 0x2), "  Front > YUV422_Interleave ");  break;
            case IMG_FORM_YUV422_SEMI_PLANAR: dbg_printk((_DEBUG_CONVERT & 0x2), "  Front > YUV422_Semi-Planar "); break;
            case IMG_FORM_YUV420_SEMI_PLANAR: dbg_printk((_DEBUG_CONVERT & 0x2), "  Front > YUV420_Semi-Planar "); break;
            case IMG_FORM_RGB888:             dbg_printk((_DEBUG_CONVERT & 0x2), "  Front > RGB888 ");             break;
            case IMG_FORM_RGB565:             dbg_printk((_DEBUG_CONVERT & 0x2), "  Front > RGB565 ");             break;
        }

        if(img_resize == RESIZE_ON_FRONT) dbg_printk((_DEBUG_CONVERT & 0x2), ": ZOOM ");
        if(img_mode & IMG_DITHER_F_BIT)   dbg_printk((_DEBUG_CONVERT & 0x2), ": dither ");
        if(img_mode & IMG_RFILTER_F_BIT)  dbg_printk((_DEBUG_CONVERT & 0x2), ": smooth filter ");
        else                              dbg_printk((_DEBUG_CONVERT & 0x2), ": radical filter ");
        if(img_maskcolor)                 dbg_printk((_DEBUG_CONVERT & 0x2), ": mask color ");
        dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
    }


    switch(dst_format)
    {
        case IMG_FORM_YUV422_SEMI_PLANAR: dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > YUV422_Semi-Planar "); break;
        case IMG_FORM_YUV422_INTERLEAVE:  dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > YUV422_Interleave ");  break;
        case IMG_FORM_YUV420_SEMI_PLANAR: dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > YUV420_Semi-Planar "); break;
        case IMG_FORM_YUV420_PLANAR:      dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > YUV420_Planar ");      break;
        case IMG_FORM_RGB888:             dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > RGB888 ");             break;
        case IMG_FORM_RGB666:             dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > RGB666 ");             break;
        case IMG_FORM_RGB565:             dbg_printk((_DEBUG_CONVERT & 0x2), "  Dest  > RGB565 ");             break;
    }
    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");

    dbg_printk((_DEBUG_CONVERT & 0x2), "    Back  > x = %3ld  y = %3ld  w = %3ld   h = %3ld  p = %3ld\n", src_x, src_y, src_width, src_height, ulSrcScreenWidth);
    if(mix_front_image) dbg_printk((_DEBUG_CONVERT & 0x2), "    Front > x = %3ld  y = %3ld  w = %3ld   h = %3ld  p = %3ld\n", src_x_F, src_y_F, src_width_F, src_height_F, ulSrcScreenWidth_F);
    dbg_printk((_DEBUG_CONVERT & 0x2), "    Dest  > x = %3ld  y = %3ld  w = %3ld   h = %3ld  p = %3ld\n", dest_x, dest_y, dest_width, dest_height, ulDestScreenWidth);

    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCYRGBADR_B > 0x%08lx\n", ulSrcStarAddrYRGB);       index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCUVADR_B   > 0x%08lx\n", ulSrcStarAddrU);          index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCVADR_B    > 0x%08lx\n", ulSrcStarAddrV_B);        index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCSIZE_B    > %ld\n",     ulSrcScreenWidth);        index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCHSIZE_B   > %ld\n",     src_width);               index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCVSIZE_B   > %ld\n",     src_height);              index_req++;

    if(mix_front_image){
        dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCYRGBADR_F > 0x%08lx\n", ulSrcStarAddrYRGB_F); index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCUVADR_F   > 0x%08lx\n", ulSrcStarAddrU_F);    index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCVADR_F    > 0x%08lx\n", ulSrcStarAddrV_F);    index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCSIZE_F    > %ld\n",     ulSrcScreenWidth_F);  index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCHSIZE_F   > %ld\n",     src_width_F);         index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_SRCVSIZE_F   > %ld\n",     src_height_F);        index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_OFFSETX      > +%ld\n",  offset_x);              index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_OFFSETY      > +%ld\n",  offset_y);              index_req++;

        if(img_maskcolor){
            dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
            dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_MASKCOLOR    > 0x%04x\n", mask_color);       index_req++;
            dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_ALPHA        > 0x%04x\n", mask_alpha);       index_req++;
        }
    }

    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_DSTYRGBADR   > 0x%08lx\n", ulDestStarAddrYRGB);      index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_DSTUVADR     > 0x%08lx\n", ulDestStarAddrU);         index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_DSTVADR      > 0x%08lx\n", ulDestStarAddrV);         index_req++;
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_DSTSIZE      > %ld\n",     ulDestScreenWidth);       index_req++;

    if(img_resize){
        dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_DSTHSIZE     > %ld\n", dest_width);              index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_DSTVSIZE     > %ld\n", dest_height);             index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_HSTEP        > %ld\n", step_h);                  index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_VSTEP        > %ld\n", step_v);                  index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_HFOLD        > %ld\n", fold_h);                  index_req++;
        dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_VFOLD        > %ld\n", fold_v);                  index_req++;
    }

    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_INDATABYTE   > 0x%08lx\n", in_byte);
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_OUTDATABYTE  > 0x%08lx\n", out_byte);
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_PEL_ENDIAN   > 0x%08lx\n", endian);
    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), " IMG_REQ          > 0x%08x\n", IMG_REQ_BIT);
    dbg_printk((_DEBUG_CONVERT & 0x2), "======================================================================\n");
    dbg_printk((_DEBUG_CONVERT & 0x2), "\n");
}
#endif /* _DEBUG_CONVERT */

#endif /* _MP200_IMG_CONVERT_H_ */
