/* 2010-08-25: File added and changed by Sony Corporation */
/*
*  File Name       : /drivers/ipu/mp200_ipu.c
*  Function        : IPU Driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                           */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                  */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                  */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/


#define _DEBUG_IPU       0x00 /* 80218421(bit) */
                              /* 0x01: debug function in
                               * 0x02: debug function out
                               * 0x04: debug reschedule
                               * 0x08: debug mp200_ipu_request()
                               * 0x10: debug queue
                               * 0x20: debug IMG/ROT/GDMA register
                               * 0x80: debug DUAL_FF mode
                               */

#if (_DEBUG_IPU)
#define dbg_printk(level, fmt, arg...) \
        if(level > 0) printk(KERN_DEBUG "mp200_ipu: " fmt, ## arg);
#else
#define dbg_printk(level, fmt, arg...) \
	;
#endif


#include "mp200_ipu.h"

#define DRV_IPU DRV_IPU_IMG


/****************************************************************************/
/* Flags                                                                    */
/****************************************************************************/
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
static int DPM_suspend_flg = 0;
#endif /* CONFIG_PM || CONFIG_DPM */
static int img_empty_flg = 1;
static int rot_empty_flg = 1;
static int gdma_empty_flg = 1;

#define TEMPBUF_LOG_PADDR  0x37F1C300
#define TEMPBUF_LOG_LENGTH 0xE3D00

/* IMG MMIO */
static ulong IMGMmio        = IMAGEPROC_BASE;    /* paddr */
#if (_DEBUG_IPU)
static ulong IMGMmioLength  = IMAGEPROC_SIZE;
#endif
static char *IMGMmioV;                           /* vaddr */

/* ROT MMIO */
static ulong ROTMmio        = ROTATOR_BASE;      /* paddr */
#if (_DEBUG_IPU)
static ulong ROTMmioLength  = ROTATOR_SIZE;
#endif
static char *ROTMmioV;                           /* vaddr */

/* GDMA MMIO */
static ulong GDMAMmio        = GDMA_BASE;        /* paddr */
#if (_DEBUG_IPU)
static ulong GDMAMmioLength  = GDMA_SIZE;
#endif
static char *GDMAMmioV;                          /* vaddr */

/* thread flag */
static int thread_img_flag  = 1;                 /* img  flag */
static int thread_rot_flag  = 1;                 /* rot  flag */
static int thread_gdma_flag = 1;                 /* gdma flag */

static unsigned int dev_major=194;
static char *dev_name = "mp200_ipu";             /* dev_name            */


MP200_IPU_DEV        *mp200_ipu_devices;         /* allocated in mp200_ipu_init   */
MP200_IPU_REQ_REGIST *mp200_ipu_req_bufs;        /* allocated in mp200_ipu_init   */
MP200_IPU_QUEUE      *mp200_ipu_que;             /* allocated in mp200_ipu_init   */

int mp200_ipu_devs = MP200_IPU_DEVS;             /* number of bare IPU devices    */


/****************************************************************************/
/* Spinlock                                                                 */
/****************************************************************************/
static spinlock_t ipu_lock;


/****************************************************************************/
/* IPU request informations                                                 */
/****************************************************************************/
#ifdef CONFIG_MP200_IPU_DUAL
#define INDEX_MAX 2

/* Informations of first IPU request */
struct list_head  index_img;
struct list_head  index_rot;
struct list_head  index_gdma;

/* Informations of execution IPU request */
#define REQ_FREE 0
#define REQ_USED 1

struct mp200_ipu_task{
    struct list_head             req_list;
    struct mp200_ipu_req_regist *request;                  /* request running flag */
    int                          status;
};

/* Informations of last IPU request */
static struct mp200_ipu_task request_img[INDEX_MAX];
static struct mp200_ipu_task request_rot[INDEX_MAX];
static struct mp200_ipu_task request_gdma[INDEX_MAX];
#else
/* Informations of last IPU request */
static struct mp200_ipu_req_regist *request_img  = NULL;   /* request running img  flag*/
static struct mp200_ipu_req_regist *request_rot  = NULL;   /* request running rot  flag*/
static struct mp200_ipu_req_regist *request_gdma = NULL;   /* request running gdma flag*/
#endif

/* Informations of IPU request which suspended with left callback */
struct mp200_ipu_req_leftcall {
    ipu_callback_func   callback;
    unsigned long       sequence;
    unsigned long       data;
    int                 ipu_hw_func;
    int                 flag;
};
static struct mp200_ipu_req_leftcall IPU_REQ_leftcall;


#if _DEBUG_IPU_LOG
static ulong  IPUSmem       = TEMPBUF_LOG_PADDR;           /* paddr */
static ulong  IPUSmemLength = TEMPBUF_LOG_LENGTH;
static char  *IPUSmemV;                                    /* vaddr */
static ulong *last_regist;
static ulong *last_remove;
static ulong  num_regist;
static ulong  num_remove;
#endif


/****************************************************************************/
/* Prototype declarations                                                   */
/****************************************************************************/
/* ------------------------------------------- */
/* external function prottype                  */
/* ------------------------------------------- */
extern int  mp200_ipu_open( struct inode *inode, struct file *file );
extern int  mp200_ipu_close( struct inode *inode, struct file *file );
extern int  mp200_ipu_ioctl( struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg );
extern int  mp200_ipu_mmap( struct file *file, struct vm_area_struct * vma );
extern void mp200_ipu_callback_img(unsigned long data, int flag);
extern void mp200_ipu_callback_rot(unsigned long data, int flag);
extern void mp200_ipu_callback_gdma(unsigned long data, int flag);


/* ------------------------------------------- */
/* file_operations                             */
/* ------------------------------------------- */
static struct file_operations mp200_ipu_fops = {
    .owner   = THIS_MODULE,
    .ioctl   = mp200_ipu_ioctl,
    .mmap    = mp200_ipu_mmap,
    .open    = mp200_ipu_open,
    .release = mp200_ipu_close,
};


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,00)           // Linux 2.6.00 later
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)           // Linux 2.6.21 later
static struct class        *mp200_ipu_class;
#else
static struct class_simple *mp200_ipu_class;
#endif
static struct class_device *mp200_ipu_class_dev;
              dev_t         t_dev;

/* cdev fops */
static struct cdev mp200_ipu_cdev = {
    .owner =  THIS_MODULE,
    .ops   = &mp200_ipu_fops,
};
#else // Linux 2.6.00 before
#endif


/* ------------------------------------------- */
/*   Public functions                          */
/* ------------------------------------------- */
int mp200_ipu_regist( struct mp200_ipu_req *p );
int mp200_ipu_cancel( unsigned long sequence, int flag );


/* ------------------------------------------- */
/*   Initialize function                       */
/* ------------------------------------------- */
static int  mp200_ipu_init( void );


/* ------------------------------------------- */
/*   Private functions                         */
/* ------------------------------------------- */
/* initialize function */
static int                          mp200_ipu_hw_init( void );
static int                          mp200_ipu_reschedule_img( void * );
static int                          mp200_ipu_reschedule_rot( void * );
static int                          mp200_ipu_reschedule_gdma( void * );
static void                         mp200_ipu_hw_int_enable( int ipu_hw_func );
static void                         mp200_ipu_hw_int_disable( int ipu_hw_func );
#ifdef CONFIG_MP200_IPU_DUAL
static void                         mp200_ipu_hw_int_ffclr( int ipu_hw_func );
#endif
/* Queue operation function */
static int                          mp200_ipu_init_queue( void );
static int                          mp200_ipu_regist_queue( struct mp200_ipu_queue **pQueue, struct mp200_ipu_req *p );
static void                         mp200_ipu_remove_queue( unsigned long sequence, int ipu_hw_func );
static void                         mp200_ipu_release_buf( struct mp200_ipu_req_regist *p );
static struct mp200_ipu_req_regist *mp200_ipu_search_queue( unsigned long sequence, int ipu_hw_func );
static struct mp200_ipu_req_regist *mp200_ipu_search_queue_func( unsigned long sequence, struct list_head *list );
#ifdef CONFIG_MP200_IPU_DUAL
/* Queue operation function for "DUAL_FF" */
static void                         mp200_ipu_reqqueue_addtail(struct list_head *index, struct mp200_ipu_task *request, struct mp200_ipu_req_regist *new_req);
static void                         mp200_ipu_reqqueue_del(struct mp200_ipu_task *request);
static void                         mp200_ipu_reqqueue_del_next_sequence(struct mp200_ipu_task *request, unsigned long sequence);
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
static int                          mp200_ipu_reqqueue_search_empty(struct list_head *index);
#endif /* CONFIG_PM || CONFIG_DPM */
static int                          mp200_ipu_reqqueue_search_index(struct mp200_ipu_task *request);
static int                          mp200_ipu_reqqueue_search_sequence(struct mp200_ipu_task *request, unsigned long sequence);
#endif
/* request function */
static int                          mp200_ipu_request( struct mp200_ipu_req_regist *p );
static void                         mp200_ipu_request_sub( struct mp200_ipu_req_regist *p, int callback_flag );
static void                         mp200_ipu_request_remove_sub( struct mp200_ipu_req_regist *p );
static void                         mp200_ipu_request_release_sub( int ipu_hw_func );
static void                         mp200_ipu_request_wakeup( int ipu_hw_func );

static int                          mp200_ipu_req_finalize_img( struct mp200_ipu_req_regist *p );
static int                          mp200_ipu_req_finalize_rot( struct mp200_ipu_req_regist *p );
static int                          mp200_ipu_req_finalize_gdma( struct mp200_ipu_req_regist *p );
static void                         mp200_ipu_req_finalize( struct mp200_ipu_req_regist *p, int *ret );


#if defined(CONFIG_PM) || defined(CONFIG_DPM)
/* ------------------------------------------- */
/*   DPM functions                             */
/* ------------------------------------------- */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)           // kernel 2.6.21 later
static int  mp200_ipu_suspend(struct platform_device *dev, pm_message_t state);
static int  mp200_ipu_suspend_late(struct platform_device *dev, pm_message_t state);
static int  mp200_ipu_resume_early(struct platform_device *dev);
static int  mp200_ipu_resume(struct platform_device *dev);
#else // kernel 2.6.21 before
static int  mp200_ipu_suspend(struct device *dev, u32 state, u32 level);
static int  mp200_ipu_resume(struct device *dev, u32 level);
#endif

#ifdef CONFIG_MP200_IPU_DUAL
static int mp200_ipu_suspend_set_hwreq_flg(struct list_head *list_hw);
static void mp200_ipu_resume_kick_next(struct list_head *list_hw, int irq);
#else
static int mp200_ipu_suspend_set_hwreq_flg(struct mp200_ipu_req_regist *request_hw);
static void mp200_ipu_resume_kick_next(struct mp200_ipu_req_regist *request_hw, int irq);
#endif /* CONFIG_MP200_IPU_DUAL */
static void mp200_ipu_resume_kick(unsigned long num);
static void mp200_ipu_resume_kick_finalize(void *task, int irq);
static void mp200_ipu_resume_kick_wakeup(int hw_func);


static struct platform_driver mp200_ipu_driver = {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)           // kernel 2.6.21 later
        .driver.name  = "mp200_image",
        .driver.bus   = &platform_bus_type,
        .suspend      = mp200_ipu_suspend,
        .suspend_late = mp200_ipu_suspend_late,
        .resume_early = mp200_ipu_resume_early,
        .resume       = mp200_ipu_resume,
#else // kernel 2.6.21 before
        .name         = "mp200_image",
        .bus          = &platform_bus_type,
        .suspend      = mp200_ipu_suspend,
        .resume       = mp200_ipu_resume,
#endif
};


static struct platform_device mp200_ipu_device = {
        .name        = "MP200 IPU",
        .dev.release = NULL,
        .id          = 0,
};


/* Tasklet */
static DECLARE_TASKLET(ipu_nexttask, mp200_ipu_resume_kick, 0);



/* Macros */
static int ctrl_func = 0;
#define Call_StatusCtrlFunc_ON() \
        do{                      \
            ctrl_func = 1;       \
        }while(0)
#ifdef CONFIG_MP200_IPU_DUAL
#define Call_StatusCtrlFunc_OFF()                             \
        do{                                                   \
            if(mp200_ipu_reqqueue_search_empty(&index_img) && \
               mp200_ipu_reqqueue_search_empty(&index_rot) && \
               mp200_ipu_reqqueue_search_empty(&index_gdma))  \
            {                                                 \
                ctrl_func = 0;                                \
            }                                                 \
        }while(0)
#else  /* CONFIG_MP200_IPU_DUAL */
#define Call_StatusCtrlFunc_OFF()                             \
        do{                                                   \
            if(!request_img && !request_rot && !request_gdma) \
            {                                                 \
                ctrl_func = 0;                                \
            }                                                 \
        }while(0)
#endif /* CONFIG_MP200_IPU_DUAL */
#else
#define Call_StatusCtrlFunc_ON()
#define Call_StatusCtrlFunc_OFF()
#endif /* CONFIG_PM || CONFIG_DPM */


/* ------------------------------------------- */
/*   Interrupt Management functions            */
/* ------------------------------------------- */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)           // Kernel 2.6.21 later
static irqreturn_t mp200_ipu_irq_handler( int irq, void *dev_id );                      /* IPU interrupt handler */
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,00)         // Kernel 2.6.00 later
static irqreturn_t mp200_ipu_irq_handler( int irq, void *dev_id, struct pt_regs *fp );  /* IPU interrupt handler */
#else // Kernel 2.6.00 before
static void mp200_ipu_irq_handler( int irq, void *dev_id, struct pt_regs *fp );         /* IPU interrupt handler */
#endif
static void mp200_ipu_irq_handler_sub( int irq, void *dev_id );


/*****************************************************************************
* MODULE   : mp200_ipu_regist
* FUNCTION : register request in IPU Request's queue by other drivers.
* RETURN   : IPU_REQ_REGISTERED(0) : the request is registered
*          : IPU_REQ_BUSY(-1)      : fail to registered request busy
*          : IPU_REQ_PARAMERR(-2)  : fail to registered request Parameter error
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
int mp200_ipu_regist(struct mp200_ipu_req *p)
{
    struct mp200_ipu_dev *dev = mp200_ipu_devices;
    struct mp200_ipu_queue *pQueue = mp200_ipu_que;
    int ret;
    unsigned long flags;

    dbg_GetStartTime(2);
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_regist() <start>\n");

    if (p == NULL){
        printk(KERN_INFO " @ipu: mp200_ipu_regist() Error! p(%p) is incorrect.\n", p);
        return IPU_REQ_PARAMERR;
    }
    if ((p->ipu_hw_func != IPU_HW_FUNC_IMG) &&
        (p->ipu_hw_func != IPU_HW_FUNC_ROT) &&
        (p->ipu_hw_func != IPU_HW_FUNC_GDMA))
    {
        printk(KERN_INFO " @ipu: mp200_ipu_regist() Error! p->ipu_hw_func(%d) is incorrect.\n", p->ipu_hw_func);
        return IPU_REQ_PARAMERR;
    }

    spin_lock_irqsave(&ipu_lock, flags);                   /* prohibit interrupts  */
    ret = mp200_ipu_regist_queue(&pQueue, p);              /* register request     */
    if (ret == IPU_REQ_REGISTERED)
    {
        switch (p->ipu_hw_func)                            /* kernel thread wakeup */
        {
            case IPU_HW_FUNC_IMG:
                img_empty_flg = 0;
                break;
            case IPU_HW_FUNC_ROT:
                rot_empty_flg = 0;
                break;
            case IPU_HW_FUNC_GDMA:
                gdma_empty_flg = 0;
                break;
        }
    }
    spin_unlock_irqrestore(&ipu_lock, flags);              /* permit interrupts    */
    if (ret != IPU_REQ_REGISTERED){
        printk(KERN_INFO " @ipu: mp200_ipu_regist() Error! mp200_ipu_regist_queue(%d) is incorrect.\n", ret);
        return ret;
    }

    switch (p->ipu_hw_func)                                /* kernel thread wakeup */
    {
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist() wake up kernel_thread <IMG>\n");
            wake_up_interruptible(&dev->wait_idle_img);
            break;
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist() wake up kernel_thread <ROT>\n");
            wake_up_interruptible(&dev->wait_idle_rot);
            break;
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist() wake up kernel_thread <GDMA>\n");
            wake_up_interruptible(&dev->wait_idle_gdma);
            break;
    }
    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist() <end>\n");
    return ret;
}


/*****************************************************************************
* MODULE   : mp200_ipu_cancel
* FUNCTION : cancel registered IPU request
*            target IPU request has not been started :
*                remove the target request from the queue.
*            target IPU request has been started (running) :
*              without IPU_CANCEL_SYNC :
*                set IPU_CANCEL_WAIT to the status of the request,
*                and wait for the request to be done.
*                Then call the callback function with IPU_CALLBACK_CANCEL flag.
*              with IPU_CANCEL_SYNC    :
*                remove the target request from the queue immediately.
*                and clear callback_flag (don't call the callback function).
* RETURN   :  IPU_CANCEL_DONE(0)   : cancelled
*             IPU_CANCEL_WAIT(1)   : waiting for cancelled request to be done
*             IPU_CANCEL_NO_REQ(2) : target request does not exist
* NOTE     : 1) Pay attention to the exclusive control between mp200_ipu_cancel and mp200_ipu_irq_handler.
*               ex.)prohibit interrupts while updating the status.
*            2) if serious delay problem occurs with Preferred Request, It should be required
*               to add new function which resets IPU through SMU.
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
int mp200_ipu_cancel( unsigned long sequence, int flag )
{
    struct mp200_ipu_req_regist *p_tmp;
    unsigned long flags;
    int ret;

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_cancel() <start>\n");

    if (!sequence) return IPU_CANCEL_NO_REQ;

    /* prohibit interrupts */
    spin_lock_irqsave(&ipu_lock, flags);                        /* disable interrupts */

    p_tmp = mp200_ipu_search_queue(sequence,IPU_HW_FUNC_ALL);   /* target request all search */
    if (p_tmp != NULL)
    {
        if (p_tmp->ipu_req.status == IPU_REQ_STATUS_RUNNING)
        /* target request is running */
        {
            if (flag == IPU_CANCEL_SYNC)
            {
                p_tmp->ipu_req.status = IPU_REQ_STATUS_CANCEL;  /* cansel(remove)start */
                mp200_ipu_remove_queue(sequence,p_tmp->ipu_req.ipu_hw_func);
                p_tmp->ipu_req.status = IPU_REQ_STATUS_CANCELLED;

                p_tmp->callback_flag = FALSE;
                ret = IPU_CANCEL_DONE;                          /* cancelled */
            }
            else
            {
                p_tmp->ipu_req.status = IPU_REQ_STATUS_WAIT_CANCEL;
                ret = IPU_CANCEL_WAIT;                          /* waiting for cancelled request to be done */
            }
        }
        else
        /* target request is not running */
        {
            p_tmp->ipu_req.status = IPU_REQ_STATUS_CANCEL;      /* cansel(remove)start */
            mp200_ipu_remove_queue(sequence, p_tmp->ipu_req.ipu_hw_func);
            p_tmp->ipu_req.status = IPU_REQ_STATUS_CANCELLED;
            mp200_ipu_release_buf(p_tmp);
            ret = IPU_CANCEL_DONE;                              /* cancelled */
        }
    }
    else
    {
        ret = IPU_CANCEL_NO_REQ;                                /* target request does not exist */
    }

    /* permit interrupts */
    spin_unlock_irqrestore(&ipu_lock, flags);                   /* enable interrupts */

    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_cancel() <end>\n");
    return ret;
}


/* ------------------------------------------- */
/*   kernel-thread main function               */
/* ------------------------------------------- */
#if _IPU_RT_THREAD /* RT thread */
#define IPU_THREAD_PRIORITY 1
#else /* Normal thread */
#define IPU_THREAD_PRIORITY 99
#endif
/*****************************************************************************
* MODULE   : mp200_ipu_reschedule_img
* FUNCTION : start to run request in IMG Request's queue.(IMG thread)
* RETURN   : 0
* NOTE     : none
* CREATE   : 2006/10/04 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_reschedule_img( void *dev_p)
{
    static int isfirst_img = 1;         /* flag for request_irq(INT_IMG) */
#if _IPU_RT_THREAD /* RT thread */
    struct sched_param param = { .sched_priority = IPU_THREAD_PRIORITY };
#endif
    struct mp200_ipu_dev *dev = dev_p;
    struct mp200_ipu_queue *pQueue = mp200_ipu_que;
    struct mp200_ipu_req_regist *p_tmp = NULL;
    struct list_head *list_queue;
    unsigned long flags;
    int ret;
#ifdef CONFIG_MP200_IPU_DUAL
    int req_index = -1;
    int find_list = 0;
#endif

    strcpy(current->comm, "kipu_img");
#if _IPU_RT_THREAD /* RT thread */
    sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
    current->static_prio = IPU_THREAD_PRIORITY;
#endif

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reschedule_img() <start>\n");

    do {
        if (!thread_img_flag) break;

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
        try_to_freeze();
#endif /* CONFIG_PM || CONFIG_DPM */

#ifdef CONFIG_MP200_IPU_DUAL
        req_index = mp200_ipu_reqqueue_search_index(request_img);
        if (req_index == -1)    /* IMG task is running full */
#else
        if (request_img)        /* IMG is running */
#endif
        {
            /* waiting (executing operaion) */
            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() <IMG running:sleep>\n");
            dbg_SetSleepStatus(SLEEP_2);
#ifdef CONFIG_MP200_IPU_DUAL
            wait_event_interruptible(dev->wait_busy_img, (mp200_ipu_reqqueue_search_index(request_img) != -1));
#else
            wait_event_interruptible(dev->wait_busy_img, (request_img == NULL));
#endif
            dbg_ClrSleepStatus(SLEEP_2);
            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() <IMG running:wake up>\n");
        }
        else
        {
            /* prohibit interrupts */
            spin_lock_irqsave(&ipu_lock, flags);                           /* disable interrupts */

#ifdef CONFIG_MP200_IPU_DUAL
            find_list = 0;
#endif
            if(!list_empty(&pQueue->ipu_queue_img))                        /* not exist:1  exist:0 */
            {
            /* Requests exist */
                list_for_each(list_queue, &pQueue->ipu_queue_img)
                {
                    p_tmp = list_entry(list_queue, struct mp200_ipu_req_regist, ipu_req_regist);
                    if(p_tmp->ipu_req.status == IPU_REQ_STATUS_REGISTERED)
                    {
                        Call_StatusCtrlFunc_ON();
#ifdef CONFIG_MP200_IPU_DUAL
                        find_list = 1;
                        break;
                    }
                }
                if(find_list){ dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() QueueSearch success.\n");
                }else{         dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() QueueSearch failed.\n");
                }
#else
                        request_img = p_tmp;
                        p_tmp->callback_flag  = TRUE;
                        p_tmp->ipu_req.status = IPU_REQ_STATUS_RUNNING;    /* update status */
                        break;
                    }
                }
                if(request_img){ dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() QueueSearch success.\n");
                }else{           dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() QueueSearch failed.\n");
#if _DEBUG_IPU
                    {
                        int i;
                        printk("\n");
                        for(i = 0; i < IPU_BUFS_MAX; i++){
                            if(!pQueue->bufs[i]) continue;
                            dbg_printk(_DEBUG_IPU, "reschedule_img: %2d(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%ld)\n",
                                    i, &pQueue->bufs[i]->ipu_req_regist, pQueue->bufs[i]->ipu_req_regist.prev, pQueue->bufs[i]->ipu_req_regist.next,
                                    pQueue->bufs[i]->ipu_req.status, pQueue->bufs[i]->ipu_req.sequence);
                        }
                        printk("\n");
                    }
#endif
                }
#endif

#ifdef CONFIG_MP200_IPU_DUAL
                if(find_list)
                {
                    dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_reschedule_img() ->hw_func(0x%02x) ->sequence(%ld)\n", p_tmp->ipu_req.ipu_hw_func, p_tmp->ipu_req.sequence);
                    /* permit interrupts */
                    spin_unlock_irqrestore(&ipu_lock, flags);              /* enable interrupts */

                    spin_lock_irqsave(&ipu_lock, flags);                   /* disable interrupts */
                    p_tmp->callback_flag  = TRUE;
                    p_tmp->ipu_req.status = IPU_REQ_STATUS_RUNNING;        /* update status */
                    mp200_ipu_reqqueue_addtail(&index_img, &request_img[req_index], p_tmp);
                    spin_unlock_irqrestore(&ipu_lock, flags);              /* enable interrupts */

                    if(isfirst_img)
                    {
                        mp200_ipu_hw_int_disable(IPU_HW_FUNC_IMG);
                        ret = request_irq(INT_IMG, mp200_ipu_irq_handler, 0, dev_name, NULL);
                        if (ret < 0)
                        {
                             printk(KERN_ERR " @ipu: fail in request_irq(%d)\n",INT_IMG);
                        }
                        dbg_printk((_DEBUG_IPU & 0x08), "request_irq <IMG>\n");
                        isfirst_img = 0;
                    }
dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() ipu_request in\n");
                    ret = mp200_ipu_request(p_tmp);                        /* set IPU registers & run IPU */
dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() ipu_request out\n");
                    if(ret == -2){
                        spin_lock_irqsave(&ipu_lock, flags);               /* disable interrupts */
                        mp200_ipu_reqqueue_del(&request_img[req_index]);
                        p_tmp->ipu_req.status = IPU_REQ_STATUS_REGISTERED; /* STATUS_RUNNING -> STATUS_REGISTERED */
                        spin_unlock_irqrestore(&ipu_lock, flags);          /* enable interrupts */
                    }
                    schedule();
                }
                else
                {
                    goto img_empty;
                }
#else
                /* permit interrupts */
                spin_unlock_irqrestore(&ipu_lock, flags);                  /* enable interrupts */

                if (request_img)
                {
                    if(isfirst_img)
                    {
                        mp200_ipu_hw_int_disable(IPU_HW_FUNC_IMG);
                        ret = request_irq(INT_IMG, mp200_ipu_irq_handler, 0, dev_name, NULL);
                        if (ret < 0)
                        {
                             printk(KERN_ERR " @ipu: fail in request_irq(%d)\n",INT_IMG);
                        }
                        dbg_printk((_DEBUG_IPU & 0x08), "request_irq <IMG>\n");
                        isfirst_img = 0;
                    }
                    mp200_ipu_request(p_tmp);                              /* set IPU registers & run IPU */
                }
#endif
            }
            else
            {
            /* Requests not exist */
#ifdef CONFIG_MP200_IPU_DUAL
img_empty:
#endif
                img_empty_flg = 1;

                /* permit interrupts */
                spin_unlock_irqrestore(&ipu_lock, flags);                  /* enable interrupts */

                dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() <QueueEmpty:sleep>\n");
                dbg_SetSleepStatus(SLEEP_4);
                wait_event_interruptible(dev->wait_idle_img, (img_empty_flg == 0));
                dbg_ClrSleepStatus(SLEEP_4);
                dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_img() <QueueEmpty:wake up>\n");
            }
        }
    } while(1);

    dbg_printk((_DEBUG_IPU & 0x02), "reschedule_img() <END>\n");

    return 0;
}


/*****************************************************************************
* MODULE   : mp200_ipu_reschedule_rot
* FUNCTION : start to run request in ROT Request's queue.(ROT thread)
* RETURN   : 0
* NOTE     : none
* CREATE   : 2006/10/04 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_reschedule_rot( void *dev_p)
{
    static int isfirst_rot = 1;         /* flag for request_irq(INT_ROT) */
#if _IPU_RT_THREAD /* RT thread */
    struct sched_param param = { .sched_priority = IPU_THREAD_PRIORITY };
#endif
    struct mp200_ipu_dev *dev = dev_p;
    struct mp200_ipu_queue *pQueue = mp200_ipu_que;
    struct mp200_ipu_req_regist *p_tmp = NULL;
    struct list_head *list_queue;
    unsigned long flags;
    int ret;
#ifdef CONFIG_MP200_IPU_DUAL
    int req_index = -1;
    int find_list = 0;
#endif

    strcpy(current->comm, "kipu_rot");
#if _IPU_RT_THREAD /* RT thread */
    sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
    current->static_prio = IPU_THREAD_PRIORITY;
#endif

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reschedule_rot() <start>\n");

    do {
        if (!thread_rot_flag) break;

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
        try_to_freeze();
#endif /* CONFIG_PM || CONFIG_DPM */

#ifdef CONFIG_MP200_IPU_DUAL
        req_index = mp200_ipu_reqqueue_search_index(request_rot);
        if (req_index == -1)    /* ROT task is running full */
#else
        if (request_rot)        /* ROT is running */
#endif
        {
            /* waiting (executing operaion) */
            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() <ROT running:sleep>\n");
            dbg_SetSleepStatus(SLEEP_6);
#ifdef CONFIG_MP200_IPU_DUAL
            wait_event_interruptible(dev->wait_busy_rot, (mp200_ipu_reqqueue_search_index(request_rot) != -1));
#else
            wait_event_interruptible(dev->wait_busy_rot, (request_rot == NULL));
#endif
            dbg_ClrSleepStatus(SLEEP_6);
            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() <ROT running:wake up>\n");
        }
        else
        {
            /* prohibit interrupts */
            spin_lock_irqsave(&ipu_lock, flags);                           /* disable interrupts */

#ifdef CONFIG_MP200_IPU_DUAL
            find_list = 0;
#endif
            if(!list_empty(&pQueue->ipu_queue_rot))                        /* not exist:1  exist:0 */
            {
            /* Requests exist */
                list_for_each(list_queue, &pQueue->ipu_queue_rot)
                {
                    p_tmp = list_entry(list_queue, struct mp200_ipu_req_regist, ipu_req_regist);
                    if(p_tmp->ipu_req.status == IPU_REQ_STATUS_REGISTERED)
                    {
                        Call_StatusCtrlFunc_ON();
#ifdef CONFIG_MP200_IPU_DUAL
                        mp200_ipu_reqqueue_addtail(&index_rot, &request_rot[req_index], p_tmp);
                        find_list = 1;
#else
                        request_rot = p_tmp;
#endif
                        p_tmp->callback_flag  = TRUE;
                        p_tmp->ipu_req.status = IPU_REQ_STATUS_RUNNING;    /* update status */
                        break;
                    }
                }
#ifdef CONFIG_MP200_IPU_DUAL
                if(find_list){ dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() QueueSearch success.\n");
                }else{         dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() QueueSearch failed.\n");
#else
                if(request_rot){ dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() QueueSearch success.\n");
                }else{           dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() QueueSearch failed.\n");
#if _DEBUG_IPU
                    {
                        int i;
                        printk("\n");
                        for(i = 0; i < IPU_BUFS_MAX; i++){
                            if(!pQueue->bufs[i]) continue;
                            dbg_printk(_DEBUG_IPU, "reschedule_rot: %2d(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%ld)\n",
                                    i, &pQueue->bufs[i]->ipu_req_regist, pQueue->bufs[i]->ipu_req_regist.prev, pQueue->bufs[i]->ipu_req_regist.next,
                                    pQueue->bufs[i]->ipu_req.status, pQueue->bufs[i]->ipu_req.sequence);
                        }
                        printk("\n");
                    }
#endif
#endif
                }

#ifdef CONFIG_MP200_IPU_DUAL
                if(find_list)
                {
                    dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_reschedule_rot() ->hw_func(0x%02x) ->sequence(%ld)\n", p_tmp->ipu_req.ipu_hw_func, p_tmp->ipu_req.sequence);
                    /* permit interrupts */
                    spin_unlock_irqrestore(&ipu_lock, flags);              /* enable interrupts */

                    if(isfirst_rot)
                    {
                        mp200_ipu_hw_int_disable(IPU_HW_FUNC_ROT);
                        ret = request_irq(INT_ROT, mp200_ipu_irq_handler, 0, dev_name, NULL);
                        if (ret < 0)
                        {
                             printk(KERN_ERR " @ipu: fail in request_irq(%d)\n",INT_ROT);
                        }
                        dbg_printk((_DEBUG_IPU & 0x08), "request_irq <ROT>\n");
                        isfirst_rot = 0;
                    }
                    ret = mp200_ipu_request(p_tmp);                        /* set IPU registers & run IPU */
                    if(ret == -2){
                        spin_lock_irqsave(&ipu_lock, flags);               /* disable interrupts */
                        mp200_ipu_reqqueue_del(&request_rot[req_index]);
                        p_tmp->ipu_req.status = IPU_REQ_STATUS_REGISTERED;
                        spin_unlock_irqrestore(&ipu_lock, flags);          /* enable interrupts */
                    }
                    schedule();
                }
                else
                {
                    goto rot_empty;
                }
#else
                /* permit interrupts */
                spin_unlock_irqrestore(&ipu_lock, flags);                  /* enable interrupts */

                if (request_rot)
                {
                    if(isfirst_rot)
                    {
                        mp200_ipu_hw_int_disable(IPU_HW_FUNC_ROT);
                        ret = request_irq(INT_ROT, mp200_ipu_irq_handler, 0, dev_name, NULL);
                        if (ret < 0)
                        {
                             printk(KERN_ERR " @ipu: fail in request_irq(%d)\n",INT_ROT);
                        }
                        dbg_printk((_DEBUG_IPU & 0x08), "request_irq <ROT>\n");
                        isfirst_rot = 0;
                    }
                    mp200_ipu_request(p_tmp);                              /* set IPU registers & run IPU */
                }
#endif
            }
            else
            {
                /* Requests not exist */
#ifdef CONFIG_MP200_IPU_DUAL
rot_empty:
#endif
                rot_empty_flg = 1;

                /* permit interrupts */
                spin_unlock_irqrestore(&ipu_lock, flags);                  /* enable interrupts */

                dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() <QueueEmpty:sleep>\n");
                dbg_SetSleepStatus(SLEEP_8);
                wait_event_interruptible(dev->wait_idle_rot, (rot_empty_flg == 0));
                dbg_ClrSleepStatus(SLEEP_8);
                dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_rot() <QueueEmpty:wake up>\n");
            }
        }
    } while(1);

    dbg_printk((_DEBUG_IPU & 0x02), "reschedule_rot() <END>\n");

    return 0;
}


/*****************************************************************************
* MODULE   : mp200_ipu_reschedule_gdma
* FUNCTION : start to run request in GDMA Request's queue.(GDMA thread)
* RETURN   : 0
* NOTE     : none
* CREATE   : 2006/10/04 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_reschedule_gdma( void *dev_p)
{
    static int isfirst_gdma = 1;        /* flag for request_irq(INT_IPU_DMA) */
#if _IPU_RT_THREAD /* RT thread */
    struct sched_param param = { .sched_priority = IPU_THREAD_PRIORITY };
#endif
    struct mp200_ipu_dev *dev = dev_p;
    struct mp200_ipu_queue *pQueue = mp200_ipu_que;
    struct mp200_ipu_req_regist *p_tmp = NULL;
    struct list_head *list_queue;
    unsigned long flags;
    int ret;
#ifdef CONFIG_MP200_IPU_DUAL
    int req_index = -1;
    int find_list = 0;
#endif

    strcpy(current->comm, "kipu_gdma");
#if _IPU_RT_THREAD /* RT thread */
    sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
    current->static_prio = IPU_THREAD_PRIORITY;
#endif

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reschedule_gdma() <start>\n");

    do {
        if (!thread_gdma_flag) break;

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
        try_to_freeze();
#endif /* CONFIG_PM || CONFIG_DPM */

#ifdef CONFIG_MP200_IPU_DUAL
        req_index = mp200_ipu_reqqueue_search_index(request_gdma);
        if (req_index == -1)    /* GDMA task is running full */
#else
        if (request_gdma)       /* GDMA is running */
#endif
        {
            /* waiting (executing operaion) */
            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() <GDMA running:sleep>\n");
            dbg_SetSleepStatus(SLEEP_10);
#ifdef CONFIG_MP200_IPU_DUAL
            wait_event_interruptible(dev->wait_busy_gdma, (mp200_ipu_reqqueue_search_index(request_gdma) != -1));
#else
            wait_event_interruptible(dev->wait_busy_gdma, (request_gdma == NULL));
#endif
            dbg_ClrSleepStatus(SLEEP_10);
            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() <GDMA running:wake up>\n");
        }
        else
        {
            /* prohibit interrupts */
            spin_lock_irqsave(&ipu_lock, flags);                           /* disable interrupts */

#ifdef CONFIG_MP200_IPU_DUAL
            find_list = 0;
#endif
            if(!list_empty(&pQueue->ipu_queue_gdma))                       /* not exist:1  exist:0 */
            {
            /* Requests exist */
                list_for_each(list_queue, &pQueue->ipu_queue_gdma)
                {
                    p_tmp = list_entry(list_queue, struct mp200_ipu_req_regist, ipu_req_regist);
                    if(p_tmp->ipu_req.status == IPU_REQ_STATUS_REGISTERED)
                    {
                        Call_StatusCtrlFunc_ON();
#ifdef CONFIG_MP200_IPU_DUAL
                        mp200_ipu_reqqueue_addtail(&index_gdma, &request_gdma[req_index], p_tmp);
                        find_list = 1;
#else
                        request_gdma = p_tmp;
#endif
                        p_tmp->callback_flag  = TRUE;
                        p_tmp->ipu_req.status = IPU_REQ_STATUS_RUNNING;    /* update status */
                        break;
                    }
                }
#ifdef CONFIG_MP200_IPU_DUAL
                if(find_list){ dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() QueueSearch success.\n");
                }else{         dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() QueueSearch failed.\n");
#else
                if(request_gdma){ dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() QueueSearch success.\n");
                }else{            dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() QueueSearch failed.\n");
#if _DEBUG_IPU
                    {
                        int i;
                        printk("\n");
                        for(i = 0; i < IPU_BUFS_MAX; i++){
                            if(!pQueue->bufs[i]) continue;
                            dbg_printk(_DEBUG_IPU, "reschedule_gdma: %2d(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%ld)\n",
                                    i, &pQueue->bufs[i]->ipu_req_regist, pQueue->bufs[i]->ipu_req_regist.prev, pQueue->bufs[i]->ipu_req_regist.next,
                                    pQueue->bufs[i]->ipu_req.status, pQueue->bufs[i]->ipu_req.sequence);
                        }
                        printk("\n");
                    }
#endif
#endif
                }

#ifdef CONFIG_MP200_IPU_DUAL
                if(find_list)
                {
                    dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_reschedule_gdma() ->hw_func(0x%02x) ->sequence(%ld)\n", p_tmp->ipu_req.ipu_hw_func, p_tmp->ipu_req.sequence);
                    /* permit interrupts */
                    spin_unlock_irqrestore(&ipu_lock, flags);              /* enable interrupts */

                    if(isfirst_gdma)
                    {
                        mp200_ipu_hw_int_disable(IPU_HW_FUNC_GDMA);
                        ret = request_irq(INT_IPU_DMA, mp200_ipu_irq_handler, 0, dev_name, NULL);
                        if (ret < 0)
                        {
                             printk(KERN_ERR " @ipu: fail in request_irq(%d)\n",INT_IPU_DMA);
                        }
                        dbg_printk((_DEBUG_IPU & 0x08), "request_irq <GDMA>\n");
                        isfirst_gdma = 0;
                    }
                    ret = mp200_ipu_request(p_tmp);                        /* set IPU registers & run IPU */
                    if(ret == -2){
                        spin_lock_irqsave(&ipu_lock, flags);               /* disable interrupts */
                        mp200_ipu_reqqueue_del(&request_gdma[req_index]);
                        p_tmp->ipu_req.status = IPU_REQ_STATUS_REGISTERED;
                        spin_unlock_irqrestore(&ipu_lock, flags);          /* enable interrupts */
                    }
                    schedule();
                }
                else
                {
                    goto gdma_empty;
                }
#else
                /* permit interrupts */
                spin_unlock_irqrestore(&ipu_lock, flags);                  /* enable interrupts */

                if (request_gdma)
                {
                    if(isfirst_gdma)
                    {
                        mp200_ipu_hw_int_disable(IPU_HW_FUNC_GDMA);
                        ret = request_irq(INT_IPU_DMA, mp200_ipu_irq_handler, 0, dev_name, NULL);
                        if (ret < 0)
                        {
                             printk(KERN_ERR " @ipu: fail in request_irq(%d)\n",INT_IPU_DMA);
                        }
                        dbg_printk((_DEBUG_IPU & 0x08), "request_irq <GDMA>\n");
                        isfirst_gdma = 0;
                    }
                    mp200_ipu_request(p_tmp);                              /* set IPU registers & run IPU */
                }
#endif
            }
            else
            {
            /* Requests not exist */
#ifdef CONFIG_MP200_IPU_DUAL
gdma_empty:
#endif
                gdma_empty_flg = 1;

                /* permit interrupts */
                spin_unlock_irqrestore(&ipu_lock, flags);                  /* enable interrupts */

                dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() <QueueEmpty:sleep>\n");
                dbg_SetSleepStatus(SLEEP_11);
                wait_event_interruptible(dev->wait_idle_gdma, (gdma_empty_flg == 0));
                dbg_ClrSleepStatus(SLEEP_11);
                dbg_printk((_DEBUG_IPU & 0x04), "mp200_ipu_reschedule_gdma() <QueueEmpty:wake up>\n");
           }
        }
    } while(1);

    dbg_printk((_DEBUG_IPU & 0x02), "reschedule_gdma() <END>\n");

    return 0;
}


/* ------------------------------------------- */
/*   Initialize function                       */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_init_queue
* FUNCTION : initialize IPU Queue
* RETURN   :        0 : success
*          : negative : fail
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_init_queue( void )
{
    MP200_IPU_REQ_REGIST *que_regist;
    size_t size_que;
    int ret = 0;
    int i;

    dbg_printk((_DEBUG_IPU & 0x01), "ipu queue initializing <start>\n");

    /* queue struct */
    size_que = sizeof(MP200_IPU_QUEUE);
    mp200_ipu_que = kmalloc(size_que, GFP_KERNEL);
    if (!mp200_ipu_que)
    {
        ret = -ENOMEM;
        goto fail_que;
    }
    memset(mp200_ipu_que, 0, size_que);
    INIT_LIST_HEAD(&mp200_ipu_que->ipu_queue_img);
    INIT_LIST_HEAD(&mp200_ipu_que->ipu_queue_rot);
    INIT_LIST_HEAD(&mp200_ipu_que->ipu_queue_gdma);
    mp200_ipu_que->sequence = 0;

#ifdef CONFIG_MP200_IPU_DUAL
    /* Informations of execution IPU request */
    INIT_LIST_HEAD(&index_img);
    INIT_LIST_HEAD(&index_rot);
    INIT_LIST_HEAD(&index_gdma);

    memset(request_img,  0, sizeof(struct mp200_ipu_task) * INDEX_MAX);
    memset(request_rot,  0, sizeof(struct mp200_ipu_task) * INDEX_MAX);
    memset(request_gdma, 0, sizeof(struct mp200_ipu_task) * INDEX_MAX);
#endif

    /* queue request struct */
    mp200_ipu_req_bufs = kmalloc(MAX_SIZE_MP200_IPU_REQ_REGIST * IPU_BUFS_MAX, GFP_KERNEL);
    if (!mp200_ipu_req_bufs)
    {
        ret = -ENOMEM;
        goto fail_regist;
    }
    memset(mp200_ipu_req_bufs, 0, MAX_SIZE_MP200_IPU_REQ_REGIST * IPU_BUFS_MAX);
    que_regist = mp200_ipu_req_bufs;
    for (i=0; i<IPU_BUFS_MAX; i++)
    {
        mp200_ipu_que->bufs[i] = que_regist;
#if _DEBUG_IPU_LOG
        printk(KERN_INFO " @ipu: queue->bufs[%2d].ipu_req = %p\n", i, &mp200_ipu_que->bufs[i]->ipu_req);
#endif
        INIT_LIST_HEAD(&que_regist ->ipu_req_regist);
        que_regist->ipu_req.status = IPU_REQ_STATUS_IDLE;
        que_regist = (MP200_IPU_REQ_REGIST *)((char *)que_regist + MAX_SIZE_MP200_IPU_REQ_REGIST);
    }

    dbg_printk((_DEBUG_IPU & 0x02), "ipu queue initializing <end>\n");

    goto success;

 fail_regist:
     kfree(mp200_ipu_que);

 fail_que:

 success:

    return ret;
}


/*****************************************************************************
* MODULE   : mp200_ipu_init
* FUNCTION : initialize IPU Driver
* RETURN   :        0 : success
*          : negative : fail
* NOTE     : none
* CREATE   : 2006/10/04 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_init( void )
{
    MP200_IPU_DEV      *dev;
    MP200_IPU_DEV_INFO *dev_info;
    int ret = 0;
    int i;

#if _DEBUG_IPU_LOG
    IPUSmemV = ioremap(IPUSmem, IPUSmemLength);
    if(!IPUSmemV){
        ret = -EBUSY;
        printk(KERN_ERR " @ipu: cannot ioremap IPU smem %ld KB @ 0x%lx\n", IPUSmemLength >> 10, IPUSmem);
        goto fail_ipusmem;
    }
#endif

    /* allocate and initialize Device Information Structure */
    size_t size_dev_info = sizeof (MP200_IPU_DEV_INFO) + sizeof(MP200_IPU_REG_INFO) * IPU_REG_INFO_MAX;
    size_t size_dev = sizeof (MP200_IPU_DEV) + size_dev_info * mp200_ipu_devs;

    dbg_printk((_DEBUG_IPU & 0x01), "ipu driver initializing <start>\n");

    mp200_ipu_devices = kmalloc(size_dev, GFP_KERNEL);

    if (!mp200_ipu_devices) {
        ret = -ENOMEM;
        printk(KERN_ERR " @ipu: allocate failure Device Information Structure\n");
        goto fail_malloc;
    }
    memset(mp200_ipu_devices, 0, size_dev);
    dev = mp200_ipu_devices;
    dev_info = dev->ipu_dev_info;
    for (i = 0; i < mp200_ipu_devs; i++)
    {
        sema_init(&dev_info->sem, 1);
        sema_init(&dev_info->sem_vm, 1);
        init_waitqueue_head(&dev_info->wait_que);
        dev_info = (MP200_IPU_DEV_INFO *)((char *)dev_info + size_dev_info);
    }

    init_waitqueue_head(&dev->wait_idle_img);
    init_waitqueue_head(&dev->wait_idle_rot);
    init_waitqueue_head(&dev->wait_idle_gdma);
    init_waitqueue_head(&dev->wait_busy_img);
    init_waitqueue_head(&dev->wait_busy_rot);
    init_waitqueue_head(&dev->wait_busy_gdma);


    /* IMG Mmio */
    IMGMmioV = (char *)IO_ADDRESS(IMGMmio);
    /* ROT Mmio */
    ROTMmioV = (char *)IO_ADDRESS(ROTMmio);
    /* DMA Mmio */
    GDMAMmioV = (char *)IO_ADDRESS(GDMAMmio);

    dbg_printk(_DEBUG_IPU, "IMGMmio (0x%lx)  IMGMmioV (0x%p)  IMGMmioLength (0x%lx)\n", IMGMmio,  IMGMmioV,  IMGMmioLength);
    dbg_printk(_DEBUG_IPU, "ROTMmio (0x%lx)  ROTMmioV (0x%p)  ROTMmioLength (0x%lx)\n", ROTMmio,  ROTMmioV,  ROTMmioLength);
    dbg_printk(_DEBUG_IPU, "GDMAMmio(0x%lx)  GDMAMmioV(0x%p)  GDMAMmioLength(0x%lx)\n", GDMAMmio, GDMAMmioV, GDMAMmioLength);

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
    dev_set_drvdata(&mp200_ipu_device.dev, (void *)dev);
    if(platform_device_register(&mp200_ipu_device) < 0){
        printk(KERN_ERR " @ipu: could not register platform_device\n");
        goto fail_platform_device;
    }
    if(platform_driver_register(&mp200_ipu_driver) < 0){
        printk(KERN_ERR " @ipu: could not register platform_driver\n");
        goto fail_platform_driver;
    }
#endif /* CONFIG_PM || CONFIG_DPM */

    /* initialize IPU register settings */
    ret = mp200_ipu_hw_init();
    if(ret < 0)
    {
        goto fail_init;
    }

    /* initialize IPU Queue settings */
    ret = mp200_ipu_init_queue();
    if(ret < 0)
    {
        goto fail_init;
    }

#if _DEBUG_IPU_LOG
    sleep_status = (ulong *)(IPUSmemV + (IPUSmemLength - 24));
    *sleep_status = 0;
    last_regist = (ulong *)(IPUSmemV + (IPUSmemLength - 20));
    last_remove = (ulong *)(IPUSmemV + (IPUSmemLength - 12));
    *last_regist = 0; *(last_regist+1) = 0;
    *last_remove = 0; *(last_remove+1) = 0;
    num_regist = 0;
    num_remove = 0;
    printk(KERN_INFO " @ipu: sleep status flag  ->sleep_status(%p)\n", sleep_status);
    printk(KERN_INFO " @ipu: sequence flag  ->last_regist(%p)  ->last_remove(%p)\n", last_regist, last_remove);
#endif

    /* initialize spinlock */
    spin_lock_init(&ipu_lock);

    IPU_REQ_leftcall.callback = NULL;    /* no suspended request */

/*------------------------------*/
/* Create a kernel thread Start */
/*------------------------------*/

    kernel_thread(mp200_ipu_reschedule_img,  dev, CLONE_FS|CLONE_FILES|CLONE_SIGHAND);
    kernel_thread(mp200_ipu_reschedule_rot,  dev, CLONE_FS|CLONE_FILES|CLONE_SIGHAND);
    kernel_thread(mp200_ipu_reschedule_gdma, dev, CLONE_FS|CLONE_FILES|CLONE_SIGHAND);
/*----------------------------*/
/* Create a kernel thread End */
/*----------------------------*/

    goto success;


fail_init:
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
    platform_driver_unregister(&mp200_ipu_driver);
fail_platform_driver:
    platform_device_unregister(&mp200_ipu_device);
fail_platform_device:
#endif /* CONFIG_PM || CONFIG_DPM */
    kfree(mp200_ipu_devices);

fail_malloc:
#if _DEBUG_IPU_LOG
    iounmap(IPUSmemV);
    IPUSmemV = NULL;
fail_ipusmem:
#endif
success:

    return ret;
}


/* ------------------------------------------- */
/*   Private functions (initialize)            */
/* ------------------------------------------- */
/*******************************************************************************
* MODULE   : mp200_ipu_hw_init
* FUNCTION : initialize IPU register settings (only essential parameters)
* RETURN   :  0 : success
*          : -1 : fail (IPU is running)
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
********************************************************************************/
static int mp200_ipu_hw_init( void )
{
    unsigned long ulRegVal32_IMG, ulRegVal32_ROT, ulRegVal32_DMA;

    /* Unreset IMG */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMG1);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMG2);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMGPCLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_IMG_1_CLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_IMG_2_CLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_IMG_PCLK);
    mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUIMG);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUIMG1);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUIMG2);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUIMGPCLK);
    dbg_printk(_DEBUG_IPU, "mp200_ipu_hw_init <Unreset IMG>\n");

    /* Unreset ROT */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUROT);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUROTPCLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_ROT_CLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_ROT_PCLK);
    mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUROT);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUROT);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUROTPCLK);
    dbg_printk(_DEBUG_IPU, "mp200_ipu_hw_init <Unreset ROT>\n");

    /* Unreset DMA */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUDMA);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUDMACLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_IPUDMA_CLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_IPUDMA_PCLK);
    mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUDMA);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUDMA);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUDMACLK);
    dbg_printk(_DEBUG_IPU, "mp200_ipu_hw_init <Unreset GDMA>\n");

    /* Unreset IPUAHB */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUAHB);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_IPUAHB_CLK);
    mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUAHB);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUAHB);
    dbg_printk(_DEBUG_IPU, "mp200_ipu_hw_init <Unreset IPUAHB>\n");

    // check img working
    ulRegVal32_IMG = readl( IMGMmioV + IMG_ACK );
    if((ulRegVal32_IMG & IMG_ACK_BIT) == IMG_ACK_BIT){
        printk(KERN_ERR " @ipu: img working now... cannot use img\n");
        return -1;
    }
    // check rot working
    ulRegVal32_ROT = readl( ROTMmioV + ROT_STATUS );
    if((ulRegVal32_ROT & ROT_STATUS_BIT) == ROT_STATUS_BIT){
        printk(KERN_ERR " @ipu: rot working now... cannot use rot\n");
        return -1;
    }
    // check dma working
    ulRegVal32_DMA = readl( GDMAMmioV + DMA_ACK );
    if((ulRegVal32_DMA & DMA_STAT_BIT ) == DMA_ACK_BIT){
        printk(KERN_ERR " @ipu: dma working now... cannot use dma\n");
        return -1;
    }

    mp200_ipu_hw_int_disable(IPU_HW_FUNC_ALL);

    return 0;
}


/*******************************************************************************
* MODULE   : mp200_ipu_hw_int_enable
* FUNCTION : permit IPU interrupts
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
********************************************************************************/
static void mp200_ipu_hw_int_enable( int ipu_hw_func )
{
    // Interuppt Enable Set
    switch ( ipu_hw_func )
    {
        case IPU_HW_FUNC_ALL:
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_enable() <IMG>\n");
            writel(IMG_ENSETSTAT_BIT | IMG_ENSETERROR_BIT | IMG_ENSETSTOP_BIT, IMGMmioV + IMG_ENSET);
            if(ipu_hw_func == IPU_HW_FUNC_IMG)
                break;
            /* FALL THROUGH */
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_enable() <ROT>\n");
            writel(ROT_DMASTOPEN_BIT  | ROT_DMAERREN_BIT  | ROT_ROTENDEN_BIT,  ROTMmioV + ROT_INTENSET);
            if(ipu_hw_func == IPU_HW_FUNC_ROT)
                break;
            /* FALL THROUGH */
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_enable() <GDMA>\n");
            writel(DMA_ENSETAHBERR_BIT | DMA_ENSETSTAT_BIT, GDMAMmioV + DMA_ENSET);
            break;
    }
}


/*******************************************************************************
* MODULE   : mp200_ipu_hw_int_disable
* FUNCTION : prohibit IPU interrupts
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   : 2007/02/26 Y.Utsuhara
********************************************************************************/
static void mp200_ipu_hw_int_disable( int ipu_hw_func )
{
    // Interuppt Enable Clear and Interuppt Status Clear
    switch ( ipu_hw_func )
    {
        case IPU_HW_FUNC_ALL:
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_disable() <IMG>\n");
            writel(IMG_ENCLRSTAT_BIT | IMG_ENCLRERROR_BIT | IMG_ENCLRSTOP_BIT, IMGMmioV + IMG_ENCLR);
            writel(IMG_FFCLRSTAT_BIT | IMG_FFCLRERROR_BIT | IMG_FFCLRSTOP_BIT, IMGMmioV + IMG_FFCLR);
            if(ipu_hw_func == IPU_HW_FUNC_IMG)
                break;
            /* FALL THROUGH */
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_disable() <ROT>\n");
            writel(ROT_DMASTOPMASK_BIT | ROT_DMAERRMASK_BIT | ROT_ROTENDMASK_BIT, ROTMmioV + ROT_INTENCLR);
            writel(ROT_DMASTOPCLR_BIT  | ROT_DMAERRCLR_BIT  | ROT_ROTENDCLR_BIT,  ROTMmioV + ROT_INTFFCLR);
            if(ipu_hw_func == IPU_HW_FUNC_ROT)
                break;
            /* FALL THROUGH */
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_disable() <GDMA>\n");
            writel(DMA_ENCLRAHBERR_BIT | DMA_ENCLRSTAT_BIT, GDMAMmioV + DMA_ENCLR);
            writel(DMA_FFCLRAHBERR_BIT | DMA_FFCLRSTAT_BIT, GDMAMmioV + DMA_FFCLR);
            break;
    }
}


#ifdef CONFIG_MP200_IPU_DUAL
/*******************************************************************************
* MODULE   : mp200_ipu_hw_int_ffclr
* FUNCTION : prohibit IPU interrupts
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/05/24 M.Haguro
* UPDATE   :
********************************************************************************/
static void mp200_ipu_hw_int_ffclr( int ipu_hw_func )
{
    // Interuppt Enable Clear and Interuppt Status Clear
    switch ( ipu_hw_func )
    {
        case IPU_HW_FUNC_ALL:
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_ffclr() <IMG>\n");
            writel(IMG_FFCLRSTAT_BIT | IMG_FFCLRERROR_BIT | IMG_FFCLRSTOP_BIT, IMGMmioV + IMG_FFCLR);
            if(ipu_hw_func == IPU_HW_FUNC_IMG)
                break;
            /* FALL THROUGH */
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_ffclr() <ROT>\n");
            writel(ROT_DMASTOPCLR_BIT  | ROT_DMAERRCLR_BIT  | ROT_ROTENDCLR_BIT,  ROTMmioV + ROT_INTFFCLR);
            if(ipu_hw_func == IPU_HW_FUNC_ROT)
                break;
            /* FALL THROUGH */
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_hw_int_ffclr() <GDMA>\n");
            writel(DMA_FFCLRAHBERR_BIT | DMA_FFCLRSTAT_BIT, GDMAMmioV + DMA_FFCLR);
            break;
    }
}
#endif


/* ------------------------------------------- */
/*   Private functions (Queue operation)       */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_regist_queue
* FUNCTION : regist the IPU request in target queue
* RETURN   : IPU_REQ_REGISTERED(0) : the request is registered
*          : IPU_REQ_BUSY(-1)      : fail to registered request busy
*          : IPU_REQ_PARAMERR(-2)  : fail to registered request Parameter error
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_regist_queue( struct mp200_ipu_queue **pQueue, struct mp200_ipu_req *p )
{

    struct mp200_ipu_req_regist *buf;
    struct list_head            *list_queue = NULL;

    int index;

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_regist_queue() <start>\n");

    for (index = 0; index < IPU_BUFS_MAX; index++)
    {
        buf = (*pQueue)->bufs[index];
        if (NULL == buf)
            continue;
        if (buf->ipu_req.status == IPU_REQ_STATUS_IDLE)
        {
            goto index_regist;
        }
    }
    goto done;

 index_regist:

    /* copy of request */
    for (index = 0 ; index < IPU_REG_INFO_MAX ; index++)
    {
        buf->ipu_req.ipu_reg_info[index].reg = p->ipu_reg_info[index].reg;
        buf->ipu_req.ipu_reg_info[index].data = p->ipu_reg_info[index].data;
        dbg_printk((_DEBUG_IPU & 0x10), "  ipu_reg_info[%2d] ->reg(0x%08lx) ->data(0x%08lx)\n", index, buf->ipu_req.ipu_reg_info[index].reg, buf->ipu_req.ipu_reg_info[index].data);

        if (buf->ipu_req.ipu_reg_info[index].reg == IPU_REG_INFO_END){
            dbg_printk((_DEBUG_IPU & 0x10), "***break!!!\n");
            break;
        }
    }
    if (index <= 0 || index >= IPU_REG_INFO_MAX)
    {
        mp200_ipu_release_buf(buf);
        return IPU_REQ_PARAMERR;
    }

    (*pQueue)->sequence ++;
    if((*pQueue)->sequence == 0) (*pQueue)->sequence = 1;
    p->sequence = (*pQueue)->sequence;
    p->status = IPU_REQ_STATUS_REGISTERED;
    buf->ipu_req.sequence = p->sequence;
    buf->ipu_req.data     = p->data;
    buf->ipu_req.status   = p->status;
    buf->ipu_req.callback = p->callback;
    buf->ipu_req.ipu_hw_func = p->ipu_hw_func;
#if _DEBUG_IPU_LOG
    *last_regist     = p->sequence;
    num_regist++;
    *(last_regist+1) = num_regist;
#endif

    switch (buf->ipu_req.ipu_hw_func)
    {
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist_queue() list_add <IMG>\n");
            list_queue = &(*pQueue)->ipu_queue_img;
            break;
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist_queue() list_add <ROT>\n");
            list_queue = &(*pQueue)->ipu_queue_rot;
            break;
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_regist_queue() list_add <GDMA>\n");
            list_queue = &(*pQueue)->ipu_queue_gdma;
            break;
    }
    list_add_tail(&buf->ipu_req_regist, list_queue);

    return IPU_REQ_REGISTERED;

 done:
#if _DEBUG_IPU
    {
        int i;
        printk("\n");
        for(i = 0; i < IPU_BUFS_MAX; i++){
            if(!(*pQueue)->bufs[i]) continue;
            dbg_printk(_DEBUG_IPU, "regist_queue: %2d(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%ld)\n",
                    i, &(*pQueue)->bufs[i]->ipu_req_regist, (*pQueue)->bufs[i]->ipu_req_regist.prev, (*pQueue)->bufs[i]->ipu_req_regist.next,
                    (*pQueue)->bufs[i]->ipu_req.status, (*pQueue)->bufs[i]->ipu_req.sequence);
        }
        printk("\n");
    }
#endif

    return IPU_REQ_BUSY;
}


/*****************************************************************************
* MODULE   : mp200_ipu_remove_queue
* FUNCTION : remove the IPU requeest from target queue
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static void mp200_ipu_remove_queue( unsigned long sequence, int ipu_hw_func )
{
    struct mp200_ipu_req_regist *p;

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_remove_queue() <start>\n");

    p = mp200_ipu_search_queue(sequence,ipu_hw_func);
    if (p == NULL ) return;

    {
#if _DEBUG_IPU_LOG
        *last_remove     = sequence;
        num_remove++;
        *(last_remove+1) = num_remove;
#endif
#if (_DEBUG_IPU & 0x10)
        struct mp200_ipu_queue *pQueue = mp200_ipu_que;
        int i, j=0;
        dbg_printk((_DEBUG_IPU & 0x10), "\n");
        dbg_printk((_DEBUG_IPU & 0x10), "remove_queue: IMG(%p) ROT(%p) GDMA(%p)\n", &pQueue->ipu_queue_img, &pQueue->ipu_queue_rot, &pQueue->ipu_queue_gdma);
        for(i = 0; i < IPU_BUFS_MAX; i++){
            if(!pQueue->bufs[i]) continue;
            dbg_printk((_DEBUG_IPU & 0x10), "remove_queue: %2d(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%ld)\n",
                    i, &pQueue->bufs[i]->ipu_req_regist, pQueue->bufs[i]->ipu_req_regist.prev, pQueue->bufs[i]->ipu_req_regist.next,
                    pQueue->bufs[i]->ipu_req.status, pQueue->bufs[i]->ipu_req.sequence);
            if(j){ if(pQueue->bufs[i]->ipu_req.status == IPU_REQ_STATUS_IDLE) break;
            }else{ if(pQueue->bufs[i]->ipu_req.status != IPU_REQ_STATUS_IDLE) j++;
            }
        }
#endif
        list_del(&p->ipu_req_regist);
#if (_DEBUG_IPU & 0x10)
        dbg_printk(_DEBUG_IPU, "************************\n");
        if (i >= IPU_BUFS_MAX) i = IPU_BUFS_MAX - 1;
        for(j = 0; j <= i; j++){
            dbg_printk((_DEBUG_IPU & 0x10), "remove_queue: %2d(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%ld)\n",
                    j, &pQueue->bufs[j]->ipu_req_regist, pQueue->bufs[j]->ipu_req_regist.prev, pQueue->bufs[j]->ipu_req_regist.next,
                    pQueue->bufs[j]->ipu_req.status, pQueue->bufs[j]->ipu_req.sequence);
            if(pQueue->bufs[j]->ipu_req.status == IPU_REQ_STATUS_RUNNING){
                int index;
                struct mp200_ipu_req_regist *buf = pQueue->bufs[j];
                for(index = 0 ; index < IPU_REG_INFO_MAX ; index++){
                    dbg_printk((_DEBUG_IPU & 0x10), "  ipu_reg_info[%2d] ->reg(0x%08lx) ->data(0x%08lx)\n", index, buf->ipu_req.ipu_reg_info[index].reg, buf->ipu_req.ipu_reg_info[index].data);
                    if (buf->ipu_req.ipu_reg_info[index].reg == IPU_REG_INFO_END) break;
                }
            }
        }
        dbg_printk((_DEBUG_IPU & 0x10), "\n");
#endif
        dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_remove_queue() remove success\n");
    }
}


/*****************************************************************************
* MODULE   : mp200_ipu_release_buf
* FUNCTION : release the IPU requeest buffer
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/12/01 M.Sasaki
* UPDATE   :
******************************************************************************/
static void mp200_ipu_release_buf( struct mp200_ipu_req_regist *p )
{
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_release_buf() <start>\n");

    if (p == NULL)
    {
        dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_release_buf() p(NULL)\n");
        return;
    }

    {
        p->ipu_req.status = IPU_REQ_STATUS_IDLE;
        /* Initialization of MP200_IPU_REG_INFO */
        memset(p->ipu_req.ipu_reg_info, 0, SIZE_MP200_IPU_REG_INFO * IPU_REG_INFO_MAX);
    }
}


/*****************************************************************************
* MODULE   : mp200_ipu_search_queue
* FUNCTION : search the IPU request from queues
* RETURN   : NULL    : target request does not exist
*          : pointer : pointer to the top of queue containing target request
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static struct mp200_ipu_req_regist *mp200_ipu_search_queue( unsigned long sequence, int ipu_hw_func )
{
    struct mp200_ipu_queue *pQueue = mp200_ipu_que;
    struct mp200_ipu_req_regist *ret = NULL;

    switch (ipu_hw_func)
    {
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_search_queue() IMG <start>\n");
            ret = mp200_ipu_search_queue_func(sequence, &pQueue->ipu_queue_img);
            break;
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_search_queue() ROT <start>\n");
            ret = mp200_ipu_search_queue_func(sequence, &pQueue->ipu_queue_rot);
            break;
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_search_queue() GDMA <start>\n");
            ret = mp200_ipu_search_queue_func(sequence, &pQueue->ipu_queue_gdma);
            break;
        case IPU_HW_FUNC_ALL:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_search_queue() ALL <start>\n");
            ret = mp200_ipu_search_queue_func(sequence, &pQueue->ipu_queue_img);
            if (ret == NULL)
            {
                ret = mp200_ipu_search_queue_func(sequence, &pQueue->ipu_queue_rot);
                if (ret == NULL)
                {
                    ret = mp200_ipu_search_queue_func(sequence, &pQueue->ipu_queue_gdma);
                }
            }
            break;
    }
    return ret;
}


/*****************************************************************************
* MODULE   : mp200_ipu_search_queue_func
* FUNCTION : search the IPU request from queues(sub)
* RETURN   : NULL    : target request does not exist
*          : pointer : pointer to the top of queue containing target request
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static struct mp200_ipu_req_regist *mp200_ipu_search_queue_func( unsigned long sequence, struct list_head *list )
{
    struct mp200_ipu_req_regist *ret = NULL;
    struct mp200_ipu_req_regist *buf;
    struct list_head *list_queue;

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_search_queue_func()\n");

    list_for_each(list_queue, list)
    {
        buf = list_entry(list_queue, struct mp200_ipu_req_regist, ipu_req_regist);

        if(buf->ipu_req.sequence == sequence)
        {
            dbg_printk((_DEBUG_IPU & 0x10), "mp200_ipu_search_queue_func() <find>\n");
            ret = buf;
            break;
        }
    }
    return ret;
}


#ifdef CONFIG_MP200_IPU_DUAL
/* --------------------------------------------------- */
/*   Private functions (Queue operation for "DUAL_FF") */
/* --------------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_reqqueue_addtail
* FUNCTION :
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/05/16 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_reqqueue_addtail(struct list_head *index, struct mp200_ipu_task *request, struct mp200_ipu_req_regist *new_req)
{
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reqqueue_addtail() <start>\n");
    request->status  = REQ_USED;
    request->request = new_req;
    list_add_tail(&request->req_list, index);
    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_reqqueue_addtail() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_reqqueue_del
* FUNCTION :
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/05/16 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_reqqueue_del(struct mp200_ipu_task *request)
{
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reqqueue_del() <start>\n");
    list_del(&request->req_list);
    request->request->callback_flag = FALSE;
    request->request->request_flag  = FALSE;
    request->request->hw_req_flag   = NO_HW_REQ; /* IPU running flag */
    request->request                = NULL;
    request->status                 = REQ_FREE;
    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_reqqueue_del() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_reqqueue_del_next_sequence
* FUNCTION :
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/06/07 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_reqqueue_del_next_sequence(struct mp200_ipu_task *request, unsigned long sequence)
{
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reqqueue_del_next_sequence() <start>\n");

    if(mp200_ipu_reqqueue_search_index(request) == -1)
    {
        struct mp200_ipu_task *p_tmp;
        int index;
        index = mp200_ipu_reqqueue_search_sequence(request, sequence);
        if(index != -1){
            p_tmp = &request[((index+1) % INDEX_MAX)];
            if(p_tmp->request){
                if(p_tmp->request->request_flag){
                    p_tmp->request->ipu_req.status = IPU_REQ_STATUS_REGISTERED;
                    mp200_ipu_reqqueue_del(p_tmp);
                }
            }
        }
    }
    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_reqqueue_del_next_sequence() <end>\n");
}


#if defined(CONFIG_PM) || defined(CONFIG_DPM)
/*****************************************************************************
* MODULE   : mp200_ipu_reqqueue_search_empty
* FUNCTION :
* RETURN   : 1 : empty
*          : 0 : not empty
* NOTE     : none
* CREATE   : 2007/05/16 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_ipu_reqqueue_search_empty(struct list_head *index)
{
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reqqueue_search_empty() <start>\n");
    return (list_empty(index) ? 1 : 0);
}
#endif /* CONFIG_PM || CONFIG_DPM */


/*****************************************************************************
* MODULE   : mp200_ipu_reqqueue_search_index
* FUNCTION :
* RETURN   : i : free index no.
*          : -1: error
* NOTE     : none
* CREATE   : 2007/05/16 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_ipu_reqqueue_search_index(struct mp200_ipu_task *request)
{
    int i;
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reqqueue_search_index() <start>\n");

    for(i = 0; i < INDEX_MAX; i++){
        if(request[i].status == REQ_FREE) return i;
    }
    return -1;
}


/*****************************************************************************
* MODULE   : mp200_ipu_reqqueue_search_sequence
* FUNCTION :
* RETURN   : i : index no.
*          : -1: error
* NOTE     : none
* CREATE   : 2007/05/24 M.Haguro
* UPDATE   :
******************************************************************************/
static int mp200_ipu_reqqueue_search_sequence(struct mp200_ipu_task *request, unsigned long sequence)
{
    int i;
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_reqqueue_search_sequence() <start>\n");

    for(i = 0; i < INDEX_MAX; i++){
        if(request[i].request->ipu_req.sequence == sequence){
            if(request[i].request->ipu_req.status == IPU_REQ_STATUS_RUNNING     ||
               request[i].request->ipu_req.status == IPU_REQ_STATUS_WAIT_CANCEL ||
               request[i].request->ipu_req.status == IPU_REQ_STATUS_CANCELLED)
                return i;
        }
    }
    return -1;
}
#endif


/* ------------------------------------------- */
/*   Private functions (request)               */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_request
* FUNCTION : set IPU registers & run IPU
* RETURN   :  0 : success
*          : -1 : fail
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_request( struct mp200_ipu_req_regist *p )
{
    int write_req = 0;                /* flag for writing IMG_REQ */

    struct mp200_ipu_reg_info *ipu_reg_info;
    unsigned long ulRegVal32;
    unsigned long IPU_REQ = IMG_REQ, IPU_REQ_BIT = IMG_REQ_BIT;
    char *IPUMmioV = IMGMmioV;
    int  INT_IPU = INT_IMG;
    int  reg_cnt = 0;
    int  ret = 0;
#ifdef CONFIG_MP200_IPU_DUAL
    unsigned long  flags;

    spin_lock_irqsave(&ipu_lock, flags);      /* disable interrupts */
    p->request_flag = TRUE;
#endif

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request() <start>\n");

    switch (p->ipu_req.ipu_hw_func){
        case IPU_HW_FUNC_IMG:   dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request() IMG\n");   break;
        case IPU_HW_FUNC_ROT:   dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request() ROT\n");   break;
        case IPU_HW_FUNC_GDMA:  dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request() GDMA\n");  break;
    }

    /* initialize register */
    switch (p->ipu_req.ipu_hw_func)
    {
        case IPU_HW_FUNC_IMG:
        default:
#ifdef CONFIG_MP200_IPU_DUAL
            /* check img working */
            ulRegVal32 = readl(IMGMmioV + IMG_STATUS);
            if((ulRegVal32 & IMG_DFF_RSV_BIT) == IMG_DFF_RSV_BIT){
                printk( KERN_ERR " @ipu: IMG WORKING NOW\n");
                ret = -2;
#else
            /* check img working */
            ulRegVal32 = readl(IMGMmioV + IMG_ACK);
            if((ulRegVal32 & IMG_ACK_BIT) == IMG_ACK_BIT){
                printk( KERN_ERR " @ipu: IMG WORKING NOW\n");
                ret = -1;
#endif
                goto error_ret;
            }
            /* initialize img register */
            dbg_printk((_DEBUG_IPU & 0x08), "initialize img register\n");
            writel(IMG_MODE_RESET,        IMGMmioV + IMG_MODE);
            writel(IMG_HSTEP_RESET,       IMGMmioV + IMG_HSTEP);
            writel(IMG_VSTEP_RESET,       IMGMmioV + IMG_VSTEP);
            writel(IMG_HFOLD_RESET,       IMGMmioV + IMG_HFOLD);
            writel(IMG_VFOLD_RESET,       IMGMmioV + IMG_VFOLD);
            writel(IMG_INDATABYTE_RESET,  IMGMmioV + IMG_INDATABYTE);
            writel(IMG_OUTDATABYTE_RESET, IMGMmioV + IMG_OUTDATABYTE);
#ifdef CONFIG_MP200_IPU_DUAL
            writel(IMG_REG_EN_BIT,    IMGMmioV + IMG_DUAL_FF);
#else
            writel(IMG_DUAL_FF_RESET,     IMGMmioV + IMG_DUAL_FF);
#endif
#if _IPU_COLCONV
            writel(IMG_RGBYUV_CONF_RESET, IMGMmioV + IMG_RGBYUV_CONF);
            writel(IMG_RGBYUV00_RESET,    IMGMmioV + IMG_RGBYUV00);
            writel(IMG_RGBYUV01_RESET,    IMGMmioV + IMG_RGBYUV01);
            writel(IMG_RGBYUV02_RESET,    IMGMmioV + IMG_RGBYUV02);
            writel(IMG_RGBYUV10_RESET,    IMGMmioV + IMG_RGBYUV10);
            writel(IMG_RGBYUV11_RESET,    IMGMmioV + IMG_RGBYUV11);
            writel(IMG_RGBYUV12_RESET,    IMGMmioV + IMG_RGBYUV12);
            writel(IMG_RGBYUV20_RESET,    IMGMmioV + IMG_RGBYUV20);
            writel(IMG_RGBYUV21_RESET,    IMGMmioV + IMG_RGBYUV21);
            writel(IMG_RGBYUV22_RESET,    IMGMmioV + IMG_RGBYUV22);
            writel(IMG_YUVRGB_CONF_RESET, IMGMmioV + IMG_YUVRGB_CONF);
            writel(IMG_YUVRGB00_RESET,    IMGMmioV + IMG_YUVRGB00);
            writel(IMG_YUVRGB01_RESET,    IMGMmioV + IMG_YUVRGB01);
            writel(IMG_YUVRGB02_RESET,    IMGMmioV + IMG_YUVRGB02);
            writel(IMG_YUVRGB10_RESET,    IMGMmioV + IMG_YUVRGB10);
            writel(IMG_YUVRGB11_RESET,    IMGMmioV + IMG_YUVRGB11);
            writel(IMG_YUVRGB12_RESET,    IMGMmioV + IMG_YUVRGB12);
            writel(IMG_YUVRGB20_RESET,    IMGMmioV + IMG_YUVRGB20);
            writel(IMG_YUVRGB21_RESET,    IMGMmioV + IMG_YUVRGB21);
            writel(IMG_YUVRGB22_RESET,    IMGMmioV + IMG_YUVRGB22);
#endif
            /* IMG Variable set */
            IPU_REQ     = IMG_REQ;
            IPU_REQ_BIT = IMG_REQ_BIT;
            IPUMmioV    = IMGMmioV;
            INT_IPU     = INT_IMG;
            break;
        case IPU_HW_FUNC_ROT:
#ifdef CONFIG_MP200_IPU_DUAL
            /* check rot working */
            ulRegVal32 = readl(ROTMmioV + ROT_INTSTATUS);
            if((ulRegVal32 & ROT_DFF_RSV_BIT) == ROT_DFF_RSV_BIT){
                printk( KERN_ERR " @ipu: ROT WORKING NOW\n");
                ret = -2;
#else
            /* check rot working */
            ulRegVal32 = readl(ROTMmioV + ROT_STATUS);
            if((ulRegVal32 & ROT_STATUS_BIT) == ROT_STATUS_BIT){
                printk( KERN_ERR " @ipu: ROT WORKING NOW\n");
                ret = -1;
#endif
                goto error_ret;
            }
            /* initialize rot register */
            dbg_printk((_DEBUG_IPU & 0x08), "initialize rot register\n");
            writel(ROT_MODE_RESET,     ROTMmioV + ROT_MODE);
            writel(ROT_SRCBYTE_RESET,  ROTMmioV + ROT_SRCBYTE);
            writel(ROT_DSTBYTE_RESET,  ROTMmioV + ROT_DSTBYTE);
#ifdef CONFIG_MP200_IPU_DUAL
            writel(ROT_REG_EN_BIT,     ROTMmioV + ROT_DUAL_FF);
#else
            writel(ROT_DUAL_FF_RESET,  ROTMmioV + ROT_DUAL_FF);
#endif
            /* ROT Variable set */
            IPU_REQ     = ROT_REQ;
            IPU_REQ_BIT = ROT_REQ_BIT;
            IPUMmioV    = ROTMmioV;
            INT_IPU     = INT_ROT;
            break;
        case IPU_HW_FUNC_GDMA:
#ifdef CONFIG_MP200_IPU_DUAL
            /* check gdma working */
            ulRegVal32 = readl(GDMAMmioV + DMA_STATUS);
            if((ulRegVal32 & DMA_DFF_RSV_BIT) == DMA_DFF_RSV_BIT){
                printk(KERN_ERR " @ipu: GDMA WORKING NOW\n");
                ret = -2;
#else
            /* check gdma working */
            ulRegVal32 = readl(GDMAMmioV + DMA_ACK);
            if((ulRegVal32 & DMA_ACK_BIT) == DMA_ACK_BIT){
                printk(KERN_ERR " @ipu: GDMA WORKING NOW\n");
                ret = -1;
#endif
                goto error_ret;
            }
            /* initialize gdma register */
            dbg_printk((_DEBUG_IPU & 0x08), "initialize gdma register\n");
            writel(DMA_MODE_RESET,      GDMAMmioV + DMA_MODE);
            writel(DMA_SRCBYTE_1_RESET, GDMAMmioV + DMA_SRCBYTE_1);
            writel(DMA_SRCBYTE_2_RESET, GDMAMmioV + DMA_SRCBYTE_2);
            writel(DMA_DSTBYTE_RESET,   GDMAMmioV + DMA_DSTBYTE);
#ifdef CONFIG_MP200_IPU_DUAL
            writel(DMA_REG_EN_BIT,      GDMAMmioV + DMA_DUAL_FF);
#else
            writel(DMA_DUAL_FF_RESET,   GDMAMmioV + DMA_DUAL_FF);
#endif
            writel(DMA_AUTO_SCAN_RESET, GDMAMmioV + DMA_AUTO_SCAN);
            /* GDMA Variable set */
            IPU_REQ     = DMA_REQ;
            IPU_REQ_BIT = DMA_REQ_BIT;
            IPUMmioV    = GDMAMmioV;
            INT_IPU     = INT_IPU_DMA;
            break;
    }

    ipu_reg_info = p->ipu_req.ipu_reg_info;

    /* loop of register settings                                               */
    /*   read IPU register setting information structure, and set IPU registers */
    for(reg_cnt = 0 ; reg_cnt < IPU_REG_INFO_MAX ; reg_cnt++, ipu_reg_info++)
    {
        if(ipu_reg_info->reg == IPU_REG_INFO_END)
        /* end of IPU register setting information structure */
        {
#ifdef CONFIG_MP200_IPU_DUAL
            spin_unlock_irqrestore(&ipu_lock, flags); /* enable interrupts */
            mp200_ipu_request_sub(p, IPU_CALLBACK_SET_PARAM);
            spin_lock_irqsave(&ipu_lock, flags);      /* disable interrupts */
#else
            mp200_ipu_request_sub(p, IPU_CALLBACK_SET_PARAM);
#endif
            Call_StatusCtrlFunc_OFF();

            mp200_ipu_request_wakeup(p->ipu_req.ipu_hw_func);  /* start the next request */
            break;
        }

        /* skip accessing registers which are concerned with interrupt, or read-only, or out of scope */
        switch (p->ipu_req.ipu_hw_func)
        {
            case IPU_HW_FUNC_IMG:
                if((ipu_reg_info->reg >= IMG_ACK         &&  ipu_reg_info->reg <= IMG_FFCLR)        ||
                   (ipu_reg_info->reg >  IMG_DSTSIZE     &&  ipu_reg_info->reg <  IMG_SRCYRGBADR_B) ||
                   (ipu_reg_info->reg >  IMG_DSTYRGBADR  &&  ipu_reg_info->reg <  IMG_SRCUVADR_B)   ||
                   (ipu_reg_info->reg >  IMG_DSTUVADR    &&  ipu_reg_info->reg <  IMG_SRCHSIZE_B)   ||
                   (ipu_reg_info->reg >  IMG_VFOLD       &&  ipu_reg_info->reg <  IMG_PEL_ENDIAN)   ||
                   (ipu_reg_info->reg >  IMG_PEL_ENDIAN  &&  ipu_reg_info->reg <  IMG_RGBYUV_CONF)  ||
                   (ipu_reg_info->reg >= IMG_DUAL_FF     &&  ipu_reg_info->reg <  IMG_INDATABYTE_B_CMP) ||
                   (ipu_reg_info->reg >  IMG_B_BRITNESS))
                {
                    dbg_printk((_DEBUG_IPU & 0x08), "  skip  (data(0x%08lx) addr(0x%p + 0x%08lx)) <IMG>\n", ipu_reg_info->data, IPUMmioV, ipu_reg_info->reg);
                    continue;
                }
                break;

            case IPU_HW_FUNC_ROT:
                if((ipu_reg_info->reg >= ROT_STATUS     && ipu_reg_info->reg <  ROT_SRCSIZE)    ||
                   (ipu_reg_info->reg >  ROT_DSTSIZE    && ipu_reg_info->reg <  ROT_SRCYADR_A)  ||
                   (ipu_reg_info->reg >  ROT_SRCYADR_C  && ipu_reg_info->reg <  ROT_DSTYADR_A)  ||
                   (ipu_reg_info->reg >  ROT_DSTYADR_C  && ipu_reg_info->reg <  ROT_SRCUVADR_A) ||
                   (ipu_reg_info->reg >  ROT_SRCUVADR_C && ipu_reg_info->reg <  ROT_DSTUVADR_A) ||
                   (ipu_reg_info->reg >  ROT_DSTUVADR_C && ipu_reg_info->reg <  ROT_SRCHSIZE)   ||
                   (ipu_reg_info->reg >  ROT_SRCHSIZE   && ipu_reg_info->reg <  ROT_SRCVSIZE)   ||
                   (ipu_reg_info->reg >  ROT_SRCVSIZE   && ipu_reg_info->reg <  ROT_FORMAT)     ||
                   (ipu_reg_info->reg == ROT_DUAL_FF)                                           ||
                   (ipu_reg_info->reg >  ROT_DSTBYTE_CMP))
                {
                    dbg_printk((_DEBUG_IPU & 0x08), "  skip  (data(0x%08lx) addr(0x%p + 0x%08lx)) <ROT>\n", ipu_reg_info->data, IPUMmioV, ipu_reg_info->reg);
                    continue;
                }
                break;

            case IPU_HW_FUNC_GDMA:
                if((ipu_reg_info->reg >= DMA_ACK         && ipu_reg_info->reg <= DMA_FFCLR)         ||
                   (ipu_reg_info->reg >  DMA_DSTSIZE     && ipu_reg_info->reg <  DMA_SRCYRGBADR_1)  ||
                   (ipu_reg_info->reg >  DMA_DSTYRGBADR  && ipu_reg_info->reg <  DMA_SRCUVADR_1)    ||
                   (ipu_reg_info->reg >  DMA_DSTUVADR    && ipu_reg_info->reg <  DMA_SRCVADR_1)     ||
                   (ipu_reg_info->reg >  DMA_DSTVADR     && ipu_reg_info->reg <  DMA_HSIZE)         ||
                   (ipu_reg_info->reg >= DMA_DUAL_FF     && ipu_reg_info->reg <  DMA_SRCBYTE_1_CMP) ||
                   (ipu_reg_info->reg >  DMA_DSTBYTE_CMP))
                {
                    dbg_printk((_DEBUG_IPU & 0x08), "  skip  (data(0x%08lx) addr(0x%p + 0x%08lx)) <GDMA>\n", ipu_reg_info->data, IPUMmioV, ipu_reg_info->reg);
                    continue;
                }
                break;
        }

        if(ipu_reg_info->reg == IPU_REQ)
        /* setting IPU_REQ means end of IPU register setting information structure */
        {
            if(ipu_reg_info->data != IPU_REQ_BIT)
            /* return error if IPU_REQ = 0 */
            {
#ifdef CONFIG_MP200_IPU_DUAL
                spin_unlock_irqrestore(&ipu_lock, flags); /* enable interrupts */
                mp200_ipu_request_sub(p, IPU_CALLBACK_FAIL);
                spin_lock_irqsave(&ipu_lock, flags);      /* disable interrupts */
#else
                mp200_ipu_request_sub(p, IPU_CALLBACK_FAIL);
#endif
                Call_StatusCtrlFunc_OFF();

                switch (INT_IPU)
                {
                    case INT_IMG:
                        printk( KERN_INFO " @ipu: request data error. 'IMG_REQ data 0x%08x'\n", (unsigned int)ipu_reg_info->data );
                        break;
                    case INT_ROT:
                        printk( KERN_INFO " @ipu: request data error. 'ROT_REQ data 0x%08x'\n", (unsigned int)ipu_reg_info->data );
                        break;
                    case INT_IPU_DMA:
                        printk( KERN_INFO " @ipu: request data error. 'GDMA_REQ data 0x%08x'\n", (unsigned int)ipu_reg_info->data );
                        break;
                }
                ret = -1;
                mp200_ipu_request_wakeup(p->ipu_req.ipu_hw_func); /* start the next request */
                break;
            }
            else
            {
                switch (INT_IPU)
                {
                    case INT_IMG:
                        dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_request(): sequence(%ld) <IMG>\n", p->ipu_req.sequence);
                        mp200_ipu_hw_int_enable(IPU_HW_FUNC_IMG);
                        break;
                    case INT_ROT:
                        dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_request(): sequence(%ld) <ROT>\n", p->ipu_req.sequence);
                        mp200_ipu_hw_int_enable(IPU_HW_FUNC_ROT);
                        break;
                    case INT_IPU_DMA:
                        dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_request(): sequence(%ld) <GDMA>\n", p->ipu_req.sequence);
                        mp200_ipu_hw_int_enable(IPU_HW_FUNC_GDMA);
                        break;
                }
                p->hw_req_flag = SET_HW_REQ; /* IPU running */
                write_req = 1;
            }
        }

#if (_DEBUG_IPU & 0x20)
        if((INT_IPU == INT_IMG     && ipu_reg_info->reg == IMG_REQ && ipu_reg_info->data == IMG_REQ_BIT) ||
           (INT_IPU == INT_ROT     && ipu_reg_info->reg == ROT_REQ && ipu_reg_info->data == ROT_REQ_BIT) ||
           (INT_IPU == INT_IPU_DMA && ipu_reg_info->reg == DMA_REQ && ipu_reg_info->data == DMA_REQ_BIT))
        {
            unsigned int i, j, max;
            unsigned long ulRegVal[4];
            switch(INT_IPU){
            case INT_IMG:     max = 0x12c; dbg_printk((_DEBUG_IPU & 0x20), "  IMG(%p)\n", IPUMmioV);  break;
            case INT_ROT:     max = 0xec;  dbg_printk((_DEBUG_IPU & 0x20), "  ROT(%p)\n", IPUMmioV);  break;
            case INT_IPU_DMA: max = 0x8c;  dbg_printk((_DEBUG_IPU & 0x20), "  GDMA(%p)\n", IPUMmioV); break;
            }
            for(i = 0; i < max; i+=0x10){
                for(j = 0; j < 0x10; j+=4) ulRegVal[j/4] = readl(IPUMmioV + i + j);
                dbg_printk((_DEBUG_IPU & 0x20), "  %04x|  %08lx  %08lx  %08lx  %08lx\n", i, ulRegVal[0], ulRegVal[1], ulRegVal[2], ulRegVal[3]);
            }
        }
#endif

#if _IPU_PERF == 1
        if((INT_IPU == INT_IMG     && ipu_reg_info->reg == IMG_REQ && ipu_reg_info->data == IMG_REQ_BIT) ||
           (INT_IPU == INT_ROT     && ipu_reg_info->reg == ROT_REQ && ipu_reg_info->data == ROT_REQ_BIT) ||
           (INT_IPU == INT_IPU_DMA && ipu_reg_info->reg == DMA_REQ && ipu_reg_info->data == DMA_REQ_BIT))
        {
            dbg_GetStartTime(1);
        }
#endif

        writel(ipu_reg_info->data, IPUMmioV + ipu_reg_info->reg);
        dbg_printk((_DEBUG_IPU & 0x08), "  writel(data(0x%08lx) addr(0x%p + 0x%08lx))\n", ipu_reg_info->data, IPUMmioV, ipu_reg_info->reg);

        if(write_req) break;
    }

    if(reg_cnt >= IPU_REG_INFO_MAX)
    {
#ifdef CONFIG_MP200_IPU_DUAL
        spin_unlock_irqrestore(&ipu_lock, flags); /* enable interrupts */
        mp200_ipu_request_sub(p, IPU_CALLBACK_FAIL);
        spin_lock_irqsave(&ipu_lock, flags);      /* disable interrupts */
#else
        mp200_ipu_request_sub(p, IPU_CALLBACK_FAIL);
#endif
        Call_StatusCtrlFunc_OFF();
        printk( KERN_INFO " @ipu: request data error. 'over 230 request'\n" );
        ret = -1;
        mp200_ipu_request_wakeup(p->ipu_req.ipu_hw_func);    /* start the next request */
    }

error_ret:
#ifdef CONFIG_MP200_IPU_DUAL
    spin_unlock_irqrestore(&ipu_lock, flags); /* enable interrupts */
#endif

    switch (p->ipu_req.ipu_hw_func){
    case IPU_HW_FUNC_IMG:   dbg_printk((_DEBUG_IPU & 0x02), "ipu_request IMG<END>\n");   break;
    case IPU_HW_FUNC_ROT:   dbg_printk((_DEBUG_IPU & 0x02), "ipu_request ROT<END>\n");   break;
    case IPU_HW_FUNC_GDMA:  dbg_printk((_DEBUG_IPU & 0x02), "ipu_request GDMA<END>\n");  break;
    }

    return ret;
}


/*****************************************************************************
* MODULE   : mp200_ipu_request_sub
* FUNCTION : finalize the IPU requeest buffer
*            (mp200_ipu_request sub function)
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/12/14 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_request_sub( struct mp200_ipu_req_regist *p, int callback_flag )
{
    struct mp200_ipu_req_leftcall illegal_leftcall;

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request_sub() <start>\n");

    /* callback */
    illegal_leftcall.callback    = p->ipu_req.callback;
    illegal_leftcall.sequence    = p->ipu_req.sequence;
    illegal_leftcall.data        = p->ipu_req.data;
    illegal_leftcall.ipu_hw_func = p->ipu_req.ipu_hw_func;
    illegal_leftcall.flag        = callback_flag;

    /* calback is required */
    if (((illegal_leftcall.ipu_hw_func == IPU_HW_FUNC_IMG)   && (p->callback_flag == TRUE)) ||
        ((illegal_leftcall.ipu_hw_func == IPU_HW_FUNC_ROT)   && (p->callback_flag == TRUE)) ||
        ((illegal_leftcall.ipu_hw_func == IPU_HW_FUNC_GDMA)  && (p->callback_flag == TRUE)))
    {
        mp200_ipu_request_remove_sub(p);

        /* request running flag initialize */
        mp200_ipu_request_release_sub(illegal_leftcall.ipu_hw_func);

        if(*(illegal_leftcall.callback))
        {
            if ((*(illegal_leftcall.callback) == (void *)mp200_ipu_callback_img) ||
                (*(illegal_leftcall.callback) == (void *)mp200_ipu_callback_rot) ||
                (*(illegal_leftcall.callback) == (void *)mp200_ipu_callback_gdma))
            {
                (*(illegal_leftcall.callback))(illegal_leftcall.data, illegal_leftcall.flag);     /* callback (can't start IPU process (Error)) */
            }
            else
            {
                (*(illegal_leftcall.callback))(illegal_leftcall.sequence, illegal_leftcall.flag); /* callback (can't start IPU process (Error)) */
            }
        }
    }
    else
    {
        /* request running flag initialize */
        mp200_ipu_request_release_sub(illegal_leftcall.ipu_hw_func);
    }

    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_request_sub() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_request_remove_sub
* FUNCTION : remove the IPU requeest from target queue
*            (mp200_ipu_request sub function)
* RETURN   : none
*          :
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static void mp200_ipu_request_remove_sub( struct mp200_ipu_req_regist *p )
{
    unsigned long flags;

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request_remove_sub() <start>\n");
    spin_lock_irqsave(&ipu_lock, flags);              /* prohibit interrupts */

    /* remove from queue */
    p->ipu_req.status = IPU_REQ_STATUS_DONE;
    mp200_ipu_remove_queue(p->ipu_req.sequence, p->ipu_req.ipu_hw_func);
    spin_unlock_irqrestore(&ipu_lock, flags);         /* permit interrupts */
    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_request_remove_sub() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_request_release_sub
* FUNCTION : release the IPU requeest buffer
*            (mp200_ipu_request sub function)
* RETURN   : none
*          :
* NOTE     : none
* CREATE   : 2006/12/14 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_request_release_sub( int ipu_hw_func )
{
#ifdef CONFIG_MP200_IPU_DUAL
    struct mp200_ipu_task *p_tmp;
#endif

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request_release_sub() <start>\n");
    switch (ipu_hw_func)
    {
        case IPU_HW_FUNC_IMG:
#ifdef CONFIG_MP200_IPU_DUAL
            p_tmp = list_entry(index_img.next, struct mp200_ipu_task, req_list);
            mp200_ipu_release_buf(p_tmp->request);
            mp200_ipu_reqqueue_del(p_tmp);
#else
            mp200_ipu_release_buf(request_img);
            request_img = NULL;
#endif
            break;
        case IPU_HW_FUNC_ROT:
#ifdef CONFIG_MP200_IPU_DUAL
            p_tmp = list_entry(index_rot.next, struct mp200_ipu_task, req_list);
            mp200_ipu_release_buf(p_tmp->request);
            mp200_ipu_reqqueue_del(p_tmp);
#else
            mp200_ipu_release_buf(request_rot);
            request_rot = NULL;
#endif
            break;
        case IPU_HW_FUNC_GDMA:
#ifdef CONFIG_MP200_IPU_DUAL
            p_tmp = list_entry(index_gdma.next, struct mp200_ipu_task, req_list);
            mp200_ipu_release_buf(p_tmp->request);
            mp200_ipu_reqqueue_del(p_tmp);
#else
            mp200_ipu_release_buf(request_gdma);
            request_gdma = NULL;
#endif
            break;
    }
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_request_release_sub() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_request_wakeup
* FUNCTION : start of processing instruction(thread wakeup)
*            (mp200_ipu_request sub function)
* RETURN   : none
*          :
* NOTE     : none
* CREATE   : 2006/10/02 M.Sasaki
* UPDATE   :
******************************************************************************/
static void mp200_ipu_request_wakeup( int ipu_hw_func )
{
    struct mp200_ipu_dev *dev = mp200_ipu_devices;
    switch(ipu_hw_func)
    {
        case IPU_HW_FUNC_IMG:
            wake_up_interruptible(&dev->wait_busy_img);
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_request_wakeup() <IMG:wake up>\n");
            break;
        case IPU_HW_FUNC_ROT:
            wake_up_interruptible(&dev->wait_busy_rot);
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_request_wakeup() <ROT:wake up>\n");
            break;
        case IPU_HW_FUNC_GDMA:
            wake_up_interruptible(&dev->wait_busy_gdma);
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_request_wakeup() <GDMA:wake up>\n");
            break;
    }
}


/* ------------------------------------------- */
/*   Interrupt Management functions            */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_irq_handler
* FUNCTION : IPU interrupt handler
*            analyze the cause of IPU interrupt and execute each operations
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,00)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
static irqreturn_t mp200_ipu_irq_handler( int irq, void *dev_id )
#else
static irqreturn_t mp200_ipu_irq_handler( int irq, void *dev_id, struct pt_regs *fp )
#endif
{
    unsigned long flags;
    spin_lock_irqsave(&ipu_lock, flags);         /* prohibit interrupts  */
    mp200_ipu_irq_handler_sub(irq, dev_id);
    spin_unlock_irqrestore(&ipu_lock, flags);    /* prohibit interrupts  */
    return IRQ_HANDLED;
}
#else
static void mp200_ipu_irq_handler( int irq, void *dev_id, struct pt_regs *fp )
{
    unsigned long flags;
    spin_lock_irqsave(&ipu_lock, flags);         /* prohibit interrupts  */
    mp200_ipu_irq_handler_sub(irq, dev_id);
    spin_unlock_irqrestore(&ipu_lock, flags);    /* prohibit interrupts  */
}
#endif


/*****************************************************************************
* MODULE   : mp200_ipu_irq_handler_sub
* FUNCTION : IPU interrupt handler
*            analyze the cause of IPU interrupt and execute each operations
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/07/06 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_irq_handler_sub( int irq, void *dev_id )
{
    struct mp200_ipu_dev *dev = mp200_ipu_devices;
    int status_tmp;
#ifdef CONFIG_MP200_IPU_DUAL
    struct mp200_ipu_task *p_tmp;
#endif
    int    tmp_hw_func = 0;

    switch (irq)
    {
        case INT_IMG:
            dbg_GetStopTime(1, "IMG");
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_irq_handler() IMG <start>\n");
#ifdef CONFIG_MP200_IPU_DUAL
            p_tmp = list_entry(index_img.next, struct mp200_ipu_task, req_list);
            dbg_printk((_DEBUG_IPU & 0x80), "mp200_irq_handler(): sequence(%ld) <IMG>\n", p_tmp->request->ipu_req.sequence);
            status_tmp = mp200_ipu_req_finalize_img(p_tmp->request);
#else
            status_tmp = mp200_ipu_req_finalize_img(request_img);
            tmp_hw_func = request_img->ipu_req.ipu_hw_func;
#endif

            break;
        case INT_ROT:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_irq_handler() ROT <start>\n");
            dbg_GetStopTime(1, "ROT");
#ifdef CONFIG_MP200_IPU_DUAL
            p_tmp = list_entry(index_rot.next, struct mp200_ipu_task, req_list);
            dbg_printk((_DEBUG_IPU & 0x80), "mp200_irq_handler(): sequence(%ld) <ROT>\n", p_tmp->request->ipu_req.sequence);
            status_tmp = mp200_ipu_req_finalize_rot(p_tmp->request);
#else
            status_tmp = mp200_ipu_req_finalize_rot(request_rot);
            tmp_hw_func = request_rot->ipu_req.ipu_hw_func;
#endif
            break;
        case INT_IPU_DMA:
            dbg_GetStopTime(1, "GDMA");
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_irq_handler() GDMA <start>\n");
#ifdef CONFIG_MP200_IPU_DUAL
            p_tmp = list_entry(index_gdma.next, struct mp200_ipu_task, req_list);
            dbg_printk((_DEBUG_IPU & 0x80), "mp200_irq_handler(): sequence(%ld) <GDMA>\n", p_tmp->request->ipu_req.sequence);
            status_tmp = mp200_ipu_req_finalize_gdma(p_tmp->request);
#else
            status_tmp = mp200_ipu_req_finalize_gdma(request_gdma);
            tmp_hw_func = request_gdma->ipu_req.ipu_hw_func;
#endif
            break;
        default:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_irq_handler() --- <return>\n");
            status_tmp = 0;
            return;
    }

#ifdef CONFIG_MP200_IPU_DUAL
    tmp_hw_func = p_tmp->request->ipu_req.ipu_hw_func;
#endif

    /* request running flag initialize and release buffer */
#ifdef CONFIG_MP200_IPU_DUAL
    mp200_ipu_release_buf(p_tmp->request);
    mp200_ipu_reqqueue_del(p_tmp);
#else
    switch (irq)
    {
        case INT_IMG:
            mp200_ipu_release_buf(request_img);
            request_img = NULL;
            break;
        case INT_ROT:
            mp200_ipu_release_buf(request_rot);
            request_rot = NULL;
            break;
        case INT_IPU_DMA:
            mp200_ipu_release_buf(request_gdma);
            request_gdma = NULL;
            break;
    }
#endif

    if (IPU_REQ_leftcall.callback)
    /* with callback*/
    {
        dbg_GetStopTime(2, "callback");
        if ((IPU_REQ_leftcall.callback == (void *)mp200_ipu_callback_img) ||
            (IPU_REQ_leftcall.callback == (void *)mp200_ipu_callback_rot) ||
            (IPU_REQ_leftcall.callback == (void *)mp200_ipu_callback_gdma))
        {
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() data = 0x%lx\n", IPU_REQ_leftcall.data);
            (IPU_REQ_leftcall.callback)(IPU_REQ_leftcall.data, IPU_REQ_leftcall.flag);
        }
        else
        {
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() sequence = %ld\n", IPU_REQ_leftcall.sequence);
            (IPU_REQ_leftcall.callback)(IPU_REQ_leftcall.sequence, IPU_REQ_leftcall.flag);
        }

        IPU_REQ_leftcall.callback = NULL;
    }

    Call_StatusCtrlFunc_OFF();

    /* start to run request in IPU Request's queue */
    switch (tmp_hw_func)
    {
        case IPU_HW_FUNC_IMG:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(rot)\n");
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(gdma)\n");
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(img)\n");
            wake_up_interruptible(&dev->wait_busy_rot);
            wake_up_interruptible(&dev->wait_busy_gdma);
            wake_up_interruptible(&dev->wait_busy_img);
            break;
        case IPU_HW_FUNC_ROT:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(gdma)\n");
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(img)\n");
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(rot)\n");
            wake_up_interruptible(&dev->wait_busy_gdma);
            wake_up_interruptible(&dev->wait_busy_img);
            wake_up_interruptible(&dev->wait_busy_rot);
            break;
        case IPU_HW_FUNC_GDMA:
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(img)\n");
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(rot)\n");
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() wake_up(gdma)\n");
            wake_up_interruptible(&dev->wait_busy_img);
            wake_up_interruptible(&dev->wait_busy_rot);
            wake_up_interruptible(&dev->wait_busy_gdma);
            break;
    }

    switch (irq)
    {
    case INT_IMG:     dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() IMG <END>\n");  break;
    case INT_ROT:     dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() ROT <END>\n");  break;
    case INT_IPU_DMA: dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_irq_handler() GDMA <END>\n"); break;
    }
}


/*****************************************************************************
* MODULE   : mp200_ipu_req_finalize
* FUNCTION : finalize the request without callback
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
static void mp200_ipu_req_finalize( struct mp200_ipu_req_regist *p, int *ret )
{
    int flag_tmp = ret[0];
    int status_tmp = ret[1];

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_req_finalize() <start>\n");

    if(p){
        char *cHW = "";
        switch(p->ipu_req.ipu_hw_func){
        case IPU_HW_FUNC_IMG:   cHW = "IMG"; break;
        case IPU_HW_FUNC_ROT:   cHW = "ROT"; break;
        case IPU_HW_FUNC_GDMA:  cHW = "GDMA"; break;
        }
        dbg_printk((_DEBUG_IPU & 0x01), "  hw(%s) callback_flag(%d) callback(%p)\n", cHW, p->callback_flag, p->ipu_req.callback);
    }
    if ((p != NULL) &&
        (((p->ipu_req.ipu_hw_func == IPU_HW_FUNC_IMG)   && (p->callback_flag == TRUE)) ||
         ((p->ipu_req.ipu_hw_func == IPU_HW_FUNC_ROT)   && (p->callback_flag == TRUE)) ||
         ((p->ipu_req.ipu_hw_func == IPU_HW_FUNC_GDMA)  && (p->callback_flag == TRUE))))
    {
        /* callback is required */
        if ((flag_tmp == IPU_CALLBACK_SUCCESS) &&
            (p->ipu_req.status == IPU_REQ_STATUS_WAIT_CANCEL))
        {
            flag_tmp = IPU_CALLBACK_CANCEL;
        }

        /* update status */
        p->ipu_req.status = status_tmp;

        /* callback */
        IPU_REQ_leftcall.callback    = p->ipu_req.callback;
        IPU_REQ_leftcall.sequence    = p->ipu_req.sequence;
        IPU_REQ_leftcall.data        = p->ipu_req.data;
        IPU_REQ_leftcall.ipu_hw_func = p->ipu_req.ipu_hw_func;
        IPU_REQ_leftcall.flag        = flag_tmp;

        /* remove from queue */
        mp200_ipu_remove_queue(p->ipu_req.sequence, p->ipu_req.ipu_hw_func);
    }
    return;
}


/*****************************************************************************
* MODULE   : mp200_ipu_req_finalize_img
* FUNCTION : finalize the request without callback(IMG)
* RETURN   : status of request
              success : IPU_REQ_STATUS_DONE
              fail    : IPU_REQ_STATUS_ERROR
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_req_finalize_img( struct mp200_ipu_req_regist *p )
{
    int ret[2] = {IPU_CALLBACK_SUCCESS, IPU_REQ_STATUS_DONE};

    // check Interuppt Status
    unsigned long ulRegVal32 = readl( IMGMmioV + IMG_STATUS );

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_req_finalize_img() <start>\n");

    /* interrupt by ERROR of inner bus I/F and KERNEL */
    /* interrupt by RETRY/SPLIT of inner bus I/F */
    if( (ulRegVal32 & IMG_ERROR_BIT)      == IMG_ERROR_BIT      ||
        (ulRegVal32 & IMG_FLT_BIT)        == IMG_FLT_BIT        ||
        (ulRegVal32 & IMG_STOP_BIT)       == IMG_STOP_BIT )
    {

#ifdef CONFIG_MP200_IPU_DUAL
        /* remove next request */
        mp200_ipu_reqqueue_del_next_sequence(request_img, p->ipu_req.sequence);
#endif

        /* Reset IMG */
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMG1);
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMG2);
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMGPCLK);
        mp200_pmu_reset_device(MP200_RESETDEVICE_IPUIMG);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_1_CLK);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_2_CLK);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_PCLK);
        /* Unreset IMG */
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_IMG_1_CLK);
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_IMG_2_CLK);
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_IMG_PCLK);
        mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUIMG);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUIMG1);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUIMG2);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUIMGPCLK);

        /* Error Message */
        if((ulRegVal32 & IMG_ERROR_BIT) == IMG_ERROR_BIT)
            printk( KERN_INFO " @ipu: IMG AHB ERROR. Reset IMG.\n" );
        else if((ulRegVal32 & IMG_FLT_BIT) == IMG_FLT_BIT)
            printk( KERN_INFO " @ipu: IMG KERNEL ACCESS_ERROR. Reset IMG.\n" );
        else if((ulRegVal32 & IMG_STOP_BIT) == IMG_STOP_BIT)
            printk( KERN_INFO " @ipu: IMG AHB RETRY/SPLIT. Reset IMG.\n" );

        if((ulRegVal32 & IMG_STOP_BIT) == IMG_STOP_BIT)
        {
            ret[0] = IPU_CALLBACK_IPU_STOP;                 /* flag of callback  */
        }
        else
        {
            ret[0] = IPU_CALLBACK_IPU_ERROR;                /* flag of callback  */
        }
        ret[1] = IPU_REQ_STATUS_ERROR;                      /* status of request */
    }
    else if((ulRegVal32 & IMG_STAT_BIT) == IMG_STAT_BIT )
    /* interrupt that IMG process is completed */
    {
        dbg_printk((_DEBUG_IPU & 0x02), "interrupt that IMG process is completed\n");
#ifdef CONFIG_MP200_IPU_DUAL
        mp200_ipu_hw_int_ffclr(IPU_HW_FUNC_IMG);                /* Interuppt Enable Clear and Interuppt Status Clear */
#else
        mp200_ipu_hw_int_disable(IPU_HW_FUNC_IMG);              /* Interuppt Enable Clear and Interuppt Status Clear */
#endif
        ret[0] = IPU_CALLBACK_SUCCESS;                          /* flag of callback  */
        ret[1] = IPU_REQ_STATUS_DONE;                           /* status of request */
    }
    mp200_ipu_req_finalize(p, ret);

    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_req_finalize_img() <end>\n");
    return ret[1];
}


/*****************************************************************************
* MODULE   : mp200_ipu_req_finalize_rot
* FUNCTION : finalize the request without callback(ROT)
* RETURN   : status of request
              success : IPU_REQ_STATUS_DONE
              fail    : IPU_REQ_STATUS_ERROR
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_req_finalize_rot( struct mp200_ipu_req_regist *p )
{
    int ret[2] = {IPU_CALLBACK_SUCCESS, IPU_REQ_STATUS_DONE};

    // check Interuppt Status
    unsigned long ulRegVal32 = readl( ROTMmioV + ROT_INTSTATUS );

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_req_finalize_rot() <start>\n");
    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_req_finalize_rot() *p(%p)\n", p);

    if( (ulRegVal32 & ROT_DMAERR_BIT)     == ROT_DMAERR_BIT     ||
        (ulRegVal32 & ROT_DMASTOP_BIT)    == ROT_DMASTOP_BIT)
    /* interrupt by ERROR of inner bus I/F       */
    /* interrupt by RETRY/SPLIT of inner bus I/F */
    {

#ifdef CONFIG_MP200_IPU_DUAL
        /* remove next request */
        mp200_ipu_reqqueue_del_next_sequence(request_rot, p->ipu_req.sequence);
#endif

        /* Reset ROT */
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUROT);
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUROTPCLK);
        mp200_pmu_reset_device(MP200_RESETDEVICE_IPUROT);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_ROT_CLK);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_ROT_PCLK);
        /* Unreset ROT */
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_ROT_CLK);
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_ROT_PCLK);
        mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUROT);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUROT);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUROTPCLK);

        /* Error Message */
        if((ulRegVal32 & ROT_DMAERR_BIT) == ROT_DMAERR_BIT)
            printk( KERN_INFO " @ipu: ROT AHB ERROR. Reset ROT.\n" );
        else if( (ulRegVal32 & ROT_DMASTOP_BIT) == ROT_DMASTOP_BIT )
            printk( KERN_INFO " @ipu: ROT AHB RETRY/SPLIT. Reset ROT.\n" );

        if( (ulRegVal32 & ROT_DMASTOP_BIT) == ROT_DMASTOP_BIT )
        {
            ret[0] = IPU_CALLBACK_IPU_STOP;                 /* flag of callback  */
        }
        else
        {
            ret[0] = IPU_CALLBACK_IPU_ERROR;                /* flag of callback  */
        }
        ret[1] = IPU_REQ_STATUS_ERROR;                      /* status of request */
    }
    else if( (ulRegVal32 & ROT_ROTEND_BIT) == ROT_ROTEND_BIT )
    /* interrupt that ROT process is completed */
    {
        dbg_printk((_DEBUG_IPU & 0x02), "interrupt that ROT process is completed\n");
#ifdef CONFIG_MP200_IPU_DUAL
        mp200_ipu_hw_int_ffclr(IPU_HW_FUNC_ROT);            /* Interuppt Enable Clear and Interuppt Status Clear */
#else
        mp200_ipu_hw_int_disable(IPU_HW_FUNC_ROT);          /* Interuppt Enable Clear and Interuppt Status Clear */
#endif
        ret[0] = IPU_CALLBACK_SUCCESS;                          /* flag of callback  */
        ret[1] = IPU_REQ_STATUS_DONE;                           /* status of request */
    }
    mp200_ipu_req_finalize(p, ret);

    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_req_finalize_rot() <end>\n");
    return ret[1];
}


/*****************************************************************************
* MODULE   : mp200_ipu_req_finalize_gdma
* FUNCTION : finalize the request without callback(GDMA)
* RETURN   : status of request
              success : IPU_REQ_STATUS_DONE
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
static int mp200_ipu_req_finalize_gdma( struct mp200_ipu_req_regist *p )
{
    int ret[2] = {IPU_CALLBACK_SUCCESS, IPU_REQ_STATUS_DONE};

    // check Interuppt Status
    unsigned long ulRegVal32 = readl( GDMAMmioV + DMA_STATUS );

    dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_req_finalize_gdma() <start>\n");
    if((ulRegVal32 & DMA_AHBERR_BIT)     == DMA_AHBERR_BIT )
    /* interrupt by ERROR of inner bus I/F */
    {

#ifdef CONFIG_MP200_IPU_DUAL
        /* remove next request */
        mp200_ipu_reqqueue_del_next_sequence(request_gdma, p->ipu_req.sequence);
#endif

        /* Reset GDMA */
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUDMA);
        mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUDMACLK);
        mp200_pmu_reset_device(MP200_RESETDEVICE_IPUDMA);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_IPUDMA_CLK);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_IPUDMA_PCLK);
        /* Unreset GDMA */
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_IPUDMA_CLK);
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_IPUDMA_PCLK);
        mp200_pmu_unreset_device(MP200_RESETDEVICE_IPUDMA);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUDMA);
        mp200_pmu_clkctrl_on(MP200_CLKCTRL_IPUDMACLK);

        /* Error Message */
        if((ulRegVal32 & DMA_AHBERR_BIT) == DMA_AHBERR_BIT)
            printk( KERN_INFO " @ipu: DMA AHB ERROR. Reset DMA.\n" );

        ret[0] = IPU_CALLBACK_IPU_ERROR;                        /* flag of callback  */
        ret[1] = IPU_REQ_STATUS_ERROR;                          /* status of request */
    }
    else if( (ulRegVal32 & DMA_STAT_BIT) == DMA_STAT_BIT )
    /* interrupt that GDMA process is completed */
    {
        dbg_printk((_DEBUG_IPU & 0x02), "interrupt that GDMA process is completed\n");
#ifdef CONFIG_MP200_IPU_DUAL
        mp200_ipu_hw_int_ffclr(IPU_HW_FUNC_GDMA);               /* Interuppt Enable Clear and Interuppt Status Clear */
#else
        mp200_ipu_hw_int_disable(IPU_HW_FUNC_GDMA);             /* Interuppt Enable Clear and Interuppt Status Clear */
#endif
        ret[0] = IPU_CALLBACK_SUCCESS;                          /* flag of callback  */
        ret[1] = IPU_REQ_STATUS_DONE;                           /* status of request */
    }
    mp200_ipu_req_finalize(p, ret);

    dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_req_finalize_gdma() <end>\n");
    return ret[1];
}


#if defined(CONFIG_PM) || defined(CONFIG_DPM)
/* ------------------------------------------- */
/*   DPM functions                             */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_suspend
* FUNCTION : callback function when to suspend
* RETURN   : 0
* NOTE     : none
* CREATE   : 2006/10/05 M.Sasaki
* UPDATE   :
******************************************************************************/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21) // kernel 2.6.21 later
static int  mp200_ipu_suspend(struct platform_device *dev, pm_message_t state)
{
    if (state.event == DEV_SUSPEND_IDLE_1)
    {
        if(ctrl_func) return -EBUSY;
        else          return 0;
    }
    else /* state.event == PM_EVENT_SUSPEND */
    {
//      if (mp200_sleep_while_idle) ;
//      else                        ;
        return 0;
    }
}


/*****************************************************************************
* MODULE   : mp200_ipu_suspend_set_hwreq_flag
* FUNCTION : search the IPU request from queues(sub)
* RETURN   : NULL    : target request does not exist
*          : pointer : pointer to the top of queue containing target request
* NOTE     : none
* CREATE   : 2006/11/21 M.Haguro
* UPDATE   :
******************************************************************************/
#ifdef CONFIG_MP200_IPU_DUAL
static int mp200_ipu_suspend_set_hwreq_flg(struct list_head *list_hw)
{
    struct mp200_ipu_req_regist *request_hw;
    struct list_head            *list_queue;
    int ret = 0;

    list_for_each(list_queue, list_hw){
        request_hw = list_entry(list_queue, struct mp200_ipu_req_regist, ipu_req_regist);
#else

static int mp200_ipu_suspend_set_hwreq_flg(struct mp200_ipu_req_regist *request_hw)
{
    int ret = 0;
#endif /* CONFIG_MP200_IPU_DUAL */
        if(request_hw->hw_req_flag == SET_HW_REQ){
            request_hw->hw_req_flag = DONE_HW_REQ;
            ret = 1;
        }
#ifdef CONFIG_MP200_IPU_DUAL
    }
#endif /* CONFIG_MP200_IPU_DUAL */

    return ret;
}


/*****************************************************************************
* MODULE   : mp200_ipu_suspend_late
* FUNCTION : callback function when to suspend_late
* RETURN   : 0
* NOTE     : none
* CREATE   : 2007/11/20 M.Haguro
* UPDATE   :
******************************************************************************/
static int  mp200_ipu_suspend_late(struct platform_device *dev, pm_message_t state)
{
    int i = 0;
    int ret_i, ret_r, ret_g;
    unsigned long ulRegVal32_i = 0, ulRegVal32_r = 0, ulRegVal32_g = 0, flags;
#ifdef CONFIG_MP200_IPU_DUAL
    struct mp200_ipu_queue      *pQueue = mp200_ipu_que;
#endif /* CONFIG_MP200_IPU_DUAL */

    if (!mp200_sleep_while_idle)
    {
        DPM_suspend_flg = 1;
        spin_lock_irqsave(&ipu_lock, flags);               /* disable interrupts */

#ifdef CONFIG_MP200_IPU_DUAL
        ret_i = mp200_ipu_suspend_set_hwreq_flg(&pQueue->ipu_queue_img);
        ret_r = mp200_ipu_suspend_set_hwreq_flg(&pQueue->ipu_queue_rot);
        ret_g = mp200_ipu_suspend_set_hwreq_flg(&pQueue->ipu_queue_gdma);
#else
        ret_i = mp200_ipu_suspend_set_hwreq_flg(request_img);
        ret_r = mp200_ipu_suspend_set_hwreq_flg(request_rot);
        ret_g = mp200_ipu_suspend_set_hwreq_flg(request_gdma);
#endif /* CONFIG_MP200_IPU_DUAL */

        do{
            if(ret_i) ulRegVal32_i = readl(IMGMmioV  + IMG_ACK);
            if(ret_r) ulRegVal32_r = readl(ROTMmioV  + ROT_STATUS);
            if(ret_g) ulRegVal32_g = readl(GDMAMmioV + DMA_ACK);
            if(ulRegVal32_i || ulRegVal32_r || ulRegVal32_g) mdelay(16);
            else break;
            i++;
        }while(i < 5);

        spin_unlock_irqrestore(&ipu_lock, flags);          /* enable interrupts */
    }
    return 0;
}
#else // kernel 2.6.21 before
static int  mp200_ipu_suspend(struct device *dev, u32 state, u32 level)
{
    switch (level)
    {
        case SUSPEND_DISABLE:
            break;
        case SUSPEND_SAVE_STATE:
            if(ctrl_func) mdelay(20);
            break;
        case SUSPEND_POWER_DOWN:
            break;
        default:
            break;
    }
    return 0;
}
#endif


/*****************************************************************************
* MODULE   : mp200_ipu_resume
* FUNCTION : callback function when to resume
* RETURN   : 0
* NOTE     : none
* CREATE   : 2006/10/05 M.Sasaki
* UPDATE   :
******************************************************************************/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21) // kernel 2.6.21 later
static int  mp200_ipu_resume(struct platform_device *dev)
{
    if (DPM_suspend_flg == 1)
    {
        wake_up_interruptible(&mp200_ipu_devices->wait_busy_img);   /* start to run request in IMG Request's queue */
        wake_up_interruptible(&mp200_ipu_devices->wait_busy_rot);   /* start to run request in ROT Request's queue */
        wake_up_interruptible(&mp200_ipu_devices->wait_busy_gdma);  /* start to run request in GDMA Request's queue */
        tasklet_schedule(&ipu_nexttask); /* start the next request */
    }
    return 0;
}


/*****************************************************************************
* MODULE   : mp200_ipu_resume_early
* FUNCTION : callback function when to resume
* RETURN   : 0
* NOTE     : none
* CREATE   : 2006/11/21 M.Sasaki
* UPDATE   :
******************************************************************************/
static int  mp200_ipu_resume_early(struct platform_device *dev)
{
    return 0;
}
#else // kernel 2.6.21 before
static int  mp200_ipu_resume(struct device *dev, u32 level)
{
    switch (level){
    case RESUME_POWER_ON:
        wake_up_interruptible(&mp200_ipu_devices->wait_busy_img);   /* start to run request in IMG Request's queue */
        wake_up_interruptible(&mp200_ipu_devices->wait_busy_rot);   /* start to run request in ROT Request's queue */
        wake_up_interruptible(&mp200_ipu_devices->wait_busy_gdma);  /* start to run request in GDMA Request's queue */
        break;
    case RESUME_RESTORE_STATE:
        break;
    case RESUME_ENABLE:
        tasklet_schedule(&ipu_nexttask); /* start the next request */
        break;
    default:
        break;
    }
    return 0;
}
#endif


/*****************************************************************************
* MODULE   : mp200_ipu_resume_kick
* FUNCTION : start the next request
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/11/21 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_resume_kick(unsigned long num)
{
#ifdef CONFIG_MP200_IPU_DUAL
    struct mp200_ipu_queue *pQueue = mp200_ipu_que;
    mp200_ipu_resume_kick_next(&pQueue->ipu_queue_img,  INT_IMG);
    mp200_ipu_resume_kick_next(&pQueue->ipu_queue_rot,  INT_ROT);
    mp200_ipu_resume_kick_next(&pQueue->ipu_queue_gdma, INT_IPU_DMA);
#else
    mp200_ipu_resume_kick_next(request_img,  INT_IMG);
    mp200_ipu_resume_kick_next(request_rot,  INT_ROT);
    mp200_ipu_resume_kick_next(request_gdma, INT_IPU_DMA);
#endif
}


/*****************************************************************************
* MODULE   : mp200_ipu_resume_kick_next
* FUNCTION : start the next request
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/11/21 M.Haguro
* UPDATE   :
******************************************************************************/
#ifdef CONFIG_MP200_IPU_DUAL
static void mp200_ipu_resume_kick_next(struct list_head *list_hw, int irq)
{
    struct mp200_ipu_req_regist *request_hw;
    struct list_head            *list_queue;
    int    tmp_hw_func;

    list_for_each(list_queue, list_hw){
        request_hw  = list_entry(list_queue, struct mp200_ipu_req_regist, ipu_req_regist);
#else

static void mp200_ipu_resume_kick_next(struct mp200_ipu_req_regist *request_hw, int irq)
{
    int    tmp_hw_func;
#endif /* CONFIG_MP200_IPU_DUAL */
        if(request_hw->hw_req_flag == DONE_HW_REQ){
            tmp_hw_func = request_hw->ipu_req.ipu_hw_func;
            mp200_ipu_resume_kick_finalize((void *)request_hw, irq);
        }
#ifdef CONFIG_MP200_IPU_DUAL
    }
#endif /* CONFIG_MP200_IPU_DUAL */
}


/*****************************************************************************
* MODULE   : mp200_ipu_resume_kick_finalize
* FUNCTION : start the next request
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/11/21 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_resume_kick_finalize(void *task, int irq)
{
    int ret[2]     = {IPU_CALLBACK_SUCCESS, IPU_REQ_STATUS_DONE};
    int status_tmp = ret[1];

#ifdef CONFIG_MP200_IPU_DUAL
    struct mp200_ipu_task *p_tmp = (struct mp200_ipu_task *)task;
    int    tmp_hw_func = p_tmp->request->ipu_req.ipu_hw_func;
#else
    struct mp200_ipu_req_regist *p_tmp = (struct mp200_ipu_req_regist *)task;
    int    tmp_hw_func = p_tmp->ipu_req.ipu_hw_func;
#endif /* CONFIG_MP200_IPU_DUAL */

    switch (irq)
    {
        case INT_IMG:
            dbg_GetStopTime(1, "IMG");
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_resume_kick_finalize() IMG <start>\n");
#ifdef CONFIG_MP200_IPU_DUAL
            dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_resume_kick_finalize(): sequence(%ld) <IMG>\n", p_tmp->request->ipu_req.sequence);
            mp200_ipu_req_finalize(p_tmp->request, ret);
#else
            mp200_ipu_req_finalize(p_tmp, ret);
#endif
            break;
        case INT_ROT:
            dbg_GetStopTime(1, "ROT");
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_resume_kick_finalize() ROT <start>\n");
#ifdef CONFIG_MP200_IPU_DUAL
            dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_resume_kick_finalize(): sequence(%ld) <ROT>\n", p_tmp->request->ipu_req.sequence);
            mp200_ipu_req_finalize(p_tmp->request, ret);
#else
            mp200_ipu_req_finalize(p_tmp, ret);
#endif
            break;
        case INT_IPU_DMA:
            dbg_GetStopTime(1, "GDMA");
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_resume_kick_finalize() GDMA <start>\n");
#ifdef CONFIG_MP200_IPU_DUAL
            dbg_printk((_DEBUG_IPU & 0x80), "mp200_ipu_resume_kick_finalize(): sequence(%ld) <GDMA>\n", p_tmp->request->ipu_req.sequence);
            mp200_ipu_req_finalize(p_tmp->request, ret);
#else
            mp200_ipu_req_finalize(p_tmp, ret);
#endif
            break;
        default:
            dbg_printk((_DEBUG_IPU & 0x01), "mp200_ipu_resume_kick_finalize() --- <return>\n");
            status_tmp = 0;
            return;
    }

    /* request running flag initialize and release buffer */
#ifdef CONFIG_MP200_IPU_DUAL
    mp200_ipu_release_buf(p_tmp->request);
    mp200_ipu_reqqueue_del(p_tmp);
#else
    mp200_ipu_release_buf(p_tmp);
    p_tmp = NULL;
#endif /* CONFIG_MP200_IPU_DUAL */

    Call_StatusCtrlFunc_OFF();

    if(IPU_REQ_leftcall.callback){
        /* with callback*/
        dbg_GetStopTime(2, "callback");
        if ((IPU_REQ_leftcall.callback == (void *)mp200_ipu_callback_img) ||
            (IPU_REQ_leftcall.callback == (void *)mp200_ipu_callback_rot) ||
            (IPU_REQ_leftcall.callback == (void *)mp200_ipu_callback_gdma))
        {
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_resume_kick() data = 0x%lx\n", IPU_REQ_leftcall.data);
            (IPU_REQ_leftcall.callback)(IPU_REQ_leftcall.data, IPU_REQ_leftcall.flag);
        }
        else{
            dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_resume_kick() sequence = %ld\n", IPU_REQ_leftcall.sequence);
            (IPU_REQ_leftcall.callback)(IPU_REQ_leftcall.sequence, IPU_REQ_leftcall.flag);
        }
        memset(&IPU_REQ_leftcall, 0, sizeof(struct mp200_ipu_req_leftcall));
    }

    mp200_ipu_resume_kick_wakeup(tmp_hw_func);

    switch(irq){
    case INT_IMG:     dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_resume_kick_finalize() IMG <END>\n");  break;
    case INT_ROT:     dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_resume_kick_finalize() ROT <END>\n");  break;
    case INT_IPU_DMA: dbg_printk((_DEBUG_IPU & 0x02), "mp200_ipu_resume_kick_finalize() GDMA <END>\n"); break;
    }
    return;
}


/*****************************************************************************
* MODULE   : mp200_ipu_resume_kick_wakeup
* FUNCTION : start the next request
* RETURN   : none
* NOTE     : none
* CREATE   : 2007/11/21 M.Haguro
* UPDATE   :
******************************************************************************/
static void mp200_ipu_resume_kick_wakeup(int hw_func)
{
    struct mp200_ipu_dev *dev = mp200_ipu_devices;

    /* start to run request in IPU Request's queue */
    switch (hw_func)
    {
        case IPU_HW_FUNC_IMG:
            wake_up_interruptible(&dev->wait_busy_rot);
            wake_up_interruptible(&dev->wait_busy_gdma);
            wake_up_interruptible(&dev->wait_busy_img);
            break;
        case IPU_HW_FUNC_ROT:
            wake_up_interruptible(&dev->wait_busy_gdma);
            wake_up_interruptible(&dev->wait_busy_img);
            wake_up_interruptible(&dev->wait_busy_rot);
            break;
        case IPU_HW_FUNC_GDMA:
            wake_up_interruptible(&dev->wait_busy_img);
            wake_up_interruptible(&dev->wait_busy_rot);
            wake_up_interruptible(&dev->wait_busy_gdma);
            break;
    }
}
#endif /* CONFIG_PM || CONFIG_DPM */


/*****************************************************************************
* MODULE   : mp200_ipu_init_module
* FUNCTION : initialize module
* RETURN   :        0 : success
*          : negative : fail
* NOTE     : none
* CREATE   : 2006/10/05 M.Sasaki
* UPDATE   :
******************************************************************************/
int mp200_ipu_init_module( void )
{
    int result;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,00) /* Linux 2.6.00 later */
    if(dev_major){
        t_dev  = MKDEV(dev_major, 0);
        result = register_chrdev_region(t_dev, 1, dev_name);
    }else{
        result = alloc_chrdev_region(&t_dev, 0, 1, dev_name);
        dev_major  = MAJOR(t_dev);
    }
    if(result < 0){
        printk(KERN_ERR "mp200_ipu: can't get major number.\n");
        goto fail_get_major;
    }

    cdev_init(&mp200_ipu_cdev, &mp200_ipu_fops);
    mp200_ipu_cdev.owner = THIS_MODULE;
    result = cdev_add(&mp200_ipu_cdev, t_dev, 1);
    if(result){
        goto fail_cdev_add;
    }

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21) /* Linux 2.6.21 later */
    mp200_ipu_class = class_create(THIS_MODULE, dev_name);
    if(IS_ERR(mp200_ipu_class)){
        printk(KERN_ERR "mp200_ipu: Unable to create ipu class.\n");
        result = PTR_ERR(mp200_ipu_class);
        goto fail_class_create;
    }

    mp200_ipu_class_dev = class_device_create(mp200_ipu_class, NULL, t_dev, NULL, "ipu");
    if(IS_ERR(mp200_ipu_class_dev)){
        printk(KERN_ERR "mp200_ipu: Unable to create class_device.\n");
        result = PTR_ERR(mp200_ipu_class_dev);
        goto fail_class_device_create;
    }
#else /* Linux 2.6.00 later */
    mp200_ipu_class = class_simple_create(THIS_MODULE, dev_name);
    if(IS_ERR(mp200_ipu_class)){
        printk(KERN_ERR "mp200_ipu: Unable to create ipu class.\n");
        result = PTR_ERR(mp200_ipu_class);
        goto fail_class_simple_create;
    }

    mp200_ipu_class_dev = class_simple_device_add(mp200_ipu_class, t_dev, NULL, dev_name);
    if(IS_ERR(mp200_ipu_class_dev)){
        printk(KERN_ERR "mp200_ipu: Unable to create ipu class device.\n");
        result = PTR_ERR(mp200_ipu_class_dev);
        goto fail_class_simple_device_add;
    }

    result = devfs_mk_cdev(t_dev, S_IFCHR | S_IRUGO | S_IWUGO, dev_name);
    if(result){
        printk(KERN_ERR "mp200_ipu: Unable to make cdev.\n");
        goto fail_devfs_mk_cdev;
    }
#endif
#else /* Linux 2.6.00 before */
    /* Set up owner pointers.*/
    SET_MODULE_OWNER(&mp200_ipu_fops);

    /* Register the device */
    result = register_chrdev(dev_major, dev_name, &mp200_ipu_fops);
    if (result < 0)
    {
        printk(KERN_ERR "mp200_ipu: device registration error\n");
        goto fail_get_major;
    }

    if (dev_major == 0) dev_major = result;                 /* dynamic */
#endif

    /* initialize ipu driver */
    result = mp200_ipu_init();
    if(result < 0)
    {
        printk(KERN_ERR "mp200_ipu: ipu driver initialize <failed>\n");
        goto fail_ipu_init;
    }

    printk(KERN_INFO "mp200_ipu: registered device ipu [ipu]\n");
    goto success;

fail_ipu_init:
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21) /* Linux 2.6.21 later */
    class_device_unregister(mp200_ipu_class_dev);
fail_class_device_create:
    class_destroy(mp200_ipu_class);
fail_class_create:
    cdev_del(&mp200_ipu_cdev);
fail_cdev_add:
    unregister_chrdev(dev_major, dev_name);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,00) /* Linux 2.6.00 later */
    devfs_remove(dev_name);
fail_devfs_mk_cdev:
    class_simple_device_remove(t_dev);
fail_class_simple_device_add:
    class_simple_destroy(mp200_ipu_class);
fail_class_simple_create:
    cdev_del(&mp200_ipu_cdev);
fail_cdev_add:
    unregister_chrdev_region(t_dev, 1);
#else /* Linux 2.6.00 before */
    unregister_chrdev(dev_major, dev_name);
#endif
fail_get_major:
success:

    return result;
}


#ifdef MODULE
/*****************************************************************************
* MODULE   : mp200_ipu_cleanup_module
* FUNCTION : cleanup module
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/05 M.Sasaki
* UPDATE   :
******************************************************************************/
void mp200_ipu_cleanup_module( void )
{

    thread_img_flag = 0;               /* thread exit IMG  */
    thread_rot_flag = 0;               /* thread exit ROT  */
    thread_gdma_flag = 0;              /* thread exit GDMA */

    mp200_ipu_hw_int_disable(IPU_HW_FUNC_ALL);
    free_irq(INT_IMG, NULL);
    free_irq(INT_ROT, NULL);
    free_irq(INT_IPU_DMA, NULL);

    /* Reset IMG */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMG1);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMG2);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUIMGPCLK);
    mp200_pmu_reset_device(MP200_RESETDEVICE_IPUIMG);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_1_CLK);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_2_CLK);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_PCLK);

    /* Reset ROT */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUROT);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUROTPCLK);
    mp200_pmu_reset_device(MP200_RESETDEVICE_IPUROT);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_ROT_CLK);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_ROT_PCLK);

    /* Reset GDMA */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUDMA);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUDMACLK);
    mp200_pmu_reset_device(MP200_RESETDEVICE_IPUDMA);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_IPUDMA_CLK);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_IPUDMA_PCLK);

    /* Reset IPUAHB */
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_IPUAHB);
    mp200_pmu_reset_device(MP200_RESETDEVICE_IPUAHB);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_IPUAHB_CLK);

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
    platform_driver_unregister(&mp200_ipu_driver);
    platform_device_unregister(&mp200_ipu_device);
#endif /* CONFIG_PM || CONFIG_DPM */

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21) /* Linux 2.6.21 later */
    class_device_unregister(mp200_ipu_class_dev);
    class_destroy(mp200_ipu_class);
    cdev_del(&mp200_ipu_cdev);
    unregister_chrdev(dev_major, dev_name);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,00) /* Linux 2.6.00 later */
    devfs_remove(dev_name);
    class_simple_device_remove(t_dev);
    class_simple_destroy(mp200_ipu_class);
    cdev_del(&mp200_ipu_cdev);
    unregister_chrdev_region(t_dev, 1);
#else /* Linux 2.6.00 before */
    unregister_chrdev(dev_major, dev_name);
#endif

    kfree(mp200_ipu_devices);
    kfree(mp200_ipu_req_bufs);
    kfree(mp200_ipu_que);

#if _DEBUG_IPU_LOG
    iounmap(IPUSmemV);
    IPUSmemV = NULL;
#endif
}


module_init(mp200_ipu_init_module);
module_exit(mp200_ipu_cleanup_module);
#else
__initcall(mp200_ipu_init_module);
#endif

EXPORT_SYMBOL( mp200_ipu_regist );
EXPORT_SYMBOL( mp200_ipu_cancel );


MODULE_AUTHOR("NEC Electronics Corporation.");
MODULE_DESCRIPTION("IPU control driver");
MODULE_LICENSE("GPL");

