/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /drivers/ipu/mp200_ipu.h
*  Function        : IPU Driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/

#ifndef _MP200_IPU_H_
#define _MP200_IPU_H_

#define _IPU_COLCONV 1 /* 1:enable  0:disable */
#define _IPU_RT_THREAD 1 /* 1:enable  0:disable */

#ifndef TRUE
#define TRUE 1
#endif /* TRUE */
#ifndef FALSE
#define FALSE 0
#endif /* FALSE */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>  /* using /proc file system */
#include <linux/errno.h>
#include <linux/string.h>

#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) /* Linux 2.6.00 later */
#include <linux/cdev.h>
#else
#include <linux/wrapper.h>
#include <linux/tqueue.h>
#endif
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <asm/uaccess.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/timer.h>
#include <linux/sched.h>

#include <asm/semaphore.h>
#include <linux/completion.h>
#include <linux/wait.h>
#include <linux/list.h>
#if _IPU_RT_THREAD
#include <linux/sched.h>
#endif

/* to provide ioctl */
#include <asm/ioctl.h>
#include <asm/types.h>
#include <asm/io.h>


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20) /* Linux 2.6.20 later */
#include <linux/platform_device.h>
#endif

/* platform related */
#include <asm/arch/ipu.h>
#include <asm/arch/ipu_user.h>

#include <asm/arch/irqs.h>        //#define INT_IMG     22
                                  //#define INT_ROT     23
                                  //#define INT_IPU_DMA 37
#include <asm/arch/pmu.h>

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20) /* Linux 2.6.20 later */
#include <linux/freezer.h>
#include <asm/arch/pm.h>
#else // Lunux 2.6.0 before
#include <linux/device.h>
#include <asm/arch/bus.h>
#include <linux/pm.h>
#endif
#endif /* CONFIG_PM || CONFIG_DPM */




#define _DEBUG_IPU_LOG   0 /* 1:enable  0:disable */

#define _IPU_PERF        0 /* 0: disable
                              1: enable IMG/ROT/GDMA
                              2: enable regist - callback
                              3: enable ioctl  - return
                              4: enable DMA
                            */




/****************************************************************************/
/* File information structure                                               */
/****************************************************************************/
typedef struct mp200_ipu_dev_info {
    int                  flag;          /* status of device information structure    */
                                        /*   FALSE(0):usable TRUE(1):used            */
    int vmas;                           /* active mappings                           */
    struct semaphore     sem_vm;        /* exclusive resource (for mmap)             */
    struct semaphore     sem;           /* exclusive resource                        */
    wait_queue_head_t    wait_que;      /* wait queue                                */
    int                  sleep_mode;    /* sleep status                              */
                                        /*   FALSE(0):wakeup TRUE(1):sleep           */
    int                  status;        /* status of IPU request process             */
                                        /*   IPU_STATUS_DONE(0)     : done           */
                                        /*   IPU_STATUS_WAIT(1)     : running        */
                                        /*   IPU_STATUS_NO_REQ(2)   : no request     */
    struct mp200_ipu_req req;           /* IPU request structure                     */
                                        /*   sequence:request sequence no            */
                                        /*   status:status of IPU request            */
                                        /*   callback:callback function              */
                                        /*   data:optional data                      */
                                        /*   ipu_hw_func:request H/W                 */
                                        /*   ipu_reg_info:IPU register setting info  */
}MP200_IPU_DEV_INFO;


/****************************************************************************/
/* Device information structure                                             */
/****************************************************************************/
typedef struct mp200_ipu_dev {
    wait_queue_head_t    wait_idle_img;  /* wait queue img(idle)            */
    wait_queue_head_t    wait_idle_rot;  /* wait queue rot(idle)            */
    wait_queue_head_t    wait_idle_gdma; /* wait queue gdma(idle)           */
    wait_queue_head_t    wait_busy_img;  /* wait queue img(busy)            */
    wait_queue_head_t    wait_busy_rot;  /* wait queue rot(busy)            */
    wait_queue_head_t    wait_busy_gdma; /* wait queue gdma(busy)           */
    struct mp200_ipu_dev_info ipu_dev_info[];
}MP200_IPU_DEV;

#define MP200_IPU_DEVS 8              /* mp200_ipu0 through mp200_ipu  */


/*===============================================================*/
/* IPU request information queue structure                       */
/*===============================================================*/
enum hw_req {
        NO_HW_REQ,   /* 0 */
        SET_HW_REQ,  /* 1 */
        DONE_HW_REQ, /* 2 */
};

typedef struct mp200_ipu_req_regist {
     struct list_head      ipu_req_regist; /* head of IPU request information structure is secured */
     int                   callback_flag;  /*   TRUE  : callback is required                       */
                                           /*   FALSE : callback is not required                   */
     enum hw_req           hw_req_flag;    /*   NO_HW_REQ   : REQ(HW) is not set                   */
                                           /*   SET_HW_REQ  : REQ(HW) is set                       */
                                           /*   DONE_HW_REQ : REQ(HW) is complete                  */
#ifdef CONFIG_MP200_IPU_DUAL
     int                   request_flag;   /*   TRUE  : request is set                             */
                                           /*   FALSE : request is not set                         */
#endif
     struct mp200_ipu_req  ipu_req;        /* IPU request information                              */
}MP200_IPU_REQ_REGIST;

#ifdef CONFIG_MP200_IPU_DUAL
#define MAX_SIZE_MP200_IPU_REQ_REGIST (sizeof(struct list_head) + sizeof(int) + sizeof(enum hw_req) + sizeof(int) + (MAX_SIZE_MP200_IPU_REQ))
#else
#define MAX_SIZE_MP200_IPU_REQ_REGIST (sizeof(struct list_head) + sizeof(int) + sizeof(enum hw_req) + (MAX_SIZE_MP200_IPU_REQ))
#endif


/*===============================================================*/
/* IPU queue structure                                           */
/*===============================================================*/
#define IPU_BUFS_MAX 32                          /* max number of IPU request queue structure */
                                                 /* setting informations */

typedef struct mp200_ipu_queue {
     unsigned long               sequence;       /* sequence No                                          */
     struct list_head            ipu_queue_img;  /* head of IPU request information structure is secured */
     struct list_head            ipu_queue_rot;  /* head of IPU request information structure is secured */
     struct list_head            ipu_queue_gdma; /* head of IPU request information structure is secured */
     struct mp200_ipu_req_regist *bufs[IPU_BUFS_MAX];
                                                 /* IPU request information structure Size               */
}MP200_IPU_QUEUE;

#define SIZE_MP200_IPU_QUEUE (28 + 4 * IPU_BUFS_MAX)


/****************************************************************************/
/* debug option                                                             */
/****************************************************************************/
/*===============================================================*/
/* performance measurement                                       */
/*===============================================================*/
#if _DEBUG_IPU_LOG > 0
#ifdef _DEBUG_IPU
       ulong *sleep_status;
#else
extern ulong *sleep_status;
#endif

#define SLEEP_1    0x00000001
#define SLEEP_2    0x00000002
#define SLEEP_3    0x00000004
#define SLEEP_4    0x00000008
#define SLEEP_5    0x00000010
#define SLEEP_6    0x00000020
#define SLEEP_7    0x00000040
#define SLEEP_8    0x00000080
#define SLEEP_9    0x00000100
#define SLEEP_10   0x00000200
#define SLEEP_11   0x00000400
#define SLEEP_12   0x00000800
#define SLEEP_13   0x00001000
#define SLEEP_14   0x00002000
#define SLEEP_15   0x00004000
#define SLEEP_16   0x00008000
#define WAIT_DMA_1 0x00010000
#define WAIT_DMA_2 0x00020000

#define dbg_SetSleepStatus(FLG) *sleep_status |= FLG;
#define dbg_ClrSleepStatus(FLG) *sleep_status &= ~FLG;
#else
#define dbg_SetSleepStatus(FLG) ;
#define dbg_ClrSleepStatus(FLG) ;
#endif


/*===============================================================*/
/* performance measurement                                       */
/*===============================================================*/
#if _IPU_PERF > 0
#ifdef _DEBUG_IPU
       struct timeval  tv_start, tv_stop;
       struct timezone tz;
unsigned long   delta;
#else
extern struct timeval  tv_start, tv_stop;
extern struct timezone tz;
extern unsigned long   delta;
#endif

#define dbg_CalcTime(num)                                                         \
	do{                                                                       \
		if(tv_stop.tv_sec > tv_start.tv_sec){                             \
			delta = (tv_stop.tv_sec - tv_start.tv_sec) * 1000 * 1000; \
			if(tv_stop.tv_usec > tv_start.tv_usec){                   \
				delta += (tv_stop.tv_usec - tv_start.tv_usec);    \
			}else{	delta -= (tv_start.tv_usec - tv_stop.tv_usec);    \
			}                                                         \
		}else if(tv_stop.tv_sec == tv_start.tv_sec){                      \
			if(tv_stop.tv_usec > tv_start.tv_usec){                   \
				delta = (tv_stop.tv_usec -tv_start.tv_usec);      \
			}else{	delta = 0;                                        \
			}                                                         \
		}else{	delta = 0;                                                \
		}                                                                 \
		printk(KERN_DEBUG " @ipu: " num ": ->cycle(%ld.%03ld)msec  ->start(%ld)  ->end(%ld)\n", delta / 1000, delta % 1000, tv_start.tv_usec, tv_stop.tv_usec);\
	}while(0);

#define dbg_GetStartTime(level)  \
	if(_IPU_PERF == level){ do_gettimeofday(&tv_start); }
#define dbg_GetStopTime(level, num)        \
	if(_IPU_PERF == level){            \
		do_gettimeofday(&tv_stop); \
		dbg_CalcTime(num);         \
	}
#else
#define dbg_GetStartTime(level)      ;
#define dbg_GetStopTime(level, num)  ;
#endif /* _IPU_PERF */



#endif /* _MP200_IPU_H_ */

