/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /drivers/ipu/mp200_ipu_user.c
*  Function        : IPU Driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                           */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                  */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                  */
/*-----------+------+------------------+-------------------+--------------------------------------------------*/


#define _DEBUG_IPU_AP    0x00 /* 00008401(bit) */
                              /* 1: debug function in
                               * 4: debug mmap
                               * 8: debug function out
                               */
#if _DEBUG_IPU_AP > 0
#define dbg_printk(level, fmt, arg...) \
	if(level > 0) printk(KERN_DEBUG " @ipu: " fmt, ## arg);
#else
#define dbg_printk(level, fmt, arg...) \
	;
#endif

#include "mp200_ipu.h"


/****************************************************************************/
/* Prototype declarations                                                   */
/****************************************************************************/
/*===============================================================*/
/* external function prottype                                    */
/*===============================================================*/
extern int mp200_ipu_regist( struct mp200_ipu_req *p );
extern int mp200_ipu_cancel( unsigned long sequence, int flag );

int  mp200_ipu_open( struct inode *inode, struct file *file );
int  mp200_ipu_close( struct inode *inode, struct file *file );
int  mp200_ipu_ioctl( struct inode *inode, struct file *file, unsigned int request, unsigned long arg );
int  mp200_ipu_mmap( struct file *file, struct vm_area_struct * vma );

/*===============================================================*/
/* callback function                                             */
/*===============================================================*/
void mp200_ipu_callback_img(unsigned long data, int flag);
void mp200_ipu_callback_rot(unsigned long data, int flag);
void mp200_ipu_callback_gdma(unsigned long data, int flag);


/****************************************************************************/
/* Flags                                                                    */
/****************************************************************************/
/* ------------------------------------------- */
/*   external flags                            */
/* ------------------------------------------- */
extern MP200_IPU_DEV        *mp200_ipu_devices;    /* allocated in mp200_ipu_init   */
extern int                  mp200_ipu_devs;        /* number of bare IPU devices    */


#include "mp200_img_convert.h"
#include "mp200_rot_rotation.h"


/* ------------------------------------------- */
/*   Callback function                         */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_callback_img
* FUNCTION : Callback function for IPUIO_MP200_IPU_REGIST
*            (register IPU request by User Process)
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
void mp200_ipu_callback_img(unsigned long data, int flag)
{
    MP200_IPU_DEV_INFO *dev_info; /* device information */

    dbg_printk((_DEBUG_IPU_AP & 0x1), "mp200_ipu_callback_img() <start>\n");
    dbg_printk((_DEBUG_IPU_AP), "mp200_ipu_callback_img() data = 0x%lx\n", data);

    /* get pointer to device information structure */
    dev_info = (MP200_IPU_DEV_INFO *)data;

    /* update status of IPU request process to IPU_STATUS_DONE */
    dev_info->status = IPU_STATUS_DONE;

    /* update status of IPU request */
    dev_info->req.status = flag;

    if (dev_info->sleep_mode == TRUE)
    /* wake up from IPU_STATUS_WAIT */
        wake_up_interruptible(&dev_info->wait_que);

    dbg_printk((_DEBUG_IPU_AP & 0x8), "mp200_ipu_callback_img() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_callback_rot
* FUNCTION : Callback function for IPUIO_MP200_IPU_REGIST
*            (register IPU request by User Process)
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
void mp200_ipu_callback_rot(unsigned long data, int flag)
{
    MP200_IPU_DEV_INFO *dev_info; /* device information */

    dbg_printk((_DEBUG_IPU_AP & 0x1), "mp200_ipu_callback_rot() <start>\n");

    /* get pointer to device information structure */
    dev_info = (MP200_IPU_DEV_INFO *)data;

    /* update status of IPU request process to IPU_STATUS_DONE */
    dev_info->status = IPU_STATUS_DONE;

    /* update status of IPU request */
    dev_info->req.status = flag;

    if (dev_info->sleep_mode == TRUE)
    /* wake up from IPU_STATUS_WAIT */
        wake_up_interruptible(&dev_info->wait_que);

    dbg_printk((_DEBUG_IPU_AP & 0x8), "mp200_ipu_callback_rot() <end>\n");
}


/*****************************************************************************
* MODULE   : mp200_ipu_callback_gdma
* FUNCTION : Callback function for IPUIO_MP200_IPU_REGIST
*            (register IPU request by User Process)
* RETURN   : none
* NOTE     : none
* CREATE   : 2006/10/03 M.Sasaki
* UPDATE   :
******************************************************************************/
void mp200_ipu_callback_gdma(unsigned long data, int flag)
{
    MP200_IPU_DEV_INFO *dev_info; /* device information */

    dbg_printk((_DEBUG_IPU_AP & 0x1), "mp200_ipu_callback_gdma() <start>\n");

    /* get pointer to device information structure */
    dev_info = (MP200_IPU_DEV_INFO *)data;

    /* update status of IPU request process to IPU_STATUS_DONE */
    dev_info->status = IPU_STATUS_DONE;

    /* update status of IPU request */
    dev_info->req.status = flag;

    if (dev_info->sleep_mode == TRUE)
    /* wake up from IPU_STATUS_WAIT */
        wake_up_interruptible(&dev_info->wait_que);

    dbg_printk((_DEBUG_IPU_AP & 0x8), "mp200_ipu_callback_gdma() <end>\n");
}


/* ------------------------------------------- */
/*   Module Entries                            */
/* ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_ipu_open
* FUNCTION : open IPU device
* RETURN   :       0 : success
*          : -ENODEV : fail
* NOTE     : none
* CREATE   : 2006/10/04 M.Sasaki
* UPDATE   :
******************************************************************************/
int mp200_ipu_open( struct inode *inode, struct file *file )
{
    MP200_IPU_DEV *dev = NULL;                  /* device information */
    MP200_IPU_DEV_INFO *dev_info = NULL;        /* device information */
    size_t size_dev_info;
    int i;

    dbg_printk((_DEBUG_IPU_AP & 0x1), " @ipu: mp200_ipu_open() <start>");
    dbg_printk((_DEBUG_IPU_AP & 0x1), "         file->f_version     : %lu \n",file->f_version);
    dbg_printk((_DEBUG_IPU_AP & 0x1), "         MINOR(inode->i_rdev): %d \n",MINOR(inode->i_rdev));

    size_dev_info = sizeof (MP200_IPU_DEV_INFO) + sizeof(MP200_IPU_REG_INFO) * IPU_REG_INFO_MAX;
    dev = mp200_ipu_devices;
    dev_info = dev->ipu_dev_info;
    for (i = 0; i < mp200_ipu_devs; i++)
    {
        if (dev_info->flag == FALSE) /* search unused device information structure */
        {
            break;
        }
        dev_info = (MP200_IPU_DEV_INFO *)((char *)dev_info + size_dev_info);
    }
    if (i >= mp200_ipu_devs) return -ENODEV;

    memset(dev_info, 0, size_dev_info);
    dev_info->flag = TRUE;           /* set used */

    sema_init(&dev_info->sem, 1);
    sema_init(&dev_info->sem_vm, 1);
    init_waitqueue_head(&dev_info->wait_que);
    dev_info->sleep_mode = FALSE;
    dev_info->status = IPU_STATUS_NO_REQ;

    /* and use file->private_data to point to the device data */
    file->private_data = dev_info;
    return 0;
}


/*****************************************************************************
* MODULE   : mp200_ipu_close
* FUNCTION : close IPU device
* RETURN   : 0 :
* NOTE     : none
* CREATE   : 2006/10/04 M.Sasaki
* UPDATE   :
******************************************************************************/
int mp200_ipu_close( struct inode *inode, struct file *file )
{
    MP200_IPU_DEV_INFO *dev_info = file->private_data;

    dbg_printk((_DEBUG_IPU_AP & 0x1), " @ipu: mp200_ipu_close() <start>");
    dbg_printk((_DEBUG_IPU_AP & 0x1), "         file->f_version     : %lu \n",file->f_version);

    dev_info->flag = FALSE;                     /* set unused */
    return 0;
}


/*****************************************************************************
* MODULE   : mp200_ipu_ioctl
* FUNCTION : IOCTL main function. analyze IOCTL request and execute each operations.
* RETURN   : positive : status
*            negative : fail
* NOTE     : none
* CREATE   : 2006/10/05 M.Sasaki
* UPDATE   : 2008/03/13 M.Sasaki
******************************************************************************/
int mp200_ipu_ioctl( struct inode *inode, struct file *file, unsigned int request, unsigned long arg )
{
    int   ret = -1;
    void *alloc_obj;
    size_t request_size;

    signed char buf_data[MAX_SIZE_IPUIOCTL];    /* max size :                */
                                                /* _IPUIO_MP200_IPU_REGIST   */
                                                /* _IPUIO_MP200_IPU_CALLBACK */
                                                /* _IPUIO_MP200_IPU_CANCEL   */
                                                /* _IMGIO_CONVERT            */
                                                /* _ROTIO_ROTATION           */
    MP200_IPU_DEV_INFO *dev_info = file->private_data;

    dbg_printk((_DEBUG_IPU_AP & 0x1), " @ipu: mp200_ipu_ioctl() <requset = 0x%x>\n", request);

    switch(request)
    {
        case IPUIO_MP200_IPU_REGIST:
            dbg_GetStartTime(3);
            alloc_obj = &buf_data[0];
            if( copy_from_user( alloc_obj, (void *)arg, sizeof(_IPUIO_MP200_IPU_REGIST)))
            {
                return -EFAULT;
            }

            request_size = ((_IPUIO_MP200_IPU_REGIST*)alloc_obj)->request_length;
            if (request_size > (sizeof(MP200_IPU_REQ) + sizeof(MP200_IPU_REG_INFO) * IPU_REG_INFO_MAX))
            {
                printk(KERN_INFO " @ipu: mp200_ipu_ioctl() Error! request_length is too big. <request_length = %d>\n", request_size);
                return -EINVAL;
            }

            if (dev_info->status != IPU_STATUS_NO_REQ)
            {
                return -EBUSY;
            }

            if (down_interruptible (&dev_info->sem))        /* get semafore */
            {
                return -ERESTARTSYS;
            }

            /* copy IPU request information from user memory area */
            if(copy_from_user(&dev_info->req, ((_IPUIO_MP200_IPU_REGIST*)alloc_obj)->request_arg, request_size))
            {
                up(&dev_info->sem);                     /* release semafore */
                return -EFAULT;
            }

            /* set pointer to Device information into dev_info->req.data */
            dev_info->req.data = (unsigned long)dev_info;

            /* set callback function into dev->req.callback */
            switch(dev_info->req.ipu_hw_func)
            {
                case IPU_HW_FUNC_IMG:
                    dev_info->req.callback = (void *)mp200_ipu_callback_img;
                    break;
                case IPU_HW_FUNC_ROT:
                    dev_info->req.callback = (void *)mp200_ipu_callback_rot;
                    break;
                case IPU_HW_FUNC_GDMA:
                    dev_info->req.callback = (void *)mp200_ipu_callback_gdma;
                    break;
                default:
                    printk(KERN_INFO " @ipu: mp200_ipu_ioctl() Error! arg->request_arg->ipu_hw_func(%d) is incorrect.\n", dev_info->req.ipu_hw_func);
                    up(&dev_info->sem);                     /* release semafore */
                    return -EINVAL;
            }

            /* update status of IPU request process */
            dev_info->status = IPU_STATUS_WAIT;             /* running */

            /* regist IPU request */
            ret = mp200_ipu_regist( &dev_info->req );
            if (ret != IPU_REQ_REGISTERED)           /* Registration failure */
            {
                dev_info->status = IPU_STATUS_NO_REQ;
                up(&dev_info->sem);                     /* release semafore */
                break;
            }

            /* copy dev_info->req.sequence */
            if(copy_to_user(((_IPUIO_MP200_IPU_REGIST*)alloc_obj)->request_arg, &dev_info->req, sizeof(MP200_IPU_REQ)))
            {
                up(&dev_info->sem);                     /* release semafore */
                return -EFAULT;
            }

            if(file->f_flags & O_NONBLOCK)                  /* non block */
            {
                ret = IPU_REQ_REGISTERED;
            }
            else                                            /* block until IPU process is completed */
            {
                dev_info->sleep_mode = TRUE;
                wait_event_interruptible(dev_info->wait_que, (dev_info->status != IPU_STATUS_WAIT)); /* wait until IPU process is completed */
                dev_info->sleep_mode = FALSE;

                if (dev_info->status == IPU_STATUS_DONE)    /* wake up by IPU_STATUS_DONE */
                {
                    /* set result of Write request information structure */
                    ((_IPUIO_MP200_IPU_REGIST*)alloc_obj)->result_data = dev_info->req.status;
                    if(copy_to_user( &((_IPUIO_MP200_IPU_REGIST*)arg)->result_data,
                                  &((_IPUIO_MP200_IPU_REGIST*)alloc_obj)->result_data,
                                  sizeof(((_IPUIO_MP200_IPU_REGIST*)arg)->result_data) ))
                    {
                        up(&dev_info->sem);                     /* release semafore */
                        return -EFAULT;
                    }

                    /* update status of IPU request process */
                    dev_info->status = IPU_STATUS_NO_REQ;   /* target request does not exist */

                    up(&dev_info->sem);                     /* release semafore */
                    ret = IPU_REQ_COMPLETE;
                }
                else if (dev_info->status == IPU_STATUS_CANCELLED)
                {
                    /* update status of IPU request process */
                    dev_info->status = IPU_STATUS_NO_REQ;   /* target request does not exist */

                    up(&dev_info->sem);                     /* release semeafore */
                    ret = IPU_REQ_CANCELLED;
                }
                else                                        /* wake up by signal */
                {
                    ret = IPU_REQ_REGISTERED;
                }
            }
            dbg_GetStopTime(3, "IPU_REGIST");
            break;
        case IPUIO_MP200_IPU_CALLBACK:
            alloc_obj = &buf_data[0];
            switch(dev_info->status)
            {
                case IPU_STATUS_DONE:                            /* IPU operation is done */

                    /* update status of IPU request process */
                    dev_info->status = IPU_STATUS_NO_REQ;        /* target request does not exist */

                    up(&dev_info->sem);                          /* release semafore */
                    ret = IPU_STATUS_DONE;
                    break;

                case IPU_STATUS_WAIT:                            /* waiting (executing operation) */
                    if(file->f_flags & O_NONBLOCK)               /* non block */
                    {
                        ret = IPU_STATUS_WAIT;
                    }
                    else                                         /* block until IPU operation is finished */
                    {
                        dev_info->sleep_mode = TRUE;
                        wait_event_interruptible(dev_info->wait_que, (dev_info->status != IPU_STATUS_WAIT)); /* waiting (executing operaion) */
                        dev_info->sleep_mode = FALSE;

                        if (dev_info->status == IPU_STATUS_DONE) /* wake up by IPU_STATUS_DONE */
                        {
                            /* update status of IPU request process */
                            dev_info->status = IPU_STATUS_NO_REQ;/* target request does not exist */

                            up(&dev_info->sem);                  /* release semafore */
                            ret = IPU_STATUS_DONE;
                        }
                        else if (dev_info->status == IPU_STATUS_CANCELLED)
                        {
                            /* update status of IPU request process */
                            dev_info->status = IPU_STATUS_NO_REQ;/* target request does not exist */

                            up(&dev_info->sem);                  /* release semafore */
                            ret = IPU_STATUS_CANCELLED;
                        }
                        else                                     /* wake up by signal */
                        {
                            ret = IPU_STATUS_WAIT;
                        }
                    }
                    break;

                case IPU_STATUS_NO_REQ:                         /* target request does not exist */
                    ret = IPU_STATUS_NO_REQ;
                    break;

                default:
                    break;
            }
            if ((ret == IPU_STATUS_DONE) || (ret == IPU_STATUS_WAIT))
            {
                /* set result of Write request information structure */
                ((_IPUIO_MP200_IPU_CALLBACK*)alloc_obj)->result_data = dev_info->req.status;
                if(copy_to_user( &((_IPUIO_MP200_IPU_CALLBACK*)arg)->result_data,
                              &((_IPUIO_MP200_IPU_CALLBACK*)alloc_obj)->result_data,
                              sizeof(((_IPUIO_MP200_IPU_CALLBACK*)arg)->result_data) ))
                {
                    return -EFAULT;
                }
            }
            break;

        case IPUIO_MP200_IPU_CANCEL:
            alloc_obj = &buf_data[0];
            switch(dev_info->status)
            {
                case IPU_STATUS_DONE:                           /* IPU process is completed */
                    ret = IPU_CANCEL_DONE;

                    /* update status of IPU request process */
                    dev_info->status = IPU_STATUS_NO_REQ;       /* target request does not exist */

                    up(&dev_info->sem);                         /* release semafore */
                    break;

                case IPU_STATUS_WAIT:                           /* IPU process is running (already registered) */
                    ret = mp200_ipu_cancel( dev_info->req.sequence, ((_IPUIO_MP200_IPU_CANCEL*)alloc_obj)->flag );

                    if (ret == IPU_CANCEL_DONE)
                    {
                        if (dev_info->sleep_mode == TRUE)
                        {
                            /* update status of IPU request process */
                            dev_info->status = IPU_STATUS_CANCELLED; /* target request is cancelled */

                            wake_up_interruptible(&dev_info->wait_que);
                        }
                        else
                        {
                        /* update status of IPU request process */
                        dev_info->status = IPU_STATUS_NO_REQ;   /* target request does not exist */

                        up(&dev_info->sem);                     /* release semafore */
                        }
                    }
                    break;

                case IPU_STATUS_NO_REQ:                         /* target request does not exist */
                    ret = IPU_CANCEL_NO_REQ;
                    break;
            }
            if ((ret == IPU_STATUS_DONE) || (ret == IPU_STATUS_WAIT))
            {
                /* set result of Write request information structure */
                ((_IPUIO_MP200_IPU_CANCEL*)alloc_obj)->result_data = dev_info->req.status;
                if(copy_to_user( &((_IPUIO_MP200_IPU_CANCEL*)arg)->result_data,
                              &((_IPUIO_MP200_IPU_CANCEL*)alloc_obj)->result_data,
                              sizeof(((_IPUIO_MP200_IPU_CANCEL*)arg)->result_data) ))
                {
                    return -EFAULT;
                }
            }
            break;

        case IMGIO_CONVERT:
            dbg_GetStartTime(3);
            alloc_obj = &buf_data[0];
            if( copy_from_user( alloc_obj, (void *)arg, sizeof(_IMGIO_CONVERT) ) )
            {
                return -EFAULT;
            }
            ret = mp200_img_convert(dev_info, (_IMGIO_CONVERT *)alloc_obj);
            dbg_GetStopTime(3, "IMGIO_CONVERT");
            break;

        case ROTIO_ROTATION:
            dbg_GetStartTime(3);
            alloc_obj = &buf_data[0];
            if( copy_from_user( alloc_obj, (void *)arg, sizeof(_ROTIO_ROTATION) ) )
            {
                return -EFAULT;
            }
            ret = mp200_rot_rotation(dev_info, (_ROTIO_ROTATION *)alloc_obj);
            dbg_GetStopTime(3, "ROTIO_ROTATION");
            break;

        default:
            ret = -EINVAL;
            break;
    }
    return ret;
}


/*****************************************************************************
* MODULE   : mp200_ipu_mmap
* FUNCTION : mmap main function.
*            allocate I/O buffer and map it to user area
*              with physical address    :
*                map target memory area to user area
*              without physical address :
*                allocate free area of RAM and map it to user area
* INPUT    : physical address (or 0)
*          : range (byte)
* RETURN   : user area virtual address
* NOTE     : none
* CREATE   : 2006/10/06 M.Sasaki
* UPDATE   : 2007/03/12 Y.Utsuhara
******************************************************************************/
int mp200_ipu_mmap( struct file *file, struct vm_area_struct * vma )
{
    unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;
//  unsigned long size = vma->vm_end - vma->vm_start;

    dbg_printk((_DEBUG_IPU_AP & 0x4), " @ipu: mp200_ipu_mmap() <start>\n");
    dbg_printk((_DEBUG_IPU_AP & 0x4), "         vma->vm_start: 0x%x\n", (unsigned int)vma->vm_start);
    dbg_printk((_DEBUG_IPU_AP & 0x4), "         vma->vm_end  : 0x%x\n", (unsigned int)vma->vm_end);
    dbg_printk((_DEBUG_IPU_AP & 0x4), "         offset       : 0x%x\n", (unsigned int)offset);

    /*
     * Accessing memory above the top the kernel knows about or
     * through a file pointer that was marked O_SYNC will be
     * done non-cached.
     */
    //if (offset >= __pa(high_memory) || (file->f_flags & O_SYNC))
    vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);

    /* Don't try to swap out physical pages.. */
    vma->vm_flags |= VM_RESERVED;

    /*
     * Dump addresses that are real memory to a core file.
     */
    if (offset >= __pa(high_memory) || (file->f_flags & O_SYNC))
        vma->vm_flags |= VM_IO;

    if (remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff, vma->vm_end-vma->vm_start, vma->vm_page_prot))
        return -EAGAIN;

    dbg_printk((_DEBUG_IPU_AP & 0x4), " @ipu: mp200_ipu_mmap() <end>\n");
    return 0;
}

