/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /drivers/ipu/mp200_rot_rotation.h
*  Function        : IPU Driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/


#ifndef _MP200_ROT_ROTATION_H_
#define _MP200_ROT_ROTATION_H_


/****************************************************************************/
/* Macro                                                                    */
/****************************************************************************/
#define RotRequest(DATA, REG)                        \
do{                                                  \
	pReq->ipu_reg_info[index_req].reg  = (REG);  \
	pReq->ipu_reg_info[index_req].data = (DATA); \
	index_req++;                                 \
}while(0)


/****************************************************************************/
/* Flags                                                                    */
/****************************************************************************/
/* ------------------------------------------- */
/*   internal flags                            */
/* ------------------------------------------- */
static unsigned long rot_mode;
static unsigned long ulSrcStarAddrY, ulSrcStarAddrUV, ulSrcStarAddrV;
static unsigned long ulDstStarAddrY, ulDstStarAddrUV, ulDstStarAddrV;
static unsigned long ulSrcPitch, ulDstPitch;
static unsigned long rot_src_width, rot_src_height;
static unsigned long in_bytelane, out_bytelane;
static unsigned long rot_format_in, rot_format_out, rot_format;


/* ------------------------------------------- */
/*   definition                                */
/* ------------------------------------------- */
#define MAX_ROT_SCREEN_WIDTH      0x3FFC
#define MAX_ROT_SCREEN_WIDTH_ES1x 0xFFC


/****************************************************************************/
/* Prototype declarations                                                   */
/****************************************************************************/
/* ------------------------------------------- */
/*   Private functions                         */
/* ------------------------------------------- */
static int mp200_rot_rotation( MP200_IPU_DEV_INFO *dev_info, _ROTIO_ROTATION *p );
static int mp200_rot_mkdata(_ROTIO_ROTATION *p);
static int mp200_rot_mkreq(struct mp200_ipu_req *pReq, int index);


/*****************************************************************************
* MODULE   : mp200_rot_rotation
* FUNCTION : ioctl ROTIO_ROTATION sub function. rotate image with IPU.
* RETURN   : ROT_COMPLETE(0) : Success
*            -EINVAL         : Fail (parameter error)
*            -EBUSY          : Fail (ROT busy)
*            -EIO            : Fail (ROT error, or rotation is not completed)
*            -ERESTARTSYS    : Fail (device busy)
* NOTE     : none
* CREATE   : 2006/10/05 M.Sasaki
* UPDATE   : 2008/03/13 M.Sasaki
******************************************************************************/
static int mp200_rot_rotation( MP200_IPU_DEV_INFO *dev_info, _ROTIO_ROTATION *p )
{
    struct mp200_ipu_req *pReq;
    int ret;

    /* make ROT data. */
    if(mp200_rot_mkdata(p) == FALSE) return -EINVAL;

    if (dev_info->status != IPU_STATUS_NO_REQ)
    {
        return -EBUSY;
    }

    if (down_interruptible (&dev_info->sem))            /* get semafore */
    {
        return -ERESTARTSYS;
    }

    /* set ROT data to register. */
    pReq = &dev_info->req;
    mp200_rot_mkreq(pReq, 0);

    /* set pointer to Device information into dev_info->req.data */
    pReq->data = (unsigned long)dev_info;

    /* set callback function into dev_info->req.callback */
    pReq->callback = (void *)mp200_ipu_callback_rot;

    /* set hw_func */
    pReq->ipu_hw_func = IPU_HW_FUNC_ROT;

    /* update status of IPU request process */
    dev_info->status = IPU_STATUS_WAIT;  /* running */

    ret = mp200_ipu_regist(pReq);        /* register request (reserve)*/
    if (ret != IPU_REQ_REGISTERED)
    {
        dev_info->status = IPU_STATUS_NO_REQ;           /* target request does not exist */
        up(&dev_info->sem);                             /* release semafore */
        return ret;
    }

wait_rot:
    dev_info->sleep_mode = TRUE;
    wait_event_interruptible(dev_info->wait_que, (dev_info->status != IPU_STATUS_WAIT)); /* wait until IPU process is completed */
    dev_info->sleep_mode = FALSE;

    switch (dev_info->status)
    {
        case IPU_STATUS_DONE:                           /* wake up by IPU_STATUS_DONE */
            switch (dev_info->req.status)
            {
                case IPU_CALLBACK_SUCCESS:
                    ret = ROT_COMPLETE;
                    break;

                case IPU_CALLBACK_IPU_ERROR:
                case IPU_CALLBACK_IPU_STOP:
                    printk( KERN_INFO " @ipu: ROTIO_ROTATION: ROT Error! callback flag = %d\n", dev_info->req.status);
                    ret = -EIO;
                    break;

                case IPU_CALLBACK_SET_PARAM:
                case IPU_CALLBACK_CANCEL:
                case IPU_CALLBACK_FAIL:
                default:
                    printk( KERN_INFO " @ipu: ROTIO_ROTATION: rotation is not completed! callback flag = %d\n", dev_info->req.status);
                    ret = -EIO;
                    break;
            }
            break;

        case IPU_STATUS_CANCELLED:                      /* wake up by IPU_STATUS_CANCELLED */
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: rotation process is cancelled!\n");
            ret = -EIO;
            break;

        case IPU_STATUS_WAIT:                           /* wake up by signal */
        default:
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: wake up by signal. retry to wait until ROT process is completed\n");
            goto wait_rot;
            break;
    }
    /* update status of IPU request process */
    dev_info->status = IPU_STATUS_NO_REQ;               /* target request does not exist */
    up(&dev_info->sem);                                 /* release semafore */
    return ret;
}


/*****************************************************************************
* MODULE   : mp200_rot_mkreq
* FUNCTION : make ROT request data
* IMPORT   :
* RETURN   :
* NOTE     : none
* CREATE   : 2006/11/07 M.Sasaki
* UPDATE   : 2008/03/13 M.Sasaki
******************************************************************************/
static int mp200_rot_mkreq(struct mp200_ipu_req *pReq, int index)
{
    int index_req = index;

/*******************************
    set ROT register
********************************/
    /*==============================
        ROT_MODE
    ================================*/
    RotRequest(rot_mode,                ROT_MODE);

    /*==============================
        image frame
    ================================*/
    RotRequest(ROT_SRCSEL_FRAME_A | ROT_DSTSEL_FRAME_A, ROT_FRAME);


    /*==============================
        source image
    ================================*/
    RotRequest(ulSrcPitch,              ROT_SRCSIZE);
    RotRequest(ulSrcStarAddrY,          ROT_SRCYADR_A);
    RotRequest(ulSrcStarAddrUV,         ROT_SRCUVADR_A);
    RotRequest(ulSrcStarAddrV,          ROT_SRCVADR_A);
    RotRequest(rot_src_width,           ROT_SRCHSIZE);
    RotRequest(rot_src_height,          ROT_SRCVSIZE);

    /*==============================
        destination image
    ================================*/
    RotRequest(ulDstPitch,              ROT_DSTSIZE);
    RotRequest(ulDstStarAddrY,          ROT_DSTYADR_A);
    RotRequest(ulDstStarAddrUV,         ROT_DSTUVADR_A);
    RotRequest(ulDstStarAddrV,          ROT_DSTVADR_A);

    /*==============================
        format
    ================================*/
    rot_format  = (rot_format_in  << ROT_SRCFMT_SFT);
    rot_format |= (rot_format_out << ROT_DSTFMT_SFT);
    RotRequest(rot_format,              ROT_FORMAT);

    /*==============================
        bytelane
    ================================*/
    RotRequest(in_bytelane,         ROT_SRCBYTE);
    RotRequest(out_bytelane,            ROT_DSTBYTE);

    /*==============================
        Start ROT
    ================================*/
    RotRequest(ROT_REQ_BIT,         ROT_REQ);

    /*==============================
        end mark
    ================================*/
    RotRequest(IPU_REG_INFO_END,        -1);

    return index_req;
}


/*****************************************************************************
* MODULE   : mp200_rot_mkdata
* FUNCTION : make ROT register data
* IMPORT   : _ROTIO_ROTATION
* RETURN   : TRUE  : check OK
*          : FALSE : check NG
* NOTE     : none
* CREATE   : 2006/11/07 M.Sasaki
* UPDATE   : 2008/03/13 M.Sasaki
******************************************************************************/
static int mp200_rot_mkdata(_ROTIO_ROTATION *p)
{
    int input_bpp, output_bpp;

    rot_mode       = ROT_NEWEN_BIT;

    rot_format_in    = p->input_data_format;
    rot_format_out   = p->output_data_format;

    rot_src_width    = p->input_data_width;
    rot_src_height   = p->input_data_height;
    ulSrcStarAddrY   = p->input_data_addr_y;
    ulSrcStarAddrUV  = p->input_data_addr_uv;
    ulSrcStarAddrV   = p->input_data_addr_v;
    ulDstStarAddrY   = p->output_data_addr_y;
    ulDstStarAddrUV  = p->output_data_addr_uv;
    ulDstStarAddrV   = p->output_data_addr_v;

    in_bytelane     = p->input_byte;
    out_bytelane    = p->output_byte;

/*====================*/
/* Image Format check */
/*====================*/
    if((rot_format_in != ROT_FORM_RGB888            ) &&
       (rot_format_in != ROT_FORM_RGB565            ) &&
       (rot_format_in != ROT_FORM_YUV420_PLANAR     ) &&
       (rot_format_in != ROT_FORM_YUV420_SEMI_PLANAR) &&
       (rot_format_in != ROT_FORM_YUV422_SEMI_PLANAR) &&
       (rot_format_in != ROT_FORM_YUV422_INTERLEAVE ))
    {
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_format is incorrect. intput_data_format = 0x%lx\n", rot_format_in);
    }

    if((rot_format_out != ROT_FORM_RGB888            ) &&
       (rot_format_out != ROT_FORM_RGB565            ) &&
       (rot_format_out != ROT_FORM_YUV420_PLANAR     ) &&
       (rot_format_out != ROT_FORM_YUV420_SEMI_PLANAR) &&
       (rot_format_out != ROT_FORM_YUV422_SEMI_PLANAR) &&
       (rot_format_out != ROT_FORM_YUV422_INTERLEAVE ))
    {
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_format is incorrect. output_data_format = 0x%lx\n", rot_format_out);
    }

    if (rot_format_in == ROT_FORM_YUV422_SEMI_PLANAR)
    {
        if ((rot_format_out != ROT_FORM_YUV422_SEMI_PLANAR) && (rot_format_out != ROT_FORM_YUV420_SEMI_PLANAR))
        {
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_format and output_data_format is incorrect.\n intput_data_format = 0x%lx output_data_format = 0x%lx\n",
                    rot_format_in, rot_format_out);
        }
    }
    else
    {
        if (rot_format_in != rot_format_out)
        {
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_format and output_data_format is incorrect.\n intput_data_format = 0x%lx output_data_format = 0x%lx\n",
                    rot_format_in, rot_format_out);
        }
    }

/*===============*/
/* Address check */
/*===============*/
    /* check input Y/RGB data start address by 4 byte */
    if (ulSrcStarAddrY == 0 ||
        (ulSrcStarAddrY & 0x3) != 0)
    {
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_addr_y is incorrect. input_data_addr_y = 0x%lx\n", ulSrcStarAddrY);
        return FALSE;
    }

    /* check input UV data start address by 4 byte */
    if ((rot_format_in == ROT_FORM_YUV420_PLANAR)      || (rot_format_in == ROT_FORM_YUV420_SEMI_PLANAR) ||
        (rot_format_in == ROT_FORM_YUV422_SEMI_PLANAR))
    {
        if (ulSrcStarAddrUV == 0 ||
            (ulSrcStarAddrUV & 0x3) != 0)
        {
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_addr_uv is incorrect. input_data_addr_uv = 0x%lx\n", ulSrcStarAddrUV);
            return FALSE;
        }
    }

    /* check input V data start address by 4 byte */
    if ((rot_format_in == ROT_FORM_YUV420_PLANAR))
    {
        if (ulSrcStarAddrV == 0 ||
            (ulSrcStarAddrV & 0x3) != 0)
        {
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_addr_v is incorrect. input_data_addr_v = 0x%lx\n", ulSrcStarAddrV);
            return FALSE;
        }
    }

    /* check output Y/RGB data start address by 4 byte */
    if (ulDstStarAddrY == 0 ||
        (ulDstStarAddrY  & 0x3) != 0)
    {
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_addr_y is incorrect. output_data_addr_y = 0x%lx\n", ulDstStarAddrY);
        return FALSE;
    }

    /* check output UV data start address by 4 byte */
    if ((rot_format_out == ROT_FORM_YUV420_PLANAR) || (rot_format_out == ROT_FORM_YUV420_SEMI_PLANAR) ||
        (rot_format_out == ROT_FORM_YUV422_SEMI_PLANAR))
    {
        if (ulDstStarAddrUV == 0 ||
            (ulDstStarAddrUV  & 0x3) != 0)
        {
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_addr_uv is incorrect. output_data_addr_uv = 0x%lx\n", ulDstStarAddrUV);
            return FALSE;
        }
    }

    /* check output V data start address by 4 byte */
    if ((rot_format_out == ROT_FORM_YUV420_PLANAR))
    {
        if (ulDstStarAddrV == 0 ||
            (ulDstStarAddrV & 0x3) != 0)
        {
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_addr_v is incorrect. output_data_addr_u = 0x%lx\n", ulDstStarAddrV);
            return FALSE;
        }
    }

/*==================*/
/* Image Size check */
/*==================*/
    /* source width check */
    if (rot_src_width < 8 || rot_src_width > 4092)
    {
        goto fail_size_width;
    }
    if (rot_format_in == ROT_FORM_YUV420_PLANAR)
    {
        if (rot_src_width & 0x7)
        {
            goto fail_size_width;
        }
    }
    else if (rot_format_in == ROT_FORM_YUV422_INTERLEAVE || rot_format_in == ROT_FORM_RGB565)
    {
        if (rot_src_width & 0x1)
        {
            goto fail_size_width;
        }
    }
    else
    {
        if (rot_src_width & 0x3)
        {
            goto fail_size_width;
        }
    }

    /* source height check */
    if (rot_src_height < 8 || rot_src_height > 4092)
    {
        goto fail_size_height;
    }
    if (rot_format_in == ROT_FORM_YUV420_PLANAR)
    {
        if (rot_src_height & 0x7)
        {
            goto fail_size_height;
        }
    }
    else if (rot_format_in == ROT_FORM_YUV422_INTERLEAVE || rot_format_in == ROT_FORM_RGB565)
    {
        if (rot_src_height & 0x1)
        {
            goto fail_size_height;
        }
    }
    else
    {
        if (rot_src_height & 0x3)
        {
            goto fail_size_height;
        }
    }

/*======================*/
/* Rotation Angle check */
/*======================*/
    switch (p->rotation_angle)
    {
        case ROT_ANGLE_0:
            rot_mode |= ROT_MOD_0;
            ulDstPitch = rot_src_width;
            break;
        case ROT_ANGLE_90:
            rot_mode |= ROT_MOD_90;
            ulDstPitch = rot_src_height;
            break;
        case ROT_ANGLE_180:
            rot_mode |= ROT_MOD_180;
            ulDstPitch = rot_src_width;
            break;
        case ROT_ANGLE_270:
            rot_mode |= ROT_MOD_270;
            ulDstPitch = rot_src_height;
            break;
        default:
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! rotation_angle is incorrect. rotation_angle = %d\n", p->rotation_angle);
            return FALSE;
            break;
    }

/*=============*/
/* pitch check */
/*=============*/
    /* input image pitch */
    if(rot_format_in == ROT_FORM_RGB888)
        input_bpp = 3;
    else if((rot_format_in == ROT_FORM_RGB565) || (rot_format_in == ROT_FORM_YUV422_INTERLEAVE))
        input_bpp = 2;
    else
        input_bpp = 1;
    ulSrcPitch = rot_src_width * input_bpp;

    if(p->input_data_size < ulSrcPitch || p->input_data_size > MAX_ROT_SCREEN_WIDTH){
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_size is incorrect. input_data_size = %d\n", p->input_data_size);
        return FALSE;
    }

    if(rot_format_in == ROT_FORM_YUV420_PLANAR){
        if(p->input_data_size & 0x7){
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_size is incorrect. input_data_size = %d\n", p->input_data_size);
            return FALSE;
        }
    }
    else if(rot_format_in == ROT_FORM_RGB888){
        if(p->input_data_size % 0xC){
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_size is incorrect. input_data_size = %d\n", p->input_data_size);
            return FALSE;
        }
    }
    else{
        if(p->input_data_size & 0x3){
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_data_size is incorrect. input_data_size = %d\n", p->input_data_size);
            return FALSE;
        }
    }
    ulSrcPitch = p->input_data_size;

    /* output image pitch */
    if(rot_format_out == ROT_FORM_RGB888)
        output_bpp = 3;
    else if((rot_format_out == ROT_FORM_RGB565) || (rot_format_out == ROT_FORM_YUV422_INTERLEAVE))
        output_bpp = 2;
    else
        output_bpp = 1;
    ulDstPitch *= output_bpp;

    if(p->output_data_size < ulDstPitch || p->output_data_size > MAX_ROT_SCREEN_WIDTH){
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_size is incorrect. output_data_size = %d\n", p->output_data_size);
        return FALSE;
    }
    if (rot_format_out == ROT_FORM_YUV420_PLANAR){
        if(p->output_data_size & 0x7){
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_size is incorrect. output_data_size = %d\n", p->output_data_size);
            return FALSE;
        }
    }
    else if(rot_format_out == ROT_FORM_RGB888){
        if(p->output_data_size % 0xC){
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_size is incorrect. output_data_size = %d\n", p->output_data_size);
            return FALSE;
        }
    }
    else{
        if(p->output_data_size & 0x3){
            printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_data_size is incorrect. output_data_size = %d\n", p->output_data_size);
            return FALSE;
        }
    }
    ulDstPitch = p->output_data_size;

/*================*/
/* Bytelane check */
/*================*/
    /* input_bytelane */
    if ((in_bytelane & ROT_SRCBYTE_DATA0_BIT) == ((in_bytelane & ROT_SRCBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT)||
        (in_bytelane & ROT_SRCBYTE_DATA0_BIT) == ((in_bytelane & ROT_SRCBYTE_DATA2_BIT) >> ROT_SRCBYTE_DATA2_SFT)||
        (in_bytelane & ROT_SRCBYTE_DATA0_BIT) == ((in_bytelane & ROT_SRCBYTE_DATA3_BIT) >> ROT_SRCBYTE_DATA3_SFT)||
        ((in_bytelane & ROT_SRCBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT) == ((in_bytelane & ROT_SRCBYTE_DATA2_BIT) >> ROT_SRCBYTE_DATA2_SFT)||
        ((in_bytelane & ROT_SRCBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT) == ((in_bytelane & ROT_SRCBYTE_DATA3_BIT) >> ROT_SRCBYTE_DATA3_SFT)||
        ((in_bytelane & ROT_SRCBYTE_DATA2_BIT) >> ROT_SRCBYTE_DATA2_SFT) == ((in_bytelane & ROT_SRCBYTE_DATA3_BIT) >> ROT_SRCBYTE_DATA3_SFT))
    {
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! input_byte is incorrect.\n");
        printk( KERN_INFO " input_byte ROT_SRCBYTE_DATA0 = 0x%lx\n", (in_bytelane & ROT_SRCBYTE_DATA0_BIT));
        printk( KERN_INFO " input_byte ROT_SRCBYTE_DATA1 = 0x%lx\n", (in_bytelane & ROT_SRCBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT);
        printk( KERN_INFO " input_byte ROT_SRCBYTE_DATA2 = 0x%lx\n", (in_bytelane & ROT_SRCBYTE_DATA2_BIT) >> ROT_SRCBYTE_DATA2_SFT);
        printk( KERN_INFO " input_byte ROT_SRCBYTE_DATA3 = 0x%lx\n", (in_bytelane & ROT_SRCBYTE_DATA3_BIT) >> ROT_SRCBYTE_DATA3_SFT);
       return FALSE;
    }

    /* output_bytelane */
    if ((out_bytelane & ROT_DSTBYTE_DATA0_BIT) == ((out_bytelane & ROT_DSTBYTE_DATA1_BIT)  >> ROT_SRCBYTE_DATA1_SFT)||
        (out_bytelane & ROT_DSTBYTE_DATA0_BIT) == ((out_bytelane & ROT_DSTBYTE_DATA2_BIT)  >> ROT_SRCBYTE_DATA2_SFT)||
        (out_bytelane & ROT_DSTBYTE_DATA0_BIT) == ((out_bytelane & ROT_DSTBYTE_DATA3_BIT)  >> ROT_SRCBYTE_DATA3_SFT)||
        ((out_bytelane & ROT_DSTBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT)== ((out_bytelane & ROT_DSTBYTE_DATA2_BIT)  >> ROT_SRCBYTE_DATA2_SFT)||
        ((out_bytelane & ROT_DSTBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT)== ((out_bytelane & ROT_DSTBYTE_DATA3_BIT)  >> ROT_SRCBYTE_DATA3_SFT)||
        ((out_bytelane & ROT_DSTBYTE_DATA2_BIT) >> ROT_SRCBYTE_DATA2_SFT)== ((out_bytelane & ROT_DSTBYTE_DATA3_BIT)  >> ROT_SRCBYTE_DATA3_SFT))
    {
        printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! output_byte is incorrect.\n");
        printk( KERN_INFO " output_byte ROT_DSTBYTE_DATA0 = 0x%lx\n", (out_bytelane & ROT_DSTBYTE_DATA0_BIT));
        printk( KERN_INFO " output_byte ROT_DSTBYTE_DATA1 = 0x%lx\n", (out_bytelane & ROT_DSTBYTE_DATA1_BIT) >> ROT_SRCBYTE_DATA1_SFT);
        printk( KERN_INFO " output_byte ROT_DSTBYTE_DATA2 = 0x%lx\n", (out_bytelane & ROT_DSTBYTE_DATA2_BIT) >> ROT_SRCBYTE_DATA2_SFT);
        printk( KERN_INFO " output_byte ROT_DSTBYTE_DATA3 = 0x%lx\n", (out_bytelane & ROT_DSTBYTE_DATA3_BIT) >> ROT_SRCBYTE_DATA3_SFT);
       return FALSE;
    }

    rot_mode |= (ROT_SRCENDIAN_LITTLE | ROT_DSTENDIAN_LITTLE);

    return TRUE;

fail_size_width:
    printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! src_width is incorrect. src_width = 0x%lx\n", rot_src_width);
    return FALSE;

fail_size_height:
    printk( KERN_INFO " @ipu: ROTIO_ROTATION: Error! src_height is incorrect. src_height = 0x%lx\n", rot_src_height);
    return FALSE;
}


#endif /* _MP200_ROT_ROTATION_H_ */

