/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : drivers/media/video/mp200/mp200_v4l2_core.c
 * Function        : Video for Linux driver for EM1 driver core
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>
#include <linux/ctype.h>
#include <linux/pagemap.h>
#include <linux/mm.h>
#include <linux/device.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/kthread.h>
#include <linux/fb.h>
#include <linux/videodev.h>

#include <asm/semaphore.h>
#include <asm/processor.h>
#include <asm/io.h>
#include <asm/byteorder.h>
#include <asm/irq.h>

#include <linux/platform_device.h>
#include <media/v4l2-common.h>

#include <asm/arch/ipu.h>
#include <asm/arch/rot.h>
#include <asm/arch/img.h>
#include <asm/arch/em1_mem.h>
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
#include <linux/freezer.h>
#include <asm/arch/pm.h>
#endif /* CONFIG_PM || CONFIG_DPM */
#include "../drivers/video/mp200/mp200_common.h"
#include "../drivers/video/mp200/nts/mp200_nts.h"

#define _V4L2_RT_THREAD 1 /* 1:enable  0:disable */
#if _V4L2_RT_THREAD
#include <linux/sched.h>
#endif
#include "mp200_v4l2_core.h"


/*===============================================================*/
/* v4l2 local configuration                                      */
/*===============================================================*/
#define VF_YUV422PL  0 /* input yuv422planer     */ /* 0: disable  1: enable */ /***** ROT not support *****/
#define VF_YUV422PX  0 /* input yuv422interleave */ /* 0: disable  1: enable */ /***** not care *****/
#define RESIZE_CHECK 0 /* resize scaling check   */ /* 0: disable  1: enable */ /***** not care *****/


/*===============================================================*/
/* debug parameters                                              */
/*===============================================================*/
#define _DPM_DBG   0 /* debug suspend/resume */   /* 0: disable  1: enable */
#define _LCK_DBG   0 /* debug spinlock */         /* 0: disable  1: enable */
#define _WKQ_DBG   0 /* debug workqueue */        /* 0: disable  1: enable */
#define _VBQ_DBG   0 /* debug videobuf_queue */   /* 0: disable  1: enable */
#define _ROT_DBG   0 /* debug IPU-ROT thread */   /* 0: disable  1: print  2: callback print */
#define _IMG_DBG   0 /* debug IPU-IMG thread */   /* 0: disable  1: print  2: callback print */
#define _TMR_DBG   0 /* debug timer thread */     /* 0: disable  1: enable */
#define _LCD_DBG   0 /* debug LCD thread */       /* 0: disable  1: print  2: callback print */
#define _PERF1_DBG 0 /* debug v4l2 performance */ /* 0: disable  1: enable */
#define _PERF2_DBG 0 /* debug v4l2 performance */ /* 0: disable  1: enable */
#define _TRACE_DBG 0 /* debug back trace */       /* 0: disable  1: enable */


#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_INFO DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define dbg_lock(lock, fmt, arg...) \
	if(_LCK_DBG > 0){   \
		if(spin_is_locked(lock)) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}


/*===============================================================*/
/* external function prottyped                                   */
/*===============================================================*/
extern int  mp200_ipu_regist(struct mp200_ipu_req *p);                  /* ROT|IMG request function */
extern int  mp200_lcd_set_v4l2_image(V4L2_IMAGE_INFO *v4l2_iamge_info); /* LCD request function     */
#ifdef CONFIG_MP200_NTS
extern int  mp200_nts_set_v4l2_image(V4L2_IMAGE_INFO *v4l2_iamge_info); /* NTS request function     */
extern int  mp200_nts_reserve(int iActiveDevice, int iSetOutmode);
extern void mp200_nts_free(int iActiveDevice);
extern int  mp200_nts_getmode(void);
#endif /* CONFIG_MP200_NTS */


/*===============================================================*/
/* kernel thread function                                        */
/*===============================================================*/
static        int   mp200_v4l2_thread_startup(struct mp200_v4l2_device *dev);
static        void  mp200_v4l2_thread_cleanup(struct mp200_v4l2_device *dev);
static        void  mp200_v4l2_thread_init(struct mp200_v4l2_device *dev);
static        void  mp200_v4l2_thread_main(struct mp200_v4l2_device *dev, struct th_object *th_obj);
static inline int   mp200_v4l2_thread_rot(void *dev);
static inline int   mp200_v4l2_thread_img(void *dev);
static inline int   mp200_v4l2_thread_tmr(void *dev);
static inline int   mp200_v4l2_thread_lcd(void *dev);

static        void *mp200_v4l2_reschedule(struct mp200_v4l2_device *dev, struct th_object *th_obj);
static        int   mp200_v4l2_next_proccess(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, enum buffer_select buf_sel, enum call_func_num call_func);
static        int   mp200_v4l2_chk_ipuimg(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);
#ifdef CONFIG_VIDEO_MP200_FRAMESKIP
static        int   mp200_v4l2_chk_frameskip(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, struct th_object *th_obj);
#endif /* CONFIG_VIDEO_MP200_FRAMESKIP */


/*===============================================================*/
/* kernel timer function                                         */
/*===============================================================*/
static inline long  mp200_v4l2_tmr_cmpare_time(struct timeval *start, struct timeval *end);
static inline long  mp200_v4l2_tmr_calc_waittime(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);
static inline void  mp200_v4l2_tmr_chk_waittime(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);
static inline long  mp200_v4l2_tmr_calc_adjusttime(struct mp200_v4l2_device *dev);
static inline void  mp200_v4l2_tmr_init(struct mp200_v4l2_device *dev);
static        int   mp200_v4l2_tmr_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int dummy);
static inline void  mp200_v4l2_tmr_callback_top(unsigned long data);
#if ENABLE_DELAY
static        void  mp200_v4l2_tmr_callback_bottom_do(struct work_struct *num);
#else
static inline void  mp200_v4l2_tmr_callback_bottom_do(struct mp200_v4l2_device *dev);
#endif


/*===============================================================*/
/* IPU proccessing function prottyped                            */
/*===============================================================*/
static        void  mp200_v4l2_ipu_callback_main(struct mp200_v4l2_device *dev, struct th_object *th_obj, enum call_func_num call_func, int flag);
static        int   mp200_v4l2_ipu_request_main(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, struct th_object *th_obj, enum call_func_num call_func);
/* ------------------ ROT request ops ------------------------------------------- */
static inline void  mp200_v4l2_rot_callback(unsigned long sequence, int flag);
static inline int   mp200_v4l2_rot_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int dummy);
static        int   mp200_v4l2_rot_make_req(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int index, enum buffer_select buf_sel);
/* ------------------ IMG request ops ------------------------------------------- */
static inline void  mp200_v4l2_img_callback(unsigned long sequence, int flag);
static inline int   mp200_v4l2_img_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int dummy);
static        int   mp200_v4l2_img_make_req(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int index, enum buffer_select buf_sel);


/*===============================================================*/
/* LCD proccessing function prottyped                            */
/*===============================================================*/
              void  mp200_v4l2_lcd_callback(FRAME_DATA frame_data);
              void  mp200_v4l2_lcd_refresh_callback(FRAME_DATA frame_data);
              void  mp200_v4l2_ntsc_ready_callback(FRAME_DATA frame_data);
static inline void *mp200_v4l2_lcd_set_outfunc(int output);
static        int   mp200_v4l2_lcd_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int call_flg);
static        void  mp200_v4l2_lcd_stop_do(struct work_struct *num);


/*===============================================================*/
/* videobuf queue operate function prottyped                     */
/*===============================================================*/
static inline void  mp200_v4l2_core_vbq_release(struct videobuf_queue *q, struct videobuf_buffer *vb);
static inline void  mp200_v4l2_core_vbq_dqueue(struct videobuf_queue *q, struct videobuf_buffer *vb);
static        int   mp200_v4l2_core_vbq_cancel(struct videobuf_queue *q, struct videobuf_buffer *vb);
static        void  mp200_v4l2_core_vbq_complete(void *arg1, void *arg, int err_flg);
static inline void  mp200_v4l2_core_vbq_complete_skipped(struct list_head *vbq_stream);
static        void  mp200_v4l2_core_vbq_queue(struct videobuf_queue *q, struct videobuf_buffer *vb);
static        int   mp200_v4l2_core_vbq_prepare(struct videobuf_queue *q, struct videobuf_buffer *vb, enum v4l2_field field);
static        int   mp200_v4l2_core_vbq_sizecheck_pix(struct videobuf_buffer *vb);
static        int   mp200_v4l2_core_vbq_sizecheck_crop(struct videobuf_buffer *vb);
static        int   mp200_v4l2_core_vbq_sizecheck(struct videobuf_queue *q, struct videobuf_buffer *vb);
static        int   mp200_v4l2_core_vbq_setup(struct videobuf_queue *q, unsigned int *cnt, unsigned int *size);


/*===============================================================*/
/* IPU workbuffer                                                */
/*===============================================================*/
static inline void  mp200_v4l2_core_wkb_init(struct mp200_v4l2_device *dev);
static        int   mp200_v4l2_core_wkb_active(struct mp200_v4l2_device *dev, unsigned int cmd,  void *arg);
static        int   mp200_v4l2_core_wkb_chkbuf(struct v4l2_workbuffer *wbuf);
static        int   mp200_v4l2_core_wkb_chksize(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);


/*===============================================================*/
/* systemcall function prottyped                                 */
/*===============================================================*/
static inline int   mp200_v4l2_ioc_g_output(struct mp200_v4l2_fh *fh, unsigned int *output);
static        int   mp200_v4l2_ioc_s_output(struct mp200_v4l2_fh *fh, unsigned int *output);
static        int   mp200_v4l2_ioc_g_fmt(struct mp200_v4l2_device *dev, struct v4l2_format *fmt);
static        int   mp200_v4l2_ioc_s_fmt(struct mp200_v4l2_device *dev, struct v4l2_format *fmt);
static        int   mp200_v4l2_ioc_g_crop(struct mp200_v4l2_device *dev, struct v4l2_crop *vc);
static        int   mp200_v4l2_ioc_s_crop(struct mp200_v4l2_device *dev, struct v4l2_crop *vc);
static        int   mp200_v4l2_ioc_g_ctrl(struct mp200_v4l2_device *dev, struct v4l2_control *vc);
static        int   mp200_v4l2_ioc_s_ctrl(struct mp200_v4l2_device *dev, struct v4l2_control *vc);
static        int   mp200_v4l2_ioc_g_effect(struct mp200_v4l2_device *dev, struct v4l2_effect *vef);
static        int   mp200_v4l2_ioc_s_effect(struct mp200_v4l2_device *dev, struct v4l2_effect *vef);
static inline int   mp200_v4l2_ioc_reqbufs(struct mp200_v4l2_device *dev, struct videobuf_queue *vbq, struct v4l2_requestbuffers *req);
static inline int   mp200_v4l2_ioc_qbuf(struct mp200_v4l2_device *dev, struct videobuf_queue *vbq, struct v4l2_buffer *vb);
static inline int   mp200_v4l2_ioc_dqbuf(struct mp200_v4l2_device *dev, struct videobuf_queue *vbq, struct v4l2_buffer *vb, int f_flags);
static        int   mp200_v4l2_ioc_streamon(struct mp200_v4l2_fh *fh, struct mp200_v4l2_device *dev);
static        int   mp200_v4l2_ioc_streamoff(struct mp200_v4l2_fh *fh, struct mp200_v4l2_device *dev);
static        int   mp200_v4l2_core_do_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg);
static inline int   mp200_v4l2_core_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
static inline unsigned int mp200_v4l2_core_poll(struct file *file, struct poll_table_struct *wait);
static        int   mp200_v4l2_core_release(struct inode *inode, struct file *file);
static        int   mp200_v4l2_core_open(struct inode *inode, struct file *file);
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
static        int   mp200_v4l2_core_suspend(struct platform_device *pdev, pm_message_t state);
static        int   mp200_v4l2_core_resume(struct platform_device *pdev);
#endif /* CONFIG_PM || CONFIG_DPM */
void        mp200_v4l2_core_cleanup(void);
int __init  mp200_v4l2_core_init(void);


/*===============================================================*/
/* file_operations                                               */
/*===============================================================*/
static struct file_operations mp200_v4l2_core_fops = {
	.owner		= THIS_MODULE,
	.llseek		= no_llseek,
	.ioctl		= mp200_v4l2_core_ioctl,
	.poll		= mp200_v4l2_core_poll,
	.open		= mp200_v4l2_core_open,
	.release	= mp200_v4l2_core_release,
};


/*===============================================================*/
/* device_driver                                                 */
/*===============================================================*/
static struct platform_driver mp200_v4l2_core_driver = {
	.probe		= NULL,
	.remove		= NULL,
	.shutdown	= NULL,

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
	.suspend	= mp200_v4l2_core_suspend,
	.resume		= mp200_v4l2_core_resume,
#else
	.suspend	= NULL,
	.resume		= NULL,
#endif /* CONFIG_PM || CONFIG_DPM */

	.driver.name	= DEV_NAME,
	.driver.bus	= &platform_bus_type,
};


/*===============================================================*/
/* platform_device                                               */
/*===============================================================*/
static struct platform_device mp200_v4l2_core_device = {
	.name	= DEV_NAME,
	.dev	= {
			.release 	= NULL,
		  },
	.id	= 0,
};


/*===============================================================*/
/* module parameters                                             */
/*===============================================================*/
struct mp200_v4l2_device *mp200_dev;
static int video_nr = -1;	/* video device minor (-1 ==> auto assign) */




#include "mp200_v4l2_perf.h"
#include "mp200_v4l2_debug.h"


/*===============================================================*/
/* kernel thread function                                        */
/*===============================================================*/
/*****************************************************************************
* MODULE   : mp200_v4l2_thread_startup
* FUNCTION : 
* RETURN   : 
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_thread_startup(struct mp200_v4l2_device *dev)
{
	mp200_v4l2_thread_init(dev);

	init_waitqueue_head(&dev->th_rot.th_idle);
	init_waitqueue_head(&dev->th_rot.th_busy);
	init_waitqueue_head(&dev->th_img.th_idle);
	init_waitqueue_head(&dev->th_img.th_busy);
	init_waitqueue_head(&dev->th_tmr.th_idle);
	init_waitqueue_head(&dev->th_tmr.th_busy);
	init_waitqueue_head(&dev->th_lcd.th_idle);
	init_waitqueue_head(&dev->th_lcd.th_busy);

	/*----------------------*/
	/* create kernel thread */
	/*----------------------*/
	dev->th_rot.th = kthread_run(mp200_v4l2_thread_rot, dev, dev->th_rot.th_name);
	if(IS_ERR(dev->th_rot.th)){
		return -1;
	}
	dev->th_img.th = kthread_run(mp200_v4l2_thread_img, dev, dev->th_img.th_name);
	if(IS_ERR(dev->th_img.th)){
		return -1;
	}
	dev->th_tmr.th = kthread_run(mp200_v4l2_thread_tmr, dev, dev->th_tmr.th_name);
	if(IS_ERR(dev->th_tmr.th)){
		return -1;
	}
	dev->th_lcd.th = kthread_run(mp200_v4l2_thread_lcd, dev, dev->th_lcd.th_name);
	if(IS_ERR(dev->th_lcd.th)){
		return -1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_thread_cleanup
* FUNCTION : 
* RETURN   : 
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_thread_cleanup(struct mp200_v4l2_device *dev)
{
	mp200_v4l2_thread_init(dev);

	/*-------------------------*/
	/* discreate kernel thread */
	/*-------------------------*/
	wake_up_interruptible(&dev->th_rot.th_idle);
	wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
	wake_up_interruptible(&dev->th_rot.th_busy);
	wake_up_interruptible(&dev->th_img.th_idle);
	wake_up_interruptible(&dev->th_img.th_buf->buf_busy);
	wake_up_interruptible(&dev->th_img.th_busy);
	wake_up_interruptible(&dev->th_tmr.th_idle);
	wake_up_interruptible(&dev->th_tmr.th_busy);
	wake_up_interruptible(&dev->th_lcd.th_idle);
	wake_up_interruptible(&dev->th_lcd.th_busy);

	/*-------------------------*/
	/* ROT thread uninitialize */
	/*-------------------------*/
	if(dev->th_lcd.th) kthread_stop(dev->th_lcd.th);
	kfree(dev->th_rot.th_buf);
	memset(&dev->th_rot, 0, sizeof(struct th_object));

	/*-------------------------*/
	/* IMG thread uninitialize */
	/*-------------------------*/
	if(dev->th_lcd.th) kthread_stop(dev->th_lcd.th);
	kfree(dev->th_img.th_buf);
	memset(&dev->th_img, 0, sizeof(struct th_object));

	/*-------------------------*/
	/* TMR thread uninitialize */
	/*-------------------------*/
	if(dev->th_lcd.th) kthread_stop(dev->th_lcd.th);
	memset(&dev->th_tmr, 0, sizeof(struct th_object));

	/*-------------------------*/
	/* LCD thread uninitialize */
	/*-------------------------*/
	if(dev->th_lcd.th) kthread_stop(dev->th_lcd.th);
	memset(&dev->th_lcd, 0, sizeof(struct th_object));
}


/*****************************************************************************
* MODULE   : mp200_v4l2_thread_init
* FUNCTION : 
* RETURN   : 
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_thread_init(struct mp200_v4l2_device *dev)
{
	int i;

	/*-----------------------*/
	/* ROT thread initialize */
	/*-----------------------*/
	dev->th_rot.th_name                   = "kv4l2_rot";
	dev->th_rot.th_idle_flag              = STATE_QUEUED_IDLE;
	dev->th_rot.th_idle_fixed             = STATE_QUEUED_ROT_PREPARED;
	dev->th_rot.th_busy_fixed             = STATE_QUEUED_ROT_QUEUED;
	dev->th_rot.call_request              = (void *)mp200_v4l2_rot_request;
	/* ROT temp buffer init */
	if(dev->th_rot.th_buf == NULL){
		dev->th_rot.th_buf            = kmalloc(sizeof(struct buf_object), GFP_KERNEL);
	}
	memset(dev->th_rot.th_buf, 0, sizeof(struct buf_object));
	init_waitqueue_head(&dev->th_rot.th_buf->buf_busy);

	for(i = 0; i < NUM_BUF_V4L2; i++){
		dev->th_rot.th_buf->buf[i].state  = TMPBUF_IDLE;
		dev->th_rot.th_buf->buf[i].paddr  = dev->workbuf.rot_addr + (((dev->workbuf.rot_size / NUM_BUF_V4L2) & ~0x3) * i);
	}
	dev->th_rot.th_buf->make_request      = (void *)mp200_v4l2_rot_make_req;
	dev->th_rot.th_buf->callback          = (void *)mp200_v4l2_rot_callback;
	dev->th_rot.th_buf->ipu_hw_func       = IPU_HW_FUNC_ROT;

	/*-----------------------*/
	/* IMG thread initialize */
	/*-----------------------*/
	dev->th_img.th_name                   = "kv4l2_img";
	dev->th_img.th_idle_flag              = STATE_QUEUED_IDLE;
	dev->th_img.th_idle_fixed             = STATE_QUEUED_IMG_PREPARED;
	dev->th_img.th_busy_fixed             = STATE_QUEUED_IMG_QUEUED;
	dev->th_img.call_request              = (void *)mp200_v4l2_img_request;
	/* IMG temp buffer init */
	if(dev->th_img.th_buf == NULL){
		dev->th_img.th_buf            = kmalloc(sizeof(struct buf_object), GFP_KERNEL);
	}
	memset(dev->th_img.th_buf, 0, sizeof(struct buf_object));
	init_waitqueue_head(&dev->th_img.th_buf->buf_busy);

	for(i = 0; i < NUM_BUF_V4L2; i++){
		dev->th_img.th_buf->buf[i].state  = TMPBUF_IDLE;
		dev->th_img.th_buf->buf[i].paddr  = dev->workbuf.img_addr + (((dev->workbuf.img_size / NUM_BUF_V4L2) & ~0x3) * i);
	}
	dev->th_img.th_buf->make_request      = (void *)mp200_v4l2_img_make_req;
	dev->th_img.th_buf->callback          = (void *)mp200_v4l2_img_callback;
	dev->th_img.th_buf->ipu_hw_func       = IPU_HW_FUNC_IMG;

	/*-----------------------*/
	/* TMR thread initialize */
	/*-----------------------*/
	dev->th_tmr.th_name                   = "kv4l2_tmr";
	dev->th_tmr.th_idle_flag              = STATE_QUEUED_IDLE;
	dev->th_tmr.th_idle_fixed             = STATE_QUEUED_TMR_PREPARED;
	dev->th_tmr.th_busy_fixed             = STATE_QUEUED_TMR_QUEUED;
	dev->th_tmr.call_request              = (void *)mp200_v4l2_tmr_request;
	/* temp buffer not used */
	dev->th_tmr.th_buf                    = NULL;

	/*-----------------------*/
	/* LCD thread initialize */
	/*-----------------------*/
	dev->th_lcd.th_name                   = "kv4l2_lcd";
	dev->th_lcd.th_idle_flag              = STATE_QUEUED_IDLE;
	dev->th_lcd.th_idle_fixed             = STATE_QUEUED_LCD_PREPARED;
	dev->th_lcd.th_busy_fixed             = STATE_QUEUED_LCD_QUEUED;
	dev->th_lcd.call_request              = (void *)mp200_v4l2_lcd_request;
	/* temp buffer not used */
	dev->th_lcd.th_buf                    = NULL;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_thread_main
* FUNCTION : 
* RETURN   : 
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_thread_main(struct mp200_v4l2_device *dev, struct th_object *th_obj)
{
	struct videobuf_buffer *vb;

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
	try_to_freeze();
#endif /* CONFIG_PM || CONFIG_DPM */

	vb = mp200_v4l2_reschedule(dev, th_obj);
	if(vb == NULL){
		/* Queue is free */
		trace_add_th(0, "thread_main: wait_event(", "th_idle) <--");
		wait_event_interruptible(th_obj->th_idle, (th_obj->th_idle_flag == th_obj->th_idle_fixed));
		trace_add_th(0, "thread_main: wait_event(", "th_idle) -->");
	}
	else{
#ifdef CONFIG_VIDEO_MP200_FRAMESKIP
		if(mp200_v4l2_chk_frameskip(dev, vb, th_obj) != STATE_FRAME_SKIPPED){
#endif /* CONFIG_VIDEO_MP200_FRAMESKIP */
			if(th_obj->th_buf){
				int i;
				for(i = 0; i < NUM_BUF_V4L2; i++){
					if(th_obj->th_buf->buf[i].state == TMPBUF_IDLE){
						break;
					}
				}
				if (i >= NUM_BUF_V4L2){
					trace_add_th(vb, "thread_main: wait_event(", "buf_busy) <--");
#if (NUM_BUF_V4L2 == 3)
					wait_event_interruptible(th_obj->th_buf->buf_busy, (th_obj->th_buf->buf[BUF_A].state == TMPBUF_IDLE || 
											    th_obj->th_buf->buf[BUF_B].state == TMPBUF_IDLE || 
											    th_obj->th_buf->buf[BUF_C].state == TMPBUF_IDLE));
#else /* (NUM_BUF_V4L2 == 2) */
					wait_event_interruptible(th_obj->th_buf->buf_busy, (th_obj->th_buf->buf[BUF_A].state == TMPBUF_IDLE || th_obj->th_buf->buf[BUF_B].state == TMPBUF_IDLE));
#endif
					trace_add_th(vb, "thread_main: wait_event(", "buf_busy) -->");
				}
			}

			/* call request function */
			if(!th_obj->call_request(dev, vb, STREAM_PLAYBACK)){
				/* processing request */
				trace_add_th(vb, "thread_main: wait_event(", "th_busy) <--");
				wait_event_interruptible(th_obj->th_busy, (vb->state_queued != th_obj->th_busy_fixed));
				trace_add_th(vb, "thread_main: wait_event(", "th_busy) -->");
			}
#ifdef CONFIG_VIDEO_MP200_FRAMESKIP
		}
		else{
			trace_add(vb, "thread_main: frame skipped!");
		}
#endif /* CONFIG_VIDEO_MP200_FRAMESKIP */
	}
}

/*****************************************************************************
* MODULE   : mp200_v4l2_thread_rot
* FUNCTION : start to run request in IPU-ROT Request's queue.(ROT thread)
* RETURN   : 0
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_thread_rot(void *dev)
{
	/* set thread priority */
#if _V4L2_RT_THREAD /* RT thread */
	struct sched_param param = { .sched_priority = V4L2_THREAD_PRIORITY };
	sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
	set_user_nice(current, V4L2_THREAD_NICE);
#endif

	while(!kthread_should_stop()){
		mp200_v4l2_thread_main(dev, &((struct mp200_v4l2_device *)dev)->th_rot);
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_thread_img
* FUNCTION : start to run request in IPU-IMG Request's queue.(IMG thread)
* RETURN   : 0
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_thread_img(void *dev)
{
	/* set thread priority */
#if _V4L2_RT_THREAD /* RT thread */
	struct sched_param param = { .sched_priority = V4L2_THREAD_PRIORITY };
	sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
	set_user_nice(current, V4L2_THREAD_NICE);
#endif

	while(!kthread_should_stop()){
		mp200_v4l2_thread_main(dev, &((struct mp200_v4l2_device *)dev)->th_img);
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_thread_tmr
* FUNCTION : start to run request in TIMER Request's queue.(TMR thread)
* RETURN   : 0
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_thread_tmr(void *dev)
{
	/* set thread priority */
#if _V4L2_RT_THREAD /* RT thread */
	struct sched_param param = { .sched_priority = V4L2_THREAD_PRIORITY };
	sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
	set_user_nice(current, V4L2_THREAD_NICE);
#endif

	while(!kthread_should_stop()){
		mp200_v4l2_thread_main(dev, &((struct mp200_v4l2_device *)dev)->th_tmr);
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_thread_lcd
* FUNCTION : start to run request in LCD Request's queue.(LCD thread)
* RETURN   : 0
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_thread_lcd(void *dev)
{
	/* set thread priority */
#if _V4L2_RT_THREAD /* RT thread */
	struct sched_param param = { .sched_priority = V4L2_THREAD_PRIORITY };
	sched_setscheduler(current, SCHED_FIFO, &param);
#else /* Normal thread */
	set_user_nice(current, V4L2_THREAD_NICE);
#endif

	while(!kthread_should_stop()){
		mp200_v4l2_thread_main(dev, &((struct mp200_v4l2_device *)dev)->th_lcd);
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_reschedule
* FUNCTION : call next queue.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static void *mp200_v4l2_reschedule(struct mp200_v4l2_device *dev, struct th_object *th_obj)
{
	struct videobuf_queue  *q  = NULL;
	struct videobuf_buffer *vb = NULL;
	struct list_head       *list;
	unsigned long flags;

	printk_dbg((_ROT_DBG | _IMG_DBG | _TMR_DBG | _LCD_DBG), "call in.\n");
	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if(dev->streaming){
		q = &dev->streaming->vbq;
		list_for_each(list, &q->stream)
		{
			vb = list_entry(list, struct videobuf_buffer, stream);
			switch(vb->state){
			case STATE_PREPARED:
				vb->state = STATE_QUEUED;
				mp200_v4l2_next_proccess(dev, vb, 0, RESCHEDULE);
				/* FALL THROUGH */
			case STATE_QUEUED:
				printk_dbg(_VBQ_DBG, "(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%d)\n", &vb->stream, vb->stream.prev, vb->stream.next, vb->state, vb->sequence);
				if(vb->state_queued < th_obj->th_idle_fixed){
					/* don't process it for status_queued that is smaller than call_function. */
					trace_add_th(vb, "reschedule:" , "busy...");
					th_obj->th_idle_flag = STATE_QUEUED_IDLE;
					spin_unlock_irqrestore(&dev->vbq_lock, flags);
					return NULL;
				}
				else if(vb->state_queued == th_obj->th_idle_fixed){
					trace_add_th(vb, "reschedule:" , "next find.");
					spin_unlock_irqrestore(&dev->vbq_lock, flags);
					return vb;
				}
				else{
					/* search next queue */
					break;
				}
			case STATE_CANCELED:
				if(vb->state_queued != STATE_QUEUED_DONE){
					if(vb->state_queued == th_obj->th_idle_fixed){
						trace_add_th(vb, "reschedule:" , "CANCELED.");
						spin_unlock_irqrestore(&dev->vbq_lock, flags);
						return vb;
					}
				}
				break;
			default:
				/* search next queue */
				break;
			}
		}
	}

	trace_add(0, "reschedule: not find.");
	th_obj->th_idle_flag = STATE_QUEUED_IDLE;
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return NULL;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_next_proccess
* FUNCTION : Allocate & Set next Queue status.
* RETURN   : 
* NOTE     : request() & callback() allways call this function.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_next_proccess(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, enum buffer_select buf_sel, enum call_func_num call_func)
{
	unsigned long flags;
	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if(call_func == VBQ_DQUEUE){
		vb->state_queued    = STATE_QUEUED_IDLE;
		vb->state_proccess |= STATE_PROCCESS_IDLE;
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -1;
	}

	/* phase1 **************/
	/* check CANCELED task */
	if((vb->state == STATE_CANCELED || vb->state == STATE_ERROR) ||
	   (vb->state == STATE_DQBUF_PERMIT && dev->vb_old_refresh == NULL)){
		trace_add_func(vb, "cancel_proccess", call_func);

		/* initialize buffer */
		switch(vb->state_queued){
		case STATE_QUEUED_LCD_PREPARED:	/* FALL THROUGH */
		case STATE_QUEUED_TMR_QUEUED:	/* FALL THROUGH */
		case STATE_QUEUED_TMR_PREPARED:	/* FALL THROUGH */
		case STATE_QUEUED_IMG_QUEUED:	/* FALL THROUGH */
		case STATE_QUEUED_DONE:
			/* free buffer for IMG */
			if(vb->img_buf){
				vb->img_buf->state = TMPBUF_IDLE;
				vb->img_buf        = NULL;
				trace_add(vb, "IMG: buffer -->");
			}
			break;
		default:
			break;
		}
		switch(vb->state_queued){
		case STATE_QUEUED_LCD_PREPARED:	/* FALL THROUGH */
		case STATE_QUEUED_TMR_QUEUED:	/* FALL THROUGH */
		case STATE_QUEUED_TMR_PREPARED:	/* FALL THROUGH */
		case STATE_QUEUED_IMG_QUEUED:	/* FALL THROUGH */
		case STATE_QUEUED_IMG_PREPARED:	/* FALL THROUGH */
		case STATE_QUEUED_ROT_QUEUED:	/* FALL THROUGH */
		case STATE_QUEUED_DONE:
			/* free buffer for ROT */
			if(vb->rot_buf){
				vb->rot_buf->state = TMPBUF_IDLE;
				vb->rot_buf        = NULL;
				trace_add(vb, "ROT: buffer -->");
			}
			break;
		default:
			break;
		}

		/* to next proccess */
		switch(call_func){
		case ROT_REQUEST: /* FALL THROUGH */
		case ROT_CALLBACK:
			DEL_MIXING(ROT_MIXING, ROT_MIXING_SFT);
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE || vb->state_proccess & STATE_PROCCESS_ROT_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(ROT: buf_busy)");
				wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
			}
			trace_add(vb, "cancel_proccess: wake_up(ROT: th_busy)");
			wake_up_interruptible(&dev->th_rot.th_busy);
			break;
		case IMG_REQUEST: /* FALL THROUGH */
		case IMG_CALLBACK:
			DEL_MIXING(IMG_MIXING, IMG_MIXING_SFT);
			if(vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE || vb->state_proccess & STATE_PROCCESS_IMG_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(IMG: buf_busy)");
				wake_up_interruptible(&dev->th_img.th_buf->buf_busy);
			}
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE || vb->state_proccess & STATE_PROCCESS_ROT_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(ROT: buf_busy)");
				wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
			}
			trace_add(vb, "cancel_proccess: wake_up(IMG: th_busy)");
			wake_up_interruptible(&dev->th_img.th_busy);
			break;
		case TMR_REQUEST: /* FALL THROUGH */
		case TMR_CALLBACK:
			DEL_MIXING(TMR_MIXING, TMR_MIXING_SFT);
			if(vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE || vb->state_proccess & STATE_PROCCESS_IMG_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(IMG: buf_busy)");
				wake_up_interruptible(&dev->th_img.th_buf->buf_busy);
			}
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE || vb->state_proccess & STATE_PROCCESS_ROT_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(ROT: buf_busy)");
				wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
			}
			trace_add(vb, "cancel_proccess: wake_up(TMR: th_busy)");
			wake_up_interruptible(&dev->th_tmr.th_busy);
			break;
		case LCD_REQUEST: /* FALL THROUGH */
		case LCD_CALLBACK:
			DEL_MIXING(LCD_MIXING, LCD_MIXING_SFT);
			if(vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE || vb->state_proccess & STATE_PROCCESS_IMG_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(IMG: buf_busy)");
				wake_up_interruptible(&dev->th_img.th_buf->buf_busy);
			}
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE || vb->state_proccess & STATE_PROCCESS_ROT_QUEUED){
				trace_add(vb, "cancel_proccess: wake_up(ROT: buf_busy)");
				wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
			}
			trace_add(vb, "cancel_proccess: wake_up(LCD: th_busy)");
			wake_up_interruptible(&dev->th_lcd.th_busy);
			break;
		default:
			break;
		}

		vb->state_queued    = STATE_QUEUED_DONE;
		vb->state_proccess |= STATE_PROCCESS_LCD_COMPLETE;
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -1;
	}
	else if(vb->state != STATE_PREPARED && vb->state != STATE_QUEUED && vb->state != STATE_ACTIVE && vb->state != STATE_DONE){
		switch(call_func){
		case ROT_REQUEST: /* FALL THROUGH */
		case ROT_CALLBACK:
			DEL_MIXING(ROT_MIXING, ROT_MIXING_SFT);
			break;
		case IMG_REQUEST: /* FALL THROUGH */
		case IMG_CALLBACK:
			DEL_MIXING(IMG_MIXING, IMG_MIXING_SFT);
			break;
		case TMR_REQUEST: /* FALL THROUGH */
		case TMR_CALLBACK:
			DEL_MIXING(TMR_MIXING, TMR_MIXING_SFT);
			break;
		case LCD_REQUEST: /* FALL THROUGH */
		case LCD_CALLBACK:
			DEL_MIXING(LCD_MIXING, LCD_MIXING_SFT);
			break;
		default:
			break;
		}
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return 0;
	}

	trace_add_func(vb, "next_proccess", call_func);

	/* phase2 ********************/
	/* set flag to next proccess */
	switch(call_func){
	case VBQ_DQUEUE:
		break;

	case LCD_CALLBACK:
		DEL_MIXING(LCD_MIXING, LCD_MIXING_SFT);
		if(dev->vb_old_refresh){
			if(dev->vb_old_refresh->img_buf && (dev->vb_old_refresh->state_proccess & STATE_PROCCESS_IMG_COMPLETE)){
				dev->vb_old_refresh->img_buf->state = TMPBUF_IDLE;
				dev->vb_old_refresh->img_buf        = NULL;
				trace_add(dev->vb_old_refresh, "IMG: buffer -->");
			}
			else if(dev->vb_old_refresh->rot_buf && (dev->vb_old_refresh->state_proccess & STATE_PROCCESS_ROT_COMPLETE)){
				dev->vb_old_refresh->rot_buf->state = TMPBUF_IDLE;
				dev->vb_old_refresh->rot_buf        = NULL;
				trace_add(dev->vb_old_refresh, "ROT: buffer -->");
			}
		}
		dev->vb_old_refresh = vb;
		if(vb->img_buf && (vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE)){
			vb->img_buf->state = TMPBUF_LCD_WRITE;
			trace_add(vb, "IMG: buffer -->");
		}
		else if(vb->rot_buf && (vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE)){
			vb->rot_buf->state = TMPBUF_LCD_WRITE;
			trace_add(vb, "ROT: buffer -->");
		}
		vb->state_queued    = STATE_QUEUED_DONE;
		vb->state_proccess |= STATE_PROCCESS_LCD_COMPLETE;
		break;

	case VBQ_COMPLETE:
		break;

	case ROT_REQUEST:
		ADD_MIXING(ROT_MIXING, ROT_MIXING_SFT);
		vb->state_queued    = STATE_QUEUED_ROT_QUEUED;
		vb->state_proccess |= STATE_PROCCESS_ROT_QUEUED;

		switch(buf_sel){
		case 0: trace_add(vb, "ROT: buffer0 <--"); break;
		case 1: trace_add(vb, "ROT: buffer1 <--"); break;
		case 2: trace_add(vb, "ROT: buffer2 <--"); break;
		}
		vb->rot_buf         = &dev->th_rot.th_buf->buf[buf_sel];
		vb->rot_buf->state  = TMPBUF_ROT_WRITE;
		break;
	case IMG_REQUEST:
		ADD_MIXING(IMG_MIXING, IMG_MIXING_SFT);
		vb->state_queued    = STATE_QUEUED_IMG_QUEUED;
		vb->state_proccess |= STATE_PROCCESS_IMG_QUEUED;

		switch(buf_sel){
		case 0: trace_add(vb, "IMG: buffer0 <--"); break;
		case 1: trace_add(vb, "IMG: buffer1 <--"); break;
		case 2: trace_add(vb, "IMG: buffer2 <--"); break;
		}
		vb->img_buf         = &dev->th_img.th_buf->buf[buf_sel];
		vb->img_buf->state  = TMPBUF_IMG_WRITE;
		break;
	case TMR_REQUEST:
		ADD_MIXING(TMR_MIXING, TMR_MIXING_SFT);
		vb->state_queued    = STATE_QUEUED_TMR_QUEUED;
		vb->state_proccess |= STATE_PROCCESS_TMR_QUEUED;
		break;
	case LCD_REQUEST:
		ADD_MIXING(LCD_MIXING, LCD_MIXING_SFT);
		vb->state_queued    = STATE_QUEUED_LCD_QUEUED;
		vb->state_proccess |= STATE_PROCCESS_LCD_QUEUED;
		break;

	case VBQ_QUEUE:
		/* FALL THROUGH */
	case RESCHEDULE:
		vb->state_proccess = STATE_PROCCESS_IDLE;

		if(vb->save_effect.movie_angle){
			/* next process is IPU-ROT */
			vb->state_queued         = STATE_QUEUED_ROT_PREPARED;
			dev->th_rot.th_idle_flag = dev->th_rot.th_idle_fixed;
			break;
		}
		/* FALL THROUGH */
	case ROT_CALLBACK:
		if(call_func == ROT_CALLBACK){
			DEL_MIXING(ROT_MIXING, ROT_MIXING_SFT);
			vb->state_proccess |= STATE_PROCCESS_ROT_COMPLETE;
		}
		if(mp200_v4l2_chk_ipuimg(dev, vb)){
			/* next process is IPU-IMG */
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
				vb->rot_buf->state = TMPBUF_IMG_READ;
			}
			vb->state_queued         = STATE_QUEUED_IMG_PREPARED;
			dev->th_img.th_idle_flag = dev->th_img.th_idle_fixed;
			break;
		}
		/* FALL THROUGH */
	case IMG_CALLBACK:
		if(call_func == IMG_CALLBACK){
			DEL_MIXING(IMG_MIXING, IMG_MIXING_SFT);
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
				vb->rot_buf->state = TMPBUF_IDLE;
				vb->rot_buf        = NULL;
				trace_add(vb, "ROT: buffer -->");
			}
			vb->state_proccess |= STATE_PROCCESS_IMG_COMPLETE;
		}
		if(vb->state_frame != STATE_FRAME_IMMEDIATE){
			/* next process is TIMER */
			vb->state_queued         = STATE_QUEUED_TMR_PREPARED;
			dev->th_tmr.th_idle_flag = dev->th_tmr.th_idle_fixed;
			break;
		}
		/* FALL THROUGH */
	case TMR_CALLBACK:
		if(call_func == TMR_CALLBACK){
			DEL_MIXING(TMR_MIXING, TMR_MIXING_SFT);
			vb->state_proccess |= STATE_PROCCESS_TMR_COMPLETE;
		}
		/* next process is LCD */
		if(vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE){
			vb->img_buf->state = TMPBUF_LCD_READ;
		}
		else if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
			vb->rot_buf->state = TMPBUF_LCD_READ;
		}
		vb->state_queued         = STATE_QUEUED_LCD_PREPARED;
		dev->th_lcd.th_idle_flag = dev->th_lcd.th_idle_fixed;
		break;
	}

	/* phase3 ******************/
	/* wakeup to next proccess */
	switch(call_func){
	case VBQ_DQUEUE:
		break;

	case LCD_CALLBACK:
		if(vb->output == V4L2_OUTPUT_LCD){
			trace_add(vb, "next_proccess: wake_up(LCD: th_busy)");
			wake_up_interruptible(&dev->th_lcd.th_busy);
		}
		if(vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE){
			trace_add(vb, "next_proccess: wake_up(IMG: buf_busy)");
			wake_up_interruptible(&dev->th_img.th_buf->buf_busy);
		}
		else if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
			trace_add(vb, "next_proccess: wake_up(ROT: buf_busy)");
			wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
		}
		break;

	case VBQ_COMPLETE:
		if(vb->output != V4L2_OUTPUT_LCD){
			trace_add(vb, "next_proccess: wake_up(LCD: th_busy)");
			wake_up_interruptible(&dev->th_lcd.th_busy);
		}
		break;

	case ROT_REQUEST:
		break;
	case IMG_REQUEST:
		break;
	case TMR_REQUEST:
		break;
	case LCD_REQUEST:
		break;

	case VBQ_QUEUE:
		/* FALL THROUGH */
	case RESCHEDULE:
		if(vb->save_effect.movie_angle){
			/* next process is IPU-ROT */
			trace_add(vb, "next_proccess: wake_up(ROT: th_idle)");
			wake_up_interruptible(&dev->th_rot.th_idle);
			break;
		}
		/* FALL THROUGH */
	case ROT_CALLBACK:
		if(call_func == ROT_CALLBACK){
			trace_add(vb, "next_proccess: wake_up(ROT: th_busy)");
			wake_up_interruptible(&dev->th_rot.th_busy);
		}
		if(mp200_v4l2_chk_ipuimg(dev, vb)){
			/* next process is IPU-IMG */
			trace_add(vb, "next_proccess: wake_up(IMG: th_idle)");
			wake_up_interruptible(&dev->th_img.th_idle);
			break;
		}
		/* FALL THROUGH */
	case IMG_CALLBACK:
		if(call_func == IMG_CALLBACK){
			if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
				trace_add(vb, "next_proccess: wake_up(ROT: buf_busy)");
				wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
			}
			trace_add(vb, "next_proccess: wake_up(IMG: th_busy)");
			wake_up_interruptible(&dev->th_img.th_busy);
		}
		if(vb->state_frame != STATE_FRAME_IMMEDIATE){
			/* next process is TIMER */
			trace_add(vb, "next_proccess: wake_up(TMR: th_idle)");
			wake_up_interruptible(&dev->th_tmr.th_idle);
			break;
		}
		/* FALL THROUGH */
	case TMR_CALLBACK:
		if(call_func == TMR_CALLBACK){
			trace_add(vb, "next_proccess: wake_up(TMR: th_busy)");
			wake_up_interruptible(&dev->th_tmr.th_busy);
		}
		/* next process is LCD */
		trace_add(vb, "next_proccess: wake_up(LCD: th_idle)");
		wake_up_interruptible(&dev->th_lcd.th_idle);
		break;
	}

	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_chk_ipuimg
* FUNCTION : 
* RETURN   : -
* NOTE     : 
* CREATE   : 2007/12/06 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_chk_ipuimg(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	unsigned long width, height;

	if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
		/* use IPU-ROT */
		width  = vb->rot_buf->dst_data.hsize;
		height = vb->rot_buf->dst_data.vsize;
	}
	else{
		/* not use IPU-ROT */
		width  = vb->save_effect.source_crop.width;
		height = vb->save_effect.source_crop.height;
	}

	if((vb->save_effect.screen.width != 0) && (vb->save_effect.screen.height != 0)){
		if(vb->output == V4L2_OUTPUT_LCD){
			/* output LCD */
			trace_add(vb, "chk_ipuimg: output LCD");

			if((vb->save_effect.screen.width != width) || (vb->save_effect.screen.height != height)){
				switch(vb->pixelformat){
#if VF_YUV422PL
				case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar *//* FALL THROUGH */
#endif
				case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar */
					trace_add(vb, "chk_ipuimg: ->IMG(resize & fmt convert)");
					return IMG_OP_MODE_03_BIT; /* Resize & Color Format Convert for IMG */
				default:
					trace_add(vb, "chk_ipuimg: ->IMG(resize)");
					return IMG_OP_MODE_03_BIT; /* Resize for IMG */
				}
			}
		}
#ifdef CONFIG_MP200_NTS	
		else{
			/* output NTSC */
			trace_add(vb, "chk_ipuimg: output NTSC");
		}
#endif /* CONFIG_MP200_NTS */
	}

	trace_add(vb, "chk_ipuimg: ->not IMG");
	return 0;
}


#ifdef CONFIG_VIDEO_MP200_FRAMESKIP
#define SKIP_COUNT_MAX 1
/*****************************************************************************
* MODULE   : mp200_v4l2_chk_frameskip
* FUNCTION : check condition of frameskip function.
* RETURN   : STATE_FRAME_DONE    : not skipped
*          : STATE_FRAME_SKIPPED : skipped
* NOTE     : 
* CREATE   : 2009/04/10 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_chk_frameskip(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, struct th_object *th_obj)
{
#if (SKIP_COUNT_MAX > 0)
	static int skip_count = 0;
#endif
	unsigned long flags;
	long adjust_time;
	int  adjust;
	int  ret = STATE_FRAME_DONE;
	enum call_func_num call_func = VBQ_QUEUE;

	trace_add(vb, "chk_frameskip");

	spin_lock_irqsave(&dev->vbq_lock, flags);

	/* this process */
	if(th_obj->call_request == mp200_v4l2_rot_request){
		/* rot thread */
		adjust = 0;
		call_func = ROT_REQUEST;
	}
	else if(th_obj->call_request == mp200_v4l2_img_request){
		/* img thread */
		adjust = 0;
		call_func = IMG_REQUEST;
	}
	else if(th_obj->call_request == mp200_v4l2_tmr_request){
		/* tmr thread */
		adjust = 1; /* IMC */
		call_func = TMR_REQUEST;
	}
	else{
		/* lcd thread */
		if(vb->state_proccess & STATE_PROCCESS_TMR_COMPLETE){
			adjust = 0;
		}
		else{
			adjust = 1;
		}
		call_func = LCD_REQUEST;
	}

	if(adjust > 0){
		if(vb->state_frame == STATE_FRAME_IMMEDIATE){
#if (SKIP_COUNT_MAX > 0)
			skip_count = 0;
#endif
		}
		else{
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			down(&dev->sem_lcdout);
			trace_add(vb, "chk_frameskip:down sem_lcdout");
			spin_lock_irqsave(&dev->vbq_lock, flags);

#if (SKIP_COUNT_MAX > 0)
			if (skip_count >= SKIP_COUNT_MAX){
				skip_count = 0;
			}
			else{
#endif
				adjust_time  = mp200_v4l2_tmr_calc_waittime(dev, vb);
				adjust_time *= adjust;

#if _TRACE_DBG
				memset(cp_name, 0, sizeof(cp_name));
				sprintf(cp_name, "chk_frameskip: waittime(%ld) = wait(%ld) - adjust(%ld)", dev->timer.wait_time - adjust_time, dev->timer.wait_time, adjust_time);
				trace_add(vb, cp_name);
#endif /* _TRACE_DBG */

				if(dev->timer.wait_time < adjust_time){
#if (SKIP_COUNT_MAX > 0)
					skip_count++;
#endif
					vb->state       = STATE_CANCELED;
					vb->state_frame = STATE_FRAME_SKIPPED;
					ret             = STATE_FRAME_SKIPPED;

					up(&dev->sem_lcdout);
					trace_add(vb, "chk_frameskip:up sem_lcdout");

					mp200_v4l2_next_proccess(dev, vb, 0, call_func);
				}
#if (SKIP_COUNT_MAX > 0)
				else{
					skip_count = 0;
				}
			}
#endif
		}
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return ret;
}
#endif /* CONFIG_VIDEO_MP200_FRAMESKIP */


/*===============================================================*/
/* kernel timer function                                         */
/*===============================================================*/
/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_cmpare_time
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2009/04/06 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline long mp200_v4l2_tmr_cmpare_time(struct timeval *start, struct timeval *end)
{
	long delta = 0;

	if(end->tv_sec > start->tv_sec){
		delta = (end->tv_sec - start->tv_sec) * 1000;
		if(end->tv_usec > start->tv_usec){
			delta += (end->tv_usec - start->tv_usec) / 1000;
		}else{
			delta -= (start->tv_usec - end->tv_usec) / 1000;
		}
	}else if(end->tv_sec == start->tv_sec){
		if(end->tv_usec > start->tv_usec){
			delta = (end->tv_usec - start->tv_usec) / 1000;
		}else{
			delta = STATE_FRAME_IMMEDIATE;
		}
	}else{
		delta = STATE_FRAME_IMMEDIATE;
	}
	return delta;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_calc_waittime
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline long mp200_v4l2_tmr_calc_waittime(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	long adjust_time;

	do_gettimeofday(&dev->timer.now_time);
	dev->timer.wait_time = mp200_v4l2_tmr_cmpare_time(&dev->timer.now_time, &vb->ts);
	printk_dbg(_TMR_DBG, "waittime(%ld)\n", dev->timer.wait_time);

#if _TRACE_DBG
	memset(cp_name, 0, sizeof(cp_name));
	sprintf(cp_name, "calc_waittime: vb->ts.tv_usec     (%ld)", vb->ts.tv_usec);
	trace_add(vb, cp_name);
	memset(cp_name, 0, sizeof(cp_name));
	sprintf(cp_name, "calc_waittime: dev->timer.now_time(%ld)", dev->timer.now_time.tv_usec);
	trace_add(vb, cp_name);
#endif /* _TRACE_DBG */

	if(vb->output == V4L2_OUTPUT_LCD){
		adjust_time = dev->timer.adjust_time_lcd;
	}
	else{
		adjust_time = dev->timer.adjust_time_ntsc;
	}
	return adjust_time;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_chk_waittime
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2009/04/10 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_tmr_chk_waittime(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	long adjust_time = mp200_v4l2_tmr_calc_waittime(dev, vb);
	long roundup;

	if(vb->output == V4L2_OUTPUT_LCD)       roundup = ROUNDUP_ADJUST_LCD;
	else if(vb->output == V4L2_OUTPUT_NTSC) roundup = ROUNDUP_ADJUST_NTSC;
	else                                    roundup = ROUNDUP_ADJUST_PAL;
	adjust_time += roundup;

#if _TRACE_DBG
	memset(cp_name, 0, sizeof(cp_name));
	sprintf(cp_name, "chk_waittime: waittime(%ld) = wait(%ld) - adjust(%ld)", dev->timer.wait_time - adjust_time, dev->timer.wait_time, adjust_time);
	trace_add(vb, cp_name);
#endif /* _TRACE_DBG */

	if(dev->timer.wait_time >= adjust_time){
		dev->timer.wait_time -= adjust_time;
	}
	else{
		dev->timer.wait_time = STATE_FRAME_IMMEDIATE;
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_calc_adjusttime
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline long mp200_v4l2_tmr_calc_adjusttime(struct mp200_v4l2_device *dev)
{
	long adjust_time;

	do_gettimeofday(&dev->timer.comp_time);
	adjust_time = mp200_v4l2_tmr_cmpare_time(&dev->timer.do_time, &dev->timer.comp_time);
	printk_dbg(_TMR_DBG, "adjusttime(%ld)\n", adjust_time);

	return adjust_time;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_init
* FUNCTION : wait to call LCD driver init function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_tmr_init(struct mp200_v4l2_device *dev)
{
	dev->timer.adjust_time_lcd  = DEFAULT_ADJUST_LCD;
	dev->timer.adjust_time_ntsc = DEFAULT_ADJUST_NTSC;

	init_timer(&dev->timer.wait_timer);
	dev->timer.wait_timer.function = mp200_v4l2_tmr_callback_top;
	dev->timer.wait_timer.data = (unsigned long)dev;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_request
* FUNCTION : wait to call LCD driver function.
* RETURN   : 0: success
* NOTE     : 
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_tmr_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int dummy)
{
	int ret = 0;
	unsigned long flags;

	printk_dbg(_TMR_DBG, "in.\n");
	trace_add(vb, "tmr_request");

#ifndef CONFIG_VIDEO_MP200_FRAMESKIP
	down(&dev->sem_lcdout);
	trace_add(vb, "tmr_request:down sem_lcdout");
#endif

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	/* check next queue status */
	if(mp200_v4l2_next_proccess(dev, vb, 0, TMR_REQUEST)){
		printk_dbg(_TMR_DBG, "STATE_CANCELED.\n");
		ret = -1;
		up(&dev->sem_lcdout); /* release semafore */
		trace_add(vb, "tmr_request:up sem_lcdout");
	}
	else{
		mp200_v4l2_tmr_chk_waittime(dev, vb);
		dev->timer.vb   = vb;
		vb->state_frame = STATE_FRAME_DONE;

		if(dev->timer.wait_time >= 10){
			trace_add(vb, "tmr_request: add_timer()");
			perf_add(vb->sequence, PHS_TMR);
			dev->timer.wait_timer.expires = jiffies + (dev->timer.wait_time * HZ / 1000);
			add_timer(&dev->timer.wait_timer);
		}
		else{
			trace_add(vb, "tmr_request: queue_work()");
#if ENABLE_DELAY
			queue_work(dev->v4l2_workqueue, &dev->wk_tmr_callback_bottom);
#else
			mp200_v4l2_tmr_callback_bottom_do(dev);
#endif
		}
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return ret;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_callback_top
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_tmr_callback_top(unsigned long data)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)data;

	trace_add(((struct videobuf_buffer *)(dev->timer.vb)), "tmr_callback_top_half");
	perf_add(((struct videobuf_buffer *)(dev->timer.vb))->sequence, PHS_TMR);
#if ENABLE_DELAY
	queue_work(dev->v4l2_workqueue, &dev->wk_tmr_callback_bottom);
#else
	mp200_v4l2_tmr_callback_bottom_do(dev);
#endif
}


/*****************************************************************************
* MODULE   : mp200_v4l2_tmr_callback_bottom_do
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
#if ENABLE_DELAY
static void mp200_v4l2_tmr_callback_bottom_do(struct work_struct *num)
{
	struct mp200_v4l2_device *dev = container_of(num, struct mp200_v4l2_device, wk_tmr_callback_bottom);
	struct videobuf_buffer   *vb  = dev->timer.vb;
	unsigned long flags;
	long wait_time;
#else
static inline void mp200_v4l2_tmr_callback_bottom_do(struct mp200_v4l2_device *dev)
{
	struct videobuf_buffer *vb = dev->timer.vb;
#endif
	dev->timer.vb = NULL;

	trace_add(vb, "tmr_callback_bottom_half");
	printk_dbg(_WKQ_DBG, "in\n");
	printk_dbg(_VBQ_DBG, "(%p) ->prev(%p) ->next(%p) ->state(%d)\n", &vb->stream, vb->stream.prev, vb->stream.next, vb->state);

#if ENABLE_DELAY
	spin_lock_irqsave(&dev->vbq_lock, flags);
	mp200_v4l2_tmr_chk_waittime(dev, vb);
	wait_time = dev->timer.wait_time;
	spin_unlock_irqrestore(&dev->vbq_lock, flags);

	if(wait_time > 10){
		trace_add(vb, "tmr_callback_bottom_half: msleep(TMR) <--");
		msleep((unsigned int)wait_time);
		trace_add(vb, "tmr_callback_bottom_half: msleep(TMR) -->");
	}
#endif

	/* check next queue status */
	if(mp200_v4l2_next_proccess(dev, vb, 0, TMR_CALLBACK)){
		printk_dbg(_TMR_DBG, "STATE_CANCELED.\n");
		down_trylock(&dev->sem_lcdout);
		up(&dev->sem_lcdout); /* release semafore */
		trace_add(vb, "tmr_callback:up sem_lcdout");
	}
}


/*===============================================================*/
/* IPU proccessing function prottyped                            */
/*===============================================================*/
/*****************************************************************************
* MODULE   : mp200_v4l2_ipu_callback_main
* FUNCTION : IPU-ROT driver callback function.
* RETURN   : -
* NOTE     : When the animation rotation is completed/error, 
*          : the IPU-ROT driver calls this function.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_ipu_callback_main(struct mp200_v4l2_device *dev, struct th_object *th_obj, enum call_func_num call_func, int flag)
{
	unsigned long flags;
	struct videobuf_buffer *vb = th_obj->th_buf->vb;

	spin_lock_irqsave(&dev->vbq_lock, flags);
	th_obj->th_buf->vb = NULL;

	trace_add_th(vb, "ipu_callback(", ")");
	perf_add_th(vb->sequence);

	/* check callback flag */
	if(flag != IPU_CALLBACK_SUCCESS){
		vb->state = STATE_CANCELED;
	}

	/* check next queue status */
	if(mp200_v4l2_next_proccess(dev, vb, 0, call_func)){
		printk_dbg((_IMG_DBG || _ROT_DBG), "STATE_CANCELED.\n");
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ipu_request_main
* FUNCTION : request to IPU driver function.
* RETURN   :  0: success
*          : -1: buffer busy
* NOTE     : This function the movie rotation demand to the IPU driver.
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ipu_request_main(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, struct th_object *th_obj, enum call_func_num call_func)
{
	enum buffer_select  buf_sel;
	int index = -1;
	unsigned long flags;
	int i;

	spin_lock_irqsave(&dev->vbq_lock, flags);
	trace_add(vb, "ipu_call_func_main");
	dbg_lock(&dev->vbq_lock, "vbq_lock\n");

	/* check use buffer */
	for(i = 0; i < NUM_BUF_V4L2; i++){
		if(th_obj->th_buf->buf[i].state == TMPBUF_IDLE){
			buf_sel = i;
			break;
		}
	}
	if (i >= NUM_BUF_V4L2){
		dbg_lock(&dev->vbq_lock, "buffer busy\n");
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -1;
	}

	/* check next queue status */
	if(mp200_v4l2_next_proccess(dev, vb, buf_sel, call_func)){
		dbg_lock(&dev->vbq_lock, "STATE_CANCELED\n");
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -1;
	}

	/* make IPU request data */
	perf_add_th(vb->sequence);
	if(th_obj->th_buf->make_request(dev, vb, index, buf_sel)){
		/* check next queue status */
		if(mp200_v4l2_next_proccess(dev, vb, 0, VBQ_COMPLETE)){
			/* CANCELED */
			trace_add(vb, "ipu_request_main: next_proccess canceled.\n");
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			return -1;
		}
		mp200_v4l2_core_vbq_complete(dev, vb, 1);
		printk_info("IPU request data making error.\n\n");
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -1;
	}

	/* set pointer to callback function from rot driver */
	th_obj->th_buf->ipu_request.callback    = th_obj->th_buf->callback;
	th_obj->th_buf->ipu_request.ipu_hw_func = th_obj->th_buf->ipu_hw_func;
	/* set pointer to Device information into dev->req.data */
	th_obj->th_buf->ipu_request.data        = (unsigned long)dev;
	th_obj->th_buf->vb                      = (void *)vb;

	/* call IPU driver */
	mp200_ipu_regist((struct mp200_ipu_req *)(&th_obj->th_buf->ipu_request));

	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_rot_callback
* FUNCTION : IPU-ROT driver callback function.
* RETURN   : -
* NOTE     : When the animation rotation is completed/error, 
*          : the IPU-ROT driver calls this function.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_rot_callback(unsigned long sequence, int flag)
{
	struct mp200_v4l2_device *dev = mp200_dev;

	trace_add(dev->th_rot.th_buf->vb, "rot_callback");
	printk_dbg((_ROT_DBG & 0x2), "call in.\n");

	/* IPU callback main function */
	mp200_v4l2_ipu_callback_main(dev, &dev->th_rot, ROT_CALLBACK, flag);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_rot_request
* FUNCTION : request to IPU-ROT driver function.
* RETURN   :  0: success
*          : -1: buffer busy
* NOTE     : This function the movie rotation demand to the IPU-ROT driver.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_rot_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int dummy)
{
	trace_add(vb, "rot_call_func");
	return mp200_v4l2_ipu_request_main(dev, vb, &dev->th_rot, ROT_REQUEST);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_rot_make_req
* FUNCTION : make rotate request data to ROT driver.
* RETURN   :  0: success
*          : -1: error
* NOTE     : The register information table make, requested to the ROT driver.
* CREATE   : 2006/06/06 M.Haguro
* UPDATE   : 2006/06/14 M.Haguro
******************************************************************************/
static int mp200_v4l2_rot_make_req(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int index, enum buffer_select buf_sel)
{
	struct buf_object  *rot_buf  = dev->th_rot.th_buf;
	struct _IMAGE_DATA *rot_data = &vb->rot_buf->dst_data;
	unsigned long base_addr, src_yrgbaddr = 0, src_uvaddr = 0, src_vaddr = 0;
	unsigned long rot_mode, format;
	int index_req = 0;
	int bpp_y, bpp_uv, bpp_v; /* bit par pixel  */
	int ppl_y, ppl_uv, ppl_v; /* pixel par line */

	trace_add(vb, "make_rot_req");
	memset(&rot_buf->ipu_request, 0, sizeof(struct mp200_ipu_req_v4l2));    /* init IPU request data */
	memset(rot_data, 0, sizeof(struct _IMAGE_DATA));

	/*******************************
	    set ROT_MODE
	********************************/
	rot_mode = (ROT_NEWEN_BIT | ROT_MODE_SRCENDIAN_LITTLE | ROT_MODE_DSTENDIAN_LITTLE);

	switch(vb->save_effect.movie_angle){
	default:
	case ROT_MODE_ROT_MOD_0:   return -1;                               /*   0 deg */
	case ROT_MODE_ROT_MOD_90:  rot_mode |= ROT_MODE_ROT_MOD_90;  break; /*  90 deg */
	case ROT_MODE_ROT_MOD_180: rot_mode |= ROT_MODE_ROT_MOD_180; break; /* 180 deg */
	case ROT_MODE_ROT_MOD_270: rot_mode |= ROT_MODE_ROT_MOD_270; break; /* 270 deg */
	}

	/*******************************
	    set ROT_FORMAT
	********************************/
	switch(vb->pixelformat){
	case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
		format = ROT_DSTFMT_YUV420_SEMI_PLANAR | ROT_SRCFMT_YUV420_SEMI_PLANAR;
		bpp_y = 8;  bpp_uv = 8; bpp_v = 0; ppl_y = 1; ppl_uv = 2; ppl_v = 0;
		break;
	case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
		format = ROT_DSTFMT_YUV422_SEMI_PLANAR | ROT_SRCFMT_YUV422_SEMI_PLANAR;
		bpp_y = 8;  bpp_uv = 8; bpp_v = 0; ppl_y = 1; ppl_uv = 1; ppl_v = 0;
		break;
	case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
		format = ROT_DSTFMT_YUV420_PLANAR      | ROT_SRCFMT_YUV420_PLANAR;
		bpp_y = 8;  bpp_uv = 4; bpp_v = 4; ppl_y = 1; ppl_uv = 2; ppl_v = 2;
		break;
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
		format = ROT_DSTFMT_YUV422_PLANAR      | ROT_SRCFMT_YUV422_PLANAR;
		bpp_y = 8;  bpp_uv = 8; bpp_v = 8; ppl_y = 1; ppl_uv = 2; ppl_v = 2;
		break;
#endif
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
		format = ROT_DSTFMT_YUV422_INTERLEAVE  | ROT_SRCFMT_YUV422_INTERLEAVE;
		bpp_y = 16; bpp_uv = 0; bpp_v = 0; ppl_y = 1; ppl_uv = 0; ppl_v = 0;
		break;
#endif
	default:
		return -1;
	}

	/*******************************
	    set ROT request data
	********************************/
	if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180){
		rot_data->hsize = vb->save_effect.source_crop.width;
		rot_data->vsize = vb->save_effect.source_crop.height;
		rot_data->size  = vb->save_effect.source_crop.width * bpp_y / 8;
	}
	else{
		rot_data->hsize = vb->save_effect.source_crop.height;
		rot_data->vsize = vb->save_effect.source_crop.width;
		rot_data->size  = vb->save_effect.source_crop.height * bpp_y / 8;
	}
	vb->rot_buf->pixelformat = vb->pixelformat;

	/* check next destination buffer */
	base_addr = vb->rot_buf->paddr;
	switch(vb->pixelformat){
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
		/* src */
		src_yrgbaddr       = vb->base_addr.PhysAddr_Y  + vb->save_effect.source_crop.top * vb->bytesperline * bpp_y  / 8 / ppl_y  + vb->save_effect.source_crop.left * bpp_y  / 8;
		src_uvaddr         = 0;
		src_vaddr          = 0;
		/* dst */
		rot_data->yrgbaddr = base_addr;
		rot_data->uvaddr   = 0;
		rot_data->vaddr    = 0;
		break;
#endif
	case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
	case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
		/* src */
		src_yrgbaddr       = vb->base_addr.PhysAddr_Y  + vb->save_effect.source_crop.top * vb->bytesperline * bpp_y  / 8 / ppl_y  + vb->save_effect.source_crop.left * bpp_y  / 8;
		src_uvaddr         = vb->base_addr.PhysAddr_UV + vb->save_effect.source_crop.top * vb->bytesperline * bpp_uv / 8 / ppl_uv + vb->save_effect.source_crop.left * bpp_uv / 8;
		src_vaddr          = 0;
		/* dst */
		rot_data->yrgbaddr = base_addr;
		rot_data->uvaddr   = rot_data->yrgbaddr + rot_data->vsize * rot_data->hsize * bpp_y  / 8 / ppl_y;
		rot_data->vaddr    = 0;
		break;
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
#endif
	case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
		/* src */
		src_yrgbaddr       = vb->base_addr.PhysAddr_Y  + vb->save_effect.source_crop.top * vb->bytesperline * bpp_y  / 8 / ppl_y  + vb->save_effect.source_crop.left * bpp_y  / 8;
		src_uvaddr         = vb->base_addr.PhysAddr_UV + vb->save_effect.source_crop.top * vb->bytesperline * bpp_uv / 8 / ppl_uv + vb->save_effect.source_crop.left * bpp_uv / 8;
		src_vaddr          = vb->base_addr.PhysAddr_V  + vb->save_effect.source_crop.top * vb->bytesperline * bpp_v  / 8 / ppl_v  + vb->save_effect.source_crop.left * bpp_v  / 8;
		/* dst */
		rot_data->yrgbaddr = base_addr;
		rot_data->uvaddr   = rot_data->yrgbaddr + rot_data->vsize * rot_data->hsize * bpp_y  / 8 / ppl_y;
		rot_data->vaddr    = rot_data->uvaddr   + rot_data->vsize * rot_data->hsize * bpp_uv / 8 / ppl_uv;
		break;
	}

	/*******************************
	    set ROT register
	********************************/
	/*==============================
	    ROT_MODE
	================================*/
	RotRequest(rot_mode,                           ROT_MODE);
	RotRequest(0x05,                               ROT_FRAME);
	RotRequest(format,                             ROT_FORMAT);

	/*==============================
	    source image
	================================*/
	RotRequest(src_yrgbaddr,                       ROT_SRCYADR_A);
	RotRequest(src_uvaddr,                         ROT_SRCUVADR_A);
	RotRequest(src_vaddr,                          ROT_SRCVADR_A);
	RotRequest(vb->bytesperline,                   ROT_SRCSIZE);
	RotRequest(vb->save_effect.source_crop.width,  ROT_SRCHSIZE);
	RotRequest(vb->save_effect.source_crop.height, ROT_SRCVSIZE);

	/*==============================
	    destination image
	================================*/
	RotRequest(rot_data->yrgbaddr,                 ROT_DSTYADR_A);
	RotRequest(rot_data->uvaddr,                   ROT_DSTUVADR_A);
	RotRequest(rot_data->vaddr,                    ROT_DSTVADR_A);
	RotRequest(rot_data->size,                     ROT_DSTSIZE);

	/*==============================
	    Start ROT
	================================*/
	RotRequest(ROT_REQ_BIT,                        ROT_REQ);

	/*==============================
	    end mark
	================================*/
	RotRequest(-1, -1);

	printk_dbg(_ROT_DBG, "=== mp200_ipu_regist() =======================================\n");
	printk_dbg(_ROT_DBG, " ROT_MODE          > 0x%08lx\n", rot_mode);
	printk_dbg(_ROT_DBG, "    rotation = %ddeg\n", vb->save_effect.movie_angle *90);
	switch(vb->pixelformat){
	case V4L2_PIX_FMT_UYVY:		printk_dbg(_ROT_DBG, "    YUV422 Px -> YUV422 Px\n");	break;
	case V4L2_PIX_FMT_NV422:	printk_dbg(_ROT_DBG, "    YUV422 Pl2 -> YUV422 Pl2\n");	break;
	case V4L2_PIX_FMT_YUV422P:	printk_dbg(_ROT_DBG, "    YUV422 P -> YUV422 P\n");	break;
	case V4L2_PIX_FMT_NV12:		printk_dbg(_ROT_DBG, "    YUV420 Pl2 -> YUV420 Pl2\n");	break;
	case V4L2_PIX_FMT_YVU420:	printk_dbg(_ROT_DBG, "    YUV420 P -> YUV420 P\n");	break;
	}
	printk_dbg(_ROT_DBG, "    Src   > w = %3d  h = %3d\n", vb->save_effect.source_crop.width, vb->save_effect.source_crop.height);
	printk_dbg(_ROT_DBG, "    Dest  > w = %3d\n\n", rot_data->hsize);
	printk_dbg(_ROT_DBG, "    SrcAddrY    = 0x%08lx      DestAddrY   = 0x%08lx\n", src_yrgbaddr, rot_data->yrgbaddr);
	printk_dbg(_ROT_DBG, "    SrcAddrUV   = 0x%08lx      DestAddrUV  = 0x%08lx\n", src_uvaddr,   rot_data->uvaddr  );
	printk_dbg(_ROT_DBG, "    SrcAddrV    = 0x%08lx      DestAddrV   = 0x%08lx\n", src_vaddr,    rot_data->vaddr   );
	printk_dbg(_ROT_DBG, "\n");
	printk_dbg(_ROT_DBG, " ROT_FRAME         > 0x05\n");
	printk_dbg(_ROT_DBG, "\n");
	printk_dbg(_ROT_DBG, " ROT_SRCYADR_A     > 0x%08lx\n", src_yrgbaddr);
	printk_dbg(_ROT_DBG, " ROT_SRCUVADR_A    > 0x%08lx\n", src_uvaddr  );
	printk_dbg(_ROT_DBG, " ROT_SRCVADR_A     > 0x%08lx\n", src_vaddr   );
	printk_dbg(_ROT_DBG, " ROT_SRCSIZE       > %d\n",      vb->bytesperline);
	printk_dbg(_ROT_DBG, " ROT_SRCHSIZE      > %d\n",      vb->save_effect.source_crop.width);
	printk_dbg(_ROT_DBG, " ROT_SRCVSIZE      > %d\n",      vb->save_effect.source_crop.height);
	printk_dbg(_ROT_DBG, "\n");
	printk_dbg(_ROT_DBG, " ROT_DSTYADR_A     > 0x%08lx\n", rot_data->yrgbaddr);
	printk_dbg(_ROT_DBG, " ROT_DSTUVADR_A    > 0x%08lx\n", rot_data->uvaddr  );
	printk_dbg(_ROT_DBG, " ROT_DSTVADR_A     > 0x%08lx\n", rot_data->vaddr   );
	printk_dbg(_ROT_DBG, " ROT_DSTSIZE       > %d\n",      rot_data->size);
	printk_dbg(_ROT_DBG, "\n");
	printk_dbg(_ROT_DBG, " ROT_REQ           > 0x%08x\n", ROT_REQ_BIT);
	printk_dbg(_ROT_DBG, "==============================================================\n\n");

	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_img_callback
* FUNCTION : IPU-ROT driver callback function.
* RETURN   : -
* NOTE     : When the animation rotation is completed/error, 
*          : the IPU-IMG driver calls this function.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_img_callback(unsigned long sequence, int flag)
{
	struct mp200_v4l2_device *dev = mp200_dev;

	trace_add(dev->th_img.th_buf->vb, "img_callback");
	printk_dbg((_ROT_DBG & 0x2), "call in.\n");

	/* IPU callback main function */
	mp200_v4l2_ipu_callback_main(dev, &dev->th_img, IMG_CALLBACK, flag);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_img_request
* FUNCTION : request to IPU-IMG driver function.
* RETURN   :  0: success
*          : -1: buffer busy
* NOTE     : This function the movie rotation demand to the IPU-IMG driver.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_img_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int dummy)
{
	trace_add(vb, "img_call_func");
	return mp200_v4l2_ipu_request_main(dev, vb, &dev->th_img, IMG_REQUEST);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_img_make_req
* FUNCTION : make rotate request data to IPU-IMG driver.
* RETURN   :  0: success
*          : -1: error
* NOTE     : The register information table make, requested to the ROT driver.
* CREATE   : 2007/12/06 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_img_make_req(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int index, enum buffer_select buf_sel)
{
	struct buf_object  *img_buf  = dev->th_img.th_buf;
	struct _IMAGE_DATA *img_data = &vb->img_buf->dst_data;
	struct _IMAGE_DATA  src_data;

	unsigned long base_addr;
	unsigned long img_mode, format;
	int index_req = 0;
	int bpp_y, bpp_uv, bpp_v; /* bit par pixel  */
	int ppl_y, ppl_uv, ppl_v; /* pixel par line */

	unsigned long	endian = IMG_PEL_ENDIAN_IN_ENDI_LITTLE | IMG_PEL_ENDIAN_OUT_ENDI_LITTLE;

	trace_add(vb, "make_img_req");
	memset(&img_buf->ipu_request, 0, sizeof(struct mp200_ipu_req_v4l2));    /* init IPU request data */
	memset(img_data,  0, sizeof(struct _IMAGE_DATA));
	memset(&src_data, 0, sizeof(struct _IMAGE_DATA));

	/*******************************
	    set IMG_MODE
	********************************/
	img_mode  = IMG_NEWEN_BIT;
	img_mode |= mp200_v4l2_chk_ipuimg(dev, vb);

	switch(vb->pixelformat){
	case V4L2_PIX_FMT_NV12:   /* YUV420 Semi-Planar */
		format = IMG_FORMAT_DSTFMT_YUV420_SEMI_PLANAR | IMG_FORMAT_SRCFMT_B_YUV420_SEMI_PLANAR;
		bpp_y = 8;  bpp_uv = 8; bpp_v = 0; ppl_y = 1; ppl_uv = 2; ppl_v = 0;
		break;
	case V4L2_PIX_FMT_YVU420: /* YUV420 Planar      */
//@@		format = IMG_FORMAT_DSTFMT_YUV420_PLANAR | IMG_FORMAT_SRCFMT_B_YUV420_PLANAR;
		format = IMG_FORMAT_DSTFMT_YUV420_SEMI_PLANAR | IMG_FORMAT_SRCFMT_B_YUV420_PLANAR;
		bpp_y = 8;  bpp_uv = 4; bpp_v = 4; ppl_y = 1; ppl_uv = 2; ppl_v = 2;
		break;
	case V4L2_PIX_FMT_NV422:  /* YUV422 Semi-Planar */
		format = IMG_FORMAT_DSTFMT_YUV422_SEMI_PLANAR | IMG_FORMAT_SRCFMT_B_YUV422_SEMI_PLANAR;
		bpp_y = 8;  bpp_uv = 8; bpp_v = 0; ppl_y = 1; ppl_uv = 1; ppl_v = 0;
		break;
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
//@@		format = IMG_FORMAT_DSTFMT_YUV422_PLANAR      | IMG_FORMAT_SRCFMT_B_YUV422_PLANAR;
		format = IMG_FORMAT_DSTFMT_YUV422_SEMI_PLANAR | IMG_FORMAT_SRCFMT_B_YUV422_PLANAR;
		bpp_y = 8;  bpp_uv = 8; bpp_v = 8; ppl_y = 1; ppl_uv = 2; ppl_v = 2;
		break;
#endif
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:   /* YUV422 Interleave  */
		format = IMG_FORMAT_DSTFMT_YUV422_INTERLEAVE  | IMG_FORMAT_SRCFMT_B_YUV422_INTERLEAVE;
		bpp_y = 16; bpp_uv = 0; bpp_v = 0; ppl_y = 1; ppl_uv = 0; ppl_v = 0;
		break;
#endif
	default:
		return -1;
	}

	/*******************************
	    set IMG request data
	********************************/
	/* check next destination buffer */
	base_addr = vb->img_buf->paddr;

	if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
		memcpy(&src_data, &vb->rot_buf->dst_data, sizeof(struct _IMAGE_DATA));
		vb->img_buf->pixelformat = vb->rot_buf->pixelformat;
	}else{
		/* not use IPU-ROT */
		src_data.yrgbaddr = vb->base_addr.PhysAddr_Y;
		src_data.uvaddr   = vb->base_addr.PhysAddr_UV;
		src_data.vaddr    = vb->base_addr.PhysAddr_V;
		src_data.x        = vb->save_effect.source_crop.left;
		src_data.y        = vb->save_effect.source_crop.top;
		src_data.hsize    = vb->save_effect.source_crop.width;
		src_data.vsize    = vb->save_effect.source_crop.height;
		src_data.size     = vb->bytesperline;
		vb->img_buf->pixelformat = vb->pixelformat;
	}

	switch(vb->img_buf->pixelformat){
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
		/* src */
		src_data.yrgbaddr  = src_data.yrgbaddr  + src_data.y * src_data.size * bpp_y  / 8 / ppl_y + src_data.x * bpp_y / 8;
		src_data.uvaddr    = 0;
		src_data.vaddr     = 0;
		/* dst */
		img_data->yrgbaddr = base_addr;
		img_data->uvaddr   = 0;
		img_data->vaddr    = 0;
		break;
#endif
	case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
	case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
		/* src */
		src_data.yrgbaddr  = src_data.yrgbaddr + src_data.y * src_data.size * bpp_y  / 8 / ppl_y  + src_data.x * bpp_y  / 8;
		src_data.uvaddr    = src_data.uvaddr   + src_data.y * src_data.size * bpp_uv / 8 / ppl_uv + src_data.x * bpp_uv / 8;
		src_data.vaddr     = 0;
		/* dst */
		img_data->yrgbaddr = base_addr;
		img_data->uvaddr   = img_data->yrgbaddr + vb->save_effect.screen.width * vb->save_effect.screen.height * bpp_y  / 8 / ppl_y;
		img_data->vaddr    = 0;
		break;
	case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
#endif
		/* src */
		src_data.yrgbaddr  = src_data.yrgbaddr + src_data.y * src_data.size * bpp_y  / 8 / ppl_y  + src_data.x * bpp_y  / 8;
		src_data.uvaddr    = src_data.uvaddr   + src_data.y * src_data.size * bpp_uv / 8 / ppl_uv + src_data.x * bpp_uv / 8;
		src_data.vaddr     = src_data.vaddr    + src_data.y * src_data.size * bpp_v  / 8 / ppl_v  + src_data.x * bpp_v  / 8;
		/* dst */
		img_data->yrgbaddr = base_addr;
		img_data->uvaddr   = img_data->yrgbaddr + vb->save_effect.screen.width * vb->save_effect.screen.height * bpp_y  / 8 / ppl_y;
//@@		img_data->vaddr    = img_data->uvaddr   + vb->save_effect.screen.width * vb->save_effect.screen.height * bpp_uv / 8 / ppl_uv;
		img_data->vaddr    = 0;

		if(vb->img_buf->pixelformat == V4L2_PIX_FMT_YVU420){
			vb->img_buf->pixelformat = V4L2_PIX_FMT_NV12;    /* YUV420 Semi-Planar */
		}
#if VF_YUV422PL
		else{
			vb->img_buf->pixelformat = V4L2_PIX_FMT_NV422;   /* YUV422 Semi-Planar */
		}
#endif
		break;
	}

	img_data->hsize = vb->save_effect.screen.width;
	img_data->vsize = vb->save_effect.screen.height;
	img_data->size  = img_data->hsize * bpp_y / 8;

	/*******************************
	    set IMG register
	********************************/
	/*==============================
	    IMG_MODE
	================================*/
	ImgRequest(img_mode,                            IMG_MODE);
	ImgRequest(format,                              IMG_FORMAT);

	/*==============================
	    source image
	================================*/
	ImgRequest(src_data.yrgbaddr,                   IMG_SRCYRGBADR_B);
	ImgRequest(src_data.uvaddr,                     IMG_SRCUVADR_B);
	ImgRequest(src_data.vaddr,                      IMG_SRCVADR_B);
	ImgRequest(src_data.size,                       IMG_SRCSIZE_B);
	ImgRequest(src_data.hsize,                      IMG_SRCHSIZE_B);
	ImgRequest(src_data.vsize,                      IMG_SRCVSIZE_B);

	/*==============================
	    destination image
	================================*/
	ImgRequest(img_data->yrgbaddr,                  IMG_DSTYRGBADR);
	ImgRequest(img_data->uvaddr,                    IMG_DSTUVADR);
	ImgRequest(img_data->vaddr,                     IMG_DSTVADR);
	ImgRequest(img_data->size,                      IMG_DSTSIZE);

	if((img_mode & IMG_OP_MODE_BIT) == IMG_OP_MODE_03_BIT){
		unsigned long step_h, step_v, fold_h, fold_v;

		/* resize */
		ImgRequest(img_data->hsize,             IMG_DSTHSIZE);
		ImgRequest(img_data->vsize,             IMG_DSTVSIZE);

		/* step */
		step_h = src_data.hsize * 256 / img_data->hsize;
		step_v = src_data.vsize * 256 / img_data->vsize;
		ImgRequest(step_h,                      IMG_HSTEP);
		ImgRequest(step_v,                      IMG_VSTEP);

		/* fold */
		fold_h = img_data->hsize * 64 / src_data.hsize;
		fold_v = img_data->vsize * 64 / src_data.vsize;
		ImgRequest(fold_h,                      IMG_HFOLD);
		ImgRequest(fold_v,                      IMG_VFOLD);
	}

#if 0
//@	if(img_mode & IMG_OVERLAY_BIT){
//@		/* overlay */
//@		ImgRequest(vb->save_effect.screen.left, IMG_OFFSETX);
//@		ImgRequest(vb->save_effect.screen.top,  IMG_OFFSETY);
//@	}
//@
//@	if(img_mode & IMG_OP_MODE_BIT == IMG_OP_MODE_03_BIT){
//@		/* maskcolor */
//@		ImgRequest(uiMaskColor,                 IMG_MASKCOLR );
//@		ImgRequest(uiAlpha,                     IMG_ALPHA );
//@	}
#endif
	ImgRequest(endian,                              IMG_PEL_ENDIAN);

	/*==============================
	    Start IMG
	================================*/
	ImgRequest(IMG_REQ_BIT,                         IMG_REQ);

	/*==============================
	    end mark
	================================*/
	ImgRequest(-1, -1);

	printk_dbg(_IMG_DBG, "=== mp200_ipu_regist() =======================================\n");
	printk_dbg(_IMG_DBG, " IMG_MODE          > 0x%08lx\n", img_mode);
	printk_dbg(_IMG_DBG, "    Src   > w = %3d  h = %3d\n", src_data.hsize, src_data.vsize);
	printk_dbg(_IMG_DBG, "    Dest  > w = %3d\n\n", img_data->hsize);
	printk_dbg(_IMG_DBG, "    SrcAddrY    = 0x%08lx      DestAddrY   = 0x%08lx\n", src_data.yrgbaddr, img_data->yrgbaddr);
	printk_dbg(_IMG_DBG, "    SrcAddrUV   = 0x%08lx      DestAddrUV  = 0x%08lx\n", src_data.uvaddr,   img_data->uvaddr);
	printk_dbg(_IMG_DBG, "    SrcAddrV    = 0x%08lx      DestAddrV   = 0x%08lx\n", src_data.vaddr,    img_data->vaddr);
	printk_dbg(_IMG_DBG, "\n");
	printk_dbg(_IMG_DBG, " IMG_SRCYRGBADR_B  > 0x%08lx\n", src_data.yrgbaddr);
	printk_dbg(_IMG_DBG, " IMG_SRCUVADR_B    > 0x%08lx\n", src_data.uvaddr);
	printk_dbg(_IMG_DBG, " IMG_SRCVADR_B     > 0x%08lx\n", src_data.vaddr);
	printk_dbg(_IMG_DBG, "\n");
	printk_dbg(_IMG_DBG, " IMG_DSTYRGBADR_A  > 0x%08lx\n", img_data->yrgbaddr);
	printk_dbg(_IMG_DBG, " IMG_DSTUVADR_A    > 0x%08lx\n", img_data->uvaddr);
	printk_dbg(_IMG_DBG, " IMG_DSTVADR_A     > 0x%08lx\n", img_data->vaddr);
	printk_dbg(_IMG_DBG, " IMG_DSTSIZE       > %d\n",      vb->save_effect.screen.width);
	printk_dbg(_IMG_DBG, "\n");
	printk_dbg(_IMG_DBG, " IMG_REQ           > 0x%08x\n",  IMG_REQ_BIT);
	printk_dbg(_IMG_DBG, "==============================================================\n\n");

	return 0;
}


/*===============================================================*/
/* LCD proccessing function prottyped                            */
/*===============================================================*/
/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_stop_do
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2008/01/18 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_lcd_stop_do(struct work_struct *num)
{
	struct mp200_v4l2_device *dev = container_of(num, struct mp200_v4l2_device, wk_lcd_stop);

	printk_dbg((_WKQ_DBG || (_LCD_DBG & 0x2)), "lcd_call_func()\n");
	trace_add(0, "lcd_stop_do");

	if(dev->mixing){
		/* call lcd driver function. */
		queue_work(dev->v4l2_workqueue, &dev->wk_lcd_stop);
	}else{
		/* call lcd driver function. */
		mp200_v4l2_lcd_request(dev, NULL, STREAM_STOP);
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_callback
* FUNCTION : LCD driver callback function.
* RETURN   : -
* NOTE     : When the animation rotation is completed/error, 
*          : the LCD driver calls this function.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
void mp200_v4l2_lcd_callback(FRAME_DATA frame_data)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)frame_data.dev;
	struct videobuf_buffer   *vb  = (struct videobuf_buffer *)frame_data.buf;
	unsigned long flags;
	spin_lock_irqsave(&dev->vbq_lock, flags);

	printk_dbg((_LCD_DBG & 0x2), "call in.\n");
	perf_add(vb->sequence, PHS_LCD);

	if(vb->output != V4L2_OUTPUT_LCD && vb->state_frame != STATE_FRAME_IMMEDIATE){
		down_trylock(&dev->sem_lcdout);
		up(&dev->sem_lcdout); /* release semafore */
		trace_add(vb, "lcd_callback:up sem_lcdout");
	}

#if CALC_ADJUST
	if(vb->output == V4L2_OUTPUT_LCD){
		dev->timer.adjust_time_lcd = mp200_v4l2_tmr_calc_adjusttime(dev);
#if _TRACE_DBG
		memset(cp_name, 0, sizeof(cp_name));
		sprintf(cp_name, "lcd_callback: adjust(%ld)", dev->timer.adjust_time_lcd);
		trace_add(vb, cp_name);
#endif /* _TRACE_DBG */
	}
#endif

#if _PERF2_DBG
	Perf2Calc(dev, vb);
#endif

	mp200_v4l2_core_vbq_complete(dev, vb, 0);
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_refresh_callback
* FUNCTION : LCD driver callback function.
* RETURN   : -
* NOTE     : When the animation rotation is completed/error, 
*          : the LCD driver calls this function.
* CREATE   : 2008/01/15 M.Haguro
* UPDATE   : 
******************************************************************************/
void mp200_v4l2_lcd_refresh_callback(FRAME_DATA frame_data)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)frame_data.dev;
	struct videobuf_buffer   *vb  = (struct videobuf_buffer *)frame_data.buf;
	unsigned long flags;
	spin_lock_irqsave(&dev->vbq_lock, flags);

	printk_dbg((_LCD_DBG & 0x2), "call in.\n");
	trace_add(vb, "lcd_refresh_callback");
	perf_add(vb->sequence, PHS_LCD);

	if(vb->output == V4L2_OUTPUT_LCD && vb->state_frame != STATE_FRAME_IMMEDIATE){
		down_trylock(&dev->sem_lcdout);
		up(&dev->sem_lcdout); /* release semafore */
		trace_add(vb, "lcd_refresh_callback:up sem_lcdout");
	}

	/* check next queue status */
	if(mp200_v4l2_next_proccess(dev, vb, 0, LCD_CALLBACK)){
		/* CANCELED */
		trace_add(vb, "lcd_refresh_callback: next_proccess canceled.\n");
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ntsc_ready_callback
* FUNCTION : NTSC driver callback function. when finished IPU-IMG.
* RETURN   : -
* NOTE     : 
* CREATE   : 2009/04/03 M.Haguro
* UPDATE   : 
******************************************************************************/
void mp200_v4l2_ntsc_ready_callback(FRAME_DATA frame_data)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)frame_data.dev;
	struct videobuf_buffer   *vb  = (struct videobuf_buffer *)frame_data.buf;
	unsigned long flags;
	spin_lock_irqsave(&dev->vbq_lock, flags);
	printk_dbg((_LCD_DBG & 0x2), "call in.\n");
	trace_add(vb, "ntsc_ready_callback");

	if(vb->output == V4L2_OUTPUT_NTSC || vb->output == V4L2_OUTPUT_PAL){
		dev->timer.adjust_time_ntsc = mp200_v4l2_tmr_calc_adjusttime(dev);
#if _TRACE_DBG
		memset(cp_name, 0, sizeof(cp_name));
		sprintf(cp_name, "ntsc_ready_callback: adjust(%ld)", dev->timer.adjust_time_ntsc);
		trace_add(vb, cp_name);
#endif /* _TRACE_DBG */
	}

	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_set_outfunc
* FUNCTION : request to LCD driver function.
* RETURN   :  0: success
* NOTE     : This function the drawing demand to the LCD driver.
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void *mp200_v4l2_lcd_set_outfunc(int output)
{
	printk_dbg((_LCD_DBG || _ROT_DBG), "\n");

#ifdef CONFIG_MP200_NTS
	if(output == V4L2_OUTPUT_LCD){
		/* LCD driver call */
		return (void *)mp200_lcd_set_v4l2_image;
	}
	else{
		/* NTS driver call */
		return (void *)mp200_nts_set_v4l2_image;
	}
#else
	/* LCD driver call */
	return (void *)mp200_lcd_set_v4l2_image;
#endif /* CONFIG_MP200_NTS */
}


/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_request
* FUNCTION : request to LCD driver function.
* RETURN   :  0: success
* NOTE     : This function the drawing demand to the LCD driver.
* CREATE   : 2008/01/17 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_lcd_request(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int call_flg)
{
	int                      ret = 0;
	unsigned long            flags;
	unsigned long            pixelformat = 0;

	struct _V4L2_IMAGE_INFO  image_info;
	int (*call_outfunc)(V4L2_IMAGE_INFO *);

	switch(call_flg){
	case STREAM_PLAYBACK:
		trace_add(vb, "lcd_request(PLAYBACK)");
		printk_dbg(_VBQ_DBG, "(%p) ->prev(%p)  ->next(%p) ->state(%d)\n", &vb->stream, vb->stream.prev, vb->stream.next, vb->state);
#if CALC_ADJUST
		do_gettimeofday(&dev->timer.do_time);
#else
		if(vb->output == V4L2_OUTPUT_NTSC || vb->output == V4L2_OUTPUT_PAL){
			do_gettimeofday(&dev->timer.do_time);
		}
#endif

		/* LCD driver's request information making */
		dbg_lock(&dev->vbq_lock, "vbq_lock\n");
		spin_lock_irqsave(&dev->vbq_lock, flags);

		/* check next queue status */
		if(mp200_v4l2_next_proccess(dev, vb, 0, LCD_REQUEST)){
			printk_dbg((_LCD_DBG || _ROT_DBG), "STATE_CANCELED.\n");
			if (vb->state_frame != STATE_FRAME_IMMEDIATE){
			down_trylock(&dev->sem_lcdout);
			up(&dev->sem_lcdout); /* release semafore */
			trace_add(vb, "lcd_request:up sem_lcdout");
			}
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			return -1;
		}

		if(vb->state == STATE_QUEUED){
			vb->state = STATE_ACTIVE;
		}

		/* movie reproduction processing */
		if(vb->state_proccess & STATE_PROCCESS_IMG_COMPLETE){
			printk_dbg((_LCD_DBG || _ROT_DBG), "IMG processing.\n");
			memcpy(&image_info.image_data, &vb->img_buf->dst_data, sizeof(struct _IMAGE_DATA));
			pixelformat = vb->img_buf->pixelformat;
		}
		else if(vb->state_proccess & STATE_PROCCESS_ROT_COMPLETE){
			printk_dbg((_LCD_DBG || _ROT_DBG), "ROT processing.\n");
			memcpy(&image_info.image_data, &vb->rot_buf->dst_data, sizeof(struct _IMAGE_DATA));
			pixelformat = vb->rot_buf->pixelformat;
		}
		else{
			image_info.image_data.size     = vb->bytesperline;
			image_info.image_data.yrgbaddr = vb->base_addr.PhysAddr_Y;
			image_info.image_data.uvaddr   = vb->base_addr.PhysAddr_UV;
			image_info.image_data.vaddr    = vb->base_addr.PhysAddr_V;
			image_info.image_data.x        = vb->save_effect.source_crop.left;
			image_info.image_data.y        = vb->save_effect.source_crop.top;
			image_info.image_data.hsize    = vb->save_effect.source_crop.width;
			image_info.image_data.vsize    = vb->save_effect.source_crop.height;
			pixelformat                    = vb->pixelformat;
		}

		image_info.screen_data.x     = vb->save_effect.screen.left;
		image_info.screen_data.y     = vb->save_effect.screen.top;
		image_info.screen_data.hsize = vb->save_effect.screen.width;
		image_info.screen_data.vsize = vb->save_effect.screen.height;

		switch(pixelformat){
		case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
			image_info.yuvfmt = V4L2_FORMAT_YUV420Pl2;
			break;
		case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
			image_info.yuvfmt = V4L2_FORMAT_YUV422Pl2;
			break;
		case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
			image_info.yuvfmt = V4L2_FORMAT_YUV420Pl;
			break;
#if VF_YUV422PL
		case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
			image_info.yuvfmt = V4L2_FORMAT_YUV422Pl;
			break;
#endif
#if VF_YUV422PX
		case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
			image_info.yuvfmt = V4L2_FORMAT_YUV422Px;
			break;
#endif
		default:
			trace_add(vb, "lcd_request1");
			printk_dbg((_LCD_DBG || _ROT_DBG), "not support format.\n");
			if (vb->state_frame != STATE_FRAME_IMMEDIATE){
			down_trylock(&dev->sem_lcdout);
			up(&dev->sem_lcdout); /* release semafore */
			trace_add(vb, "lcd_request:up sem_lcdout");
			}
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			return -1;
		}

		image_info.endian         = V4L2_LITTLE_ENDIAN;
		image_info.frame_data.dev = (void *)dev;
		image_info.frame_data.buf = (void *)vb;

		printk_dbg((_LCD_DBG || _ROT_DBG), "lcd_request: LCD driver call. ->sequence(%d)\n", vb->sequence);
		printk_dbg((_LCD_DBG || _ROT_DBG), "=== mp200_lcd_set_v4l2_image() =============================== sequence(%d) ===\n", vb->sequence);
		printk_dbg((_LCD_DBG || _ROT_DBG), " src | Y(0x%08lx)  UV(0x%08lx)  V(0x%08lx)\n", image_info.image_data.yrgbaddr, image_info.image_data.uvaddr, image_info.image_data.vaddr);
		printk_dbg((_LCD_DBG || _ROT_DBG), "     | w(%3d)  h(%3d)  x(%3d)  y(%3d)\n", image_info.image_data.hsize, image_info.image_data.vsize, image_info.image_data.x, image_info.image_data.y);
		printk_dbg((_LCD_DBG || _ROT_DBG), "     | p(%d)\n", image_info.image_data.size);
		printk_dbg((_LCD_DBG || _ROT_DBG), "-----+--------------------------------------------------------------------------\n");
		printk_dbg((_LCD_DBG || _ROT_DBG), " dst | w(%3d)  h(%3d)  x(%3d)  y(%3d)\n", image_info.screen_data.hsize, image_info.screen_data.vsize, image_info.screen_data.x, image_info.screen_data.y);
		printk_dbg((_LCD_DBG || _ROT_DBG), "     | endi(%d), fmt(%d)\n", image_info.endian, image_info.yuvfmt);
		printk_dbg((_LCD_DBG || _ROT_DBG), "================================================================================\n\n");

		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		break;

	default:
	case STREAM_STOP:
		trace_add(0, "lcd_request(STOP)");
		printk_dbg((_LCD_DBG || _ROT_DBG), "STOP processing.\n");

		/* movie stop processing */
		memset(&image_info, 0, sizeof(struct _IMAGE_DATA));
		printk_dbg((_LCD_DBG || _ROT_DBG), "LCD driver call.\n");
		break;
	}

#ifdef CONFIG_FB_MP200
	if(call_flg != STREAM_STOP){
		perf_add(vb->sequence, PHS_LCD);
		call_outfunc = mp200_v4l2_lcd_set_outfunc(vb->output);
	}
	else{
		call_outfunc = mp200_v4l2_lcd_set_outfunc(dev->output);
	}

	if(call_outfunc(&image_info)){
		if(call_flg != STREAM_STOP){
			/* check next queue status */
			spin_lock_irqsave(&dev->vbq_lock, flags);
			if(mp200_v4l2_next_proccess(dev, vb, 0, VBQ_COMPLETE)){
				/* CANCELED */
				trace_add(vb, "lcd_request: next_proccess canceled.\n");
			}
			else{
				mp200_v4l2_core_vbq_complete(dev, vb, 1);
			}
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			ret = -1;
		}
	}
#endif
	return ret;
}


/* ------------------ videobuf_queue_ops ---------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_release
* FUNCTION : 
* RETURN   : -
* NOTE     : This function is called a videobuf_buffer release.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_core_vbq_release(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	videobuf_waiton(vb, 0, 0);
	mp200_v4l2_core_vbq_dqueue(q, vb);
	vb->state = STATE_NEEDS_INIT;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_dqueue
* FUNCTION : 
* RETURN   : -
* NOTE     : This function executes video buffer information.
* CREATE   : 08/01/24 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_core_vbq_dqueue(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;
	spin_lock_irqsave(&dev->vbq_lock, flags);
	mp200_v4l2_next_proccess(dev, vb, 0, VBQ_DQUEUE);
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_cancel
* FUNCTION : 
* RETURN   : 0: success
*          : 1: stop timer
* NOTE     : This function registers information to video buffer.
* CREATE   : 
* UPDATE   : 06/07/03 M.Haguro
******************************************************************************/
static int mp200_v4l2_core_vbq_cancel(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;
	unsigned int  retval = 0;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if(vb->state_queued == STATE_QUEUED_TMR_QUEUED){
		del_timer_sync(&dev->timer.wait_timer);
		vb->state_frame  = STATE_FRAME_CANCELED;
		vb->state        = STATE_CANCELED;
		vb->state_queued = STATE_QUEUED_DONE;
		mp200_v4l2_next_proccess(dev, vb, 0, TMR_CALLBACK);

		down_trylock(&dev->sem_lcdout);
		up(&dev->sem_lcdout); /* release semafore */
		trace_add(vb, "vbq_cancel:up sem_lcdout");
	}
	else{
		if(vb->state == STATE_QUEUED){
			retval = 1;
		}
		if(vb->state_queued == STATE_QUEUED_IDLE){
			vb->state_queued = STATE_QUEUED_DONE;
			retval = 0;
		}
		if(dev->vb_old_refresh) retval = 1;
	}
	
	if(retval) trace_add(vb, "vbq_cancel(1)");
	else       trace_add(vb, "vbq_cancel(0)");
	spin_unlock_irqrestore(&dev->vbq_lock, flags);

	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_complete
* FUNCTION : 
* RETURN   : -
* NOTE     : This function is called from interrupt context 
*          : when a IMG transferof a videobuf_buffer completes.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_core_vbq_complete(void *arg1, void *arg, int err_flg)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)arg1;
	struct videobuf_buffer   *vb  = (struct videobuf_buffer *)arg;

	vb->field_count   = dev->field_count;
	dev->field_count += 2;

	/* check next queue status */
	if(mp200_v4l2_next_proccess(dev, vb, 0, VBQ_COMPLETE)){
		/* CANCELED */
		trace_add(vb, "vbq_complete: next_proccess canceled.\n");
		return;
	}

	if(err_flg){
		trace_add(vb, "vbq_complete: callback error.\n");
		if(dev->vb_old_refresh){
			vb->state = STATE_CANCELED;
		}
		else{
			vb->state = STATE_DQBUF_PERMIT;
			wake_up(&vb->done);
			wake_up(&vb->clear_done);
		}
	}else{
		trace_add(vb, "vbq_complete: callback success.\n");
		vb->state = STATE_DONE;

		if(dev->vb_old){
			if(dev->vb_old->state == STATE_DONE || dev->vb_old->state == STATE_CANCELED)
				dev->vb_old->state = STATE_DQBUF_PERMIT;
			printk_dbg(_VBQ_DBG, "(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%d) :old (%p) ->state(%d) ->done(%p)\n",
					&vb->stream, vb->stream.prev, vb->stream.next, vb->state, vb->sequence, &dev->vb_old->stream, dev->vb_old->state, &dev->vb_old->done);

			mp200_v4l2_core_vbq_complete_skipped(&dev->vb_old->stream);
			wake_up(&dev->vb_old->done);
		}
		else{
			mp200_v4l2_core_vbq_complete_skipped(&((struct videobuf_queue *)(&dev->streaming->vbq))->stream);
		}
		dev->vb_old = vb;
		wake_up(&vb->clear_done);
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_complete_skipped
* FUNCTION : 
* RETURN   : -
* NOTE     : This function is called from interrupt context 
*          : when a IMG transferof a videobuf_buffer completes.
* CREATE   : 
* UPDATE   : 09/04/02 M.Haguro
******************************************************************************/
static inline void mp200_v4l2_core_vbq_complete_skipped(struct list_head *vbq_stream)
{
	struct list_head       *list;
	struct videobuf_buffer *buf;

	list_for_each(list, vbq_stream){
		buf = list_entry(list, struct videobuf_buffer, stream);
		if(buf->state == STATE_DONE){
			break;
		}
		else if(buf->state == STATE_CANCELED){
			if(buf->state_frame == STATE_FRAME_DONE || buf->state_frame == STATE_FRAME_IMMEDIATE){
				buf->state_frame = STATE_FRAME_CANCELED;
			}
			buf->state = STATE_DQBUF_PERMIT;
			wake_up(&buf->done);
		}
		printk_dbg(_VBQ_DBG, "(%p) ->prev(%p) ->next(%p) ->state(%d) ->sequence(%d)\n",
				&buf->stream, buf->stream.prev, buf->stream.next, buf->state, buf->sequence);
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_queue
* FUNCTION : 
* RETURN   : -
* NOTE     : This function executes video buffer information.
* CREATE   : 
* UPDATE   : 06/06/16 M.Haguro
******************************************************************************/
static void mp200_v4l2_core_vbq_queue(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if(dev->suspend == 0 && dev->mixing == 0 && dev->streamoff == NULL){
		trace_add(vb, "vbq_queue");
		printk_dbg(_VBQ_DBG, "(%p) ->prev(%p) ->next(%p) ->state(%d)\n", &vb->stream, vb->stream.prev, vb->stream.next, vb->state);

		vb->state = STATE_QUEUED;
		mp200_v4l2_next_proccess(dev, vb, 0, VBQ_QUEUE);
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_prepare
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 
* UPDATE   : 06/06/12 M.Haguro
******************************************************************************/
static int mp200_v4l2_core_vbq_prepare(struct videobuf_queue *q, struct videobuf_buffer *vb, enum v4l2_field field)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;
	unsigned int  retval = 0;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if (dev->pix.sizeimage > vb->bsize) {
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -EINVAL;
	}
	vb->field  = field;

	if((dev->pix.pixelformat != V4L2_PIX_FMT_NV12)       /* YUV420 Semi-Planar */
	   && (dev->pix.pixelformat != V4L2_PIX_FMT_NV422)   /* YUV422 Semi-Planar */
	   && (dev->pix.pixelformat != V4L2_PIX_FMT_YVU420)  /* YUV420 Planar      */
#if VF_YUV422PL
	   && (dev->pix.pixelformat != V4L2_PIX_FMT_YUV422P) /* YUV422 Planar      */
#endif
#if VF_YUV422PX
	   && (dev->pix.pixelformat != V4L2_PIX_FMT_UYVY)    /* YUV422 Interleave  */
#endif
	){
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		printk_info("error! pixelformat is incorrect.\n\n");
		return -EINVAL;
	}

	memcpy(&vb->save_effect, &dev->efct, sizeof(struct v4l2_effect));
	vb->pixelformat = dev->pix.pixelformat;
	vb->output      = dev->output;

	/* call size check function */
	if(mp200_v4l2_core_vbq_sizecheck(q, vb)){
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -EINVAL;
	}

	if(mp200_v4l2_core_wkb_chksize(dev, vb)){
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		return -EINVAL;
	}

	printk_dbg(_VBQ_DBG, "=== vbq_queue() ==============================================\n");
	printk_dbg(_VBQ_DBG, " timestamp: sec(%ld), usec(%ld)\n", vb->ts.tv_sec, vb->ts.tv_usec);
	printk_dbg(_VBQ_DBG, " src :      address Y(0x%08lx),  UV(0x%08lx)\n", vb->base_addr.PhysAddr_Y, vb->base_addr.PhysAddr_UV);
	printk_dbg(_VBQ_DBG, "                                 width(%3d), height(%3d),  pitch(%3d)\n", vb->width, vb->height, vb->bytesperline);
	printk_dbg(_VBQ_DBG, " crop:      left(%3d), top(%3d), width(%3d), height(%3d)\n",
			vb->save_effect.source_crop.top, vb->save_effect.source_crop.left, vb->save_effect.source_crop.width, vb->save_effect.source_crop.height);
	printk_dbg(_VBQ_DBG, "            angle(%ddeg)\n", vb->save_effect.movie_angle *90);
	printk_dbg(_VBQ_DBG, " screen:    left(%3d), top(%3d), width(%3d), height(%3d)\n",
			vb->save_effect.screen.top, vb->save_effect.screen.left, vb->save_effect.screen.width, vb->save_effect.screen.height);
	printk_dbg(_VBQ_DBG, "==============================================================\n");

	q->sequence ++;
	if(q->sequence == 0) q->sequence++;
	dev->efct.sequence = q->sequence;

	vb->sequence    = q->sequence;
	vb->state       = STATE_PREPARED;

	if(vb->ts.tv_sec == STATE_FRAME_IMMEDIATE || vb->ts.tv_usec == STATE_FRAME_IMMEDIATE){
		vb->state_frame = STATE_FRAME_IMMEDIATE;
	}
	else{
		vb->state_frame = STATE_FRAME_DONE;
	}

	trace_add(vb, "vbq_prepare");

	spin_unlock_irqrestore(&dev->vbq_lock, flags);

	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_sizecheck_pix
* FUNCTION : 
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 06/07/20 H.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_vbq_sizecheck_pix(struct videobuf_buffer *vb)
{
	unsigned int image_width_aline, image_height_aline;

	switch(vb->pixelformat){
	default:
	case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
	case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
		image_width_aline  = IMAGE_WIDTH_ALINE_SP;
		image_height_aline = IMAGE_HEIGHT_ALINE_SP;
		break;
	case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
#endif
		image_width_aline  = IMAGE_WIDTH_ALINE_PL;
		image_height_aline = IMAGE_HEIGHT_ALINE_PL;
		break;
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
		image_width_aline  = IMAGE_WIDTH_ALINE_IL;
		image_height_aline = IMAGE_HEIGHT_ALINE_IL;
		break;
#endif
	}

	if((vb->width < IMAGE_WIDTH_MIN) ||
	   (vb->width > IMAGE_WIDTH_MAX) ||
	   (vb->width & image_width_aline) != 0){
		printk_info("error! width(%3d) is incorrect.\n\n", vb->width);
		return -1;
	}
	if((vb->height < IMAGE_HEIGHT_MIN) ||
	   (vb->height > IMAGE_HEIGHT_MAX) ||
	   (vb->height & image_height_aline) != 0){
		printk_info("error! height(%3d) is incorrect.\n\n", vb->height);
		return -1;
	}
	if((vb->width * vb->height) > IMAGE_AREA_MAX){
		printk_info("error! width*height(%3d)*(%3d) is incorrect.\n\n", vb->width, vb->height);
		return -1;
	}
	if((vb->bytesperline < vb->width)     ||
	   (vb->bytesperline > IMAGE_WIDTH_MAX) ||
	   (vb->bytesperline & image_width_aline) != 0){
		printk_info("error! bytesperline(%3d) is incorrect.\n\n", vb->bytesperline);
		return -1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_sizecheck_crop
* FUNCTION : 
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 06/07/20 H.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_vbq_sizecheck_crop(struct videobuf_buffer *vb)
{
	unsigned int image_width_aline, image_height_aline;

	switch(vb->pixelformat){
	default:
	case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
	case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
		image_width_aline  = IMAGE_WIDTH_ALINE_SP;
		image_height_aline = IMAGE_HEIGHT_ALINE_SP;
		break;
	case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
#endif
		image_width_aline  = IMAGE_WIDTH_ALINE_PL;
		image_height_aline = IMAGE_HEIGHT_ALINE_PL;
		break;
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
		image_width_aline  = IMAGE_WIDTH_ALINE_IL;
		image_height_aline = IMAGE_HEIGHT_ALINE_IL;
		break;
#endif
	}

	if((vb->save_effect.source_crop.left < IMAGE_LEFT_MIN) ||
	   (vb->save_effect.source_crop.left > vb->width)      ||
	   (vb->save_effect.source_crop.left & IMAGE_LEFT_ALINE) != 0){
		printk_info("error! source_crop.left(%3d) is incorrect.\n\n", vb->save_effect.source_crop.left);
		return -1;
	}
	if((vb->save_effect.source_crop.top < IMAGE_TOP_MIN) ||
	   (vb->save_effect.source_crop.top > vb->height)    ||
	   (vb->save_effect.source_crop.top & IMAGE_TOP_ALINE) != 0){
		printk_info("error! source_crop.top(%3d) is incorrect.\n\n", vb->save_effect.source_crop.top);
		return -1;
	}

	if((vb->save_effect.source_crop.width < IMAGE_WIDTH_MIN) ||
	   (vb->save_effect.source_crop.width > vb->width)       ||
	   (vb->save_effect.source_crop.width & image_width_aline) != 0){
		printk_info("error! source_crop.width(%3d) is incorrect.\n\n", vb->save_effect.source_crop.width);
		return -1;
	}

	if((vb->save_effect.source_crop.height < IMAGE_HEIGHT_MIN) ||
	   (vb->save_effect.source_crop.height > vb->height)       ||
	   (vb->save_effect.source_crop.height & image_height_aline) != 0){
		printk_info("error! source_crop.height(%3d) is incorrect.\n\n", vb->save_effect.source_crop.height);
		return -1;
	}

	if((vb->save_effect.source_crop.left + vb->save_effect.source_crop.width) > vb->width){
		printk_info("error! source_crop.left+width(%3d) is incorrect.\n\n", vb->save_effect.source_crop.left + vb->save_effect.source_crop.width);
		return -1;
	}
	if((vb->save_effect.source_crop.top + vb->save_effect.source_crop.height) > vb->height){
		printk_info("error! source_crop.top+height(%3d) is incorrect.\n\n", vb->save_effect.source_crop.top + vb->save_effect.source_crop.height);
		return -1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_sizecheck
* FUNCTION : 
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 06/06/16 H.Nagashima
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_vbq_sizecheck(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	unsigned int src_size, dst_size, dst_pos;
	unsigned int src_org_size_w, src_org_size_h;
	unsigned int image_width_aline, image_height_aline, image_width_aline_out, image_height_aline_out;
	unsigned int front_width, front_height;

	switch(vb->pixelformat){
	default:
	case V4L2_PIX_FMT_NV12:    /* YUV420 Semi-Planar */
	case V4L2_PIX_FMT_NV422:   /* YUV422 Semi-Planar */
		image_width_aline      = IMAGE_WIDTH_ALINE_SP;
		image_height_aline     = IMAGE_HEIGHT_ALINE_SP;
		image_width_aline_out  = IMAGE_WIDTH_ALINE_SP;
		image_height_aline_out = IMAGE_HEIGHT_ALINE_SP;
		break;
	case V4L2_PIX_FMT_YVU420:  /* YUV420 Planar      */
#if VF_YUV422PL
	case V4L2_PIX_FMT_YUV422P: /* YUV422 Planar      */
#endif
		image_width_aline      = IMAGE_WIDTH_ALINE_PL;
		image_height_aline     = IMAGE_HEIGHT_ALINE_PL;
		image_width_aline_out  = IMAGE_WIDTH_ALINE_SP;  /* PL -> SP */
		image_height_aline_out = IMAGE_HEIGHT_ALINE_SP; /* PL -> SP */
		break;
#if VF_YUV422PX
	case V4L2_PIX_FMT_UYVY:    /* YUV422 Interleave  */
		image_width_aline      = IMAGE_WIDTH_ALINE_IL;
		image_height_aline     = IMAGE_HEIGHT_ALINE_IL;
		image_width_aline_out  = IMAGE_WIDTH_ALINE_IL;
		image_height_aline_out = IMAGE_HEIGHT_ALINE_IL;
		break;
#endif
	}

	switch(vb->output){
	default:
	case V4L2_OUTPUT_LCD:
		front_width  = FRONT_WIDTH_V;
		front_height = FRONT_HEIGHT_V;
		break;
#ifdef CONFIG_MP200_NTS
	case V4L2_OUTPUT_NTSC:
		front_width  = NTSC_WIDTH;
		front_height = NTSC_HEIGHT;
		break;
	case V4L2_OUTPUT_PAL:
		front_width  = PAL_WIDTH;
		front_height = PAL_HEIGHT;
		break;
#endif
	}

	/* pix size check */
	if(mp200_v4l2_core_vbq_sizecheck_pix(vb)){
		return -1;
	}

	/* crop size check */
	if(mp200_v4l2_core_vbq_sizecheck_crop(vb)){
		return -1;
	}

	/* angle check */
	if(vb->save_effect.movie_angle < ROT_MODE_ROT_MOD_0 || vb->save_effect.movie_angle > ROT_MODE_ROT_MOD_270){
		printk_info("error! movie_angle(%d) is incorrect.\n\n", vb->save_effect.movie_angle);
		return -1;
	}

	/* address check */
	if((vb->base_addr.PhysAddr_Y & IMAGE_PADDR_ALINE) != 0  || vb->base_addr.PhysAddr_Y == IMAGE_PADDR_NULL){
		printk_info("error! PhysAddrY(0x%lx) is incorrect.\n\n", vb->base_addr.PhysAddr_Y);
		return -1;
	}
	if(((vb->base_addr.PhysAddr_UV & IMAGE_PADDR_ALINE) != 0 || vb->base_addr.PhysAddr_UV == IMAGE_PADDR_NULL)
#if VF_YUV422PX
	   && (vb->pixelformat != V4L2_PIX_FMT_UYVY)    /* YUV422 Interleave  */
#endif
	){
		printk_info("error! PhysAddrUV(0x%lx) is incorrect.\n\n", vb->base_addr.PhysAddr_UV);
		return -1;
	}
	if(((vb->pixelformat == V4L2_PIX_FMT_YUV420)    /* YUV420 Planar      */
#if VF_YUV422PL
	   || (vb->pixelformat == V4L2_PIX_FMT_YUV422P) /* YUV422 Planar      */
#endif
	   ) && ((vb->base_addr.PhysAddr_V & IMAGE_PADDR_ALINE) != 0 || vb->base_addr.PhysAddr_V == IMAGE_PADDR_NULL)){
		printk_info("error! PhysAddrV(0x%lx) is incorrect.\n\n", vb->base_addr.PhysAddr_V);
		return -1;
	}

	/* dest size check */
	vb->save_effect.screen.width  -= vb->save_effect.screen.width  & image_width_aline_out;
	vb->save_effect.screen.height -= vb->save_effect.screen.height & image_height_aline_out;

	/* revise LCD output size */
	if(vb->save_effect.screen.left > front_width || vb->save_effect.screen.top > front_height ){
		vb->save_effect.screen.width  = 0;
		vb->save_effect.screen.height = 0;
		return 0;
	}

	src_org_size_w = vb->save_effect.source_crop.left + vb->save_effect.source_crop.width;
	src_org_size_h = vb->save_effect.source_crop.top  + vb->save_effect.source_crop.height;

	if(vb->save_effect.screen.left + vb->save_effect.screen.width  > front_width ){
		dst_pos  = vb->save_effect.screen.left;
		dst_size = vb->save_effect.screen.width;
		vb->save_effect.screen.width = (front_width - dst_pos) & ~image_width_aline_out;
		vb->save_effect.screen.left  = front_width - vb->save_effect.screen.width;
		if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_0 || vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180)
		{
			src_size = vb->save_effect.source_crop.width;
			vb->save_effect.source_crop.width  = vb->save_effect.screen.width * src_size / dst_size;
			vb->save_effect.source_crop.width -= (vb->save_effect.source_crop.width & image_width_aline);
			if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180){
				vb->save_effect.source_crop.left = src_org_size_w - vb->save_effect.source_crop.width;
			}
		}else{
			src_size = vb->save_effect.source_crop.height;
			vb->save_effect.source_crop.height  = vb->save_effect.screen.width * src_size / dst_size;
			vb->save_effect.source_crop.height -= (vb->save_effect.source_crop.height & image_height_aline);
			if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_90){
				vb->save_effect.source_crop.top = src_org_size_h - vb->save_effect.source_crop.height;
			}
		}
	}
	if(vb->save_effect.screen.top + vb->save_effect.screen.height  > front_height){
		dst_pos  = vb->save_effect.screen.top;
		dst_size = vb->save_effect.screen.height;
		vb->save_effect.screen.height = (front_height - dst_pos) & ~image_height_aline_out;
		vb->save_effect.screen.top    = front_height - vb->save_effect.screen.height;
		if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_0 || vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180)
		{
			src_size = vb->save_effect.source_crop.height;
			vb->save_effect.source_crop.height  = vb->save_effect.screen.height * src_size / dst_size;
			vb->save_effect.source_crop.height -= (vb->save_effect.source_crop.height & image_height_aline);
			if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180){
				vb->save_effect.source_crop.top = src_org_size_h - vb->save_effect.source_crop.height;
			}
		}else{
			src_size = vb->save_effect.source_crop.width;
			vb->save_effect.source_crop.width  = vb->save_effect.screen.height * src_size / dst_size;
			vb->save_effect.source_crop.width -= (vb->save_effect.source_crop.width & image_width_aline);
			if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_270){
				vb->save_effect.source_crop.left = src_org_size_w - vb->save_effect.source_crop.width;
			}
		}
	}
	
	/* IMG not support dest size check */
	if(vb->save_effect.screen.width < DEST_WIDTH_MIN || vb->save_effect.screen.height < DEST_HEIGHT_MIN ){
		vb->save_effect.screen.width  = 0;
		vb->save_effect.screen.height = 0;
	}
	else if(vb->save_effect.source_crop.height < IMAGE_HEIGHT_MIN || vb->save_effect.source_crop.width < IMAGE_WIDTH_MIN ){
		printk_info("error! source_crop.width(%3d) or source_crop.height(%3d) is incorrect.\n\n",
		        vb->save_effect.source_crop.width, vb->save_effect.source_crop.height);
		return -1;
	}

#if RESIZE_CHECK
	/* resize scaling check */
	if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_0 || vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180)
	{
		if(((RESIZE_WIDTH_MIN * vb->save_effect.screen.width) < vb->save_effect.source_crop.width) ||
		    (vb->save_effect.screen.width > (RESIZE_WIDTH_MAX * vb->save_effect.source_crop.width))){
			printk_info("error! source_crop.width(%3d) or screen.width(%3d) is incorrect.\n", vb->save_effect.source_crop.width, vb->save_effect.screen.width);
			return -1;
		}
		if(((RESIZE_HEIGHT_MIN * vb->save_effect.screen.height) < vb->save_effect.source_crop.height) ||
		    (vb->save_effect.screen.height > (RESIZE_HEIGHT_MAX * vb->save_effect.source_crop.height))){
			printk_info("error! source_crop.height(%3d) or screen.height(%3d) is incorrect.\n", vb->save_effect.source_crop.height, vb->save_effect.screen.height);
			return -1;
		}
	}else{
		if(((RESIZE_WIDTH_MIN * vb->save_effect.screen.width) < vb->save_effect.source_crop.height) ||
		    (vb->save_effect.screen.width > (RESIZE_WIDTH_MAX * vb->save_effect.source_crop.height))){
			printk_info("error! source_crop.height(%3d) or screen.width(%3d) is incorrect.\n", vb->save_effect.source_crop.height, vb->save_effect.screen.width);
			return -1;
		}
		if(((RESIZE_HEIGHT_MIN * vb->save_effect.screen.height) < vb->save_effect.source_crop.width) ||
		    (vb->save_effect.screen.height > (RESIZE_HEIGHT_MAX * vb->save_effect.source_crop.width))){
			printk_info("error! source_crop.width(%3d) or screen.height(%3d) is incorrect.\n", vb->save_effect.source_crop.width, vb->save_effect.screen.height);
			return -1;
		}
	}
#endif

	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_setup
* FUNCTION : 
* RETURN   : 0: success
* NOTE     : Limit the number of available kernel image capture buffers based
*          : on the number requested, the currently selected image size, 
*          : and the maximum amount of memory permitted for kernel buffers.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_vbq_setup(struct videobuf_queue *q, unsigned int *cnt, unsigned int *size)
{
	int retval = 0;
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if (*cnt <= 0)
		*cnt = VIDEO_MAX_FRAME; /* supply a default number of buffers */

	if (*cnt > VIDEO_MAX_FRAME)
		*cnt = VIDEO_MAX_FRAME;

	*size = dev->pix.sizeimage;

	if(dev->pix.sizeimage <= 0)
		retval = -1;

	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/* ------------------ work_buffer_ops ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_core_wkb_init
* FUNCTION : 
* RETURN   : 
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_FMT
* CREATE   : 2008/06/05 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline void mp200_v4l2_core_wkb_init(struct mp200_v4l2_device *dev)
{
	if(dev->workbuf.active == WORKBUF_NO_INIT){ /* not init */
		dev->workbuf.rot_addr = ROT_SMEM_START;
		dev->workbuf.rot_size = ROT_SMEM_LENGTH;
		dev->workbuf.img_addr = IMG_SMEM_START;
		dev->workbuf.img_size = IMG_SMEM_LENGTH;
		dev->workbuf.active   = WORKBUF_INIT;
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_wkb_active
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: misstype argument
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_FMT
* CREATE   : 2008/06/05 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_wkb_active(struct mp200_v4l2_device *dev, unsigned int cmd,  void *arg)
{
	mp200_v4l2_core_wkb_init(dev);

	if(cmd == VIDIOC_G_FMT || cmd == VIDIOC_G_CROP || cmd == VIDIOC_G_CTRL || cmd == VIDIOC_G_EFFECT){
		/* ioctl(VIDIOC_G_FMT) || ioctl(VIDIOC_G_CROP) || ioctl(VIDIOC_G_CTRL) || ioctl(VIDIOC_G_EFFECT) */
		;
	}
	else if(cmd == VIDIOC_S_FMT){
		/* ioctl(VIDIOC_S_FMT) */
		if(((struct v4l2_format *)arg)->type == V4L2_BUF_TYPE_PRIVATE){
			if(dev->workbuf.active == WORKBUF_ACTIVE){
				printk_info("error! this ioctl(VIDIOC_S_FMT) is invalid.\n");
				return -EPERM;
			}
		}
		else{
			/* other type */
			dev->workbuf.active = WORKBUF_ACTIVE;
		}
	}
	else{
		/* other ioctl */
		dev->workbuf.active = WORKBUF_ACTIVE;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_wkb_chkbuf
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: misstype argument
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_FMT
* CREATE   : 2008/06/05 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_wkb_chkbuf(struct v4l2_workbuffer *wbuf)
{
	/* check workbuffer size */
	if(wbuf->rot_size < 0){
		return -EINVAL;
	}
	if(wbuf->img_size < 0){
		return -EINVAL;
	}

	/* check workbuffer area */
	if(wbuf->rot_addr < wbuf->img_addr){
		if(wbuf->rot_addr + wbuf->rot_size > wbuf->img_addr){
			return -EFAULT;
		}
	}
	else{
		if(wbuf->img_addr + wbuf->img_size > wbuf->rot_addr){
			return -EFAULT;
		}
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_wkb_chksize
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: misstype argument
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_QBUF
* CREATE   : 2008/06/05 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_wkb_chksize(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	int bpp = 0;

	if(vb->pixelformat == V4L2_PIX_FMT_NV12 || vb->pixelformat == V4L2_PIX_FMT_YVU420){
		bpp = 12;	/* YUV420 Semi-Planar || YUV420 Planar */
	}
	else if(vb->pixelformat == V4L2_PIX_FMT_NV422
#if VF_YUV422PL
		|| vb->pixelformat == V4L2_PIX_FMT_YUV422P
#endif
#if VF_YUV422PX
		|| vb->pixelformat == V4L2_PIX_FMT_UYVY
#endif
	){
		bpp = 16;	/* YUV422 Semi-Planar */
	}
	else{
		return -EINVAL;
	}

	/* check use ROT */
	if(vb->save_effect.movie_angle != ROT_MODE_ROT_MOD_0){
		if((vb->save_effect.source_crop.width * vb->save_effect.source_crop.height * bpp / 8) > ((dev->workbuf.rot_size / NUM_BUF_V4L2) & ~0x3)){
			printk_info("error! WorkBuffer(IPU-ROT) is insufficient.  >>> crop(%d*%d =0x%x) > workbuf(0x%x)\n\n", 
				vb->save_effect.source_crop.width, vb->save_effect.source_crop.height,
				(vb->save_effect.source_crop.width * vb->save_effect.source_crop.height * bpp / 8), ((dev->workbuf.rot_size / NUM_BUF_V4L2) & ~0x3));
			return -EINVAL;
		}
	}

	if(vb->output == V4L2_OUTPUT_LCD){
		/* check use IMG */
		if(vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_0 || vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_180){
			if((vb->save_effect.source_crop.width  != vb->save_effect.screen.width) || /* resize */
			   (vb->save_effect.source_crop.height != vb->save_effect.screen.height))  /* resize */ 
			{
				if((vb->save_effect.screen.width * vb->save_effect.screen.height * bpp / 8) > ((dev->workbuf.img_size / NUM_BUF_V4L2) & ~0x3)){
					printk_info("error! WorkBuffer(IPU-IMG) is insufficient.  >>> screen(%d*%d =0x%x) > workbuf(0x%x)\n\n", 
						vb->save_effect.screen.width, vb->save_effect.screen.height,
						(vb->save_effect.screen.width * vb->save_effect.screen.height * bpp / 8), ((dev->workbuf.img_size / NUM_BUF_V4L2) & ~0x3));
					return -EINVAL;
				}
			}
		}
		else{	/* vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_90 || vb->save_effect.movie_angle == ROT_MODE_ROT_MOD_270 */
			if((vb->save_effect.source_crop.width  != vb->save_effect.screen.height) || /* resize */
			   (vb->save_effect.source_crop.height != vb->save_effect.screen.width))    /* resize */ 
			{
				if((vb->save_effect.screen.width * vb->save_effect.screen.height * bpp / 8) > ((dev->workbuf.img_size / NUM_BUF_V4L2) & ~0x3)){
					printk_info("error! WorkBuffer(IPU-IMG) is insufficient.  >>> screen(%d*%d =0x%x) > workbuf(0x%x)\n\n", 
						vb->save_effect.screen.width, vb->save_effect.screen.height,
						(vb->save_effect.screen.width * vb->save_effect.screen.height * bpp / 8), ((dev->workbuf.img_size / NUM_BUF_V4L2) & ~0x3));
					return -EINVAL;
				}
			}
		}
	}
	return 0;
}


/* ------------------ system_call_ops ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_g_output
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_G_OUTPUT
* CREATE   : 2008/11/18 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_ioc_g_output(struct mp200_v4l2_fh *fh, unsigned int *output)
{
	*output = ((struct mp200_v4l2_device *)fh->dev)->output;
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_s_output
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype *output
*          :    -EIO: error 
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_OUTPUT
* CREATE   : 2008/11/18 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_s_output(struct mp200_v4l2_fh *fh, unsigned int *output)
{
	struct mp200_v4l2_device *dev = fh->dev;
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	switch(*output){
	case V4L2_OUTPUT_LCD:
#ifdef CONFIG_MP200_NTS
		if(dev->output != V4L2_OUTPUT_LCD){
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			if(dev->streaming){
				mp200_v4l2_ioc_streamoff(fh, dev);
				mp200_nts_release(NTS_ACTIVE_V4L2);
				dev->output = *output;
				mp200_v4l2_ioc_streamon(fh, dev);
			}
			else{
				mp200_nts_release(NTS_ACTIVE_V4L2);
			}
			spin_lock_irqsave(&dev->vbq_lock, flags);
		}
#endif /* CONFIG_MP200_NTS */
		dev->output = *output;
		break;
#ifdef CONFIG_MP200_NTS
	case V4L2_OUTPUT_NTSC:
	case V4L2_OUTPUT_PAL:
		retval = mp200_nts_getmode();
		if(retval != V4L2_OUTPUT_LCD && retval != *output){
			/* NTS reserved in other mode. */
			retval = -EIO;
		}
		else{
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			if(dev->streaming){
				mp200_v4l2_ioc_streamoff(fh, dev);
				retval = mp200_nts_reserve(NTS_ACTIVE_V4L2, *output);
				mp200_v4l2_ioc_streamon(fh, dev);
			}
			else{
				retval = mp200_nts_reserve(NTS_ACTIVE_V4L2, *output);
			}
			spin_lock_irqsave(&dev->vbq_lock, flags);
			if(retval){
				retval = -EIO;
			}
			else{
				/* success NTS reserve */
				dev->output = *output;
			}
		}
		break;
#endif /* CONFIG_MP200_NTS */
	default:
		retval = -EINVAL;
		break;
	}

	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_g_fmt
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype fmt->type
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_G_FMT
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_g_fmt(struct mp200_v4l2_device *dev, struct v4l2_format *fmt)
{
	struct v4l2_workbuffer *wbuf = (struct v4l2_workbuffer *)fmt->fmt.raw_data;
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(fmt->type == V4L2_BUF_TYPE_VIDEO_OUTPUT){
		/* get the current format */
		fmt->fmt.pix = dev->pix;
	}
	else if(fmt->type == V4L2_BUF_TYPE_VIDEO_OVERLAY){
		/* get overlay info */
		fmt->fmt.win.w = dev->efct.screen;
	}
	else if(fmt->type == V4L2_BUF_TYPE_PRIVATE){
		/* get workbuffer info */
		memcpy(wbuf, &dev->workbuf, sizeof(struct v4l2_workbuffer));
	}
	else{
		retval = -EINVAL;
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_s_fmt
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype fmt->type
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_FMT
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_s_fmt(struct mp200_v4l2_device *dev, struct v4l2_format *fmt)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(fmt->type == V4L2_BUF_TYPE_VIDEO_OUTPUT){
		/* set image info */
		dev->pix.sizeimage    = sizeof(struct videobuf_buffer);
		dev->pix.width        = fmt->fmt.pix.width;
		dev->pix.height       = fmt->fmt.pix.height;
		dev->pix.pixelformat  = fmt->fmt.pix.pixelformat;
		dev->pix.bytesperline = fmt->fmt.pix.bytesperline;
	}
	else if(fmt->type == V4L2_BUF_TYPE_VIDEO_OVERLAY){
		if(dev->efct.source_crop.width && dev->efct.source_crop.height){
			if((dev->efct.source_crop.width != fmt->fmt.win.w.width || dev->efct.source_crop.height != fmt->fmt.win.w.height) && (!dev->workbuf.img_addr || !dev->workbuf.img_size)){
				retval = -ENOMEM;
				goto err_ret;
			}
		}
		/* set overlay info */
		dev->efct.screen.left   = fmt->fmt.win.w.left;
		dev->efct.screen.top    = fmt->fmt.win.w.top;
		dev->efct.screen.width  = fmt->fmt.win.w.width;
		dev->efct.screen.height = fmt->fmt.win.w.height;
	}
	else if(fmt->type == V4L2_BUF_TYPE_PRIVATE){
		struct v4l2_workbuffer *wbuf = (struct v4l2_workbuffer *)fmt->fmt.raw_data;
		if(mp200_v4l2_core_wkb_chkbuf(wbuf)){
			retval = -EINVAL;
			goto err_ret;
		}
		/* set workbuffer info */
		dev->workbuf.rot_addr = wbuf->rot_addr;
		dev->workbuf.rot_size = wbuf->rot_size;
		dev->workbuf.img_addr = wbuf->img_addr;
		dev->workbuf.img_size = wbuf->img_size;
	}
	else{
		retval = -EINVAL;
	}
err_ret:
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_g_crop
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype vc->type
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_G_CROP
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_g_crop(struct mp200_v4l2_device *dev, struct v4l2_crop *vc)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(vc->type == V4L2_BUF_TYPE_VIDEO_OVERLAY){
		/* get crop data of original movie */
		vc->c.left   = dev->efct.source_crop.left;
		vc->c.top    = dev->efct.source_crop.top;
		vc->c.width  = dev->efct.source_crop.width;
		vc->c.height = dev->efct.source_crop.height;
	}
	else{
		retval = -EINVAL;
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_s_crop
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype vc->type
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_CROP
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_s_crop(struct mp200_v4l2_device *dev, struct v4l2_crop *vc)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(vc->type == V4L2_BUF_TYPE_VIDEO_OVERLAY){
		if(dev->efct.screen.width && dev->efct.screen.height){
			if((dev->efct.screen.width != vc->c.width || dev->efct.screen.height != vc->c.height) && (!dev->workbuf.img_addr || !dev->workbuf.img_size)){
				retval = -ENOMEM;
				goto err_ret;
			}
		}
		/* set crop data of original movie */
		dev->efct.source_crop.left   = vc->c.left;
		dev->efct.source_crop.top    = vc->c.top;
		dev->efct.source_crop.width  = vc->c.width;
		dev->efct.source_crop.height = vc->c.height;
	}
	else{
		retval = -EINVAL;
	}
err_ret:
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_g_ctrl
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype vc->id
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_G_CTRL
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_g_ctrl(struct mp200_v4l2_device *dev, struct v4l2_control *vc)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(vc->id == V4L2_MOVIE_DISPLAY_ANGLE){
		/* get current rotation angle */
		vc->value = dev->efct.movie_angle;
	}
	else{
		retval = -EINVAL;
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_s_ctrl
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          : -EINVAL: misstype vc->id
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_CTRL
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_s_ctrl(struct mp200_v4l2_device *dev, struct v4l2_control *vc)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(vc->id == V4L2_MOVIE_DISPLAY_ANGLE){
		if(vc->value != ROT_MODE_ROT_MOD_0 && (!dev->workbuf.rot_addr || !dev->workbuf.rot_size)){
			retval = -ENOMEM;
		}
		else{
			/* set rotation angle */
			dev->efct.movie_angle = vc->value;
		}
	}
	else{
		retval = -EINVAL;
	}
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_g_effect
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_G_EFFECT
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_g_effect(struct mp200_v4l2_device *dev, struct v4l2_effect *vef)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	/* get current effect infomation */
	vef->sequence           = dev->efct.sequence;
	vef->source_crop.left   = dev->efct.source_crop.left;
	vef->source_crop.top    = dev->efct.source_crop.top;
	vef->source_crop.width  = dev->efct.source_crop.width;
	vef->source_crop.height = dev->efct.source_crop.height;
	vef->movie_angle        = dev->efct.movie_angle;
	vef->screen.left        = dev->efct.screen.left;
	vef->screen.top         = dev->efct.screen.top;
	vef->screen.width       = dev->efct.screen.width;
	vef->screen.height      = dev->efct.screen.height;
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_s_effect
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_S_EFFECT
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_s_effect(struct mp200_v4l2_device *dev, struct v4l2_effect *vef)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if((vef->screen.width != vef->source_crop.width || vef->screen.height != vef->source_crop.height) && (!dev->workbuf.img_addr || !dev->workbuf.img_size)){
		retval = -ENOMEM;
		goto err_ret;
	}
	if(vef->movie_angle != ROT_MODE_ROT_MOD_0 && (!dev->workbuf.rot_addr || !dev->workbuf.rot_size)){
		retval = -ENOMEM;
		goto err_ret;
	}

	/* set effect infomation */
	dev->efct.sequence           = 0;
	dev->efct.source_crop.left   = vef->source_crop.left;
	dev->efct.source_crop.top    = vef->source_crop.top;
	dev->efct.source_crop.width  = vef->source_crop.width;
	dev->efct.source_crop.height = vef->source_crop.height;
	dev->efct.movie_angle        = vef->movie_angle;
	dev->efct.screen.left        = vef->screen.left;
	dev->efct.screen.top         = vef->screen.top;
	dev->efct.screen.width       = vef->screen.width;
	dev->efct.screen.height      = vef->screen.height;

err_ret:
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_reqbufs
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_REQBUFS
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_ioc_reqbufs(struct mp200_v4l2_device *dev, struct videobuf_queue *vbq, struct v4l2_requestbuffers *req)
{
	/* request memory for queue */
	return videobuf_reqbufs(vbq, req);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_qbuf
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_QBUF
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_ioc_qbuf(struct mp200_v4l2_device *dev, struct videobuf_queue *vbq, struct v4l2_buffer *vb)
{
	/* enqueue videoframe-buffer */
	return videobuf_qbuf(vbq, vb);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_dqbuf
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_DQBUF
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_ioc_dqbuf(struct mp200_v4l2_device *dev, struct videobuf_queue *vbq, struct v4l2_buffer *vb, int f_flags)
{
	/* dequeue videoframe-buffer */
	return videobuf_dqbuf(vbq, vb, f_flags & O_NONBLOCK);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_streamon
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          :  -EBUSY: 
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_STREAMON
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_streamon(struct mp200_v4l2_fh *fh, struct mp200_v4l2_device *dev)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if (dev->streaming) {
		retval = -EBUSY;
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
	}
	else {
		dev->streaming = fh;
		dev->vb_old_refresh = NULL;
		dev->vb_old = NULL;
		dev->mixing = 0;

		/* init thread info */
		mp200_v4l2_thread_init(dev);

		/* init timer */
		mp200_v4l2_tmr_init(dev);
		spin_unlock_irqrestore(&dev->vbq_lock, flags);

		retval = videobuf_streamon(&fh->vbq);
	}
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_ioc_streamoff
* FUNCTION : sub function of mp200_v4l2_core_do_ioctl()
* RETURN   :       0: success
*          :     err: 
* NOTE     : this function is substance of the ioctl system call.
*          :    VIDIOC_STREAMOFF
* CREATE   : 2008/02/28 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_ioc_streamoff(struct mp200_v4l2_fh *fh, struct mp200_v4l2_device *dev)
{
	int retval = 0;
	unsigned long flags;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
#if _PERF2_DBG
	Perf2Deinit(dev);
#endif

	dev->streamoff = fh;
	spin_unlock_irqrestore(&dev->vbq_lock, flags);

	/* streamoff */
	retval = videobuf_streamoff(&fh->vbq);
	if(retval < 0){
		dbg_lock(&dev->vbq_lock, "vbq_lock\n");
		spin_lock_irqsave(&dev->vbq_lock, flags);
		dev->streamoff = NULL;
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
	}
	else{
		dbg_lock(&dev->vbq_lock, "vbq_lock\n");
		spin_lock_irqsave(&dev->vbq_lock, flags);

		/* init thread info */
		mp200_v4l2_thread_init(dev);
		wake_up_interruptible(&dev->th_rot.th_busy);
		wake_up_interruptible(&dev->th_rot.th_buf->buf_busy);
		wake_up_interruptible(&dev->th_img.th_busy);
		wake_up_interruptible(&dev->th_img.th_buf->buf_busy);
		wake_up_interruptible(&dev->th_tmr.th_busy);
		wake_up_interruptible(&dev->th_lcd.th_busy);

		if (dev->streaming == fh) {
			/* stop timer */
			del_timer_sync(&dev->timer.wait_timer);

			/* call lcd driver function. */
			spin_unlock_irqrestore(&dev->vbq_lock, flags);
			mp200_v4l2_lcd_request(dev, NULL, STREAM_STOP);
			spin_lock_irqsave(&dev->vbq_lock, flags);

			dev->streaming = NULL;
			dev->vb_old_refresh = NULL;
			dev->vb_old    = NULL;
			dev->mixing    = 0;
		}
		/* video-buffer-queue sequence number initialize */
		fh->vbq.sequence = 0;
		dev->streamoff   = NULL;

		spin_unlock_irqrestore(&dev->vbq_lock, flags);
	}
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_do_ioctl
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: Argument error
* NOTE     : this function is substance of the ioctl system call.
*          : @@ supported ioctl command
*          :    VIDIOC_S_FMT,       VIDIOC_G_FMT,       VIDIOC_S_CROP,
*          :    VIDIOC_G_CROP,      VIDIOC_S_CTRL,      VIDIOC_G_CTRL,
*          :    VIDIOC_REQBUFS,     VIDIOC_S_EFFECT,    VIDIOC_G_EFFECT,
*          :    VIDIOC_STREAMON,    VIDIOC_STREAMOFF,   VIDIOC_QBUF,
*          :    VIDIOC_DQBUF
*          : @@ not supported ioctl command
*          :    VIDIOC_ENUMINPUT,   VIDIOC_G_INPUT,     VIDIOC_S_INPUT,
*          :    VIDIOC_ENUM_FMT,    VIDIOC_TRY_FMT,     VIDIOC_QUERYCTRL,
*          :    VIDIOC_G_FBUF,      VIDIOC_S_FBUF,      VIDIOC_OVERLAY,
*          :    VIDIOC_ENUMSTD,     VIDIOC_G_STD,       VIDIOC_S_STD,
*          :    VIDIOC_QUERYSTD,    VIDIOC_G_AUDIO,     VIDIOC_S_AUDIO
*          :    VIDIOC_G_AUDOUT,    VIDIOC_S_AUDOUT,    VIDIOC_G_JPEGCOMP,
*          :    VIDIOC_S_JPEGCOMP,  VIDIOC_G_TUNER,     VIDIOC_S_TUNER,
*          :    VIDIOC_G_MODULATOR, VIDIOC_S_MODULATOR, VIDIOC_G_FREQUENCY,
*          :    VIDIOC_S_FREQUENCY, VIDIOC_QUERYCAP,    VIDIOC_ENUMOUTPUT,
*          :    VIDIOC_G_OUTPUT,    VIDIOC_S_OUTPUT,    VIDIOC_QUERYBUF
* CREATE   : 
* UPDATE   : 2006/06/16 M.Haguro
******************************************************************************/
static int mp200_v4l2_core_do_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg)
{
	struct mp200_v4l2_fh     *fh  = file->private_data;
	struct mp200_v4l2_device *dev = fh->dev;


	/* check IMG/ROT workbuffer */
	if(mp200_v4l2_core_wkb_active(dev, cmd, arg)){
		return -EPERM;
	}


	switch (cmd) {
	case VIDIOC_G_OUTPUT:
		return mp200_v4l2_ioc_g_output(fh, arg);

	case VIDIOC_S_OUTPUT:
		return mp200_v4l2_ioc_s_output(fh, arg);

	case VIDIOC_G_FMT:
		return mp200_v4l2_ioc_g_fmt(dev, arg);

	case VIDIOC_S_FMT:
		return mp200_v4l2_ioc_s_fmt(dev, arg);

	case VIDIOC_G_CROP:
		return mp200_v4l2_ioc_g_crop(dev, arg);

	case VIDIOC_S_CROP:
		return mp200_v4l2_ioc_s_crop(dev, arg);

	case VIDIOC_G_CTRL:
		return mp200_v4l2_ioc_g_ctrl(dev, arg);

	case VIDIOC_S_CTRL:
		return mp200_v4l2_ioc_s_ctrl(dev, arg);

	case VIDIOC_G_EFFECT:
		return mp200_v4l2_ioc_g_effect(dev, arg);

	case VIDIOC_S_EFFECT:
		return mp200_v4l2_ioc_s_effect(dev, arg);

	case VIDIOC_REQBUFS:
		return mp200_v4l2_ioc_reqbufs(dev, &fh->vbq, arg);

	case VIDIOC_QBUF:
		return mp200_v4l2_ioc_qbuf(dev, &fh->vbq, arg);

	case VIDIOC_DQBUF:
		return mp200_v4l2_ioc_dqbuf(dev, &fh->vbq, arg, file->f_flags);

	case VIDIOC_STREAMON:
		return mp200_v4l2_ioc_streamon(fh, dev);

	case VIDIOC_STREAMOFF:
		return mp200_v4l2_ioc_streamoff(fh, dev);

	/*--- not supported ---*/
	case VIDIOC_QUERYBUF:
		/* not supported */
		return -EINVAL;

	case VIDIOC_ENUMINPUT:	/* FALL THROUGH */
	case VIDIOC_G_INPUT:	/* FALL THROUGH */
	case VIDIOC_S_INPUT:	/* FALL THROUGH */
	case VIDIOC_ENUM_FMT:	/* FALL THROUGH */
	case VIDIOC_TRY_FMT:	/* FALL THROUGH */
	case VIDIOC_QUERYCTRL:
		/* not supported */
		return -EINVAL;

	case VIDIOC_QUERYCAP:	/* FALL THROUGH */
	case VIDIOC_G_FBUF:	/* FALL THROUGH */ /* Get the frame buffer parameters */
	case VIDIOC_S_FBUF:	/* FALL THROUGH */ /* set the frame buffer parameters */
	case VIDIOC_OVERLAY:
		/* not supported */
		return -EINVAL;

	case VIDIOC_ENUMSTD:	/* FALL THROUGH */
	case VIDIOC_G_STD:	/* FALL THROUGH */
	case VIDIOC_S_STD:	/* FALL THROUGH */
	case VIDIOC_QUERYSTD:
		/* we don't have an analog video standard, 
		 * so we don't need to implement these ioctls.
		 */
		 return -EINVAL;

	case VIDIOC_G_AUDIO:	/* FALL THROUGH */
	case VIDIOC_S_AUDIO:	/* FALL THROUGH */
	case VIDIOC_G_AUDOUT:	/* FALL THROUGH */
	case VIDIOC_S_AUDOUT:
		/* we don't have any audio inputs or outputs */
		return -EINVAL;

	case VIDIOC_G_JPEGCOMP:	/* FALL THROUGH */
	case VIDIOC_S_JPEGCOMP:
		/* JPEG compression is not supported */
		return -EINVAL;

	case VIDIOC_G_TUNER:		/* FALL THROUGH */
	case VIDIOC_S_TUNER:		/* FALL THROUGH */
	case VIDIOC_G_MODULATOR:	/* FALL THROUGH */
	case VIDIOC_S_MODULATOR:	/* FALL THROUGH */
	case VIDIOC_G_FREQUENCY:	/* FALL THROUGH */
	case VIDIOC_S_FREQUENCY:
		/* we don't have a tuner or modulator */
		return -EINVAL;

	case VIDIOC_ENUMOUTPUT:
		/* not supported */
		return -EINVAL;

	default:
		/* unrecognized ioctl */
		return -ENOIOCTLCMD;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_ioctl
* FUNCTION : a stub function of ioctl
* RETURN   :       0: success
*          : -EINVAL: Argument error
* NOTE     : 
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static inline int mp200_v4l2_core_ioctl(struct inode *inode, struct file *file, unsigned int cmd,  unsigned long arg)
{
	return video_usercopy(inode, file, cmd, arg, mp200_v4l2_core_do_ioctl);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_poll
* FUNCTION : a system call function of poll & select
* RETURN   :       0: success
*          : -EINVAL: Argument error
* NOTE     : 
* CREATE   : 06/06/09 Y.Hara
* UPDATE   : 06/06/13 M.Haguro
******************************************************************************/
static inline unsigned int mp200_v4l2_core_poll(struct file *file, struct poll_table_struct *wait)
{
	struct mp200_v4l2_fh *fh = file->private_data;

	return videobuf_poll_stream(file, &fh->vbq, wait);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_release
* FUNCTION : v4l2 driver's close processing function.
* RETURN   : 0: success
* NOTE     : 
* CREATE   : 
* UPDATE   : 06/06/14 M.Haguro
******************************************************************************/
static int mp200_v4l2_core_release(struct inode *inode, struct file *file)
{
	struct mp200_v4l2_fh     *fh  = file->private_data;
	struct mp200_v4l2_device *dev = fh->dev;
	unsigned long flags;
	unsigned int  lcdout = V4L2_OUTPUT_LCD;
	int err;

	mp200_v4l2_ioc_streamoff(fh, dev);

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);

	if (fh->vbq.read_buf) {
		mp200_v4l2_core_vbq_release(&fh->vbq, fh->vbq.read_buf);
		kfree(fh->vbq.read_buf);
	}

	spin_unlock_irqrestore(&dev->vbq_lock, flags);
	err = mp200_v4l2_ioc_s_output(fh, &lcdout);
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if(err) printk_err("NTS out disable failed.\n\n");

	err = videobuf_mmap_free(&fh->vbq);
	if(err) printk_err("videobuf_free failed.\n\n");

	file->private_data = NULL;
	kfree(fh);
	spin_unlock_irqrestore(&dev->vbq_lock, flags);

	/* workqueue uninitialize */
	destroy_workqueue(dev->v4l2_workqueue);

	dev->vb_old_refresh = NULL;
	dev->vb_old = NULL;
	memset(&dev->pix,     0, sizeof(dev->pix));
	memset(&dev->efct,    0, sizeof(dev->efct));
	memset(&dev->workbuf, 0, sizeof(dev->workbuf));

	/* v4l2 unactive */
	dev->active = 0;
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_open
* FUNCTION : v4l2 driver's open processing function.
* RETURN   :       0: success
*          : -ENODEV: corresponding device doesn't exist.
*          : -ENOMEM: memory that can be used for the kernel is insufficient.
*          : -EPERM : no privilege in the call origin.
* NOTE     : 
* CREATE   : 
* UPDATE   : 06/06/14 M.Haguro
******************************************************************************/
static int mp200_v4l2_core_open(struct inode *inode, struct file *file)
{
	int minor = iminor(inode);
	struct mp200_v4l2_device *dev = mp200_dev;
	struct mp200_v4l2_fh     *fh;
	unsigned long flags;

	if (!dev || !dev->vfd || (dev->vfd->minor != minor))
		return -ENODEV;

#if (_PERF1_DBG | _TRACE_DBG)
	if (dev->active == 1) {
#if _PERF1_DBG
		perf_out();
#endif /* _PERF1_DBG */
#if _TRACE_DBG
		fh = NULL;
		if(dev->streaming)      fh = dev->streaming;
		else if(dev->streamoff) fh = dev->streamoff;
		if(fh){
			dbg_vbqtrace(dev, &fh->vbq);
			dbg_backtrace(dev);
		}
		return -EPERM;
#endif /* _TRACE_DBG */
	}
#endif /* _PERF1_DBG | _TRACE_DBG */

	/* allocate per-filehandle data */
	fh = kmalloc(sizeof(*fh), GFP_KERNEL);

	if (NULL == fh)
		return -ENOMEM;
	file->private_data = fh;
	fh->dev = dev;
	fh->type = V4L2_BUF_TYPE_VIDEO_OUTPUT;

	dbg_lock(&dev->vbq_lock, "vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
	if (dev->active == 1) {
		printk_err("v4l2 device Active\n");
		spin_unlock_irqrestore(&dev->vbq_lock, flags);
		kfree(fh);
		return -EPERM;
	}
	dev->active = 1;
	spin_unlock_irqrestore(&dev->vbq_lock, flags);

	videobuf_queue_init(&fh->vbq, &dev->vbq_ops, NULL, &dev->vbq_lock,
		fh->type, V4L2_FIELD_NONE, sizeof(struct videobuf_buffer), fh->dev);

	/* workqueue initialize */
	dev->v4l2_workqueue = create_singlethread_workqueue("v4l2");
	INIT_WORK(&dev->wk_lcd_stop, mp200_v4l2_lcd_stop_do);
#if ENABLE_DELAY
	INIT_WORK(&dev->wk_tmr_callback_bottom, mp200_v4l2_tmr_callback_bottom_do);
#endif

	/* video-buffer-queue index & sequence number initialize */
	fh->vbq.index_max = 0;
	fh->vbq.sequence  = 0;

	trace_clr();
	perf_clr();

	return 0;
}

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
/*****************************************************************************
* MODULE   : mp200_v4l2_core_suspend
* FUNCTION : callback function when to suspend.
* RETURN   : 
* NOTE     : unsupport
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int  mp200_v4l2_core_suspend(struct platform_device *pdev, pm_message_t state)
{
	int ret;
	struct mp200_v4l2_device *dev = dev_get_drvdata(&pdev->dev);

	if (state.event == DEV_SUSPEND_IDLE_1)
	{
		ret = 0;
	}
	else /* state.event == PM_EVENT_SUSPEND */
	{
		if (!mp200_sleep_while_idle)
		{
			if(dev->streaming){
				printk_dbg(_DPM_DBG, "del_timer\n");
				del_timer_sync(&dev->timer.wait_timer);

				down_trylock(&dev->sem_lcdout);
				up(&dev->sem_lcdout); /* release semafore */
			}
			dev->suspend = 1;
		}
		ret = 0;
	}
	return ret;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_resume
* FUNCTION : callback function when to resume.
* RETURN   : 
* NOTE     : unsupport
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int  mp200_v4l2_core_resume(struct platform_device *pdev)
{
	struct mp200_v4l2_device *dev = dev_get_drvdata(&pdev->dev);
	struct videobuf_queue    *q   = &dev->streaming->vbq;
	struct videobuf_buffer   *vb;
	struct list_head         *list;

	if(dev->suspend)
	{
		if(dev->streaming){
			list_for_each(list, &q->stream){
				vb = list_entry(list, struct videobuf_buffer, stream);
				if(vb->state == STATE_QUEUED && vb->state_queued == STATE_QUEUED_TMR_QUEUED){
					vb->state_queued = STATE_QUEUED_TMR_PREPARED;
				}
			}
		}
		dev->suspend = 0;
	}
	return 0;
}
#endif /* CONFIG_PM || CONFIG_DPM */


/*****************************************************************************
* MODULE   : mp200_v4l2_core_cleanup
* FUNCTION : cleanup module
* RETURN   :       0: success
*          : -ENODEV: corresponding device doesn't exist.
* NOTE     : v4l2 driver cleanup function, when to driver ends.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
void mp200_v4l2_core_cleanup(void)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct video_device      *vfd;

	if (!dev)
		return;

	vfd = dev->vfd;
	if (vfd) {
		if (vfd->minor == -1) {
			/* The device never got registered, so release the 
			** video_device struct directly
			*/
			video_device_release(vfd);
		} else {
			/* The unregister function will release the video_device
			** struct as well as unregistering it.
			*/
			video_unregister_device(vfd);
			platform_driver_unregister(&mp200_v4l2_core_driver);
			platform_device_unregister(&mp200_v4l2_core_device);
		}
		dev->vfd = NULL;
	}

	/* uninitialize kernel thread */
	mp200_v4l2_thread_cleanup(dev);

	kfree(dev);
	mp200_dev = NULL;

	return;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_init
* FUNCTION : initialize module
* RETURN   :       0: success
*          : -ENODEV: corresponding device doesn't exist.
* NOTE     : v4l2 driver initialize function, when to driver starts.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
int __init mp200_v4l2_core_init(void)
{
	struct mp200_v4l2_device *dev;
	struct video_device      *vfd;

	dev = kmalloc(sizeof(struct mp200_v4l2_device), GFP_KERNEL);
	if (!dev) {
		printk_err("could not allocate memory\n");
		goto init_error;
	}
	memset(dev, 0, sizeof(struct mp200_v4l2_device));
	
	/* Save the pointer to device in a global variable */
	mp200_dev = dev;

	/* initialize the video_device struct */
	vfd = dev->vfd = video_device_alloc();
	if (!vfd) {
		printk_err("could not allocate video device struct\n");
		goto init_error;
	}
	
 	vfd->release = video_device_release;

 	strlcpy(vfd->name, DEV_NAME, sizeof(vfd->name));
 	vfd->type = VID_TYPE_CAPTURE | VID_TYPE_CHROMAKEY;
 	
 	/* need to register for a VID_HARDWARE_* ID in videodev.h */
 	vfd->hardware = 0;
 	vfd->fops     = &mp200_v4l2_core_fops;
 	video_set_drvdata(vfd, dev);
 	vfd->minor    = -1;

	/* initialize the semafore */
	sema_init(&dev->sem_lcdout, 1);

	/* initialize the spinlock used to image parameters */
	spin_lock_init(&dev->vbq_lock);

	mp200_v4l2_core_wkb_init(dev);

	/* initialize the kernel thread */
	if(mp200_v4l2_thread_startup(dev)){
		printk_err("failed create thread\n");
		goto init_error;
	}

	/* initialize the kernel timer */
	mp200_v4l2_tmr_init(dev);

	/* initialize the videobuf queue ops */
	dev->vbq_ops.buf_setup   = mp200_v4l2_core_vbq_setup;
	dev->vbq_ops.buf_prepare = mp200_v4l2_core_vbq_prepare;
	dev->vbq_ops.buf_queue   = mp200_v4l2_core_vbq_queue;
	dev->vbq_ops.buf_release = mp200_v4l2_core_vbq_release;
	dev->vbq_ops.buf_cancel  = mp200_v4l2_core_vbq_cancel;
	dev->vbq_ops.buf_dqueue  = mp200_v4l2_core_vbq_dqueue;

	dev_set_drvdata(&mp200_v4l2_core_device.dev, (void *)dev);
	if (platform_device_register(&mp200_v4l2_core_device) < 0) {
		printk_err("could not register platform_device\n");
		goto init_error;
	}

	if (platform_driver_register(&mp200_v4l2_core_driver) < 0) {
		printk_err("could not register driver\n");
		platform_device_unregister(&mp200_v4l2_core_device);
		goto init_error;
        }
	if (video_register_device(vfd, VFL_TYPE_GRABBER, video_nr) < 0) {
		printk_err("could not register Video for Linux device\n");
		platform_device_unregister(&mp200_v4l2_core_device);
		platform_driver_unregister(&mp200_v4l2_core_driver);
		goto init_error;
	}

	printk(KERN_INFO DEV_NAME ": registered device video%d [v4l2]\n", vfd->minor);
	return 0;

init_error:
	mp200_v4l2_core_cleanup();
	return -ENODEV;
}


MODULE_AUTHOR("NEC Electronics Corporation.");
MODULE_DESCRIPTION("EM1 Video for Linux2 driver");
MODULE_LICENSE("GPL");
module_param(video_nr, int, 0);
MODULE_PARM_DESC(video_nr, "Minor number for video device (-1 ==> auto assign)");

module_init(mp200_v4l2_core_init);
module_exit(mp200_v4l2_core_cleanup);


