/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : drivers/media/video/mp200/mp200_v4l2_core.h
 * Function        : Video for Linux driver for EM1 driver core
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef MP200_V4L2_CORE__H
#define MP200_V4L2_CORE__H


#include <media/video-buf.h>
#include <asm/scatterlist.h>

#include "mp200_v4l2_ipu.h"
#include "mp200_v4l2_time.h"


struct mp200_v4l2_fh;
struct mp200_v4l2_device;


/*===============================================================*/
/* next_step_do() call function                                  */
/*===============================================================*/
enum call_func_num{
	VBQ_QUEUE,
	RESCHEDULE,
	ROT_REQUEST,
	ROT_CALLBACK,
	IMG_REQUEST,
	IMG_CALLBACK,
	TMR_REQUEST,
	TMR_CALLBACK,
	LCD_REQUEST,
	LCD_CALLBACK,
	VBQ_COMPLETE,
	VBQ_DQUEUE,
};


/*===============================================================*/
/* main thread definition structure                              */
/*===============================================================*/
enum thread_control{
	TH_IDLE,
	TH_RUNNING,
};

/* ROT|IMG temporary buffer */
enum buffer_select{
	BUF_A,
	BUF_B,
#if (NUM_BUF_V4L2 == 3)
	BUF_C,
#endif
};

enum buffer_state{
	TMPBUF_IDLE,
	TMPBUF_ROT_WRITE,
	TMPBUF_IMG_READ,
	TMPBUF_IMG_WRITE,
	TMPBUF_LCD_READ,
	TMPBUF_LCD_WRITE,
};

/* temporary buffer infomation structure ------------------------*/
struct temp_buffer {
	enum buffer_state  state;
	unsigned long      paddr;

	/* temporary image data (from mp200_common.h) */
	unsigned long      pixelformat;
	struct _IMAGE_DATA dst_data;
};

/* temporary buffer control structure ---------------------------*/
struct buf_object {
	/* temporary buffer data */
	struct temp_buffer          buf[NUM_BUF_V4L2]; /* temporary buffer infomation structure */
	wait_queue_head_t           buf_busy;

	/* use mp200_v4l2_ipu_callback_main() */
	ipu_callback_func           callback;      /* callback function (from IPU driver) */
	int                         ipu_hw_func;   /* H/W Function (use IPU processing)   */
	int (*make_request)(struct mp200_v4l2_device *, struct videobuf_buffer *, int, enum buffer_select);

	struct videobuf_buffer     *vb;
	struct mp200_ipu_req_v4l2   ipu_request;   /* request structure (to IPU driver)    */
};


/* main structure -----------------------------------------------*/
struct th_object {
	struct task_struct         *th;
	char                       *th_name;       /* thread name */
	int (*call_request)(struct mp200_v4l2_device *, struct videobuf_buffer *, int); /* main function to thread*/

	wait_queue_head_t           th_idle;
	enum videobuf_state_queued  th_idle_fixed; /* peculiar idle state flag to thread */
	enum videobuf_state_queued  th_idle_flag;
	wait_queue_head_t           th_busy;
	enum videobuf_state_queued  th_busy_fixed; /* peculiar busy state flag to thread */

	struct buf_object          *th_buf;        /* temporary buffer control structure */
};


#if _V4L2_RT_THREAD /* RT thread */
#define V4L2_THREAD_PRIORITY  1 /* thread priority */
#else /* Normal thread */
#define V4L2_THREAD_NICE    -20 /* thread priority */
#endif


/*===============================================================*/
/* mixing flag (member of mp200_v4l2_device)                     */
/*===============================================================*/
#define ROT_MIXING	0x000F
#define IMG_MIXING	0x00F0
#define TMR_MIXING	0x0F00
#define LCD_MIXING	0xF000
#define ROT_MIXING_SFT	0x00
#define IMG_MIXING_SFT	0x04
#define TMR_MIXING_SFT	0x08
#define LCD_MIXING_SFT	0x0C

#define ADD_MIXING(BIT, SFT) \
	do{ \
		dev->mixing = (dev->mixing & ~BIT) | ((1 << SFT) & BIT); \
	}while(0)

#define DEL_MIXING(BIT, SFT) \
	do{ \
		dev->mixing = (dev->mixing & ~BIT) | ((0 << SFT) & BIT); \
	}while(0)


/*===============================================================*/
/* per-device data structure                                     */
/*===============================================================*/
struct mp200_v4l2_device {
	struct device             dev;
	struct video_device      *vfd;

	struct semaphore          sem_lcdout;   /* semafore for lcdout */

	spinlock_t                vbq_lock;     /* spinlock for videobuf queues */
	struct videobuf_queue_ops vbq_ops;      /* videobuf queue operations    */
	unsigned long             field_count;  /* field counter for videobuf_buffer */
	struct videobuf_buffer   *vb_old_refresh;
	struct videobuf_buffer   *vb_old;

	/* v4l2_workqueue defines the v4l2 driver to LCD driver or ROT driver
         * working task.
	 */
	struct workqueue_struct  *v4l2_workqueue;
	struct work_struct        wk_lcd_stop;
	struct work_struct        wk_tmr_callback_bottom;

	/* The img_lock is used to serialize access to the image parameters for 
	 * overlay and capture.  Need to use spin_lock_irq when writing to the 
	 * reading, streaming, and previewing parameters.  A regular spin_lock 
	 * will suffice for all other cases.
	 */

	/* We allow streaming from at most one filehandle at a time.
 	 * non-NULL means streaming is in progress.
 	 */
	struct mp200_v4l2_fh     *streaming;
	struct mp200_v4l2_fh     *streamoff;
	unsigned long             mixing;

	/* revise time data */
	struct mp200_v4l2_time    timer;

	/* kernel thread data */
	struct th_object          th_rot;
	struct th_object          th_img;
	struct th_object          th_tmr;
	struct th_object          th_lcd;


	/* pix defines the size and pixel format of the image captured by the
	 * sensor.  This also defines the size of the framebuffers.  The
	 * same pool of framebuffers is used for video capture and video
	 * overlay.  These parameters are set/queried by the
	 * VIDIOC_S_FMT/VIDIOC_G_FMT ioctls with a CAPTURE buffer type.
	 */
	struct v4l2_pix_format    pix;

	/* saving effet data */
	struct v4l2_effect        efct;

	/* saving waork buffer data */
	struct v4l2_workbuffer    workbuf;

	/* saving output device data */
	unsigned int              output;

	char                      suspend;
	char                      active;
};


/*===============================================================*/
/* per-filehandle data structure                                 */
/*===============================================================*/
struct mp200_v4l2_fh {
	struct mp200_v4l2_device *dev;
	enum   v4l2_buf_type      type;
	struct videobuf_queue     vbq;
};


/*===============================================================*/
/* V4L2 streaming state flag. (need to LCD request)              */
/*===============================================================*/
#define STREAM_STOP         0
#define STREAM_PLAYBACK     1
#define STREAM_PLAYBACK_ROT 2


/*===============================================================*/
/* use image sizes check.                                        */
/*===============================================================*/
#define D1_WIDTH            720
#define D1_HEIGHT           480
#define INPUT_WIDTH         D1_WIDTH
#define INPUT_HEIGHT        D1_HEIGHT
#define INPUT_MAX           (INPUT_WIDTH > INPUT_HEIGHT ? INPUT_WIDTH : INPUT_HEIGHT)
#define LCD_MAX             (FRONT_WIDTH > FRONT_HEIGHT ? FRONT_WIDTH : FRONT_HEIGHT)


#define IMAGE_WIDTH_MIN      8
#define IMAGE_WIDTH_MAX      (LCD_MAX > INPUT_MAX ? LCD_MAX : INPUT_MAX)
#define IMAGE_WIDTH_ALINE_IL 0x1
#define IMAGE_WIDTH_ALINE_SP 0x3
#define IMAGE_WIDTH_ALINE_PL 0x7

#define IMAGE_HEIGHT_MIN      8
#define IMAGE_HEIGHT_MAX      (LCD_MAX > INPUT_MAX ? LCD_MAX : INPUT_MAX)
#define IMAGE_HEIGHT_ALINE_IL 0x1
#define IMAGE_HEIGHT_ALINE_SP 0x3
#define IMAGE_HEIGHT_ALINE_PL 0x7

#define IMAGE_AREA_MAX      ((FRONT_WIDTH  > INPUT_WIDTH  ? FRONT_WIDTH  : INPUT_WIDTH) *  \
                             (FRONT_HEIGHT > INPUT_HEIGHT ? FRONT_HEIGHT : INPUT_HEIGHT))

#define IMAGE_LEFT_MIN      0
#define IMAGE_LEFT_MAX      IMAGE_WIDTH_MAX
#define IMAGE_LEFT_ALINE    0x1
#define IMAGE_TOP_MIN       0
#define IMAGE_TOP_MAX       IMAGE_HEIGHT_MAX
#define IMAGE_TOP_ALINE     0x1

#define IMAGE_PADDR_NULL    0
#define IMAGE_PADDR_ALINE   0x3

#define DEST_WIDTH_MIN      16
#define DEST_HEIGHT_MIN     16

#define RESIZE_WIDTH_MIN    40 /* *(1/40) */
#define RESIZE_WIDTH_MAX    4  /* *4      */
#define RESIZE_HEIGHT_MIN   40 /* *(1/40) */
#define RESIZE_HEIGHT_MAX   4  /* *4      */


/*===============================================================*/
/* device name definetion                                        */
/*===============================================================*/
#define DEV_NAME "mp200_v4l2"


#endif /* MP200_V4L2_CORE__H */
