/* 2010-08-25: File added by Sony Corporation */
/*
 * Copyright 2006 KMC, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Driver for the Freescale Semiconductor MXC IPU
 *
 */

/*!
 * @file ak54k0a.c
 *
 * @brief ak54k0a camera driver functions
 *
 * @ingroup Camera
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/ctype.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/i2c.h>
#include "../drivers/media/video/mxc/capture/mxc_v4l2_capture.h"
#include "mt9v111.h"
#include "asm/arch/mxc_i2c.h"
#include "asm/arch/gpio.h"

//#define KMC_DEBUG
#ifdef KMC_DEBUG

#  define DDPRINTK(fmt, args...) printk(KERN_ERR"%s :: %d :: %s - " \
          fmt, __FILE__,__LINE__,__FUNCTION__ , ## args)
#  define DPRINTK(fmt, args...) printk("%s: " fmt, __FUNCTION__ , ## args)

#  define FUNC_START DPRINTK(" func start\n")
#  define FUNC_END DPRINTK(" func end\n")

#  define FUNC_ERR printk(KERN_ERR"%s :: %d :: %s  err= %d \n", \
          __FILE__,__LINE__,__FUNCTION__ ,err)

#else /* KMC_DEBUG */

#define DDPRINTK(fmt, args...)  do {} while(0)
#define DPRINTK(fmt, args...)   do {} while(0)

#define FUNC_START
#define FUNC_END

#endif /* KMC_DEBUG */

static sensor_interface *interface_param = NULL;
static int reset_frame_rate = 30;


static int ak54k0a_attach(struct i2c_adapter *adapter);
static int ak54k0a_detach(struct i2c_client *client);

//static struct i2c_driver ak54k0a_i2c_driver = {
//        .owner          = THIS_MODULE,
//        .name           = "AK54K0A Client",
//        .flags          = I2C_DF_NOTIFY,
//        .attach_adapter = ak54k0a_attach,
//        .detach_client = ak54k0a_detach,
//};
static struct i2c_driver ak54k0a_i2c_driver = {
	.driver = {
		.name           = "AK54K0A Client",
	},
	.id = 1,
        .attach_adapter = ak54k0a_attach,
        .detach_client = ak54k0a_detach,
};

/*
 * Function definitions
 */


/*!
 * Initialize ak54k0a_sensor_lib
 * Libarary for Sensor configuration through I2C
 *
 * @param       coreReg       Core Registers
 * @param       ifpReg        IFP Register
 *
 * @return status
 */

extern void gpio_sensor_active(void);

/*!
 * ak54k0a sensor interface Initialization
 * @param param            sensor_interface *
 * @param width            u32
 * @param height           u32
 * @return  None
 */
static void ak54k0a_interface(sensor_interface *param, u32 width, u32 height)
{
        gpio_sensor_active();

        param->Vsync_pol  = 0x0;
#if defined(CONFIG_MACH_MX3KZ) /* { */
	width = 542;
	height = 496;
        param->clk_mode   = 0x3; //CCIR Interlaced mode
//        param->clk_mode   = 0x2; //CCIR Progressive mode
#else /* }{ */
        param->clk_mode   = 0x0; //gated
#endif /* } */
	 param->pixclk_pol = 0x0;
        param->data_width = 0x1;
        param->data_pol   = 0x0;
        param->ext_vsync  = 0x0;
        param->Vsync_pol  = 0x0;
        param->Hsync_pol  = 0x0;
        param->width = width - 1;
        param->height = height - 1;
        param->pixel_fmt = IPU_PIX_FMT_UYVY;
        param->mclk = 27000000;
}


/*!
 * AK54K0A sensor configuration
 *
 * @param frame_rate       int *
 * @return  sensor_interface *
 */
sensor_interface *ak54k0a_config(int *frame_rate)
{
        FUNC_START;

	ak54k0a_interface(interface_param,640,480);
        set_mclk_rate(&interface_param->mclk);

        FUNC_END;

        return interface_param;
}

/*!
 * ak54k0a sensor set color configuration
 *
 * @param bright       int
 * @param saturation   int
 * @param red          int
 * @param green        int
 * @param blue         int
 * @return  None
 */
static void
ak54k0a_set_color(int bright, int saturation, int red, int green, int blue)
{
}

/*!
 * ak54k0a sensor get color configuration
 *
 * @param bright       int *
 * @param saturation   int *
 * @param red          int *
 * @param green        int *
 * @param blue         int *
 * @return  None
 */
static void
ak54k0a_get_color(int *bright, int *saturation, int *red, int *green, int *blue)
{
}

/*!
 * ak54k0a Reset function
 *
 * @return  None
 */
static sensor_interface *ak54k0a_reset(void)
{
        return ak54k0a_config(&reset_frame_rate);
}

struct camera_sensor camera_sensor_if = {
        set_color : ak54k0a_set_color,
        get_color : ak54k0a_get_color,
        config    : ak54k0a_config,
        reset     : ak54k0a_reset,
};

/*!
 *
 * @param adapter            struct i2c_adapter *
 * @return  Error code indicating success or failure
 */
static int ak54k0a_attach(struct i2c_adapter *adapter)
{
        if (strcmp(adapter->name, MXC_ADAPTER_NAME) != 0) {
                printk("kzm_54ka0_attach: %s\n", adapter->name);
                return -1;
        }

        interface_param = (sensor_interface *)
                kmalloc(sizeof(sensor_interface), GFP_KERNEL);
        if (!interface_param) {
                printk("kzm_ak54k0a_attach: kmalloc failed \n");
                return -1;
        }

        return 0;
}

/*!
 *
 * @param client            struct i2c_client *
 * @return  Error code indicating success or failure
 */
static int ak54k0a_detach(struct i2c_client *client)
{
        int err;

        if (interface_param)
               kfree(interface_param);
        interface_param = NULL;

        return err;
}


/*!
 * AK54k0A init function
 *
 * @return  Error code indicating success or failure
 */
static __init int
ak54k0a_init(void)
{
        u8 err;


        err = i2c_add_driver(&ak54k0a_i2c_driver);

        return err;
}

/*!
 * AK54k0A cleanup function
 *
 * @return  Error code indicating success or failure
 */
static void __exit
ak54k0a_clean(void)
{
        FUNC_START;


        FUNC_END;
}

module_init(ak54k0a_init);
module_exit(ak54k0a_clean);

MODULE_AUTHOR("KMC, Inc.");
MODULE_DESCRIPTION("KZM_ak540a Camera Driver");
MODULE_LICENSE("GPL");
