/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name		: linux/drivers/mmc/host/mp200_mmc.c
 *  Function		: MMC
 *  Release Version : Ver 1.00
 *  Release Date	: 2007/12/20
 *
 *  Copyright (C) NEC Electronics Corporation 2007
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/platform_device.h>
#include <linux/mutex.h>
#include <linux/workqueue.h>
#include <linux/timer.h>
#include <linux/mmc/mmc.h>
#include <linux/mmc/host.h>
#include <linux/mmc/sd.h>
#include <linux/mmc/core.h>
#include <linux/mmc/card.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach/irq.h>
#include <asm/dma.h>
#include <asm/hardware.h>

#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/arch/pwc.h>

#ifdef CONFIG_PM
#include <linux/notifier.h>
#include <linux/pm.h>
#endif

#include "mp200_mmc.h"

#define DRIVER_NAME	"mp200_mmc"

#define MP200_MMC_BASE	(IO_ADDRESS(MP200_SDIC_BASE))
#define MP200_MMC_DMASD		(MP200_SDIC_BASE + 0x300)

#define MMC_CLK_ON	mp200_pmu_open_clockgate(MP200_CLOCKGATE_SDIC_CLK)
#define MMC_CLK_OFF	mp200_pmu_close_clockgate(MP200_CLOCKGATE_SDIC_CLK)

/* #define MMC_SPEED_NORMAL */

static inline u32 mp200_mmc_info_read(void)
{
	u32 info;

	info = inl(MP200_MMC_INFO1) & 0xffff;
	info |= ((inl(MP200_MMC_INFO2) & 0xffff) << 16);

	return info;
}

/*
 * Notify the core about command completion
 */
static void
mp200_mmc_data_done(struct mp200_mmc_host *host, struct mmc_command *cmd)
{
	outl(MMC_STOP_STOP, MP200_MMC_STOP);
	outl(MMC_CC_EXT_MODE_FIFO, MP200_MMC_CC_EXT_MODE);

	if(host->data != NULL) {
		dma_unmap_sg(mmc_dev(host->mmc), host->data->sg, host->dma_num, host->dma_dir);

		if (!host->data->error) {
			host->data->bytes_xfered = host->data_len;
		}
	}
	host->data_len = 0;

	host->data = NULL;
	host->cmd = NULL;
	host->req = NULL;
	host->stop = NULL;

	/* Clear interrupt */
	outl(0, MP200_MMC_INFO1);
	outl(0, MP200_MMC_INFO2);
	host->info_mask = MMC_INFO_MASK;
	outl(MMC_INFO_MASK & 0xffff, MP200_MMC_INFO1_MASK);
	MMC_CLK_OFF;

	mmc_request_done(host->mmc, cmd->mrq);
}

/*
 * Notify the core about command completion
 */
static void
mp200_mmc_cmd_done(struct mp200_mmc_host *host, struct mmc_command *cmd)
{
	u32 val;

	if (cmd->flags & MMC_RSP_PRESENT) {
		if (cmd->flags & MMC_RSP_136) {
			/* response type 2 */
			val = inl(MP200_MMC_RSP0);
			cmd->resp[3] = val << 8;
			val = inl(MP200_MMC_RSP1);
			cmd->resp[3] |= (val & 0xff) << 24;
			cmd->resp[2] = val >> 8;
			val = inl(MP200_MMC_RSP2);
			cmd->resp[2] |= val << 8;
			val = inl(MP200_MMC_RSP3);
			cmd->resp[2] |= (val & 0xff) << 24;
			cmd->resp[1] = val >> 8;
			val = inl(MP200_MMC_RSP4);
			cmd->resp[1] |= val << 8;
			val = inl(MP200_MMC_RSP5);
			cmd->resp[1] |= (val & 0xff) << 24;
			cmd->resp[0] = val >> 8;
			val = inl(MP200_MMC_RSP6);
			cmd->resp[0] |= val << 8;
			val = inl(MP200_MMC_RSP7);
			cmd->resp[0] |= (val & 0xff) << 24;
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d RSP %x %x %x %x\n", __FUNCTION__, cmd->opcode, cmd->resp[0], cmd->resp[1], cmd->resp[2], cmd->resp[3]);
		} else {
			/* response types 1, 1b, 3, 4, 5, 6 */
			cmd->resp[0] = inl(MP200_MMC_RSP0);
			cmd->resp[0] |= (inl(MP200_MMC_RSP1) << 16);
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d RSP %x\n", __FUNCTION__, cmd->opcode, cmd->resp[0]);
		}
	}

	if( cmd->flags & MMC_RSP_BUSY ) {
		/* wait DATA0 == 1 */
		while( !(mp200_mmc_info_read() & MMC_INFO_DAT0) ) {
		}
	}


	if (host->app_mode) {
		host->app_mode = 0;
	} else if (cmd->opcode == 55) {
		host->app_mode = 1;
	}

	if (host->data == NULL) {
		/* Clear interrupt */
		outl(0, MP200_MMC_INFO1);
		outl(0, MP200_MMC_INFO2);
		MMC_CLK_OFF;
		host->req = NULL;
		host->cmd = NULL;
		mmc_request_done(host->mmc, cmd->mrq);
	} else if (host->data->error) {
		mp200_mmc_data_done(host, host->cmd);
	} else {
		val = inl(MP200_MMC_INFO1_MASK) | MMC_INFO_EOC;
		host->info_mask = val;
		outl(val, MP200_MMC_INFO1_MASK);
	}
}

/*
 * Configure the Responce type
 */
static void
mp200_mmc_start_cmd(struct mp200_mmc_host *host, struct mmc_command *cmd, u16 cmddat)
{
	u32 val;

	host->cmd = cmd;

	cmddat |= cmd->opcode;

	switch (mmc_resp_type(cmd)) {
	case MMC_RSP_R1: /* short CRC, OPCODE */
		cmddat |= MMC_CMD_RSP_R1;
		break;
	case MMC_RSP_R1B:/* short CRC, OPCODE, BUSY */
		cmddat |= MMC_CMD_RSP_R1B;
		break;
	case MMC_RSP_R2: /* long 136 bit + CRC */
		cmddat |= MMC_CMD_RSP_R2;
		break;
	case MMC_RSP_R3: /* short */
		cmddat |= MMC_CMD_RSP_R3;
		break;
	}

	if (host->app_mode) {
		cmddat |= MMC_CMD_ACMD;
	}

	val = inl(MP200_MMC_STOP) & ~MMC_STOP_STOP;
	outl(val, MP200_MMC_STOP);

	outl(cmd->arg & 0xffff, MP200_MMC_ARG0);
	outl(cmd->arg >> 16,    MP200_MMC_ARG1);

	dev_dbg(mmc_dev(host->mmc), "CMD %d %x %x %x %x\n",
			cmd->opcode, cmd->arg, cmd->flags, cmd->retries, cmddat);

/*
	printk("CMD %d %x %x %x %x\n",
			cmd->opcode, cmd->arg, cmd->flags, cmd->retries, cmddat);
*/

	/* Send command */
	outl(cmddat, MP200_MMC_CMD);
}

/*
 * DMA clean up for command errors
 */
static void mp200_mmc_dma_cleanup(struct mp200_mmc_host *host)
{
	struct mmc_data *data = host->data;

	if (host->dma_ch != -1) {
		dma_unmap_sg(mmc_dev(host->mmc), data->sg, host->dma_num, host->dma_dir);
		mp200_reset_dma(host->dma_ch);
		host->dma_ch = -1;
	}
}

/*
 * MMC controller IRQ handler
 */
static irqreturn_t
mp200_mmc_irq(int irq, void *dev_id)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)dev_id;
	u32 info, end_trans = 0;

	info = mp200_mmc_info_read() & ~host->info_mask;

/*
	printk("%s: %x\n", __FUNCTION__, info);
*/
	if (host->cmd == NULL) {
		outl(0, MP200_MMC_INFO1);
		outl(0, MP200_MMC_INFO2);
		return IRQ_HANDLED;
	}

	if (info & MMC_INFO_ALLERR) {
		end_trans = 1;
		if (info & MMC_INFO_CTO) {
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d CMD Timeout\n", __FUNCTION__, host->cmd->opcode);
			host->cmd->error = -ETIMEDOUT;
		} else if (info & MMC_INFO_CRCERR) {
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d CRC error\n", __FUNCTION__, host->cmd->opcode);
			host->cmd->error = -EILSEQ;
		} else {
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d Other error.(0x%x)\n", __FUNCTION__, host->cmd->opcode, info);
			host->cmd->error = -EILSEQ;
		}
		if (host->data) {
			if (info & MMC_INFO_DTO) {
				dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d DATA Timeout\n", __FUNCTION__, host->cmd->opcode);
				host->data->error = -ETIMEDOUT;
			} else {
				dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d Other error.(0x%x)\n", __FUNCTION__, host->cmd->opcode, info);
				host->data->error = -EILSEQ;
			}
			mp200_mmc_dma_cleanup(host);
		}
	}

	if ((info & MMC_INFO_EOC) || end_trans) {
		mp200_mmc_cmd_done(host, host->cmd);
		return IRQ_HANDLED;
	}
	if ((info & MMC_INFO_RWEND)) {
		mp200_mmc_data_done(host, host->cmd);
		return IRQ_HANDLED;
	}

	return IRQ_HANDLED;
}

/*
 * IRQ for handling card insertion and removal
 */
#if 0 /* eMMC */
static irqreturn_t
mp200_mmc_detect_irq(int irq, void *dev_id)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)dev_id;
	mmc_detect_change(host->mmc, (HZ * 50) / 1000);

	return IRQ_HANDLED;
}
#endif

static void mp200_mmc_dma_cb(void *data, int intsts, int intrawsts)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)data;
	u32 val;

	host->dma_count++;

	if (host->dma_count >= host->dma_num) {
		val = inl(MP200_MMC_INFO1_MASK) & ~MMC_INFO_RWEND;
		host->info_mask = val;
		outl(val, MP200_MMC_INFO1_MASK);
		host->dma_ch = -1;
	} else {
		if (host->dma_ch == host->dma_rx_ch) {
			mp200_start_dma(host->dma_rx_ch, MP200_MMC_DMASD, host->dma_len[host->dma_count],
				host->dma_address[host->dma_count], MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
		} else {
			mp200_start_dma(host->dma_tx_ch, host->dma_address[host->dma_count], host->dma_len[host->dma_count],
				MP200_MMC_DMASD, MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
		}
	}
}

/*
 * Configure block leangth for MMC/SD cards and intiate the transfer.
 */
static void
mp200_mmc_setup_data(struct mp200_mmc_host *host, struct mmc_data *data)
{
	int i;

	host->data = data;

	host->data_len = data->blksz * data->blocks;

	dev_dbg(mmc_dev(host->mmc), "MMC: DATA: size=%d num=%d flags=0x%x\n", data->blksz, data->blocks, data->flags);

	outl(data->blksz,  MP200_MMC_SIZE);
	outl(data->blocks, MP200_MMC_SECCNT);

	host->dma_dir = (data->flags & MMC_DATA_WRITE) ?
				DMA_TO_DEVICE : DMA_FROM_DEVICE;

	host->dma_num = dma_map_sg(mmc_dev(host->mmc), data->sg,
				data->sg_len, host->dma_dir);

	for (i = 0; i < host->dma_num; i++) {
		host->dma_address[i] = sg_dma_address(&data->sg[i]);
		host->dma_len[i] = sg_dma_len(&data->sg[i]);
	}

	outl(MMC_CC_EXT_MODE_SD_DMA, MP200_MMC_CC_EXT_MODE);

	host->dma_count = 0;

	if (data->flags & MMC_DATA_READ) {
		host->dma_ch = host->dma_rx_ch;
		mp200_start_dma(host->dma_rx_ch, MP200_MMC_DMASD, host->dma_len[0],
			host->dma_address[0], MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
	} else {
		host->dma_ch = host->dma_tx_ch;
		mp200_start_dma(host->dma_tx_ch, host->dma_address[0], host->dma_len[0],
			MP200_MMC_DMASD, MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
	}

/*
	printk("MMC: dma_num=%d sg_len=%d\n", host->dma_num, data->sg_len);
*/
}

/*
 * Request function. for read/write operation
 */
static void
mp200_mmc_request(struct mmc_host *mmc, struct mmc_request *req)
{
	struct mp200_mmc_host *host = mmc_priv(mmc);
	u16 cmddat = 0;
	u32 info;

	host->req = req;
	host->stop = req->stop;

	MMC_CLK_ON;
	info = mp200_mmc_info_read();

	/* Clear stat */
	outl(0, MP200_MMC_INFO1);
	outl(0, MP200_MMC_INFO2);
	req->cmd->error = 0;

	if (req->data) {
		req->data->error = 0;
		mp200_mmc_setup_data(host, req->data);

		cmddat |= MMC_CMD_DATA;

		if (host->data->flags & MMC_DATA_READ) {
			cmddat |= MMC_CMD_READ;
		}

		if (host->data->blocks > 1) {
			cmddat |= MMC_CMD_MULTI;
			outl(MMC_STOP_MULTI, MP200_MMC_STOP);
		}
	}

	mp200_mmc_start_cmd(host, req->cmd, cmddat);
}

static void mp200_mmc_power(struct mp200_mmc_host *host, u32 power_on)
{
	if (power_on) {
		MMC_CLK_ON;
		mp200_pmu_unreset_device(MP200_RESETDEVICE_SDIC);

		outl(MMC_SDWAITCTRL_VAL,     SMU_AB1_SDICWAITCTRL);
		outl(MMC_SDREADCTRL_VAL,     SMU_AB1_SDICREADCTRL);
		outl(MMC_USER_VAL,           MP200_MMC_USER);
		outl(MMC_USER2_VAL,          MP200_MMC_USER2);

		outl(MMC_SOFT_RST_RESET,     MP200_MMC_SOFT_RST);
		outl(MMC_SOFT_RST_UNRESET,   MP200_MMC_SOFT_RST);
		outl(MMC_CC_EXT_MODE_FIFO,   MP200_MMC_CC_EXT_MODE);
		outl(MMC_INFO_MASK,          MP200_MMC_INFO1_MASK);
		outl(MMC_INFO_MASK >> 16,    MP200_MMC_INFO2_MASK);

		outl(0, MP200_MMC_INFO1);
		outl(0, MP200_MMC_INFO2);
		outl(0, MP200_SDIO_INFO1);
		outl(MMC_SDIO_INFO1_MASK, MP200_SDIO_INFO1_MASK);
		outl(MMC_STOP_STOP, MP200_MMC_STOP);
		outl(0x80EE, MP200_MMC_OPTION);

		MMC_CLK_OFF;
//		enable_irq(host->irq);
	} else {
//		disable_irq(host->irq);
		host->clock = 0;
		host->bus_width = 0;
		MMC_CLK_ON;
		mp200_pmu_reset_device(MP200_RESETDEVICE_SDIC);
		MMC_CLK_OFF;
	}
}


/*
 * Configuring clock values.
 */
static void
mp200_mmc_set_ios(struct mmc_host *mmc, struct mmc_ios *ios)
{
	struct mp200_mmc_host *host = mmc_priv(mmc);
	u32 val, wDiv;

/*
	printk("%s: %d %x %d %d\n", __FUNCTION__,
		ios->clock, ios->power_mode, ios->bus_width, ios->timing);
*/
	if ((host->bus_width != ios->bus_width) && (host->power_mode != MMC_POWER_OFF)) {
		MMC_CLK_ON;
		val = inl(MP200_MMC_OPTION);
		if (ios->bus_width == MMC_BUS_WIDTH_4) {
			host->bus_width = MMC_BUS_WIDTH_4;
			val &= ~MMC_OPTION_WIDTH4;
		} else {
			host->bus_width = MMC_BUS_WIDTH_1;
			val |= MMC_OPTION_WIDTH4;
		}
		outl(val, MP200_MMC_OPTION);
		MMC_CLK_OFF;
	}

	if ((ios->clock != host->clock) && (host->power_mode != MMC_POWER_OFF)) {
		MMC_CLK_ON;
		/* Stop clock */
		outl(0, MP200_MMC_CLK_CTRL);

		if (ios->clock >= 40000000) {
			wDiv = MMC_CLOCK_HIGH   | MMC_CLOCK_OFFEN;
			outl(MMC_USER2_HIGH_VAL, MP200_MMC_USER2);
		} else if (ios->clock >= 20000000) {
			wDiv = MMC_CLOCK_DIV4   | MMC_CLOCK_OFFEN;
		} else if (ios->clock >= 10000000) {
			wDiv = MMC_CLOCK_DIV8   | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=  5000000) {
			wDiv = MMC_CLOCK_DIV16  | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=  2500000) {
			wDiv = MMC_CLOCK_DIV32  | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=  1250000) {
			wDiv = MMC_CLOCK_DIV64  | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=   625000) {
			wDiv = MMC_CLOCK_DIV128 | MMC_CLOCK_OFFEN;
		} else {
			wDiv = MMC_CLOCK_DIV256;
		}

		wDiv |= MMC_CLOCK_EN;

		/* Start clock */
		outl(wDiv, MP200_MMC_CLK_CTRL);
		MMC_CLK_OFF;
		host->clock = ios->clock;
	}

	if (host->power_mode != ios->power_mode) {
		switch (ios->power_mode) {
		case MMC_POWER_OFF:
			mp200_mmc_power(host, 0);
			dev_dbg(mmc_dev(host->mmc), "power off\n");
       		break;
		case MMC_POWER_UP:
			mp200_mmc_power(host, 1);
			dev_dbg(mmc_dev(host->mmc), "power up\n");
       		break;
		case MMC_POWER_ON:
			dev_dbg(mmc_dev(host->mmc), "power on\n");
       		break;
		}
		host->power_mode = ios->power_mode;
	}
}

static struct mmc_host_ops mp200_mmc_ops = {
	.request	= mp200_mmc_request,
	.set_ios	= mp200_mmc_set_ios,
};

static int
mp200_mmc_probe(struct platform_device *pdev)
{
	struct mmc_host *mmc;
	struct mp200_mmc_host *host = NULL;
	int ret = 0;

	mmc = mmc_alloc_host(sizeof(struct mp200_mmc_host), &pdev->dev);
	if (!mmc) {
		ret = -ENOMEM;
		goto err;
	}

	mmc->ops   = &mp200_mmc_ops;
	mmc->f_min = 400000;
#ifdef MMC_SPEED_NORMAL
	mmc->f_max = 25000000;
#else
	mmc->f_max = 50000000;
#endif
	mmc->ocr_avail = MMC_VDD_28_29;
#ifdef MMC_SPEED_NORMAL
	mmc->caps = MMC_CAP_4_BIT_DATA | MMC_CAP_BYTEBLOCK | MMC_CAP_MULTIWRITE;
#else
	mmc->caps = MMC_CAP_4_BIT_DATA | MMC_CAP_BYTEBLOCK | MMC_CAP_MULTIWRITE |
				MMC_CAP_MMC_HIGHSPEED | MMC_CAP_SD_HIGHSPEED;
#endif
#ifdef CONFIG_SNSC_MMC_MULTIPLE_CARDS
	mmc->card_num = 2;
#endif /* CONFIG_SNSC_MMC_MULTIPLE_CARDS */

	/* MMC core transfer sizes tunable parameters */
	mmc->max_hw_segs   = MP200_MAX_SEGS;
	mmc->max_phys_segs = MP200_MAX_SEGS;
	mmc->max_seg_size  = MP200_MAX_SEGS * 512;
	mmc->max_req_size  = MP200_MAX_SEGS * 512;
	mmc->max_blk_size  = 512;
	mmc->max_blk_count = 65535;

	host      = mmc_priv(mmc);
	host->mmc = mmc;
	host->irq = INT_SDIC_SD;
#if 0 /* eMMC */
	host->detect_irq = INT_PWC_SDDET;
#endif
	host->power_mode = MMC_POWER_OFF;
	host->info_mask = MMC_INFO_MASK;
	mutex_init(&host->mutex);

	/* DMA setting */
	host->dma_rx_ch = MP200_DMAC_P2M_LCH5;
	host->dma_tx_ch = MP200_DMAC_M2P_LCH5;

	ret = mp200_request_dma(host->dma_rx_ch, DRIVER_NAME,
				mp200_mmc_dma_cb, (void *)host, &host->dma_rx_regs);
	if (ret < 0) {
		goto err;
	}

	ret = mp200_request_dma(host->dma_tx_ch, DRIVER_NAME,
				mp200_mmc_dma_cb, (void *)host, &host->dma_tx_regs);
	if (ret < 0) {
		goto err_dma;
	}

	/* Request IRQ for MMC operations */
	ret = request_irq(host->irq, mp200_mmc_irq, IRQF_DISABLED, pdev->name, host);
	if (ret) {
		dev_dbg(mmc_dev(host->mmc), "Unable to IRQ");
		goto err_irq;
	}
//	disable_irq(host->irq);

#if 0 /* eMMC */
	/* Request IRQ for MMC card detect */
	ret = request_irq(host->detect_irq, mp200_mmc_detect_irq, IRQF_DISABLED, pdev->name, host);
	if (ret) {
		dev_dbg(mmc_dev(host->mmc), "Unable to Card detect IRQ");
		goto err_irq2;
	}
#endif
	platform_set_drvdata(pdev, host);
	mmc_add_host(mmc);

	return 0;

#if 0 /* eMMC */
err_irq2:
	free_irq(host->irq, host);
#endif
err_irq:
	mp200_free_dma(host->dma_tx_ch);
err_dma:
	mp200_free_dma(host->dma_rx_ch);

err:
	printk("%s: error!!(%d)\n", __FUNCTION__, ret);
	if (host)
		mmc_free_host(mmc);
	return ret;

}

static int
mp200_mmc_remove(struct platform_device *pdev)
{
	struct mp200_mmc_host *host = platform_get_drvdata(pdev);

	platform_set_drvdata(pdev, NULL);
	if (host) {
		flush_scheduled_work();

		mmc_remove_host(host->mmc);

		mp200_mmc_power(host, 0);

#if 0 /* eMMC */
		free_irq(host->detect_irq, host);
#endif
		free_irq(host->irq, host);

		mp200_free_dma(host->dma_rx_ch);
		mp200_free_dma(host->dma_tx_ch);

		mmc_free_host(host->mmc);
	}

	return 0;
}

#ifdef CONFIG_PM
static unsigned int mmc_save[IDX_REGS_NUM];

static void mmc_reg_save(void)
{
	mmc_save[IDX_CLK_CTRL] = inl(MP200_MMC_CLK_CTRL);
	mmc_save[IDX_OPTION] = inl(MP200_MMC_OPTION);
	mmc_save[IDX_SDIO_INFO1_MASK] = inl(MP200_SDIO_INFO1_MASK);
	mmc_save[IDX_INFO2_MASK] = inl(MP200_MMC_INFO2_MASK);
	mmc_save[IDX_INFO1_MASK] = inl(MP200_MMC_INFO1_MASK);
	mmc_save[IDX_USER2] = inl(MP200_MMC_USER2);
	mmc_save[IDX_USER] = inl(MP200_MMC_USER);

	return;
}

static void mmc_reg_restore(void)
{
	outl(mmc_save[IDX_USER], MP200_MMC_USER);
	outl(mmc_save[IDX_USER2], MP200_MMC_USER2);

	outl(MMC_SOFT_RST_RESET,     MP200_MMC_SOFT_RST);
	outl(MMC_SOFT_RST_UNRESET,   MP200_MMC_SOFT_RST);
	outl(MMC_CC_EXT_MODE_FIFO,   MP200_MMC_CC_EXT_MODE);

	outl(0, MP200_MMC_INFO1);
	outl(0, MP200_MMC_INFO2);
	outl(0, MP200_SDIO_INFO1);
	outl(mmc_save[IDX_INFO1_MASK], MP200_MMC_INFO1_MASK);
	outl(mmc_save[IDX_INFO2_MASK], MP200_MMC_INFO2_MASK);
	outl(mmc_save[IDX_SDIO_INFO1_MASK], MP200_SDIO_INFO1_MASK);

	outl(mmc_save[IDX_OPTION], MP200_MMC_OPTION);
	outl(mmc_save[IDX_CLK_CTRL], MP200_MMC_CLK_CTRL);

	return;
}

static int
mp200_mmc_suspend(struct platform_device *pdev, pm_message_t state)
{
	struct mp200_mmc_host *host = platform_get_drvdata(pdev);
	int ret = 0;
/*
	printk("%s: %x\n", __FUNCTION__, state.event);
*/
	if (host && host->suspended)
		return 0;

	if (host) {
		if ( !mp200_sleep_while_idle ) {
			ret = mmc_suspend_host(host->mmc, state);
			if (ret == 0) {
				host->suspended = 1;
			}
		} else {
			MMC_CLK_ON;
			mmc_reg_save();
			MMC_CLK_OFF;
			host->suspended = 1;
		}
	}
	return ret;
}

/* Routine to resume the MMC device */
static int
mp200_mmc_resume(struct platform_device *pdev)
{
	struct mp200_mmc_host *host = platform_get_drvdata(pdev);
	int ret = 0;

	if (host && !host->suspended)
		return 0;

	if (host) {
		if ( !mp200_sleep_while_idle ) {
			ret = mmc_resume_host(host->mmc);
			if (ret == 0) {
				host->suspended = 0;
			}
		} else {
			MMC_CLK_ON;
			mmc_reg_restore();
			MMC_CLK_OFF;
			host->suspended = 0;
		}
	}
	return ret;
}
#else
#define mp200_mmc_suspend	NULL
#define mp200_mmc_resume	NULL
#endif

static struct platform_driver mp200_mmc_driver = {
	.probe		= mp200_mmc_probe,
	.remove		= mp200_mmc_remove,
	.suspend	= mp200_mmc_suspend,
	.resume		= mp200_mmc_resume,
	.driver		= {
			.name = DRIVER_NAME,
	},
};

static int __init mp200_mmc_init(void)
{
	/* Register the MMC driver */
	if (platform_driver_register(&mp200_mmc_driver)) {
		printk(KERN_ERR ": Failed to register MMC driver\n");
		return -ENODEV;
	}
	return 0;
}

static void __exit mp200_mmc_cleanup(void)
{
	/* Unregister MMC driver */
	platform_driver_unregister(&mp200_mmc_driver);
}

module_init(mp200_mmc_init);
module_exit(mp200_mmc_cleanup);

MODULE_DESCRIPTION("MP200 High Speed Multimedia Card driver");
MODULE_LICENSE("GPL");
MODULE_ALIAS(DRIVER_NAME);
MODULE_AUTHOR("NEC Electronics");
