/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name		: linux/drivers/mmc/host/mp200_mmc.h
 *  Function		: MMC
 *  Release Version : Ver 1.00
 *  Release Date	: 2007/12/20
 *
 *  Copyright (C) NEC Electronics Corporation 2007
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __DRIVERS_MMC_MP200_MMC_H
#define __DRIVERS_MMC_MP200_MMC_H

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/dma.h>
#include <asm/hardware.h>

#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/arch/pwc.h>

#ifdef CONFIG_PM
#include <linux/notifier.h>
#include <linux/pm.h>
#include <asm/arch/pm.h>
#endif



/* MMC Registers */
#define MP200_MMC_CMD			(MP200_MMC_BASE + 0x00)	/* SD Command */
#define MP200_MMC_ARG0			(MP200_MMC_BASE + 0x08)	/* Command Argument0 */
#define MP200_MMC_ARG1			(MP200_MMC_BASE + 0x0C)	/* Command Argument1 */
#define MP200_MMC_STOP			(MP200_MMC_BASE + 0x10)	/* Stop internal action */
#define MP200_MMC_SECCNT		(MP200_MMC_BASE + 0x14)	/* Transfer sector count register */
#define MP200_MMC_RSP0			(MP200_MMC_BASE + 0x18)	/* Response0 */
#define MP200_MMC_RSP1			(MP200_MMC_BASE + 0x1C)	/* Response1 */
#define MP200_MMC_RSP2			(MP200_MMC_BASE + 0x20)	/* Response2 */
#define MP200_MMC_RSP3			(MP200_MMC_BASE + 0x24)	/* Response3 */
#define MP200_MMC_RSP4			(MP200_MMC_BASE + 0x28)	/* Response4 */
#define MP200_MMC_RSP5			(MP200_MMC_BASE + 0x2C)	/* Response5 */
#define MP200_MMC_RSP6			(MP200_MMC_BASE + 0x30)	/* Response6 */
#define MP200_MMC_RSP7			(MP200_MMC_BASE + 0x34)	/* Response7 */
#define MP200_MMC_INFO1			(MP200_MMC_BASE + 0x38)	/* Card Status(MP200_MMC_BASE + detect,write protect,etc.) */
#define MP200_MMC_INFO2			(MP200_MMC_BASE + 0x3C)	/* SD Buffer control & Error */
#define MP200_MMC_INFO1_MASK	(MP200_MMC_BASE + 0x40)	/* Mask of MP200_MMC_INFO1 register interrupt */
#define MP200_MMC_INFO2_MASK	(MP200_MMC_BASE + 0x44)	/* Mask of MP200_MMC_INFO2 register interrupt */
#define MP200_MMC_CLK_CTRL		(MP200_MMC_BASE + 0x48)	/* SD Card Clock Control */
#define MP200_MMC_SIZE			(MP200_MMC_BASE + 0x4C)	/* SD Card Data transfer size */
#define MP200_MMC_OPTION		(MP200_MMC_BASE + 0x50)	/* Option set up for SD Card */
#define MP200_MMC_ERR_STS1		(MP200_MMC_BASE + 0x58)	/* Error status1(MP200_MMC_BASE + Command,CRC,End) */
#define MP200_MMC_ERR_STS2		(MP200_MMC_BASE + 0x5C)	/* Error status2(MP200_MMC_BASE + Timeout) */
#define MP200_MMC_BUF0			(MP200_MMC_BASE + 0x60)	/* SD Data port0 */
#define MP200_SDIO_MODE			(MP200_MMC_BASE + 0x68)	/* Select SDIO mode */
#define MP200_SDIO_INFO1		(MP200_MMC_BASE + 0x6C)	/* IRQ from SDIO card 0 */
#define MP200_SDIO_INFO1_MASK	(MP200_MMC_BASE + 0x70)	/* MASK IRQ from SDIO card 0 */

#define MP200_MMC_CC_EXT_MODE	(MP200_MMC_BASE + 0x1B0)
#define MP200_MMC_SOFT_RST		(MP200_MMC_BASE + 0x1C0)

#define MP200_MMC_USER			(MP200_MMC_BASE + 0x200)
#define MP200_MMC_USER2			(MP200_MMC_BASE + 0x204)


/* Command */
#define MMC_CMD_ACMD		(1 << 6)
#define MMC_CMD_RSP_NONE	(3 << 8)
#define MMC_CMD_RSP_R1		(4 << 8)
#define MMC_CMD_RSP_R1B		(5 << 8)
#define MMC_CMD_RSP_R2		(6 << 8)
#define MMC_CMD_RSP_R3		(7 << 8)
#define MMC_CMD_DATA		(1 << 11)
#define MMC_CMD_READ		(1 << 12)
#define MMC_CMD_MULTI		(1 << 13)
#define MMC_CMD_SDIO_NO_AUTO_STOP	(1 << 14)

/* Stop */
#define MMC_STOP_STOP		(1 << 0)
#define MMC_STOP_MULTI		(1 << 8)

/* Interrupts */
#define MMC_INFO_EOC		(1 << 0)
#define MMC_INFO_RWEND		(1 << 2)

#define MMC_INFO_CMDERR		(1 << (0+16))
#define MMC_INFO_CRCERR		(1 << (1+16))
#define MMC_INFO_ENDERR		(1 << (2+16))
#define MMC_INFO_DTO		(1 << (3+16))
#define MMC_INFO_ILAW		(1 << (4+16))
#define MMC_INFO_ILAR		(1 << (5+16))
#define MMC_INFO_CTO		(1 << (6+16))
#define MMC_INFO_DAT0		(1 << (7+16))
#define MMC_INFO_BRE		(1 << (8+16))
#define MMC_INFO_BWE		(1 << (9+16))
#define MMC_INFO_CB			(1 << (14+16))
#define MMC_INFO_ILA		(1 << (15+16))

#define MMC_INFO_ALLERR		0x807F0000
//#define MMC_INFO_MASK		0x0300FFFA
#define MMC_INFO_MASK		0x0300FFFE

#define MMC_SDIO_MODE_IOMOD       (1 << 0)
#define MMC_SDIO_INFO1_IOIRQ      (1 << 0)
#define MMC_SDIO_INFO1_IOIRQ_MASK (1 << 0)
#define MMC_SDIO_INFO1_MASK	0xFFFF

/* Clock */
#define MMC_CLOCK_DIV2		(  0 << 0)
#define MMC_CLOCK_DIV4		(  1 << 0)
#define MMC_CLOCK_DIV8		(  2 << 0)
#define MMC_CLOCK_DIV16		(  4 << 0)
#define MMC_CLOCK_DIV32		(  8 << 0)
#define MMC_CLOCK_DIV64		( 16 << 0)
#define MMC_CLOCK_DIV128	( 32 << 0)
#define MMC_CLOCK_DIV256	( 64 << 0)
#define MMC_CLOCK_DIV512	(128 << 0)
#define MMC_CLOCK_EN		(1 <<8)
#define MMC_CLOCK_OFFEN		(1 << 9)
#define MMC_CLOCK_HIGH		(1 << 10)
#define MMC_CLOCK_DIV_MASK	0xFF

/* Option */
#define MMC_OPTION_WIDTH4	(1 << 15) /* 0:WIDTH14 1:WIDTH1 */

/* Buffer mode */
#define MMC_CC_EXT_MODE_FIFO	0x0000
#define MMC_CC_EXT_MODE_SD_DMA	0x0002

/* Reset */
#define MMC_SOFT_RST_RESET		0x0000
#define MMC_SOFT_RST_UNRESET	0x0007

/* User */
#define MMC_USER_VAL		0x8104	/* SDCLK internal loop */
//#define MMC_USER_VAL		0x8004	/* SDCLK external loop */
#define MMC_USER2_VAL		0x0301
#define MMC_USER2_HIGH_VAL	0x0001

#define MMC_SDWAITCTRL_VAL		0x0300
#define MMC_SDREADCTRL_VAL		0x0000


#define MP200_MAX_SEGS	128	// MAX 64KByte

/* Private structure */
struct mp200_mmc_host {
	struct	device		*dev;
	struct	mmc_host	*mmc;
	struct	mmc_request	*req;
	struct	mmc_command	*cmd;
	struct	mmc_data	*data;
	struct	mmc_command	*stop;

	struct	mutex		mutex;

	u8		bus_width;
	u8		app_mode;
	u32		power_mode;
	u32		clock;

	u32		info_mask;

	u32		dma_num;
	u32		dma_count;
	u32		dma_len[MP200_MAX_SEGS];
	u32		dma_dir;
	u32		data_len;
	u32		dma_address[MP200_MAX_SEGS];
	u16		*buffer;

	dma_regs_t *dma_rx_regs;	/* dma rx register */
	dma_regs_t *dma_tx_regs;	/* dma tx register */
	int		dma_rx_ch;
	int		dma_tx_ch;
	int		dma_ch;

	int		suspended;
	int		irq;
	int		detect_irq;
	int		connect;

#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
	spinlock_t	lock;		/* Mutex */
	int             sdio_irq_enabled;
#endif
};

enum {
  IDX_INFO1_MASK = 0,	/* +0040 */
  IDX_INFO2_MASK,	/* +0044 */
  IDX_CLK_CTRL,		/* +0048 */
  IDX_OPTION,		/* +0050 */
  IDX_SDIO_INFO1_MASK,	/* +0070 */
  IDX_USER,		/* +0200 */
  IDX_USER2,		/* +0204 */

  IDX_REGS_NUM,
};

#endif /* __DRIVERS_MMC_MP200_MMC_H */
