/* 2010-08-25: File added and changed by Sony Corporation */
/*
 *  File Name		: linux/drivers/mmc/host/mp200_mmc_sd.c
 *  Function		: MMC
 *  Release Version : Ver 1.01
 *  Release Date	: 2009/02/04
 *
 *  Copyright (C) NEC Electronics Corporation 2009
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/platform_device.h>
#include <linux/mutex.h>
#include <linux/workqueue.h>
#include <linux/timer.h>
#include <linux/mmc/mmc.h>
#include <linux/mmc/host.h>
#include <linux/mmc/sd.h>
#include <linux/mmc/core.h>
#include <linux/mmc/card.h>
#include <linux/mmc/sdio.h>

#include <asm/io.h>
#include <asm/mach/irq.h>
#include <asm/dma.h>
#include <asm/hardware.h>

#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#ifndef CONFIG_SNSC_MP200_PLATFORM
#include <asm/arch/pwc.h>
#endif /* !CONFIG_SNSC_MP200_PLATFORM */
#include <asm/arch/gpio.h>

#ifdef CONFIG_PM
#include <linux/notifier.h>
#include <linux/pm.h>
#endif
#ifdef CONFIG_SNSC_MP200_PLATFORM
#include <asm/arch/mmc_sd.h>
#endif /* CONFIG_SNSC_MP200_PLATFORM */
#define MMC_WP_DISABLE

#include "mp200_mmc.h"

#define DRIVER_NAME	"mp200_mmc_sd"

#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
#define DECLARE_MP200_MMC_HOST(host)
#define MP200_MMC_BASE	(IO_ADDRESS(MP200_SDIA_BASE))
#define MP200_MMC_DMASD		(MP200_SDIA_BASE + 0x300)

#define MMC_CLK_ON	mp200_pmu_open_clockgate(MP200_CLOCKGATE_SDIA_CLK)
#define MMC_CLK_OFF	mp200_pmu_close_clockgate(MP200_CLOCKGATE_SDIA_CLK)
#else /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
#define DECLARE_MP200_MMC_HOST(host)			\
	struct mp200_mmc_host *__mp200_mmc_host = host
#define MP200_MMC_PLATFORM_DATA(host)		\
	((struct mp200_mmc_sd_platform_data *)	\
	 (host)->mmc->parent->platform_data)
#define MP200_MMC_SDI(host)	(MP200_MMC_PLATFORM_DATA(host)->mmc_sdi)
#define MP200_MMC_BASE						\
	(IO_ADDRESS(MP200_MMC_SDI(__mp200_mmc_host)->mmc_base))
#define MP200_MMC_DMASD						\
	(MP200_MMC_SDI(__mp200_mmc_host)->mmc_base + 0x300)

#define MMC_CLK_ON	mp200_pmu_open_clockgate(		\
		MP200_MMC_SDI(__mp200_mmc_host)->mmc_clockgate)

#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
#define MMC_CLK_OFF
#else
#define MMC_CLK_OFF	mp200_pmu_close_clockgate(		\
		MP200_MMC_SDI(__mp200_mmc_host)->mmc_clockgate)
#endif

struct mp200_mmc_sdi mp200_mmc_sdia = {
	.mmc_base        = MP200_SDIA_BASE,
	.mmc_clockgate   = MP200_CLOCKGATE_SDIA_CLK,
	.mmc_resetdevice = MP200_RESETDEVICE_SDIA,
	.mmc_waitctrl    = SMU_AB1_SDIAWAITCTRL,
	.mmc_readctrl    = SMU_AB1_SDIAREADCTRL,
	.mmc_int         = INT_SDIASD,
	.mmc_dma_rx_ch   = MP200_DMAC_P2M_LCH3,
	.mmc_dma_tx_ch   = MP200_DMAC_M2P_LCH3,
	.mmc_save        = NULL,
};
EXPORT_SYMBOL(mp200_mmc_sdia);

struct mp200_mmc_sdi mp200_mmc_sdib = {
	.mmc_base        = MP200_SDIB_BASE,
	.mmc_clockgate   = MP200_CLOCKGATE_SDIB_CLK,
	.mmc_resetdevice = MP200_RESETDEVICE_SDIB,
	.mmc_waitctrl    = SMU_AB1_SDIBWAITCTRL,
	.mmc_readctrl    = SMU_AB1_SDIBREADCTRL,
	.mmc_int         = INT_SDIBSD,
	.mmc_dma_rx_ch   = MP200_DMAC_P2M_LCH4,
	.mmc_dma_tx_ch   = MP200_DMAC_M2P_LCH4,
	.mmc_save        = NULL,
};
EXPORT_SYMBOL(mp200_mmc_sdib);
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */

//#define DEBUG_PRINT
#ifdef DEBUG_PRINT
#define FUNC_PRINT(FMT,ARGS...)		printk(KERN_INFO "%s(): " FMT, __FUNCTION__ , ##ARGS)
#else
#define FUNC_PRINT(FMT,ARGS...)		
#endif

static inline u32 mp200_mmc_info_read(struct mp200_mmc_host *host)
{
	u32 info;
	DECLARE_MP200_MMC_HOST(host);

	info = inl(MP200_MMC_INFO1) & 0xffff;
	info |= ((inl(MP200_MMC_INFO2) & 0xffff) << 16);

	return info;
}

/*
 * Notify the core about command completion
 */
static void
mp200_mmc_data_done(struct mp200_mmc_host *host, struct mmc_command *cmd)
{
	DECLARE_MP200_MMC_HOST(host);

	outl(MMC_STOP_STOP, MP200_MMC_STOP);
	outl(MMC_CC_EXT_MODE_FIFO, MP200_MMC_CC_EXT_MODE);

	if(host->data != NULL) {
		dma_unmap_sg(mmc_dev(host->mmc), host->data->sg, host->dma_num, host->dma_dir);

		if (!host->data->error) {
			host->data->bytes_xfered = host->data_len;
		}
	}
	host->data_len = 0;

	host->data = NULL;
	host->cmd = NULL;
	host->req = NULL;
	host->stop = NULL;

	/* Clear interrupt */
	outl(0, MP200_MMC_INFO1);
	outl(0, MP200_MMC_INFO2);
	host->info_mask = MMC_INFO_MASK;
	outl(MMC_INFO_MASK & 0xffff, MP200_MMC_INFO1_MASK);
	MMC_CLK_OFF;

	mmc_request_done(host->mmc, cmd->mrq);
}

/*
 * Notify the core about command completion
 */
static void
mp200_mmc_cmd_done(struct mp200_mmc_host *host, struct mmc_command *cmd)
{
	u32 val;
	DECLARE_MP200_MMC_HOST(host);

	if (cmd->flags & MMC_RSP_PRESENT) {
		if (cmd->flags & MMC_RSP_136) {
			/* response type 2 */
			val = inl(MP200_MMC_RSP0);
			cmd->resp[3] = val << 8;
			val = inl(MP200_MMC_RSP1);
			cmd->resp[3] |= (val & 0xff) << 24;
			cmd->resp[2] = val >> 8;
			val = inl(MP200_MMC_RSP2);
			cmd->resp[2] |= val << 8;
			val = inl(MP200_MMC_RSP3);
			cmd->resp[2] |= (val & 0xff) << 24;
			cmd->resp[1] = val >> 8;
			val = inl(MP200_MMC_RSP4);
			cmd->resp[1] |= val << 8;
			val = inl(MP200_MMC_RSP5);
			cmd->resp[1] |= (val & 0xff) << 24;
			cmd->resp[0] = val >> 8;
			val = inl(MP200_MMC_RSP6);
			cmd->resp[0] |= val << 8;
			val = inl(MP200_MMC_RSP7);
			cmd->resp[0] |= (val & 0xff) << 24;
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d RSP %x %x %x %x\n", __FUNCTION__, cmd->opcode, cmd->resp[0], cmd->resp[1], cmd->resp[2], cmd->resp[3]);
		} else {
			/* response types 1, 1b, 3, 4, 5, 6 */
			cmd->resp[0] = inl(MP200_MMC_RSP0);
			cmd->resp[0] |= (inl(MP200_MMC_RSP1) << 16);
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d RSP %x\n", __FUNCTION__, cmd->opcode, cmd->resp[0]);
		}
	}

	if( cmd->flags & MMC_RSP_BUSY ) {
		/* wait DATA0 == 1 */
		while( !(mp200_mmc_info_read(host) & MMC_INFO_DAT0) ) {
		}
	}

	if (host->app_mode) {
		host->app_mode = 0;
	} else if (cmd->opcode == 55) {
		host->app_mode = 1;
	}

	if (host->data == NULL) {
		/* Clear interrupt */
		outl(0, MP200_MMC_INFO1);
		outl(0, MP200_MMC_INFO2);
		MMC_CLK_OFF;
		host->req = NULL;
		host->cmd = NULL;
		mmc_request_done(host->mmc, cmd->mrq);
	} else if (host->data->error) {
		mp200_mmc_data_done(host, host->cmd);
	} else {
		val = inl(MP200_MMC_INFO1_MASK) | MMC_INFO_EOC;
		host->info_mask = val;
		outl(val, MP200_MMC_INFO1_MASK);
	}
}

/*
 * Configure the Responce type
 */
static void
mp200_mmc_start_cmd(struct mp200_mmc_host *host, struct mmc_command *cmd, u16 cmddat)
{
	u32 val;
	DECLARE_MP200_MMC_HOST(host);

	host->cmd = cmd;

	cmddat |= cmd->opcode;

	switch (mmc_resp_type(cmd)) {
	case MMC_RSP_R1: /* short CRC, OPCODE */
		cmddat |= MMC_CMD_RSP_R1;
		break;
	case MMC_RSP_R1B:/* short CRC, OPCODE, BUSY */
		cmddat |= MMC_CMD_RSP_R1B;
		break;
	case MMC_RSP_R2: /* long 136 bit + CRC */
		cmddat |= MMC_CMD_RSP_R2;
		break;
	case MMC_RSP_R3: /* short */
		cmddat |= MMC_CMD_RSP_R3;
		break;
	}

	if (host->app_mode) {
		cmddat |= MMC_CMD_ACMD;
	}

	val = inl(MP200_MMC_STOP) & ~MMC_STOP_STOP;
	outl(val, MP200_MMC_STOP);

	outl(cmd->arg & 0xffff, MP200_MMC_ARG0);
	outl(cmd->arg >> 16,    MP200_MMC_ARG1);

	if (cmd->opcode == SD_IO_RW_EXTENDED)
		cmddat |= MMC_CMD_SDIO_NO_AUTO_STOP;


	dev_dbg(mmc_dev(host->mmc), "CMD %d %x %x %x %x\n",
			cmd->opcode, cmd->arg, cmd->flags, cmd->retries, cmddat);

/*
	printk("CMD %d %x %x %x %x\n",
			cmd->opcode, cmd->arg, cmd->flags, cmd->retries, cmddat);
*/

	/* Send command */
	outl(cmddat, MP200_MMC_CMD);
}

/*
 * DMA clean up for command errors
 */
static void mp200_mmc_dma_cleanup(struct mp200_mmc_host *host)
{
	struct mmc_data *data = host->data;

	if (host->dma_ch != -1) {
		dma_unmap_sg(mmc_dev(host->mmc), data->sg, host->dma_num, host->dma_dir);
		mp200_reset_dma(host->dma_ch);
		host->dma_ch = -1;
	}
}

#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
/*
 * MMC controller SDIO IRQ handler
 */
static irqreturn_t
mp200_mmc_sdio_irq(int irq, void *dev_id)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)dev_id;
	u32 sdio_info;
	unsigned long flags;
	DECLARE_MP200_MMC_HOST(host);

	spin_lock_irqsave(&host->lock, flags);

	if (!host->sdio_irq_enabled) {

		/* clear sdio info */
		outl(0, MP200_SDIO_INFO1);

		spin_unlock_irqrestore(&host->lock, flags);
		return IRQ_HANDLED;
	}

	sdio_info = inl(MP200_SDIO_INFO1);
	outl(~sdio_info, MP200_SDIO_INFO1); /* clear sdio info */

	if (sdio_info & MMC_SDIO_INFO1_IOIRQ)
		mmc_signal_sdio_irq(host->mmc);

	spin_unlock_irqrestore(&host->lock, flags);
	return IRQ_HANDLED;
}
#endif /* CONFIG_GROVER_SUPPORT_SDIO_IRQ */

/*
 * MMC controller IRQ handler
 */
static irqreturn_t
mp200_mmc_irq(int irq, void *dev_id)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)dev_id;
	u32 info, end_trans = 0;
	DECLARE_MP200_MMC_HOST(host);

#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
	mp200_mmc_sdio_irq(irq, dev_id);
#endif

	info = mp200_mmc_info_read(host) & ~host->info_mask;

/*
	printk("%s: %x\n", __FUNCTION__, info);
*/
	if (host->cmd == NULL) {
		outl(0, MP200_MMC_INFO1);
		outl(0, MP200_MMC_INFO2);
		return IRQ_HANDLED;
	}

	if (info & MMC_INFO_ALLERR) {
		end_trans = 1;
		if (info & MMC_INFO_CTO) {
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d CMD Timeout\n", __FUNCTION__, host->cmd->opcode);
			host->cmd->error = -ETIMEDOUT;
		} else if (info & MMC_INFO_CRCERR) {
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d CRC error\n", __FUNCTION__, host->cmd->opcode);
			host->cmd->error = -EILSEQ;
		} else {
			dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d Other error.(0x%x)\n", __FUNCTION__, host->cmd->opcode, info);
			host->cmd->error = -EILSEQ;
		}
		if (host->data) {
			if (info & MMC_INFO_DTO) {
				dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d DATA Timeout\n", __FUNCTION__, host->cmd->opcode);
				host->data->error = -ETIMEDOUT;
			} else {
				dev_dbg(mmc_dev(host->mmc), "%s: CMD=%d Other error.(0x%x)\n", __FUNCTION__, host->cmd->opcode, info);
				host->data->error = -EILSEQ;
			}
			mp200_mmc_dma_cleanup(host);
		}
	}

	if ((info & MMC_INFO_EOC) || end_trans) {
		mp200_mmc_cmd_done(host, host->cmd);
		return IRQ_HANDLED;
	}
	if ((info & MMC_INFO_RWEND)) {
		mp200_mmc_data_done(host, host->cmd);
		return IRQ_HANDLED;
	}

	return IRQ_HANDLED;
}

#ifndef CONFIG_SNSC_MP200_PLATFORM
/*
 * IRQ for handling card insertion and removal
 */
static irqreturn_t
mp200_mmc_detect_irq(int irq, void *dev_id)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)dev_id;
	unsigned int tmp_data;

	FUNC_PRINT("\n");
	pwc_read(PWC_IOIN2, &tmp_data);
	if (tmp_data & 0x40) {
		/* fall edge */
		pwc_write(PWC_DETMOD2H, 0x20, 0x30);
		host->connect = 0;
	} else {
		/* rise edge */
		pwc_write(PWC_DETMOD2H, 0x10, 0x30);
		host->connect = 1;
	}
	mmc_detect_change(host->mmc, 0);

	return IRQ_HANDLED;
}
#endif /* !CONFIG_SNSC_MP200_PLATFORM */

static void mp200_mmc_dma_cb(void *data, int intsts, int intrawsts)
{
	struct mp200_mmc_host *host = (struct mp200_mmc_host *)data;
	u32 val;
	DECLARE_MP200_MMC_HOST(host);

	host->dma_count++;

	if (host->dma_count >= host->dma_num) {
		val = inl(MP200_MMC_INFO1_MASK) & ~MMC_INFO_RWEND;
		host->info_mask = val;
		outl(val, MP200_MMC_INFO1_MASK);
		host->dma_ch = -1;
	} else {
		if (host->dma_ch == host->dma_rx_ch) {
			mp200_start_dma(host->dma_rx_ch, MP200_MMC_DMASD, host->dma_len[host->dma_count],
				host->dma_address[host->dma_count], MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
		} else {
			mp200_start_dma(host->dma_tx_ch, host->dma_address[host->dma_count], host->dma_len[host->dma_count],
				MP200_MMC_DMASD, MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
		}
	}
}

/*
 * Configure block leangth for MMC/SD cards and intiate the transfer.
 */
static void
mp200_mmc_setup_data(struct mp200_mmc_host *host, struct mmc_data *data)
{
	int i;
	DECLARE_MP200_MMC_HOST(host);

	host->data = data;

	host->data_len = data->blksz * data->blocks;

	dev_dbg(mmc_dev(host->mmc), "MMC: DATA: size=%d num=%d flags=0x%x\n", data->blksz, data->blocks, data->flags);

	outl(data->blksz,  MP200_MMC_SIZE);
	outl(data->blocks, MP200_MMC_SECCNT);

	host->dma_dir = (data->flags & MMC_DATA_WRITE) ?
				DMA_TO_DEVICE : DMA_FROM_DEVICE;

	host->dma_num = dma_map_sg(mmc_dev(host->mmc), data->sg,
				data->sg_len, host->dma_dir);

	for (i = 0; i < host->dma_num; i++) {
		host->dma_address[i] = sg_dma_address(&data->sg[i]);
		host->dma_len[i] = sg_dma_len(&data->sg[i]);
	}

	outl(MMC_CC_EXT_MODE_SD_DMA, MP200_MMC_CC_EXT_MODE);

	host->dma_count = 0;

	if (data->flags & MMC_DATA_READ) {
		host->dma_ch = host->dma_rx_ch;
		mp200_start_dma(host->dma_rx_ch, MP200_MMC_DMASD, host->dma_len[0],
			host->dma_address[0], MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
	} else {
		host->dma_ch = host->dma_tx_ch;
		mp200_start_dma(host->dma_tx_ch, host->dma_address[0], host->dma_len[0],
			MP200_MMC_DMASD, MP200_DMAC_INT_ERROR_EN | MP200_DMAC_INT_LENG_EN);
	}

/*
	printk("MMC: dma_num=%d sg_len=%d\n", host->dma_num, data->sg_len);
*/
}

/*
 * Request function. for read/write operation
 */
static void
mp200_mmc_request(struct mmc_host *mmc, struct mmc_request *req)
{
	struct mp200_mmc_host *host = mmc_priv(mmc);
	u16 cmddat = 0;
	u32 info;
	DECLARE_MP200_MMC_HOST(host);

	host->req = req;
	host->stop = req->stop;

	MMC_CLK_ON;
	info = mp200_mmc_info_read(host);

	/* Clear stat */
	outl(0, MP200_MMC_INFO1);
	outl(0, MP200_MMC_INFO2);
	req->cmd->error = 0;

	if (req->data) {
		req->data->error = 0;
		mp200_mmc_setup_data(host, req->data);

		cmddat |= MMC_CMD_DATA;

		if (host->data->flags & MMC_DATA_READ) {
			cmddat |= MMC_CMD_READ;
		}

		if (host->data->blocks > 1) {
			cmddat |= MMC_CMD_MULTI;
			outl(MMC_STOP_MULTI, MP200_MMC_STOP);
		}
	}

	mp200_mmc_start_cmd(host, req->cmd, cmddat);
}

static void mp200_mmc_power(struct mp200_mmc_host *host, u32 power_on)
{
	DECLARE_MP200_MMC_HOST(host);

	if (power_on) {
		MMC_CLK_ON;
#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
		mp200_pmu_unreset_device(MP200_RESETDEVICE_SDIA);

		outl(MMC_SDWAITCTRL_VAL,     SMU_AB1_SDIAWAITCTRL);
		outl(MMC_SDREADCTRL_VAL,     SMU_AB1_SDIAREADCTRL);
#else /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
		mp200_pmu_unreset_device(MP200_MMC_SDI(host)->mmc_resetdevice);

		outl(MMC_SDWAITCTRL_VAL,     MP200_MMC_SDI(host)->mmc_waitctrl);
		outl(MMC_SDREADCTRL_VAL,     MP200_MMC_SDI(host)->mmc_readctrl);
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
		outl(MMC_USER_VAL,           MP200_MMC_USER);
		outl(MMC_USER2_VAL,          MP200_MMC_USER2);

		outl(MMC_SOFT_RST_RESET,     MP200_MMC_SOFT_RST);
		outl(MMC_SOFT_RST_UNRESET,   MP200_MMC_SOFT_RST);
		outl(MMC_CC_EXT_MODE_FIFO,   MP200_MMC_CC_EXT_MODE);
		outl(MMC_INFO_MASK,          MP200_MMC_INFO1_MASK);
		outl(MMC_INFO_MASK >> 16,    MP200_MMC_INFO2_MASK);

		outl(0, MP200_MMC_INFO1);
		outl(0, MP200_MMC_INFO2);
		outl(0, MP200_SDIO_INFO1);
#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
		outl(MMC_SDIO_MODE_IOMOD, MP200_SDIO_MODE);
#endif
		outl(MMC_SDIO_INFO1_MASK, MP200_SDIO_INFO1_MASK);
		outl(MMC_STOP_STOP, MP200_MMC_STOP);
		outl(0x80EE, MP200_MMC_OPTION);

		MMC_CLK_OFF;
//		enable_irq(host->irq);
	} else {
//		disable_irq(host->irq);
		host->clock = 0;
		host->bus_width = 0;
		MMC_CLK_ON;
#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
		mp200_pmu_reset_device(MP200_RESETDEVICE_SDIA);
#else /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
		mp200_pmu_reset_device(MP200_MMC_SDI(host)->mmc_resetdevice);
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
		MMC_CLK_OFF;
	}
}


/*
 * Configuring clock values.
 */
static void
mp200_mmc_set_ios(struct mmc_host *mmc, struct mmc_ios *ios)
{
	struct mp200_mmc_host *host = mmc_priv(mmc);
	u32 val, wDiv;
	DECLARE_MP200_MMC_HOST(host);

/*
	printk("%s: %d %x %d %d\n", __FUNCTION__,
		ios->clock, ios->power_mode, ios->bus_width, ios->timing);
*/
	if ((host->bus_width != ios->bus_width) && (host->power_mode != MMC_POWER_OFF)) {
		MMC_CLK_ON;
		val = inl(MP200_MMC_OPTION);
		if (ios->bus_width == MMC_BUS_WIDTH_4) {
			host->bus_width = MMC_BUS_WIDTH_4;
			val &= ~MMC_OPTION_WIDTH4;
		} else {
			host->bus_width = MMC_BUS_WIDTH_1;
			val |= MMC_OPTION_WIDTH4;
		}
		outl(val, MP200_MMC_OPTION);
		MMC_CLK_OFF;
	}

	if ((ios->clock != host->clock) && (host->power_mode != MMC_POWER_OFF)) {
		MMC_CLK_ON;
		/* Stop clock */
		outl(0, MP200_MMC_CLK_CTRL);

		if (ios->clock >= 40000000) {
			wDiv = MMC_CLOCK_HIGH   | MMC_CLOCK_OFFEN;
			outl(MMC_USER2_HIGH_VAL, MP200_MMC_USER2);
		} else if (ios->clock >= 20000000) {
			wDiv = MMC_CLOCK_DIV4   | MMC_CLOCK_OFFEN;
		} else if (ios->clock >= 10000000) {
			wDiv = MMC_CLOCK_DIV8   | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=  5000000) {
			wDiv = MMC_CLOCK_DIV16  | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=  2500000) {
			wDiv = MMC_CLOCK_DIV32  | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=  1250000) {
			wDiv = MMC_CLOCK_DIV64  | MMC_CLOCK_OFFEN;
		} else if (ios->clock >=   625000) {
			wDiv = MMC_CLOCK_DIV128 | MMC_CLOCK_OFFEN;
		} else {
			wDiv = MMC_CLOCK_DIV256;
		}

		wDiv |= MMC_CLOCK_EN;
#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
		wDiv &= ~MMC_CLOCK_OFFEN;
#endif

		/* Start clock */
		outl(wDiv, MP200_MMC_CLK_CTRL);
		MMC_CLK_OFF;
		host->clock = ios->clock;
	}

	if (host->power_mode != ios->power_mode) {
		switch (ios->power_mode) {
		case MMC_POWER_OFF:
			mp200_mmc_power(host, 0);
			dev_dbg(mmc_dev(host->mmc), "power off\n");
       		break;
		case MMC_POWER_UP:
			mp200_mmc_power(host, 1);
			dev_dbg(mmc_dev(host->mmc), "power up\n");
       		break;
		case MMC_POWER_ON:
			dev_dbg(mmc_dev(host->mmc), "power on\n");
       		break;
		}
		host->power_mode = ios->power_mode;
	}
}

#ifndef MMC_WP_DISABLE
static int mp200_mmc_get_ro(struct mmc_host *mmc)
{
	int ret;

#ifdef CONFIG_SNSC_MP200_PLATFORM
	ret = 0;
	if (mmc->parent->platform_data != NULL) {
		struct mp200_mmc_sd_platform_data *platform_data;
		platform_data = mmc->parent->platform_data;
		if (platform_data->mmc_sd_get_ro != NULL) {
			ret = platform_data->mmc_sd_get_ro(mmc);
		}
	}
#else /* CONFIG_SNSC_MP200_PLATFORM */
	ret = gpio_get_value(GPIO_SDWP);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	dev_dbg(mmc_dev(mmc), "write protect = %d\n", ret);

	return ret;
}
#endif

#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
static void mp200_mmc_enable_sdio_irq(struct mmc_host *mmc, int enable)
{
	struct mp200_mmc_host *host = mmc_priv(mmc);
	unsigned long flags;
	u32 mask;
	DECLARE_MP200_MMC_HOST(host);

	spin_lock_irqsave(&host->lock, flags);

	host->sdio_irq_enabled = enable;
	if (enable) {
		mask = inl(MP200_SDIO_INFO1_MASK) & 0xFFFF;
		mask &= ~MMC_SDIO_INFO1_IOIRQ_MASK;
		outl(mask, MP200_SDIO_INFO1_MASK);
		outl(MMC_SDIO_MODE_IOMOD, MP200_SDIO_MODE);
	} else {
		mask = inl(MP200_SDIO_INFO1_MASK);
		mask |= MMC_SDIO_INFO1_IOIRQ_MASK;
		outl(mask, MP200_SDIO_INFO1_MASK);
	}

	spin_unlock_irqrestore(&host->lock, flags);
}
#endif /* CONFIG_GROVER_SUPPORT_SDIO_IRQ */

static struct mmc_host_ops mp200_mmc_ops = {
	.request	= mp200_mmc_request,
	.set_ios	= mp200_mmc_set_ios,
#ifndef MMC_WP_DISABLE
	.get_ro		= mp200_mmc_get_ro,
#endif
#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
	.enable_sdio_irq = mp200_mmc_enable_sdio_irq,
#endif
};

static int
mp200_mmc_probe(struct platform_device *pdev)
{
	struct mmc_host *mmc;
	struct mp200_mmc_host *host = NULL;
	int ret = 0;
#ifndef CONFIG_SNSC_MP200_PLATFORM
	unsigned int tmp_data;
#endif /* !CONFIG_SNSC_MP200_PLATFORM */

	mmc = mmc_alloc_host(sizeof(struct mp200_mmc_host), &pdev->dev);
	if (!mmc) {
		ret = -ENOMEM;
		goto err;
	}

	mmc->ops   = &mp200_mmc_ops;
	mmc->f_min = 400000;
	mmc->f_max = 50000000;
	mmc->ocr_avail = MMC_VDD_28_29 | MMC_VDD_32_33 | MMC_VDD_33_34;
	mmc->caps = MMC_CAP_4_BIT_DATA | MMC_CAP_BYTEBLOCK | MMC_CAP_MULTIWRITE |
				MMC_CAP_MMC_HIGHSPEED | MMC_CAP_SD_HIGHSPEED;
#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
	mmc->caps |= MMC_CAP_SDIO_IRQ;
#endif

	/* MMC core transfer sizes tunable parameters */
	mmc->max_hw_segs   = MP200_MAX_SEGS;
	mmc->max_phys_segs = MP200_MAX_SEGS;
	mmc->max_seg_size  = MP200_MAX_SEGS * 512;
	mmc->max_req_size  = MP200_MAX_SEGS * 512;
	mmc->max_blk_size  = 512;
	mmc->max_blk_count = 65535;

	host      = mmc_priv(mmc);
	host->mmc = mmc;
#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
	host->irq = INT_SDIASD;
#else /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
	host->irq = MP200_MMC_SDI(host)->mmc_int;
#ifdef CONFIG_PM
	MP200_MMC_SDI(host)->mmc_save =
		kmalloc(sizeof(unsigned int) * IDX_REGS_NUM , GFP_KERNEL);
	if (MP200_MMC_SDI(host)->mmc_save == NULL) {
		ret = -ENOMEM;
		goto err;
	}
#endif /* CONFIG_PM */
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
#ifndef CONFIG_SNSC_MP200_PLATFORM
	host->detect_irq = INT_PWC_IO14;
#endif /* !CONFIG_SNSC_MP200_PLATFORM */
	host->power_mode = MMC_POWER_OFF;
	host->info_mask = MMC_INFO_MASK;
	mutex_init(&host->mutex);
#ifdef CONFIG_GROVER_SUPPORT_SDIO_IRQ
	spin_lock_init(&host->lock);
#endif

	/* DMA setting */
#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
	host->dma_rx_ch = MP200_DMAC_P2M_LCH3;
	host->dma_tx_ch = MP200_DMAC_M2P_LCH3;
#else /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
	host->dma_rx_ch = MP200_MMC_SDI(host)->mmc_dma_rx_ch;
	host->dma_tx_ch = MP200_MMC_SDI(host)->mmc_dma_tx_ch;
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */

	ret = mp200_request_dma(host->dma_rx_ch, DRIVER_NAME,
				mp200_mmc_dma_cb, (void *)host, &host->dma_rx_regs);
	if (ret < 0) {
		goto err;
	}

	ret = mp200_request_dma(host->dma_tx_ch, DRIVER_NAME,
				mp200_mmc_dma_cb, (void *)host, &host->dma_tx_regs);
	if (ret < 0) {
		goto err_dma;
	}

	/* Request IRQ for MMC operations */
	ret = request_irq(host->irq, mp200_mmc_irq, IRQF_DISABLED, pdev->name, host);
	if (ret) {
		dev_dbg(mmc_dev(host->mmc), "Unable to IRQ");
		goto err_irq;
	}
//	disable_irq(host->irq);

#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pdev->dev.platform_data != NULL) {
		struct mp200_mmc_sd_platform_data *platform_data;
		platform_data = pdev->dev.platform_data;
		if (platform_data->mmc_sd_probe != NULL) {
			ret = platform_data->mmc_sd_probe(pdev, mmc);
			if (ret < 0)
				goto err_irq2;
		}
	}
#else /* CONFIG_SNSC_MP200_PLATFORM */
	/* chattering ON */
	pwc_write(PWC_STATE2, 0x40, 0x40);

	pwc_read(PWC_IOIN2, &tmp_data);
	if (tmp_data & 0x40) {
		/* fall edge */
		set_irq_type(host->detect_irq, IRQ_TYPE_EDGE_FALLING);
		host->connect = 0;
	} else {
		/* rise edge */
		set_irq_type(host->detect_irq, IRQ_TYPE_EDGE_RISING);
		host->connect = 1;
	}

	/* Request IRQ for MMC card detect */
	ret = request_irq(host->detect_irq, mp200_mmc_detect_irq, IRQF_DISABLED, pdev->name, host);
	if (ret) {
		dev_dbg(mmc_dev(host->mmc), "Unable to Card detect IRQ");
		goto err_irq2;
	}
#endif /* CONFIG_SNSC_MP200_PLATFORM */
	platform_set_drvdata(pdev, host);
	mmc_add_host(mmc);

	return 0;

err_irq2:
	free_irq(host->irq, host);
err_irq:
	mp200_free_dma(host->dma_tx_ch);
err_dma:
	mp200_free_dma(host->dma_rx_ch);

err:
	printk("%s: error!!(%d)\n", __FUNCTION__, ret);
#if defined(CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS) && defined(CONFIG_PM)
	if (MP200_MMC_SDI(host)->mmc_save != NULL)
		kfree(MP200_MMC_SDI(host)->mmc_save);
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS && CONFIG_PM */
	if (host)
		mmc_free_host(mmc);
	return ret;

}

static int
mp200_mmc_remove(struct platform_device *pdev)
{
	struct mp200_mmc_host *host = platform_get_drvdata(pdev);

	if (host) {
		flush_scheduled_work();

		mmc_remove_host(host->mmc);

		mp200_mmc_power(host, 0);

#ifdef CONFIG_SNSC_MP200_PLATFORM
		if (pdev->dev.platform_data != NULL) {
			struct mp200_mmc_sd_platform_data *platform_data;
			platform_data = pdev->dev.platform_data;
			if (platform_data->mmc_sd_remove != NULL) {
				platform_data->mmc_sd_remove(pdev);
			}
		}
#else /* CONFIG_SNSC_MP200_PLATFORM */
		free_irq(host->detect_irq, host);
#endif /* CONFIG_SNSC_MP200_PLATFORM */
		free_irq(host->irq, host);

		mp200_free_dma(host->dma_rx_ch);
		mp200_free_dma(host->dma_tx_ch);

#if defined(CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS) && defined(CONFIG_PM)
		if (MP200_MMC_SDI(host)->mmc_save != NULL)
			kfree(MP200_MMC_SDI(host)->mmc_save);
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS && CONFIG_PM */

		mmc_free_host(host->mmc);

		platform_set_drvdata(pdev, NULL);
	}

	return 0;
}

#ifdef CONFIG_PM
#ifndef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
static unsigned int mmc_save[IDX_REGS_NUM];
#endif /* !CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */

static void mmc_reg_save(struct mp200_mmc_host *host)
{
	DECLARE_MP200_MMC_HOST(host);
#ifdef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
	unsigned int *mmc_save;

	mmc_save = MP200_MMC_SDI(host)->mmc_save;
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
	mmc_save[IDX_CLK_CTRL] = inl(MP200_MMC_CLK_CTRL);
	mmc_save[IDX_OPTION] = inl(MP200_MMC_OPTION);
	mmc_save[IDX_SDIO_INFO1_MASK] = inl(MP200_SDIO_INFO1_MASK);
	mmc_save[IDX_INFO2_MASK] = inl(MP200_MMC_INFO2_MASK);
	mmc_save[IDX_INFO1_MASK] = inl(MP200_MMC_INFO1_MASK);
	mmc_save[IDX_USER2] = inl(MP200_MMC_USER2);
	mmc_save[IDX_USER] = inl(MP200_MMC_USER);

	return;
}

static void mmc_reg_restore(struct mp200_mmc_host *host)
{
	DECLARE_MP200_MMC_HOST(host);
#ifdef CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS
	unsigned int *mmc_save;

	mmc_save = MP200_MMC_SDI(host)->mmc_save;
#endif /* CONFIG_SNSC_MMC_SD_MP200_MULTIPLE_HOSTS */
	outl(mmc_save[IDX_USER], MP200_MMC_USER);
	outl(mmc_save[IDX_USER2], MP200_MMC_USER2);

	outl(MMC_SOFT_RST_RESET,     MP200_MMC_SOFT_RST);
	outl(MMC_SOFT_RST_UNRESET,   MP200_MMC_SOFT_RST);
	outl(MMC_CC_EXT_MODE_FIFO,   MP200_MMC_CC_EXT_MODE);

	outl(0, MP200_MMC_INFO1);
	outl(0, MP200_MMC_INFO2);
	outl(0, MP200_SDIO_INFO1);
	outl(mmc_save[IDX_INFO1_MASK], MP200_MMC_INFO1_MASK);
	outl(mmc_save[IDX_INFO2_MASK], MP200_MMC_INFO2_MASK);
	outl(mmc_save[IDX_SDIO_INFO1_MASK], MP200_SDIO_INFO1_MASK);

	outl(mmc_save[IDX_OPTION], MP200_MMC_OPTION);
	outl(mmc_save[IDX_CLK_CTRL], MP200_MMC_CLK_CTRL);

	return;
}

static int
mp200_mmc_suspend(struct platform_device *pdev, pm_message_t state)
{
	struct mp200_mmc_host *host = platform_get_drvdata(pdev);
#ifdef CONFIG_SNSC_MP200_PLATFORM
	struct mp200_mmc_sd_platform_data *platform_data;
	int ret;
#endif /* CONFIG_SNSC_MP200_PLATFORM */
	DECLARE_MP200_MMC_HOST(host);

/*
	printk("%s: %x\n", __FUNCTION__, state.event);
*/
	if (!host || host->suspended)
		return 0;

	if (!mp200_sleep_while_idle) {
		/* detect interrupt disable */
		disable_irq(host->detect_irq);
#ifdef CONFIG_SNSC_MP200_PLATFORM
		if (pdev->dev.platform_data != NULL) {
			platform_data = pdev->dev.platform_data;
			if (platform_data->mmc_sd_suspend != NULL) {
				ret = platform_data->mmc_sd_suspend(pdev,
								    state);
				if (ret < 0) {
					enable_irq(host->detect_irq);
					return (ret);
				}
			}
		}
#endif /* CONFIG_SNSC_MP200_PLATFORM */
		FUNC_PRINT("in\n");
		mmc_suspend_host(host->mmc, state);
		host->suspended = 1;
		FUNC_PRINT("out\n");
	} else if (host->clock) {
		MMC_CLK_ON;
		mmc_reg_save(host);
		MMC_CLK_OFF;
		host->suspended = 1;
	}
	return 0;
}

/* Routine to resume the MMC device */
static int
mp200_mmc_resume(struct platform_device *pdev)
{
	struct mp200_mmc_host *host = platform_get_drvdata(pdev);
#ifdef CONFIG_SNSC_MP200_PLATFORM
	struct mp200_mmc_sd_platform_data *platform_data;
	int ret = 0;
#else /*CONFIG_SNSC_MP200_PLATFORM */
	unsigned int tmp_data;
	unsigned long flag;
#endif /*CONFIG_SNSC_MP200_PLATFORM */
	DECLARE_MP200_MMC_HOST(host);

	if (host) {
		if (!mp200_sleep_while_idle) {
			/* detect interrupt enable */
			enable_irq(host->detect_irq);
			
#ifdef CONFIG_SNSC_MP200_PLATFORM
			if (pdev->dev.platform_data != NULL) {
				platform_data = pdev->dev.platform_data;
				if (platform_data->mmc_sd_resume != NULL) {
					ret = platform_data->mmc_sd_resume(
						pdev);
				}
			}
#else /* CONFIG_SNSC_MP200_PLATFORM */
			local_irq_save(flag);
			pwc_read(PWC_IOIN2, &tmp_data);
			if (tmp_data & 0x40) {
				/* fall edge */
				pwc_write(PWC_DETMOD2H, 0x20, 0x30);
				host->connect = 0;
			} else {
				/* rise edge */
				pwc_write(PWC_DETMOD2H, 0x10, 0x30);
				host->connect = 1;
			}
			local_irq_restore(flag);
#endif /*CONFIG_SNSC_MP200_PLATFORM */
			
			mmc_resume_host(host->mmc);
		} else if (host->suspended){
			MMC_CLK_ON;
			mmc_reg_restore(host);
			MMC_CLK_OFF;
		}
		host->suspended = 0;
	}
#ifdef CONFIG_SNSC_MP200_PLATFORM
	return ret;
#else /* CONFIG_SNSC_MP200_PLATFORM */
	return 0;
#endif /* CONFIG_SNSC_MP200_PLATFORM */
}
#else
#define mp200_mmc_suspend	NULL
#define mp200_mmc_resume	NULL
#endif

static struct platform_driver mp200_mmc_driver = {
	.probe		= mp200_mmc_probe,
	.remove		= mp200_mmc_remove,
	.suspend	= mp200_mmc_suspend,
	.resume		= mp200_mmc_resume,
	.driver		= {
			.name = DRIVER_NAME,
	},
};

static int __init mp200_mmc_init(void)
{
	/* Register the MMC driver */
	if (platform_driver_register(&mp200_mmc_driver)) {
		printk(KERN_ERR ": Failed to register MMC driver\n");
		return -ENODEV;
	}
	return 0;
}

static void __exit mp200_mmc_cleanup(void)
{
	/* Unregister MMC driver */
	platform_driver_unregister(&mp200_mmc_driver);
}

module_init(mp200_mmc_init);
module_exit(mp200_mmc_cleanup);

MODULE_DESCRIPTION("MP200 High Speed Multimedia Card driver");
MODULE_LICENSE("GPL");
MODULE_ALIAS(DRIVER_NAME);
MODULE_AUTHOR("NEC Electronics");
