/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>

#include <asm/arch/gpio.h>
#include <asm/arch/pgpf/ax/pm.h>
#include <asm/arch/hardware.h>
#include <asm/irq.h>

#include "internal.h"
#include "ax_pm_irq.h"

static irqreturn_t pwrkey_irq_handler(int irq, void *dev_id);
static irqreturn_t detrtc_irq_handler(int irq, void *dev_id);

static void pwrkey_bh_handler(struct work_struct *work);
static void detrtc_bh_handler(struct work_struct *work);

inline static void do_callback(ax_pm_irq_cb_t irq);

static ax_pm_callback_t callback[AX_PM_IRQ_CB_NUM_MAX];

DECLARE_WORK(ax_pm_pk_ws,  pwrkey_bh_handler);
DECLARE_WORK(ax_pm_rtc_ws, detrtc_bh_handler);

static char dev_id[] = "ax_pm";

static ax_pm_irq_info_t ax_pm_irq_info[] = {
	{PGPF_GPIO_KEY_WAKE,  pwrkey_irq_handler, IRQF_DISABLED, 
	 "KEY_WAKE",   dev_id, IRQT_RISING,   TRUE },

	{INT_PWC_RTCINT,     detrtc_irq_handler, IRQF_DISABLED|IRQF_SHARED, 
	 "DETRTC_IRQ", dev_id, IRQT_RISING,   FALSE },
};

static irqreturn_t pwrkey_irq_handler(int irq, void *dev_id)
{
	return IRQ_HANDLED;
}

static irqreturn_t detrtc_irq_handler(int irq, void *dev_id)
{
	return IRQ_HANDLED;
}

static void pwrkey_bh_handler(struct work_struct *work)
{
	do_callback(AX_PM_IRQ_CB_POWER_KEY);

	return ;
}

static void detrtc_bh_handler(struct work_struct *work)
{
	do_callback(AX_PM_IRQ_CB_RTC_DET);

	return ;
}

inline static void do_callback(ax_pm_irq_cb_t irq)
{
	if(callback[irq].func){
		callback[irq].func(callback[irq].arg);
	}

	return ;
}

int ax_pm_register_irq(void)
{
	ax_pm_status_t ret;
	ax_pm_irq_info_t *p;
	int i;

	/* Register PWC IRQ */ 
	for(i = 0; i < ARRAY_SIZE(ax_pm_irq_info); i++){
		p = &ax_pm_irq_info[i]; 
		if(p->gpio_flag){
			p->irq = gpio_to_irq(p->irq);
		}

		set_irq_type(p->irq, p->trigger_type);
		ret = request_irq(p->irq, p->handler, p->flags, p->dev_name, p->dev_id);
		if(ret){
			DPRINT("## irq(%d) error\n", p->irq);
			return AX_PM_ERR_OTHER;
		}
	}

	return AX_PM_SUCCESS;
}

void ax_pm_register_callback(ax_pm_irq_cb_t irq, void (func)(void *), void *arg)
{
	if(NULL == func){
		DPRINT("## Invalid argument\n");
		return ;
	}

	callback[irq].irq  = irq;
	callback[irq].func = func;
	callback[irq].arg  = arg;

	return ;	
}

