/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/proc_fs.h>
#include <asm/uaccess.h>
#include "chrdv.h"
#include "internal.h"

static int     ax_pm_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data);
static ssize_t ax_pm_write_proc(struct file *file, const char *buffer, size_t count, loff_t * ppos);

ax_pm_status_t ax_pm_proc_register(void)
{
	struct proc_dir_entry *entry;

	/* proc for suspend and battery status */
	entry = create_proc_read_entry("ax_pm", S_IWUSR | S_IRUGO, NULL,
								   ax_pm_read_proc, 0);
	if(NULL == entry){
		return AX_PM_ERR_OTHER;
	}
	entry->write_proc = (write_proc_t *)ax_pm_write_proc;

	return AX_PM_SUCCESS;
}

static int ax_pm_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	return 0;
}

static ssize_t ax_pm_write_proc(struct file *file, const char *buffer, size_t count, loff_t * ppos)
{
    int ret = 0;
    char *buf;
    char *tok;
    char *s;
    char *str;
    char *whitespace = " \t\r\n";

    if (0 == count){
        return ret;
    }

    buf = kmalloc(count + 1, GFP_KERNEL);
    if (buf == NULL) {
        ret = -ENOMEM;
        goto exit_2;
    }

    if (copy_from_user(buf, buffer, count) != 0) {
        ret = -EFAULT;
        goto exit_1;
    }

    buf[count] = '\0';
    s = buf + strspn(buf, whitespace);
    tok = strsep(&s, whitespace);

    str = "suspend";
    if (strnicmp(tok, str, strlen(str) + 1) == 0) {
        DPRINT("Suspend\n");

		ax_pm_request_suspend();

        goto exit_1;
    }

	ret = -EINVAL;

  exit_1:    
	kfree(buf);

  exit_2:

    if (ret == 0) {
        return count;
    }

    return ret;
}

