/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/platform_device.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/wait.h>
#include <linux/freezer.h>
#include <linux/snsc_nblargs.h>
#include <asm/irq.h>
#include <asm/arch/pm.h>
#include <asm/arch/smu.h>
#include <asm/uaccess.h>
#include <asm/semaphore.h>
#include <asm/arch/pwc.h>

#include "internal.h"
#include "chrdv.h"
#include "ax_pm_irq.h"
#include "ax_pm_proc.h"
#include "get_adkey.h"

static int            ax_pm_ioctl(struct inode *inode, struct file *filep, 
								  unsigned int cmd, unsigned long arg);
static int            ax_pm_open(struct inode *inode, struct file *filep);
static int            ax_pm_release(struct inode *inode, struct file *filep);
static ax_pm_status_t ax_pm_wait_irq(ax_pm_irq_t wait_irq);
static void           ax_pm_wait_irq_cancel(ax_pm_irq_t wait_irq);
static void           init_wait_irq(void);

typedef struct {
	int  ready;
	wait_queue_head_t comp;
	int  (*judge)(void);
	int  canceled;
} wait_irq_t;

static struct file_operations ax_pm_fops = {
	.owner   = THIS_MODULE,
	.ioctl   = ax_pm_ioctl,
	.open    = ax_pm_open,
	.release = ax_pm_release,
};

static struct miscdevice ax_pm_miscdev = {
	.minor   = AX_PM_MISCDEV_MINOR,
	.name    = AX_PM_DEV_NAME,
	.fops    = &ax_pm_fops,
};

static DECLARE_MUTEX(_chrdv_mutex);

static wait_irq_t wait_irq[AX_PM_IRQ_NUM_MAX];

static int ax_pm_open(struct inode *inode, struct file *filp)
{
	return 0;
}

static int ax_pm_release(struct inode *inode, struct file *filp)
{
    return 0;
}

static int ax_pm_ioctl(struct inode *inode, struct file *filp, 
						 unsigned int cmd, unsigned long arg)
{
	ax_pm_irq_t irq;
	ax_pm_status_t ret;

	switch(cmd){
	case AX_PM_EVENT_WAIT:

		if(copy_from_user(&irq, (ax_pm_irq_t*)arg, sizeof(ax_pm_irq_t))){
			DPRINT("## Error : copy_from_user()\n");

			return AX_PM_ERR_NOMEM;
		}

		ret = ax_pm_wait_irq(irq);
		if(AX_PM_ERR_CANCEL == ret){
			return AX_PM_ERR_CANCEL;
		}else if(AX_PM_ERR_OTHER == ret){
			return AX_PM_ERR_OTHER;
		}

		break;

	case AX_PM_EVENT_WAIT_CANCEL:

		if(copy_from_user(&irq, (ax_pm_irq_t*)arg, sizeof(ax_pm_irq_t))){
			DPRINT("## Error : copy_from_user()\n");

			return AX_PM_ERR_NOMEM;
		}

		ax_pm_wait_irq_cancel(irq);

		break;

	case AX_PM_SET_MODE_SUSPEND:

		ax_pm_request_suspend();

		break;

	default:
		DPRINT("## Invalid command!!\n");

		return AX_PM_ERR_NOMEM;

		break;		
	}

	return AX_PM_SUCCESS;
}

static ax_pm_status_t ax_pm_wait_irq(ax_pm_irq_t irq)
{
	int judge;
	ax_pm_status_t ret = AX_PM_SUCCESS;

	if(irq < 0 || irq >= AX_PM_IRQ_NUM_MAX){
		DPRINT("## invalid irq num\n");

		return AX_PM_ERR_OTHER;
	}


	if(FALSE == wait_irq[irq].ready){
		init_waitqueue_head(&(wait_irq[irq].comp));
		wait_irq[irq].canceled = FALSE;
		wait_irq[irq].ready = TRUE;
	}
	
	if(NULL != wait_irq[irq].judge){
		judge = wait_irq[irq].judge();
		if(TRUE == judge){
			wait_irq[irq].ready = FALSE;	
			return AX_PM_SUCCESS;
		}
	}

	wait_event_freezable(wait_irq[irq].comp, wait_irq[irq].ready == FALSE);

	if(TRUE == wait_irq[irq].canceled){
		DPRINT("## Wait Canceled\n");

		ret = AX_PM_ERR_CANCEL;
	}

	return ret;
}

static void ax_pm_wait_irq_cancel(ax_pm_irq_t irq)
{
	if(irq < 0 || irq >= AX_PM_IRQ_NUM_MAX){
		DPRINT("## invalid irq num\n");

		return ;
	}

	if(wait_irq[irq].ready){
		wait_irq[irq].canceled = TRUE;
		wait_irq[irq].ready = FALSE;
		wake_up_all(&(wait_irq[irq].comp));
	}

	return ;
}

static void init_wait_irq(void)
{
	ax_pm_irq_t count;

	for(count = 0; count < AX_PM_IRQ_NUM_MAX; count++){
		wait_irq[count].ready = FALSE;

		init_waitqueue_head(&(wait_irq[count].comp));

		switch(count){
		default:
			wait_irq[count].judge = NULL;
			break;
		}
	}

	return ;
}

void ax_pm_request_suspend(void)
{
	mp200_pm_sleep(PM_SLEEP_MODE_S3);

	return ;
}

void complete_wait_irq(ax_pm_irq_t irq)
{
	if(irq < 0 || irq >= AX_PM_IRQ_NUM_MAX){
		return ;
	}

	if(wait_irq[irq].ready){
		wait_irq[irq].ready = FALSE;
		wake_up_all(&(wait_irq[irq].comp));
	}

	return ;
}

ax_pm_status_t chrdv_init(void)
{
	ax_pm_status_t ret;

	init_wait_irq();

	ret = misc_register(&ax_pm_miscdev);
	if(ret < 0){
		DPRINT("## Error misc_register = %d\n", ret);
		return AX_PM_ERR_OTHER;
	}

	ret = ax_pm_register_irq();
	if(AX_PM_SUCCESS != ret){
		DPRINT("## Error ax_pm_register_irq = %d\n", ret);
	}

	ret = ax_pm_proc_register();
	if(AX_PM_SUCCESS != ret){
		DPRINT("## Error ax_pm_proc_register = %d\n", ret);
	}

	ret = adkey_init();
	if(AX_PM_SUCCESS != ret){
		DPRINT("## Error adkey_init = %d\n", ret);
	}

	return AX_PM_SUCCESS;
}

void chrdv_cleanup(void)
{
	misc_deregister(&ax_pm_miscdev);

	return ;
}

