/* 2009-08-24: File added by Sony Corporation */
/**
 ** @file  ad7795.c
 ** @brief A/DC(PPU) driver 
 ** @author Hiroyuki.Motousu@jp.sony.com
 **
 ** $Id:$
 **
 **/
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/fs.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/cdev.h>
#include <linux/platform_device.h>
#include <linux/hrtimer.h>
#include <linux/syscalls.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/semaphore.h>
#include <asm/atomic.h>

#include <linux/kthread.h>
#include <asm/hardware.h>
#include <asm/arch/pgpf/devnum.h>
#include <asm/arch/timer.h>
#include <asm/arch/pmu.h>
#include <asm/arch/smu.h>
#include <asm/arch/dma.h>
#include <asm/arch/spi.h>
#include <asm/arch/pgpf/navu2/ad7795.h>
#include "../../char/mp200_spi0.h"

#ifdef CONFIG_PGPF_NAVU2_PM_SUSPEND
#include <asm/arch/pwc.h>
#define PWC_PSCNTGP2_ON     0x1
#define PWC_PSCNTGP2_OFF    0x0
#define PWC_PSCNTGP2_MASK   0x1
#endif /* CONFIG_PGPF_NAVU2_PM_SUSPEND */

#define BUF_UNREAD (-1)

/* debug print control */
#define AD7795_DEBUG  0

#if AD7795_DEBUG
#define DBG(format, args...)  printk(KERN_DEBUG "%s(): " format, __FUNCTION__, ## args)
#else
#define DBG(format, args...)
#endif

/* Power Mode */
#define ADC_POWER_DOWN  MODEMD_POWERDOWN
#define ADC_IDLE        MODEMD_IDLE

static int DevMajorNumber = NAVU2_MAJOR_ADC;
static int DevMinorNumber = NAVU2_MINOR_ADC;

static int DevCount       = 1;
static char *DevName      = "adc";

typedef struct {
	unsigned char	*buf;
	int				bufsize;
	int				elements;
	int				elementSize;
	int				wrPos;		// Next write positin
	int				rdPos;		// Next read position
	spinlock_t		buflock;
} ad7795_ringbuf;

typedef struct {
    struct semaphore      sem;
    struct cdev           cdev;
    unsigned int          TimerResource;
    adc_service_config_t  config;
    irqreturn_t          (*cb)(int, void *);
    u16                   ChipID;
    int                   used;

    ad7795_ringbuf       *DataBuf;
    PosSensor             AdData;
    Ad7795RegArg          AdSaveReg;

    int                   RdCh;
} AdcDevSt;

AdcDevSt Adc_devst;

static int AdApiFlag = 1;   //TRUE

#define SPI1_BASE_ADDR           IO_ADDRESS(MP200_SP1_BASE)
#define SPI1_MODE_REG_ADDR       (SPI1_BASE_ADDR + 0x00)
#define SPI1_POL_REG_ADDR        (SPI1_BASE_ADDR + 0x04)
#define SPI1_CONTROL_REG_ADDR    (SPI1_BASE_ADDR + 0x08)
#define SPI1_TX_TIM_REG_ADDR     (SPI1_BASE_ADDR + 0x0C) /* no use? */
#define SPI1_TX_DATA_REG_ADDR    (SPI1_BASE_ADDR + 0x10)
#define SPI1_RX_DATA_REG_ADDR    (SPI1_BASE_ADDR + 0x14)
#define SPI1_STATUS_REG_ADDR     (SPI1_BASE_ADDR + 0x18)
#define SPI1_RAW_STATUS_REG_ADDR (SPI1_BASE_ADDR + 0x1C)
#define SPI1_ENSET_REG_ADDR      (SPI1_BASE_ADDR + 0x20)
#define SPI1_ENCLR_REG_ADDR      (SPI1_BASE_ADDR + 0x24)
#define SPI1_FFCLR_REG_ADDR      (SPI1_BASE_ADDR + 0x28)
#define SPI1_TX_FIFO_P_REG_ADDR  (SPI1_BASE_ADDR + 0x2C) /* no use? */
#define SPI1_RX_FIFO_P_REG_ADDR  (SPI1_BASE_ADDR + 0x30) /* no use? */
#define SPI1_CONTROL2_REG_ADDR   (SPI1_BASE_ADDR + 0x34)
#define SPI1_TIECS_REG_ADDR      (SPI1_BASE_ADDR + 0x38)

static irqreturn_t   adc_interrupt(int irq, void *dev_id);
static irqreturn_t   adc_interrupt2(int irq, void *dev_id);
ad7795_ringbuf      *ad7795_ringbuf_init( int elements, int elementSize );
void                 ad7795_ringbuf_free( ad7795_ringbuf *ringbuf );
int                  ad7795_ringbuf_write( ad7795_ringbuf *ringbuf, void *data );
int                  ad7795_ringbuf_read( ad7795_ringbuf *ringbuf, void *data );
static unsigned int  ad7795_get_nbits(unsigned int bytes);
static unsigned int  ad7795_get_nbytes(unsigned int bits);
static u32           ad7795_spi_rw(u32 tx_buf, size_t n_tx, size_t n_rx);
void                 adcTimerService( AdcDevSt *pdev );
void                 adcTimerService_Walk( AdcDevSt *pdev );
static int           ad7795_set_modereg( AdcDevSt *pdev );
static int           adcConnectionCheck( AdcDevSt *pdev );
static int           adcTimerStop( AdcDevSt *pdev );
static void          adcTimerClean(  AdcDevSt *pdev );
static int           adcTimerInit( AdcDevSt *pdev );
static int           adcTimerReStart( AdcDevSt *pdev );
static int           adcTimerStart(AdcDevSt *pdev, adc_service_config_t __user *arg);
static int           adcInit( AdcDevSt *pdev );
static int           ad7795_reg_read(adc_regmsg __user *arg);
static int           ad7795_reg_write(adc_regmsg __user *arg);
static int           adcSetPowerMode( int mode );
static int           ad7795RestoreState( Ad7795RegArg *parg );
static int           adcReadReg( Ad7795RegArg *parg );
static int           adcDebug(void);
static int           adcGetConfig( AdcDevSt *pdev, Ad7795RegArg __user *parg );
static int           adcGetData( AdcDevSt *pdev, AdReadArg __user *parg );
static int           ad7795_get_current_tick(unsigned long __user * arg);
static int           ad7795_reset_device(AdcDevSt *pdev);


//######################################################################
/** 
 * clear ring buffer data
 * 
 * @param ringbuf : entity of ring buffer
 * 
 * @return 0 : success, other : failure
 */
static int ad7795_ringbuf_clear( ad7795_ringbuf *ringbuf )
{
    int err = 0;

    memset(ringbuf->buf, 0, ringbuf->bufsize);

	ringbuf->wrPos       = 0;
	ringbuf->rdPos       = BUF_UNREAD;
	ringbuf->buflock     = SPIN_LOCK_UNLOCKED;
    
    return err;
}

//######################################################################
/** 
 * initiailze function of ring buffer
 * 
 * @param elements    : data element size of ring buffer
 * @param elementSize : data size which you want to save  
 * 
 * @return pointer of ring buffer
 */
ad7795_ringbuf *ad7795_ringbuf_init( int elements, int elementSize )
{
	ad7795_ringbuf *ringbuf;

    DBG("called.\n");
    
	ringbuf = kmalloc( sizeof(ad7795_ringbuf), GFP_KERNEL );
	if( ringbuf == NULL ){
		printk( KERN_ERR "%s() : memory allocation(ringbuf) error.\n", __FUNCTION__ );
		return NULL;
	}
	memset( ringbuf, 0, sizeof(ad7795_ringbuf) );
    
	ringbuf->bufsize = elements * elementSize;

	ringbuf->buf = kmalloc( ringbuf->bufsize, GFP_KERNEL );
	if( ringbuf->buf == NULL ){
		printk( KERN_ERR "%s() : memory allocation (ringbuf->buf) error.\n", __FUNCTION__ );
		kfree( ringbuf );
		return NULL;
	}

	ringbuf->elements    = elements;
	ringbuf->elementSize = elementSize;
	ringbuf->wrPos       = 0;
	ringbuf->rdPos       = BUF_UNREAD;
	ringbuf->buflock     = SPIN_LOCK_UNLOCKED;

	return ringbuf;
}

//######################################################################
/** 
 * free function of ring buffer
 * 
 * @param ringbuf : entity of ring buffer
 */
void ad7795_ringbuf_free( ad7795_ringbuf *ringbuf )
{
    kfree( ringbuf->buf );
	kfree( ringbuf );
    
    return;
}

//######################################################################
/** 
 * write function to ring buffer
 * 
 * @param ringbuf : entity of ring buffer(pointer)
 * @param data    : data which is written to ring buffer
 * 
 * @return 
 */
int ad7795_ringbuf_write( ad7795_ringbuf *ringbuf, void *data )
{

    spin_lock( &ringbuf->buflock );

	memcpy( &ringbuf->buf[ringbuf->wrPos*ringbuf->elementSize],
	        data,
	        ringbuf->elementSize );

	ringbuf->wrPos ++;
	if( ringbuf->wrPos >= ringbuf->elements ){
		ringbuf->wrPos = 0;
		if( ringbuf->rdPos == BUF_UNREAD ){
			ringbuf->rdPos = 1;
		}
	}
	if( ringbuf->wrPos == ringbuf->rdPos ){
		ringbuf->rdPos ++;
		if( ringbuf->rdPos >= ringbuf->elements ){
			ringbuf->rdPos = 0;
		}
	}

	spin_unlock( &ringbuf->buflock );

    return 0;
}

//######################################################################
/** 
 * read function from ring buffer
 * 
 * wrPos > rdPos -- none overlap.
 * wrPos < rdPos -- overlaped
 * wrPos = rdPos -- none data.
 * 
 * @param ringbuf : entity of ring buffer(pointer)
 * @param data    : data which is read from ring buffer
 * 
 * @return read size
 */
int ad7795_ringbuf_read( ad7795_ringbuf *ringbuf, void *data )
{
	int readelements = 0;

	spin_lock( &ringbuf->buflock );

	if( ringbuf->rdPos == BUF_UNREAD ){
        if( ringbuf->wrPos >= 0 ){
			ringbuf->rdPos = 0;
		}
	}

	// single element read
	if( ringbuf->rdPos != ringbuf->wrPos ){
		readelements = 1;

		memcpy( data, 
		        &ringbuf->buf[ringbuf->rdPos * ringbuf->elementSize],
		        ringbuf->elementSize );

		ringbuf->rdPos++;
        
		if( ringbuf->rdPos >= ringbuf->elements ){
			ringbuf->rdPos = 0;
		}
	}

	spin_unlock( &ringbuf->buflock );

    return readelements;
}

//######################################################################
/** 
 * exchange a number of bytes to number of bits 
 * 
 * @param bytes : total byte number
 * 
 * @return total bit number
 */
static unsigned int ad7795_get_nbits(unsigned int bytes)
{
    return (bytes << 3);
}

//######################################################################
/** 
 * exchange a number of bits to number of bytes
 * 
 * @param bits : total bit number
 * 
 * @return total byte number
 */
static unsigned int ad7795_get_nbytes(unsigned int bits)
{
    if( bits == 0 ){
        return 0;
    }
    if( bits <= 8 ){
        return 1;
    }
    if( bits <= 16 ){
        return 2;
    }
    if( bits <= 24 ){
        return 3;
    }
    return 4;
}


#define DBG_SPI_MSG(a)
//######################################################################
/** 
 * spi transfer function 
 * 
 * @param tx_buf : tx buffer
 * @param n_tx   : a number of tx bits
 * @param n_rx   : a number of rx bits
 * 
 * @return rx data
 */
static u32 ad7795_spi_rw(u32 tx_buf, size_t n_tx, size_t n_rx)
{
    struct spi_regs *spi1_regs     = (struct spi_regs *)(SPI1_BASE_ADDR);
    unsigned int     mode_reg      = 0;
    unsigned int     pol_reg       = 0;
    unsigned int     tiecs_reg     = 0;
    unsigned int     nb            = 0;
    unsigned int     csw_bits      = 0;
    unsigned int     pol_bits      = 0;
    unsigned int     pol_lsb       = 0;
    unsigned int     raw_status    = 0;
    unsigned int     data          = 0;
    unsigned int     n_total_bytes = 0;
    unsigned int     n_tx_bytes    = 0;
    unsigned int     n_rx_bytes    = 0;
    u32              buffer        = 0x00000000;
    int              err           = 0;

    /* total transfer bits size */
    nb = n_tx + n_rx;

    /******* spi_config() *******/
    
    /* mode register setting */
    mode_reg |= ((nb - 1) << 8);       /* NB_A[4:0]    */
    mode_reg |= (SPI_CS_SEL_CS0 << 4); /* CS_SELA[2:0] */
    mode_reg |= (SPI_M_S_MASTER << 1); /* M_S          */
    mode_reg |= (SPI_DMA_OFF << 0);    /* DMA          */

    /* tiecs register setting */
    tiecs_reg |= SPI_TIECS_NORMAL << SPI_CS_SEL_CS0;

    /* pol register setting   */
    csw_bits = SPI_POL_SP1_CS0 & SPI_CSW_MASK;
    pol_bits = SPI_POL_SP1_CS0 & SPI_POL_MASK;
    pol_lsb  = SPI_CS_SEL_CS0 * 3;

    pol_reg = spi1_regs->pol;
    pol_reg |= SPI_POL_SP1_CS0;

    pol_reg &= ~( SPI_CSW_MASK | ( SPI_POL_MASK << pol_lsb ) );
    pol_reg |= ( csw_bits | ( pol_bits << pol_lsb ) );
    
    /* sclk setting */
    outl(SMU_DIV80, SMU_DIVSP1SCLK); /* sclk = 3[MHz] */
    /* perhaps, "mp200_spi" module is setting this... */
    
    /* register write */
    spi1_regs->mode  = mode_reg;
    spi1_regs->tiecs = tiecs_reg;
    spi1_regs->pol   = pol_reg;
    /* 
       outl(mode_reg, SPI1_MODE_REG_ADDR);
       outl(pol_reg, SPI1_POL_REG_ADDR);
       outl(tiecs_reg, SPI1_TIECS_REG_ADDR);
    */
    
    /* soft reset */
    spi1_regs->control  = (0x01 << 8);
    udelay( ( (1000 * 4) / 2867 ) + 1 );
    spi1_regs->control &= ~(0x01 << 8);
    /*
      outl((0x01 << 8), SPI1_CONTROL_REG_ADDR);
      udelay( ( (1000 * 4) / 2867 ) + 1 );
      outl((~(0x01 << 8)), SPI1_CONTROL_REG_ADDR);
    */
    /******* end of "spi_config()" *******/

    n_tx_bytes    = ad7795_get_nbytes((unsigned int)n_tx);
    n_rx_bytes    = ad7795_get_nbytes((unsigned int)n_rx);
    n_total_bytes = ad7795_get_nbytes((unsigned int)(n_tx + n_rx));
    
    memcpy((char *)&data, &tx_buf, n_tx_bytes);
    data = data << n_rx;
    //DPRINTK(KERN_INFO "%s : tx_data = 0x%08x\n",__FUNCTION__);
    
    /******* spi_xferbytes() *******/
    spi1_regs->tx_data = data;
    spi1_regs->control = (SPx_CONTROL_RD | SPx_CONTROL_WRT | SPx_CONTROL_START);

    while( (spi1_regs->control & (SPx_CONTROL_START | SPx_CONTROL_RX_EMP)) != 0 ){
        /* wait transfer done ... */
        raw_status = spi1_regs->raw_status;
        if( (raw_status & SPx_RAW_STATUS_ALLERR_RAW) != 0 ){
            /* error is occured ! */
            /* spi_sft_reset() */
            spi1_regs->control  = (0x01 << 8);
            udelay( ( (1000 * 4) / 2867 ) + 1 );
            spi1_regs->control &= ~(0x01 << 8);
            err = 1;
            break;
        }
    }

    if( err == 0 ){
        data = spi1_regs->rx_data;
    }
    //DPRINTK(KERN_INFO "%s : rx_data = 0x%08x\n",__FUNCTION__,data);
    
    memcpy(&buffer,  (char *)&data, n_total_bytes);

    /******* end of "spi_xferbytes()" *******/

    /******* spi_config_release() *******/
    /******* end of "spi_config_release()" *******/

    return buffer;
}
EXPORT_SYMBOL(ad7795_spi_rw);

#define DBG_TS_MSG(a)
//######################################################################
/** 
 * Timer call-back routine
 * 
 */
void adcTimerService( AdcDevSt *pdev )
{
    u32 data;
    int err = 0;
    struct timespec tu;
    struct timespec idle;
    cputime_t idletime = cputime_add(init_task.utime, init_task.stime);

    // Read Data
    switch( pdev->RdCh )
    {
    case 0:
        do_posix_clock_monotonic_gettime(&tu);
        monotonic_to_bootbased(&tu);
        cputime_to_timespec(idletime, &idle);

        pdev->AdData.tick = (tu.tv_sec * 1000) + (tu.tv_nsec / 1000000);
        
        data = COM_REG( COMREG_RD | COMRS_DATA );

        pdev->AdData.ch1  = ad7795_spi_rw(data, 8, 16);

        // Next:Y ch
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[1] + CONFREG_CH(ADC_CH_G_Y);

        ad7795_spi_rw(data, 24, 0);

        break;

    case 1:
        data = COM_REG( COMREG_RD | COMRS_DATA );

        pdev->AdData.ch2  = ad7795_spi_rw(data, 8, 16);

        // Next:Z ch
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[2] + CONFREG_CH(ADC_CH_G_Z);

        ad7795_spi_rw(data, 24, 0);

        break;

    case 2:
        data = COM_REG( COMREG_RD | COMRS_DATA );
        
        pdev->AdData.ch3  = ad7795_spi_rw(data, 8, 16);

        /* Yaw Gyro , bi-polar, Gain 8 */ 
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[4] + CONFREG_CH(ADC_CH_YAW_GYRO);

        ad7795_spi_rw(data, 24, 0);
        break;


    case 3:
        data = COM_REG( COMREG_RD | COMRS_DATA );

        pdev->AdData.ch5 = ad7795_spi_rw( data, 8, 16);;

        /* Pitch Gyro, bi-polar, Gain 8 */
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[5] + CONFREG_CH(ADC_CH_PITCH_GYRO);

        ad7795_spi_rw(data, 24, 0);

        break;

    case 4:
        data = COM_REG( COMREG_RD | COMRS_DATA );

        pdev->AdData.ch6  = ad7795_spi_rw(data, 8, 16);

    default:
        // Init state.(RdCh is -1)
        // Next:X ch
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[0] + CONFREG_CH(ADC_CH_G_X);

        ad7795_spi_rw(data, 24, 0);
        
        break;
    }

    // Next channel
    pdev->RdCh++;

    // save
    if( pdev->RdCh > 4 )
    {
        err = ad7795_ringbuf_write( pdev->DataBuf, &pdev->AdData );
        pdev->RdCh = 0;
    }

    return;
}

//######################################################################
/** 
 * timer service for walking mode
 * 
 * @param pdev 
 */
void adcTimerService_Walk( AdcDevSt *pdev )
{

    u32 data;
    int err = 0;
    struct timespec tu;
    struct timespec idle;
    cputime_t idletime = cputime_add(init_task.utime, init_task.stime);

    // Read Data
    switch( pdev->RdCh )
    {
    case 0:
        do_posix_clock_monotonic_gettime(&tu);
        monotonic_to_bootbased(&tu);
        cputime_to_timespec(idletime, &idle);

        pdev->AdData.tick = (tu.tv_sec * 1000) + (tu.tv_nsec / 1000000);
        
        data = COM_REG( COMREG_RD | COMRS_DATA );

        pdev->AdData.ch1  = ad7795_spi_rw(data, 8, 16);

        // Next:Y ch
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[1] + CONFREG_CH(ADC_CH_G_Y);

        ad7795_spi_rw(data, 24, 0);

        break;

    case 1:
        data = COM_REG( COMREG_RD | COMRS_DATA );

        pdev->AdData.ch2 = ad7795_spi_rw(data, 8, 16);

        // Next:Z ch
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[2] + CONFREG_CH(ADC_CH_G_Z);

        ad7795_spi_rw(data, 24, 0);
        
        break;

    case 2:
        data = COM_REG( COMREG_RD | COMRS_DATA );
        
        pdev->AdData.ch3  = ad7795_spi_rw(data, 8, 16);

    default:
        // Init state.(RdCh is -1)
        // Next:X ch
        data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
            + pdev->config.confreg[0] + CONFREG_CH(ADC_CH_G_X);

        ad7795_spi_rw(data, 24, 0);
        
        break;
    }

    // Next channel
    pdev->RdCh++;

    // save
    if( pdev->RdCh > 2 )
    {
        err = ad7795_ringbuf_write( pdev->DataBuf, &pdev->AdData );
        pdev->RdCh = 0;
    }

    return ;
}

//######################################################################
/** 
 * mode register setting function
 * 
 * @param pdev : device data structure
 * 
 * @return 0 : success, other : failure
 */
static int ad7795_set_modereg( AdcDevSt *pdev )
{
    int err;
    u32 data;

    data = ( COM_REG( COMREG_WR | COMRS_MODE ) << 16 )
        + pdev->config.modereg;
    
    err = ad7795_spi_rw(data, 24, 0);

    return 0;
}

//######################################################################
/** 
 * AD7795 Connection check.
 * 
 * 
 * @return 0 : success, other : failure
 */
static int adcConnectionCheck( AdcDevSt *pdev )
{
    u32         data;
    int         status = 0;     /* 0:OK -1:NG */

    udelay(500);

    /* reset */
    ad7795_spi_rw(0xffffffff, 32, 0);
    udelay(500);
    
    //ID Register Read.
    data = COM_REG( COMREG_RD | COMRS_ID );
    pdev->ChipID  = (u16)ad7795_spi_rw(data, 8, 8);
    printk(KERN_INFO "AD7795 ID=0x%02x\n", pdev->ChipID);

    if( (pdev->ChipID & 0x0f) != 0x0e )
    {
        printk( KERN_INFO "%s():ADC7795 Not found!(ID=0x%02x)\n", __FUNCTION__,  (pdev->ChipID & 0xff) );
        status  = -ENXIO;
    }

    return status;
}

//######################################################################
/** 
 * Timer Suspend
 * 
 * @param pdev 
 * 
 * @return 0 : success, other : failure
 */
static int adcTimerSuspend( AdcDevSt *pdev )
{
    int err = 0;
    
    if( (pdev->cb == adc_interrupt) || (pdev->cb == adc_interrupt2) ){
        printk(KERN_DEBUG "%s: called.\n",__FUNCTION__);
        err = mp200_timer_stop(pdev->TimerResource);
        if(err){
            printk(KERN_ERR "%s() : timer stop ... failed(%d).\n",__FUNCTION__,err);
            return -EBUSY;
        }

        err = mp200_timer_unregister_cb(pdev->TimerResource, (void *)pdev);
        if(err){
            printk(KERN_ERR "%s() : unregister callback function ... failed(%d).\n",
                   __FUNCTION__, err);
            return err;
        }
    }

    return err;

}

//######################################################################
/** 
*  Timer stop.
* 
* @param pdev 
* 
* @return 0 : success, other : failure
*/
static int adcTimerStop( AdcDevSt *pdev )
{
    int err = 0;
    
    if( (pdev->cb == adc_interrupt) || (pdev->cb == adc_interrupt2) ){

        err = mp200_timer_stop(pdev->TimerResource);
        if(err){
            printk(KERN_ERR "%s() : timer stop ... failed(%d).\n",__FUNCTION__,err);
            return -EBUSY;
        }

        err = mp200_timer_unregister_cb(pdev->TimerResource, (void *)pdev);
        if(err){
            printk(KERN_ERR "%s() : unregister callback function ... failed(%d).\n",
                   __FUNCTION__, err);
            return err;
        }

        pdev->config.type            = ADC_SERVICE_TYPE_UNKNOWN;
        pdev->config.sampling_period = 0;
        pdev->cb                     = NULL;
        pdev->used                   = 0;
    }

    return err;
}

//######################################################################
/** 
 * Timer Clean up function  
 * 
 * @param pdev 
 */
static void adcTimerClean(  AdcDevSt *pdev )
{
    int err = 0;

    err = adcTimerStop(pdev);

    return;
}

//######################################################################
/** 
 * Time initialize & timer start trigger
 * 
 * @param pdev 
 * 
 * @return 0 : success, other : failure
 */
static int adcTimerInit( AdcDevSt *pdev )
{
    int err = 0;
    unsigned int usecs = 0;

    //pdev->RdCh = -1;

    err = mp200_timer_register_cb(pdev->TimerResource, "xyz5-adc", pdev->cb, (void *)pdev);
    if(err){
        printk(KERN_ERR "%s() : register callback function ... failed(%d).\n",
               __FUNCTION__, err);
        return err;
    }
    
    usecs = pdev->config.sampling_period * 1000;
    
    /* set periodic interval */
    err = mp200_timer_set_period(pdev->TimerResource, usecs);
    if(err){
        printk(KERN_ERR "%s() : set timer period(%d) ... failed(%d).\n",
               __FUNCTION__, usecs, err);
        return err;
    }
    udelay(500);

    /* start a timer */
    err = mp200_timer_start(pdev->TimerResource);
    if(err){
        printk(KERN_ERR "%s() : timer start ... failed(%d).\n",
               __FUNCTION__, err);
        return err;
    }

    return err;
}

//######################################################################
/** 
 *  restart timer when resume state
 * 
 * @param pdev 
 * 
 * @return 0 : success, other : failure
 */
static int adcTimerReStart( AdcDevSt *pdev )
{
    int err = 0;

    if( (pdev->cb == adc_interrupt) || (pdev->cb == adc_interrupt2) ){

        err = ad7795_set_modereg(pdev);

        memset( &pdev->AdData, 0, sizeof(pdev->AdData) );

        err = ad7795_ringbuf_clear( pdev->DataBuf );

        err = adcTimerInit(pdev);
    }
    
    return err;
}


//######################################################################
/** 
 * Timer start.
 * 
 * @param pdev 
 * @param arg 
 * 
 * @return 0 : success, other : failure
 */
static int adcTimerStart(AdcDevSt *pdev, adc_service_config_t __user *arg)
{
    int err = 0;
    adc_service_config_t data;

    memset(&data, 0, sizeof(data));
    
    err = copy_from_user(&data, (adc_service_config_t __user *)arg, sizeof(data));
    if( err != 0 ){
        return err; /* = -EFAULT */
    }
    
    pdev->config.type            = data.type;

    switch(pdev->config.type){
    case ADC_SERVICE_TYPE_PPU:
        pdev->config.sampling_period = ADC_SERVICE_TYPE_PPU_SAMPLE_TIME;
        pdev->cb = adc_interrupt;
        break;
    case ADC_SERVICE_TYPE_WALK:
        pdev->config.sampling_period = ADC_SERVICE_TYPE_WALK_SAMPLE_TIME;
        pdev->cb = adc_interrupt2;
        break;
    default:
        printk(KERN_ERR "%s : no such device service(%d)\n",__FUNCTION__, pdev->config.type);
        return -EINVAL;
    }

    pdev->config.use_default = data.use_default;
    
    if(pdev->config.use_default){
        /* use a default setting */
        pdev->config.modereg    = MODE_REG( MODEMD_CONTINUE | MODEREG_PSW | MODEREG_AMP_CM | MODEREG_CHOP_DIS | MODEREG_FS(1) );
        pdev->config.confreg[0] = CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF );
        pdev->config.confreg[1] = CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF );
        pdev->config.confreg[2] = CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF );
        pdev->config.confreg[3] = CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF );
        pdev->config.confreg[4] = CONF_REG( CONG_REF_REF1 | CONFREG_BUF | CONFREG_GAIN(3) );
        pdev->config.confreg[5] = CONF_REG( CONG_REF_REF1 | CONFREG_BUF | CONFREG_GAIN(3) );
        
    }
    else{
        /* use a user setting */
        pdev->config.sampling_period = data.sampling_period;
        pdev->config.modereg         = data.modereg;
        pdev->config.confreg[0]      = data.confreg[0] & 0xFFF0;
        pdev->config.confreg[1]      = data.confreg[1] & 0xFFF0;
        pdev->config.confreg[2]      = data.confreg[2] & 0xFFF0;
        pdev->config.confreg[3]      = data.confreg[3] & 0xFFF0;
        pdev->config.confreg[4]      = data.confreg[4] & 0xFFF0;
        pdev->config.confreg[5]      = data.confreg[5] & 0xFFF0;
    }
    
    err = ad7795_set_modereg(pdev);
    
    DBG("service mode    : %d\n", pdev->config.type);
    DBG("sampling period : %d\n", pdev->config.sampling_period);
    DBG("mode reg        : 0x%04x\n", pdev->config.modereg);
    DBG("config reg[0]   : 0x%04x\n", pdev->config.confreg[0]);
    DBG("config reg[1]   : 0x%04x\n", pdev->config.confreg[1]);
    DBG("config reg[2]   : 0x%04x\n", pdev->config.confreg[2]);
    DBG("config reg[3]   : 0x%04x\n", pdev->config.confreg[3]);
    DBG("config reg[4]   : 0x%04x\n", pdev->config.confreg[4]);
    DBG("config reg[5]   : 0x%04x\n", pdev->config.confreg[5]);

    memset( &pdev->AdData, 0, sizeof(pdev->AdData) );
    err = ad7795_ringbuf_clear( pdev->DataBuf );

    err = adcTimerInit(pdev);
    
    return err;
}


//######################################################################
/** 
 * AD7795 Initialize.
 * 
 * 
 * @return 0 : success, other : failure
 */
static int adcInit( AdcDevSt *pdev )    
{
    return 0;
}

//######################################################################
/** 
 * ioctl method for read register
 * 
 * @param arg : register data transfer structure 
 * 
 * @return 0 : usccess, other : failure
 */
static int ad7795_reg_read(adc_regmsg __user *arg)
{
    int          err       = 0;
    size_t       n_tx      = 0;
    size_t       n_rx      = 0;
    u32          tx_buf    = 0;
    u32          rx_buf    = 0;
    adc_regmsg   msg;

    /* data from user */
    err = copy_from_user(&msg, (adc_regmsg __user *)arg, sizeof(msg));
    if( err != 0 ){
        return err; /* = -EFAULT */
    }
    n_tx   = (size_t)ad7795_get_nbits(msg.tx_nbytes);
    n_rx   = (size_t)ad7795_get_nbits(msg.rx_nbytes);
    tx_buf = (u32)msg.tx_buf;

    rx_buf = ad7795_spi_rw(tx_buf, n_tx, n_rx);

    msg.rx_buf = (unsigned int)rx_buf;

    err = copy_to_user(arg, &msg, sizeof(msg));
    if( err != 0 ){
        return err;
    }
    
    return err;
}

//######################################################################
/** 
 * ioctl method for writing register
 * 
 * @param arg 
 * 
 * @return 
 */
static int ad7795_reg_write(adc_regmsg __user *arg)
{
    int          err       = 0;
    size_t       n_tx      = 0;
    size_t       n_rx      = 0;
    u32          tx_buf    = 0;
    u32          rx_buf    = 0;
    adc_regmsg   msg;

    /* data from user */
    err = copy_from_user(&msg, (adc_regmsg __user *)arg, sizeof(msg));
    if( err != 0 ){
        return err; /* = -EFAULT */
    }
    n_tx   = (size_t)ad7795_get_nbits(msg.tx_nbytes);
    tx_buf = (u32)msg.tx_buf;

    rx_buf = ad7795_spi_rw(tx_buf, n_tx, n_rx);

    return err;
}


//######################################################################
/** 
 * ADC Power mode setting
 * 
 * @param mode : power mode
 * 
 * @return 0 : success, other : failure
 */
static int adcSetPowerMode( int mode )
{
    u32 data;

    switch( mode )
    {
    case ADC_POWER_DOWN:
    case ADC_IDLE:
        break;
    default:
        printk( KERN_ERR "%s():Invalid argument!(%d)\n", __FUNCTION__, mode );
        return -EINVAL;
    }
    data = ( COM_REG( COMREG_WR | COMRS_MODE ) << 16 )
        + MODE_REG( mode | MODEREG_PSW | MODEREG_AMP_CM | MODEREG_CHOP_DIS | MODEREG_FS(1) );

    ad7795_spi_rw(data, 24, 0);

    udelay(500);
    return 0;
}

//######################################################################
/** 
 * take shelter with a/d converter register
 * 
 * @param parg : argument
 * 
 * @return 0 : success, other : failure
 */
static int ad7795RestoreState( Ad7795RegArg *parg )
{
    u32 data;

    //reset
    ad7795_spi_rw(0xffffffff, 32, 0);

    udelay(500);

    // Idle mode
    adcSetPowerMode( ADC_IDLE );

    // 1 ch: Offset / Full Scale register restore
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_X) );
    ad7795_spi_rw(data, 24, 0);

    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->offset[0] & 0xffff);
    ad7795_spi_rw(data, 24, 0);
    
    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->fullscale[0] & 0xffff);
    ad7795_spi_rw(data, 24, 0);

    
    // 2 ch: Offset / Full Scale register restore
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_Y) );
    ad7795_spi_rw(data, 24, 0);
 
    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->offset[1] & 0xffff);
    ad7795_spi_rw(data, 24, 0);
 
    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->fullscale[1] & 0xffff);
    ad7795_spi_rw(data, 24, 0);


    // 3 ch: Offset / Full Scale register restore
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_Z) );
    ad7795_spi_rw( data, 24, 0 );

    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->offset[2] & 0xffff);
    ad7795_spi_rw( data, 24, 0);

    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->fullscale[2] & 0xffff);
    ad7795_spi_rw( data, 24, 0);


    // 4-6 ch: Offset / Full Scale register restore
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_YAW_GYRO) );
    ad7795_spi_rw(data, 24, 0);

 
    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->offset[3] & 0xffff);
    ad7795_spi_rw(data, 24, 0);
 
    data = ( COM_REG( COMREG_RD | COMRS_OFFSET ) << 16 )
        + (parg->fullscale[3] & 0xffff);
    ad7795_spi_rw(data, 24, 0);

    
    // Mode register restore
    data = ( COM_REG( COMREG_WR | COMRS_MODE ) << 16 ) + (parg->mode & 0xffff);
    ad7795_spi_rw(data, 24, 0);

    udelay(500);

    // Configuratiorn register restore
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 ) + (parg->config & 0xffff);
    ad7795_spi_rw(data, 24, 0);

    udelay(500);

    DBG(" status ......... 0x%x\n", parg->status);
    DBG(" mode ........... 0x%x\n", parg->mode);
    DBG(" config ......... 0x%x\n", parg->config);
    DBG(" id ............. 0x%x\n", parg->id);
    DBG(" io ............. 0x%x\n", parg->io);
    DBG(" id ............. 0x%x\n", parg->id);
    DBG(" offset[0] ...... 0x%x\n", parg->offset[0]);
    DBG(" offset[1] ...... 0x%x\n", parg->offset[1]);
    DBG(" offset[2] ...... 0x%x\n", parg->offset[2]);
    DBG(" offset[3] ...... 0x%x\n", parg->offset[3]);
    DBG(" fullscale[0] ... 0x%x\n", parg->fullscale[0]);
    DBG(" fullscale[1] ... 0x%x\n", parg->fullscale[1]);
    DBG(" fullscale[2] ... 0x%x\n", parg->fullscale[2]);
    DBG(" fullscale[3] ... 0x%x\n", parg->fullscale[3]);
    
    return 0;
}

#define DBG_REGREAD_MSG(a)
//######################################################################
/** 
 * ADC internal register read.
 * 
 * @param parg 
 * 
 * @return 0 : success, other : failure
 */
static int adcReadReg( Ad7795RegArg *parg )
{
    u32 data;

    // Status Reg
    data = COM_REG( COMREG_RD | COMRS_STATUS );
    parg->status = ad7795_spi_rw(data, 8, 8);


    // Mode Reg
    data = COM_REG( COMREG_RD | COMRS_MODE );
    parg->mode = ad7795_spi_rw(data, 8, 16);

        
    // Configration Reg
    data = COM_REG( COMREG_RD | COMRS_CONFIG );
    parg->config = ad7795_spi_rw(data, 8, 16);

    
    // ID Reg
    data = COM_REG( COMREG_RD | COMRS_ID );
    parg->id = ad7795_spi_rw(data, 8, 8);


    // IO Reg
    data = COM_REG( COMREG_RD | COMRS_IO );
    parg->io = ad7795_spi_rw(data, 8, 8);


    // 1 ch: Offset / Full Scale
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_X) );
    ad7795_spi_rw(data, 24, 0);

    data = COM_REG( COMREG_RD | COMRS_OFFSET );
    parg->offset[0] = ad7795_spi_rw(data, 8, 16);
        
    data = COM_REG( COMREG_RD | COMRS_F_SCALE );
    parg->fullscale[0] = ad7795_spi_rw(data, 8, 16);


    // 2 ch: Offset / Full Scale
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_Y) );
    ad7795_spi_rw(data, 24, 0);

    data = COM_REG( COMREG_RD | COMRS_OFFSET );
    parg->offset[1] = ad7795_spi_rw(data, 8, 16);

    data = COM_REG( COMREG_RD | COMRS_F_SCALE );
    parg->fullscale[1] = ad7795_spi_rw(data, 8, 16);


    // 3 ch: Offset / Full Scale
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_Z) );
    ad7795_spi_rw(data, 24, 0);

    data = COM_REG( COMREG_RD | COMRS_OFFSET );
    parg->offset[2] = ad7795_spi_rw(data, 8, 16);
    
    data = COM_REG( COMREG_RD | COMRS_F_SCALE );
    parg->fullscale[2] = ad7795_spi_rw(data, 8, 16);


    // 4-6 ch: Offset / Full Scale
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_YAW_GYRO) );
    ad7795_spi_rw(data, 24, 0);

    data = COM_REG( COMREG_RD | COMRS_OFFSET );
    parg->offset[3] = ad7795_spi_rw(data, 8, 16);
    
    data = COM_REG( COMREG_RD | COMRS_F_SCALE );
    parg->fullscale[3] = ad7795_spi_rw(data, 8, 16);
    
    // Config Reg is returned.
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( parg->config );
    ad7795_spi_rw(data, 24, 0);

    DBG(" status ......... 0x%x\n", parg->status);
    DBG(" mode ........... 0x%x\n", parg->mode);
    DBG(" config ......... 0x%x\n", parg->config);
    DBG(" id ............. 0x%x\n", parg->id);
    DBG(" io ............. 0x%x\n", parg->io);
    DBG(" id ............. 0x%x\n", parg->id);
    DBG(" offset[0] ...... 0x%x\n", parg->offset[0]);
    DBG(" offset[1] ...... 0x%x\n", parg->offset[1]);
    DBG(" offset[2] ...... 0x%x\n", parg->offset[2]);
    DBG(" offset[3] ...... 0x%x\n", parg->offset[3]);
    DBG(" fullscale[0] ... 0x%x\n", parg->fullscale[0]);
    DBG(" fullscale[1] ... 0x%x\n", parg->fullscale[1]);
    DBG(" fullscale[2] ... 0x%x\n", parg->fullscale[2]);
    DBG(" fullscale[3] ... 0x%x\n", parg->fullscale[3]);

    return 0;
}

//######################################################################
/** 
 * for debug
 * 
 * 
 * @return 0 : success, other : failure
 */
static int adcDebug(void)
{
    u32 data, read_data;
    
    // Next:X ch
    data = ( COM_REG( COMREG_WR | COMRS_CONFIG ) << 16 )
        + CONF_REG( CONFREG_UNI | CONG_REF_REF1 | CONFREG_BUF | CONFREG_CH(ADC_CH_G_X) );
    ad7795_spi_rw(data, 24, 0);

    msleep(4);
    
    data = COM_REG( COMREG_RD | COMRS_DATA );

    read_data = ad7795_spi_rw(data, 8, 16);
    
    DBG("tx : 0x%x, rx : 0x%x\n",data,read_data);

    return 0;
}

//######################################################################
/** 
 * Get AD7795 configuration
 * 
 * @param pdev : device data structure
 * @param parg : user space data buffer
 * 
 * @return 0 : success, other : failure
 */
static int adcGetConfig( AdcDevSt *pdev, Ad7795RegArg __user *parg )
{
    int err = 0;
    Ad7795RegArg data;

    memset(&data, 0, sizeof(Ad7795RegArg));
    
    err = adcReadReg(&data);

    err = copy_to_user(parg, &data, sizeof(Ad7795RegArg));
    
    return err;
}

//######################################################################
/** 
 * Get Sensor data.
 * 
 * @param pdev 
 * @param parg 
 * 
 * @return 0 : success, other : failure
 */
static int adcGetData( AdcDevSt *pdev, AdReadArg __user *parg )
{
    int i;
    int readcnt;
    int maxcnt;
    PosSensor *rdbuf;
    unsigned long ret;

    ret = get_user(maxcnt, &(parg->maxReadCount));
    if( ret != 0 ){
        return ret;
    }
    if( maxcnt <= 0 )
    {
        return -EINVAL;
    }

    if( maxcnt > POS_BUF_NUM )
    {
        maxcnt = POS_BUF_NUM;
    }

    rdbuf = kmalloc( sizeof(PosSensor) * maxcnt, GFP_KERNEL );
    if( rdbuf == NULL )
    {
        return -ENOMEM;
    }
    memset( rdbuf, 0, sizeof(PosSensor) * maxcnt );

    readcnt = 0;
    for( i=0; i<maxcnt; i++ )
    {
        if( ad7795_ringbuf_read( pdev->DataBuf, &rdbuf[i] ) == 0 )
        {
            break;
        }
        readcnt++;
    }

    ret = copy_to_user( parg->buf, rdbuf, sizeof(PosSensor) * readcnt );
    kfree( rdbuf );
    if( ret != 0 )
    {
        return ret;
    }

    return readcnt;
}

//######################################################################
/** 
 * ioctl function for acquiring a current tick value
 * 
 * @param arg : tick 
 * 
 * @return 0 : success, other : failure
 */
static int ad7795_get_current_tick(unsigned long __user * arg)
{
    int             err      = 0;
    unsigned long   tick     = 0;
    struct timespec uptime;
    struct timespec idle;
    cputime_t       idletime = cputime_add(init_task.utime, init_task.stime);

    do_posix_clock_monotonic_gettime(&uptime);
	monotonic_to_bootbased(&uptime);
	cputime_to_timespec(idletime, &idle);

    tick = (uptime.tv_sec * 1000) + (uptime.tv_nsec / 1000000);

    err = copy_to_user(arg, &tick, sizeof(unsigned long));
    
    return err;
}

//######################################################################
/** 
 * ioctl function for reset AD7795 device
 * 
 * 
 * @return 0 
 */
static int ad7795_reset_device(AdcDevSt *pdev)    
{
    int err = 0;
    u32 buf = 0;
    
    if( (pdev->cb == adc_interrupt) || (pdev->cb == adc_interrupt2) ){
        DBG("Sampling service is working.\n");
        return -EBUSY;
    }

    buf = ad7795_spi_rw(0xffffffff, 32, 0);
    udelay(500);

    return err;
}

//######################################################################
/** 
 * the ioctl method
 * 
 * @param inode : inode pointer
 * @param filp  : file pointer
 * @param cmd   : ioctl commands
 * @param arg   : ioctl argment
 * 
 * @return 0 : success, other : failure
 */
static int adc_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
    int stat = 0;
    AdcDevSt    *pdev = (AdcDevSt *)filp->private_data;

    if( AdApiFlag == 0 )
    {
        return -EPERM;
    }
    switch( cmd )
    {
    case XYZ5ADC_GET_CONFIG:
        stat = adcGetConfig( pdev, (Ad7795RegArg __user *)arg );
        break;

    case XYZ5ADC_READ:
        stat = adcGetData( pdev, (AdReadArg __user *)arg );
        break;

    case XYZ5ADC_STOP:
        stat = adcTimerStop(pdev);
        break;
        
    case XYZ5ADC_START:
        stat = adcTimerStart(pdev, (adc_service_config_t __user *)arg);
        break;
    case XYZ5ADC_DEBUG :
        // for debug
        stat = adcDebug();
        break;
    case XYZ5ADC_REG_READ:
        stat = ad7795_reg_read((adc_regmsg __user *)arg);
        break;
    case XYZ5ADC_REG_WRITE:
        stat = ad7795_reg_write((adc_regmsg __user *)arg);
        break;
    case XYZ5ADC_GET_CURRENT_TICK:
        stat = ad7795_get_current_tick((unsigned long __user *)arg);
        break;
    case XYZ5ADC_RESET_DEVICE:
        stat = ad7795_reset_device(pdev);
        break;
    default:
        stat = -EINVAL;
        break;
    }
    return stat;
}

//######################################################################
/** 
 * the open method
 * 
 * @param inode : inode pointer
 * @param filp  : file pointer
 * 
 * @return 0 : success, other : failure
 */
static int adc_open(struct inode *inode, struct file *filp)
{
    AdcDevSt    *pdev;

    pdev = container_of(inode->i_cdev, AdcDevSt, cdev);
    filp->private_data = pdev;

    if( pdev->used )
    {
        printk( KERN_ERR "%s:AD7795 using.\n", __FUNCTION__ );
        return -EBUSY;
    }

    if( down_interruptible( &pdev->sem ) )
    {
        printk( KERN_ERR "%s:down_interruptible failed!!\n", __FUNCTION__ );
        return -ERESTARTSYS;
    }

    pdev->used = 1;
    up( &pdev->sem );

    return 0;
}

//######################################################################
/** 
 * ther release method
 * 
 * @param inode : inode pointer
 * @param filp  : file pointer
 * 
 * @return 0 : success, other : failure
 */
static int adc_release( struct inode *inode, struct file *filp )
{
    AdcDevSt    *pdev;

    pdev = container_of(inode->i_cdev, AdcDevSt, cdev);

    if( down_interruptible( &pdev->sem ) )
    {
        printk( KERN_ERR "%s:down_interruptible failed!!\n", __FUNCTION__ );
        return -ERESTARTSYS;
    }

    pdev->used = 0;
    up( &pdev->sem );

    return 0;
}


static struct file_operations adc_fops = {
    owner:   THIS_MODULE,
    ioctl:   adc_ioctl,
    open:    adc_open,
    release: adc_release,
};


//######################################################################
/** 
 * callback handler function for NEC EM1 timer framework 
 * 
 * @param irq    : irq number
 * @param dev_id : device id
 * 
 * @return : IRQ_HANDLED
 */
static irqreturn_t adc_interrupt(int irq, void *dev_id)
{
    AdcDevSt *pdev = (AdcDevSt *)dev_id;

    adcTimerService(pdev);
    
    return IRQ_HANDLED;
}

//######################################################################
/** 
 * callback handler function for NEC EM1 timer framework(walking mode) 
 * 
 * @param irq    : irq number
 * @param dev_id : device id
 * 
 * @return : IRQ_HANDLED
 */
static irqreturn_t adc_interrupt2(int irq, void *dev_id)
{
    AdcDevSt *pdev = (AdcDevSt *)dev_id;

    adcTimerService_Walk(pdev);
    
    return IRQ_HANDLED;
}

//######################################################################
/** 
 *  EM1 spi (bus:1) block exit function 
 * 
 */
static void adc_spi_exit(void)
{
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_SP1SCLK);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_SP1PCLK);
    mp200_pmu_reset_device(MP200_RESETDEVICE_SP1);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_SP1_SCLK);
    mp200_pmu_close_clockgate(MP200_CLOCKGATE_SP1_PCLK);

    return;
}

//######################################################################
/** 
 * EM1 spi(bus:1) block init function 
 * 
 */
static void adc_spi_init(void)
{
    struct spi_regs *spi1_regs =  (struct spi_regs *)(SPI1_BASE_ADDR);

    mp200_pmu_clkctrl_off(MP200_CLKCTRL_SP1SCLK);
    mp200_pmu_clkctrl_off(MP200_CLKCTRL_SP1PCLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_SP1_PCLK);
    mp200_pmu_open_clockgate(MP200_CLOCKGATE_SP1_SCLK);
    mp200_pmu_unreset_device(MP200_RESETDEVICE_SP1);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_SP1PCLK);
    mp200_pmu_clkctrl_on(MP200_CLKCTRL_SP1SCLK);

    /* POL setting */
    spi1_regs->pol = SPI_POL_SP1_CS0;
    /* software reset */
    spi1_regs->control |= SPx_CONTROL_RST;
    udelay(5);
    spi1_regs->control &= ~SPx_CONTROL_RST;

    return;
}

//######################################################################
/*
 *  Power manager Support Routines
 */
int __devinit xyz5_adc_probe(struct platform_device *spi)    
{
    return 0;
}

//######################################################################
/*
 *
 */
static int xyz5_adc_remove(struct platform_device *dev)
{
    return 0;
}

//######################################################################
/*
 *
 */
static int xyz5_adc_suspend(struct platform_device *dev, pm_message_t state)    
{
    u32 data;

    DBG("%s() called.\n", __FUNCTION__);

    if( AdApiFlag )
    {
        adcTimerSuspend(&Adc_devst);

        /* save all register value in the meanwhile... */
        adcReadReg( &Adc_devst.AdSaveReg );
        
        /* set a power-down mode to MODE register */
        data = ( COM_REG( COMREG_WR | COMRS_MODE ) << 16 )
            + MODE_REG( MODEMD_POWERDOWN | MODEREG_AMP_CM | MODEREG_CHOP_DIS | MODEREG_FS(1) );
        ad7795_spi_rw(data, 24, 0);
    }

#ifdef CONFIG_PGPF_NAVU2_PM_SUSPEND
    pwc_write(PWC_PSCNT2, PWC_PSCNTGP2_OFF, PWC_PSCNTGP2_MASK);
#endif /* CONFIG_PGPF_NAVU2_PM_SUSPEND */

    return 0;
}

//######################################################################
/*
 *
 */
static int xyz5_adc_resume(struct platform_device *dev)    
{
    int rc = 0;
    u32 data;

    DBG("%s() called.\n", __FUNCTION__ );

#ifdef CONFIG_PGPF_NAVU2_PM_SUSPEND
    pwc_write(PWC_PSCNT2, PWC_PSCNTGP2_ON, PWC_PSCNTGP2_MASK);
#endif /* CONFIG_PGPF_NAVU2_PM_SUSPEND */

    if( AdApiFlag )
    {
        ad7795RestoreState( &Adc_devst.AdSaveReg );

        /* set a Continuous Mode & PSW */
        data = ( COM_REG( COMREG_WR | COMRS_MODE ) << 16 )
            + Adc_devst.config.modereg;

        ad7795_spi_rw(data, 24, 0);
        /* wait 30 ms for PSW sink */
        mdelay(30);

        rc = adcTimerReStart(&Adc_devst);
    }
    return rc;
}

static struct platform_driver xyz5adc_driver = {
    .probe      = xyz5_adc_probe,
    .remove     = __devexit_p(xyz5_adc_remove),
    .suspend    = xyz5_adc_suspend,
    .resume     = xyz5_adc_resume,
    .driver     = {
        .name   = "xyz5-adc",
        .owner  = THIS_MODULE,
    },
};

//######################################################################
/*
 * Driver cleanup routine.
 */
static void __exit xyz5_adc_cleanup(void)
{
    dev_t devno = MKDEV(DevMajorNumber, DevMinorNumber);

    adcTimerClean(&Adc_devst);
    
    ad7795_ringbuf_free( Adc_devst.DataBuf );

    /* for PND  EM1 platform */
    platform_driver_unregister(&xyz5adc_driver);

    cdev_del(&Adc_devst.cdev);

    unregister_chrdev_region(devno, DevCount);

    adc_spi_exit();

    return;
}

//######################################################################
/*
 * Driver initalize routine.
 */
static int __init xyz5_adc_init( void )
{
    int result;
    int err;
    int devno;
    dev_t dev = 0;
    
    printk(KERN_INFO "Starting ADC(PPU)\n");

    adc_spi_init();

    // Append this device driver to system.
    if (DevMajorNumber)
    {
        dev = MKDEV(DevMajorNumber, DevMinorNumber);
        result = register_chrdev_region(dev, DevCount, DevName);
    }
    else
    {
        result = alloc_chrdev_region(&dev, DevMinorNumber, DevCount, DevName);
        DevMajorNumber = MAJOR(dev);
    }

    if (result < 0)
    {
        printk( KERN_ERR "%s:can't get major %d\n", __FUNCTION__, DevMinorNumber);
        return result;
    }

    memset( &Adc_devst, 0, sizeof(AdcDevSt) );
    memset( &Adc_devst.AdData, 0, sizeof(Adc_devst.AdData) );
    memset( &Adc_devst.AdSaveReg, 0, sizeof(Adc_devst.AdSaveReg) );

    init_MUTEX(&Adc_devst.sem);

    devno = MKDEV(DevMajorNumber, DevMinorNumber);

    cdev_init(&Adc_devst.cdev, &adc_fops);
    Adc_devst.cdev.owner = THIS_MODULE;
    Adc_devst.cdev.ops = &adc_fops;

    err = cdev_add (&Adc_devst.cdev, devno, 1);
    /* Fail gracefully if need be */
    if (err)
    {
        printk(KERN_NOTICE "%s:Error %d adding adc0\n", __FUNCTION__, err);
    }

    result = platform_driver_register(&xyz5adc_driver);
    if( result != 0 )
    {
        printk(KERN_ERR "%s:driver_register() failed. (%d)\n", __FUNCTION__, result);
    }

    // ADC check
    if( adcConnectionCheck(&Adc_devst) != 0 )
    {
        printk(KERN_ERR "%s:adcConnectionCheck() failed!\n", __FUNCTION__);
        AdApiFlag = 0;  // FALSE
        return result;
    }
    AdApiFlag = 1;      //TRUE
    
    // data buffer.
    Adc_devst.DataBuf = ad7795_ringbuf_init( POS_BUF_NUM, sizeof(PosSensor) );

    if( Adc_devst.DataBuf < 0 )
    {
        printk(KERN_ERR "%s() : failed to initialize ring-buffer.\n", __FUNCTION__);
        goto error;
    }

    /* other device data structure init */
    Adc_devst.TimerResource          = TIMER_TG1;
    Adc_devst.config.type            = ADC_SERVICE_TYPE_UNKNOWN;
    Adc_devst.config.sampling_period = 0;
    Adc_devst.used                   = 0;
    Adc_devst.cb                     = NULL;

    // Read channel Init state.
    Adc_devst.RdCh = -1;

    adcInit(&Adc_devst);

    return result;

  error:
    xyz5_adc_cleanup();
    return result;
}

module_init(xyz5_adc_init);
module_exit(xyz5_adc_cleanup);

MODULE_DESCRIPTION("A/DC driver");
MODULE_AUTHOR("Sony Corporation");
MODULE_LICENSE("GPL");
