/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __NAVU2_PM_CHARGERIF_H__
#define __NAVU2_PM_CHARGERIF_H__

#include <asm/arch/pgpf/navu2/pm.h>
#include <asm/arch/pmic_external.h>
#include <asm/arch/pmic_adc.h>

#define SAMPLE_NUMBER      8
#define MILI_SCALE         1000   

#define ADC_RESOLUTION     1024   /* 10bit */
#define ADC_VMAX_MILI      2300   /* ADC Max sensed voltage */
#define ADC_VMIN_MILI      0      /* ADC Min sensed voltage */
#define ADC_MSB            0x200
#define ADC_MAX_ABS        0x1ff
#define ADC_STEP           ADC_VMAX_MILI / ADC_RESOLUTION  

#define BATT_CUR_SCALE     20     /* Scaling of Battery Current */
#define BATT_CUR_MILIOHMIC 20     /* Resistence value in mili ohm */

#define BATT_VOL_OFFSET    2400   /* Battery voltage offset in mV */

#define LED_DUTY_MAX       31
#define LED_DUTY_MIN       0

#define BIT_GPO1EN         6
#define BIT_GPO1STBY       7
#define BIT_GPO2EN         8
#define BIT_GPO2STBY       9

typedef enum {
	ATLAS_ICHRG_0 = 0,
	ATLAS_ICHRG_70,
	ATLAS_ICHRG_177,
	ATLAS_ICHRG_266,
	ATLAS_ICHRG_355,
	ATLAS_ICHRG_443,
	ATLAS_ICHRG_532,
	ATLAS_ICHRG_621,
	ATLAS_ICHRG_709,
	ATLAS_ICHRG_NUM_MAX,
} atlas_ichrg_t;

typedef struct {
	char * desc;
	type_event event;
	void (*callback)(void *);
	void *data;
} chargerif_irq_callback_t;

navu2pm_status_t chargerif_init(void);
navu2pm_status_t chargerif_get_battery_current(int *bc, unsigned short *bc_hex);
navu2pm_status_t chargerif_get_battery_voltage(int *bv, unsigned short *bv_hex);
navu2pm_status_t chargerif_get_battery_temperature(int *bt, unsigned short *bt_hex);
navu2pm_status_t chargerif_get_supply_voltage(int *sv, unsigned short *sv_hex);
navu2pm_status_t chargerif_get_ichrg(unsigned char *val);
navu2pm_status_t chargerif_set_ichrg(atlas_ichrg_t val);
navu2pm_status_t chargerif_set_charger_led_red(int on);
navu2pm_status_t chargerif_set_charger_led_green(int on);
navu2pm_status_t chargerif_register_callback(chargerif_irq_callback_t *cb);
navu2pm_status_t chargerif_get_acpin(int *connect);
atlas_ichrg_t    chargerif_get_usb_ilimit_setting(int ilimit);

#endif /* __NAVU2_PM_CHARGERIF_H__ */
