/* 2009-08-24: File added and changed by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __NAVU2_PM_CONTROLLER_H__
#define __NAVU2_PM_CONTROLLER_H__

#include <asm/arch/pgpf/navu2/pm.h>
#include <asm/arch/pgpf/navu2/suspend.h>
#include "chargerif.h"


#define MAIN_THREAD_SLEEP_TIME      3000 /* msec */
#define SENSOR_THREAD_SLEEP_TIME    3000 /* msec */

#define PWC_PSW_STATE_SDETUSB       0x20
#define PWC_PSW_OUT_GPO00           0x01
#define PWC_IOIN2_GPO08             0x01
#define PWC_XACC_IN                 PWC_IOIN2_GPO08             

/* Before RS */
#define THERMO_CHARGE_MAX_H_0       0x3AB /*  0 degree C */
#define THERMO_CHARGE_MAX_L_0       0x38C /*  2 degree C */
#define THERMO_CHARGE_MIN_H_0       0x127 /* 44 degree C */ 
#define THERMO_CHARGE_MIN_L_0       0x11D /* 45 degree C */
#define THERMO_DISCHARGE_MIN_H_0    0xE9  /* 51 degree C */

/* After RS */
#define THERMO_CHARGE_MAX_H_1       0x380 /*  0 degree C */
#define THERMO_CHARGE_MAX_L_1       0x370 /*  2 degree C */
#define THERMO_CHARGE_MIN_H_1       0x180 /* 43 degree C */ 
#define THERMO_CHARGE_MIN_L_1       0x170 /* 45 degree C */
#define THERMO_DISCHARGE_MIN_H_1    0x132 /* 51 degree C */

#define BATT_V_CHARGE_RESTART       0x29B /* 3900 mV */
#define BATT_V_DISCHARGE_L          0x1FB /* 3540 mV */
#define BATT_V_LOW                  0x1BD /* 3400 mV */ 

#define BATT_C_FULL                 0x3f2 /* -78  mA */

#define TIME_STOP_CHARGE            36000 /* sec (10 hours) */
#define COUNT_STOP_CHARGE           TIME_STOP_CHARGE / SENSOR_THREAD_SLEEP_TIME * 1000

#define SAMPLE_COUNT                5 

typedef enum {
	PLUGIN_NONE = 0,
	PLUGIN_DC_ONLY,
	PLUGIN_USB_ONLY,
	PLUGIN_DC_USB,
	PLUGIN_USB_DET_ONLY,
	PLUGIN_INVALID,
	PLUGIN_NUM_MAX
} plugin_status_t;

typedef struct {
	int dc;
	int cradle;
	int usb;
} connect_status_t;

typedef struct {
	int flag;
	atlas_ichrg_t ichrg;
} usb_status_t;

typedef struct {
	int bv;
	int bc;
	int bt;
	int sv;
	unsigned short bv_hex;
	unsigned short bc_hex;
	unsigned short bt_hex;
	unsigned short sv_hex;

	unsigned short history_bc[SAMPLE_COUNT];
	unsigned short history_bv[SAMPLE_COUNT];

	connect_status_t         connect;
	usb_status_t             usb;
	plugin_status_t          plugin;
	navu2pm_mode_t           mode;
	navu2pm_charge_status_t  charge_status;
	navu2pm_battery_status_t battery_status;
	navu2_factor_t           factor;
	
	int det_pkpress;
	int det_hightemp;
	int det_suspend;
	int det_rtc;
	int det_usb;
	int det_dc;

	int dsm;
	int supply_low;
	int acc_wake;
	int judge_pk_press;
	int usb_suspend;

	unsigned int count_charge_time;
} controller_status_t;

navu2pm_status_t         controller_init(void);
navu2pm_status_t         controller_resume(void);
navu2pm_status_t         controller_get_controller_status(controller_status_t *cs);
navu2pm_charge_status_t  controller_get_charge_status(void);
unsigned short           controller_get_battery_voltage(void);
unsigned short           controller_get_battery_current(void);
unsigned short           controller_get_battery_temperature(void);
unsigned short           controller_get_supply_voltage(void);
navu2pm_mode_t           controller_get_mode(void);
navu2pm_battery_status_t controller_get_battery_status(void);
int                      controller_get_dc_connect(void);
int                      controller_get_usb_connect(void);
int                      controller_get_supply_low(void);
void                     controller_request_suspend(void);
void                     controller_set_acc_wake(int);
int                      controller_get_acc_wake(void);
int                      controller_judge_dc_in(void);
int                      controller_judge_dc_out(void);
int                      controller_judge_usb_in(void);
int                      controller_judge_usb_out(void);
int                      controller_judge_supply_low(void);
int                      controller_judge_pk_press(void);
navu2_factor_t           controller_get_resume_factor(void);

#ifdef CONFIG_PGPF_NAVU2_PM_APPTHERM
void start_fake_therm(unsigned short val);
void stop_fake_therm(void);
#endif /* CONFIG_PGPF_NAVU2_PM_APPTHERM */

#endif  /* __NAVU2_PM_CONTROLLER_H__ */
