/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/list.h>

#include <asm/semaphore.h>
#include "event.h"
#include "internal.h"

static LIST_HEAD(_eventset_list);
static DECLARE_MUTEX(_eventset_mutex);

navu2pm_status_t event_add_eventset_list(navu2pm_eventset_t *eventset)
{
	eventset_list_t *eventset_list;

	if(NULL == eventset){
		DPRINT("## Error Invalid address\n");
		return NAVU2PM_ERR_ADDR;
	}

	eventset_list = kmalloc(sizeof(eventset_list_t), GFP_KERNEL);
	if(NULL == eventset_list){
		return NAVU2PM_ERR_NOMEM;
	}

	eventset_list->eventset = eventset;
	list_add_tail(&eventset_list->list, &_eventset_list);

	return NAVU2PM_SUCCESS;	
}

void event_del_eventset_list(navu2pm_eventset_t *eventset)
{
	struct list_head *pos;
	eventset_list_t  *eventset_list;

	if(NULL == eventset){
		DPRINT("## Error Invalid address\n");
		return ;
	}

	list_for_each(pos, &_eventset_list){
		eventset_list = list_entry(pos, eventset_list_t, list);
		if(eventset_list->eventset == eventset){
			list_del(&eventset_list->list);
			kfree(eventset_list);
			break;
		}
	}

	return ;
}

void eventset_append(navu2pm_event_t event)
{
	struct list_head   *pos;
	unsigned long      now;
	navu2pm_eventset_t *eventset;
	eventset_list_t    *eventset_list;

	down(&_eventset_mutex);
	now = jiffies;

	list_for_each(pos, &_eventset_list){
		eventset_list = list_entry(pos, eventset_list_t, list);
		eventset = eventset_list->eventset;
		eventset->event[eventset->index]     = event;
		eventset->timestamp[eventset->index] = now;
		eventset->num_event++;
		eventset->index++;

		if(eventset->num_event > NAVU2PM_MAX_EVENTS){
			eventset->num_event = NAVU2PM_MAX_EVENTS;
		}

		if(eventset->index >= NAVU2PM_MAX_EVENTS){
			eventset->index = 0;
		}
	}

	up(&_eventset_mutex);

	return ;
}

void eventset_clear(void)
{
	struct list_head   *pos;
	eventset_list_t    *eventset_list;
	navu2pm_eventset_t *eventset;

	down(&_eventset_mutex);
	list_for_each(pos, &_eventset_list){
		eventset_list = list_entry(pos, eventset_list_t, list);
		eventset = eventset_list->eventset;
		eventset->num_event = 0;
		eventset->index     = 0;
	}

	up(&_eventset_mutex);

	return ;
}
