/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/platform_device.h>

#include <asm/arch/pgpf/navu2/pm.h>
#include "internal.h"
#include "chrdv.h"
#include "controller.h"

#ifdef CONFIG_PGPF_NAVU2_PM_PROC_FS
#include "navu2pm_proc.h"
#endif /* CONFIG_PGPF_NAVU2_PM_PROC_FS */

static struct platform_device *navu2pm_device;

static int __devinit navu2pm_probe(struct platform_device *dev)
{
	navu2pm_status_t ret;

	ret = chrdv_init();
	if(NAVU2PM_SUCCESS != ret){
		/* error handling */
	}

	ret = controller_init();
	if(NAVU2PM_SUCCESS != ret){
		/* error handling */
	}

	return 0;
}

static int navu2pm_remove(struct platform_device *dev)
{
	return 0;
}

static int navu2pm_suspend(struct platform_device *dev, pm_message_t state)
{
	return 0;
}

static int navu2pm_resume(struct platform_device *dev)
{
	navu2pm_status_t ret;

	ret = controller_resume();
	if(NAVU2PM_SUCCESS != ret){
		DPRINT("## Error controller resume\n");
		return -1;
	}
	
	return 0;
}

static struct platform_driver navu2pm_driver = {
	.driver    = {
		.name  = "navu2pm",
		.owner = THIS_MODULE,
	},
	.probe     = navu2pm_probe,
	.remove    = navu2pm_remove,
    .suspend   = navu2pm_suspend,
    .resume    = navu2pm_resume,
};

static int __init navu2pm_init_module(void)
{
	int ret = 0;

	ret = platform_driver_register(&navu2pm_driver);
	if(0 != ret){
		DPRINT( "## platform_driver_register\n"); 
		goto error;
	}

	navu2pm_device = platform_device_register_simple("navu2pm", -1, NULL, 0);
	if(NULL == navu2pm_device){
		DPRINT("## platform_device_register\n"); 
		ret = -1;
		goto error;
	}
	
#ifdef CONFIG_PGPF_NAVU2_PM_PROC_FS
	navu2pm_proc_register();
#endif /* CONFIG_PGPF_NAVU2_PM_PROC_FS */

	return ret;

  error:
	platform_driver_unregister(&navu2pm_driver);
	platform_device_unregister(navu2pm_device);

	return ret;
}

static void __exit navu2pm_cleanup_module(void)
{
	platform_driver_unregister(&navu2pm_driver);
	platform_device_unregister(navu2pm_device);
}

late_initcall(navu2pm_init_module);
module_exit(navu2pm_cleanup_module);

MODULE_LICENSE("GPL");
