/* 2009-08-24: File added by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>

#include <asm/arch/gpio.h>
#include <asm/arch/pgpf/navu2/pm.h>
#include <asm/arch/hardware.h>
#include <asm/irq.h>
#include <asm/arch/pgpf/board.h>

#include "internal.h"
#include "chargerif.h"
#include "navu2pm_irq.h"
#include "controller.h"

static irqreturn_t pwrkey_irq_handler(int irq, void *dev_id);
static irqreturn_t detusb_irq_handler(int irq, void *dev_id);
static irqreturn_t hightemp_irq_handler(int irq, void *dev_id);
static irqreturn_t detrtc_irq_handler(int irq, void *dev_id);
static irqreturn_t detxacc_irq_handler(int irq, void *dev_id);

static void detusb_bh_handler(struct work_struct *work);
static void pwrkey_bh_handler(struct work_struct *work);
static void hightemp_bh_handler(struct work_struct *work);
static void detrtc_bh_handler(struct work_struct *work);
static void detxacc_bh_handler(struct work_struct *work);
static void detdc_bh_handler(void *arg);     
static void det_dc_usb_out_bh_handler(void *arg);     

inline static void do_callback(navu2pm_irq_cb_t irq);

static navu2pm_callback_t callback[NAVU2PM_IRQ_CB_NUM_MAX];

DECLARE_WORK(navu2pm_usb_ws,  detusb_bh_handler);
DECLARE_WORK(navu2pm_pk_ws,   pwrkey_bh_handler);
DECLARE_WORK(navu2pm_ht_ws,   hightemp_bh_handler);
DECLARE_WORK(navu2pm_rtc_ws,  detrtc_bh_handler);
DECLARE_WORK(navu2pm_xacc_ws, detxacc_bh_handler);

static char dev_id[] = "navu2pm";

static navu2pm_irq_info_t navu2pm_irq_info[] = {
	{INT_PWC_DETUSBINT,       detusb_irq_handler,   IRQF_DISABLED|IRQF_SHARED, 
	 "DETUSB_IRQ", dev_id, IRQT_RISING,   FALSE},

	{PGPF_GPIO_PWRKEY_XDET,   pwrkey_irq_handler,   IRQF_DISABLED, 
	 "PWRKEY_IRQ", dev_id, IRQT_FALLING,  TRUE },

	{PGPF_GPIO_HIGHTEMP_XDET, hightemp_irq_handler, IRQF_DISABLED, 
	 "HITEMP_IRQ", dev_id, IRQT_BOTHEDGE, TRUE },

	{INT_PWC_RTCINT,          detrtc_irq_handler,   IRQF_DISABLED|IRQF_SHARED, 
	 "DETRTC_IRQ", dev_id, IRQT_RISING,   FALSE },

	{PGPF_GPIO_XACC_IN,       detxacc_irq_handler,  IRQF_DISABLED, 
	 "DETDC_IRQ",  dev_id, IRQT_RISING,   TRUE },
};

static chargerif_irq_callback_t navu2pm_chargerif_irq_info[] = {
	{"EVENT_SE1I",      EVENT_SE1I,      detdc_bh_handler,          NULL},
	{"EVENT_CHRGCURRI", EVENT_CHRGCURRI, det_dc_usb_out_bh_handler, NULL},
};

static irqreturn_t pwrkey_irq_handler(int irq, void *dev_id)
{
	schedule_work(&navu2pm_pk_ws);

	return IRQ_HANDLED;
}

static irqreturn_t detusb_irq_handler(int irq, void *dev_id)
{
	int usb_plugin;

	do{
		usb_plugin = controller_get_usb_connect();

		if(usb_plugin){
			set_irq_type(irq, IRQT_FALLING);
		}else{
			set_irq_type(irq, IRQT_RISING);
		}

	}while(usb_plugin != controller_get_usb_connect());

	schedule_work(&navu2pm_usb_ws);

	return IRQ_HANDLED;
}

static irqreturn_t detxacc_irq_handler(int irq, void *dev_id)
{
	int dc_plugin;

	do{
		dc_plugin = controller_get_dc_connect();

		if(dc_plugin){
			set_irq_type(irq, IRQT_RISING);
		}else{
			set_irq_type(irq, IRQT_FALLING);
		}

	}while(dc_plugin != controller_get_dc_connect());

	schedule_work(&navu2pm_xacc_ws);

	return IRQ_HANDLED;
}

static irqreturn_t detrtc_irq_handler(int irq, void *dev_id)
{
	schedule_work(&navu2pm_rtc_ws);

	return IRQ_HANDLED;
}

static irqreturn_t hightemp_irq_handler(int irq, void *dev_id)
{
	schedule_work(&navu2pm_ht_ws);

	return IRQ_HANDLED;
}

static void detdc_bh_handler(void *arg)
{
	do_callback(NAVU2PM_IRQ_CB_DC_DET);

	return ;
}

static void detusb_bh_handler(struct work_struct *work)
{
	do_callback(NAVU2PM_IRQ_CB_USB_DET);

	return ;
}

static void pwrkey_bh_handler(struct work_struct *work)
{
	do_callback(NAVU2PM_IRQ_CB_POWER_KEY);

	return ;
}

static void hightemp_bh_handler(struct work_struct *work)
{
	do_callback(NAVU2PM_IRQ_CB_HIGH_TEMP);

	return ;
}

static void detrtc_bh_handler(struct work_struct *work)
{
	do_callback(NAVU2PM_IRQ_CB_RTC_DET);

	return ;
}

static void detxacc_bh_handler(struct work_struct *work)
{
	do_callback(NAVU2PM_IRQ_CB_DC_DET);

	return; 
}

static void det_dc_usb_out_bh_handler(void *arg)
{
	do_callback(NAVU2PM_IRQ_CB_DC_USB_OUT_DET);

	return ;
}

inline static void do_callback(navu2pm_irq_cb_t irq)
{
	if(callback[irq].func){
		callback[irq].func(callback[irq].arg);
	}

	return ;
}

int navu2pm_register_irq(void)
{
	navu2pm_status_t ret;
	navu2pm_irq_info_t *p;
	int i;

	/* Register RICOH PWC IRQ */ 
	for(i = 0; i < ARRAY_SIZE(navu2pm_irq_info); i++){
		p = &navu2pm_irq_info[i]; 
		
		/* xacc_in handler is not registered Before RS */
		if(PGPF_GPIO_XACC_IN == p->irq &&
		   PGPF_PRODUCTION_LEVEL_RS > pgpf_get_production_level()){
			continue;
		}

		if(p->gpio_flag){
			p->irq = gpio_to_irq(p->irq);
		}

		set_irq_type(p->irq, p->trigger_type);
		ret = request_irq(p->irq, p->handler, p->flags, p->dev_name, p->dev_id);
		if(ret){
			DPRINT("## irq%d error\n", p->irq);
			return NAVU2PM_ERR_OTHER;
		}
	}

	/* Register Atlas IRQ */ 
	for(i = 0; i < ARRAY_SIZE(navu2pm_chargerif_irq_info); i++){

		/* se1i handler is not registered Before RS */
		if(EVENT_SE1I == navu2pm_chargerif_irq_info[i].event &&
		   PGPF_PRODUCTION_LEVEL_RS <= pgpf_get_production_level()){
			continue;
		}

		ret = chargerif_register_callback(&navu2pm_chargerif_irq_info[i]);
		if(NAVU2PM_SUCCESS != ret){
			return NAVU2PM_ERR_OTHER;
		}
	}

	return NAVU2PM_SUCCESS;
}

void navu2pm_register_callback(navu2pm_irq_cb_t irq, void (func)(void *), void *arg)
{
	if(NULL == func){
		DPRINT("## Invalid argument\n");
		return ;
	}

	callback[irq].irq  = irq;
	callback[irq].func = func;
	callback[irq].arg  = arg;

	return ;	
}
