/* 2009-08-24: File added and changed by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <linux/platform_device.h>
#include <linux/delay.h>

#include <asm/arch/pm.h>
#include <asm/arch/smu.h>
#include <asm/arch/pgpf/navu2/pm.h>

#include "internal.h"
#include "chargerif.h"

static void gps_suspend(void);
static void gps_resume(void);
static void bt_suspend(void);
static void ecompass_suspend(void);
static void ecompass_resume(void);
static void oneseg_suspend(void);
static void oneseg_resume(void);

static inline void mask_write(unsigned int addr, unsigned int val, unsigned int mask);

static int gpio_ecompass_setting;
static int gpio_oneseg_setting;

static void gps_suspend(void)
{
	/* GPS_XRST -> L */
	gpio_set_value(PGPF_GPIO_GPS_XRST, 0);

	/* GPS_TX, RX -> GPIO */
	mask_write(0x00000000, 0x00003c00, CHG_PINSEL_G80); 

	/* SYS_XOFF -> L */
	gpio_set_value(PGPF_GPIO_SYS_XOFF, 0);

	return;
}

static void gps_resume(void)
{
	/* SYS_XOFF -> H*/
	gpio_set_value(PGPF_GPIO_SYS_XOFF, 1);

	msleep(1);

	/* GPS_TX, RX -> GPIO */
	mask_write(0x00002800, 0x00003c00, CHG_PINSEL_G80); 

	msleep(100);

	/* GPS_XRST -> H */
	gpio_set_value(PGPF_GPIO_GPS_XRST, 1);

	return;
}

static void bt_suspend(void)
{
	/* BT_XRST -> L */
	gpio_set_value(PGPF_GPIO_BT_XRST, 0);

	/* BT_XRTS, BT_XCTS, BT_TXD, BT_RXD -> GPIO */
	mask_write(0x00000000, 0xff000000, CHG_PINSEL_G96);

	/* BT_PWRON -> L */
	gpio_set_value(PGPF_GPIO_BT_PWRON, 0);

	return;
}

static void ecompass_suspend(void)
{
	gpio_ecompass_setting = gpio_get_value(PGPF_GPIO_COMPASS_XRST);

	/* COMPASS_XRST -> L */
	gpio_set_value(PGPF_GPIO_COMPASS_XRST, 0);
		
	return ;
}

static void ecompass_resume(void)
{
	if(gpio_ecompass_setting){
		gpio_set_value(PGPF_GPIO_COMPASS_XRST, 1);
	}
	
	return ;
}

static void oneseg_suspend(void)
{
	gpio_oneseg_setting = gpio_get_value(PGPF_GPIO_1SEG_PWRON);

	gpio_set_value(PGPF_GPIO_1SEG_PWRON, 0);
	gpio_set_value(PGPF_GPIO_1SEG_XRST,  0);
		
	return ;
}

static void oneseg_resume(void)
{
	if(gpio_oneseg_setting){
		gpio_set_value(PGPF_GPIO_1SEG_PWRON, 1);
		msleep(10);
		gpio_set_value(PGPF_GPIO_1SEG_XRST,  1);
	}
	
	return ;
}

static inline void mask_write(unsigned int data, unsigned int mask, unsigned int addr)
{
	unsigned int tmp_data;
	
	if(NULL == addr){
		return ;
	}

	tmp_data = inl(addr);
	outl((data & mask) | (tmp_data & ~mask), addr);

    return ;
}

navu2pm_status_t navu2pm_suspend(void)
{
 	bt_suspend(); 
	gps_suspend();
	ecompass_suspend();
	oneseg_suspend();

	/* Kernel Suspend */
    mp200_pm_sleep(PM_SLEEP_MODE_S3);

	oneseg_resume();
	ecompass_resume();
	gps_resume();

    return NAVU2PM_SUCCESS;
}
EXPORT_SYMBOL(navu2pm_suspend);

