/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * mp200_udc.c -- for MP200 High speed USB device controller.
 *	This driver doesn't support OTG.
 *
 *	Copyright (C) NEC Electronics Corporation 2009
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* #define UDC_FS */
//#define DUMMY_PRIME
//	#define DUMMY_PRIME_NOT_EP0
//	#define DUMMY_PRIME_NAK_INT
//#define	DEBUG_COUNT
//#define	DEBUG_LOG
//#define	MEMORY_LOG

#undef	DEBUG
#undef	VERBOSE

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <linux/moduleparam.h>
#include <linux/device.h>
#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>
#include <linux/usb/otg.h>
#include <linux/dma-mapping.h>
#include <linux/clk.h>
#include <linux/seq_file.h>
#include <linux/irq.h>

#include <asm/errno.h>
#include <asm/arch/pmu.h>
#include <asm/arch/irqs.h>
#include <asm/bitops.h>
#include <asm/hardware.h>
#include <asm/arch/spi.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/signal.h>
#include <asm/arch/smu.h>
#ifdef CONFIG_MP200_PWC
#include <asm/arch/pwc.h>
#endif
#include <asm/arch/gpio.h>
#include <asm/arch/ahb.h>
#include <asm/arch/pm.h>
#ifdef CONFIG_SNSC_MP200_PLATFORM
#include <asm/arch/udc.h>
#endif /* CONFIG_SNSC_MP200_PLATFORM */

#include "mp200_udc.h"

/* ISO too */
#ifdef DUMMY_PRIME
 #undef		USE_ISO
#else /* DUMMY_PRIME */
 #define	USE_ISO
#endif /* DUMMY_PRIME */

#define	DRIVER_DESC	"MP200 UDC driver"
#define	DRIVER_VERSION	"20080526"

#define	DMA_ADDR_INVALID	(~(dma_addr_t)0)

static const char proc_filename[] = "driver/udc";
static const char driver_name[] = "mp200_udc";
static const char driver_desc[] = DRIVER_DESC;

/* context */
static struct mp200_udc		g_udc = {{0}};

/* prototype */
static int
 mp200_udcgad_get_frame(struct usb_gadget *pgadget);
static int
 mp200_udcgad_wakeup(struct usb_gadget *pgadget);
static int
 mp200_udcgad_set_selfpowered(struct usb_gadget *pgadget, int is_selfpowered);
static int
 mp200_udcgad_vbus_session(struct usb_gadget *pgadget, int is_active);
static int
 mp200_udcgad_vbus_draw(struct usb_gadget *pgadget, unsigned mA);
static int
 mp200_udcgad_pullup(struct usb_gadget *pgadget, int is_on);
static int
 mp200_udcgad_ioctl(struct usb_gadget *pgadget,
 				unsigned code, unsigned long param);

static int
 mp200_udcep_ep_enable(struct usb_ep *puep,
			   const struct usb_endpoint_descriptor *pdesc);
static int
 mp200st_udc_create_CBI_dTD(struct mp200_udc *pudc,
			   struct mp200_ep *pep,
			   struct mp200_req *preq,
			   PT_MEMORY_VP_INFO pdTD_mem,
			   u8 ep_adrs);
static int
 mp200st_udc_create_ISO_dTD(struct mp200_udc *pudc,
			   struct mp200_ep *pep,
			   struct mp200_req *preq,
			   PT_MEMORY_VP_INFO pdTD_mem,
			   u8 ep_adrs);
static int
 mp200st_udc_create_dTD(struct mp200_udc *pudc,
			   struct mp200_ep *pep,
			   struct mp200_req *preq,
			   PT_MEMORY_VP_INFO pdTD_mem,
			   u8 ep_adrs);
static void
 mp200st_udc_delete_dTD(struct mp200_udc *pudc, u8 ep_adrs);
static void
 mp200st_udc_cancel_dTD(struct mp200_udc *pudc, u8 ep_adrs);
static int
 mp200st_udc_link_dTD(struct mp200_udc *pudc,
			  struct mp200_req *preq,
			  PT_MEMORY_VP_INFO pdTD_mem,
			  u8 ep_adrs);
static int
 mp200st_udc_unlink_dTD(struct mp200_udc *pudc, u8 ep_adrs);
static int
 mp200st_udc_done_transfer(struct mp200_udc *pudc, u8 ep_adrs);
static int
 mp200st_udc_set_start_dTD(struct mp200_udc *pudc,
			   struct mp200_ep *pep,
			   struct mp200_req *preq,
			   u8 ep_adrs);
static int
 mp200st_udc_submit_dTD(struct mp200_udc *pudc, struct mp200_ep *pep);
static int
 mp200st_udc_abort_dTD(struct mp200_udc *pudc, u8 ep_adrs);
static void
 mp200st_udc_done_callback(struct mp200_udc *pudc,
			   struct mp200_ep *pep,
			   struct mp200_req *preq,
			   int comp_status,
			   bool bstart_next);
static void
 mp200st_udc_nuke(struct mp200_udc *pudc, struct mp200_ep *pep, int status);
static void
 mp200st_udc_quiesce(struct mp200_udc *pudc);
static int
 mp200_udcep_ep_disable(struct usb_ep *puep);
static struct usb_request *
 mp200_udcep_alloc_request(struct usb_ep *puep, gfp_t gfp_flags);
static void
 mp200_udcep_free_request(struct usb_ep *puep, struct usb_request *pureq);
static int
 mp200st_udc_list_add_and_submit_dTD(struct mp200_udc *pudc,
					 struct mp200_ep *pep,
					 struct mp200_req *preq);
static int
 mp200_udcep_ep_queue(struct usb_ep *puep,
			  struct usb_request *pureq,
			  gfp_t gfp_flags);
static int
 mp200_udcep_ep_dequeue(struct usb_ep *puep, struct usb_request *pureq);
static int
 mp200_udcep_ep_set_halt(struct usb_ep *puep, int bstall);
static int
 mp200st_udc_get_ep_req_actual_length(struct mp200_udc *pudc,
					u8 ep_adrs,
					u32 use_flag,
					struct mp200_ep **pep_ret,
					struct mp200_req **preq_ret);
static int
 mp200_udcep_fifo_status(struct usb_ep *puep);
static void
 mp200_udcep_fifo_flush(struct usb_ep *puep);

#ifdef CONFIG_USB_NWM_MODULE
static int
 mp200_udcep_ep_clear_stall(struct usb_ep *puep);
#endif	// CONFIG_USB_NWM_MODULE

static int
 mp200st_udcprc_proc_udc_show(struct seq_file *s, void *pram);
static int
 mp200_udcprc_proc_udc_open(struct inode *inode, struct file *file);
#ifdef DEBUG
static ssize_t
 mp200_udcprc_proc_udc_write(struct file *fd, const char __user *buf, size_t len, loff_t *ptr);
#endif /* DEBUG */
static PT_ENDPOINT_QUEUE_HEAD
 mp200st_udc_ep_get_dQH_adrs(struct mp200_udc *pudc,
				  u8 ep_adrs,
				  bool bphy);
static bool
 mp200st_udc_get_endpoint_stall(struct mp200_udc *pudc,
				 u8 ep_adrs);
static void
 mp200st_udc_set_endpoint_stall(struct mp200_udc *pudc,
				   u8 ep_adrs,
				   bool bstall);
static void
 mp200st_udcint_EP0_complete(struct usb_ep *puep, struct usb_request *pureq);
static irqreturn_t
 mp200_udcint_irq(int irq, void *id_udc);
#ifndef CONFIG_SNSC_MP200_PLATFORM
static irqreturn_t
 mp200_udcint_vbus_irq(int irq, void *id_udc);
#endif /* !CONFIG_SNSC_MP200_PLATFORM */
static int
 __init mp200_udcdev_probe(struct platform_device *pdev);
static void
 mp200_udcgaddev_release(struct device *pgaddev);
static int
 __exit mp200_udcdev_remove(struct platform_device *pdev);
static int
 mp200_udcdev_suspend(struct platform_device *pdev, pm_message_t message);
static int
 mp200_udcdev_resume(struct platform_device *pdev);
int
 usb_gadget_register_driver(struct usb_gadget_driver *pdriver);
int
 usb_gadget_unregister_driver(struct usb_gadget_driver *pdriver);
static int
 __init mp200_udc_init(void);
static void __exit
 mp200_udc_exit(void);

static void
 mp200st_udc_set_ep_ctrlreg(struct mp200_udc *pudc,
			u8 ep_adrs,
			u8 bm_attributes
			);

#ifdef CONFIG_MACH_PGPF
extern void pgpf_usb_phy_enable(int flag);
extern void pgpf_usb_low_power_mode_enable(int flag);
#endif /* CONFIG_MACH_PGPF */

#ifdef CONFIG_PGPF_NAVU2
#define ENABLE_DEDICATED_CHARGER_DETECTION
#endif

#ifdef ENABLE_DEDICATED_CHARGER_DETECTION
extern void pgpf_usb_notify_supplied_power(bool flag, int power);
static int mp200_udc_detect_charger(struct mp200_udc *pudc);
#endif /* ENABLE_DEDICATED_CHARGER_DETECTION */

/* inline */

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udcint_data_packet(
 	struct mp200_udc *pudc,
 	void *pbuf,
 	unsigned length,
 	dma_addr_t dma)
{
	memset(&pudc->ep0_req, 0, sizeof(struct mp200_req));
	pudc->ep0_req.req.buf		= pbuf;
	pudc->ep0_req.req.length	= length;
	pudc->ep0_req.req.dma		= dma;
	pudc->ep0_req.req.zero		= TRUE;
	pudc->ep0_req.req.complete	= mp200st_udcint_EP0_complete;
	pudc->ep0_req.req.status	= -EINPROGRESS;
	mp200st_udc_list_add_and_submit_dTD(pudc, &pudc->ep[0], &pudc->ep0_req);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udc_ep_clear_dQH(struct mp200_udc *pudc, u8 ep_adrs)
{
	PT_ENDPOINT_QUEUE_HEAD	p_ep_dQH;

	p_ep_dQH = mp200st_udc_ep_get_dQH_adrs(pudc, ep_adrs, FALSE);

	/* clear */
	memset(p_ep_dQH, 0, sizeof(T_ENDPOINT_QUEUE_HEAD));

	return 0;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_clear_EP0(struct mp200_udc *pudc)
{
	/* EP0 out(setup) */
	mp200st_udc_ep_clear_dQH(pudc, (0 | USB_DIR_OUT));

	/* EP0 in */
	mp200st_udc_ep_clear_dQH(pudc, (0 | USB_DIR_IN));

	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_pullup_enable(struct mp200_udc *pudc)
{
	mp200st_udc_set_bit(&pudc->pfc_reg->USBCMD, USBCMD_RS);
	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_pullup_disable(struct mp200_udc *pudc)
{
	mp200st_udc_clear_bit(&pudc->pfc_reg->USBCMD, USBCMD_RS);
	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_interrupt_enable(struct mp200_udc *pudc)
{
#ifdef UDC_FS
	mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->PORTSC[0], PORTSC_PFSC);		/* USB1.1(FS) */
#endif /* UDC_FS */

#ifdef DUMMY_PRIME
 #ifdef DUMMY_PRIME_NAK_INT
	mp200st_udc_writel(&pudc->pfc_reg->USBINTR, USB_FC_INT_ENABLE | USBINTR_NAKE);
 #else /* DUMMY_PRIME_NAK_INT */
	mp200st_udc_writel(&pudc->potg_reg->GPTIMER0LD, 374);
	mp200st_udc_writel(&pudc->potg_reg->GPTIMER0CTRL, 0xc1000000);
	mp200st_udc_writel(&pudc->pfc_reg->USBINTR, USB_FC_INT_ENABLE | USBINTR_TIE0);
 #endif /* DUMMY_PRIME_NAK_INT */
#else /* DUMMY_PRIME */
	mp200st_udc_writel(&pudc->pfc_reg->USBINTR, USB_FC_INT_ENABLE);
#endif /* DUMMY_PRIME */
	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_interrupt_disable(struct mp200_udc *pudc)
{
	mp200st_udc_writel(&pudc->pfc_reg->USBINTR, 0);
	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_reset(struct mp200_udc *pudc)
{
	//---------------------------------------------------------------
	// Link disable
	mp200_pmu_reset_device(MP200_RESETDEVICE_USB);	// Reset ON
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_USB_CLK);	// CLK OFF

	//---------------------------------------------------------------
	// PHY disable
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pudc->config && pudc->config->udc_disable_phy)
		pudc->config->udc_disable_phy(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
#ifdef CONFIG_MACH_PGPF
	pgpf_usb_phy_enable(0);
#else /* CONFIG_MACH_PGPF */
	gpio_direction_output(UDC_PHY_CS, 1);			// CS
	pwc_write(PWC_PSCNT1, 0, PHY_POWER_BIT);		// Power OFF
#endif /* CONFIG_MACH_PGPF */
#endif /* CONFIG_SNSC_MP200_PLATFORM */

//	//---------------------------------------------------------------
//	// clock auto change(resume status)
//	outl(pudc->clk_reg, SMU_AUTO_FRQ_CHANGE);

	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_unreset(struct mp200_udc *pudc)
{
#if 1	/* for SMSC */
	int	status;
#else
	int	status, status2;
	int	status_d, status_d2;
#endif

//	//---------------------------------------------------------------
//	// clock auto change OFF
//	pudc->clk_reg = inl(SMU_AUTO_FRQ_CHANGE);
//	outl(pudc->clk_reg & 0xFFFFFFFE, SMU_AUTO_FRQ_CHANGE);

	//---------------------------------------------------------------
	// PHY
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pudc->config && pudc->config->udc_enable_phy)
		pudc->config->udc_enable_phy(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
#ifdef CONFIG_MACH_PGPF
	pgpf_usb_phy_enable(0);
	mdelay(UDC_PHY_CS_MDELAY);
	pgpf_usb_phy_enable(1);
#else /* CONFIG_MACH_PGPF */
	pwc_write(PWC_PSCNT1, PHY_POWER_BIT, PHY_POWER_BIT);	// Power ON
	gpio_direction_output(UDC_PHY_CS, 0);					// CS
#endif /* CONFIG_MACH_PGPF */
	mdelay(UDC_PHY_CS_MDELAY);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	//---------------------------------------------------------------
	// Link
	status = mp200_pmu_open_clockgate(MP200_CLOCKGATE_USB_CLK);	// CLK ON
	if(status < 0){
		DBG("mp200_pmu_open_clockgate ERR\n");
	}

	status = mp200_pmu_unreset_device(MP200_RESETDEVICE_USB);	// Reset OFF
	if(status < 0){
		DBG("mp200_pmu_unreset_device ERR\n");
	}

#if 1	/* for SMSC */
	mdelay(UDC_LINK_UNRESET_MDELAY);
#else
	udelay(UDC_LINK_UNRESET_UDELAY);
#endif

#ifdef CONFIG_PGPF_USB_LOW_POWER_MODE_ENABLE
	pgpf_usb_low_power_mode_enable(0);
#endif

	//---------------------------------------------------------------
	// Link
	//	 Setting for Device(Function)
	mp200st_udc_set_bit(&pudc->pfc_reg->USBMODE, UDBMODE_CM_FC | USBMODE_SLOM);
	mp200st_udc_mask_set_bit(&pudc->pfc_reg->USBCMD, USBCMD_ITC, USBCMD_ITC_01);	/*	1 uFrame interrupt */

	status = mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPOINTLISTADDR, (u32)pudc->dQH.padrs);
	if(status < 0){
		DBG("mp200st_udc_ulc_chk_writel ERR\n");
	}

	/*---------------------------------------------------------------*/
	/**/
#if 1	/* for SMSC */
	status = mp200st_udc_ulpi_write(pudc, 0x0F, BIT01);	/* VbusValid Rise INT Disable */
	if(status < 0){
		printk("VbusValid Rise INT Disable Error\n");
	}

	status = mp200st_udc_ulpi_write(pudc, 0x12, BIT01);	/* VbusValid Fall INT Disable */
	if(status < 0){
		printk("VbusValid Fall INT Disable Error\n");
	}
#endif

	return;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udc_ep_set_dQH(
	struct mp200_udc *pudc,
	u8 ep_adrs,
	u8 bm_attributes,
	u16 w_max_packet_size)
{
	PT_ENDPOINT_QUEUE_HEAD	p_ep_dQH;

	p_ep_dQH = mp200st_udc_ep_get_dQH_adrs(pudc, ep_adrs, FALSE);

	/* clear */
	p_ep_dQH->max_length = 0;

	/*	no zero length */
	p_ep_dQH->max_length |= EQH_ZLT;

	/*	max data */
	p_ep_dQH->max_length |= ((w_max_packet_size & USB_ENDPOINT_MPS_SIZE_MASK) << EQH_MPLENGTH_BIT_POS);

#ifdef USE_ISO
	/*	Multi */
	if((bm_attributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_ISOC){
		switch(w_max_packet_size & USB_ENDPOINT_MPS_MULTI_MASK){
		case USB_ENDPOINT_MPS_MULTI_NONE:
			p_ep_dQH->max_length |= EQH_MPS_ISO_MULTI_NONE;
			break;

		case USB_ENDPOINT_MPS_MULTI_1ADD:
			p_ep_dQH->max_length |= EQH_MPS_ISO_MULTI_1ADD;
			break;

		case USB_ENDPOINT_MPS_MULTI_2ADD:
			p_ep_dQH->max_length |= EQH_MPS_ISO_MULTI_2ADD;
			break;

		default:
			ERR("Isochronous MULTI error\n");
			p_ep_dQH->max_length = 0;
			return -EINVAL;
		}
	}
#endif /* USE_ISO */

	/*	SETUP */
	if((bm_attributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_CONTROL){
		p_ep_dQH->max_length |= EQH_SETUP_INT;
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_set_EP0(struct mp200_udc *pudc)
{

	/* EP0 out(setup) */
	mp200st_udc_ep_set_dQH(pudc,
				   (0 | USB_DIR_OUT),
				   USB_ENDPOINT_XFER_CONTROL,
				   UDC_EP0_MAXPACKETSIZE);

	/* EP0 in */
	mp200st_udc_ep_set_dQH(pudc,
				   (0 | USB_DIR_IN),
				   USB_ENDPOINT_XFER_CONTROL,
				   UDC_EP0_MAXPACKETSIZE);
	return;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 __init mp200st_udcdev_dtd_dqh_mem_init(
	struct mp200_udc *pudc,
	struct platform_device *pdev,
	PT_MEMORY_VP_INFO p_align_mem,
	PT_MEMORY_VP_INFO p_mem,
	size_t alloc_mem_size,
	size_t align_size)
{
	u32		mem_alignment;

	/* allocate DMA memory */
	p_align_mem->p_vadrs = dma_alloc_coherent(&pdev->dev,
						  alloc_mem_size,
						  &p_align_mem->padrs,
						  GFP_KERNEL|__GFP_DMA);

	/* check allocate ERROR */
	if(p_align_mem->p_vadrs == NULL) return -ENOMEM;
	MEM_LOG(pudc,"%s dma_alloc_coherent(%08x)\n",__FUNCTION__,(int)p_align_mem->p_vadrs);

	/* check memory alignment */
	mem_alignment = (u32)p_align_mem->padrs & (align_size - 1);
	if(mem_alignment > 0){
		/* shift phy address */
		p_mem->padrs = p_align_mem->padrs + (align_size - mem_alignment);
		/* shift virtual address */
		p_mem->p_vadrs = (u32*)(((u8*)p_align_mem->p_vadrs) + (align_size - mem_alignment));
	}else{
		/* no shift address (struct copy) */
		memcpy(p_mem, p_align_mem, sizeof(T_MEMORY_VP_INFO));
	}

	/* clear DMA memory */
	memset(p_mem->p_vadrs , 0, (alloc_mem_size - align_size));

	return 0;
}

/*-------------------------------------------------------------------------*/
static bool __inline__
 mp200st_udc_can_pullup(struct mp200_udc *pudc)
{
	if((pudc->pdriver != NULL) && pudc->softconnect && pudc->vbus_active && (pudc->linux_suspended == 0)){
		return TRUE;
	}else{
		return FALSE;
	}
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_device_start(struct mp200_udc *pudc)
{
	if(mp200st_udc_can_pullup(pudc)){
		mp200st_udc_set_EP0(pudc);
		mp200st_udc_unreset(pudc);
		mp200st_udc_interrupt_enable(pudc);
		mp200st_udc_pullup_enable(pudc);
		mp200st_udc_ulpi_write(pudc, 0x0C, BIT02);
	}

#ifdef ENABLE_DEDICATED_CHARGER_DETECTION
	mp200_udc_detect_charger(pudc);
#endif /* ENABLE_DEDICATED_CHARGER_DETECTION */

	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_device_stop(struct mp200_udc *pudc)
{
#ifdef ENABLE_DEDICATED_CHARGER_DETECTION
	pgpf_usb_notify_supplied_power(0, 0);
#endif /* ENABLE_DEDICATED_CHARGER_DETECTION */
#ifdef DUMMY_PRIME
	pudc->dummy_prime_transferring = 0;
	pudc->dummy_prime_waiting = 0;
#endif /* DUMMY_PRIME */

	mp200st_udc_pullup_disable(pudc);
	mp200st_udc_interrupt_disable(pudc);
#ifdef CONFIG_PGPF_USB_LOW_POWER_MODE_ENABLE
	if( pudc->linux_suspended != 1 ){
		pgpf_usb_low_power_mode_enable(1);
	}else{
		mp200st_udc_reset(pudc);
	}
#else
	mp200st_udc_reset(pudc);
#endif
	mp200st_udc_clear_EP0(pudc);

	/* clear devstate */
	pudc->devstate		= USB_STATE_NOTATTACHED;

	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_req_feature_complete(struct mp200_udc *pudc, bool bset)
{
	u32	data;

	if((pudc->ctrl.bRequestType & USB_RECIP_MASK) == USB_RECIP_DEVICE){

		switch (le16_to_cpu(pudc->ctrl.wValue)){
		case USB_DEVICE_TEST_MODE:
			if(bset){
				data = ((le16_to_cpu(pudc->ctrl.wIndex)) << 8) & PORTSC_PTC;
				mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->PORTSC[0], data);
			}
			break;
		default:
			break;
		}
	}

	return ;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udcint_status_packet(struct mp200_udc *pudc)
{
	return mp200st_udcint_data_packet(pudc, NULL, 0, DMA_ADDR_INVALID);
}

/*-------------------------------------------------------------------------*/
static enum usb_device_speed __inline__
 mp200st_udcint_get_speed(struct mp200_udc *pudc)
{
	enum usb_device_speed speed;

	switch(mp200st_udc_readl(&pudc->pfc_reg->PORTSC[0]) & PORTSC_PSPD){
	case PORTSC_PSPD_FS:
		speed = USB_SPEED_FULL;
		break;

	case PORTSC_PSPD_LS:
		speed = USB_SPEED_LOW;
		break;

	case PORTSC_PSPD_HS:
		speed = USB_SPEED_HIGH;
		break;

	default:
		speed = USB_SPEED_UNKNOWN;
		ERR("PORTSC speed ERR\n");
		break;
	}

	return speed;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_soft_connect_change(struct mp200_udc *pudc, bool is_on)
{
	/* Already status change */
	if((!pudc->softconnect) == (!is_on)){
		DBG("%s  Already status change\n",__FUNCTION__);
		return;
	}

	pudc->usb_suspended = 0;
	if(is_on){
		/* soft connect */
		pudc->softconnect = 1;

		mp200st_udc_device_start(pudc);
	}
	else{
		/* soft disconnect */
		pudc->softconnect = 0;

		/* callback */
		if((pudc->linux_suspended == 0) && pudc->vbus_active && (pudc->pdriver->disconnect != NULL)){
			spin_unlock(&pudc->lock);
			pudc->pdriver->disconnect(&pudc->gadget);
			spin_lock(&pudc->lock);
		}

		mp200st_udc_quiesce(pudc);
		mp200st_udc_device_stop(pudc);
	}

	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_vbus_change(struct mp200_udc *pudc, bool bvbus_on)
{
	/* Already status change */
	if((!pudc->vbus_active) == (!bvbus_on)){
		DBG("%s  Already status change\n",__FUNCTION__);
		if(pudc->linux_resume){
			pudc->linux_resume = 0;
			mp200st_udc_device_start(pudc);
		}
		return;
	}

	pudc->usb_suspended		= 0;
	if(bvbus_on){
		/* vbus active */
		mdelay(VBUS_CHATTERING_RETRY_MDELAY);	// ULPI Wait
		pudc->vbus_active = 1;
		pudc->devstate = USB_STATE_POWERED;
		mp200st_udc_device_start(pudc);
	}
	else{
		/* vbus inactive */
		pudc->vbus_active = 0;
		pudc->devstate = USB_STATE_ATTACHED;
		pudc->remote_wakeup = 0;

		/* callback */
		if((pudc->linux_suspended == 0) && (pudc->softconnect != 0)){
			spin_unlock(&pudc->lock);
			if(pudc->pdriver->disconnect != NULL){
				pudc->pdriver->disconnect(&pudc->gadget);
			}
#ifdef CONFIG_USB_NWM_MODULE
			if(pudc->pdriver->vbusoff != NULL){
				pudc->pdriver->vbusoff(&pudc->gadget);
			}
#endif	// CONFIG_USB_NWM_MODULE
			spin_lock(&pudc->lock);
		}

		mp200st_udc_quiesce(pudc);
		mp200st_udc_device_stop(pudc);
	}

	return;
}

#ifdef DUMMY_PRIME
/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_set_dummy_prime_transferring(struct mp200_udc *pudc, u32 bit_pos)
{
 #ifdef DEBUG
	if(!(pudc->dummy_prime_waiting & bit_pos)){
		DBG("%s pit_pos(%08x) dummy_prime_waiting(%08x) bug.\n",__FUNCTION__,bit_pos,pudc->dummy_prime_waiting);
	}
 #endif /* DEBUG */

	/* set */
	pudc->dummy_prime_transferring |= bit_pos;
	pudc->dummy_prime_waiting &= ~(bit_pos);

 #if !defined(DUMMY_PRIME_NOT_EP0)
	/* EP0 && SETUP Receiving */
	if(bit_pos & (ENDPT_BIT_POS(0 | USB_DIR_IN) | ENDPT_BIT_POS(0 | USB_DIR_OUT))){
		if(mp200st_udc_readl(&pudc->pfc_reg->ENDPTSETUPSTAT) & ENDPTSETUPSTAT_ENDPTSETUPSTAT_EP0){
			DBG("%s Receiving ENDPTSETUPSTAT_EP0 (3)(%08x)\n",__FUNCTION__, bit_pos);
			return;
		}
	}
 #endif /* !defined(DUMMY_PRIME_NOT_EP0) */
	/* prime */
	mp200st_udc_writel(&pudc->pfc_reg->ENDPTPRIME, pudc->dummy_prime_transferring);
	DBG_LOG(pudc,"PRIME5(%08x)\n",pudc->dummy_prime_transferring);

	return;
}
#endif /* DUMMY_PRIME */

#ifdef DUMMY_PRIME
/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_set_dummy_prime_waiting(struct mp200_udc *pudc, u32 bit_pos)
{
	pudc->dummy_prime_waiting |= bit_pos;
 #ifdef DUMMY_PRIME_NAK_INT
	if(pudc->dummy_prime_transferring == 0){
		mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAK, bit_pos);
		mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->ENDPTNAKEN, bit_pos);
	}
 #endif /* DUMMY_PRIME_NAK_INT */

	return;
}
#endif /* DUMMY_PRIME */

/* function */

/*-------------------------------------------------------------------------*/
static struct usb_gadget_ops mp200_gadget_ops = {
	.get_frame		= mp200_udcgad_get_frame,
	.wakeup			= mp200_udcgad_wakeup,
	.set_selfpowered	= mp200_udcgad_set_selfpowered,
	.vbus_session		= mp200_udcgad_vbus_session,
	.vbus_draw		= mp200_udcgad_vbus_draw,
	.pullup			= mp200_udcgad_pullup,
	.ioctl			= mp200_udcgad_ioctl,
};

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udc_linux_suspended_change(struct mp200_udc *pudc, bool is_suspended)
{
	/* Already status change */
	if((!pudc->linux_suspended) == (!is_suspended)){
		DBG("%s  Already status change\n",__FUNCTION__);
		return;
	}

	pudc->usb_suspended		= 0;
	if(is_suspended == FALSE){
		/* resume */
		pudc->linux_suspended = 0;
		mp200st_udc_device_start(pudc);
	}
	else{
		/* suspend */
		pudc->linux_suspended = 1;

		/* callback */
		if(pudc->softconnect && pudc->vbus_active && (pudc->pdriver->disconnect != NULL)){
			spin_unlock(&pudc->lock);
			pudc->pdriver->disconnect(&pudc->gadget);
			spin_lock(&pudc->lock);
		}

		mp200st_udc_quiesce(pudc);
		mp200st_udc_device_stop(pudc);
	}

	return;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcgad_get_frame(struct usb_gadget *pgadget)
{
	struct mp200_udc	*pudc;

	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pudc = container_of(pgadget, struct mp200_udc, gadget);

	return (int)(mp200st_udc_readl(&pudc->pfc_reg->FRINDEX) & FRINDEX_FRINDEX);
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcgad_wakeup(struct usb_gadget *pgadget)
{
	struct mp200_udc	*pudc;
	unsigned long		flags;
	u32			data;
	int			status = -EHOSTUNREACH;

	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pudc = container_of(pgadget, struct mp200_udc, gadget);

	spin_lock_irqsave(&pudc->lock, flags);

	if(!pudc->remote_wakeup){
		DBG("%s fail(EHOSTUNREACH)",__FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return status;
	}

	/* check suspend */
	data = mp200st_udc_readl(&pudc->pfc_reg->PORTSC[0]) & PORTSC_SUSP;
	if(data != 0){
		/* remote wakeup */
		mp200st_udc_set_bit(&pudc->pfc_reg->PORTSC[0], PORTSC_FPR);
		status = 0;
	}

	spin_unlock_irqrestore(&pudc->lock, flags);

	return status;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcgad_set_selfpowered(struct usb_gadget *pgadget, int is_selfpowered)
{
	struct mp200_udc	*pudc;
	unsigned long		flags;

	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pudc = container_of(pgadget, struct mp200_udc, gadget);

	spin_lock_irqsave(&pudc->lock, flags);
	pudc->self_powered = (is_selfpowered != 0);
	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
/*
 * Called by whatever detects VBUS sessions:  external transceiver
 * driver, or maybe GPIO0 VBUS IRQ.  May request 48 MHz clock.
 */
static int
 mp200_udcgad_vbus_session(struct usb_gadget *pgadget, int is_active)
{
	struct mp200_udc	*pudc;
	unsigned long		flags;

	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pudc = container_of(pgadget, struct mp200_udc, gadget);

	spin_lock_irqsave(&pudc->lock, flags);
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if(pudc->vbus_change)
		pudc->vbus_change(pudc, is_active);
#else
	mp200st_udc_vbus_change(pudc, (bool)is_active);
#endif /* CONFIG_SNSC_MP200_PLATFORM */
	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcgad_vbus_draw(struct usb_gadget *pgadget, unsigned mA)
{
	struct mp200_udc	*pudc;
	unsigned long		flags;

	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pudc = container_of(pgadget, struct mp200_udc, gadget);

	spin_lock_irqsave(&pudc->lock, flags);
	pudc->mA = mA;
	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcgad_ioctl(
	struct usb_gadget *pgadget,
	unsigned code,
	unsigned long param)
{
	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	return -EOPNOTSUPP;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcgad_pullup(struct usb_gadget *pgadget, int is_on)
{
	struct mp200_udc	*pudc;
	unsigned long		flags;

	if(pgadget == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pudc = container_of(pgadget, struct mp200_udc, gadget);

	spin_lock_irqsave(&pudc->lock, flags);
	mp200st_udc_soft_connect_change(pudc, (bool)is_on);
	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/

static struct usb_ep_ops mp200_ep_ops = {
	.enable		= mp200_udcep_ep_enable,
	.disable	= mp200_udcep_ep_disable,

	.alloc_request	= mp200_udcep_alloc_request,
	.free_request	= mp200_udcep_free_request,

	.queue		= mp200_udcep_ep_queue,
	.dequeue	= mp200_udcep_ep_dequeue,

	.set_halt	= mp200_udcep_ep_set_halt,
	.fifo_status	= mp200_udcep_fifo_status,
	.fifo_flush	= mp200_udcep_fifo_flush,

#ifdef CONFIG_USB_NWM_MODULE
	.clear_stall	= mp200_udcep_ep_clear_stall,
#endif	// CONFIG_USB_NWM_MODULE
};

/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_ep_enable(
	struct usb_ep *puep,
	const struct usb_endpoint_descriptor *pdesc)
{
	struct mp200_ep		*pep;
	struct mp200_udc	*pudc;
	unsigned long		flags;
	int			status;

	/* catch various bogus parameters */
	if((puep == NULL) || (pdesc == NULL)){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	spin_lock_irqsave(&pudc->lock, flags);

	if((pep == &pudc->ep[0])
	   || (pep->pdesc != NULL)
	   || (pdesc->bDescriptorType != USB_DT_ENDPOINT)){

		DBG("%s descriptor(EINVAL)",__FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -EINVAL;
	}

	/* connect */
	if((pudc->pdriver == NULL)
	   || (pudc->gadget.speed == USB_SPEED_UNKNOWN)){

		DBG("%s, bogus device state\n", __FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -ESHUTDOWN;
	}

	/* check */
#if defined(DUMMY_PRIME) || !defined(USE_ISO)
	if((pdesc->bmAttributes | USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_ISOC){
		DBG("%s, not Support Isochronous\n", __FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -EINVAL;
	}
#endif /* defined(DUMMY_PRIME) || !defined(USE_ISO) */
#ifdef USB_TRANS_DIVISION
	if(pdesc->wMaxPacketSize > USB_DIVISION_SIZE){
		DBG("%s, USB_TRANS_DIVISION not Support wMaxPacketSize > 512\n", __FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -EINVAL;
	}
#endif /* USB_TRANS_DIVISION */

	pep->pdesc = pdesc;

	/* set endpoint to initial state */

	status = mp200st_udc_ep_set_dQH(pudc,
					pdesc->bEndpointAddress,
					pdesc->bmAttributes,
					le16_to_cpu(pdesc->wMaxPacketSize));

	if(status < 0){
		DBG("%s, bogus dQH\n", __FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return status;
	}

	mp200st_udc_set_ep_ctrlreg(pudc,
				   pdesc->bEndpointAddress,
				   pdesc->bmAttributes);



	/* IO<->DMA memory alloc */
	if(pep->mem.p_vadrs == NULL){
		VDBG("%s auto DMA allocate\n",__FUNCTION__);
		pep->mem.p_vadrs = dma_alloc_coherent(&pudc->pdev->dev, UDC_EP_AUTO_BUFSIZE, &pep->mem.padrs, GFP_ATOMIC|__GFP_DMA);

		if(pep->mem.p_vadrs == NULL){
			ERR("%s dma_alloc_coherent NOMEM)\n",__FUNCTION__);
			spin_unlock_irqrestore(&pudc->lock, flags);
			return -ENOMEM;
		}
		MEM_LOG(pudc,"%s dma_alloc_coherent(%08x)\n",__FUNCTION__,(int)pep->mem.p_vadrs);
	}else{
		VDBG("%s Already auto DMA memory allocated\n",__FUNCTION__);
	}
	spin_unlock_irqrestore(&pudc->lock, flags);
	VDBG("%s enabled\n", puep->name);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_ep_disable(struct usb_ep *puep)
{
	struct mp200_ep		*pep;
	struct mp200_udc	*pudc;
	unsigned long		flags;

	if(puep == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	spin_lock_irqsave(&pudc->lock, flags);

	if((pep == &pudc->ep[0])
	   || (pep->pdesc == NULL)){
		DBG("%s descriptor(EINVAL)",__FUNCTION__);
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -EINVAL;
	}

	/* clear dQH */
	mp200st_udc_ep_clear_dQH(pudc, pep->pdesc->bEndpointAddress);

	/* dequeue */
	DBG_LOG(pudc,"EP DISABLE NUKE\n");
	mp200st_udc_nuke(pudc, pep, -ESHUTDOWN);
	pep->pdesc = NULL;

	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static struct usb_request *
 mp200_udcep_alloc_request(struct usb_ep *puep, gfp_t gfp_flags)
{
	struct mp200_req	*preq;
	struct mp200_ep		*pep;
	struct mp200_udc	*pudc;

	if(puep == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return NULL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	preq = kzalloc(sizeof(struct mp200_req), gfp_flags);

	if(preq == NULL){
		DBG("%s memory(NOMEM)",__FUNCTION__);
		return NULL;
	}
	MEM_LOG(pudc,"%s kzalloc(%08x)\n",__FUNCTION__,(int)preq);

	preq->req.dma = DMA_ADDR_INVALID;
	INIT_LIST_HEAD (&preq->queue);

	return &preq->req;
}

/*-------------------------------------------------------------------------*/
static void
 mp200_udcep_free_request(struct usb_ep *puep, struct usb_request *pureq)
{
	struct mp200_req	*preq;
	struct mp200_ep		*pep;
	struct mp200_udc	*pudc;

	if(puep == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	if(pureq != NULL){
		preq = container_of(pureq, struct mp200_req, req);
		MEM_LOG(pudc,"%s kfree(%08x)\n",__FUNCTION__,(int)preq);
		kfree(preq);
	}
	return;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_ep_queue(
	struct usb_ep *puep,
	struct usb_request *pureq,
	gfp_t gfp_flags)
{
	struct mp200_ep		*pep;
	struct mp200_req	*preq;
	struct mp200_udc	*pudc;
	unsigned long		flags;
	__u8			usb_dir_in;

	/* catch various bogus parameters */
	if((pureq == NULL) || (puep == NULL)){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	preq = container_of(pureq, struct mp200_req, req);

	if((preq->req.complete == NULL)
	   || (preq->req.buf == NULL)
	   || !list_empty(&preq->queue)){

		ERR("%s, bad req\n", __FUNCTION__);
		return -EINVAL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	/* set DMA */
	usb_dir_in = (pep->pdesc != NULL) /* not EP0 */
		? (pep->pdesc->bEndpointAddress & USB_DIR_IN)
		: (pudc->ctrl.bRequestType & USB_DIR_IN);

	if((unsigned long)preq->req.buf & (dma_get_cache_alignment() - 1)) {
		/* copy data between usb_request buf and mp200_ep mem */
	}
	else if(preq->req.dma == DMA_ADDR_INVALID) {
		preq->req.dma = dma_map_single(pudc->gadget.dev.parent,
					       preq->req.buf,
					       preq->req.length,
					       usb_dir_in
					       ? DMA_TO_DEVICE
					       : DMA_FROM_DEVICE);
		preq->mapped = 1;
	} else {
		dma_sync_single_for_device(pudc->gadget.dev.parent,
					   preq->req.dma,
					   preq->req.length,
					   usb_dir_in
					   ? DMA_TO_DEVICE
					   : DMA_FROM_DEVICE);
		preq->mapped = 0;
	}

	/* length check */
	if((preq->req.dma == DMA_ADDR_INVALID) || (preq->req.dma == 0)){
		if(preq->req.length > UDC_EP_AUTO_BUFSIZE){
			ERR("%s length size over! %s size=%d\n",__FUNCTION__,pep->name,preq->req.length);
			return -EMSGSIZE;
		}
	}

	/* normal EP check (EP0 threw) */
	if(pep != &pudc->ep[0]){
		if(pep->pdesc == NULL){
			ERR("(pep->pdesc == NULL)\n");
			return -EINVAL;
		}

		if((pep->pdesc->bmAttributes | USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_ISOC){
			if(preq->req.length > (le16_to_cpu(pep->pdesc->wMaxPacketSize) | USB_ENDPOINT_MPS_SIZE_MASK)){
				ERR("USB_ENDPOINT_XFER_ISOC length > wMaxPacketSize \n");
				return -EMSGSIZE;
			}
		}
	}


	VDBG("%s queue req %p, len %d buf %p\n",
		pep->ep.name, pureq, pureq->length, pureq->buf);

	spin_lock_irqsave(&pudc->lock, flags);
	if((pudc->pdriver == NULL) || (pudc->gadget.speed == USB_SPEED_UNKNOWN)){
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -ESHUTDOWN;
	}

	preq->req.status = -EINPROGRESS;
	preq->req.actual = 0;

#ifdef USB_TRANS_DIVISION
	preq->act_len = 0;
	if((pep != &pudc->ep[0]) && (pep->pdesc->bEndpointAddress & USB_DIR_IN)){
//	if(pep != &pudc->ep[0]){

		if(pep->pdesc->wMaxPacketSize == 512){
			//-------------------------------------------------------
			// division transport support only High Speed
			preq->division_len = USB_DIVISION_SIZE;
		}
		else{
			preq->division_len = 0;
		}
	}
#endif /* USB_TRANS_DIVISION */
	VDBG("%s %s length=%d\n",__FUNCTION__ ,pep->name,preq->req.length);

	mp200st_udc_list_add_and_submit_dTD(pudc, pep, preq);

	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_ep_dequeue(struct usb_ep *puep, struct usb_request *pureq)
{
	struct mp200_ep		*pep;
	struct mp200_req	*preq;
	struct mp200_udc	*pudc;
	unsigned long		flags;
	int			status;

	if((puep == NULL) || (pureq == NULL)){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	spin_lock_irqsave(&pudc->lock, flags);

	/* make sure it's actually queued on this endpoint */
	list_for_each_entry(preq, &pep->queue, queue){
		if(&preq->req == pureq){
			break;
		}
	}
	if(&preq->req != pureq){
		spin_unlock_irqrestore(&pudc->lock, flags);
		DBG("%s no queue(EINVAL)",__FUNCTION__);
		return -EINVAL;
	}

	/* fowarding */
	if(pep->queue.next == &preq->queue){
		status = mp200st_udc_abort_dTD(pudc, pep->pdesc->bEndpointAddress);
		if(status < 0){
			DBG("%s abort fail(%d)",__FUNCTION__,status);
			spin_unlock_irqrestore(&pep->pudc->lock, flags);
			return status;
		}
	}

	DBG_LOG(pudc,"DEQUEUE(%02x)\n",pep->pdesc->bEndpointAddress);

	mp200st_udc_done_callback(pudc, pep, preq, -ECONNABORTED, TRUE);

	spin_unlock_irqrestore(&pep->pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_ep_set_halt(struct usb_ep *puep, int bstall)
{
	struct mp200_ep		*pep;
	unsigned long		flags;
	struct mp200_udc	*pudc;
	u8			ep_adrs;

	if(puep == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	spin_lock_irqsave(&pudc->lock, flags);

	if(pep == &pudc->ep[0]){
		ep_adrs = 0;
	}
	else{
		if(pep->pdesc == NULL){
			spin_unlock_irqrestore(&pudc->lock, flags);
			return -EINVAL;
		}

		ep_adrs = pep->pdesc->bEndpointAddress;
	}

	if((bstall) && (!list_empty(&pep->queue))){
		spin_unlock_irqrestore(&pudc->lock, flags);
		return -EAGAIN;
	}

	if(ep_adrs == 0){
		mp200st_udc_set_endpoint_stall(pudc, (0 | USB_DIR_OUT), (bool)bstall);
		mp200st_udc_set_endpoint_stall(pudc, (0 | USB_DIR_IN), (bool)bstall);
	}
	else{
		mp200st_udc_set_endpoint_stall(pudc, ep_adrs, (bool)bstall);
	}

	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_fifo_status(struct usb_ep *puep)
{
	struct mp200_ep		*pep;
	struct mp200_ep		*pep_ret = NULL;
	struct mp200_udc	*pudc;
	struct mp200_req	*preq = NULL;
	int			status;
	u8			ep_adrs;
	unsigned long		flags;

	if(puep == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return -EINVAL;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	spin_lock_irqsave(&pudc->lock, flags);

	if(pep->pdesc == NULL){
		spin_unlock_irqrestore(&pudc->lock, flags);
		return 0;
	}

	ep_adrs = pep->pdesc->bEndpointAddress;

	status = mp200st_udc_get_ep_req_actual_length(pudc,
							ep_adrs,
							DTD_MANAGEMENT_USE, /* not change */
							&pep_ret,
							&preq);

	if(pep_ret == NULL){
		spin_unlock_irqrestore(&pudc->lock, flags);
		return 0;
	}

	/* not match */
	if(pep != pep_ret){
		spin_unlock_irqrestore(&pudc->lock, flags);
		ERR("%s mp200st_udc_get_ep_req_actual_length  pep(%08x)!=pep_ret(%08x)\n",__FUNCTION__,(int)pep,(int)pep_ret);
		return -EINVAL;
	}

	/* error */
	if((status < 0) || (preq == NULL)){
		spin_unlock_irqrestore(&pudc->lock, flags);
		ERR("%s status = %d preq(%08x)\n",__FUNCTION__, status,(int)preq);
		return status;
	}

	spin_unlock_irqrestore(&pudc->lock, flags);

	return preq->req.actual;
}

/*-------------------------------------------------------------------------*/
static void
 mp200_udcep_fifo_flush(struct usb_ep *puep)
{
	struct mp200_ep		*pep;
	struct mp200_udc	*pudc;
	unsigned long	flags;

	if(puep == NULL){
		ERR("%s, bad param\n", __FUNCTION__);
		return;
	}

	pep = container_of(puep, struct mp200_ep, ep);

	/* udc */
	pudc = pep->pudc;

	spin_lock_irqsave(&pudc->lock, flags);

	if(pep == &pudc->ep[0] || (pep->pdesc == NULL) || !list_empty(&pep->queue)){
		spin_unlock_irqrestore(&pudc->lock, flags);
		DBG("%s, bad ep\n", __FUNCTION__);
		return;
	}

	/* FLUSH */
	mp200st_udc_delete_dTD(pudc, pep->pdesc->bEndpointAddress);

	spin_unlock_irqrestore(&pudc->lock, flags);

	return;
}


#ifdef CONFIG_USB_NWM_MODULE
/*-------------------------------------------------------------------------*/
static int
 mp200_udcep_ep_clear_stall(struct usb_ep *puep)
{
	int		nret;

	nret = mp200_udcep_ep_set_halt(puep, FALSE);

	return nret;
}
#endif	// CONFIG_USB_NWM_MODULE


/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_enable_endpoint(struct mp200_udc *pudc,u8 ep_adrs)
{
	u32		data;
	u32		epnum;

	epnum = (ep_adrs & USB_ENDPOINT_NUMBER_MASK);

	if(ep_adrs & USB_DIR_IN){
		data = ENDPTCTRL_TXE | ENDPTCTRL_TXR;
	}
	else{
		data = ENDPTCTRL_RXE | ENDPTCTRL_RXR;
	}

	mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->ENDPTCTRL[epnum], data);

	return;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_set_ep_ctrlreg(
	struct mp200_udc *pudc,
	u8 ep_adrs,
	u8 bm_attributes)
{
	u32		data;
	u32		epnum;
	u32		type;

	epnum = (ep_adrs & USB_ENDPOINT_NUMBER_MASK);
	data = mp200st_udc_readl(&pudc->pfc_reg->ENDPTCTRL[epnum]);

	type = (bm_attributes & USB_ENDPOINT_XFERTYPE_MASK);

	if(ep_adrs & USB_DIR_IN){
		if(data & ENDPTCTRL_RXT){
			data = (type << ENDPTCTRL_TXT_SHIFT);
		}else{
			data = (type << ENDPTCTRL_TXT_SHIFT) | (type << ENDPTCTRL_RXT_SHIFT);
		}
	}else{
		if(data & ENDPTCTRL_TXT){
			data = (type << ENDPTCTRL_RXT_SHIFT);
		}else{
			data = (type << ENDPTCTRL_TXT_SHIFT) | (type << ENDPTCTRL_RXT_SHIFT);
		}
	}

	mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->ENDPTCTRL[epnum], data);
	mp200st_udc_enable_endpoint(pudc, ep_adrs);

	return;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_create_CBI_dTD(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	struct mp200_req *preq,
	PT_MEMORY_VP_INFO pdTD_mem,
	u8 ep_adrs)
{
	int			i, k;
	u8			direct, epnum;
	bool			bdTD_mem_set = FALSE;
	u32			data_len, len_tmp;
	u32			buffer_addr, offset_addr;
	PT_DTD_MANAGEMENT	p_dTD_m;
	PT_EP_TRANS_DESC	p_dTD, p_tmp_dTD;

	/* dTD */
	p_tmp_dTD = NULL;
	epnum		= ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	direct		= (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;
#ifdef USB_TRANS_DIVISION
	if(preq->division_len != 0){
		data_len = min(preq->req.length - preq->act_len, preq->division_len);
	}
	else{
		data_len	= preq->req.length;
	}

	if(data_len == 0){
		buffer_addr = preq->req.dma & DTD_BUFFER_PTR;
		offset_addr = preq->req.dma & DTD_CURR_OFFSET;
	}
	else{
		buffer_addr = (preq->req.dma + preq->act_len) & DTD_BUFFER_PTR;
		offset_addr = (preq->req.dma + preq->act_len) & DTD_CURR_OFFSET;
	}
#else /* USB_TRANS_DIVISION */
	data_len	= preq->req.length;
	buffer_addr = preq->req.dma & DTD_BUFFER_PTR;
	offset_addr = preq->req.dma & DTD_CURR_OFFSET;
#endif /* USB_TRANS_DIVISION */

	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		if(p_dTD_m->use_flag == 0){

			p_dTD_m->use_flag  = DTD_MANAGEMENT_RESERVE;
			p_dTD_m->p_context = pep;

			if(bdTD_mem_set == FALSE){
				*pdTD_mem = p_dTD_m->mem_info;
				bdTD_mem_set = TRUE;
			}

			/* clear dTD memory */
			p_dTD = (PT_EP_TRANS_DESC)p_dTD_m->mem_info.p_vadrs;
			memset(p_dTD, 0, sizeof(T_EP_TRANS_DESC));

			/* Data length */
			if(data_len > DTD_MAX_TRANSFER_SIZE){
				len_tmp = DTD_MAX_TRANSFER_SIZE;
			}else{
				len_tmp = data_len;
			}
			data_len -= len_tmp;

			/* Terminal */
			p_dTD->next_ptr = DTD_NEXT_TERMINATE;

			/* Total Byte */
			p_dTD->total_bytes = ((len_tmp << DTD_LENGTH_BIT_POS) | DTD_STATUS_ACTIVE);

			/* Buffer Pointer & Offset */
			p_dTD->buf_ptr[0] = buffer_addr + offset_addr;
			for(i=1; i<5; i++){
				buffer_addr += 0x1000;
				p_dTD->buf_ptr[i] = buffer_addr;
			}

			/* Next Link Pointer */
			if(p_tmp_dTD != NULL){
				p_tmp_dTD->next_ptr = (u32)p_dTD_m->mem_info.padrs;
			}
			p_tmp_dTD = p_dTD;

			if(data_len == 0){
				p_dTD->total_bytes |= DTD_IOC;	/* Interrupt */
				break;
			}
		}
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_create_ISO_dTD(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	struct mp200_req *preq,
	PT_MEMORY_VP_INFO pdTD_mem,
	u8 ep_adrs)
{
	int			i, k;
	u8			direct, epnum;
	bool			bdTD_mem_set = FALSE;
	u32			data_len, len_tmp;
	u32			buffer_addr, offset_addr;
	PT_DTD_MANAGEMENT	p_dTD_m;
	PT_EP_TRANS_DESC	p_dTD, p_tmp_dTD;

	/* dTD */
	p_tmp_dTD = NULL;
	epnum		= ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	direct		= (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;

	data_len	= preq->req.length;
	buffer_addr = preq->req.dma & DTD_BUFFER_PTR;
	offset_addr = preq->req.dma & DTD_CURR_OFFSET;

	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		if(p_dTD_m->use_flag == 0){

			p_dTD_m->use_flag  = DTD_MANAGEMENT_RESERVE;
			p_dTD_m->p_context = pep;

			if(bdTD_mem_set == FALSE){
				*pdTD_mem = p_dTD_m->mem_info;
				bdTD_mem_set = TRUE;
			}

			/* clear dTD memory */
			p_dTD = (PT_EP_TRANS_DESC)p_dTD_m->mem_info.p_vadrs;
			memset(p_dTD, 0, sizeof(T_EP_TRANS_DESC));

			/* Data length */
			if(data_len > DTD_MAX_TRANSFER_SIZE){
				len_tmp = DTD_MAX_TRANSFER_SIZE;
			}else{
				len_tmp = data_len;
			}
			data_len -= len_tmp;

			/* Terminal */
			p_dTD->next_ptr = DTD_NEXT_TERMINATE;

			/* Total Byte */
			p_dTD->total_bytes = ((len_tmp << DTD_LENGTH_BIT_POS) | DTD_STATUS_ACTIVE);

			/* Buffer Pointer & Offset */
			p_dTD->buf_ptr[0] = buffer_addr + offset_addr;
			for(i=1; i<5; i++){
				buffer_addr += 0x1000;
				p_dTD->buf_ptr[i] = buffer_addr;
			}

			/* Next Link Pointer */
			if(p_tmp_dTD != NULL){
				p_tmp_dTD->next_ptr = (u32)p_dTD_m->mem_info.padrs;
			}
			p_tmp_dTD = p_dTD;

			if(data_len == 0){
				p_dTD->total_bytes |= DTD_IOC;	/* Interrupt */
				break;
			}
		}
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_create_dTD(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	struct mp200_req *preq,
	PT_MEMORY_VP_INFO pdTD_mem,
	u8 ep_adrs)
{
	int			k;
	u8			direct, epnum;
	u16			dTD_num=0;
	u32			data_len;
	PT_DTD_MANAGEMENT	p_dTD_m;

	/* search empty dtd num */
	epnum		= ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	direct		= (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;
	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		if(p_dTD_m->use_flag == 0) dTD_num++;
	}

	/* check transfer size */
	data_len = dTD_num * DTD_MAX_TRANSFER_SIZE;
	if(preq->req.length > data_len){
		return -EINVAL;
	}

	if((pep != &pudc->ep[0])
	   && (pep->pdesc != NULL)
	   && ((pep->pdesc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_ISOC)){

		/* Isochronous */
		mp200st_udc_create_ISO_dTD(pudc, pep, preq, pdTD_mem, ep_adrs);
	}else{
		/* Control Bulk Interrupt */
		mp200st_udc_create_CBI_dTD(pudc, pep, preq, pdTD_mem, ep_adrs);
	}

	return 0;
}

#define DEL_DTD_REPEATED_WCOUNT	10
#define DEL_DTD_REPEATED_RCOUNT	1000
/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_delete_dTD(struct mp200_udc *pudc, u8 ep_adrs)
{
	u32	bit_pos;
	u32	data;
	int	i,j;

	bit_pos = ENDPT_BIT_POS(ep_adrs);
	for(i=0; i<DEL_DTD_REPEATED_WCOUNT; i++){
		mp200st_udc_writel(&pudc->pfc_reg->ENDPTFLUSH, bit_pos);
		for(j=0; j<DEL_DTD_REPEATED_RCOUNT; j++){
			data = mp200st_udc_readl(&pudc->pfc_reg->ENDPTFLUSH);
			if(data == 0){
				break;
			}
		}

		data = mp200st_udc_readl(&pudc->pfc_reg->ENDPTSTATUS) & bit_pos;
		if(data == 0){
			break;
		}
	}

	if(i >= DEL_DTD_REPEATED_WCOUNT){
		DBG("%s fail\n",__FUNCTION__);
	}

	return;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_cancel_dTD(struct mp200_udc *pudc, u8 ep_adrs)
{
	int			k;
	PT_DTD_MANAGEMENT	p_dTD_m;
	u8			epnum = ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	u8			direct = (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;

	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		if(p_dTD_m->use_flag == DTD_MANAGEMENT_RESERVE){
			p_dTD_m->use_flag = 0;
		}
	}

	return;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_link_dTD(
	struct mp200_udc *pudc,
	struct mp200_req *preq,
	PT_MEMORY_VP_INFO pdTD_mem,
	u8 ep_adrs)
{
	int			k;
	u8			direct, epnum;
	u32			bit_pos;
	u32			data;
	PT_ENDPOINT_QUEUE_HEAD	p_dQH;
	PT_EP_TRANS_DESC	p_dTD;
	PT_DTD_MANAGEMENT	p_dTD_m, p_dTD_m_tails;

	epnum	= ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	direct	= (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;
	bit_pos = ENDPT_BIT_POS(ep_adrs);

	p_dQH = mp200st_udc_ep_get_dQH_adrs(pudc, ep_adrs, FALSE);

	p_dTD_m_tails = NULL;
	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		switch(p_dTD_m->use_flag){
			case DTD_MANAGEMENT_RESERVE:
				p_dTD_m->use_flag = DTD_MANAGEMENT_USE;
				break;

			case DTD_MANAGEMENT_USE:
				p_dTD_m_tails = p_dTD_m;
				break;

			default:
				break;
		}
	}

	if(p_dTD_m_tails == NULL){
		/* not transfer */
#ifdef USB_TRANS_DIVISION

		if(preq->division_len != 0){
			p_dQH->max_length |= EQH_ZLT;
		}else{
			if(preq->req.zero){
				p_dQH->max_length &= ~(EQH_ZLT);
			}else{
				p_dQH->max_length |= EQH_ZLT;
			}
		}

#else /* USB_TRANS_DIVISION */
		if(preq->req.zero){
			p_dQH->max_length &= ~(EQH_ZLT);
		}else{
			p_dQH->max_length |= EQH_ZLT;
		}
#endif /* USB_TRANS_DIVISION */
		p_dQH->next_ptr = (u32)pdTD_mem->padrs;
		p_dQH->total_bytes = 0;

		if(epnum == 0){
			if(mp200st_udc_readl(&pudc->pfc_reg->ENDPTSETUPSTAT) & ENDPTSETUPSTAT_ENDPTSETUPSTAT_EP0){
				DBG("%s Receiving ENDPTSETUPSTAT_EP0 (1)(%08x)\n",__FUNCTION__, bit_pos);
				return 0;
			}
		}
#ifdef DUMMY_PRIME
 #ifdef DUMMY_PRIME_NOT_EP0
		if(epnum == 0){
			mp200st_udc_writel(&pudc->pfc_reg->ENDPTPRIME, bit_pos);
			if(bit_pos){
				DBG_LOG(pudc,"PRIME1(%08x)\n",bit_pos);
			}
		}else{
			mp200st_udc_set_dummy_prime_waiting(pudc, bit_pos);
		}
 #else /* DUMMY_PRIME_NOT_EP0 */
		mp200st_udc_set_dummy_prime_waiting(pudc, bit_pos);
 #endif /* DUMMY_PRIME_NOT_EP0 */
#else /* DUMMY_PRIME */
		mp200st_udc_writel(&pudc->pfc_reg->ENDPTPRIME, bit_pos);
		if(bit_pos){
			DBG_LOG(pudc,"PRIME2(%08x)\n",bit_pos);
		}
#endif /* DUMMY_PRIME */

		return 0;
	}
	else{
		/* transfer */
		p_dTD = (PT_EP_TRANS_DESC)p_dTD_m_tails->mem_info.p_vadrs;
		p_dTD->next_ptr = pdTD_mem->padrs;

		data = mp200st_udc_readl(&pudc->pfc_reg->ENDPTPRIME);
		if(data & bit_pos){
			return 0;
		}

		mp200st_udc_set_bit(&pudc->pfc_reg->USBCMD, USBCMD_ATDTW);
		data = mp200st_udc_readl(&pudc->pfc_reg->ENDPTSTATUS);
		mp200st_udc_clear_bit(&pudc->pfc_reg->USBCMD, USBCMD_ATDTW);

		if(data & bit_pos){
			return 0;

		}else{
#ifdef USB_TRANS_DIVISION

			if(preq->division_len != 0){
				p_dQH->max_length |= EQH_ZLT;
			}else{
				if(preq->req.zero){
					p_dQH->max_length &= ~(EQH_ZLT);
				}else{
					p_dQH->max_length |= EQH_ZLT;
				}
			}

#else /* USB_TRANS_DIVISION */
			if(preq->req.zero){
				p_dQH->max_length &= ~(EQH_ZLT);
			}else{
				p_dQH->max_length |= EQH_ZLT;
			}
#endif /* USB_TRANS_DIVISION */
			p_dQH->next_ptr = (u32)pdTD_mem->padrs;
			p_dQH->total_bytes = 0;

			if(epnum == 0){
				if(mp200st_udc_readl(&pudc->pfc_reg->ENDPTSETUPSTAT) & ENDPTSETUPSTAT_ENDPTSETUPSTAT_EP0){
					DBG("%s Receiving ENDPTSETUPSTAT_EP0 (2)(%08x)\n",__FUNCTION__, bit_pos);
					return 0;
				}
			}
#ifdef DUMMY_PRIME
 #ifdef DUMMY_PRIME_NOT_EP0
			if(epnum == 0){
				mp200st_udc_writel(&pudc->pfc_reg->ENDPTPRIME, bit_pos);
				if(bit_pos){
					DBG_LOG(pudc,"PRIME3(%08x)\n",bit_pos);
				}
			}else{
				mp200st_udc_set_dummy_prime_waiting(pudc, bit_pos);
			}
 #else /* DUMMY_PRIME_NOT_EP0 */
			mp200st_udc_set_dummy_prime_waiting(pudc, bit_pos);
 #endif /* DUMMY_PRIME_NOT_EP0 */
#else /* DUMMY_PRIME */
			mp200st_udc_writel(&pudc->pfc_reg->ENDPTPRIME, bit_pos);
			if(bit_pos){
				DBG_LOG(pudc,"PRIME4(%08x)\n",bit_pos);
			}
#endif /* DUMMY_PRIME */

			return 0;
		}
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_unlink_dTD(struct mp200_udc *pudc, u8 ep_adrs)
{
	int			k;
	PT_DTD_MANAGEMENT	p_dTD_m;
	PT_EP_TRANS_DESC	p_dTD;
	u8			epnum = ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	u8			direct = (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;
#ifdef DUMMY_PRIME
 #ifdef DUMMY_PRIME_NAK_INT
	u32			epnaken;
 #endif /* DUMMY_PRIME_NAK_INT */
#endif/* DUMMY_PRIME */

#ifdef DUMMY_PRIME
	pudc->dummy_prime_transferring &= ~(ENDPT_BIT_POS(ep_adrs));
	pudc->dummy_prime_waiting &= ~(ENDPT_BIT_POS(ep_adrs));
 #ifdef DUMMY_PRIME_NAK_INT
	epnaken = mp200st_udc_readl(&pudc->pfc_reg->ENDPTNAKEN);

	if((pudc->dummy_prime_transferring == 0)
	  &&(epnaken != pudc->dummy_prime_waiting)){
		mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAKEN, pudc->dummy_prime_waiting);
	}

 #endif /* DUMMY_PRIME_NAK_INT */
#endif/* DUMMY_PRIME */

	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		p_dTD_m->use_flag = 0;
		p_dTD = (PT_EP_TRANS_DESC)p_dTD_m->mem_info.p_vadrs;
		p_dTD->total_bytes = 0;
		p_dTD->next_ptr = DTD_NEXT_TERMINATE;
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_done_transfer(struct mp200_udc *pudc, u8 ep_adrs)
{
	struct mp200_ep		*pep = NULL;
	struct mp200_req	*preq = NULL;
	int			status;

	status = mp200st_udc_get_ep_req_actual_length(pudc,
							ep_adrs,
							DTD_MANAGEMENT_FREE,
							&pep,
							&preq);

	if(status < 0){
		if((ep_adrs & USB_ENDPOINT_NUMBER_MASK) == 0){
			return 0;
		}
	}

	mp200st_udc_done_callback(pudc, pep, preq, status, TRUE);

	return status;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_set_start_dTD(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	struct mp200_req *preq,
	u8 ep_adrs)
{
	T_MEMORY_VP_INFO	dTD_mem;
	int			status;

	dTD_mem.p_vadrs = NULL;
	dTD_mem.padrs = DMA_ADDR_INVALID;

	status = mp200st_udc_create_dTD(pudc, pep, preq, &dTD_mem, ep_adrs);
	if(status < 0){
		mp200st_udc_cancel_dTD(pudc, ep_adrs);
		ERR("%s fail ep=%02x\n",__FUNCTION__,ep_adrs);
		return status;
	}

	status = mp200st_udc_link_dTD(pudc, preq, &dTD_mem, ep_adrs);

	return status;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udcint_dummy_EP0_complete(struct usb_ep *puep, struct usb_request *pureq)
{
	return ;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_submit_dTD(struct mp200_udc *pudc, struct mp200_ep *pep)
{
	int			status = -EINVAL;
	struct mp200_req	*preq;
	u8			ep_adrs;

	if(list_empty(&pep->queue)){
		ERR("%s list_empty\n",__FUNCTION__);
		return status;
	}

	if(pudc->vbus_active == 0){
		DBG("%s VBUS OFF\n",__FUNCTION__);
		return -ESHUTDOWN;
	}

	preq = list_entry(pep->queue.next, struct mp200_req, queue);

	/* check EP0 */
	if(pep != &pudc->ep[0]){
		/* not EP0 */
		ep_adrs = pep->pdesc->bEndpointAddress;

		/* not DMA mem */
		if((preq->req.dma == DMA_ADDR_INVALID) || (preq->req.dma == 0)){
			if(preq->req.length > UDC_EP_AUTO_BUFSIZE){
				ERR("%s length size over! ep=%02x size=%d\n",__FUNCTION__,ep_adrs,preq->req.length);
				return -EINVAL;
			}
			preq->req.dma = pep->mem.padrs;
			if(ep_adrs & USB_DIR_IN){
				memcpy(pep->mem.p_vadrs, preq->req.buf, preq->req.length);
			}
		}

		status = mp200st_udc_set_start_dTD(pudc, pep, preq, ep_adrs);
		if(status < 0){
			DBG("%s data in ERR\n",__FUNCTION__);
		}
	}else{
		/* make dummy req */
		memset(&pudc->null_packet_req, 0, sizeof(struct mp200_req));
		pudc->null_packet_req.req.buf		= NULL;
		pudc->null_packet_req.req.length	= 0;
		pudc->null_packet_req.req.dma		= DMA_ADDR_INVALID;
		pudc->null_packet_req.req.zero	= FALSE;
		pudc->null_packet_req.req.complete	= mp200st_udcint_dummy_EP0_complete;
		pudc->null_packet_req.req.status	= -EINPROGRESS;

		if(pudc->ctrl.bRequestType & USB_DIR_IN){
			/* data in -> status out */
			ep_adrs = (0 | USB_DIR_IN);
			/* not DMA mem */
			if((preq->req.dma == DMA_ADDR_INVALID) || (preq->req.dma == 0)){
				if(UDC_EP_AUTO_BUFSIZE < preq->req.length){
					ERR("%s length size over! ep=%02x size=%d\n",__FUNCTION__,ep_adrs,preq->req.length);
					return -EINVAL;
				}
				preq->req.dma = pep->mem.padrs;
				memcpy(pep->mem.p_vadrs, preq->req.buf, preq->req.length);
			}
			status = mp200st_udc_set_start_dTD(pudc, pep, preq, (0 | USB_DIR_IN));
			if(status < 0){
				DBG("%s data in ERR\n",__FUNCTION__);
			}
			status = mp200st_udc_set_start_dTD(pudc, pep, &pudc->null_packet_req, (0 | USB_DIR_OUT));
			if(status < 0){
				DBG("%s data in -> status out ERR\n",__FUNCTION__);
			}
		}else{
			if(pudc->ctrl.wLength){
				ep_adrs = (0 | USB_DIR_OUT);
				/* not DMA mem */
				if((preq->req.dma == DMA_ADDR_INVALID) || (preq->req.dma == 0)){
					if(UDC_EP_AUTO_BUFSIZE < preq->req.length){
						ERR("%s length size over! ep=%02x size=%d\n",__FUNCTION__,ep_adrs,preq->req.length);
						return -EINVAL;
					}
					preq->req.dma = pep->mem.padrs;
				}
				status = mp200st_udc_set_start_dTD(pudc, pep, preq, (0 | USB_DIR_OUT));
				if(status < 0){
					DBG("%s data out ERR\n",__FUNCTION__);
				}
				status = mp200st_udc_set_start_dTD(pudc, pep, &pudc->null_packet_req, (0 | USB_DIR_IN));
				if(status < 0){
					DBG("%s data out -> status in ERR\n",__FUNCTION__);
				}
			}else{
				/* status_in */
				status = mp200st_udc_set_start_dTD(pudc, pep, preq, (0 | USB_DIR_IN));
				if(status < 0){
					DBG("%s status_in ERR\n",__FUNCTION__);
				}
			}
		}
	}

	return status;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_abort_dTD(struct mp200_udc *pudc, u8 ep_adrs)
{
	/* check comp */
	if(mp200st_udc_readl(&pudc->pfc_reg->ENDPTCOMPLETE) & ENDPT_BIT_POS(ep_adrs)){
		return -EBUSY;
	}

	/* flush */
	mp200st_udc_delete_dTD(pudc, ep_adrs);

	/* check comp */
	if(mp200st_udc_readl(&pudc->pfc_reg->ENDPTCOMPLETE) & ENDPT_BIT_POS(ep_adrs)){
		return -EBUSY;
	}

	/* unlink */
	mp200st_udc_unlink_dTD(pudc, ep_adrs);

	return 0;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_done_callback(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	struct mp200_req *preq,
	int comp_status,
	bool bstart_next)
{
	bool	bdel_queue_head_next;
	int	status;
	__u8	usb_dir_in;

	if(!pudc || !pep || !preq){
		DBG("%s -EINVAL (%08x,%08x,%08x)\n",__FUNCTION__,(int)pudc,(int)pep,(int)preq);
		return;
	}

	/* finish DMA */
	usb_dir_in = (pep->pdesc != NULL) /* not EP0 */
		? (pep->pdesc->bEndpointAddress & USB_DIR_IN)
		: (pudc->ctrl.bRequestType & USB_DIR_IN);

	if(preq->req.dma != DMA_ADDR_INVALID) {
		if (preq->mapped) {
			dma_unmap_single(pudc->gadget.dev.parent,
					 preq->req.dma,
					 preq->req.length,
					 usb_dir_in
					 ? DMA_TO_DEVICE
					 : DMA_FROM_DEVICE);
			/* preq->req.dma = DMA_ADDR_INVALID; */
			/* preq->mapped = 0; */
		} else {
			dma_sync_single_for_cpu(pudc->gadget.dev.parent,
						preq->req.dma,
						preq->req.length,
						usb_dir_in
						? DMA_TO_DEVICE
						: DMA_FROM_DEVICE);
		}
	}

	/* check NULL empty */
	if(list_empty(&pep->queue)){
		goto end;
	}

	/* check queue head element */
	bdel_queue_head_next = (pep->queue.next == &preq->queue);

#ifdef USB_TRANS_DIVISION

	/* set status */
	if(preq->req.status == -EINPROGRESS){
		preq->req.status = comp_status;
	}else{
		DBG("req.status != -EINPROGRESS (%d)\n",preq->req.status);
	}

	if(comp_status == 0){

		/* auto DMA */
		if(preq->req.dma == pep->mem.padrs){
			if(pep->pdesc != NULL){
				if(!(pep->pdesc->bEndpointAddress & USB_DIR_IN)){
					if(preq->division_len != 0){
						memcpy(preq->req.buf + preq->act_len, ((u8 *)pep->mem.p_vadrs) + preq->act_len, preq->req.actual);
					}
					else{
						memcpy(preq->req.buf, pep->mem.p_vadrs, preq->req.actual);
					}
				}
			}else{
				if(!(pudc->ctrl.bRequestType & USB_DIR_IN)){
					memcpy(preq->req.buf, pep->mem.p_vadrs, preq->req.actual);
				}
			}
			preq->req.dma = DMA_ADDR_INVALID;
		}

		preq->act_len += preq->req.actual;
	}

	{
		bool	bEnd=TRUE;

		if((preq->division_len != 0) && pep != &pudc->ep[0]){
			if(preq->req.actual != 0){
				if(preq->act_len == preq->req.length){
					if((preq->req.zero != 0) && ((preq->req.actual % pep->pdesc->wMaxPacketSize) == 0)){
						preq->req.zero = 0;	/* Zero Length Packet Flag Clear*/
						bEnd = FALSE;		/* Zero Length Packet Restart */
					}
				}
				else{
					if((preq->division_len == preq->req.actual)){
						bEnd = FALSE;		/* MaxPacketSize Restart */
					}

				}
			}
		}

		if(bEnd == FALSE){
			preq->req.status = -EINPROGRESS;
			mp200st_udc_submit_dTD(pudc, pep);		/* Restart */
		}
		else {
			/* list del */
			list_del_init(&preq->queue);
 #ifdef DEBUG_COUNT
			pep->done_count++;
 #endif /* DEBUG_COUNT */

			/* next start */
			if(!list_empty(&pep->queue) && bstart_next && bdel_queue_head_next){
				status = mp200st_udc_submit_dTD(pudc, pep);
				if(status < 0){
					DBG("next start fail\n");
				}
			}

 #ifdef DEBUG_COUNT
			pep->done_callback_count++;
			if(preq->req.status == 0){
				pep->done_callback_ok_count++;
			}else if(preq->req.status == -ECONNRESET){
				pep->done_callback_reset_count++;
			}else{
				pep->done_callback_fail_count++;
			}
 #endif /* DEBUG_COUNT */

			preq->req.actual = preq->act_len;

			/* don't modify queue heads during completion callback */
			spin_unlock(&pudc->lock);
			preq->req.complete(&pep->ep, &preq->req);
			spin_lock(&pudc->lock);
		}
	}

#else /* USB_TRANS_DIVISION */

	/* list del */
	list_del_init(&preq->queue);
 #ifdef DEBUG_COUNT
	pep->done_count++;
 #endif /* DEBUG_COUNT */

	/* set status */
	if(preq->req.status == -EINPROGRESS){
		preq->req.status = comp_status;
	}else{
		DBG("req.status != -EINPROGRESS (%d)\n",preq->req.status);
	}

	/* auto DMA */
	if(preq->req.dma == pep->mem.padrs){
		if(pep->pdesc != NULL){
			if(!(pep->pdesc->bEndpointAddress & USB_DIR_IN)){
				memcpy(preq->req.buf, pep->mem.p_vadrs, preq->req.actual);
			}
		}else{
			if(!(pudc->ctrl.bRequestType & USB_DIR_IN)){
				memcpy(preq->req.buf, pep->mem.p_vadrs, preq->req.actual);
			}
		}
		preq->req.dma = DMA_ADDR_INVALID;
	}

 #ifdef DEBUG_COUNT
	pep->done_callback_count++;
	if(preq->req.status == 0){
		pep->done_callback_ok_count++;
	}else if(preq->req.status == -ECONNRESET){
		pep->done_callback_reset_count++;
	}else{
		pep->done_callback_fail_count++;
	}
 #endif /* DEBUG_COUNT */

	/* next start */
	if(!list_empty(&pep->queue) && bstart_next && bdel_queue_head_next){
		status = mp200st_udc_submit_dTD(pudc, pep);
		if(status < 0){
			DBG("next start fail\n");
		}
	}

	/* don't modify queue heads during completion callback */
	spin_unlock(&pudc->lock);
	preq->req.complete(&pep->ep, &preq->req);
	spin_lock(&pudc->lock);
#endif	/* USB_TRANS_DIVISION */

end:
	return;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_nuke(struct mp200_udc *pudc, struct mp200_ep *pep, int status)
{
	struct mp200_req	*preq;

	if(pep->pdesc != NULL){
		mp200st_udc_abort_dTD(pudc, pep->pdesc->bEndpointAddress);
	}else{
		mp200st_udc_abort_dTD(pudc, (0 | USB_DIR_OUT));
		mp200st_udc_abort_dTD(pudc, (0 | USB_DIR_IN));
	}

	while(!list_empty(&pep->queue)){
		preq = list_entry(pep->queue.next, struct mp200_req, queue);
		mp200st_udc_done_callback(pudc, pep, preq, status, FALSE);
	}

	return;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_quiesce(struct mp200_udc *pudc)
{
	struct mp200_ep		*pep;

	pudc->gadget.speed = USB_SPEED_UNKNOWN;

	DBG_LOG(pudc,"UDC QUIESCE NUKE\n");
	/* Endpoint0 */
	mp200st_udc_nuke(pudc, &pudc->ep[0], -ESHUTDOWN);

	/* Endpoint1-15 */
	list_for_each_entry(pep, &pudc->gadget.ep_list, ep.ep_list){
		mp200st_udc_nuke(pudc, pep, -ESHUTDOWN);
	}

	return;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_list_add_and_submit_dTD(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	struct mp200_req *preq)
{
	bool	b_able_transfer;
	int	status = -EBUSY;

	/* check able_transfer */
	b_able_transfer = list_empty(&pep->queue);

	/* maybe kickstart non-iso i/o queues */
	list_add_tail(&preq->queue, &pep->queue);
#ifdef DEBUG_COUNT
	pep->start_count++;
#endif /* DEBUG_COUNT */

	if(b_able_transfer){
		status = mp200st_udc_submit_dTD(pudc, pep);
	}

	return status;
}

/*-------------------------------------------------------------------------*/
static int
 mp200st_udc_get_ep_req_actual_length(
	struct mp200_udc *pudc,
	u8 ep_adrs,
	u32 use_flag,
	struct mp200_ep **pep_ret,
	struct mp200_req **preq_ret)
{
	int			k;
	int			done_num=0;
	u32			status_data=0;
	u32			data_len=0;
	PT_DTD_MANAGEMENT	p_dTD_m;
	PT_EP_TRANS_DESC	p_dTD;
	u8			epnum = ep_adrs & USB_ENDPOINT_NUMBER_MASK;
	u8			direct = (ep_adrs & USB_ENDPOINT_DIR_MASK) >> UDC_DIRECTION_SHIFT;
	int			status = 0;

	for(k=0; k<SUPPORT_DTD_NUM; k++){
		p_dTD_m = &pudc->dTD_Manage[direct][epnum][k];
		if(p_dTD_m->use_flag == DTD_MANAGEMENT_USE){
			p_dTD = (PT_EP_TRANS_DESC)p_dTD_m->mem_info.p_vadrs;
			status_data |= (p_dTD->total_bytes & DTD_STATUS);
			data_len += ((p_dTD->total_bytes & DTD_TOTAL_BYTES) >> DTD_LENGTH_BIT_POS);
			done_num++;
			*pep_ret = (struct mp200_ep *)p_dTD_m->p_context;
			if(list_empty(&(*pep_ret)->queue)){
				*preq_ret = NULL;
			}else{
				*preq_ret = list_entry((*pep_ret)->queue.next, struct mp200_req, queue);
			}
			p_dTD_m->use_flag = use_flag;
		}
	}

	/* check err */
	if(done_num == 0){
		/*	transfer ABORT */
		DBG("%s (done_num == 0) -ECONNABORTED ep=%02x\n",__FUNCTION__,ep_adrs);
		DBG_LOG(pudc, "(done_num == 0) ep=%02x\n",ep_adrs);
		return -ECONNABORTED;
	}else if(*preq_ret == NULL){
		if(epnum == 0){
			return -EINVAL;
		}
		ERR("%s (*preq_ret == NULL) -EINVAL ep=%02x\n",__FUNCTION__,ep_adrs);
		return -EINVAL;
	}


	/* set actual length and status */
	if(status_data == 0){
#ifdef USB_TRANS_DIVISION
		if((*preq_ret)->division_len != 0){
			u32		len;

			len = min((*preq_ret)->division_len, (*preq_ret)->req.length - (*preq_ret)->act_len);
			(*preq_ret)->req.actual = len - data_len;
		}
		else{
			(*preq_ret)->req.actual = (*preq_ret)->req.length - data_len;
		}
#else /* USB_TRANS_DIVISION */
		(*preq_ret)->req.actual = (*preq_ret)->req.length - data_len;
#endif /* USB_TRANS_DIVISION */
	}else{
		if(!(status_data & (DTD_STATUS_DATA_BUFFER_ERR | DTD_STATUS_TRANSACION_ERR))){
			/* ABORT */
			DBG("%s ABORT -ECONNABORTED ep=%02x\n",__FUNCTION__,ep_adrs);
			status = -ECONNABORTED;
		}else if(status_data & DTD_STATUS_DATA_BUFFER_ERR){
			/* DATA_BUFFER_ERR */
			ERR("%s DATA_BUFFER_ERR -EOVERFLOW ep=%02x\n",__FUNCTION__,ep_adrs);
			status = -EOVERFLOW;
		}else if(status_data & DTD_STATUS_TRANSACION_ERR){
			/* TRANSACION_ERR */
			ERR("%s TRANSACION_ERR -EIO ep=%02x\n",__FUNCTION__,ep_adrs);
			status = -EIO;
		}else{
			/* UNKNOWN */
			ERR("%s UNKNOWN -EBADMSG ep=%02x\n",__FUNCTION__,ep_adrs);
			status = -EBADMSG;
		}
	}

	return status;
}

/*-------------------------------------------------------------------------*/
/* ULPI */
#define ULPI_VID_READ_L		0x00	/* VendorID Low */
#define ULPI_VID_READ_H		0x01	/* VendorID High */
#define ULPI_PID_READ_L		0x02	/* ProductID Low */
#define ULPI_PID_READ_H		0x03	/* ProductID High */
#define ULPI_VID_WRITE_L	0x30	/* VendorID Low */
#define ULPI_VID_WRITE_H	0x31	/* VendorID High */
#define ULPI_PID_WRITE_L	0x32	/* ProductID Low */
#define ULPI_PID_WRITE_H	0x33	/* ProductID High */
/*-------------------------------------------------------------------------*/
static int
 mp200st_udcprc_proc_udc_show(struct seq_file *s, void *pram)
{
	struct mp200_udc	*pudc = &g_udc;
	unsigned long	flags;
	u8	ndata;
	unsigned char tmp[100];
	u32 reg_data;
	int i;

	if(pudc == NULL){
		ERR("%s pudc == NULL\n",__FUNCTION__);
		return -ENODEV;
	}

	spin_lock_irqsave(&pudc->lock, flags); /* DI */

	/* USB REG dump */
	if(mp200st_udc_can_pullup(pudc)){
		spin_unlock_irqrestore(&pudc->lock, flags); /* EI */
		seq_printf(s, "\n-USB REG-\n" );
		for(i=0x80 ; i<0x200 ; i+=16){
			reg_data = mp200st_udc_readl((u32 *)IO_ADDRESS(MP200_USB_BASE + i));
			sprintf( tmp, "USB%04x =%08x", i, (int)reg_data );
			seq_printf(s, tmp );

			reg_data = mp200st_udc_readl((u32 *)IO_ADDRESS(MP200_USB_BASE + i +4));
			sprintf( tmp, " %08x", (int)reg_data );
			seq_printf(s, tmp );

			reg_data = mp200st_udc_readl((u32 *)IO_ADDRESS(MP200_USB_BASE + i +8));
			sprintf( tmp, " %08x", (int)reg_data );
			seq_printf(s, tmp );

			reg_data = mp200st_udc_readl((u32 *)IO_ADDRESS(MP200_USB_BASE + i +12));
			sprintf( tmp, " %08x\n", (int)reg_data );
			seq_printf(s, tmp );

		}

		/* ULPI */
		seq_printf(s, "\n-ULPI REG-\n" );

		ndata = mp200st_udc_ulpi_read(pudc, ULPI_VID_READ_L);
		sprintf( tmp, "VID_L = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, ULPI_VID_READ_H);
		sprintf( tmp, "VID_H = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, ULPI_PID_READ_L);
		sprintf( tmp, "PID_L = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, ULPI_PID_READ_H);
		sprintf( tmp, "PID_H = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, 0x04);
		sprintf( tmp, "ULPI 0x04 = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, 0x07);
		sprintf( tmp, "ULPI 0x07 = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, 0x0F);
		sprintf( tmp, "ULPI 0x0F = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, 0x12);
		sprintf( tmp, "ULPI 0x12 = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, 0x15);
		sprintf( tmp, "ULPI 0x15 = %02x\n", ndata );
		seq_printf(s, tmp );

		ndata = mp200st_udc_ulpi_read(pudc, 0x16);
		sprintf( tmp, "ULPI 0x16 = %02x\n", ndata );
		seq_printf(s, tmp );
	}else{
		spin_unlock_irqrestore(&pudc->lock, flags); /* EI */
		seq_printf(s, "\n-PULLUP OFF no USB REG-\n" );
	}

	{
		u32	*g_dqh_ptr = pudc->dQH.p_vadrs;
		int	g_dqh_len = DQH_MEMORY_SIZE;
		int	ep_count = -1;
		int	direct_count = -1;

		/* DQH dump */
		if(g_dqh_ptr != NULL){
			sprintf( tmp, "\n-DQH size=%d (%08x)-\n",g_dqh_len, (int)g_dqh_ptr);
			seq_printf(s, tmp );
			for(i=0x0 ; i<g_dqh_len ; i+=16){

				if((i % (sizeof(T_ENDPOINT_QUEUE_HEAD) * NUM_DIRECTIONS))  == 0){
					ep_count++;
					direct_count = -1;
				}
				if((i % (sizeof(T_ENDPOINT_QUEUE_HEAD)))  == 0){
					direct_count++;
				}

				if( (*((u32 *)(((u8*)g_dqh_ptr) + i)) & ~(BIT00))
					|| *((u32 *)(((u8*)g_dqh_ptr) + i +4))
					|| *((u32 *)(((u8*)g_dqh_ptr) + i +8))
					|| *((u32 *)(((u8*)g_dqh_ptr) + i +12))){

					if((i % (sizeof(T_ENDPOINT_QUEUE_HEAD)))  == 0){
						sprintf( tmp, "** EP =%d **\n", ep_count );
						seq_printf(s, tmp );
						sprintf( tmp, "** DIRECT =%d **\n", direct_count );
						seq_printf(s, tmp );
					}

					reg_data = *((u32 *)(((u8*)g_dqh_ptr) + i));
					sprintf( tmp, "DQH%04x =%08x", i, (int)reg_data );
					seq_printf(s, tmp );

					reg_data = *((u32 *)(((u8*)g_dqh_ptr) + i +4));
					sprintf( tmp, " %08x", (int)reg_data );
					seq_printf(s, tmp );

					reg_data = *((u32 *)(((u8*)g_dqh_ptr) + i +8));
					sprintf( tmp, " %08x", (int)reg_data );
					seq_printf(s, tmp );

					reg_data = *((u32 *)(((u8*)g_dqh_ptr) + i +12));
					sprintf( tmp, " %08x\n", (int)reg_data );
					seq_printf(s, tmp );
				}

			}
		}else{
			seq_printf(s, "\n--dqh_ptr == NULL no DQH--\n" );
		}
	}

	{
		u32	*g_dtd_ptr = pudc->dTD.p_vadrs;
		int	g_dtd_len = DTD_MEMORY_SIZE;
		int	ep_count = -1;
		int	direct_count = -1;
		int	dtd_count = -1;

		/* DTD dump */
		if(g_dtd_ptr != NULL){
			sprintf( tmp, "\n-DTD size=%d (%08x)-\n",g_dtd_len,(int)g_dtd_ptr);
			seq_printf(s, tmp );
			for(i=0x0 ; i<g_dtd_len ; i+=16){

				if((i % (sizeof(T_EP_TRANS_DESC) * SUPPORT_DTD_NUM * NUM_ENDPOINTS)) == 0){
					direct_count++;
					ep_count = -1;
				}
				if((i % (sizeof(T_EP_TRANS_DESC) * SUPPORT_DTD_NUM)) == 0){
					ep_count++;
					dtd_count = -1;
				}
				if((i % (sizeof(T_EP_TRANS_DESC))) == 0){
					dtd_count++;
				}

				if( (*((u32 *)(((u8*)g_dtd_ptr) + i)) & ~(BIT00))
					|| *((u32 *)(((u8*)g_dtd_ptr) + i +4))
					|| *((u32 *)(((u8*)g_dtd_ptr) + i +8))
					|| *((u32 *)(((u8*)g_dtd_ptr) + i +12))){

					if((i % (sizeof(T_EP_TRANS_DESC) * SUPPORT_DTD_NUM)) == 0){
						sprintf( tmp, "** EP =%d **\n", ep_count );
						seq_printf(s, tmp );
						sprintf( tmp, "** DIRECT =%d **\n", direct_count );
						seq_printf(s, tmp );
					}
					if((i % (sizeof(T_EP_TRANS_DESC))) == 0){
						sprintf( tmp, "** DTD =%d **\n", dtd_count );
						seq_printf(s, tmp );
					}

					reg_data = *((u32 *)(((u8*)g_dtd_ptr) + i));
					sprintf( tmp, "DTD%04x =%08x", i, (int)reg_data );
					seq_printf(s, tmp );

					reg_data = *((u32 *)(((u8*)g_dtd_ptr) + i +4));
					sprintf( tmp, " %08x", (int)reg_data );
					seq_printf(s, tmp );

					reg_data = *((u32 *)(((u8*)g_dtd_ptr) + i +8));
					sprintf( tmp, " %08x", (int)reg_data );
					seq_printf(s, tmp );

					reg_data = *((u32 *)(((u8*)g_dtd_ptr) + i +12));
					sprintf( tmp, " %08x\n", (int)reg_data );
					seq_printf(s, tmp );
				}


			}
		}else{
			seq_printf(s, "\n--dqh_ptr == NULL no DTD--\n" );
		}
	}

#ifdef DEBUG_COUNT
	sprintf( tmp, "\n-EP COMP -\n");
	seq_printf(s, tmp );
	for(i=0 ; i<((NUM_ENDPOINTS)*(NUM_DIRECTIONS) -1) ; i++){
		if((pudc->ep[i].name[0] != 0) && (pudc->ep[i].start_count)){
			sprintf( tmp, " %s\n", pudc->ep[i].name );
			seq_printf(s, tmp );
			sprintf( tmp, "  start=%d (n=%d)  done=%d   comp=%d(ok=%d reset=%d fail=%d)\n",
							 pudc->ep[i].start_count,
							 pudc->ep[i].normal_start_count,
							 pudc->ep[i].done_count,
							 pudc->ep[i].done_callback_count,
							 pudc->ep[i].done_callback_ok_count,
							 pudc->ep[i].done_callback_reset_count,
							 pudc->ep[i].done_callback_fail_count);

			seq_printf(s, tmp );
		}
	}

#endif /* DEBUG_COUNT */

#ifdef DUMMY_PRIME
	sprintf( tmp, "\n dummy_prime_waiting=%08x   dummy_prime_transferring=%08x\n",
					 pudc->dummy_prime_waiting,
					 pudc->dummy_prime_transferring);

	seq_printf(s, tmp );

#endif /* DUMMY_PRIME */

#ifdef DEBUG_LOG
	{
		seq_printf(s, "\ndbg_log[%d]:\n", pudc->dbg_log_count );
		seq_printf(s, pudc->dbg_log );

	}
#endif /* DEBUG_LOG */
#ifdef MEMORY_LOG
	{
		seq_printf(s, "\nmem_log[%d]:\n", pudc->mem_log_count );
		seq_printf(s, pudc->mem_log );

	}
#endif /* MEMORY_LOG */

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcprc_proc_udc_open(struct inode *inode, struct file *file)
{
	return single_open(file, mp200st_udcprc_proc_udc_show, &g_udc);
}

#ifdef DEBUG
/*-------------------------------------------------------------------------*/
static ssize_t
 mp200_udcprc_proc_udc_write(
	struct file *fd,
	const char __user *buf,
	size_t len,
	loff_t *ptr)
{
	struct mp200_udc	*pudc = &g_udc;
#ifdef DEBUG_LOG
	unsigned long	flags;
#endif /* DEBUG_LOG */

	if(pudc == NULL){
		ERR("%s pudc == NULL\n",__FUNCTION__);
		return -ENODEV;
	}

#ifdef DEBUG_LOG
	spin_lock_irqsave(&pudc->lock, flags); /* DI */
	pudc->dbg_log[0] = '\0';
	pudc->dbg_log_count = 0;
	spin_unlock_irqrestore(&pudc->lock, flags); /* EI */
#endif /* DEBUG_LOG */

	return len;
}
#endif	/* DEBUG */

/*-------------------------------------------------------------------------*/
static const struct file_operations proc_ops = {
	.open		= mp200_udcprc_proc_udc_open,
	.read		= seq_read,
#ifdef DEBUG
	.write		= mp200_udcprc_proc_udc_write,
#endif	/* DEBUG */
	.llseek		= seq_lseek,
	.release	= single_release,
};

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcdev_remove_proc_file(void)
{
	remove_proc_entry(proc_filename, NULL);

	return;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcdev_create_proc_file(void)
{
	struct proc_dir_entry *pde;

	pde = create_proc_entry (proc_filename, 0, NULL);
	if(pde){
		pde->proc_fops = &proc_ops;
	}

	return;
}

/*-------------------------------------------------------------------------*/
static struct platform_driver udc_driver = {
	.probe		= mp200_udcdev_probe,
	.remove		= __exit_p(mp200_udcdev_remove),
	.suspend	= mp200_udcdev_suspend,
	.resume		= mp200_udcdev_resume,
	.driver		= {
		.owner	= THIS_MODULE,
		.name	= (char *) driver_name,
	},
};

/*-------------------------------------------------------------------------*/
static void __inline__
 __init mp200st_udcdev_ep_init(
	struct mp200_udc *pudc,
	struct mp200_ep *pep,
	u8 *pname)
{
	INIT_LIST_HEAD(&pep->queue);
	strcpy(pep->name, pname);
	pep->pudc		= pudc;
	pep->pdesc		= NULL;

	pep->ep.driver_data	= NULL;
	pep->ep.name		= pep->name;
	pep->ep.ops		= &mp200_ep_ops;
#ifdef USB_TRANS_DIVISION
	pep->ep.maxpacket	= 512;
#else /* USB_TRANS_DIVISION */
	pep->ep.maxpacket	= 1024;
#endif /* USB_TRANS_DIVISION */

	return;
}

/*-------------------------------------------------------------------------*/
static char g_ep0_name[] ="ep0";
static char g_ep1in_name[] ="ep1in";
static char g_ep1out_name[]="ep1out";
static char g_ep2in_name[] ="ep2in";
static char g_ep2out_name[]="ep2out";
static char g_ep3in_name[] ="ep3in";
static char g_ep3out_name[]="ep3out";
static char g_ep4in_name[] ="ep4in";
static char g_ep4out_name[]="ep4out";
static char g_ep5in_name[] ="ep5in";
static char g_ep5out_name[]="ep5out";
static char g_ep6in_name[] ="ep6in";
static char g_ep6out_name[]="ep6out";
static char g_ep7in_name[] ="ep7in";
static char g_ep7out_name[]="ep7out";
static char g_ep8in_name[] ="ep8in";
static char g_ep8out_name[]="ep8out";
static char g_ep9in_name[] ="ep9in";
static char g_ep9out_name[]="ep9out";
static char g_epain_name[] ="epain";
static char g_epaout_name[]="epaout";
static char g_epbin_name[] ="epbin";
static char g_epbout_name[]="epbout";
static char g_epcin_name[] ="epcin";
static char g_epcout_name[]="epcout";
static char g_epdin_name[] ="epdin";
static char g_epdout_name[]="epdout";
static char g_epein_name[] ="epein";
static char g_epeout_name[]="epeout";
static char g_epfin_name[] ="epfin";
static char g_epfout_name[]="epfout";

/*-------------------------------------------------------------------------*/
static char* gp_ep_name[((NUM_ENDPOINTS)*(NUM_DIRECTIONS) -1)] = {
	g_ep0_name,
	g_ep1out_name,
	g_ep1in_name,
	g_ep2out_name,
	g_ep2in_name,
	g_ep3out_name,
	g_ep3in_name,
	g_ep4out_name,
	g_ep4in_name,
	g_ep5out_name,
	g_ep5in_name,
	g_ep6out_name,
	g_ep6in_name,
	g_ep7out_name,
	g_ep7in_name,
	g_ep8out_name,
	g_ep8in_name,
	g_ep9out_name,
	g_ep9in_name,
	g_epaout_name,
	g_epain_name,
	g_epbout_name,
	g_epbin_name,
	g_epcout_name,
	g_epcin_name,
	g_epdout_name,
	g_epdin_name,
	g_epeout_name,
	g_epein_name,
	g_epfout_name,
	g_epfin_name,
};

/*-------------------------------------------------------------------------*/
static int __inline__
 __init mp200st_udcdev_udc_context_init(
 	struct mp200_udc *pudc,
	struct platform_device *pdev)
{
	int	i;
	struct mp200_ep *pep = &pudc->ep[0];

	/* init irq lock */
	spin_lock_init(&pudc->lock);
	pudc->potg_reg		= (PT_OTG_REGS)IO_ADDRESS(MP200_USB_BASE);
	pudc->softconnect	= 0;
	pudc->usb_suspended	= 0;
	pudc->linux_suspended	= 0;
	pudc->self_powered	= 1;
	pudc->remote_wakeup	= 0;
	pudc->devstate		= USB_STATE_NOTATTACHED;
	pudc->pfc_reg		= (PT_FC_REGS)(&pudc->potg_reg->USBCMD);
	pudc->pdev		= pdev;
	pudc->mA		= 0;


	/* init EP0 (IN OUT Both) */
	mp200st_udcdev_ep_init(pudc, &pudc->ep[0], gp_ep_name[0]);

	pudc->ep[0].ep.maxpacket = 64;

	pudc->pdev->dev.coherent_dma_mask = 0xFFFFFFFF;

	/* init gadget */
	pudc->gadget.ops = &mp200_gadget_ops;
	pudc->gadget.ep0 = &pudc->ep[0].ep;
	pudc->gadget.speed = USB_SPEED_UNKNOWN;
	pudc->gadget.name = driver_name;
	pudc->gadget.is_dualspeed = 1;

	/* init dev */
	device_initialize(&pudc->gadget.dev);

	/* init gadget dev */
	strcpy (pudc->gadget.dev.bus_id, "gadget");
	pudc->gadget.dev.release = mp200_udcgaddev_release;
	pudc->gadget.dev.parent = &pdev->dev;
	pudc->gadget.dev.dma_mask = pdev->dev.dma_mask;

	/* init EP0_list (no use)*/
	INIT_LIST_HEAD(&pudc->gadget.ep0->ep_list);

	/* init EP1-15_list */
	INIT_LIST_HEAD(&pudc->gadget.ep_list);

	/* init EP1-15 (IN OUT) */
	for(i=1 ; i<((NUM_ENDPOINTS)*(NUM_DIRECTIONS) -1) ; i++){
		mp200st_udcdev_ep_init(pudc, &pudc->ep[i], gp_ep_name[i]);
		list_add_tail(&pudc->ep[i].ep.ep_list, &pudc->gadget.ep_list);
	}

	/*	IO<->DMA memory alloc */
	if(pep->mem.p_vadrs == NULL){
		pep->mem.p_vadrs = dma_alloc_coherent(&pudc->pdev->dev, UDC_EP_AUTO_BUFSIZE, &pep->mem.padrs, GFP_KERNEL|__GFP_DMA);

		if(pep->mem.p_vadrs == NULL){
			ERR("%s dma_alloc_coherent NOMEM\n",__FUNCTION__);
			return -ENOMEM;
		}
		MEM_LOG(pudc,"%s dma_alloc_coherent(%08x)\n",__FUNCTION__,(int)pep->mem.p_vadrs);
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 __init mp200st_udcdev_dtd_manage_init(
	struct mp200_udc *pudc,
	struct platform_device *pdev,
	T_DTD_MANAGEMENT dTD_Manage[NUM_DIRECTIONS][NUM_ENDPOINTS][SUPPORT_DTD_NUM],
	PT_MEMORY_VP_INFO p_align_mem,
	PT_MEMORY_VP_INFO p_mem,
	size_t alloc_mem_size,
	size_t align_size)
{
	int			i, j, k;
	int			status;
	dma_addr_t		padrs;		/* phy address */
	u8			*p_vadrs;	/* virtual address */
	PT_DTD_MANAGEMENT	p_dTD_m;

	/* dTD */
	status = mp200st_udcdev_dtd_dqh_mem_init(pudc, pdev, p_align_mem, p_mem, alloc_mem_size, align_size);
	if(status < 0){
		return status;
	}

	/* init dTD MANAGEMENT */
	padrs  = p_mem->padrs;
	p_vadrs = (u8 *)p_mem->p_vadrs;

	for(i=0; i<NUM_DIRECTIONS; i++){
		for(j=0; j<NUM_ENDPOINTS; j++){
			for(k=0; k<SUPPORT_DTD_NUM; k++){
				p_dTD_m = &(dTD_Manage[i][j][k]);
				p_dTD_m->mem_info.padrs = padrs;
				p_dTD_m->mem_info.p_vadrs = (u32 *)p_vadrs;

				padrs += sizeof(T_EP_TRANS_DESC);
				p_vadrs += sizeof(T_EP_TRANS_DESC);
			}
		}
	}

	return 0;
}

/*-------------------------------------------------------------------------*/
static PT_ENDPOINT_QUEUE_HEAD
 mp200st_udc_ep_get_dQH_adrs(
	struct mp200_udc *pudc,
	u8 ep_adrs,
	bool bphy)
{
	u32		data;
	PT_ENDPOINT_QUEUE_HEAD p_ep_dQH;

	data = (ep_adrs & USB_ENDPOINT_NUMBER_MASK) << 1;
	if(ep_adrs & USB_DIR_IN){
		data++;
	}

	if(bphy == FALSE){
		p_ep_dQH = (PT_ENDPOINT_QUEUE_HEAD)pudc->dQH.p_vadrs;
	}else{
		p_ep_dQH = (PT_ENDPOINT_QUEUE_HEAD)pudc->dQH.padrs;
	}
	p_ep_dQH += data;

	return p_ep_dQH;
}

/*-------------------------------------------------------------------------*/
#ifndef CONFIG_SNSC_MP200_PLATFORM
static void __inline__
 mp200st_udc_check_vbus(struct mp200_udc *pudc)
{
	int		nret;
	int		is_active;
	u8		pwc_state=0;

	//---------------------------------------------------------------
	// interrupt clear
	pwc_write(PWC_CLRFACT3, PWC_CLRFACT3_VBUS, PWC_CLRFACT3_VBUS);

	//---------------------------------------------------------------
	// VBUS Check
	nret = mp200st_udc_trans_read(PWC_PSW_STATE, &pwc_state);
	if(nret < 0){
		return;
	}

	//---------------------------------------------------------------
	// interrupt setting
	if(pwc_state & PWC_PSW_VBUS){
		pwc_write(PWC_DETMOD3H, PWC_VBUS_FALLING_EDGE, PWC_DETMOD3H_VBUS_MASK);	// edge of fall down
	}
	else{
		pwc_write(PWC_DETMOD3H, PWC_VBUS_RISING_EDGE, PWC_DETMOD3H_VBUS_MASK);	// edge of stand up
	}

	//---------------------------------------------------------------
	// chattering
	mdelay(VBUS_CHATTERING_MDELAY);
	nret = mp200st_udc_trans_read(PWC_PSW_STATE, &pwc_state);
	if(nret < 0){
		return;
	}

	if(pwc_state & PWC_PSW_VBUS){
		//-----------------------------------------------------------
		// VBUS ON
		pwc_write(PWC_DETMOD3H, PWC_VBUS_FALLING_EDGE, PWC_DETMOD3H_VBUS_MASK);	// edge of fall down
		is_active = 1;
	}
	else{
		//-----------------------------------------------------------
		// VBUS OFF
		pwc_write(PWC_DETMOD3H, PWC_VBUS_RISING_EDGE, PWC_DETMOD3H_VBUS_MASK);	// edge of stand up
		is_active = 0;
	}

#ifdef CONFIG_SNSC_MP200_PLATFORM
	if(pudc->vbus_change)
		pudc->vbus_change(pudc, is_active);
#else
	mp200st_udc_vbus_change(pudc, is_active);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	return;
}
#endif /* CONFIG_SNSC_MP200_PLATFORM */

#define CTRL_REQUEST_REPEATED_COUNT	100000

/*-------------------------------------------------------------------------*/
/*	SetupDevReqRead */
static void __inline__
 mp200st_udcint_read_ctrl_request(
	struct mp200_udc *pudc,
	u8 ep_adrs,
	struct usb_ctrlrequest *pctrl)
{
	u32			data;
	PT_ENDPOINT_QUEUE_HEAD	p_dQH;
	int			i;

	for(i=0; i<CTRL_REQUEST_REPEATED_COUNT; i++){
		mp200st_udc_set_bit(&pudc->pfc_reg->USBCMD, USBCMD_SUTW);

		p_dQH = mp200st_udc_ep_get_dQH_adrs(pudc, ep_adrs, FALSE);

		/*	read device request */
		memcpy(pctrl, p_dQH->setup_buf, sizeof(struct usb_ctrlrequest));

		data = mp200st_udc_readl(&pudc->pfc_reg->USBCMD);
		if(data & USBCMD_SUTW){
			break;
		}
	}

	mp200st_udc_clear_bit(&pudc->pfc_reg->USBCMD, USBCMD_SUTW);

	return;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udcint_set_address(struct mp200_udc *pudc, u8 devaddr, bool bafter_ep0in)
{
	u32	data;

	/*	DevStat */
	if(devaddr > 0){
		pudc->devstate = USB_STATE_ADDRESS;
	}else{
		pudc->devstate = USB_STATE_DEFAULT;
	}

	/*	EP0 status in after SetAddress */
	if(bafter_ep0in){
		data = ((devaddr << DEVICEADDR_USBADR_SHIFT) | DEVICEADDR_USBADRA);
	}else{
		data = ((devaddr << DEVICEADDR_USBADR_SHIFT));
	}

	return mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->DEVICEADDR, data);
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udcint_req_set_address(struct mp200_udc *pudc)
{
	u8	devaddr = UDC_LE16_GET_L(pudc->ctrl.wValue);
	int	status;

	status = mp200st_udcint_set_address(pudc, devaddr, TRUE);

	if(status < 0){
		return status;
	}

	/*	make req */
	mp200st_udcint_status_packet(pudc);

	return 0;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_req_set_configuration(struct mp200_udc *pudc)
{
	u8	confnum = UDC_LE16_GET_L(pudc->ctrl.wValue);

	if(confnum > 0){
		pudc->devstate = USB_STATE_CONFIGURED;
	}else{
		pudc->devstate = USB_STATE_ADDRESS;
	}

	return;
}


/*-------------------------------------------------------------------------*/
/*	Endpoint STALL get */
static bool
 mp200st_udc_get_endpoint_stall(
	struct mp200_udc *pudc,
	u8 ep_adrs)
{
	u32	data;
	bool	bstall = FALSE;

	data = mp200st_udc_readl(&pudc->pfc_reg->ENDPTCTRL[(ep_adrs & USB_ENDPOINT_NUMBER_MASK)]);

	if((ep_adrs & USB_ENDPOINT_DIR_MASK) == USB_DIR_OUT){
		if(data & ENDPTCTRL_RXS){
			bstall = TRUE;
		}
	}else{
		if(data & ENDPTCTRL_TXS){
			bstall = TRUE;
		}
	}

	return bstall;
}

/*-------------------------------------------------------------------------*/
/*	Endpoint STALL set */
static void
 mp200st_udc_set_endpoint_stall(
	struct mp200_udc *pudc,
	u8 ep_adrs,
	bool bstall)
{
	u32	data;

	if((ep_adrs & USB_ENDPOINT_DIR_MASK) == USB_DIR_OUT){
		data = ENDPTCTRL_RXS;
	}else{
		data = ENDPTCTRL_TXS;
	}

	if(bstall){
		mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->ENDPTCTRL[(ep_adrs & USB_ENDPOINT_NUMBER_MASK)], data);
	}else{
		mp200st_udc_ulc_chk_clear_bit(pudc, &pudc->pfc_reg->ENDPTCTRL[(ep_adrs & USB_ENDPOINT_NUMBER_MASK)], data);
	}

	return;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udc_set_endpoint_toggle_reset(
	struct mp200_udc *pudc,
	u8 ep_adrs)
{
	u32	data;

	if((ep_adrs & USB_ENDPOINT_DIR_MASK) == USB_DIR_OUT){
		data = ENDPTCTRL_RXR;
	}else{
		data = ENDPTCTRL_TXR;
	}

	mp200st_udc_ulc_chk_set_bit(pudc, &pudc->pfc_reg->ENDPTCTRL[(ep_adrs & USB_ENDPOINT_NUMBER_MASK)], data);

	return;
}

/*-------------------------------------------------------------------------*/
static void
 mp200st_udcint_EP0_complete(struct usb_ep *puep, struct usb_request *pureq)
{
	struct mp200_ep		*pep;
	struct mp200_req	*preq;
	struct mp200_udc	*pudc;

	if((puep == NULL) || (pureq == NULL)){
		return ;
	}

	if(pureq->status != 0){
		DBG("%s (pureq->status != 0)(%d)\n",__FUNCTION__, pureq->status);
		return ;
	}

	pep = container_of(puep, struct mp200_ep, ep);
	preq = container_of(pureq, struct mp200_req, req);

	/* udc */
	pudc = pep->pudc;

	/* status after */
	if((pudc->ctrl.bRequestType & USB_TYPE_MASK) == USB_TYPE_STANDARD){
		switch (pudc->ctrl.bRequest){
		case USB_REQ_SET_FEATURE:
			mp200st_udcint_req_feature_complete(pudc, TRUE);
			break;
		default:
			break;
		}
	}

	return ;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udcint_req_feature(struct mp200_udc *pudc, bool bset)
{

	if((pudc->ctrl.bRequestType & USB_RECIP_MASK) == USB_RECIP_DEVICE){

		switch (le16_to_cpu(pudc->ctrl.wValue)){
		case USB_DEVICE_TEST_MODE:
			if(!bset){
				return -EOPNOTSUPP;
			}
			break;
		case USB_DEVICE_REMOTE_WAKEUP:
			pudc->remote_wakeup = bset;
			break;
		case USB_DEVICE_B_HNP_ENABLE:
			pudc->gadget.b_hnp_enable = bset;
			break;
		case USB_DEVICE_A_HNP_SUPPORT:
			pudc->gadget.a_hnp_support = bset;
			break;
		case USB_DEVICE_A_ALT_HNP_SUPPORT:
			pudc->gadget.a_alt_hnp_support = bset;
			break;
		default:
			return -EOPNOTSUPP;
		}

	}else if((pudc->ctrl.bRequestType & USB_RECIP_MASK) == USB_RECIP_ENDPOINT){
		/* clear endpoint halt */
		mp200st_udc_set_endpoint_stall(pudc, UDC_LE16_GET_L(pudc->ctrl.wIndex), bset);
		if(bset == FALSE){
			mp200st_udc_set_endpoint_toggle_reset(pudc, UDC_LE16_GET_L(pudc->ctrl.wIndex));
		}
	}else{
		return -EOPNOTSUPP;
	}

	/*	status in (Not STALL) */
	mp200st_udcint_status_packet(pudc);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int __inline__
 mp200st_udcint_req_get_status(struct mp200_udc *pudc)
{
	u16		req_status_data = 0;
	unsigned	length;

	/*	Data Low */
	if((pudc->ctrl.bRequestType & USB_RECIP_MASK) == USB_RECIP_DEVICE){

		if(pudc->self_powered){
			req_status_data |= (1 << USB_DEVICE_SELF_POWERED);
		}

		if(pudc->remote_wakeup){
			req_status_data |= (1 << USB_DEVICE_REMOTE_WAKEUP);
		}

	}else if((pudc->ctrl.bRequestType & USB_RECIP_MASK) == USB_RECIP_ENDPOINT){
		/* endpoint halt */

		/* check EP */
		if(mp200st_udc_get_endpoint_stall(pudc, UDC_LE16_GET_L(pudc->ctrl.wIndex))){
			req_status_data |= (1 << USB_ENDPOINT_HALT);
		}
	}

	/*	copy DMA mem */
	*( (u16*)pudc->ep0_mem.p_vadrs) = cpu_to_le16(req_status_data);

	length = min((u16)le16_to_cpu(pudc->ctrl.wLength), (u16)UDC_EP0_BUFSIZE);

	/*	data in (Not STALL) */
	mp200st_udcint_data_packet(pudc, pudc->ep0_mem.p_vadrs, length, pudc->ep0_mem.padrs);

	return 0;
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_EP0_setup_done(struct mp200_udc *pudc)
{
	int	status = -EINVAL;
	bool	bcall_driver_setup = TRUE;

	DBG_LOG(pudc,"SETUP DONE NUKE\n");
	mp200st_udc_nuke(pudc, &pudc->ep[0], -ETIME);

	/*	device request */
	mp200st_udcint_read_ctrl_request(pudc, (0 | USB_DIR_OUT), &pudc->ctrl);

	/*	driver->setup before */
	if((pudc->ctrl.bRequestType & USB_TYPE_MASK) == USB_TYPE_STANDARD){
		switch (pudc->ctrl.bRequest){
		case USB_REQ_SET_ADDRESS:
			bcall_driver_setup = FALSE;
			status = mp200st_udcint_req_set_address(pudc);
			break;

		case USB_REQ_CLEAR_FEATURE:
			bcall_driver_setup = FALSE;
			status = mp200st_udcint_req_feature(pudc, FALSE);
			break;

		case USB_REQ_SET_FEATURE:
			bcall_driver_setup = FALSE;
			status = mp200st_udcint_req_feature(pudc, TRUE);
			break;

		case USB_REQ_GET_STATUS:
			bcall_driver_setup = FALSE;
			status = mp200st_udcint_req_get_status(pudc);
			break;

		default:
			break;
		}
	}

	if(bcall_driver_setup){
		spin_unlock(&pudc->lock);
		status = pudc->pdriver->setup(&pudc->gadget, &pudc->ctrl);
		spin_lock(&pudc->lock);
	}

	/*	driver->setup after */
	if((pudc->ctrl.bRequestType & USB_TYPE_MASK) == USB_TYPE_STANDARD){
		switch (pudc->ctrl.bRequest){
		case USB_REQ_SET_CONFIGURATION:
			/* udc needs to know when ep != 0 is valid */
			if(status >= 0){
				mp200st_udcint_req_set_configuration(pudc);
			}
			break;

		default:
			break;
		}
	}

	if(status < 0){
		/* stall */
		mp200st_udc_set_endpoint_stall(pudc, (0 | USB_DIR_OUT), TRUE);
		mp200st_udc_set_endpoint_stall(pudc, (0 | USB_DIR_IN), TRUE);
	}

	return;
}

/*-------------------------------------------------------------------------*/
// Device Resume
static void  __inline__
 mp200st_udcint_resume_device(struct mp200_udc *pudc)	// __fn__
{
	if(pudc->usb_suspended != 0){
		pudc->usb_suspended = 0;

		//-----------------------------------------------------------
		// Power Save Mode
#ifdef CONFIG_SNSC_MP200_PLATFORM
		if (pudc->config && pudc->config->udc_disable_power_save_mode)
			pudc->config->udc_disable_power_save_mode(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
		pwc_write(PWC_MODESET1, 0, PWC_MODESET1_VUSBPS);
		udelay(POWER_SAVE_MODE_WAIT_TIME);		//
#endif /* CONFIG_SNSC_MP200_PLATFORM */

		//-----------------------------------------------------------
		// Link
		mp200st_udc_set_bit(&pudc->pfc_reg->PORTSC[0], PORTSC_FPR);
		mp200st_udc_writel(&pudc->pfc_reg->USBINTR, USB_FC_INT_ENABLE);
	}
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_bus_reset(struct mp200_udc *pudc)
{
	mp200st_udcint_resume_device(pudc);

	pudc->devstate		= USB_STATE_DEFAULT;
	pudc->remote_wakeup	= 0;

#ifdef CONFIG_MACH_PGPF
	if(pudc->pdriver->disconnect != NULL){
		spin_unlock(&pudc->lock);
		pudc->pdriver->disconnect(&pudc->gadget);
		spin_lock(&pudc->lock);
	}
#elif defined CONFIG_USB_NWM_MODULE
	if(pudc->pdriver->reset != NULL){
		spin_unlock(&pudc->lock);
		pudc->pdriver->reset(&pudc->gadget);
		spin_lock(&pudc->lock);
	}
#endif

#ifdef CONFIG_SNSC_USB_MP200_NO_VBUS_INTR
	if (pudc->pdriver->disconnect != NULL) {
		spin_unlock(&pudc->lock);
		pudc->pdriver->disconnect(&pudc->gadget);
		spin_lock(&pudc->lock);
	}
#endif /* CONFIG_SNSC_USB_MP200_NO_VBUS_INTR */

	mp200st_udc_quiesce(pudc);

	return;
}

/*-------------------------------------------------------------------------*/
#ifdef CONFIG_SNSC_USB_MP200_NO_VBUS_INTR
static void mp200st_udcint_disconnect(struct mp200_udc *pudc)
{
	if(pudc->pdriver->disconnect != NULL){
		spin_unlock(&pudc->lock);
		pudc->pdriver->disconnect(&pudc->gadget);
		spin_lock(&pudc->lock);
	}

	mp200st_udc_quiesce(pudc);

	pudc->devstate = USB_STATE_NOTATTACHED;
}
#endif

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_resume(struct mp200_udc *pudc)
{
#ifdef CONFIG_SNSC_USB_MP200_NO_VBUS_INTR
	u32 data;

	data = mp200st_udc_readl(&pudc->pfc_reg->PORTSC[0]);
	if (!(data & PORTSC_CCS)) {
		WARN("%s called, but not connected. PORTSC=%08x\n",
		     __func__, data);
		mp200st_udcint_disconnect(pudc);
		return;
	}
#endif /* CONFIG_SNSC_USB_MP200_NO_VBUS_INTR */

	mp200st_udcint_resume_device(pudc);

	if(pudc->pdriver->resume) {
		spin_unlock(&pudc->lock);
		pudc->pdriver->resume(&pudc->gadget);
		spin_lock(&pudc->lock);
	}

	return;
}

/*-------------------------------------------------------------------------*/
// USB Suspend
static void __inline__
 mp200st_udcint_suspend(struct mp200_udc *pudc)
{
	u32		data;
	u32		*p_port;

	//---------------------------------------------------------------
	// VBUS
	if(pudc->vbus_active == 0){
		return;
	}

	switch(pudc->timer_mode[1]){
	case TIMER_MODE_INIT:
	case TIMER_MODE_END:
		//-----------------------------------------------------------
		// if Device is Default_State then don't care
		if(pudc->devstate == USB_STATE_DEFAULT){
			break;
		}

		//-----------------------------------------------------------
		// Timer Start
		pudc->timer_mode[1] = TIMER_MODE_SUSPEND;
		data = TIMER_COUNT_X_USEC(CONFIG_SNSC_USB_MP200_VBUSCLR_WAIT);
		mp200st_udc_writel(&pudc->potg_reg->GPTIMER1LD, data);

		data = GPTIMER1CTRL_GPTRUN | GPTIMER1CTRL_GPTRST;
		mp200st_udc_writel(&pudc->potg_reg->GPTIMER1CTRL, data);
		break;

	case TIMER_MODE_SUSPEND:
		pudc->timer_mode[1] = TIMER_MODE_END;
		p_port = &pudc->pfc_reg->PORTSC[0];
		data = mp200st_udc_readl(p_port);
		if(data & PORTSC_SUSP){
			//-------------------------------------------------------
			// Low Power Mode
			mp200st_udc_set_bit(p_port, PORTSC_PHCD);

#ifdef CONFIG_SNSC_USB_MP200_FIX_RESUME
			//-------------------------------------------------------
			// SOF interrupt ON
			mp200st_udc_writel(&pudc->pfc_reg->USBINTR, USB_FC_INT_SUS_ENABLE);
#endif /* CONFIG_SNSC_USB_MP200_FIX_RESUME */
			//-------------------------------------------------------
			// Power Save Mode
#ifdef CONFIG_SNSC_MP200_PLATFORM
			if (pudc->config &&
			    pudc->config->udc_enable_power_save_mode)
				pudc->config->udc_enable_power_save_mode(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
			pwc_write(PWC_MODESET1, PWC_MODESET1_VUSBPS, PWC_MODESET1_VUSBPS);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

			pudc->usb_suspended = 1;
			if(pudc->pdriver->suspend) {
				spin_unlock(&pudc->lock);
				pudc->pdriver->suspend(&pudc->gadget);
				spin_lock(&pudc->lock);
			}
		}
		else{
			pudc->usb_suspended = 0;
#ifdef CONFIG_SNSC_USB_MP200_NO_VBUS_INTR
			if(!(data & PORTSC_CCS))
				mp200st_udcint_disconnect(pudc);
#endif /* CONFIG_SNSC_USB_MP200_NO_VBUS_INTR */
		}
		break;

	default:
		break;
	}
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_timer_1(struct mp200_udc *pudc)
{
	switch(pudc->timer_mode[1]){
	case TIMER_MODE_SUSPEND:
		mp200st_udcint_suspend(pudc);
		break;

	default:
		break;
	}
}

#ifdef DUMMY_PRIME
/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_dummy_prime_complete(struct mp200_udc *pudc, u8 ep_adrs)	// __fn__
{
#ifdef DUMMY_PRIME_NAK_INT
	u32			epnak;
#endif /* DUMMY_PRIME_NAK_INT */
#ifdef DUMMY_PRIME_NOT_EP0
	u8			epnum = ep_adrs & USB_ENDPOINT_NUMBER_MASK;
#endif /* DUMMY_PRIME_NAK_INT */

	/*	check */

#ifdef DUMMY_PRIME_NOT_EP0

	if(epnum != 0){
		mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAK, ENDPT_BIT_POS(ep_adrs));
		pudc->dummy_prime_transferring &= ~(ENDPT_BIT_POS(ep_adrs));

  #ifdef DUMMY_PRIME_NAK_INT
		epnak = mp200st_udc_readl(&pudc->pfc_reg->ENDPTNAK);
		epnak &= ~(ENDPT_BIT_POS(0 | USB_DIR_IN) | ENDPT_BIT_POS(0 | USB_DIR_OUT));

		if(!(epnak & pudc->dummy_prime_waiting)){
			mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAKEN, pudc->dummy_prime_waiting);
		}
  #endif /* DUMMY_PRIME_NAK_INT */

	}

#else /* DUMMY_PRIME_NOT_EP0 */

	mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAK, ENDPT_BIT_POS(ep_adrs));
	pudc->dummy_prime_transferring &= ~(ENDPT_BIT_POS(ep_adrs));

  #ifdef DUMMY_PRIME_NAK_INT
	epnak = mp200st_udc_readl(&pudc->pfc_reg->ENDPTNAK);
	if(!(epnak & pudc->dummy_prime_waiting)){
		mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAKEN, pudc->dummy_prime_waiting);
	}
  #endif /* DUMMY_PRIME_NAK_INT */

#endif /* DUMMY_PRIME_NOT_EP0 */
}

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_dummy_prime_check(struct mp200_udc *pudc)	// __fn__
{
	int		i;
	u32		epnak;
	u32		epnak_prime;
	u32		ep_adrs;

	if(pudc->dummy_prime_transferring == 0){
		epnak = mp200st_udc_readl(&pudc->pfc_reg->ENDPTNAK);
		epnak_prime = epnak & pudc->dummy_prime_waiting;

		if(epnak_prime != 0){
 #ifdef DUMMY_PRIME_NAK_INT
			mp200st_udc_ulc_chk_writel(pudc, &pudc->pfc_reg->ENDPTNAKEN, 0);
 #endif /* DUMMY_PRIME_NAK_INT */

			/* in (EP0-EP15) */
			for(i=0; (i<NUM_ENDPOINTS); i++){

				/*	IN check */
				ep_adrs = (i | USB_DIR_IN);

				if(epnak_prime & ENDPT_BIT_POS(ep_adrs)){
					mp200st_udc_set_dummy_prime_transferring(pudc, ENDPT_BIT_POS(ep_adrs));
					break;
				}

				/* OUT check */
				ep_adrs = (i | USB_DIR_OUT);

				if(epnak_prime & ENDPT_BIT_POS(ep_adrs)){
					mp200st_udc_set_dummy_prime_transferring(pudc, ENDPT_BIT_POS(ep_adrs));
					break;
				}

			}
		}

	}
}
#endif	// DUMMY_PRIME

/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_endpt_complete(struct mp200_udc *pudc, u8 ep_adrs)	// __fn__
{

#ifdef DUMMY_PRIME
	mp200st_udcint_dummy_prime_complete(pudc, ep_adrs);
#endif	// DUMMY_PRIME

	mp200st_udc_done_transfer(pudc, ep_adrs);

	return;
}


/*-------------------------------------------------------------------------*/
static void __inline__
 mp200st_udcint_usb_interrupt(struct mp200_udc *pudc)
{
	u32	intr_status;
	u32	endpt_setup;
	u32	endpt_complete;
	int	i;

	//---------------------------------------------------------------
	// Read & Clear Interrupt Status
	intr_status = mp200st_udc_readl(&pudc->pfc_reg->USBSTS);
	mp200st_udc_writel(&pudc->pfc_reg->USBSTS, intr_status);

	endpt_setup = mp200st_udc_readl(&pudc->pfc_reg->ENDPTSETUPSTAT);
	mp200st_udc_writel(&pudc->pfc_reg->ENDPTSETUPSTAT, endpt_setup);

	endpt_complete = mp200st_udc_readl(&pudc->pfc_reg->ENDPTCOMPLETE);
	mp200st_udc_writel(&pudc->pfc_reg->ENDPTCOMPLETE, endpt_complete);

	//---------------------------------------------------------------
	// Debug Message
	if(endpt_setup){
		DBG_LOG(pudc,"SETUP(%08x)\n",endpt_setup);
	}
	if(endpt_complete){
		DBG_LOG(pudc,"COMP(%08x)\n",endpt_complete);
	}

	//---------------------------------------------------------------
	// Endpoint Complete
	if(endpt_complete){
		u32		out_done=(endpt_complete & ENDPTCOMPLETE_ERCE) >> ENDPTCOMPLETE_ERCE_SHIFT;
		u32		in_done=(endpt_complete & ENDPTCOMPLETE_ETCE) >> ENDPTCOMPLETE_ETCE_SHIFT;

		if(pudc->ctrl.bRequestType & USB_DIR_IN){
			for(i=0; (out_done | in_done)>0; i++){
				if(in_done & 0x01){
					mp200st_udcint_endpt_complete(pudc, (i | USB_DIR_IN));
				}

				if(out_done & 0x01){
					mp200st_udcint_endpt_complete(pudc, (i | USB_DIR_OUT));
				}

				out_done = out_done >> 1;
				in_done = in_done >> 1;
			}
		}
		else{
			for(i=0; (out_done | in_done)>0; i++){
				if(out_done & 0x01){
					mp200st_udcint_endpt_complete(pudc, (i | USB_DIR_OUT));
				}

				if(in_done & 0x01){
					mp200st_udcint_endpt_complete(pudc, (i | USB_DIR_IN));
				}

				out_done = out_done >> 1;
				in_done = in_done >> 1;
			}
		}
	}

	/* SETUP interrupt (High Frequency) (EP0 only) */
	if(endpt_setup & ENDPTSETUPSTAT_ENDPTSETUPSTAT_EP0){
		mp200st_udcint_EP0_setup_done(pudc);
	}

	/* BUS RESET interrupt (Low Frequency) */
	if(intr_status & USBSTS_URI){
		mp200st_udcint_bus_reset(pudc);
	}
	else if(intr_status & USBSTS_TI1){
		mp200st_udcint_timer_1(pudc);
		intr_status = 0;
	}

	if(intr_status & USBSTS_SLI){
		if(pudc->usb_suspended == 0){
			mp200st_udcint_suspend(pudc);
		}else{
			DBG("SUSPEND -> SUSPEND ERR\n");
		}
	}

	/* PORT CHANGE(RESUME) interrupt (Low Frequency) */
	if(intr_status & USBSTS_PCI){
		if(pudc->gadget.speed == USB_SPEED_UNKNOWN){
			pudc->gadget.speed = mp200st_udcint_get_speed(pudc);
		}

		if(pudc->usb_suspended){
			mp200st_udcint_resume(pudc);
		}
	}

	//---------------------------------------------------------------
	// Suspend -> Resume
	if(intr_status & USBSTS_SRI){
		if(pudc->usb_suspended){
			mp200st_udcint_resume(pudc);
		}
	}

#ifdef DUMMY_PRIME
	mp200st_udcint_dummy_prime_check(pudc);
#endif	// DUMMY_PRIME

	return;
}

/*-------------------------------------------------------------------------*/
static irqreturn_t
 mp200_udcint_irq(int irq, void *id_udc)
{
	struct mp200_udc	*pudc = &g_udc;
	unsigned long		flags;

	spin_lock_irqsave(&pudc->lock, flags);

	if(pudc->vbus_active == 0){
		spin_unlock_irqrestore(&pudc->lock, flags);
		return IRQ_NONE;
	}

	mp200st_udcint_usb_interrupt(pudc);

	spin_unlock_irqrestore(&pudc->lock, flags);

	return IRQ_HANDLED;
}

/*-------------------------------------------------------------------------*/
#ifndef CONFIG_SNSC_MP200_PLATFORM
static irqreturn_t
 mp200_udcint_vbus_irq(int irq, void *id_udc)
{
	struct mp200_udc	*pudc = &g_udc;
	unsigned long		flags;

	spin_lock_irqsave(&pudc->lock, flags);
	mp200st_udc_check_vbus(pudc);
	spin_unlock_irqrestore(&pudc->lock, flags);

	return IRQ_HANDLED;
}
#endif /* !CONFIG_SNSC_MP200_PLATFORM */

/*-------------------------------------------------------------------------*/
static int
 __init mp200_udcdev_probe(struct platform_device *pdev)
{
	struct mp200_udc	*pudc = &g_udc;
	int			status = -ENODEV;
#ifndef CONFIG_SNSC_MP200_PLATFORM
	unsigned long		flags;
#endif /* !CONFIG_SNSC_MP200_PLATFORM */
	struct mp200_ep *pep = &pudc->ep[0];

	memset(pudc, 0, sizeof(struct mp200_udc));

#ifdef CONFIG_SNSC_MP200_PLATFORM
	pudc->vbus_change = &mp200st_udc_vbus_change;
	pudc->config = pdev->dev.platform_data;
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	/* set device_data */
	platform_set_drvdata(pdev, pudc);

	/* NOTE:  "knows" the order of the resources! */
	if(!request_mem_region(MP200_USB_BASE, sizeof(T_OTG_REGS), driver_name)){
		ERR("request_mem_region failed\n");
		return -EBUSY;
	}

	/* a "gadget" abstracts/virtualizes the controller */
	status = mp200st_udcdev_udc_context_init(pudc, pdev);
	if(status != 0){
		ERR("can't mp200st_udcdev_udc_context_init, err %d\n", status);
		goto cleanup1;
	}

	/* dQH */
	status = mp200st_udcdev_dtd_dqh_mem_init(pudc,
						 pdev,
						 &pudc->dQH_align_mem,
						 &pudc->dQH,
						 DQH_ALLOC_MEMORY_SIZE,
						 DQH_MEMORY_SIZE);
	if(status != 0){
		ERR("can't mp200st_udcdev_dtd_dqh_mem_init dQH, err %d\n", status);
		goto cleanup2;
	}

	/* dTD */
	status = mp200st_udcdev_dtd_manage_init(pudc,
						 pdev,
						 pudc->dTD_Manage,
						 &pudc->dTD_align_mem,
						 &pudc->dTD,
						 DTD_ALLOC_MEMORY_SIZE,
						 DTD_STRUCT_SIZE);
	if(status != 0){
		ERR("can't mp200st_udcdev_dtd_manage_init dTD, err %d\n", status);
		goto cleanup3;
	}

	/* EP0 buf */
	pudc->ep0_mem.p_vadrs = dma_alloc_coherent(&pudc->pdev->dev,
						UDC_EP0_BUFSIZE,
						&pudc->ep0_mem.padrs,
						GFP_KERNEL|__GFP_DMA);
	if(pudc->ep0_mem.p_vadrs == NULL){
		status = -ENOMEM;
		ERR("can't dma_alloc_coherent ep0 buf, err %d\n", status);
		goto cleanup4;
	}
	MEM_LOG(pudc,"%s dma_alloc_coherent(%08x)\n",__FUNCTION__,(int)pudc->ep0_mem.p_vadrs);

	/* USB general purpose IRQ:  ep0, state changes, dma, etc */
	status = request_irq(
		UDC_USB_INT,
		mp200_udcint_irq,
		IRQF_DISABLED,
		driver_name,
		pudc
	);
	if(status != 0){
		ERR("can't get irq %d, err %d\n", UDC_USB_INT, status);
		goto cleanup5;
	}

#ifdef CONFIG_SNSC_MP200_PLATFORM
	status = 0;
	if (pudc->config && pudc->config->udc_probe)
		status = pudc->config->udc_probe(pudc);
	if (status < 0) {
		ERR("can't probe, err %d\n", status);
		goto cleanup6;
	}
#else /* CONFIG_SNSC_MP200_PLATFORM */
	/* USB "VBUS(USBINT)" IRQ */
	status = request_irq(
		UDC_VBUS_INT,
		mp200_udcint_vbus_irq,
#ifdef CONFIG_MACH_PGPF
		IRQF_DISABLED|IRQF_SHARED,
#else
		IRQF_DISABLED,
#endif
		"USB VBUS",
		pudc
	);
	if(status != 0){
		ERR("can't get irq %d, err %d\n", UDC_VBUS_INT, status);
		goto cleanup6;
	}
	pudc->usbint_intno = UDC_VBUS_INT;
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	/* proc (info) */
	mp200st_udcdev_create_proc_file();

	/* gadget dev add */
	status = device_add(&pudc->gadget.dev);
	if(status != 0){
		ERR("can't device_add gadget.dev err %d\n", status);
		goto cleanup7;
	}

	//---------------------------------------------------------------
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pudc->config && pudc->config->udc_check_vbus)
		pudc->config->udc_check_vbus(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
	// VBUS check
	// at first, checks already connected USB, and already supply VBUS
	//
	spin_lock_irqsave(&pudc->lock, flags);
	mp200st_udc_check_vbus(pudc);
	spin_unlock_irqrestore(&pudc->lock, flags);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	//---------------------------------------------------------------
	// VBUS interrupt start
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pudc->config && pudc->config->udc_enable_vbus_int)
		pudc->config->udc_enable_vbus_int(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
	pwc_write(PWC_BMASK3, PWC_BMASK3_VBUS, PWC_BMASK3_VBUS);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

	return 0;

	/* err */
cleanup7:
	mp200st_udcdev_remove_proc_file();
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pudc->config && pudc->config->udc_remove)
		pudc->config->udc_remove(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
	free_irq(pudc->usbint_intno, pudc);
#endif /* CONFIG_SNSC_MP200_PLATFORM */

cleanup6:
	free_irq(UDC_USB_INT, pudc);

cleanup5:
	MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->ep0_mem.p_vadrs);
	dma_free_coherent(&pudc->pdev->dev,
			  UDC_EP0_BUFSIZE,
			  pudc->ep0_mem.p_vadrs,
			  pudc->ep0_mem.padrs);
	pudc->ep0_mem.p_vadrs = NULL;
	pudc->ep0_mem.padrs = DMA_ADDR_INVALID;

cleanup4:
	MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->dTD_align_mem.p_vadrs);
	dma_free_coherent(&pudc->pdev->dev,
			  DTD_ALLOC_MEMORY_SIZE,
			  pudc->dTD_align_mem.p_vadrs,
			  pudc->dTD_align_mem.padrs);
	pudc->dTD_align_mem.p_vadrs = NULL;
	pudc->dTD_align_mem.padrs = DMA_ADDR_INVALID;
	pudc->dTD.p_vadrs = NULL;
	pudc->dTD.padrs = DMA_ADDR_INVALID;

cleanup3:
	MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->dQH_align_mem.p_vadrs);
	dma_free_coherent(&pudc->pdev->dev,
			  DQH_ALLOC_MEMORY_SIZE,
			  pudc->dQH_align_mem.p_vadrs,
			  pudc->dQH_align_mem.padrs);
	pudc->dQH_align_mem.p_vadrs = NULL;
	pudc->dQH_align_mem.padrs = DMA_ADDR_INVALID;
	pudc->dQH.p_vadrs = NULL;
	pudc->dQH.padrs = DMA_ADDR_INVALID;

cleanup2:
	if(pep->mem.p_vadrs != NULL){
		MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pep->mem.p_vadrs);
		dma_free_coherent(&pudc->pdev->dev, UDC_EP_AUTO_BUFSIZE, pep->mem.p_vadrs, pep->mem.padrs);
		pep->mem.p_vadrs = NULL;
		pep->mem.padrs = DMA_ADDR_INVALID;
	}
	release_mem_region(MP200_USB_BASE, sizeof(T_OTG_REGS));

cleanup1:
	return status;
}

/*-------------------------------------------------------------------------*/
static void
 mp200_udcgaddev_release(struct device *pgaddev)
{
	struct mp200_udc	*pudc;

	pudc = container_of(container_of(pgaddev, struct usb_gadget, dev), struct mp200_udc, gadget);
	/* gadget exit done */
	complete(pudc->pdone);

	return;
}

/*-------------------------------------------------------------------------*/
static int
 __exit mp200_udcdev_remove(struct platform_device *pdev)
{
	struct mp200_udc	*pudc = platform_get_drvdata(pdev);
	struct mp200_ep *pep;
	DECLARE_COMPLETION_ONSTACK(dev_done);

	/* gadget started */
	if(pudc->pdriver != NULL){
		DBG("%s -EBUSY",__FUNCTION__);
		return -EBUSY;
	}

	/* gadget comp wait */
	pudc->pdone = &dev_done;

	/* proc (info) */
	mp200st_udcdev_remove_proc_file();

	//---------------------------------------------------------------
	// VBUS interrupt end.
#ifdef CONFIG_SNSC_MP200_PLATFORM
	if (pudc->config && pudc->config->udc_disable_vbus_int)
		pudc->config->udc_disable_vbus_int(pudc);

	if (pudc->config && pudc->config->udc_remove)
		pudc->config->udc_remove(pudc);
#else /* CONFIG_SNSC_MP200_PLATFORM */
	pwc_write(PWC_BMASK3, 0, PWC_BMASK3_VBUS);

	/* USB "VBUS(USBINT)" IRQ */
	free_irq(pudc->usbint_intno, pudc);
#endif /* CONFIG_SNSC_MP200_PLATFORM */
	/* USB general purpose IRQ:  ep0, state changes, dma, etc */
	free_irq(UDC_USB_INT, pudc);

	/* EP0 buf */
	MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->ep0_mem.p_vadrs);
	dma_free_coherent(&pudc->pdev->dev,
			  UDC_EP0_BUFSIZE,
			  pudc->ep0_mem.p_vadrs,
			  pudc->ep0_mem.padrs);
	pudc->ep0_mem.p_vadrs = NULL;
	pudc->ep0_mem.padrs = DMA_ADDR_INVALID;

	/* dTD */
	MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->dTD_align_mem.p_vadrs);
	dma_free_coherent(&pudc->pdev->dev,
			  DTD_ALLOC_MEMORY_SIZE,
			  pudc->dTD_align_mem.p_vadrs,
			  pudc->dTD_align_mem.padrs);
	pudc->dTD_align_mem.p_vadrs = NULL;
	pudc->dTD_align_mem.padrs = DMA_ADDR_INVALID;
	pudc->dTD.p_vadrs = NULL;
	pudc->dTD.padrs = DMA_ADDR_INVALID;

	/* dQH */
	MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->dQH_align_mem.p_vadrs);
	dma_free_coherent(&pudc->pdev->dev,
			  DQH_ALLOC_MEMORY_SIZE,
			  pudc->dQH_align_mem.p_vadrs,
			  pudc->dQH_align_mem.padrs);
	pudc->dQH_align_mem.p_vadrs = NULL;
	pudc->dQH_align_mem.padrs = DMA_ADDR_INVALID;
	pudc->dQH.p_vadrs = NULL;
	pudc->dQH.padrs = DMA_ADDR_INVALID;

	/* EP0 buf free */
	if(pudc->ep[0].mem.p_vadrs != NULL){
		MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pudc->ep[0].mem.p_vadrs);
		dma_free_coherent(&pudc->pdev->dev, UDC_EP_AUTO_BUFSIZE, pudc->ep[0].mem.p_vadrs, pudc->ep[0].mem.padrs);
		pudc->ep[0].mem.p_vadrs = NULL;
		pudc->ep[0].mem.padrs = DMA_ADDR_INVALID;
	}

	/* EP1-15 buf free */
	list_for_each_entry(pep, &pudc->gadget.ep_list, ep.ep_list){
		if(pep->mem.p_vadrs != NULL){
			MEM_LOG(pudc,"%s dma_free_coherent(%08x)\n",__FUNCTION__,(int)pep->mem.p_vadrs);
			dma_free_coherent(&pudc->pdev->dev, UDC_EP_AUTO_BUFSIZE, pep->mem.p_vadrs, pep->mem.padrs);
			pep->mem.p_vadrs = NULL;
			pep->mem.padrs = DMA_ADDR_INVALID;
		}
	}

	/* NOTE: release "knows" the order of the resources! */
	release_mem_region(MP200_USB_BASE, sizeof(T_OTG_REGS));

	/* gadget device_unregister */
	device_unregister(&pudc->gadget.dev);
	/* gadget exit comp wait */
	wait_for_completion(&dev_done);

#ifdef MEMORY_LOG
	{
		char tmp[101];
		int i;

		for(i=0; i<pudc->mem_log_count; i+=100){
			memcpy(tmp, pudc->mem_log+i, 100);
			tmp[100] = '\0';
			printk("%s",tmp);
		}
	}
#endif /* MEMORY_LOG */

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcdev_suspend(struct platform_device *pdev, pm_message_t message)
{
	struct mp200_udc	*pudc = &g_udc;
	unsigned long		flags;

	spin_lock_irqsave(&pudc->lock, flags);

	if((message.event == DEV_SUSPEND_IDLE_1) && mp200st_udc_can_pullup(pudc)){

		spin_unlock_irqrestore(&pudc->lock, flags);

		/* USB PULLUP(VBUS ON) & DEV_SUSPEND_IDLE_1 */
		DBG("%s -EBUSY",__FUNCTION__);
		return -EBUSY;
	}
	DBG("%s suspend",__FUNCTION__);

	/* USB STOP */
	mp200st_udc_linux_suspended_change(pudc, TRUE);

	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 mp200_udcdev_resume(struct platform_device *pdev)
{
	struct mp200_udc	*pudc = &g_udc;
	unsigned long		flags;

	spin_lock_irqsave(&pudc->lock, flags);
	DBG("%s call resume",__FUNCTION__);
	/* USB START */
	pudc->linux_resume = 1;
	mp200st_udc_linux_suspended_change(pudc, FALSE);
	spin_unlock_irqrestore(&pudc->lock, flags);

	return 0;
}

/*-------------------------------------------------------------------------*/
int
 usb_gadget_register_driver(struct usb_gadget_driver *pdriver)
{
	struct mp200_udc	*pudc = &g_udc;
	int			status = -ENODEV;
	unsigned long		flags;

	/* basic sanity tests */
	if((pdriver == NULL)
	   /*  FIXME if otg, check:  driver->is_otg */
	   || (pdriver->speed < USB_SPEED_FULL)
	   || (pdriver->bind == NULL)
	   || (pdriver->setup == NULL)){

		DBG("%s -EINVAL",__FUNCTION__);
		return -EINVAL;
	}

	if(pudc->potg_reg == NULL){
		DBG("%s -ENODEV",__FUNCTION__);
		return -ENODEV;
	}

	spin_lock_irqsave(&pudc->lock, flags);

	if(pudc->pdriver != NULL){
		spin_unlock_irqrestore(&pudc->lock, flags);
		DBG("%s -EBUSY",__FUNCTION__);
		return -EBUSY;
	}

	/* hook up the driver */
	pdriver->driver.bus = NULL;
	pudc->pdriver = pdriver;
	pudc->gadget.dev.driver = &pdriver->driver;

	spin_unlock_irqrestore(&pudc->lock, flags);

	status = pdriver->bind(&pudc->gadget);
	if(status < 0){
		DBG("bind to %s --> %d\n", pdriver->driver.name, status);
		pudc->gadget.dev.driver = NULL;
		pudc->pdriver = NULL;
		return status;
	}
	DBG("bound to driver %s\n", pdriver->driver.name);

	/* connect to bus through transceiver */
	/* reset state */
	spin_lock_irqsave(&pudc->lock, flags);

	mp200st_udc_soft_connect_change(pudc, TRUE);

	spin_unlock_irqrestore(&pudc->lock, flags);

	return status;
}

/*-------------------------------------------------------------------------*/
int
 usb_gadget_unregister_driver(struct usb_gadget_driver *pdriver)
{
	struct mp200_udc	*pudc = &g_udc;
	unsigned long		flags;

	if((pdriver == NULL)
	   || (pdriver != pudc->pdriver)
	   || (pdriver->unbind == NULL) ){

		return -EINVAL;
	}

	if(pudc->potg_reg == NULL){
		return -ENODEV;
	}

	spin_lock_irqsave(&pudc->lock, flags);

	mp200st_udc_soft_connect_change(pudc, FALSE);

	spin_unlock_irqrestore(&pudc->lock, flags);

	pdriver->unbind(&pudc->gadget);
	pudc->gadget.dev.driver = NULL;
	pudc->pdriver = NULL;

	DBG("unregistered driver '%s'\n", pdriver->driver.name);

	return 0;
}

/*-------------------------------------------------------------------------*/
static int
 __init mp200_udc_init(void)
{
	return platform_driver_register(&udc_driver);
}

/*-------------------------------------------------------------------------*/
static void __exit
 mp200_udc_exit(void)
{
	platform_driver_unregister(&udc_driver);

	return;
}

#ifdef ENABLE_DEDICATED_CHARGER_DETECTION
static int mp200_udc_detect_charger(struct mp200_udc *pudc)
{
	u8 reg, tmp;
	u32 portsc, ls;
	int err;
	int ret = 0;

	if (!pudc || !pudc->pfc_reg)
		return -EINVAL;

	/* tri-state D+/D- and enable FS tranceiver */
	err = mp200st_udc_ulpi_read(pudc, 0x04);
	if (err < 0)
		return err;
	reg = (u8)err;
	tmp = (reg & 0xe4) | 0x09;
	err = mp200st_udc_ulpi_write(pudc, 0x04, tmp);
	if (err < 0)
		return err;
	/* enable charger detect D+ pull-up */
	err = mp200st_udc_ulpi_write(pudc, 0x3a, BIT04);
	if (err < 0) {
		mp200st_udc_ulpi_write(pudc, 0x04, reg);
		return err;
	}

	/* check line status */
	mdelay(5);
	portsc = mp200st_udc_readl(&pudc->pfc_reg->PORTSC[0]);
	ls = (portsc >> 10) & 0x03;
	DBG("Line status = 0x%x\n", ls);
	if (ls == 0x03) { /* SE1 */
		DBG("detect charger !!\n");
		pgpf_usb_notify_supplied_power(1, 500); /* draw 500mA */
	}

	/* restore regs */
	mp200st_udc_ulpi_write(pudc, 0x3b, BIT04);
	mp200st_udc_ulpi_write(pudc, 0x04, reg);

	return ret;
}
#endif /* ENABLE_DEDICATED_CHARGER_DETECTION */


EXPORT_SYMBOL(usb_gadget_register_driver);
EXPORT_SYMBOL(usb_gadget_unregister_driver);
module_init(mp200_udc_init);
module_exit(mp200_udc_exit);

MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");
