/* 2010-08-25: File added and changed by Sony Corporation */
//=============================================================================
//
//	File Name		: linux/drivers/usb/gadget/nwm_mtp.c
//	Function		: USB MTP Transport Driver for M2
//	Release Version : Ver 0.90
//	Release Date	: 2008/06/09
//
//=============================================================================
//	2008/06/09	Ver 0.91
//			add member "*org_gadget" to mtp_device structure
//	2008/04/15	Ver 0.90
//=============================================================================

/*
 *	Copyright (C) NEC Electronics Corporation 2007
 *
 *
 *	This program is free software;you can redistribute it and/or modify it under the terms of
 *	the GNU General Public License as published by Free Softwere Foundation; either version 2
 *	of License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *	without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *	See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this program;
 *	If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *	MA 02111-1307, USA.
 *
 */



#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/semaphore.h>

#include <linux/bitops.h>
#include <linux/blkdev.h>
#include <linux/compiler.h>
#include <linux/completion.h>
#include <linux/dcache.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/fcntl.h>
#include <linux/file.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/kref.h>
#include <linux/kthread.h>
#include <linux/limits.h>
#include <linux/list.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/pagemap.h>
#include <linux/rwsem.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/string.h>
#include <linux/suspend.h>
#include <linux/utsname.h>
#include <linux/wait.h>
#include <linux/cdev.h>

#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>

#include "usbclass_common.h"

//----------------------------------
// Debug message ON
//#define USE_DEBUG_MESSAGE
//#define USE_INFOMATION_MESSAGE
#include "usb_debug_message.h"
//----------------------------------


//-----------------------------------------------------------------------------
// macro
#define	NWM_DEVICE_TYPE_MTP			1
#define USB_MTP_ERROR				-1
#define USB_DEVICE_NOT_CONNECTED	-2
#define USB_HOST_CANCELING			-3

#define	READ_BUFFER_SIZE			(1024*16)
#define	WRITE_BUFFER_SIZE			(1024*16)
#define	EVENT_BUFFER_SIZE			(1024*16)

#define USB_HOST_CANCEL_TIME		(200)		// 200ms
#define USB_HOST_CANCEL_COUNT		(1000/USB_HOST_CANCEL_TIME)
#define USB_HOST_CANCEL_TIMEOUT		(HZ/USB_HOST_CANCEL_COUNT)

/*-------------------------------------------------------------------------*/
// switch

/*-------------------------------------------------------------------------*/
#define MTP_DEVICE_MAJOR		220		// LOCAL/EXPERIMENTAL USE (-> Documentation/devices.txt)
#define MTP_DEVICE_MINOR		129		//
#define USB_MTP_MODULE_NAME		"g_mtp_transport"

#define DRIVER_DESC				"Media Transport Protcol Transport Gadget"
#define DRIVER_NAME				"g_mtp_transport"
#define DRIVER_VERSION			"0.1"

//-----------------------------------------------------------------------------
// I/O control
#define MTP_IOCTL_BASE 'M'

#define MTP_IOCTL_INITIALIZE		_IOR(MTP_IOCTL_BASE, 0,int)
#define MTP_IOCTL_RESET				_IOR(MTP_IOCTL_BASE, 1,int)
//#define MTP_IOCTL_SENDDATA		_IOR(MTP_IOCTL_BASE, 2,int)		// same as write op.
//#define MTP_IOCTL_RECEIVEDATA		_IOR(MTP_IOCTL_BASE, 3,int)		// same as read op.
#define MTP_IOCTL_STALL				_IOR(MTP_IOCTL_BASE, 4,int)
#define MTP_IOCTL_UNSTALL			_IOR(MTP_IOCTL_BASE, 5,int)
#define MTP_IOCTL_DISCONNECT		_IOR(MTP_IOCTL_BASE, 6,int)
#define MTP_IOCTL_CONNECT			_IOR(MTP_IOCTL_BASE, 7,int)
#define MTP_IOCTL_IS_CONNECTED		_IOR(MTP_IOCTL_BASE, 8,int)
#define MTP_IOCTL_INITWAIT			_IOR(MTP_IOCTL_BASE, 9,int)
#define MTP_IOCTL_SENDEVENT			_IOR(MTP_IOCTL_BASE,10,int)
#define MTP_IOCTL_SET_TX_FLAG		_IOR(MTP_IOCTL_BASE,11,int)
#define MTP_IOCTL_XCHG_STATUS		_IOR(MTP_IOCTL_BASE,12,int)
#define MTP_IOCTL_GET_VBUS_LEVEL	_IOR(MTP_IOCTL_BASE,13,int)
#define MTP_IOCTL_ABORT_TRANSFER	_IOR(MTP_IOCTL_BASE,14,int)

#define MTP_BULK_OUT_STALL		(0)
#define MTP_BULK_IN_STALL		(1)

// class request
#define USB_PTPREQUEST_TYPE_OUT   0x21	  /* Host to Device */
#define USB_PTPREQUEST_TYPE_IN	  0xA1	  /* Device to Host */
#define USB_PTPREQUEST_CANCELIO   0x64	  /* Cancel request */
#define USB_PTPREQUEST_GETEVENT   0x65	  /* Get extened event data */
#define USB_PTPREQUEST_RESET	  0x66	  /* Reset Device */
#define USB_PTPREQUEST_GETSTATUS  0x67	  /* Get Device Status */

#define USB_PTP_CANCELL_CODE	0x4001		// PTP Cancellation Code


//-----------------------------------------------------------------------------
// for Event
typedef struct mtp_event_type {
	char			*pBuffer;
	unsigned int	size;
	int				count;
} T_MTP_EVENT;

//-----------------------------------------------------------------------------
// cancel data(USB_PTPREQUEST_CANCELIO)
typedef struct tagCancelReqData {
	__u16	wCancelIOCode;
	__u32	TransactionId;
} __attribute__ ((packed))	USB_PTP_CANCELREQ_DATA, *PUSB_PTP_CANCELREQ_DATA;

//-----------------------------------------------------------------------------
// status data(USB_PTPREQUEST_GETSTATUS)
#define	STATUS_DEVICE_OK				0x2001
#define	STATUS_DEVICE_BUSY				0x2019
#define	STATUS_TRANSACTION_CANCELED		0x201F

#define GETSTATUS_HEADER_SIZE			(4)
#define DELAYED_STATUS	(512 + 999)	// An impossibly large value

typedef struct tagStatusReqData {
	__u16	wLength;
	__u16	Code;
	__u32	Params[4];
} __attribute__	((packed))	USB_PTP_STATUSREQ_DATA, *PUSB_PTP_STATUSREQ_DATA;

//-----------------------------------------------------------------------------
// MTP status(for check disconnect)
typedef enum {
	MTP_STATUS_OK = 0,
	MTP_STATUS_BULK_IN_WAIT,
	MTP_STATUS_BULK_OUT_WAIT,
	MTP_STATUS_INTR_WAIT
} eMTP_STATUS;

//-----------------------------------------------------------------------------
// for trade status MTP App and Device

/**
 * \enum eDevicePhase from MTP App.
 *
 * Defines which phase the device is currently in.
 */
typedef enum DevicePhase {
	DEVICE_PHASE_NOTREADY = 0,	/**< Device is in a busy state. */
	DEVICE_PHASE_IDLE = 1,		/**< Device is in an idle state. */
	DEVICE_PHASE_COMMAND = 2,	/**< Device is in the command phase. */
	DEVICE_PHASE_DATAIN = 3,	/**< Device is in the data in phase. */
	DEVICE_PHASE_DATAOUT = 4,	/**< Device is in the data out phase. */
	DEVICE_PHASE_RESP0NSE = 5	/**< Device is in the response phase. */
} eDevicePhase;

/**
 * \enum eDeviceStatus from MTP App.
 *
 * Defines the status a PTP camera could be in.
 */
typedef enum DeviceStatus {
	DEVICE_STATUSOK = 0,	  /**< Device OK. */
	DEVICE_LOWBATTERY = 1,	  /**< Fatal error, can't continue operation. */
	DEVICE_OUTOFMEMORY = 2,   /**< Out of memory, Can't continue. */
	DEVICE_STOREREMOVED = 3,  /**< Fatal error, especially if during a data transfer. */
	DEVICE_DEVICEERROR = 4	  /**< Unidentified fatal device error, can't continue. */
} eDeviceStatus;

typedef	struct {
	// app side
	__u32			TransactionID;
	eDevicePhase	Phase;
	__u8			CancelClear;		// MtpBool at app
	__u8			SuspendClear;
	__u8			ResetClear;
	eDeviceStatus	Status;

	// device size
	BOOL			b_Cancelling;
	BOOL			bBulkInCanceled;
	BOOL			bBulkOutCanceled;
	BOOL			bSuspend;
	BOOL			bReset;
	BOOL			bBusy;
} USB_MTP_XCHG_STATUS, *PUSB_MTP_XCHG_STATUS;


//-----------------------------------------------------------------------------
// MTP context for Device
struct mtp_device {
	/* list of usb_mtptrans devices */
	//struct list_head devs;
	dev_t	devcode;
	struct cdev cdev;
	int		major;
	int		minor;

	spinlock_t			lock;
	int		open_count;
	struct usb_gadget	*gadget;		// for Class
	struct usb_gadget	*org_gadget;		//

	struct completion	mtpthread_comp;		// wait disconnect and stop request

	struct semaphore	interrupt_comp;
	struct semaphore	bulk_comp;
	struct semaphore	mtpinit_comp;		// MTPdriver wait start(wait is app)
	struct semaphore	drv_reg_comp;		// MTP wait gadget regist

	BOOL			b_Vbus;			// VBus level (0:connect, 1:disconnect)
	__u8			config;			// connecting? (0:connect, 1:disconnect)
	BOOL			b_BulkInSetStall;	// Bulk IN is STALL?
	BOOL			b_BulkOutSetStall;	// Bulk OUT is STALL?
	BOOL			b_InitSuccess;		// Driver initialize is success?

	USB_MTP_XCHG_STATUS	devStatus;

	BOOL			b_Firstflg;		// first data receive flag
	BOOL			b_DataZeroRequired;	// data length'0' to do send?(1:send data length'0', set at host.)
	BOOL			b_BulkInWorking;
	BOOL			b_BulkOutWorking;
	BOOL			b_LastTransferIsIn;	// last transfer is in?

	struct usb_ep		*ep0;		// Handy copy of gadget->ep0
	struct usb_request	*ep0req;	// For control responses

	struct usb_ep		*bulk_in;
	struct usb_ep		*bulk_out;
	struct usb_ep		*intr_in;

	struct usb_request	*inreq;
	struct usb_request	*outreq;
	struct usb_request	*intreq;

	struct usb_endpoint_descriptor *bulk_in_desc;		//
	struct usb_endpoint_descriptor *bulk_out_desc;		//
	struct usb_endpoint_descriptor *intr_in_desc;		//

	struct semaphore	bulk_in_use_sem;
	struct semaphore	bulk_out_use_sem;
	struct semaphore	intr_in_use_sem;

	eMTP_STATUS			tx_status;		// transfer status

	__u8 *p_send_buf;
	__u8 *p_recv_buf;
	__u8 *p_event_buf;

	BOOL	b_RegistFlag;
	__u32	class_req_type;

	// cancel resource
	struct timer_list	cancel_timer;			//
	__u32	host_cancel_count;
	BOOL	b_timer_flag;

	__u32	status_count;						//
};

T_USB_CLASS_DRIVER g_mtp_regist_driver;




static const char longname[] = DRIVER_DESC;
static const char shortname[] = DRIVER_NAME;

MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_AUTHOR("NEC Electronics");
MODULE_LICENSE("Dual BSD/GPL");

/*******************************************************************************
*  Switch for Debug
*******************************************************************************/


/*******************************************************************************
*  Function
*******************************************************************************/

struct	mtp_device the_mtp;
struct	mtp_device *g_mtp = &the_mtp;

/* --------------------------------------------------------------------- */
// Proto type
static int _mtp_ioctl_waitinit(void);
static int _mtp_ioctl_initialize(void);
static int _mtp_ioctl_reset(void);
static int _mtp_ioctl_sendevent(char *pBuffer, unsigned int size, int *count);
static int _mtp_ioctl_disconnect(void);
static int _mtp_ioctl_connect(void);
static int _mtp_ioctl_isconnected(void);
static int _mtp_ioctl_stall(long arg);
static int _mtp_ioctl_unstall(long arg);
static int _mtp_ioctl_abort_transfer(void);

static int _mtp_ioctl_get_vbus_level(void);
static int _mtp_ioctl_xchg_status(PUSB_MTP_XCHG_STATUS devStatus);

static int usb_mtptrans_start(void);
static void usb_mtptrans_stop(void);

// callback for EP
static void ep0_complete(struct usb_ep *ep, struct usb_request *req);
static void bulk_in_complete(struct usb_ep *ep, struct usb_request *req);
static void bulk_out_complete(struct usb_ep *ep, struct usb_request *req);
static void intr_in_complete(struct usb_ep *ep, struct usb_request *req);

static int alloc_endpoint(void);
static int free_endpoint(void);

static void mtp_main_thread(struct usb_class_driver *p_class);
static	int do_set_config(u8 new_config);
static	int do_set_interface(int altsetting);

static	int mtp_bind(struct usb_gadget *gadget);
static void mtp_unbind(struct usb_gadget *gadget);
static void mtp_disconnect(struct usb_gadget *gadget);
static	int mtp_setup(struct usb_gadget *gadget, const struct usb_ctrlrequest *ctrl);
static void mtp_suspend(struct usb_gadget *gadget);
static void mtp_resume(struct usb_gadget *gadget);
static void mtp_vbusoff(struct usb_gadget *gadget);
static void mtp_usb_reset(struct usb_gadget *gadget);

static void mtp_free(void);

static void exe_cancel_io_req(struct usb_request *req);
static void exe_reset_req(struct usb_request *req);

static	int start_transfer(struct usb_ep *ep, struct usb_request *req);
static	int class_setup_req(const struct usb_ctrlrequest *ctrl);
static	int standard_setup_req(const struct usb_ctrlrequest *ctrl);

static void _nwm_mtp_cancel_timer_start(void);
static void _nwm_mtp_cancel_timer_restart(void);
static void _nwm_mtp_cancel_timer_stop(void);
static void _nwm_mtp_cancel_timer_fn(unsigned long arg);

static int _usb_mtp_read(char *buffer, size_t count);
static int _usb_mtp_write(const char *buffer, size_t count);


static struct usb_gadget_driver mtp_driver = {
#ifdef CONFIG_USB_GADGET_DUALSPEED
	.speed		= USB_SPEED_HIGH,
#else
	.speed		= USB_SPEED_FULL,
#endif
	.function	= (char *) longname,
	.bind		= mtp_bind,
	.unbind		= mtp_unbind,
	.disconnect	= mtp_disconnect,
	.setup		= mtp_setup,
	.suspend	= mtp_suspend,
	.vbusoff	= mtp_vbusoff,
	.reset		= mtp_usb_reset,
	.resume		= mtp_resume,

	.driver		= {
		.name		= (char *) shortname,
		// .release = ...
		// .suspend = ...
		// .resume = ...
	},
};

///////////////////////////////////////////////////////////////////////////////
// wait initialize
static int _mtp_ioctl_waitinit(void)	// __fn__
{
	int retcode;

	DBG_MSG("----- %s wait", __FUNCTION__);

	//---------------------------------------------------------------
	// MTP Driver wait initialize
	retcode = down_interruptible(&g_mtp->mtpinit_comp);
	if(retcode != 0){
		DBG_MSG("=== WAIT INIT Failed(INTERRUPT SIGNAL)");
		return -EINTR;
	}

	DBG_MSG("----- %s wakeup", __FUNCTION__);

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// initialize
static int _mtp_ioctl_initialize(void)		// __fn__
{
	DBG_MSG("%s() Call", __FUNCTION__);
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Reset
static int _mtp_ioctl_reset(void)			// __fn__
{
	int return_value;

	DBG_MSG("%s() Call", __FUNCTION__);

	return_value = _mtp_ioctl_disconnect();

	if(return_value != 0)
		return 1;

	return_value = _mtp_ioctl_initialize();

	if(return_value != 0)
		return 2;
	else
		return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Report a MTP event block through MTP Interrupt Endpoint
static int _mtp_ioctl_sendevent(char *pBuffer, unsigned int size, int *count)		// __fn__
{
	int retcode;

	DBG_MSG("%s() Call", __FUNCTION__);
	//
	// Do sanity checks
	//
	if(count){
		*count = 0;
	}

	if(!pBuffer){
		return 1;
	}

	down(&g_mtp->intr_in_use_sem);

	if (!g_mtp->config || g_mtp->devStatus.bReset || g_mtp->devStatus.bSuspend) {
		DBG_MSG("*** MTP Device Disconnected ***");
		up(&g_mtp->intr_in_use_sem);
		return USB_DEVICE_NOT_CONNECTED;
	}

	if (size > EVENT_BUFFER_SIZE) {
		DBG_MSG("%s too large write size", __FUNCTION__);
		up(&g_mtp->intr_in_use_sem);
		return USB_MTP_ERROR;
	}

	// User Memory -> DMA Buffer
	retcode = copy_from_user(g_mtp->intreq->buf, pBuffer, size);
	if(retcode != 0){
		DBG_MSG("=== %s copy_from_user() Failed", __FUNCTION__);
		up(&g_mtp->intr_in_use_sem);
		return USB_MTP_ERROR;
	}
	g_mtp->intreq->length = size;
	g_mtp->intreq->zero = 1;

	g_mtp->tx_status = MTP_STATUS_INTR_WAIT;		// set status
	retcode = start_transfer(g_mtp->intr_in, g_mtp->intreq);
	if(retcode != 0){
		ERR_MSG("*** _mtp_ioctl_sendevent() start_transfer retcode=%d",(int)retcode);
		g_mtp->tx_status = MTP_STATUS_OK;
		up(&g_mtp->intr_in_use_sem);
		return USB_MTP_ERROR;
	}

	retcode = down_interruptible(&g_mtp->interrupt_comp);	//wait
	if(retcode != 0){
		DBG_MSG("=== IOCTL Failed(INTERRUPT SIGNAL)");
		_mtp_ioctl_abort_transfer();
		retcode = -EINTR;
	}
	else{
		//-----------------------------------------------------------
		if(g_mtp->intreq->status != 0){
			INFO_MSGG("---- Interuupt IN status = %d",(int)g_mtp->intreq->status);
			retcode = USB_MTP_ERROR;
		}
		else{
			if(g_mtp->config && !g_mtp->devStatus.bReset){
				if( count ) {
					*count = g_mtp->intreq->actual;
				}
			}
			else{
				retcode = USB_DEVICE_NOT_CONNECTED;
			}
		}
	}
	up(&g_mtp->intr_in_use_sem);

	return retcode;
}

///////////////////////////////////////////////////////////////////////////////
// request disconnect
static int _mtp_ioctl_disconnect(void)	// __fn__
{
	int		nret=0;

	DBG_MSG("%s() Enter", __FUNCTION__);

	if(g_mtp->org_gadget != NULL)
		nret = usb_gadget_disconnect(g_mtp->org_gadget);

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
// request connect
static int _mtp_ioctl_connect(void)		// __fn__
{
	int		nret=0;

	DBG_MSG("%s() Enter", __FUNCTION__);

	if(g_mtp->org_gadget != NULL)
		nret = usb_gadget_connect(g_mtp->org_gadget);

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
// check connect status
static int _mtp_ioctl_isconnected(void)	// __fn__
{
	DBG_MSGG("%s() Enter", __FUNCTION__);
	if(g_mtp->config != 0)
		return 0;	// connect
	else
		return 1;	// disconnect
}

///////////////////////////////////////////////////////////////////////////////
// check Vbus status
static int _mtp_ioctl_get_vbus_level(void)	// __fn__
{
	if(g_mtp->b_Vbus == FALSE)
		return 1;	// 1 : disconnect
	else
		return 0;	// 0 : connect
}

///////////////////////////////////////////////////////////////////////////////
// change status from app
static int _mtp_ioctl_xchg_status(PUSB_MTP_XCHG_STATUS devStatus)	// __fn__
{
	PUSB_MTP_XCHG_STATUS	ds = &g_mtp->devStatus;

	DBG_MSGG("---  _mtp_ioctl_xchg_status");

	// status from app
	ds->Phase = devStatus->Phase;

	if (ds->bReset && devStatus->ResetClear) {
		DBG_MSGG("!!! Reset Clear !!!");
		ds->bReset = FALSE;
	}

	if(devStatus->CancelClear != FALSE){
		if((ds->bBulkInCanceled == FALSE) && (ds->bBulkOutCanceled == FALSE))
			ds->b_Cancelling = FALSE;
	}

	ds->Status = devStatus->Status;
	ds->bBusy  = devStatus->bBusy;

	if(ds->bBusy != FALSE){
		ds->TransactionID = devStatus->TransactionID;
	}

	// return to app
	devStatus->b_Cancelling		= ds->b_Cancelling;
	devStatus->bBulkInCanceled	= ds->bBulkInCanceled;
	devStatus->bBulkOutCanceled	= ds->bBulkOutCanceled;
	devStatus->bSuspend			= ds->bSuspend;
	devStatus->bReset			= ds->bReset;

	return	0;
}

///////////////////////////////////////////////////////////////////////////////
// Stall the BULKIN/OUT endpoints
static int _mtp_ioctl_stall(long arg)			// __fn__
{
	int		nret=0;

	if(g_mtp->b_Vbus == FALSE){
		return nret;
	}

	if (arg) {
		DBG_MSG("--- Bulk IN Set Halt");
		g_mtp->b_BulkInSetStall = TRUE;
		nret = usb_ep_set_halt(g_mtp->bulk_in);
	} else {
		DBG_MSG("--- Bulk OUT Set Halt");
		g_mtp->b_BulkOutSetStall = TRUE;
		nret = usb_ep_set_halt(g_mtp->bulk_out);
	}

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
// unstall the BULKIN/OUT Endpoints
static int _mtp_ioctl_unstall(long arg)			// __fn__
{
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;
	int		nret=0;
	struct usb_ep *ep;

	if(g_mtp->b_Vbus == FALSE){
		return nret;
	}

	if (arg) {
		if(g_mtp->b_BulkInSetStall){
			ep = g_mtp->bulk_in;
			usb_ep_fifo_flush(ep);
			DBG_MSG("--- Bulk IN Clear Stall");
			nret = usb_ep_clear_stall(ep);
			g_mtp->b_BulkInSetStall = FALSE;
		}
		ds->bBulkInCanceled = FALSE;
	}
	else{
		if(g_mtp->b_BulkOutSetStall){
			ep = g_mtp->bulk_out;
			usb_ep_fifo_flush(ep);
			DBG_MSG("--- Bulk OUT Clear Stall");
			nret = usb_ep_clear_stall(ep);
			g_mtp->b_BulkOutSetStall = FALSE;
		}
		ds->bBulkOutCanceled = FALSE;
	}

//	if((ds->bBulkInCanceled == FALSE) && (ds->bBulkOutCanceled == FALSE)){
//		ds->b_Cancelling = FALSE;
//	}

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
//
static int _mtp_ioctl_abort_transfer()		// __fn__
{
	int		nret=0;

	DBG_MSGG("%s() Enter", __FUNCTION__);

	if(g_mtp->b_Vbus == FALSE){
		return nret;
	}

	_mtp_ioctl_stall(MTP_BULK_IN_STALL);
	if(g_mtp->b_BulkInWorking){
		nret = usb_ep_dequeue(g_mtp->bulk_in, g_mtp->inreq);
	}

	_mtp_ioctl_stall(MTP_BULK_OUT_STALL);
	if(g_mtp->b_BulkOutWorking){
		nret = usb_ep_dequeue(g_mtp->bulk_out, g_mtp->outreq);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
static void ep0_complete(struct usb_ep *ep, struct usb_request *req)	// __fn__
{
	switch(g_mtp->class_req_type){
		case USB_PTPREQUEST_CANCELIO:
			exe_cancel_io_req(req);
			break;

		case USB_PTPREQUEST_RESET:
			exe_reset_req(req);
			break;

		default:
			break;
	}

	DBG_MSGG("----- %s()", __FUNCTION__);

	g_mtp->class_req_type = 0;
}

///////////////////////////////////////////////////////////////////////////////
//
static int start_transfer(struct usb_ep *ep, struct usb_request *req)	// __fn__
{
	int		rc;

	rc = usb_ep_queue(ep, req, GFP_KERNEL);

	return rc;
}

///////////////////////////////////////////////////////////////////////////////
// callback BULK_IN finished
static void bulk_in_complete(struct usb_ep *ep, struct usb_request *req)	// __fn__
{
	DBG_MSGG("----- %s(%d)", __FUNCTION__, req->actual);

	if(req->status == -ECONNRESET){
		usb_ep_fifo_flush(ep);

		if(req->actual > ep->maxpacket)
			req->actual -= ep->maxpacket;
		else
			req->actual = 0;
	}

	if(g_mtp->tx_status == MTP_STATUS_BULK_IN_WAIT){
		g_mtp->tx_status = MTP_STATUS_OK;
		up(&g_mtp->bulk_comp);
	}
	else{
		DBG_MSG("%s() req->status = %d", __FUNCTION__, req->status);
		DBG_MSG("%s() tx_status = %d", __FUNCTION__, g_mtp->tx_status);
	}
}

///////////////////////////////////////////////////////////////////////////////
// callback BULK_OUT finished
static void bulk_out_complete(struct usb_ep *ep, struct usb_request *req)		// __fn__
{
	DBG_MSGG("----- %s(%d)", __FUNCTION__, req->actual);

	if(req->status == -ECONNRESET){
		INFO_MSGG("----- %s(%d)", __FUNCTION__, req->actual);
		usb_ep_fifo_flush(ep);
	}

	if(g_mtp->tx_status == MTP_STATUS_BULK_OUT_WAIT){
		g_mtp->tx_status = MTP_STATUS_OK;	// set status
		if(g_mtp->host_cancel_count != 0){
			g_mtp->host_cancel_count = 0;
			req->status = -ETIMEDOUT;
		}
		up(&g_mtp->bulk_comp);
	}
	else{
		DBG_MSGG("%s() req->status = %d", __FUNCTION__, req->status);
		DBG_MSGG("%s() tx_status = %d", __FUNCTION__, g_mtp->tx_status);
	}
}

///////////////////////////////////////////////////////////////////////////////
// callback INTERRUPT_IN finished
static void intr_in_complete(struct usb_ep *ep, struct usb_request *req)		// __fn__
{
	if(req->status == -ECONNRESET){
		usb_ep_fifo_flush(ep);
	}

	if(g_mtp->tx_status == MTP_STATUS_INTR_WAIT){
		g_mtp->tx_status = MTP_STATUS_OK;		// set status
		up(&g_mtp->interrupt_comp);
	}
}

/////////////////////////////////////////////////////////////////////
// memory allocate for Endpoint
//	 Endpoint information and allocate transfer memory buffer
static int alloc_endpoint()	// __fn__
{
	sema_init(&g_mtp->interrupt_comp, 0);
	sema_init(&g_mtp->bulk_comp, 0);

	//---------------------------------------------------------------
	// Bulk IN
	g_mtp->inreq = usb_ep_alloc_request(g_mtp->bulk_in, GFP_ATOMIC);
	if(g_mtp->inreq == NULL) return -ENOMEM;

	g_mtp->p_send_buf = kmalloc(
			WRITE_BUFFER_SIZE,
			GFP_KERNEL);
	if(g_mtp->p_send_buf == NULL) return -ENOMEM;

	g_mtp->inreq->complete = bulk_in_complete;
	g_mtp->inreq->buf = g_mtp->p_send_buf;

	//---------------------------------------------------------------
	// Bulk OUT
	g_mtp->outreq = usb_ep_alloc_request(g_mtp->bulk_out, GFP_ATOMIC);
	if(g_mtp->outreq == NULL) return -ENOMEM;

	g_mtp->p_recv_buf = kmalloc(
			READ_BUFFER_SIZE,
			GFP_KERNEL);
	if(g_mtp->p_recv_buf == NULL) return -ENOMEM;

	g_mtp->outreq->complete = bulk_out_complete;
	g_mtp->outreq->buf = g_mtp->p_recv_buf;

	//---------------------------------------------------------------
	// Interrupt IN
	g_mtp->intreq = usb_ep_alloc_request(g_mtp->intr_in, GFP_ATOMIC);
	if(g_mtp->intreq == NULL) return -ENOMEM;

	g_mtp->p_event_buf = kmalloc(
			EVENT_BUFFER_SIZE,
			GFP_KERNEL);
	if(g_mtp->p_event_buf == NULL) return -ENOMEM;

	g_mtp->intreq->complete = intr_in_complete;
	g_mtp->intreq->buf = g_mtp->p_event_buf;

	return 0;
}

/////////////////////////////////////////////////////////////////////
// free memory for Endpoint
//	 free Endpoint information and transfer memory buffer
static int free_endpoint()	// __fn__
{
	//---------------------------------------------------------------
	// Interrupt IN
	if(g_mtp->intreq){
		down(&g_mtp->intr_in_use_sem);

		if(g_mtp->p_event_buf != NULL){
			kfree(g_mtp->intreq->buf);
			g_mtp->p_event_buf = NULL;
		}

		usb_ep_free_request(g_mtp->intr_in, g_mtp->intreq);
		g_mtp->intreq = NULL;
		up(&g_mtp->intr_in_use_sem);
	}

	//---------------------------------------------------------------
	// Bulk OUT
	if(g_mtp->outreq){
		down(&g_mtp->bulk_out_use_sem);

		if(g_mtp->p_recv_buf != NULL){
			kfree(g_mtp->outreq->buf);
			g_mtp->p_recv_buf = NULL;
		}

		usb_ep_free_request(g_mtp->bulk_out, g_mtp->outreq);
		g_mtp->outreq = NULL;
		up(&g_mtp->bulk_out_use_sem);
	}

	//---------------------------------------------------------------
	// Bulk IN
	if(g_mtp->inreq){
		down(&g_mtp->bulk_in_use_sem);

		if(g_mtp->p_send_buf != NULL){
			kfree(g_mtp->inreq->buf);
			g_mtp->p_send_buf = NULL;
		}

		usb_ep_free_request(g_mtp->bulk_in, g_mtp->inreq);
		g_mtp->inreq = NULL;
		up(&g_mtp->bulk_in_use_sem);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Main Loop
static void mtp_main_thread(struct usb_class_driver *p_class)	// __fn__
{
	DBG_MSG("=================================");
	DBG_MSG("---  mtp_main_thread START");

	g_mtp->config = 1;
	alloc_endpoint();

	g_mtp->bulk_in_desc = p_class->bulk_in_desc;
	g_mtp->bulk_out_desc = p_class->bulk_out_desc;
	g_mtp->intr_in_desc = p_class->intr_in_desc;

	g_mtp->devStatus.bBulkInCanceled	= FALSE;
	g_mtp->devStatus.bBulkOutCanceled	= FALSE;
	g_mtp->devStatus.b_Cancelling		= FALSE;
	g_mtp->devStatus.bReset				= FALSE;
	g_mtp->devStatus.bBusy				= FALSE;
	g_mtp->b_BulkInWorking				= FALSE;
	g_mtp->b_BulkOutWorking				= FALSE;

	p_class->running = 1;

	if(g_mtp->b_InitSuccess != FALSE){
		up(&g_mtp->mtpinit_comp);	// initialize finished
	}

	//---------------------------------------------------------------
	// first data is received NWM, analyze is this
	g_mtp->outreq->actual = p_class->bulk_out_req->actual;
	g_mtp->outreq->status = p_class->bulk_out_req->status;
	memcpy(g_mtp->outreq->buf, p_class->bulk_out_req->buf, g_mtp->outreq->actual);
	DBG_MSG("---  Got first bulk data, actual=%d", g_mtp->outreq->actual);

	if(p_class->running != 0){
		//-----------------------------------------------------------
		// wait request discconect and stop
		wait_for_completion(&g_mtp->mtpthread_comp);
	}

	free_endpoint();
	p_class->running = 0;

	DBG_MSG("---  mtp_main_thread END");
	DBG_MSG("=================================");
}

///////////////////////////////////////////////////////////////////////////////
// SET_CONFIGURATION
static int do_set_config(u8 new_config)	// __fn__
{
	int		rc=0;

	DBG_MSGG("----- %s() Start", __FUNCTION__);
	DBG_MSGG("----- %s() new_config = %d, config = %d", __FUNCTION__, new_config, g_mtp->config);

	if((new_config == 0) && (g_mtp->config == 0)){
		return rc;
	}

	rc = do_set_interface(-1);

	DBG_MSGG("----- %s(%d) End", __FUNCTION__, new_config);

	return rc;
}

///////////////////////////////////////////////////////////////////////////////
// SET_INTERFACE
static int do_set_interface(int altsetting)	// __fn__
{
	PT_USB_CLASS_DRIVER p_class;

	DBG_MSGG("----- %s(%d) Start", __FUNCTION__, altsetting);
	DBG_MSGG("----- %s(%d) Enter", __FUNCTION__, altsetting);

	g_mtp->b_Firstflg = TRUE;
	g_mtp->config = 0;

	//---------------------------------------------------------------
	if(g_mtp->tx_status == MTP_STATUS_INTR_WAIT){
		g_mtp->tx_status = MTP_STATUS_OK;
		up(&g_mtp->interrupt_comp);	// wake up wait interrupt
	}

	if((g_mtp->tx_status == MTP_STATUS_BULK_IN_WAIT) || (g_mtp->tx_status == MTP_STATUS_BULK_OUT_WAIT)){
		g_mtp->tx_status = MTP_STATUS_OK;
		up(&g_mtp->bulk_comp);		// wakeup wait read/write
	}

	p_class = &g_mtp_regist_driver;
	if(p_class->running){
		complete(&g_mtp->mtpthread_comp);	// wake up main
	}

	memset(&g_mtp->devStatus, 0, sizeof(USB_MTP_XCHG_STATUS));
	if(altsetting < 0) return 0;

	DBG_MSGG("----- %s() End", __FUNCTION__);

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Class Cancel 
static int do_cancel_io_req(struct usb_request *req)	// __fn__
{
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	if(g_mtp->b_LastTransferIsIn){
		ds->bBulkInCanceled = TRUE;
	}
	else{
		ds->bBulkOutCanceled = TRUE;
	}

	return sizeof(USB_PTP_CANCELREQ_DATA);
}

///////////////////////////////////////////////////////////////////////////////
// Class Reset
static int do_reset_req(struct usb_request *req)	// __fn__
{
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	ds->bReset = TRUE;

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Class Status
static int do_get_status_req(struct usb_request *req)	// __fn__
{
	USB_PTP_STATUSREQ_DATA	s_Status;
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	s_Status.wLength = GETSTATUS_HEADER_SIZE;

	if(ds->b_Cancelling != FALSE){
		s_Status.Code = STATUS_DEVICE_BUSY;
	}
	else{
		s_Status.Code = STATUS_DEVICE_OK;
	}

	if((ds->bBulkOutCanceled == FALSE) && (ds->bBulkInCanceled == FALSE)){
		ds->b_Cancelling = FALSE;

		if(g_mtp->b_BulkInSetStall) _mtp_ioctl_unstall(MTP_BULK_IN_STALL);
		if(g_mtp->b_BulkOutSetStall) _mtp_ioctl_unstall(MTP_BULK_OUT_STALL);
	}

	memcpy(req->buf, &s_Status, s_Status.wLength);

	return s_Status.wLength;
}

///////////////////////////////////////////////////////////////////////////////
// Class Cancel
static void exe_cancel_io_req(struct usb_request *req)		// __fn__
{
	USB_PTP_CANCELREQ_DATA	s_Cancel;
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	DBG_MSG("----- %s() Start", __FUNCTION__);

	memcpy(&s_Cancel, req->buf, sizeof(USB_PTP_CANCELREQ_DATA));
	INFO_MSGG("wCancelIOCode = 0x%x, TransactionId = 0x%x", s_Cancel.wCancelIOCode, s_Cancel.TransactionId);
	INFO_MSGG("                      TransactionID = 0x%x", ds->TransactionID);

	if((s_Cancel.wCancelIOCode == USB_PTP_CANCELL_CODE)
			&& (s_Cancel.TransactionId == ds->TransactionID)){

		g_mtp->host_cancel_count = 0;

		if(ds->bBulkInCanceled != FALSE){
			ds->b_Cancelling = TRUE;
			if(g_mtp->b_BulkInWorking != FALSE){
				usb_ep_dequeue(g_mtp->bulk_in, g_mtp->inreq);
				g_mtp->b_BulkInWorking = FALSE;
			}
		}

		if(ds->bBulkOutCanceled != FALSE){
			ds->b_Cancelling = TRUE;
			if(g_mtp->b_BulkOutWorking != FALSE){
				usb_ep_dequeue(g_mtp->bulk_out, g_mtp->outreq);
				g_mtp->b_BulkOutWorking = FALSE;
			}
		}
	}
	else{
		INFO_MSG(" TransactionID Failed !!");
		ds->bBulkInCanceled = FALSE;
		ds->bBulkOutCanceled = FALSE;
	}

	DBG_MSGG("----- %s() End", __FUNCTION__);
}

///////////////////////////////////////////////////////////////////////////////
// Class Reset
static void exe_reset_req(struct usb_request *req)		// __fn__
{
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	if(ds->bReset != FALSE){
		if(g_mtp->b_BulkInWorking){
			usb_ep_dequeue(g_mtp->bulk_in, g_mtp->inreq);
		}

		if(g_mtp->b_BulkOutWorking){
			usb_ep_dequeue(g_mtp->bulk_out, g_mtp->outreq);
		}
	}

	ds->bBulkInCanceled = FALSE;
	ds->bBulkOutCanceled = FALSE;
	ds->b_Cancelling = FALSE;
}

///////////////////////////////////////////////////////////////////////////////
// Class request
static int class_setup_req(const struct usb_ctrlrequest *ctrl)	// __fn__
{
	struct usb_request *req = g_mtp->ep0req;
	int		value = -EOPNOTSUPP;
	u16		w_index = le16_to_cpu(ctrl->wIndex);
	u16		w_length = le16_to_cpu(ctrl->wLength);
	u16		w_value = le16_to_cpu(ctrl->wValue);

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	g_mtp->class_req_type = ctrl->bRequest;

	/* Handle Bulk-only class-specific requests */
	switch (ctrl->bRequest) {
	case USB_PTPREQUEST_CANCELIO:		// 0x64
		DBG_MSGG("USB_PTPREQUEST_CANCELIO");

		if ((w_value == 0x00) && (w_index == 0x00) && (w_length == 0x06)) {
			value = do_cancel_io_req(req);
		}
		break;

	case USB_PTPREQUEST_GETEVENT:		// 0x65
		DBG_MSGG("USB_PTPREQUEST_GETEVENT");
		break;

	case USB_PTPREQUEST_RESET:			// 0x66
		DBG_MSGG("USB_PTPREQUEST_RESET");

		if ((w_value == 0x00) && (w_index == 0x00) && (w_length == 0x00)) {
			value = do_reset_req(req);
		}
		break;

	case USB_PTPREQUEST_GETSTATUS:		// 0x67
		DBG_MSGG("USB_PTPREQUEST_GETSTATUS");
		if ((w_value == 0x00) && (w_index == 0x00)){
			value = do_get_status_req(req);
		}
		break;

	default:
		DBG_MSGG("Unknown Request");
		break;
	}

	DBG_MSGG("----- %s() End", __FUNCTION__);

	return value;
}

///////////////////////////////////////////////////////////////////////////////
// standard request
static int standard_setup_req(const struct usb_ctrlrequest *ctrl)	// __fn__
{
	int			value = -EOPNOTSUPP;
	u16			w_index = le16_to_cpu(ctrl->wIndex);
	u16			w_value = le16_to_cpu(ctrl->wValue);

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	switch (ctrl->bRequest) {
		case USB_REQ_GET_DESCRIPTOR:
			if (ctrl->bRequestType != (USB_DIR_IN | USB_TYPE_STANDARD |
					USB_RECIP_DEVICE))
				break;
			switch (w_value >> 8) {

				case USB_DT_DEVICE:
					value = 0;
					break;
#ifdef CONFIG_USB_GADGET_DUALSPEED
				case USB_DT_DEVICE_QUALIFIER:
					value = 0;
					break;

				case USB_DT_OTHER_SPEED_CONFIG:
					value = 0;
					break;
#endif
				case USB_DT_CONFIG:
					value = 0;
					break;

				case USB_DT_STRING:
					value = 0;
					break;
				}
			break;

		case USB_REQ_SET_CONFIGURATION:
			if (ctrl->bRequestType != (USB_DIR_OUT | USB_TYPE_STANDARD |
					USB_RECIP_DEVICE))
				break;

			if (w_value == 1 || w_value == 0) {
				do_set_config(w_value);
				value = DELAYED_STATUS;
			}
			break;

		case USB_REQ_GET_CONFIGURATION:
			if (ctrl->bRequestType != (USB_DIR_IN | USB_TYPE_STANDARD |
					USB_RECIP_DEVICE))
				break;
			value = 0;
			break;

		case USB_REQ_SET_INTERFACE:
			if (ctrl->bRequestType != (USB_DIR_OUT| USB_TYPE_STANDARD |
					USB_RECIP_INTERFACE))
				break;
			if (g_mtp->config && w_index == 0) {
				do_set_interface(-1);
				value = DELAYED_STATUS;
			}
			break;

		case USB_REQ_GET_INTERFACE:
			value = 0;
			break;

		default:
			break;
	}

	DBG_MSGG("----- %s() End", __FUNCTION__);

	return value;
}

///////////////////////////////////////////////////////////////////////////////
// bind() callback
static int mtp_bind(struct usb_gadget *gadget)	// __fn__
{
	PT_USB_CLASS_DRIVER p_class;

	DBG_MSG("----- %s() Start", __FUNCTION__);

	g_mtp->gadget = gadget;
	set_gadget_data(gadget, g_mtp);
	g_mtp->ep0 = gadget->ep0;

	p_class = &g_mtp_regist_driver;
	g_mtp->org_gadget = p_class->org_gadget;
	g_mtp->ep0req = p_class->ep0req;
	g_mtp->bulk_in = p_class->bulk_in;
	g_mtp->bulk_out = p_class->bulk_out;
	g_mtp->intr_in = p_class->intr_in;

	up(&g_mtp->drv_reg_comp);

	usb_mtptrans_start();

	DBG_MSG("----- %s() End", __FUNCTION__);

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// unbind() callback
static void mtp_unbind(struct usb_gadget *gadget)	// __fn__
{
	DBG_MSG("----- %s() Start", __FUNCTION__);

	set_gadget_data(gadget, NULL);
	g_mtp->gadget = NULL;
	g_mtp->org_gadget = NULL;

	DBG_MSG("----- %s() End", __FUNCTION__);
}

///////////////////////////////////////////////////////////////////////////////
// disconnect
static void mtp_disconnect(struct usb_gadget *gadget)	// __fn__
{
	unsigned long	flags;

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	spin_lock_irqsave(&g_mtp->lock, flags);
	do_set_config(0);
	g_mtp->b_Vbus = FALSE;
	spin_unlock_irqrestore(&g_mtp->lock, flags);

	DBG_MSGG("----- %s() End", __FUNCTION__);
}

///////////////////////////////////////////////////////////////////////////////
// SETUP
static int mtp_setup(struct usb_gadget *gadget,	// __fn__
		const struct usb_ctrlrequest *ctrl)
{
	int			rc;

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	g_mtp->b_Vbus = TRUE;
	g_mtp->devStatus.bSuspend = FALSE;

	if ((ctrl->bRequestType & USB_TYPE_MASK) == USB_TYPE_CLASS)
		rc = class_setup_req(ctrl);
	else
		rc = standard_setup_req(ctrl);

	DBG_MSGG("----- %s(%d) End", __FUNCTION__, rc);

	return rc;
}

///////////////////////////////////////////////////////////////////////////////
// SUSPEND
static void mtp_suspend(struct usb_gadget *gadget)	// __fn__
{
	unsigned long	flags;
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	ds->bSuspend = TRUE;

	if(g_mtp->b_BulkInWorking){
		usb_ep_dequeue(g_mtp->bulk_in, g_mtp->inreq);
	}

	if(g_mtp->b_BulkOutWorking){
		usb_ep_dequeue(g_mtp->bulk_out, g_mtp->outreq);
	}

	//---------------------------------------------------------------
	// this call is for return to NWM Driver at USB Suspend
	spin_lock_irqsave(&g_mtp->lock, flags);
	do_set_config(0);
	spin_unlock_irqrestore(&g_mtp->lock, flags);
}


///////////////////////////////////////////////////////////////////////////////
// RESUME
static void mtp_resume(struct usb_gadget *gadget)	// __fn__
{
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	ds->bSuspend = FALSE;
}

///////////////////////////////////////////////////////////////////////////////
// Vbus OFF callback
static void mtp_vbusoff(struct usb_gadget *gadget)	// __fn__
{
	unsigned long	flags;

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	spin_lock_irqsave(&g_mtp->lock, flags);
	do_set_config(0);
	g_mtp->b_Vbus = FALSE;
	spin_unlock_irqrestore(&g_mtp->lock, flags);

	DBG_MSGG("----- %s() End", __FUNCTION__);
}

///////////////////////////////////////////////////////////////////////////////
// USB RESET
static void mtp_usb_reset(struct usb_gadget *gadget)	// __fn__
{
	unsigned long	flags;

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	spin_lock_irqsave(&g_mtp->lock, flags);
	do_set_config(0);
	spin_unlock_irqrestore(&g_mtp->lock, flags);

	DBG_MSGG("----- %s() End", __FUNCTION__);
}

///////////////////////////////////////////////////////////////////////////////
// Timer Start
static void _nwm_mtp_cancel_timer_restart()	// __fn__
{
	if(g_mtp->b_timer_flag != FALSE){
		mod_timer(&g_mtp->cancel_timer, jiffies + USB_HOST_CANCEL_TIMEOUT);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Timer Init&Start
static void _nwm_mtp_cancel_timer_start()	// __fn__
{
	init_timer(&g_mtp->cancel_timer);

	g_mtp->host_cancel_count = 0;
	g_mtp->b_timer_flag = TRUE;

	g_mtp->cancel_timer.data = (unsigned long)g_mtp;
	g_mtp->cancel_timer.function = _nwm_mtp_cancel_timer_fn;
	g_mtp->cancel_timer.expires  = jiffies + USB_HOST_CANCEL_TIMEOUT;
	add_timer(&g_mtp->cancel_timer);
}

///////////////////////////////////////////////////////////////////////////////
// Timer Stop
static void _nwm_mtp_cancel_timer_stop()	// __fn__
{
	if(g_mtp->b_timer_flag != FALSE){
		del_timer(&g_mtp->cancel_timer);
		g_mtp->b_timer_flag = FALSE;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Timer
static void _nwm_mtp_cancel_timer_fn(unsigned long arg)	// __fn__
{
	g_mtp->host_cancel_count++;
	if(g_mtp->b_BulkOutWorking)
		usb_ep_dequeue(g_mtp->bulk_out, g_mtp->outreq);
	_nwm_mtp_cancel_timer_stop();
}

///////////////////////////////////////////////////////////////////////////////
// Cancel
static int _usb_mtp_host_cancel(size_t count)	// __fn__
{
	int		nret=0;

	DBG_MSGG("----- %s() Start", __FUNCTION__);

	g_mtp->b_BulkOutWorking = TRUE;
	g_mtp->outreq->length = g_mtp->bulk_out->maxpacket << 2;

	_nwm_mtp_cancel_timer_start();

	while(1){
		if(g_mtp->b_BulkOutWorking == FALSE){
			break;
		}

		g_mtp->tx_status = MTP_STATUS_BULK_OUT_WAIT;
		nret = start_transfer(g_mtp->bulk_out, g_mtp->outreq);
		if(nret != 0){
			ERR_MSG("%s() start_transfer nret=%d", __FUNCTION__, nret);
			g_mtp->b_BulkOutWorking = FALSE;
			nret = USB_MTP_ERROR;
		}

		_nwm_mtp_cancel_timer_restart();
		nret = down_interruptible(&g_mtp->bulk_comp);	//wait
		if(nret != 0){
			INFO_MSGG("=== Bulk OUT Failed(INTERRUPT SIGNAL)");
			_mtp_ioctl_abort_transfer();
			g_mtp->b_BulkOutWorking = FALSE;
			nret = -EINTR;
		}

		if(g_mtp->outreq->status == -ETIMEDOUT){
			INFO_MSGG("----- Host Cancel Timeout");
			g_mtp->b_BulkOutWorking = FALSE;
			nret = 0;
		}
	}

	DBG_MSGG("----- %s() End", __FUNCTION__);

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
// Read
static int _usb_mtp_read(char *buffer, size_t count)	// __fn__
{
	int		retcode;
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	DBG_MSGG("----- %s() Enter", __FUNCTION__);

	if (!g_mtp->config || ds->bReset || ds->bSuspend) {
		INFO_MSG("=== MTP Device Disconnected");
		ds->bReset = FALSE;		// case reset
		return USB_DEVICE_NOT_CONNECTED;
	}

	if (count > READ_BUFFER_SIZE) {
		ERR_MSG("%s too large read size", __FUNCTION__);
		return USB_MTP_ERROR;
	}

	//---------------------------------------------------------------
	// Host Cancel
	if(ds->b_Cancelling != FALSE){
		INFO_MSGG("--- _usb_mtp_host_cancel()");
		retcode = _usb_mtp_host_cancel(count);
		if(retcode == 0){
			ds->bBulkOutCanceled = FALSE;
			ds->bBulkInCanceled = FALSE;
			INFO_MSGG("----- Host Cancel End");
			return USB_HOST_CANCELING;
		}
		return 0;
	}

	g_mtp->b_LastTransferIsIn = FALSE;

	if (g_mtp->b_DataZeroRequired) {
		g_mtp->outreq->zero = 1;
	} else {
		g_mtp->outreq->zero = 0;
	}

	//---------------------------------------------------------------
	// data transfer
	if (g_mtp->b_Firstflg == FALSE) {
		g_mtp->b_BulkOutWorking = TRUE;
		g_mtp->outreq->length = count;
		g_mtp->tx_status = MTP_STATUS_BULK_OUT_WAIT;		// set status
		retcode = start_transfer(g_mtp->bulk_out, g_mtp->outreq);
		if(retcode != 0){
			ERR_MSG("%s() start_transfer retcode=%d", __FUNCTION__, retcode);
			g_mtp->b_BulkOutWorking = FALSE;
			g_mtp->tx_status = MTP_STATUS_OK;
			return USB_MTP_ERROR;
		}

		//---------------------------------------------------------------
		// wait transfer finish
		retcode = down_interruptible(&g_mtp->bulk_comp);	//wait
		if(retcode != 0){
			INFO_MSG("=== Bulk OUT Failed(INTERRUPT SIGNAL)");
			if(g_mtp->outreq->status != 0){
				if(g_mtp->outreq->status == -EINPROGRESS){
					usb_ep_dequeue(g_mtp->bulk_out, g_mtp->outreq);
					down_interruptible(&g_mtp->bulk_comp);
				}
				else{
					g_mtp->b_BulkOutWorking = FALSE;
					return -EINTR;
				}
			}
		}
	}
	else{
		//-----------------------------------------------------------
		// first Read(Bulk OUT) is exec at NWM driver,
		// therefore not call start_transfer()
		g_mtp->b_Firstflg = FALSE;
	}

	g_mtp->b_BulkOutWorking = FALSE;

	//---------------------------------------------------------------
	// check data transfer status
	if(g_mtp->outreq->status != 0){
		return USB_MTP_ERROR;
	}

	//---------------------------------------------------------------
	// check transfer data
	if (g_mtp->config && !ds->bReset) {
		DBG_MSGG("---- bulkout actual_len = %d",(int)g_mtp->outreq->actual);
		DBG_MSGG("---- bulkout status = %d",(int)g_mtp->outreq->status);

		//-----------------------------------------------------------
		// happen cancel, but alreay finished data send transfer.
		// this case not set Error
		if(ds->bBulkOutCanceled != FALSE){
			if(g_mtp->outreq->actual != count){
				return USB_MTP_ERROR;
			}
		}

		retcode = copy_to_user(buffer, g_mtp->outreq->buf, g_mtp->outreq->actual);
		if(retcode != 0){
			return USB_MTP_ERROR;
		}
		else{
			return g_mtp->outreq->actual;
		}
	}
	else {
		INFO_MSGG("=== Bulk OUT Failed(Disconnect/Aborted)");
		return USB_MTP_ERROR;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Write
static int _usb_mtp_write(const char *buffer, size_t count)	// __fn__
{
	int		retcode;
	PUSB_MTP_XCHG_STATUS	ds=&g_mtp->devStatus;

	DBG_MSGG("----- %s(%d)", __FUNCTION__, count);

	if (!g_mtp->config || ds->bReset || ds->bSuspend) {
		DBG_MSG("=== MTP Device Disconnected");
		return USB_DEVICE_NOT_CONNECTED;
	}

	if (count > WRITE_BUFFER_SIZE) {
		DBG_MSG("%s too large write size", __FUNCTION__);
		return USB_MTP_ERROR;
	}

	if ((ds->b_Cancelling) || (ds->bBulkInCanceled)) {
		DBG_MSG("=== MTP device write cancelling");
		return USB_MTP_ERROR;
	}

	g_mtp->b_LastTransferIsIn = TRUE;

	// User Memory -> Kernel Buffer
	retcode = copy_from_user(g_mtp->inreq->buf, buffer, count);
	if(retcode != 0){
		DBG_MSG("=== %s copy_from_user() Failed", __FUNCTION__);
		return USB_MTP_ERROR;
	}

	g_mtp->inreq->length = count;
	if (g_mtp->b_DataZeroRequired) {
		g_mtp->inreq->zero = 1;
	} else {
		g_mtp->inreq->zero = 0;
	}

	//---------------------------------------------------------------
	// data transfer
	g_mtp->b_BulkInWorking = TRUE;
	g_mtp->tx_status = MTP_STATUS_BULK_IN_WAIT;		// set status
	retcode = start_transfer(g_mtp->bulk_in, g_mtp->inreq);
	if(retcode != 0){
		ERR_MSG("usb_mtptrans_write write_transfer retcode=%d",(int)retcode);
		g_mtp->b_BulkInWorking = FALSE;
		g_mtp->tx_status = MTP_STATUS_OK;
		return USB_MTP_ERROR;
	}
	else{
		retcode = down_interruptible(&g_mtp->bulk_comp);	//wait
		if(retcode != 0){
			DBG_MSG("=== Bulk IN Failed(INTERRUPT SIGNAL)");
			_mtp_ioctl_abort_transfer();
			g_mtp->b_BulkInWorking = FALSE;
			return -EINTR;
		}
	}

	g_mtp->b_BulkInWorking = FALSE;

	//---------------------------------------------------------------
	// check data transfer status
	if(g_mtp->inreq->status != 0){
		return USB_MTP_ERROR;
	}

	//---------------------------------------------------------------
	// check data transfer
	if (g_mtp->config && !ds->bReset && !ds->bBulkInCanceled){
		return g_mtp->inreq->actual;
	}
	else {
		DBG_MSG("=== Bulk IN Failed(Disconnect/Aborted)");
		return USB_MTP_ERROR;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Bulk-out read
static ssize_t usb_mtptrans_read(	// __fn__
	struct file *file,
	char *buffer,
	size_t count,
	loff_t *ppos
)
{
	int		nret;

	DBG_MSGG("");
	DBG_MSGG("----- %s() Enter", __FUNCTION__);
	DBG_MSGG("----- read size (%d)", count);

	down(&g_mtp->bulk_out_use_sem);
	nret = _usb_mtp_read(buffer, count);
	up(&g_mtp->bulk_out_use_sem);

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
// Bulk-in write
static ssize_t usb_mtptrans_write(	// __fn__
	struct file *file,
	const char *buffer,
	size_t count,
	loff_t *ppos
)
{
	int		nret;

	DBG_MSGG("");
	DBG_MSGG("----- %s(%d)", __FUNCTION__, count);
	DBG_MSGG("----- write size (%d)", count);

	down(&g_mtp->bulk_in_use_sem);
	nret = _usb_mtp_write(buffer, count);
	up(&g_mtp->bulk_in_use_sem);

	return nret;
}

///////////////////////////////////////////////////////////////////////////////
// I/O control
static int usb_mtptrans_ioctl(	// __fn__
	struct inode *inode,
	struct file *file,
	unsigned int cmd,
	unsigned long arg
)
{
	int						ret = 0;
	long					dir;
	T_MTP_EVENT				*event;
	PUSB_MTP_XCHG_STATUS	devStatus;

	DBG_MSGG("--- %s called.",__FUNCTION__);

	switch (cmd) {
	case MTP_IOCTL_INITIALIZE:			// initialize
		ret = _mtp_ioctl_initialize();
		break;

	case MTP_IOCTL_RESET:				// Resset
		ret = _mtp_ioctl_reset();
		break;

	case MTP_IOCTL_STALL:				// Bulk Stall
		dir = *(long *)arg;
		ret = _mtp_ioctl_stall(dir);
		break;

	case MTP_IOCTL_UNSTALL:				// Bulk UnStall
		dir = *(long *)arg;
		ret = _mtp_ioctl_unstall(dir);
		break;

	case MTP_IOCTL_DISCONNECT:			// Disconnect
		ret = _mtp_ioctl_disconnect();
		break;

	case MTP_IOCTL_CONNECT:				// Connect
		ret = _mtp_ioctl_connect();
		break;

	case MTP_IOCTL_IS_CONNECTED:		// connect
		ret = _mtp_ioctl_isconnected();
		break;

	case MTP_IOCTL_INITWAIT:			// wait MTP Driver initialize finish
		ret = _mtp_ioctl_waitinit();
		break;

	case MTP_IOCTL_SENDEVENT:			// Send Event
		event = (T_MTP_EVENT*)arg;
		ret = _mtp_ioctl_sendevent(event->pBuffer, event->size, &event->count);
		break;

	case MTP_IOCTL_SET_TX_FLAG:
		g_mtp->b_DataZeroRequired = *(u32 *)arg;
		break;

	case MTP_IOCTL_XCHG_STATUS:
		devStatus = (PUSB_MTP_XCHG_STATUS) arg;
		ret = _mtp_ioctl_xchg_status(devStatus);
		break;

	case MTP_IOCTL_GET_VBUS_LEVEL:		// Vbus
		ret = _mtp_ioctl_get_vbus_level();
		break;

	case MTP_IOCTL_ABORT_TRANSFER:
		ret = _mtp_ioctl_abort_transfer();
		break;

	default:
		ERR_MSG("MTP driver, Illegal IOCTL");
		break;
	}

	return ret;
}

///////////////////////////////////////////////////////////////////////////////
//	MTP driver open
static int usb_mtptrans_open(struct inode *inode, struct file *file)	// __fn__
{

	DBG_MSG("%s() Enter", __FUNCTION__);

	// check driver opened
	spin_lock(&g_mtp->lock);
	if (g_mtp->open_count) {
		spin_unlock(&g_mtp->lock);
		ERR_MSG("Already MTP driver open !!");
		return -EBUSY;
	}
	g_mtp->open_count++;
	spin_unlock(&g_mtp->lock);

	if (MINOR(inode->i_rdev) != MTP_DEVICE_MINOR && !g_mtp->b_InitSuccess) {
		ERR_MSG("MTP driver Illegal devnode or init failed.");
		return -ENODEV;
	}

	file->private_data = g_mtp;

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//	MTP driver close
static int usb_mtptrans_release(struct inode *inode, struct file *file)	// __fn__
{
	DBG_MSG("%s() Enter", __FUNCTION__);

	spin_lock(&g_mtp->lock);
	g_mtp->open_count--;
	spin_unlock(&g_mtp->lock);

	return 0;
}

static struct file_operations usb_mtptrans_fops = {
	owner:		  THIS_MODULE,
	read:		  usb_mtptrans_read,
	write:		  usb_mtptrans_write,
	ioctl:		  usb_mtptrans_ioctl,
	open:		  usb_mtptrans_open,
	release:	  usb_mtptrans_release,
};

///////////////////////////////////////////////////////////////////////////////
// start
static int usb_mtptrans_start(void)	// __fn__
{
	int ret = 0;

	DBG_MSG("%s() Enter", __FUNCTION__);

	if(g_mtp->open_count){
		return ret;
	}

	//---------------------------------------------------------------
	ret = down_interruptible(&g_mtp->drv_reg_comp);
	if(ret != 0){
		DBG_MSG("=== START Failed(INTERRUPT SIGNAL)");
		return -EINTR;
	}
	g_mtp->b_InitSuccess = FALSE;

	//---------------------------------------------------------------
	// regist device
	g_mtp->major = MTP_DEVICE_MAJOR;
	if(g_mtp->major != 0){
		g_mtp->devcode = MKDEV(MTP_DEVICE_MAJOR, MTP_DEVICE_MINOR);
		ret = register_chrdev_region(g_mtp->devcode, 1, USB_MTP_MODULE_NAME);
	}
	else{
		ret = alloc_chrdev_region(&g_mtp->devcode, g_mtp->minor, 1, USB_MTP_MODULE_NAME);
		g_mtp->major =	MAJOR(g_mtp->devcode);
	}

	if(ret < 0){
		goto err_dev1;
	}

	cdev_init(&g_mtp->cdev, &usb_mtptrans_fops);
	g_mtp->cdev.owner = THIS_MODULE;
	g_mtp->cdev.ops = &usb_mtptrans_fops;

	ret = cdev_add(&g_mtp->cdev, g_mtp->devcode, 1);
	if(ret != 0){
		ERR_MSG("cdev_add() = %d", ret);
		goto err_dev1;
	}

#ifdef CONFIG_DEVFS_FS
	ret = devfs_mk_cdev(
		g_mtp->devcode,
		S_IFCHR|S_IRUSR|S_IWUSR|S_IRGRP,
		USB_MTP_MODULE_NAME
	);
	if(ret != 0) {
		ERR_MSG("devfs_mk_cdev() = %d", ret);
		goto err_mk_cdev;
	}
#endif

	g_mtp->open_count = 0;
	g_mtp->b_InitSuccess = TRUE;

	DBG_MSG("Register MTP Device Success, ret = %d", ret);

	return 0;

#ifdef CONFIG_DEVFS_FS
err_mk_cdev:
	cdev_del(&g_mtp->cdev);
#endif

err_dev1:
	ERR_MSG("Cannot register mtp device ret = %d", ret);
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
// stop
static void usb_mtptrans_stop(void)	// __fn__
{
	DBG_MSG("%s() Enter", __FUNCTION__);

	g_mtp->b_InitSuccess = FALSE;
	cdev_del(&g_mtp->cdev);
	unregister_chrdev_region(g_mtp->devcode, 1);
}

///////////////////////////////////////////////////////////////////////////////
// free MTP memory
static void mtp_free(void)	// __fn__
{
	//---------------------------------------------------------------
	// this version use gloval memroy, not exec free function.
	//---------------------------------------------------------------
}

///////////////////////////////////////////////////////////////////////////////
// initialize MTP driver
static int __init mtp_init(void)	// __fn__
{
	int		rc = 0;
	PT_USB_CLASS_DRIVER p_class;

	DBG_MSG("%s() Start", __FUNCTION__);

	//---------------------------------------------------------------
	memset(g_mtp, 0, sizeof(struct mtp_device));
	spin_lock_init(&g_mtp->lock);

	init_completion(&g_mtp->mtpthread_comp);

	sema_init(&g_mtp->interrupt_comp, 0);
	sema_init(&g_mtp->bulk_comp, 0);
	sema_init(&g_mtp->mtpinit_comp, 0);
	sema_init(&g_mtp->drv_reg_comp, 0);
	sema_init(&g_mtp->bulk_in_use_sem, 1);
	sema_init(&g_mtp->bulk_out_use_sem, 1);
	sema_init(&g_mtp->intr_in_use_sem, 1);
	g_mtp->tx_status = MTP_STATUS_OK;

	//---------------------------------------------------------------
	// regist to NWM Driver
	p_class = &g_mtp_regist_driver;
	memset(p_class, 0, sizeof(T_USB_CLASS_DRIVER));
	p_class->pDriver = &mtp_driver;
	p_class->ClassType = USB_MTP_TYPE;
	p_class->pMainLoop = mtp_main_thread;
	p_class->pContext = g_mtp;
	p_class->ep0_complete = ep0_complete;

	rc = fc_nwm_register_driver(p_class);
	if(rc != 0){
		g_mtp->b_RegistFlag = FALSE;
		mtp_free();
		ERR_MSG("Regist Error for NWM");
		return rc;
	}
	g_mtp->b_RegistFlag = TRUE;

	DBG_MSG("%s() End", __FUNCTION__);

	return rc;
}
module_init(mtp_init);


///////////////////////////////////////////////////////////////////////////////
// MTP driver finish
static void __exit mtp_cleanup(void)	// __fn__
{
	PT_USB_CLASS_DRIVER p_class;

	DBG_MSG("%s() Enter", __FUNCTION__);

	if(g_mtp->b_RegistFlag != FALSE){
		p_class = &g_mtp_regist_driver;
		fc_nwm_unregister_driver(p_class);
	}

	usb_mtptrans_stop();
	unregister_chrdev(MTP_DEVICE_MAJOR, USB_MTP_MODULE_NAME);
	mtp_free();
#ifdef CONFIG_DEVFS_FS
	devfs_remove(USB_MTP_MODULE_NAME); /* Device File */
#endif

}
module_exit(mtp_cleanup);

