/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/imc.h
 * Function        : IMC MMIO definitions
 * Release Version : Ver 1.00
 * Release Date    : 2007/06/15
 *
 * Copyright (C) NEC Electronics Corporation 2007
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _IMC_H_
#define _IMC_H_


/*****************************************************************************
 * IMC MMIO definitions
 *****************************************************************************/

#include <asm/arch/hardware.h>
#define IMC_OFFSET	MP200_IMC_OFFSET
#define IMC_BASE	MP200_IMC_BASE
#define IMC_SIZE	MP200_IMC_SIZE


//                                                           R/W   Bit  Reset
/*----------------------------------------------------------------------------
 * Function Settings Register
 */
// Control for IMC                                           R/W  31:0  0x0000
#define IMC_CONTROL             (0x00)

// Update Reservation for IMC                                R/W   0:0  0x0000
#define IMC_REFRESH             (0x04)

/*----------------------------------------------------------------------------
 * Synthetic Start Register
 */
// Start to IMC                                              R/W   0:0  0x0000
#define IMC_START               (0x10)

// Status for IMC                                             R    1:0  0x0000
#define IMC_STATUS              (0x14)

// Double Buffer Control                                     R/W   0:0  0x0000
#define IMC_CPUBUFSEL           (0x18)

/*----------------------------------------------------------------------------
 * Gamma Control Register
 */
// Gamma Correction Control                                  R/W   0:0  0x0000
#define IMC_GAMMA_EN            (0x20)

// Gamma Control Table Address                               R/W   5:0  0x0000
#define IMC_GAMMA_ADR           (0x24)

// Gamma Control Table Data                                  R/W  21:0  0x0000
#define IMC_GAMMA_DATA          (0x28)

/*----------------------------------------------------------------------------
 * Start Exclusive Use Immediately Setting Register
 */
// Display Area Address                                      R/W  31:0  0x0000
#define IMC_WB_AREADDR          (0x40)

// Address Increment Value                                   R/W  12:4  0x0000
#define IMC_WB_HOFFSET          (0x44)

// Format                                                    R/W   0:0  0x0000
#define IMC_WB_FORMAT           (0x48)

// Image Size                                                R/W  26:0  0x0000
#define IMC_WB_SIZE             (0x4C)

/*----------------------------------------------------------------------------
 * All Layer Setting Register
 */
// Horizontal, Vertical Mirroring Control                    R/W   1:0  0x0000
#define IMC_MIRROR              (0x100)

// Y Gain Offset                                             R/W  15:0  0x0000
#define IMC_YGAINOFFSET         (0x104)

// U Gain Offset                                             R/W  15:0  0x0000
#define IMC_UGAINOFFSET         (0x108)

// V Gain Offset                                             R/W  15:0  0x0000
#define IMC_VGAINOFFSET         (0x10C)

// YUV to RGB Conversion method                              R/W   8:0  0x0000
#define IMC_YUV2RGB             (0x110)

// Custom factor (Coef R0)                                   R/W  10:0  0x0000
#define IMC_COEF_R0             (0x114)

// Custom factor (Coef R1)                                   R/W  10:0  0x0000
#define IMC_COEF_R1             (0x118)

// Custom factor (Coef R2)                                   R/W  10:0  0x0000
#define IMC_COEF_R2             (0x11C)

// Custom factor (Coef R3)                                   R/W  10:0  0x0000
#define IMC_COEF_R3             (0x120)

// Custom factor (Coef G0)                                   R/W  10:0  0x0000
#define IMC_COEF_G0             (0x124)

// Custom factor (Coef G1)                                   R/W  10:0  0x0000
#define IMC_COEF_G1             (0x128)

// Custom factor (Coef G2)                                   R/W  10:0  0x0000
#define IMC_COEF_G2             (0x12C)

// Custom factor (Coef G3)                                   R/W  10:0  0x0000
#define IMC_COEF_G3             (0x130)

// Custom factor (Coef B0)                                   R/W  10:0  0x0000
#define IMC_COEF_B0             (0x134)

// Custom factor (Coef B1)                                   R/W  10:0  0x0000
#define IMC_COEF_B1             (0x138)

// Custom factor (Coef B2)                                   R/W  10:0  0x0000
#define IMC_COEF_B2             (0x13C)

// Custom factor (Coef B3)                                   R/W  10:0  0x0000
#define IMC_COEF_B3             (0x140)

/*----------------------------------------------------------------------------
 * Layer0 Setting Register
 */
// Layer0 Control                                            R/W   0:0  0x0000
#define IMC_L0_CONTROL          (0x200)

// Layer0 Format                                             R/W   0:0  0x0000
#define IMC_L0_FORMAT           (0x204)

// Layer0 Color Key Control                                  R/W   0:0  0x0000
#define IMC_L0_KEYENABLE        (0x210)

// Layer0 Color Key                                          R/W  21:0  0x0000
#define IMC_L0_KEYCOLOR         (0x214)

// Layer0 Alpha                                              R/W   8:0  0x0000
#define IMC_L0_ALPHA            (0x218)

// Layer0 Resize                                             R/W   0:0  0x0000
#define IMC_L0_RESIZE           (0x220)

// Layer0 Address increment value                            R/W   9:0  0x0000
#define IMC_L0_OFFSET           (0x230)

// Layer0 Address                                            R/W  31:0  0x0000
#define IMC_L0_FRAMEADR         (0x234)

// Layer0 LCD Out Positon                                    R/W  23:0  0x0000
#define IMC_L0_POSITION         (0x250)

// Layer0 LCD Out Size                                       R/W   7:0  0x0000
#define IMC_L0_SIZE             (0x254)

/*----------------------------------------------------------------------------
 * Layer1A Setting Register
 */
// Layer1A Control                                           R/W   0:0  0x0000
#define IMC_L1A_CONTROL         (0x300)

// Layer1X Format                                            R/W   0:0  0x0000
#define IMC_L1X_FORMAT          (0x304)

// Layer1A Color Key Control                                 R/W   0:0  0x0000
#define IMC_L1A_KEYENABLE       (0x310)

// Layer1X Color Key                                         R/W  21:0  0x0000
#define IMC_L1X_KEYCOLOR        (0x314)

// Layer1A Alpha                                             R/W   8:0  0x0000
#define IMC_L1A_ALPHA           (0x318)

// Layer1X Resize                                            R/W   0:0  0x0000
#define IMC_L1X_RESIZE          (0x320)

// Layer1X Address increment value                           R/W  10:0  0x0000
#define IMC_L1X_OFFSET          (0x330)

// Layer1A Address                                           R/W  31:0  0x0000
#define IMC_L1A_FRAMEADR        (0x334)

// Layer1A LCD Out Positon                                   R/W  26:0  0x0000
#define IMC_L1A_POSITION        (0x350)

// Layer1A LCD Out Size                                      R/W   9:0  0x0000
#define IMC_L1A_SIZE            (0x354)

/*----------------------------------------------------------------------------
 * Layer1B Setting Register
 */
// Layer1B Control                                           R/W   0:0  0x0000
#define IMC_L1B_CONTROL         (0x400)

// Layer1B Color Key Control                                 R/W   0:0  0x0000
#define IMC_L1B_KEYENABLE       (0x410)

// Layer1B Alpha                                             R/W   8:0  0x0000
#define IMC_L1B_ALPHA           (0x418)

// Layer1B Address                                           R/W  31:0  0x0000
#define IMC_L1B_FRAMEADR        (0x434)

// Layer1B LCD Out Positon                                   R/W  26:0  0x0000
#define IMC_L1B_POSITION        (0x450)

// Layer1B LCD Out Size                                      R/W   9:0  0x0000
#define IMC_L1B_SIZE            (0x454)

/*----------------------------------------------------------------------------
 * Layer1C Setting Register
 */
// Layer1C Control                                           R/W   0:0  0x0000
#define IMC_L1C_CONTROL         (0x500)

// Layer1C Color Key Control                                 R/W   0:0  0x0000
#define IMC_L1C_KEYENABLE       (0x510)

// Layer1C Alpha                                             R/W   8:0  0x0000
#define IMC_L1C_ALPHA           (0x518)

// Layer1C Address                                           R/W  31:0  0x0000
#define IMC_L1C_FRAMEADR        (0x534)

// Layer1C LCD Out Positon                                   R/W  26:0  0x0000
#define IMC_L1C_POSITION        (0x550)

// Layer1C LCD Out Size                                      R/W   9:0  0x0000
#define IMC_L1C_SIZE            (0x554)

/*----------------------------------------------------------------------------
 * Layer2A Setting Register
 */
// Layer2A Control                                           R/W   0:0  0x0000
#define IMC_L2A_CONTROL          (0x600)

// Layer2A Format                                            R/W   2:0  0x0000
#define IMC_L2A_FORMAT           (0x604)

// Layer2A Double Buffer Control                             R/W   0:0  0x0000
#define IMC_L2A_BUFSEL           (0x608)

// Layer2A Bytelane                                          R/W   0:0  0x0000
#define IMC_L2A_BYTELANE         (0x60C)

// Layer2A Resize                                            R/W   0:0  0x0000
#define IMC_L2A_RESIZE           (0x620)

// Layer2A Horizontal, Vertical Mirroring Control            R/W   0:0  0x0000
#define IMC_L2A_MIRROR           (0x624)

// Layer2A Address increment value                           R/W  12:0  0x0000
#define IMC_L2A_OFFSET           (0x630)

// Layer2A Address (YP)                                      R/W  31:0  0x0000
#define IMC_L2A_FRAMEADR_YP      (0x634)

// Layer2A Address (UP)                                      R/W  31:0  0x0000
#define IMC_L2A_FRAMEADR_UP      (0x638)

// Layer2A Address (VP)                                      R/W  31:0  0x0000
#define IMC_L2A_FRAMEADR_VP      (0x63C)

// Layer2A Address (YQ)                                      R/W  31:0  0x0000
#define IMC_L2A_FRAMEADR_YQ      (0x640)

// Layer2A Address (UQ)                                      R/W  31:0  0x0000
#define IMC_L2A_FRAMEADR_UQ      (0x644)

// Layer2A Address (VQ)                                      R/W  31:0  0x0000
#define IMC_L2A_FRAMEADR_VQ      (0x648)

// Layer2A LCD Out Positon                                   R/W  23:0  0x0000
#define IMC_L2A_POSITION         (0x650)

// Layer2A LCD Out Size                                      R/W  10:0  0x0000
#define IMC_L2A_SIZE             (0x654)

/*----------------------------------------------------------------------------
 * Layer2B Setting Register
 */
// Layer2B Control                                           R/W   0:0  0x0000
#define IMC_L2B_CONTROL          (0x700)

// Layer2B Format                                            R/W   2:0  0x0000
#define IMC_L2B_FORMAT           (0x704)

// Layer2B Double Buffer Control                             R/W   0:0  0x0000
#define IMC_L2B_BUFSEL           (0x708)

// Layer2B Bytelane                                          R/W   0:0  0x0000
#define IMC_L2B_BYTELANE         (0x70C)

// Layer2B Resize                                            R/W   0:0  0x0000
#define IMC_L2B_RESIZE           (0x720)

// Layer2B Horizontal, Vertical Mirroring Control            R/W   0:0  0x0000
#define IMC_L2B_MIRROR           (0x724)

// Layer2B Address increment value                           R/W  12:0  0x0000
#define IMC_L2B_OFFSET           (0x730)

// Layer2B Address (YP)                                      R/W  31:0  0x0000
#define IMC_L2B_FRAMEADR_YP      (0x734)

// Layer2B Address (UP)                                      R/W  31:0  0x0000
#define IMC_L2B_FRAMEADR_UP      (0x738)

// Layer2B Address (VP)                                      R/W  31:0  0x0000
#define IMC_L2B_FRAMEADR_VP      (0x73C)

// Layer2B Address (YQ)                                      R/W  31:0  0x0000
#define IMC_L2B_FRAMEADR_YQ      (0x740)

// Layer2B Address (UQ)                                      R/W  31:0  0x0000
#define IMC_L2B_FRAMEADR_UQ      (0x744)

// Layer2B Address (VQ)                                      R/W  31:0  0x0000
#define IMC_L2B_FRAMEADR_VQ      (0x748)

// Layer2B LCD Out Positon                                   R/W  23:0  0x0000
#define IMC_L2B_POSITION         (0x750)

// Layer2B LCD Out Size                                      R/W  10:0  0x0000
#define IMC_L2B_SIZE             (0x754)

/*----------------------------------------------------------------------------
 * LayerBG Setting Register
 */
// LayerBG Format                                            R/W   2:0  0x0000
#define IMC_BG_FORMAT            (0x804)

// LayerBG Resize                                            R/W   0:0  0x0000
#define IMC_BG_RESIZE            (0x820)

// LayerBG Address increment value                           R/W  12:0  0x0000
#define IMC_BG_OFFSET            (0x830)

// Layer2B Address (YP)                                      R/W  31:0  0x0000
#define IMC_BG_FRAMEADR          (0x834)

/*----------------------------------------------------------------------------
 * Interuppt Setting Register
 */
// Interuppt Status                                           R    4:0  0x0000
#define IMC_INTSTATUS            (0x900)

// Interuppt Raw Status                                       R    4:0  0x0000
#define IMC_INTRAWSTATUS         (0x904)

// Interuppt Enable Set                                      R/W   4:0  0x0000
#define IMC_INTENSET             (0x908)

// Interuppt Enable Clear                                     W    4:0  0x0000
#define IMC_INTENCLR             (0x90C)

// Interuppt FF Clear                                         W    4:0  0x0000
#define IMC_INTFFCLR             (0x910)

// AHB(R) Error Address                                      R/W   1:0  0x0000
#define IMC_ERRORADR_R           (0x914)

// AHB(W) Error Address                                      R/W   1:0  0x0000
#define IMC_ERRORADR_W           (0x918)

#ifdef CONFIG_MACH_PGPF
// ARGB Mode                                                 R/W  11:8  0x0000  
#define IMC_ARGBMODE             (0x1000)
#endif /* CONFIG_MACH_PGPF */









/*----------------------------------------------------------------------------
 * Function Settings Register
 */
//------------------------------
// IMC_CONTROL
//------------------------------
#define IMC_DBGMODE_BIT		0x80000000
#define IMC_DBGMODE_SFT		0x1F

#define IMC_CLKCNT_BIT		0x00001F00
#define IMC_CLKCNT_SFT		0x08
#define IMC_CLKCNT_FRONT_ON	0x0100
#define IMC_CLKCNT_MIDDLE_ON	0x0200
#define IMC_CLKCNT_Y2R_ON	0x0400
#define IMC_CLKCNT_GAMMA_ON	0x0800
#define IMC_CLKCNT_WB_ON	0x1000
#define IMC_CLKCNT_ALL_ON	( IMC_CLKCNT_Y2R_ON | IMC_CLKCNT_FRONT_ON | IMC_CLKCNT_MIDDLE_ON | IMC_CLKCNT_GAMMA_ON |IMC_CLKCNT_WB_ON )

#define IMC_FORMAT_BIT		0x00000002
#define IMC_FORMAT_SFT		0x01
#define IMC_FORMAT_RGB565	0x02
#define IMC_FORMAT_RGB666	0x00

#define IMC_START_MODE_BIT	0x80000001
#define IMC_START_MODE_SFT	0x00
#define IMC_START_MODE_NOSYNC	0x01
#define IMC_START_MODE_SYNCHRO	0x00


//------------------------------
// IMC_REFRESH
//------------------------------
#define IMC_UPDATE_BIT		0x00000001
#define IMC_UPDATE_SFT		0x00

#define IMC_UPDATE_ON		0x01
#define IMC_UPDATE_OFF		0x00


/*----------------------------------------------------------------------------
 * Synthetic Start Register
 */
//------------------------------
// IMC_START
//------------------------------
#define IMC_IMCSTART_BIT	0x00000001
#define IMC_IMCSTART_SFT	0x00

#define IMC_IMCSTART_ON		0x01
#define IMC_IMCSTART_OFF	0x00

//------------------------------
// IMC_STATUS
//------------------------------
#define IMC_STATUS_BIT		0x00000003
#define IMC_STATUS_SFT		0x00

#define IMC_STATUS_SINGLE_WB	0x03
#define IMC_STATUS_LCDOUT_WB	0x02
#define IMC_STATUS_LCDOUT	0x01
#define IMC_STATUS_STOP		0x00

//------------------------------
// IMC_CPUBUFSEL
//------------------------------

/*----------------------------------------------------------------------------
 * Gamma Control Register
 */
//------------------------------
// IMC_GAMMA_EN
//------------------------------
#define IMC_GAMMAEN_BIT		0x00000001
#define IMC_GAMMAEN_SFT		0x00

#define IMC_GAMMAEN_ON		0x01
#define IMC_GAMMAEN_OFF		0x00

//------------------------------
// IMC_GAMMA_ADR
//------------------------------
#define IMC_GAMMA_ADR_BIT	0x0000002F
#define IMC_GAMMA_ADR_SFT	0x00

//------------------------------
// IMC_GAMMA_DATA
//------------------------------
#define IMC_GAMMA_RED_BIT	0x002F0000
#define IMC_GAMMA_RED_SFT	0x10
#define IMC_GAMMA_GREEN_BIT	0x00002F00
#define IMC_GAMMA_GREEN_SFT	0x08
#define IMC_GAMMA_BLUE_BIT	0x0000002F
#define IMC_GAMMA_BLUE_SFT	0x00

/*----------------------------------------------------------------------------
 * Start Exclusive Use Immediately Setting Register
 */
//------------------------------
// IMC_WB_AREADDR
//------------------------------
#define IMC_WB_AREADDR_BIT	0xFFFFFFFFC
#define IMC_WB_AREADDR_SFT	0x00

//------------------------------
// IMC_WB_HOFFSET
//------------------------------
#define IMC_WB_HOFFSET_BIT	0x00001FFC
#define IMC_WB_HOFFSET_SFT	0x00

//------------------------------
// IMC_WB_FORMAT
//------------------------------
#define IMC_WB_FORMAT_BIT	0x00000001
#define IMC_WB_FORMAT_SFT	0x00

#define IMC_WB_FORMAT_RGB565	0x01
#define IMC_WB_FORMAT_RGB666	0x00

//------------------------------
// IMC_WB_SIZE
//------------------------------
#define IMC_WB_VSIZE_BIT	0x07FF0000
#define IMC_WB_VSIZE_SFT	0x10

#define IMC_WB_HSIZE_BIT	0x000007FE
#define IMC_WB_HSIZE_SFT	0x00


/*----------------------------------------------------------------------------
 * All Layer Setting Register
 */
//------------------------------
// IMC_MIRROR
//------------------------------
#define IMC_MIRROR_BIT		0x00000003
#define IMC_MIRROR_SFT		0x00

#define IMC_MIRROR_LRUD		0x03
#define IMC_MIRROR_UD		0x02
#define IMC_MIRROR_LR		0x01
#define IMC_MIRROR_OFF		0x00

//------------------------------
// IMC_Y/U/VGAINOFFSET
//------------------------------
#define IMC_YUVGAIN_OFFSET_BIT	0x0000FF00
#define IMC_YUVGAIN_OFFSET_SFT	0x10
#define IMC_YUVGAIN_GAIN_BIT	0x000000FF
#define IMC_YUVGAIN_GAIN_SFT	0x00

//------------------------------
// IMC_YUV2RGB
//------------------------------
#define IMC_DITHER_BIT		0x00000100
#define IMC_DITHER_SFT		0x08

#define IMC_DITHER_ON		0x01
#define IMC_DITHER_OFF		0x00


#define IMC_TRANS_MODE_BIT	0x00000003
#define IMC_TRANS_MODE_SFT	0x00

#define IMC_TRANS_MODE_CUSTOM2	0x03
#define IMC_TRANS_MODE_CUSTOM1	0x02
#define IMC_TRANS_MODE_BT709	0x01
#define IMC_TRANS_MODE_BT601	0x00


/*----------------------------------------------------------------------------
 * LayerX Setting Register
 */
//------------------------------
// IMC_L0/1A/1B/1C/2A/2B_CONTROL
//------------------------------
#define IMC_Lx_CONTROL_BIT	0x00000001
#define IMC_Lx_CONTROL_SFT	0x00
#define IMC_Lx_CONTROL_ON	0x01
#define IMC_Lx_CONTROL_OFF	0x00

//------------------------------
// IMC_L0/1A/1B/1C/2A/2B_FORMAT
//------------------------------
#define IMC_L0_FORMAT_BIT	0x00000001
#define IMC_L1x_FORMAT_BIT	0x00000001
#define IMC_L2x_FORMAT_BIT	0x00000003
#define IMC_Lx_FORMAT_SFT	0x00
#define IMC_Lx_FORMAT_YUV420Pl	0x06
#define IMC_Lx_FORMAT_YUV420Pl2	0x05
#define IMC_Lx_FORMAT_YUV422Pl	0x04
#define IMC_Lx_FORMAT_YUV422Pl2	0x03
#define IMC_Lx_FORMAT_YUV422Px	0x02
#define IMC_Lx_FORMAT_RGB565	0x01
#define IMC_Lx_FORMAT_RGB666	0x00

//------------------------------
// IMC_BG_FORMAT
//------------------------------
#define IMC_LBG_FORMAT_BIT	0x00000002
#define IMC_LBG_FORMAT_SFT	0x00
#define IMC_LBG_FORMAT_OPTIONAL	0x03
#define IMC_LBG_FORMAT_BLACK	0x02
#define IMC_LBG_FORMAT_RGB565	0x01
#define IMC_LBG_FORMAT_RGB666	0x00

//------------------------------
// IMC_L0/1A/1B/1C_KEYENABLE
//------------------------------
#define IMC_Lx_KEYEN_BIT	0x00000001
#define IMC_Lx_KEYEN_SFT	0x00
#define IMC_Lx_KEYEN_ENABLE	0x01
#define IMC_Lx_KEYEN_DISABLE	0x00

//------------------------------
// IMC_L0/1X_KEYCOLOR
//------------------------------
#define IMC_Lx_KEYR_BIT		0x002F0000
#define IMC_Lx_KEYR_SFT		0x10
#define IMC_Lx_KEYG_BIT		0x00002F00
#define IMC_Lx_KEYG_SFT		0x08
#define IMC_Lx_KEYB_BIT		0x0000002F
#define IMC_Lx_KEYB_SFT		0x00

//------------------------------
// IMC_L0/1A/1B/1C_ALPHA
//------------------------------
#define IMC_Lx_REVERSE_BIT		0x00000100
#define IMC_Lx_REVERSE_SFT		0x08
#define IMC_Lx_ALPHA_BIT		0x0000002F
#define IMC_Lx_ALPHA_SFT		0x00
#define IMC_Lx_ALPHA_OPAQUE		0x2F
#define IMC_Lx_ALPHA_TRANSPARENT	0x00

//------------------------------
// IMC_L0/1X/2A/2B/BG_RESIZE
//------------------------------
#define IMC_Lx_RESIZE_BIT	0x00000001
#define IMC_Lx_RESIZE_SFT	0x00
#define IMC_Lx_RESIZE_ENABLE	0x01
#define IMC_Lx_RESIZE_DISABLE	0x00

//------------------------------
// IMC_L0/1X/2A/2B_POSITION
//------------------------------
#define IMC_Lx_POSY_BIT		0x07FF0000
#define IMC_Lx_POSY_SFT		0x10
#define IMC_Lx_POSX_BIT		0x000007FF
#define IMC_Lx_POSX_SFT		0x00

//------------------------------
// IMC_L0/1X/2A/2B_SIZE
//------------------------------
#define IMC_L0_SIZEY_BIT	0x00FF0000
#define IMC_L0_SIZEX_BIT	0x000000FF
#define IMC_L1_SIZEY_BIT	0x07FF0000
#define IMC_L1_SIZEX_BIT	0x000002FF
#define IMC_L2_SIZEY_BIT	0x07FF0000
#define IMC_L2_SIZEX_BIT	0x000002FF

#define IMC_Lx_SIZEY_SFT	0x10
#define IMC_Lx_SIZEX_SFT	0x00

//------------------------------
// IMC_L2A/2B_BYTELANE
//------------------------------
#define IMC_L2x_BYTELANE_Y_BIT  0x0000FF00
#define IMC_L2x_BYTELANE_Y_SFT  0x08
#define IMC_L2x_BYTELANE_UV_BIT 0x000000FF
#define IMC_L2x_BYTELANE_UV_SFT 0x00

#define IMC_L2x_BYTELANE_422PX_YYUV 0xE400
#define IMC_L2x_BYTELANE_422PX_YUYV 0xD800
#define IMC_L2x_BYTELANE_INIT       0xE4E4

//------------------------------
// IMC_L2A/2B_MIRROR
//------------------------------
#define IMC_L2x_MIRROR_BIT	0x00000003
#define IMC_L2x_MIRROR_SFT	0x00

#define IMC_L2x_MIRROR_LRUD	0x03
#define IMC_L2x_MIRROR_UD	0x02
#define IMC_L2x_MIRROR_LR	0x01
#define IMC_L2x_MIRROR_OFF	0x00


/*----------------------------------------------------------------------------
 * Interuppt Control Register
 */
//------------------------------
// IMC_INTSTATUS
//------------------------------
#define IMC_AHBRERR_BIT		0x00000010
#define IMC_AHBRERR_SFT		0x04
#define IMC_AHBWERR_BIT		0x00000008
#define IMC_AHBWERR_SFT		0x03
#define IMC_WBEND_BIT		0x00000004
#define IMC_WBEND_SFT		0x02
#define IMC_OVERRUN_BIT		0x00000002
#define IMC_OVERRUN_SFT		0x01
#define IMC_REFRESH_BIT		0x00000001
#define IMC_REFRESH_SFT		0x00

//------------------------------
// IMC_INTENSET
//------------------------------
#define IMC_AHBRERR_EN_BIT	0x00000010
#define IMC_AHBRERR_EN_SFT	0x04
#define IMC_AHBWERR_EN_BIT	0x00000008
#define IMC_AHBWERR_EN_SFT	0x03
#define IMC_WBEND_EN_BIT	0x00000004
#define IMC_WBEND_EN_SFT	0x02
#define IMC_OVERRUN_EN_BIT	0x00000002
#define IMC_OVERRUN_EN_SFT	0x01
#define IMC_REFRESH_EN_BIT	0x00000001
#define IMC_REFRESH_EN_SFT	0x00

#define IMC_INTENSET_ALL_BIT   (IMC_AHBRERR_EN_BIT | IMC_AHBWERR_EN_BIT | IMC_WBEND_EN_BIT | IMC_OVERRUN_EN_BIT | IMC_REFRESH_EN_BIT)

//------------------------------
// IMC_INTENCLR
//------------------------------
#define IMC_AHBRERR_MASK_BIT	0x00000010
#define IMC_AHBRERR_MASK_SFT	0x04
#define IMC_AHBWERR_MASK_BIT	0x00000008
#define IMC_AHBWERR_MASK_SFT	0x03
#define IMC_WBEND_MASK_BIT	0x00000004
#define IMC_WBEND_MASK_SFT	0x02
#define IMC_OVERRUN_MASK_BIT	0x00000002
#define IMC_OVERRUN_MASK_SFT	0x01
#define IMC_REFRESH_MASK_BIT	0x00000001
#define IMC_REFRESH_MASK_SFT	0x00

#define IMC_INTENCLR_ALL_BIT   (IMC_AHBRERR_MASK_BIT | IMC_AHBWERR_MASK_BIT | IMC_WBEND_MASK_BIT | IMC_OVERRUN_MASK_BIT | IMC_REFRESH_MASK_BIT)

//------------------------------
// IMC_INTFFCLR
//------------------------------
#define IMC_AHBRERR_CLR_BIT	0x00000010
#define IMC_AHBRERR_CLR_SFT	0x04
#define IMC_AHBWERR_CLR_BIT	0x00000008
#define IMC_AHBWERR_CLR_SFT	0x03
#define IMC_WBEND_CLR_BIT	0x00000004
#define IMC_WBEND_CLR_SFT	0x02
#define IMC_OVERRUN_CLR_BIT	0x00000002
#define IMC_OVERRUN_CLR_SFT	0x01
#define IMC_REFRESH_CLR_BIT	0x00000001
#define IMC_REFRESH_CLR_SFT	0x00

#define IMC_INTFFCLR_ALL_BIT   (IMC_AHBRERR_CLR_BIT | IMC_AHBWERR_CLR_BIT | IMC_WBEND_CLR_BIT | IMC_OVERRUN_CLR_BIT | IMC_REFRESH_CLR_BIT)

#ifdef CONFIG_MACH_PGPF
//------------------------------
// IMC_ARGBMODE
//------------------------------
#define IMC_L1C_ARGB_BIT 0x00000800
#define IMC_L1B_ARGB_BIT 0x00000400
#define IMC_L1A_ARGB_BIT 0x00000200
#define IMC_L0_ARGB_BIT  0x00000800
#endif /* CONFIG_MACH_PGPF */

#endif /* _IMC_H_ */
