/* 2009-08-24: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/lcdc.h
 * Function        : LCDC MMIO definitions
 * Release Version : Ver 1.00
 * Release Date    : 2007/07/03
 *
 * Copyright (C) NEC Electronics Corporation 2007
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _LCDC_H_
#define _LCDC_H_


/*****************************************************************************
 * LCDC MMIO definitions
 *****************************************************************************/

#include <asm/arch/hardware.h>
#define LCDC_OFFSET	MP200_LCD_OFFSET
#define LCDC_BASE	MP200_LCD_BASE
#define LCDC_SIZE	MP200_LCD_SIZE


//                                                           R/W   Bit  Reset
/*----------------------------------------------------------------------------
 * Function Settings Register
 */
// Control for LCDC                                          R/W   4:0  0x00
#define LCD_CONTROL             (0x00)

// QoS for LCDC Register                                     R/W   8:0  0x000
#define LCD_QOS                 (0x04)

// Data Request for LCDC Register                            R/W   2:0  0x0
#define LCD_DATAREQ             (0x08)

/*----------------------------------------------------------------------------
 * Output Settings Register
 */
// LCD Output                                                R/W   0:0  0x0
#define LCD_LCDOUT              (0x10)

// Access Bus Select                                         R/W   2:0  0x0
#define LCD_BUSSEL              (0x14)

// Status for LCDC                                            R   10:0  0x000
#define LCD_STATUS              (0x18)

// LCD BackGround Color                                      R/W  21:0  0x0
#define LCD_BACKCOLOR           (0x1C)

/*----------------------------------------------------------------------------
 * Frame Buffer Settings Register
 */
// Input Format select                                       R/W   0:0  0x0
#define LCD_IFORMAT             (0x28)

/*----------------------------------------------------------------------------
 * Output Settings Register
 */
// Horizontal Synchronizing Signal Total Width               R/W  11:0  0x000
#define LCD_HTOTAL              (0x30)

// Horizontal Display Area Width                             R/W  10:4  0x000
#define LCD_HAREA               (0x34)

// Horizontal Synchronizing Signal Edge Point 1              R/W  11:0  0x000
#define LCD_HEDGE1              (0x38)

// Horizontal Synchronizing Signal Edge Point 2              R/W  11:0  0x000
#define LCD_HEDGE2              (0x3C)

// Vertical Synchronizing Signal Total Width                 R/W  11:0  0x000
#define LCD_VTOTAL              (0x40)

// Vertical Display Area Width                               R/W  10:0  0x000
#define LCD_VAREA               (0x44)

// Vertical Synchronizing Signal Edge Point 1                R/W  11:0  0x000
#define LCD_VEDGE1              (0x48)

// Vertical Synchronizing Signal Edge Point 2                R/W  11:0  0x000
#define LCD_VEDGE2              (0x4C)

/*----------------------------------------------------------------------------
 * Interuppt Setting Register
 */
// Interrupt Status                                           R    4:0  0x0
#define LCD_INTSTATUS           (0x60)

// Interrupt Raw Status                                       R    4:0  0x0
#define LCD_INTRAWSTATUS        (0x64)

// Interrupt Enable Set                                      R/W   4:0  0x0
#define LCD_INTENSET            (0x68)

// Interrupt Enable Clear                                     W    4:0  0x0
#define LCD_INTENCLR            (0x6C)

// Interrupt Status Clear                                     W    4:0  0x0
#define LCD_INTFFCLR            (0x70)

// Interrupt for Framecount                                  R/W  24:0  0x0000000
#define LCD_FRAMECOUNT          (0x74)










/*----------------------------------------------------------------------------
 * Function Settings Register
 */
//------------------------------
// LCD_CONTROL
//------------------------------
#define LCD_OFORMAT_BIT		0x00000010
#define LCD_OFORMAT_SFT		0x04
#define LCD_OFORMAT_RGB565	0x10
#define LCD_OFORMAT_RGB666	0x00

#define LCD_CLKPOL_BIT		0x00000008
#define LCD_CLKPOL_SFT		0x03
#define LCD_HPOL_BIT		0x00000004
#define LCD_HPOL_SFT		0x02
#define LCD_VPOL_BIT		0x00000002
#define LCD_VPOL_SFT		0x01
#define LCD_ENPOL_BIT		0x00000001
#define LCD_ENPOL_SFT		0x00

#ifdef CONFIG_MACH_PGPF
#define	LCD_CLKPOL_POS (0 << LCD_CLKPOL_SFT)
#define	LCD_CLKPOL_NEG (1 << LCD_CLKPOL_SFT)
#define	LCD_HPOL_POS   (0 << LCD_HPOL_SFT)
#define	LCD_HPOL_NEG   (1 << LCD_HPOL_SFT)
#define	LCD_VPOL_POS   (0 << LCD_VPOL_SFT)
#define	LCD_VPOL_NEG   (1 << LCD_VPOL_SFT)
#define	LCD_ENPOL_HI   (0 << LCD_ENPOL_SFT)
#define	LCD_ENPOL_LOW  (1 << LCD_ENPOL_SFT)
#endif /* CONFIG_MACH_PGPF */

//------------------------------
// LCD_QOS
//------------------------------
#define LCD_QOSEN_BIT		0x00000100
#define LCD_QOSEN_SFT		0x08
#define LCD_QOSEN_ENABLE	0x100
#define LCD_QOSEN_DISABLE	0x000
#define LCD_QOSVALUE_BIT	0x000000FF
#define LCD_QOSVALUE_SFT	0x00

//------------------------------
// LCD_DATAREQ
//------------------------------
#define LCD_DATAREQ_BIT		0x00000007
#define LCD_DATAREQ_SFT		0x00

/*----------------------------------------------------------------------------
 * Output Settings Register
 */
//------------------------------
// LCD_LCDOUT
//------------------------------
#define LCD_LCDOUT_BIT		0x00000001
#define LCD_LCDOUT_SFT		0x00
#define LCD_LCDOUT_ENABLE	0x1
#define LCD_LCDOUT_DISABLE	0x0

//------------------------------
// LCD_BUSSEL
//------------------------------
#define LCD_BUSSEL_BIT		0x00000007
#define LCD_BUSSEL_SFT		0x00
#define LCD_BUSSEL_BACKCOLOR	0x5
#define LCD_BUSSEL_BLACK	0x4
#define LCD_BUSSEL_WB_LOCAL	0x2
#define LCD_BUSSEL_LOCAL	0x0

//------------------------------
// LCD_STATUS
//------------------------------
#define LCD_MODSTATUS_BIT	0x00000700
#define LCD_MODSTATUS_SFT	0x08
#define LCD_MODSTATUS_BACKCOLOR	0x0500
#define LCD_MODSTATUS_BLACK	0x0400
#define LCD_MODSTATUS_WB_LOCAL	0x0200
#define LCD_MODSTATUS_LOCAL	0x0000

#define LCD_STATUS_BIT		0x00000001
#define LCD_STATUS_SFT		0x00
#define LCD_STATUS_ON		0x01
#define LCD_STATUS_OFF		0x00

//------------------------------
// LCD_BACKCOLOR
//------------------------------
#define LCD_BGRED_BIT		0x003F0000
#define LCD_BGRED_SFT		0x10
#define LCD_BGGREEN_BIT		0x00003F00
#define LCD_BGGREEN_SFT		0x08
#define LCD_BGBLUE_BIT		0x0000003F
#define LCD_BGBLUE_SFT		0x00

/*----------------------------------------------------------------------------
 * Frame Buffer Settings Register
 */
//------------------------------
// LCD_IFORMAT
//------------------------------
#define LCD_IFORMAT_BIT		0x00000001
#define LCD_IFORMAT_SFT		0x00
#define LCD_IFORMAT_RGB565	0x1
#define LCD_IFORMAT_RGB666	0x0

/*----------------------------------------------------------------------------
 * Interuppt Setting Register
 */
//------------------------------
// LCD_INTSTATUS
//------------------------------
#define LCD_WBTRACE_BIT		0x00000010
#define LCD_WBTRACE_SFT		0x04
#define LCD_FRMCOUNT_BIT	0x00000008
#define LCD_FRMCOUNT_SFT	0x03
#define LCD_LCDSTOP_BIT		0x00000004
#define LCD_LCDSTOP_SFT		0x02
#define LCD_UNDERRUN_BIT	0x00000002
#define LCD_UNDERRUN_SFT	0x01
#define LCD_LCDVS_BIT		0x00000001
#define LCD_LCDVS_SFT		0x00

//------------------------------
// LCD_INTRAWSTATUS
//------------------------------
#define LCD_WBTRACERAW_BIT	0x00000010
#define LCD_WBTRACERAW_SFT	0x04
#define LCD_FRMCOUNTRAW_BIT	0x00000008
#define LCD_FRMCOUNTRAW_SFT	0x03
#define LCD_LCDSTOPRAW_BIT	0x00000004
#define LCD_LCDSTOPRAW_SFT	0x02
#define LCD_UNDERRUNRAW_BIT	0x00000002
#define LCD_UNDERRUNRAW_SFT	0x01
#define LCD_LCDVSRAW_BIT	0x00000001
#define LCD_LCDVSRAW_SFT	0x00

//------------------------------
// LCD_INTENSET
//------------------------------
#define LCD_WBTRACEEN_BIT	0x00000010
#define LCD_WBTRACEEN_SFT	0x04
#define LCD_FRMCOUNTEN_BIT	0x00000008
#define LCD_FRMCOUNTEN_SFT	0x03
#define LCD_LCDSTOPEN_BIT	0x00000004
#define LCD_LCDSTOPEN_SFT	0x02
#define LCD_UNDERRUNEN_BIT	0x00000002
#define LCD_UNDERRUNEN_SFT	0x01
#define LCD_LCDVSEN_BIT		0x00000001
#define LCD_LCDVSEN_SFT		0x00

#define LCD_INTENSET_ALL_BIT   (LCD_WBTRACEEN_BIT | LCD_FRMCOUNTEN_BIT | LCD_LCDSTOPEN_BIT | LCD_UNDERRUNEN_BIT | LCD_LCDVSEN_BIT)

//------------------------------
// LCD_INTENCLR
//------------------------------
#define LCD_WBTRACEMASK_BIT	0x00000010
#define LCD_WBTRACEMASK_SFT	0x04
#define LCD_FRMCOUNTMASK_BIT	0x00000008
#define LCD_FRMCOUNTMASK_SFT	0x03
#define LCD_LCDSTOPMASK_BIT	0x00000004
#define LCD_LCDSTOPMASK_SFT	0x02
#define LCD_UNDERRUNMASK_BIT	0x00000002
#define LCD_UNDERRUNMASK_SFT	0x01
#define LCD_LCDVSMASK_BIT	0x00000001
#define LCD_LCDVSMASK_SFT	0x00

#define LCD_INTENCLR_ALL_BIT   (LCD_WBTRACEMASK_BIT | LCD_FRMCOUNTMASK_BIT | LCD_LCDSTOPMASK_BIT | LCD_UNDERRUNMASK_BIT | LCD_LCDVSMASK_BIT)

//------------------------------
// LCD_INTFFCLR
//------------------------------
#define LCD_WBTRACECLR_BIT	0x00000010
#define LCD_WBTRACECLR_SFT	0x04
#define LCD_FRMCOUNTCLR_BIT	0x00000008
#define LCD_FRMCOUNTCLR_SFT	0x03
#define LCD_LCDSTOPCLR_BIT	0x00000004
#define LCD_LCDSTOPCLR_SFT	0x02
#define LCD_UNDERRUNCLR_BIT	0x00000002
#define LCD_UNDERRUNCLR_SFT	0x01
#define LCD_LCDVSCLR_BIT	0x00000001
#define LCD_LCDVSCLR_SFT	0x00

#define LCD_INTFFCLR_ALL_BIT   (LCD_WBTRACECLR_BIT | LCD_FRMCOUNTCLR_BIT | LCD_LCDSTOPCLR_BIT | LCD_UNDERRUNCLR_BIT | LCD_LCDVSCLR_BIT)

//------------------------------
// LCD_FRAMECOUNT
//------------------------------
#define LCD_FCCLR_BIT		0x01000000
#define LCD_FCCLR_SFT		0x18
#define LCD_FCEN_BIT		0x00010000
#define LCD_FCEN_SFT		0x10
#define LCD_ACTF_BIT		0x0000FF00
#define LCD_ACTF_SFT		0x08
#define LCD_INTF_BIT		0x000000FF
#define LCD_INTF_SFT		0x00




#endif /* _LCDC_H_ */
