/* 2009-08-24: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/mp200_common.h
 * Function        : Common parameters for LCD Driver
 * Release Version : 
 * Release Date    : 
 *
 * Copyright (C) NEC Electronics Corporation 2005
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _MP200_COMMON_H_
#define _MP200_COMMON_H_


/********************************************************
 *  Include Files                                       *
 *******************************************************/
#include <asm/arch/img.h>
#include <asm/arch/fbcommon.h>
#ifdef CONFIG_PGPF_FB_PND_EXT
#include <asm/arch/navufb.h>
#endif /* CONFIG_PGPF_FB_PND_EXT */


/********************************************************
 *  Definitions                                         *
 *******************************************************/
/* image infomation from V4L2 */
/* YUV format */
#define V4L2_FORMAT_YUV420Pl	IMG_FORMAT_YUV420_PLANAR
#define V4L2_FORMAT_YUV420Pl2	IMG_FORMAT_YUV420_SEMI_PLANAR
#define V4L2_FORMAT_YUV422Pl	IMG_FORMAT_YUV422_PLANAR
#define V4L2_FORMAT_YUV422Pl2	IMG_FORMAT_YUV422_SEMI_PLANAR
#define V4L2_FORMAT_YUV422Px	IMG_FORMAT_YUV422_INTERLEAVE
/* endian */
#define V4L2_BIG_ENDIAN		0
#define V4L2_LITTLE_ENDIAN	1


/* image infomation from fb driver */
#define FB_MASK_COLOR_DISP_OFF	TC_COLOR_DISABLE
#define FB_MASK_COLOR_DISP_ON	TC_COLOR_ENABLE

#ifdef CONFIG_MACH_PGPF
/* layer control */
#define FB_CONTROL_ON  1
#define FB_CONTROL_OFF 0
#endif /* CONFIG_MACH_PGPF */

/* mix_image_mode */
#define ONLY_2D_MODE		0
#define DSP_AND_2D_MODE		1


/* fb data update ON/OFF */
#define FB_UPDATE_OFF		0
#define FB_UPDATE_ON		1
#define FB_ABSOLUTERY_UPDATE	2


/********************************************************
 *  NTS Definitions                                     *
 *******************************************************/
/*     mp200_nts_reserve(int iActiveDevice, int iSetOutmode); */
/*     ntshw_reserve(int iSetOutmode); */
/* for iActiveDevice */
#define NTS_NONACTIVE		0x00
#define NTS_ACTIVE_FB		0x01
#define NTS_ACTIVE_V4L2		0x02

/* for iSetOutmode */
#define NTS_OUTPUT_DISABLE	0
#define NTS_OUTPUT_NTSC		1
#define NTS_OUTPUT_PAL		2


/********************************************************
 *  Structure                                           *
 *******************************************************/
/* Image data (use I/F V4L2 -> LCD  or  fb -> LCD ) */
typedef struct _IMAGE_DATA {
	unsigned int  size;		/* address addition               */
	unsigned long yrgbaddr;		/* Y  plane address               */
	unsigned long uvaddr;		/* UV plane address               */
	unsigned long vaddr;		/* V  plane address               */
	unsigned int  x;		/* x start pixcel                 */
	unsigned int  y;		/* y start pixcel                 */
	unsigned int  hsize;		/* image width                    */
	unsigned int  vsize;		/* image height                   */
} IMAGE_DATA;


typedef struct _SCREEN_DATA {
	unsigned int  x;		/* x outputt pixcel               */
	unsigned int  y;		/* y outputt pixcel               */
	unsigned int  hsize;		/* output width                   */
	unsigned int  vsize;		/* output height                  */
} SCREEN_DATA;


typedef struct _FRAME_DATA {
	void *dev;			/* mp200_v4l2_device               */
	void *buf;			/* videobuf_buffer                 */
} FRAME_DATA;


/* Image data from V4L2   (use I/F V4L2 -> LCD) */
typedef struct _V4L2_IMAGE_INFO {
#ifdef CONFIG_MACH_PGPF
	unsigned int  control;
#endif /* CONFIG_MACH_PGPF */
	IMAGE_DATA    image_data;	/* input  image data from v4l2     */
	SCREEN_DATA   screen_data;	/* output image data to LCD/NTSC   */
	FRAME_DATA    frame_data;	/* v4l2 data delivery              */
	unsigned int  yuvfmt;		/* input image data format         */
//@	unsigned long offsetx;		/* offset x                        */
//@	unsigned long offsety;		/* offset y                        */
	unsigned int  endian;		/* endian type                     */
} V4L2_IMAGE_INFO;


/* Image data from fb driver  (use I/F fb -> LCD) */
typedef struct _FB_IMAGE_INFO {
#ifdef CONFIG_MACH_PGPF
	unsigned int  control;      /* disp on/off                     */
#endif /* CONFIG_MACH_PGPF*/
	IMAGE_DATA    image_data;	/* input image data from fb driver */
	unsigned int  maskcolr;		/* maskcolor                       */
	unsigned int  maskcolrflg;	/* maskcolor flag                  */
	unsigned int  alpha;		/* alpha                           */
	unsigned int  invflg;		/* inverse mode flag               */
	unsigned int  mix_buf_page;	/* mix 2D buffer frame page        */
	unsigned int  update_flag;	/* update flag ON/Off              */
} FB_IMAGE_INFO;


#endif /* _MP200_COMMON_H_ */
