/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/mp200_imc.h
 * Function        : IMC Driver corresponding to V4L2  definitions
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _MP200_IMC_H_
#define _MP200_IMC_H_


/********************************************************
 *  Definitions                                         *
 *******************************************************/
#define SET_UPDATE_ATTR 1
#define SET_INIT_ATTR   0


/********************************************************
 *  Macros                                              *
 *******************************************************/


/********************************************************
 *  Sub Structure                                       *
 *******************************************************/
/* ----- IMC GAMMA Set Structure --------------------- */
struct imc_gamma_param
{
	unsigned long en;
	unsigned long adr;
	unsigned long *data;
};

/* ----- IMC WB Set Structure ------------------------ */
struct imc_wb_param
{
	unsigned long areaadr;
	unsigned long hoffset;
	unsigned long format;
	unsigned long hsize;
	unsigned long vsize;
};

/* ----- IMC all Layer Set Structure ----------------- */
struct imc_yuv_param
{
	unsigned long ygain;
	unsigned long ugain;
	unsigned long vgain;
	unsigned long yuv2rgb;
	unsigned long coef_r[4];
	unsigned long coef_g[4];
	unsigned long coef_b[4];
};

/* ----- IMC Kercolor Structure ---------------------- */
struct imc_keycolor_param
{
	unsigned long l0_keycolor;
	unsigned long l1x_keycolor;
};

/* ----- IMC Bytelane Structure ---------------------- */
struct imc_bytelane_param
{
	unsigned long l2a_bytelane;
	unsigned long l2b_bytelane;
};

/* ----- IMC Layer0/1 Set Structure ------------------ */
struct l01_param
{
	unsigned long control;
	unsigned long format;
	unsigned long keyenable;
/*	unsigned long keycolor; */
	unsigned long alpha;
	unsigned long resize;
	unsigned long offset;
	unsigned long frameadr;
	unsigned long position;
	unsigned long size;
};

/* ----- IMC Layer2 Set Structure -------------------- */
struct l2_param
{
	unsigned long control;
	unsigned long format;
	unsigned long bufsel;
	unsigned long resize;
	unsigned long mirror;
	unsigned long offset;
	unsigned long frameadr_yp;
	unsigned long frameadr_up;
	unsigned long frameadr_vp;
	unsigned long frameadr_yq;
	unsigned long frameadr_uq;
	unsigned long frameadr_vq;
	unsigned long position;
	unsigned long size;
};

/* ----- IMC LayerBG Set Structure ------------------- */
struct bg_param
{
	unsigned long format;
	unsigned long resize;
	unsigned long offset;
	unsigned long frameadr;
};


/********************************************************
 *  Main Structure                                      *
 *******************************************************/
/* ----- IMC Pre-Ser Structure ----------------------- */
struct mp200_imc_preset
{
	unsigned long             *imc_control;
	struct imc_gamma_param    *gamma;
	struct imc_wb_param       *wb;
	struct imc_yuv_param      *yuv;
	struct imc_keycolor_param *keycolor;
	struct imc_bytelane_param *bytelane;
#ifdef CONFIG_MACH_PGPF
	unsigned long             *imc_argb4444_mode;
#endif /* CONFIG_MACH_PGPF */
};

/* ----- IMC Update VSYNC Structure ------------------ */
struct mp200_imc_update_vsync
{
	unsigned long cpubufsel;
	unsigned long mirror;
};

/* ----- IMC Update Reserve Structure ---------------- */
struct mp200_imc_update_reserve
{
	struct l01_param *lo;
	struct l01_param *l1a;
	struct l01_param *l1b;
	struct l01_param *l1c;
	struct l2_param  *l2a;
	struct l2_param  *l2b;
	struct bg_param  *bg;
};




/********************************************************
 *  Variables                                           *
 *******************************************************/
typedef void (*imc_callback_func_refresh)(void);
typedef void (*imc_callback_func_wb)(int status);


/********************************************************
 *  external function                                   *
 *******************************************************/
extern int  mp200_imc_init_module(void);
extern void mp200_imc_exit_module(void);
extern int  mp200_imc_set_preset(struct mp200_imc_preset *imc_preset, int setmode);
extern int  mp200_imc_set_update_vsync(struct mp200_imc_update_vsync *imc_vsync);
extern int  mp200_imc_set_update_reserve(struct mp200_imc_update_reserve *imc_reserve, imc_callback_func_refresh callback_refresh);
extern int  mp200_imc_set_callback(imc_callback_func_refresh callback_refresh, imc_callback_func_wb callback_wb);
#if 0 /* this function used in IMC-LCD asynchronous mode. */
extern int  mp200_imc_start(void);
#endif
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
extern int  mp200_imc_suspend(struct platform_device *dev, pm_message_t state);
extern int  mp200_imc_resume(struct platform_device *dev);
#endif /* CONFIG_PM || CONFIG_DPM */



#endif /* _MP200_FB_H_ */
