/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/mp200_lcd.c
 * Function        : LCD driver corresponding to V4L2
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


/********************************************************
 *  Include Files                                       *
 *******************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/timer.h>
#include <linux/sched.h>
#include <linux/device.h>
#include <linux/platform_device.h>

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
#include <asm/arch/pm.h>
#endif /* CONFIG_PM || CONFIG_DPM */

#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>
#include <asm/arch/pmu.h>
#include <asm/arch/em1_mem.h>

#include "mp200_common.h"
#include "mp200_fb.h"

#include "lcdc.h"
#include "mp200_lcd_common.h"
#include "mp200_lcdhw.h"

#include "imc.h"
#include "mp200_imc.h"

#ifdef CONFIG_MACH_PGPF
#include <asm/arch/pgpf/lcdhw.h>
#endif /* CONFIG_MACH_PGPF */

/********************************************************
 *  Definitions                                         *
 *******************************************************/
#define _DEBUG_LCD  0x00 /* 00008421(bit) */
                         /* 0x01: debug function in
                          * 0x02: debug function out
                          * 0x80: debug semafore
                          */


/********************************************************
 *  Definitions                                         *
 *******************************************************/
#define DEV_NAME "mp200_lcd"


/********************************************************
 *  Macros                                              *
 *******************************************************/
#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_LCD
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif


/********************************************************
 *  Structure                                           *
 *******************************************************/
/* structure lcd device status */
typedef struct mp200_lcd_dev {
	struct semaphore      sem_image_data;		/* semafore v4l2/fb/img local data */
	struct semaphore      sem_image_refresh;	/* semafore IMC refresh */
	spinlock_t	      lcd_lock;		/* spin lock LCD timer flag        */
} MP200_LCD_DEV;


/********************************************************
 *  Variables                                           *
 *******************************************************/
/* input image data from fb driver */
static  unsigned int  uiMixFrameBufferPage_tmp;		/* (2D) name of mix frame buffer temp   */

/* other img infomation */
static  unsigned int  uiMaskColor_tmp;			/* mask color */
static  unsigned int  uiAlpha_tmp;			/* alpha      */

/* Flags */
static           int  iMixDSPFlg_tmp;			/* DSP mix ON/OFF            */
static           int  iDSPPauseFlg;			/* DSP pause status          */
static  unsigned int  uiMaskColorFlag_tmp;		/* mask color ON/OFF Flag    */
static  unsigned int  uiAbsolutelyUpFlag;		/* absolutely update flag    */
        unsigned int  uiInverseFlag_tmp;		/* inverse mode Flag         */

/*
 * Smem
 */
/* LCDC MMIO */
static  unsigned long LCDCMmio       = LCDC_BASE;
static  unsigned long LCDCMmioLength = LCDC_SIZE;
        char         *LCDCMmioV;

#if 0
/* FrameBuf(LCD) */
//@     unsigned long FrameBuf       = SMEM_START + FRAMEBUF_OFFSET;
//@     unsigned long FrameBufLength = FRAMEBUF_LENGTH;
//@     char         *FrameBufV;
#endif

/* LCD device status */
static MP200_LCD_DEV       lcd_dev;

/*
 * V4L2 dev data
 */
static void *pvDevData_tmp;		/* mp200_v4l2_device               */
static void *pvVideoBuf_tmp;		/* videobuf_buffer                 */

/*
 * timer
 */
static  struct timer_list lcd_timer;
static  int               iTimerFlag;

/* lcd_refresh_flg */
enum imc_refresh_status {
	IMC_R_IDLE         = 0,
	IMC_R_WAIT_REFRESH = 1,
	IMC_R_REFRESHED    = 2,
	IMC_R_DISPLAYED    = 3,
};

typedef struct mp200_lcd_var {
	struct list_head      list;
	struct semaphore      sem_image_flag;		/* semafore v4l2/fb/img local flag */

	/* input image data from fb driver */
	unsigned int  uiMixFrameBufferPage;		/* (2D) name of mix frame buffer        */
	/* Flags */
	int  iMixDSPFlgToFB;			/* return FB DSP mix ON/OFF  */
	int  iCallbackV4L2Flg;			/* need to callback to V4L2  */
	/* V4L2 dev data */
	void *pvDevData;			/* mp200_v4l2_device               */
	void *pvVideoBuf;		/* videobuf_buffer                 */

	enum imc_refresh_status lcd_refresh_flg;

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
	int          ctrl_func;
#endif /* CONFIG_PM || CONFIG_DPM */
} MP200_LCD_VAR;
static MP200_LCD_VAR lcd_var[2];
static int index_lcd_var_next = 0;

static struct list_head list_lcd_var;

/********************************************************
 *  Prototype declarations of function                  *
 *******************************************************/
/*
 * LCD Driver
 */
/* Public function */
       int          mp200_lcd_set_v4l2_image(V4L2_IMAGE_INFO *v4l2_image_info);
       int          mp200_lcd_set_fb_image(FB_IMAGE_INFO *fb_image_info);
       int  __init  mp200_lcd_init_module(void);
static void         mp200_lcd_remove(void);
       void         mp200_lcd_exit_module(void);
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
       int          mp200_lcd_suspend(struct platform_device *dev, pm_message_t state);
static int          lcd_suspend_chk_lcdout(void);
       int          mp200_lcd_resume(struct platform_device *dev);

       int          IMC_reset_flg   = 0;
#endif /* CONFIG_PM || CONFIG_DPM */
static int          DPM_suspend_flg = 0;

/* Private function */
static void         mix_image(int iCallbackV4L2_tmp, int update, int type);
static void         lcd_irq_handler_callback(void);
       void         lcd_callback_imc_refresh(void);
       void         lcd_callback_imc_wb(int status);

static void         set_val_init(void);
static int          lcd_probe(void);
static void         lcd_probe_error(int iErrorMode);
static void         lcd_timer_init(void);
static void         lcd_timeout(unsigned long data);


/*
 * work queue
 */
static struct workqueue_struct *mp200_lcd_workqueue;
static struct work_struct       wk_timeout_bottom_half;
static void         lcd_timeout_bottom_half_do(struct work_struct *num);


/*
 * extern V4L2 Driver function
 */
extern void         mp200_v4l2_lcd_callback(FRAME_DATA frame_data);
extern void         mp200_v4l2_lcd_refresh_callback(FRAME_DATA frame_data);


/******/
#include "mp200_lcd_perf.h"


/********************************************************
 *  Function Definitions                                *
 *******************************************************/
/******************************************************************************
* MODULE   : mp200_lcd_set_v4l2_image
* FUNCTION : set image data from v4l2 to LCD local
* RETURN   : 0 : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
int mp200_lcd_set_v4l2_image(V4L2_IMAGE_INFO *v4l2_image_info)
{
	unsigned long flags;

	int bpp_y, bpp_uv, bpp_v; /* bit par pixel  */
	int ppl_y, ppl_uv, ppl_v; /* pixel par line */

#if LCD_PERF
if(!start_perf){
printk("log_start!\n");
	map_nowtime();
	cnt_time = 0;
	start_perf = 1;
}
set_nowtime(SEQ_V4L2);
#endif /* LCD_PERF */
	printk_dbg((_DEBUG_LCD & 0x01), "<start>\n");

	/* Stop pause timer */
	spin_lock_irqsave(&lcd_dev.lcd_lock, flags);
	del_timer(&lcd_timer);
	iTimerFlag = LCD_TIMER_STOP;
	spin_unlock_irqrestore(&lcd_dev.lcd_lock, flags);
	iDSPPauseFlg = PAUSE_DSP_OFF;

	/* get v4l2 data semafore */
	down(&lcd_dev.sem_image_data);
	printk_dbg((_DEBUG_LCD & 0x80), "down(A) set_v4l2\n");

	if(init_is_first){
		/* LCD not initialize */
		up(&lcd_dev.sem_image_data);
		printk_dbg((_DEBUG_LCD & 0x80), "up(A) set_v4l2\n");
		return -1;
	}

	/* DSP stop */
	if(v4l2_image_info->image_data.yrgbaddr == 0){
		printk_dbg((_DEBUG_LCD & 0x01), "<stop DSP>\n");
		/* movie off */
		iMixDSPFlg_tmp = MIX_DSP_OFF;
		/* update Layer data */
		mix_image(CALLBACK_V4L2_OFF, UPDATE_ON, CLR_LAYER_V4L2);
		return 0;
	}

	/* select 2D(1) */
	memset(&v4l2_layer, 0, sizeof(struct l2_param));

	/* movie on */
	iMixDSPFlg_tmp   = MIX_DSP_ON;

	v4l2_layer.offset      = v4l2_image_info->image_data.size;
	v4l2_layer.position    = (v4l2_image_info->screen_data.y     << IMC_Lx_POSY_SFT  | v4l2_image_info->screen_data.x     << IMC_Lx_POSX_SFT);  /* y, x */
	v4l2_layer.size        = (v4l2_image_info->screen_data.vsize << IMC_Lx_SIZEY_SFT | v4l2_image_info->screen_data.hsize << IMC_Lx_SIZEX_SFT); /* h, w */
	v4l2_layer.control     = IMC_Lx_CONTROL_ON;
	v4l2_layer.resize      = IMC_Lx_RESIZE_DISABLE;
	v4l2_layer.mirror      = IMC_L2x_MIRROR_OFF;

	switch(v4l2_image_info->yuvfmt){
	case V4L2_FORMAT_YUV420Pl2: /* YUV420 Semi-Planar */
		printk_dbg((_DEBUG_LCD & 0x02), "format(YUV420Pl2)\n");
		bpp_y = 8;  bpp_uv = 8; bpp_v = 0; ppl_y = 1; ppl_uv = 2; ppl_v = 0;
		v4l2_layer.format      = IMC_Lx_FORMAT_YUV420Pl2;
		v4l2_layer.frameadr_yp = v4l2_image_info->image_data.yrgbaddr + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_y  / 8 / ppl_y  + v4l2_image_info->image_data.x * bpp_y  / 8;
		v4l2_layer.frameadr_up = v4l2_image_info->image_data.uvaddr   + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_uv / 8 / ppl_uv + v4l2_image_info->image_data.x * bpp_uv / 8;
		v4l2_layer.frameadr_vp = 0;
		break;
	case V4L2_FORMAT_YUV422Pl2: /* YUV422 Semi-Planar */
		printk_dbg((_DEBUG_LCD & 0x02), "format(YUV422Pl2)\n");
		bpp_y = 8;  bpp_uv = 8; bpp_v = 0; ppl_y = 1; ppl_uv = 1; ppl_v = 0;
		v4l2_layer.format      = IMC_Lx_FORMAT_YUV422Pl2;
		v4l2_layer.frameadr_yp = v4l2_image_info->image_data.yrgbaddr + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_y  / 8 / ppl_y  + v4l2_image_info->image_data.x * bpp_y  / 8;
		v4l2_layer.frameadr_up = v4l2_image_info->image_data.uvaddr   + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_uv / 8 / ppl_uv + v4l2_image_info->image_data.x * bpp_uv / 8;
		v4l2_layer.frameadr_vp = 0;
		break;
	case V4L2_FORMAT_YUV420Pl: /* YUV420 Planar      */
		printk_dbg((_DEBUG_LCD & 0x02), "format(YUV420Pl)\n");
		bpp_y = 8;  bpp_uv = 4; bpp_v = 4; ppl_y = 1; ppl_uv = 4; ppl_v = 4;
		v4l2_layer.format      = IMC_Lx_FORMAT_YUV420Pl;
		v4l2_layer.frameadr_yp = v4l2_image_info->image_data.yrgbaddr + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_y  / 8 / ppl_y  + v4l2_image_info->image_data.x * bpp_y  / 8;
		v4l2_layer.frameadr_up = v4l2_image_info->image_data.uvaddr   + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_uv / 8 / ppl_uv + v4l2_image_info->image_data.x * bpp_uv / 8;
		v4l2_layer.frameadr_vp = v4l2_image_info->image_data.vaddr    + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_v  / 8 / ppl_v  + v4l2_image_info->image_data.x * bpp_v  / 8;
		break;
	case V4L2_FORMAT_YUV422Px: /* YUV422 Interleave  */
		printk_dbg((_DEBUG_LCD & 0x02), "format(YUV422Px)\n");
		bpp_y = 16; bpp_uv = 0; bpp_v = 0; ppl_y = 1; ppl_uv = 0; ppl_v = 0;
		v4l2_layer.format      = IMC_Lx_FORMAT_YUV422Px;
		v4l2_layer.frameadr_yp = v4l2_image_info->image_data.yrgbaddr + v4l2_image_info->image_data.y * v4l2_layer.offset * bpp_y  / 8 / ppl_y  + v4l2_image_info->image_data.x * bpp_y  / 8;
		v4l2_layer.frameadr_up = 0;
		v4l2_layer.frameadr_vp = 0;
		break;
	default:
		printk_dbg((_DEBUG_LCD & 0x02), "format(%ld)\n", v4l2_layer.format);
		up(&lcd_dev.sem_image_data); /* release data semafore */
		printk_dbg((_DEBUG_LCD & 0x80), "up(A) set_v4l2\n");
		return -1;
	}
	v4l2_layer.frameadr_yq = 0;
	v4l2_layer.frameadr_uq = 0;
	v4l2_layer.frameadr_vq = 0;

	/* set frame data from V4L2 */
	pvDevData_tmp  = v4l2_image_info->frame_data.dev;
	pvVideoBuf_tmp = v4l2_image_info->frame_data.buf;

	/* mix - image data from v4l2 and image data from fb driver (DSP and 2D) */
	mix_image(CALLBACK_V4L2_ON, UPDATE_ON, SET_LAYER_V4L2);

	/* start PAUSE timer */
	spin_lock_irqsave(&lcd_dev.lcd_lock, flags);
	if(iTimerFlag == LCD_TIMER_STOP && !DPM_suspend_flg)
	{
		lcd_timer.expires = jiffies + (MIX_DSP_TIMEOUT * 10 * HZ / 1000);
		add_timer(&lcd_timer);
		iTimerFlag = LCD_TIMER_START;
	}
	spin_unlock_irqrestore(&lcd_dev.lcd_lock, flags);
#if LCD_PERF
set_nowtime(SEQ_V4L2);
#endif /* LCD_PERF */

	return 0;
}


/******************************************************************************
* MODULE   : mp200_lcd_set_fb_image
* FUNCTION : set image data from Frame Buffer to LCD local
* RETURN   :  0 : success
*            -1 : faile
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
#if defined(CONFIG_PGPF_FB_PND_EXT) || defined(CONFIG_PGPF_FB_UX_EXT)
int mp200_lcd_set_fb_image_ex2(FB_IMAGE_INFO *fb_image_info, FB_IMAGE_INFO* bg_image_info, V4L2_IMAGE_INFO* l2_image_info, FB_IMAGE_INFO* console_image_info)
#else /* CONFIG_PGPF_FB_*_EXT */
int mp200_lcd_set_fb_image(FB_IMAGE_INFO *fb_image_info)
#endif /* CONFIG_PGPF_FB_*_EXT */
{
	printk_dbg((_DEBUG_LCD & 0x01), "<start>\n");

#if LCD_PERF
if(!start_perf){
printk("log_start!\n");
	map_nowtime();
	cnt_time = 0;
	start_perf = 1;
}
set_nowtime(SEQ_FB);
#endif /* LCD_PERF */
	/* get data semafore */
#if 1
	down(&lcd_dev.sem_image_data);
#else
	if(down_interruptible(&lcd_dev.sem_image_data)){
		printk_dbg((_DEBUG_LCD & 0x80), "down(A) FAILED!! set_fb\n");
		return -1;
	}
#endif
	printk_dbg((_DEBUG_LCD & 0x80), "down(A) set_fb\n");

	/* Set other infomation of input data from fb */
	uiAlpha_tmp              = fb_image_info->alpha;
	uiInverseFlag_tmp        = fb_image_info->invflg;
	uiMixFrameBufferPage_tmp = fb_image_info->mix_buf_page;
	printk_dbg((_DEBUG_LCD & 0x01), "MixFrameBuffer(%d)\n", uiMixFrameBufferPage_tmp);

	uiMaskColorFlag_tmp      = fb_image_info->maskcolrflg;
	uiMaskColor_tmp          = fb_image_info->maskcolr;
	uiMaskColor_tmp          = (((((uiMaskColor_tmp & 0xF800) >> 11) << 1) + ((uiMaskColor_tmp & 0x8000) >> 15)) << IMC_Lx_KEYR_SFT) | 
				     (((uiMaskColor_tmp & 0x07E0) >>  5)                                             << IMC_Lx_KEYG_SFT) |
				   (((((uiMaskColor_tmp & 0x001F) >>  0) << 1) + ((uiMaskColor_tmp & 0x0010) >>  4)) << IMC_Lx_KEYB_SFT);

	/* select 2D(1) */
	memset(&fb_layer, 0, sizeof(struct l01_param));

#ifdef CONFIG_PGPF_FB_LAYER2
	if (l2_image_info != NULL) {
		l2_layer.control = l2_image_info->control;
		l2_layer.format = l2_image_info->yuvfmt;
		switch (l2_layer.format) {
		case IMC_Lx_FORMAT_RGB666:
		case IMC_Lx_FORMAT_RGB565:
		case IMC_Lx_FORMAT_YUV422Px:
			/* Packed Format */
			l2_layer.frameadr_yp = l2_image_info->image_data.yrgbaddr
				+ (l2_image_info->image_data.x * 2 + l2_image_info->image_data.y * l2_image_info->image_data.size);
			l2_layer.frameadr_up = 0;
			l2_layer.frameadr_vp = 0;
			break;
		case IMC_Lx_FORMAT_YUV422Pl2:
			l2_layer.frameadr_yp = l2_image_info->image_data.yrgbaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size);
			l2_layer.frameadr_up = l2_image_info->image_data.uvaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size);
			l2_layer.frameadr_vp = 0;
			break;
		case IMC_Lx_FORMAT_YUV422Pl:
			l2_layer.frameadr_yp = l2_image_info->image_data.yrgbaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size);
			l2_layer.frameadr_up = l2_image_info->image_data.uvaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size) / 2;
			l2_layer.frameadr_vp = l2_image_info->image_data.vaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size) / 2;
			break;
		case IMC_Lx_FORMAT_YUV420Pl2:
			l2_layer.frameadr_yp = l2_image_info->image_data.yrgbaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size);
			l2_layer.frameadr_up = l2_image_info->image_data.uvaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size) / 2;
			l2_layer.frameadr_vp = 0;
			break;
		case IMC_Lx_FORMAT_YUV420Pl:
			l2_layer.frameadr_yp = l2_image_info->image_data.yrgbaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size);
			l2_layer.frameadr_up = l2_image_info->image_data.uvaddr
				+ (l2_image_info->image_data.x + l2_image_info->image_data.y * l2_image_info->image_data.size) / 4;
			l2_layer.frameadr_vp = l2_image_info->image_data.vaddr
				+ (l2_image_info->image_data.x +  l2_image_info->image_data.y * l2_image_info->image_data.size) / 4;
			break;
		default:
			printk_err("invalid format!\n");
			up(&lcd_dev.sem_image_data);
			return -1;
		}
//		l2_layer.bufsel = 0;
		l2_layer.offset = l2_image_info->image_data.size;
		l2_layer.position = l2_image_info->screen_data.y << IMC_Lx_POSY_SFT | l2_image_info->screen_data.x << IMC_Lx_POSX_SFT; /* Disp Position */
		l2_layer.size = (l2_image_info->screen_data.vsize << IMC_Lx_SIZEY_SFT | l2_image_info->screen_data.hsize << IMC_Lx_SIZEX_SFT); /* Disp Size */
	}
#endif /* CONFIG_PGPF_FB_LAYER2 */

#ifdef CONFIG_PGPF_FB_PND_EXT
	if (bg_image_info != NULL) {
		if (bg_image_info->control) { 
			bg_layer.format = IMC_LBG_FORMAT_RGB565; 
		} else { 
			bg_layer.format = IMC_LBG_FORMAT_BLACK; 
		} 
		bg_layer.frameadr = bg_image_info->image_data.yrgbaddr /* Base Address */
			+ (bg_image_info->image_data.x * BASE_BPP / 8 + bg_image_info->image_data.y * bg_image_info->image_data.size); /* Frame Position */
		bg_layer.offset = bg_image_info->image_data.size;
	}

	if (console_image_info != NULL) {
		console_layer.control = console_image_info->control;
		console_layer.frameadr = console_image_info->image_data.yrgbaddr /* Base Address */
			+ (console_image_info->image_data.x * FB_BPP / 8 + console_image_info->image_data.y * console_image_info->image_data.size); /* Frame Position */
		console_layer.offset = console_image_info->image_data.size;
		console_layer.alpha = (console_image_info->alpha * ((1UL << 6) -1) + ((1UL << 8) -1) / 2) / ((1UL << 8) -1);
		console_layer.position = 0;
		console_layer.size = (console_image_info->image_data.vsize << IMC_Lx_SIZEY_SFT | console_image_info->image_data.hsize << IMC_Lx_SIZEX_SFT);
	}
#endif /* CONFIG_PGPF_FB_PND_EXT */

	fb_layer.format   = IMC_Lx_FORMAT_RGB565;
	fb_layer.resize   = IMC_Lx_RESIZE_DISABLE;
	fb_layer.offset   = fb_image_info->image_data.size;

#ifdef CONFIG_MACH_PGPF
	fb_layer.control  = fb_image_info->control;
#else /* CONFIG_MACH_PGPF */
	fb_layer.control  = IMC_Lx_CONTROL_ON;
#endif /* CONFIG_MACH_PGPF */

#ifdef CONFIG_PGPF_FB_PND_EXT
	fb_layer.keyenable = IMC_Lx_KEYEN_ENABLE;
	fb_layer.alpha = (0xFF * ((1UL << 6) -1) + ((1UL << 8) -1) / 2) / ((1UL << 8) -1);
#else /* CONFIG_PGPF_FB_PND_EXT */
	if(fb_image_info->maskcolrflg == TC_COLOR_ENABLE)
		fb_layer.keyenable = IMC_Lx_KEYEN_ENABLE;
	else	fb_layer.keyenable = IMC_Lx_KEYEN_DISABLE;
	fb_layer.alpha    = (uiAlpha_tmp * ((1UL << 6) -1) + ((1UL << 8) -1) / 2) / ((1UL << 8) -1); /* 8bit to 6bit */
#endif /* CONFIG_PGPF_FB_PND_EXT */
#ifdef CONFIG_MACH_PGPF
	fb_layer.position = 0;
	fb_layer.frameadr = fb_image_info->image_data.yrgbaddr /* Base Address */
#ifdef CONFIG_PGPF_FB_PND_EXT
		+ (fb_image_info->image_data.x * OVERLAY_BPP / 8 + fb_image_info->image_data.y * fb_image_info->image_data.size); /* Frame Poisition */
#else /* CONFIG_PGPF_FB_PND_EXT */
		+ (fb_image_info->image_data.x * FB_BPP / 8 + fb_image_info->image_data.y * fb_image_info->image_data.size); /* Frame Poisition */
#endif /* CONFIG_PGPF_FB_PND_EXT */
#else /* CONFIG_MACH_PGPF */
	fb_layer.position = (fb_image_info->image_data.y     << IMC_Lx_POSY_SFT  | fb_image_info->image_data.x     << IMC_Lx_POSX_SFT);  /* y, x */
	fb_layer.frameadr = fb_image_info->image_data.yrgbaddr; /* image area Y/RGB plane address */
#endif  /* CONFIG_MACH_PGPF */
	fb_layer.size     = (fb_image_info->image_data.vsize << IMC_Lx_SIZEY_SFT | fb_image_info->image_data.hsize << IMC_Lx_SIZEX_SFT); /* h, w */


	/* check update flag */
	switch(fb_image_info->update_flag){
	case FB_UPDATE_ON:  /*    ioctl(MP200_FB_UPDATE_SCRN) & update = 0 */
	                    /* or interval update                          */
		printk_dbg((_DEBUG_LCD & 0x02), "FB_UPDATE_ON\n");

		/* movie on */
		if(iMixDSPFlg_tmp == MIX_DSP_ON){
			/* pause status */
			if (iDSPPauseFlg == PAUSE_DSP_ON){
				mix_image(CALLBACK_V4L2_OFF, UPDATE_ON, SET_LAYER_2D);
			}
			/* not pause status */
			else{
				mix_image(CALLBACK_V4L2_OFF, UPDATE_OFF, SET_LAYER_2D);
			}
		}
		/* movie off */
		else{
			mix_image(CALLBACK_V4L2_OFF, UPDATE_ON, SET_LAYER_2D);
		}
		break;
	case FB_ABSOLUTERY_UPDATE:  /* ioctl(MP200_FB_UPDATE_SCRN) & update = 1 */
		printk_dbg((_DEBUG_LCD & 0x02), "FB_ABSOLUTERY_UPDATE\n");
		/* movie on */
		mix_image(CALLBACK_V4L2_OFF, UPDATE_ON, SET_LAYER_2D);
		break;
	case FB_UPDATE_OFF:  /* ioctl(MP200_FB_SET_MODES) */
	default           :  /* other                     */
		printk_dbg((_DEBUG_LCD & 0x02), "FB_UPDATE_OFF\n");
		mix_image(CALLBACK_V4L2_OFF, UPDATE_OFF, SET_LAYER_2D);
		break;
	}

#if LCD_PERF
set_nowtime(SEQ_FB);
#endif /* LCD_PERF */
	printk_dbg((_DEBUG_LCD & 0x02), "<end>\n");
	return 0;
}

/******************************************************************************
* MODULE   : mix_image
* FUNCTION : By using IMG, mix 2D (and DSP) image
* RETURN   :  0 : success
*          : -1 : failed
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
static void mix_image(int iCallbackV4L2_tmp, int update, int type)
{
	unsigned long flags;
	int i = 0;

	if(update == UPDATE_ON){
		/* get refresh semafore */
		down(&lcd_dev.sem_image_refresh);
		printk_dbg((_DEBUG_LCD & 0x80), "down(C) mix_image\n");

		spin_lock_irqsave(&lcd_dev.lcd_lock, flags);
		i = index_lcd_var_next;
		index_lcd_var_next++;
		if (index_lcd_var_next == 2) index_lcd_var_next = 0;
		spin_unlock_irqrestore(&lcd_dev.lcd_lock, flags);

		/* get flag semafore */
		down(&lcd_var[i].sem_image_flag);
		printk_dbg((_DEBUG_LCD & 0x80), "down(B) mix_image\n");

		lcd_var[i].lcd_refresh_flg = IMC_R_IDLE;

		/* set flag and temp data */
		lcd_var[i].iCallbackV4L2Flg      = iCallbackV4L2_tmp;
		lcd_var[i].uiMixFrameBufferPage  = uiMixFrameBufferPage_tmp;
		lcd_var[i].iMixDSPFlgToFB        = iMixDSPFlg_tmp;

		if(lcd_var[i].iCallbackV4L2Flg == CALLBACK_V4L2_ON){
			/* set frame data from V4L2 */
			lcd_var[i].pvDevData  = pvDevData_tmp;
			lcd_var[i].pvVideoBuf = pvVideoBuf_tmp;
		}
	}

	if(iMixDSPFlg_tmp == MIX_DSP_OFF){
#ifndef CONFIG_MACH_PGPF
		fb_layer.alpha  = (0xFF * ((1UL << 6) -1) + ((1UL << 8) -1) / 2) / ((1UL << 8) -1); /* 8bit to 6bit */
#endif /* !CONFIG_MACH_PGPF */

#ifdef CONFIG_PGPF_FB_LAYER2
		if (l2_layer.control) { 
			if (l2_layer.format == IMC_Lx_FORMAT_YUV422Px) {
				ImcNxtBytelane.l2a_bytelane = IMC_L2x_BYTELANE_422PX_YUYV;
			} else{
				ImcNxtBytelane.l2a_bytelane = IMC_L2x_BYTELANE_INIT;
			}
			/* update bytelane data */
			if (imc_hw_set_update_bytelane(&ImcNxtBytelane)) {
				printk_wrn("IMC update_bytelane() failed.\n");
			}
		}
#endif /* CONFIG_PGPF_FB_LAYER2 */

#if 0
		ImcNxtKeycol.l0_keycolor  = iKeycolor_tmp;
#endif
		ImcNxtKeycol.l1x_keycolor = uiMaskColor_tmp;

		/* update colorkey data */
		if(imc_hw_set_update_keycolor(&ImcNxtKeycol)){
			printk_wrn("IMC update_keycolor() failed.\n");
		}
	}
	else{
#ifndef CONFIG_MACH_PGPF
		fb_layer.alpha  = (uiAlpha_tmp * ((1UL << 6) -1) + ((1UL << 8) -1) / 2) / ((1UL << 8) -1); /* 8bit to 6bit */
#endif /* !CONFIG_MACH_PGPF */

#ifdef CONFIG_PGPF_FB_LAYER2
		if(l2_layer.format == IMC_Lx_FORMAT_YUV422Px){
			ImcNxtBytelane.l2a_bytelane = IMC_L2x_BYTELANE_422PX_YUYV;
		}
		else{
			ImcNxtBytelane.l2a_bytelane = IMC_L2x_BYTELANE_INIT;
		}
#endif /* CONFIG_PGPF_FB_LAYER2 */

		if(v4l2_layer.format == IMC_Lx_FORMAT_YUV422Px){
#ifndef CONFIG_PGPF_FB_LAYER2
			ImcNxtBytelane.l2a_bytelane = IMC_L2x_BYTELANE_INIT;
#endif /* !CONFIG_PGPF_FB_LAYER2 */
			ImcNxtBytelane.l2b_bytelane = IMC_L2x_BYTELANE_422PX_YUYV;
		}
		else{
#ifndef CONFIG_PGPF_FB_LAYER2
			ImcNxtBytelane.l2a_bytelane = IMC_L2x_BYTELANE_INIT;
#endif /* !CONFIG_PGPF_FB_LAYER2 */
			ImcNxtBytelane.l2b_bytelane = IMC_L2x_BYTELANE_INIT;
		}

		/* update bytelane data */
		if(imc_hw_set_update_bytelane(&ImcNxtBytelane)){
			printk_wrn("IMC update_bytelane() failed.\n");
		}
	}

	spin_lock_irqsave(&lcd_dev.lcd_lock, flags);
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
	if(update == UPDATE_ON){
		lcd_var[i].ctrl_func = 2;
	}
#endif /* CONFIG_PM || CONFIG_DPM */

	if(update == UPDATE_ON){
		lcd_var[i].lcd_refresh_flg = IMC_R_WAIT_REFRESH;
		list_add_tail(&lcd_var[i].list, &list_lcd_var);
	}

	/* update Layer data */
	if(imc_hw_set_update_reserve(type, iMixDSPFlg_tmp, update)){
		printk_dbg((_DEBUG_LCD & 0x02), "update_reserve() failed.\n");
	}

	if(update == UPDATE_ON){
		/* set IMC all layer data */
		ImcNxtVsync.mirror = uiInverseFlag_tmp;
		if(imc_hw_set_update_vsync(&ImcNxtVsync)){
			printk_wrn("IMC update_vsync() failed.\n");
		}
	}

	if(update == UPDATE_ON){
		change_frame();
	}

	/* release data semafore */
	up(&lcd_dev.sem_image_data);
	printk_dbg((_DEBUG_LCD & 0x80), "up(A) mix_image\n");

	spin_unlock_irqrestore(&lcd_dev.lcd_lock, flags);
}


/*****************************************************************************
* MODULE   : lcd_irq_handler_callback
* FUNCTION : LCDC interrupt handler
* RETURN   : 
* NOTE     : none
* CREATE   : 2007/07/09 M.Haguro
* UPDATE   :
******************************************************************************/
static void lcd_irq_handler_callback(void)
{
	FRAME_DATA frame_data;
	MP200_LCD_VAR *p_lcd_var;
	unsigned long flags;

#if LCD_PERF
set_nowtime(SEQ_IMC);
#endif /* LCD_PERF */

	printk_dbg((_DEBUG_LCD & 0x01), "<start>\n");

	if (list_empty(&list_lcd_var)) return;

	spin_lock_irqsave(&lcd_dev.lcd_lock, flags);

	p_lcd_var = list_entry(list_lcd_var.next, struct mp200_lcd_var, list);
	list_del(&p_lcd_var->list);

	p_lcd_var->lcd_refresh_flg = IMC_R_DISPLAYED;

	/* when kick mix_image by V4L2 */
	if(p_lcd_var->iCallbackV4L2Flg == CALLBACK_V4L2_ON)
	{
		p_lcd_var->iCallbackV4L2Flg = CALLBACK_V4L2_OFF;
		/* set frame date to V4L2 */
		frame_data.dev = p_lcd_var->pvDevData;
		frame_data.buf = p_lcd_var->pvVideoBuf;

		printk_dbg((_DEBUG_LCD & 0x02), "-> v4l2\n");
		/*call v4l2 callback function */
		mp200_v4l2_lcd_callback(frame_data);
	}

#if defined(CONFIG_PM) || defined(CONFIG_DPM)
	p_lcd_var->ctrl_func = 0;
#endif /* CONFIG_PM || CONFIG_DPM */

	/* release semafore */
	up(&p_lcd_var->sem_image_flag);
	printk_dbg((_DEBUG_LCD & 0x80), "up(B) lcd_irq\n");

	spin_unlock_irqrestore(&lcd_dev.lcd_lock, flags);

	printk_dbg((_DEBUG_LCD & 0x02), "<end>\n");
}


/*******************************************************************************
* MODULE   : lcd_callback_imc_refresh
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.09 M.Haguro
* UPDATE   :
********************************************************************************/
void lcd_callback_imc_refresh(void)
{
	FRAME_DATA    frame_data;
	unsigned long flags;
	struct list_head *list_tmp;
	MP200_LCD_VAR *p_lcd_var;
	int find = 0;

	spin_lock_irqsave(&lcd_dev.lcd_lock, flags);
	printk_dbg((_DEBUG_LCD & 0x01), "<start>\n");

	list_for_each(list_tmp, &list_lcd_var)
	{
		p_lcd_var = list_entry(list_tmp, struct mp200_lcd_var, list);
		if (p_lcd_var->lcd_refresh_flg == IMC_R_WAIT_REFRESH)
		{
			find = 1;
			break;
		}
	}

	if (find)
	{
		/* release refresh semafore */
		up(&lcd_dev.sem_image_refresh);
		printk_dbg((_DEBUG_LCD & 0x80), "up(C) lcd_callback_imc_refresh\n");

		/* call FB Driver function "mp200_fb_callback" */
		mp200_fb_callback((int)p_lcd_var->uiMixFrameBufferPage, p_lcd_var->iMixDSPFlgToFB, EM1_FB_DEVICE_LCD);

		/* when kick mix_image by V4L2 */
		if(p_lcd_var->iCallbackV4L2Flg == CALLBACK_V4L2_ON)
		{
			/* set frame date to V4L2 */
			frame_data.dev = p_lcd_var->pvDevData;
			frame_data.buf = p_lcd_var->pvVideoBuf;

			printk_dbg((_DEBUG_LCD & 0x02), "-> v4l2\n");
			/* call v4l2 callback function */
			mp200_v4l2_lcd_refresh_callback(frame_data);
		}

		p_lcd_var->lcd_refresh_flg = IMC_R_REFRESHED;

		lcd_irq_handler_callback();
	}

	printk_dbg((_DEBUG_LCD & 0x02), "<end>\n");
	spin_unlock_irqrestore(&lcd_dev.lcd_lock, flags);
}


/*******************************************************************************
* MODULE   : lcd_callback_imc_wb
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.09 M.Haguro
* UPDATE   :
********************************************************************************/
void lcd_callback_imc_wb(int status)
{
	;
}


/********************************************************
 *                                                      *
 *  Init Function Definitions                           *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : mp200_lcd_init_module
* FUNCTION : initialize LCD Driver
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
int __init mp200_lcd_init_module(void)
{
	int iRet;

	/* lcd device memset */
	memset(&lcd_dev, 0, sizeof(MP200_LCD_DEV));

	/* semafore initialize */
	sema_init(&lcd_dev.sem_image_data, 1);
	sema_init(&lcd_dev.sem_image_refresh, 1);

	/* list head initialize */
	INIT_LIST_HEAD(&list_lcd_var);

	/* spin lock initialize */
	spin_lock_init(&lcd_dev.lcd_lock);

	/* call lcd_probe */
	iRet = lcd_probe();
	if(iRet)
	{
		printk_err("lcd probe error\n");
		lcd_probe_error(iRet);
		return iRet;
	}

	/* variable initialized */
	set_val_init();

	/* timer initiarize */
	lcd_timer_init();

	return 0;
}

/******************************************************************************
* MODULE   : lcd_probe
* FUNCTION : LCD Driver probe
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
static int lcd_probe()
{
	/* reserve LCDC Mmio region */
#ifdef IO_ADDRESS
	LCDCMmioV = (char *)IO_ADDRESS(LCDCMmio);
#else
	if (!request_mem_region(LCDCMmio, LCDCMmioLength, DEV_NAME))
	{
		printk_wrn("cannnot request LCDC mem region\n");
		return LCD_INIT_ERROR__LCDC_MMIO_NOT_RESERVE;
	}
	LCDCMmioV = ioremap_nocache(LCDCMmio, LCDCMmioLength);
	if (!LCDCMmioV)
	{
		printk_wrn("cannot ioremap_nocache LCDC mmio\n");
		return LCD_INIT_ERROR__LCDC_MMIO_NOT_REMAP;
	}
#endif
	printk_dbg((_DEBUG_LCD), "LCDCMmio(0x%08lx)  LCDCMmioV(0x%p)  LCDCMmioLength(0x%lx)\n", LCDCMmio, LCDCMmioV, LCDCMmioLength);

	/* initiarize work queue */
	mp200_lcd_workqueue = create_singlethread_workqueue(DEV_NAME);
	INIT_WORK(&wk_timeout_bottom_half, lcd_timeout_bottom_half_do);

	return 0;
}


/******************************************************************************
* MODULE   : lcd_probe_error
* FUNCTION : When fail in lcd_probe, remove lcd resource
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
static void lcd_probe_error(int iErrorMode)
{
	switch(iErrorMode)
	{
		case LCD_INIT_ERROR__LCDM_START:		/* FALL THROUGH */
		case LCD_INIT_ERROR__LCD_IRQ_NOT_REQUEST:	/* FALL THROUGH */
		case LCD_INIT_ERROR__IMC_INIT:			/* FALL THROUGH */
		case LCD_INIT_ERROR__LCDM_INIT:			/* FALL THROUGH */
			exit_lcdhw();
			free_irq(INT_LCD, NULL);
			/* Reset LCD */
			lcd_hw_reset();
#ifndef IO_ADDRESS
			iounmap(LCDCMmioV);
		case LCD_INIT_ERROR__LCDC_MMIO_NOT_REMAP:	/* FALL THROUGH */
			release_mem_region(LCDCMmio, LCDCMmioLength);
		case LCD_INIT_ERROR__LCDC_MMIO_NOT_RESERVE:	/* FALL THROUGH */
#endif
		default:
			break;
	}

	return;
}


/******************************************************************************
* MODULE   : set_val_init
* FUNCTION : variable default set
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
static void set_val_init(void)
{
	int i;

	/* other img infomation */
	uiMaskColor_tmp      = 0x0020;
	uiAlpha_tmp          = 0xFF;

	/* flags */
	iMixDSPFlg_tmp       = MIX_DSP_OFF;
	iDSPPauseFlg         = PAUSE_DSP_OFF;
	uiMaskColorFlag_tmp  = FB_MASK_COLOR_DISP_OFF;
	uiAbsolutelyUpFlag   = FB_UPDATE_OFF;

	for (i = 0; i < 2; i++)
	{
		sema_init(&lcd_var[i].sem_image_flag, 1);

		/* input image data from fb driver */
		lcd_var[i].uiMixFrameBufferPage = DISPLAY_FRAME_NO_A;

		/* flags */
		lcd_var[i].iMixDSPFlgToFB       = MIX_DSP_OFF;
		lcd_var[i].iCallbackV4L2Flg     = CALLBACK_V4L2_OFF;

		/* frame data */
		lcd_var[i].pvDevData            = 0x00;
		lcd_var[i].pvVideoBuf           = 0x00;

		lcd_var[i].lcd_refresh_flg = IMC_R_IDLE;
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
		lcd_var[i].ctrl_func       = 0;
#endif /* CONFIG_PM || CONFIG_DPM */
	}

	return;
}


/********************************************************
 *  Timer Function Definitions                          *
 *******************************************************/
/******************************************************************************
* MODULE   : lcd_timer_init
* FUNCTION : timer function init
* RETURN   : none
* NOTE     : private function
* CREATE   : 2006.06.07 H.Nagashima
* UPDATE   :
******************************************************************************/
static void lcd_timer_init(void)
{
	init_timer(&lcd_timer);
	lcd_timer.function = lcd_timeout;
	lcd_timer.data = 0;
	iTimerFlag = LCD_TIMER_STOP;

	return;
}


/******************************************************************************
* MODULE   : lcd_timeout
* FUNCTION : when call mp200_fb_call_timer
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.06.07 H.Nagashimia
* UPDATE   :
******************************************************************************/
static void lcd_timeout(unsigned long data)
{
	/* change LCD status to PAUSE */
	iDSPPauseFlg = PAUSE_DSP_ON;

	queue_work(mp200_lcd_workqueue, &wk_timeout_bottom_half);
	return;
}


/******************************************************************************
* MODULE   : lcd_timeout_bottom_half_do
* FUNCTION : when call mp200_fb_call_timer
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.07.21 M.Haguro
* UPDATE   :
******************************************************************************/
static void lcd_timeout_bottom_half_do(struct work_struct *num)
{
	/* get semafore */
#if 0
//@	if(down_trylock(&lcd_dev.sem_image_data))
//@	{
//@		printk_dbg((_DEBUG_LCD & 0x80), "down_trylock(A) FAILED!! lcd_timeout\n");
//@		return ;
//@	}
//@	printk_dbg((_DEBUG_LCD & 0x80), "down_trylock(A) lcd_timeout\n");
#else
	down(&lcd_dev.sem_image_data);
	printk_dbg((_DEBUG_LCD & 0x80), "down(A) lcd_timeout\n");
#endif

	/* mix image */
	mix_image(CALLBACK_V4L2_OFF, UPDATE_ON, SET_LAYER_2D);

	return;
}


/********************************************************
 *  Exit Function Definitions                           *
 *******************************************************/
/******************************************************************************
* MODULE   : mp200_lcd_exit_module
* FUNCTION : cleanup LCD module
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
void mp200_lcd_exit_module()
{
#ifndef CONFIG_MACH_PGPF
	lcd_hw_stop();
#endif /* !CONFIG_MACH_PGPF */
	mp200_lcd_remove();
}


/******************************************************************************
* MODULE   : mp200_lcd_remove
* FUNCTION : release LCD resource
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.29 H.Nagashima
* UPDATE   :
******************************************************************************/
static void mp200_lcd_remove()
{
	/* power off LCD */
#ifdef CONFIG_MACH_PGPF
	pgpf_lcdhw_suspend(0);
#else /* CONFIG_MACH_PGPF */
	lcd_hw_backlight_off();
	lcd_hw_power_off();
#endif /* CONFIG_MACH_PGPF */

	free_irq(INT_LCD, NULL);
	iounmap(LCDCMmioV);
	release_mem_region(LCDCMmio, LCDCMmioLength);

	/* Reset LCD */
	lcd_hw_reset();

	return;
}


#if defined(CONFIG_PM) || defined(CONFIG_DPM)
/********************************************************
 *  Suspend/Resume Function Definitions                 *
 *******************************************************/
/******************************************************************************
* MODULE   : mp200_lcd_suspend
* FUNCTION : suspend LCD driver
* RETURN   : 0 : success
* NOTE     : none
* CREATE   : 2006.08.15 H.Nagashima
* UPDATE   :
******************************************************************************/
int mp200_lcd_suspend(struct platform_device *dev, pm_message_t state)
{
	unsigned long ulRegVal32, ulBUSSEL_save;

	if ((state.event == DEV_SUSPEND_IDLE_1) &&
	    ((lcd_var[0].ctrl_func | lcd_var[1].ctrl_func) != 0))
	{
		return -EBUSY;
	}

	ulRegVal32 = lcd_hw_chk_bussel();
	ulBUSSEL_save = ulRegVal32;
	switch (ulRegVal32 & LCD_BUSSEL_BIT){
	case LCD_BUSSEL_BACKCOLOR:	/* FALL THROUGH */
	case LCD_BUSSEL_BLACK:
		break;
	case LCD_BUSSEL_WB_LOCAL:	/* FALL THROUGH */
	case LCD_BUSSEL_LOCAL:		/* FALL THROUGH */
	default:
		writel(LCD_BUSSEL_BLACK, LCDCMmioV + LCD_BUSSEL);
		break;
	}

	if (state.event == DEV_SUSPEND_IDLE_1)
	{
		ulRegVal32 = lcd_hw_chk_status();
		switch (ulRegVal32 & LCD_MODSTATUS_BIT){
		case LCD_MODSTATUS_BACKCOLOR:	/* FALL THROUGH */
		case LCD_MODSTATUS_BLACK:
			return 0;
		case LCD_MODSTATUS_WB_LOCAL:	/* FALL THROUGH */
		case LCD_MODSTATUS_LOCAL:	/* FALL THROUGH */
		default:
			return -EBUSY;
		}
	}
	else /* state.event == PM_EVENT_SUSPEND */
	{
		if (mp200_sleep_while_idle)
		{
			IMC_reset_flg = 1;	/* Restore IMC H/W register data after resume */
			return 0;
		}
		else
		{
			DPM_suspend_flg = 1;
			/* LCD timer stop */
			del_timer(&lcd_timer);
			/* LCD status chesk */
#if 1
			lcd_suspend_chk_lcdout();
#else	
			if(lcd_suspend_chk_lcdout()){
				/* LCD timer start */
				if((iTimerFlag == LCD_TIMER_START) && (iMixDSPFlg_tmp == MIX_DSP_ON)){
					lcd_timer.expires = jiffies + (MIX_DSP_TIMEOUT * 10 * HZ / 1000);
					add_timer(&lcd_timer);
				}
				writel(ulBUSSEL_save,  LCDCMmioV + LCD_BUSSEL);
				DPM_suspend_flg = 0;
				return -EBUSY;
			}
#endif
#ifdef CONFIG_MACH_PGPF
			pgpf_lcdhw_suspend(1);
#else /* CONFIG_MACH_PGPF */
			/* BackLight OFF  */
			lcd_hw_backlight_off();
			/* LCD POWER OFF */
			lcd_hw_power_standby();
#endif /* CONFIG_MACH_PGPF */
			/* Reset LCD */
			lcd_hw_reset();

			IMC_reset_flg = 1;	/* Restore IMC H/W register data after resume */
			return 0;
		}
	}
}


/******************************************************************************
* MODULE   : lcd_suspend_chk_lcdout
* FUNCTION : enabled suspend
* RETURN   :  0 : success
*            -1 : failed
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
int lcd_suspend_chk_lcdout(void)
{
	int i = 0;
	int iRet = -1;
	int wait_max  = 32; /* ms */
	int wait_time = 2;  /* ms */
	int wait_cnt  = wait_max / wait_time;
	unsigned long ulRegVal32;

	do{
		ulRegVal32 = lcd_hw_chk_status();
		switch (ulRegVal32 & LCD_MODSTATUS_BIT){
		case LCD_MODSTATUS_BACKCOLOR:	/* FALL THROUGH */
		case LCD_MODSTATUS_BLACK:	/* FALL THROUGH */
			i    = wait_cnt;
			iRet = 0;
			break;
		case LCD_MODSTATUS_WB_LOCAL:	/* FALL THROUGH */
		case LCD_MODSTATUS_LOCAL:
		default:
			if(i == wait_cnt){
#if 1
				printk_dbg((_DEBUG_LCD & 0x01), "failed(%08lx)\n", ulRegVal32);
#else
				printk_err("failed(%08lx)\n", ulRegVal32);
#endif
			}
			else{
				mdelay(wait_time);
			}
			break;
		}
		i++;
	}while(i <= wait_cnt);

	return iRet;
}


/******************************************************************************
* MODULE   : mp200_lcd_resume
* FUNCTION : suspend LCD driver
* RETURN   : 0 : success
* NOTE     : none
* CREATE   : 2006.08.15 H.Nagashima
* UPDATE   :
******************************************************************************/
int mp200_lcd_resume(struct platform_device *dev)
{
	struct list_head *list_tmp;
	MP200_LCD_VAR *p_lcd_var;
	void (*callback_func[4])(void) = {NULL, NULL, NULL, NULL};
	int i = 0;

	if(DPM_suspend_flg){
		/* UnReset LCD */
		lcd_hw_unreset();

#ifdef CONFIG_MACH_PGPF
		pgpf_lcdhw_resume(1);
#else /* CONFIG_MACH_PGPF */
		/* LCD POWER ON */
		lcd_hw_power_wakeup();
		/* BackLight ON */
		lcd_hw_backlight_on();
#endif /* CONFIG_MACH_PGPF */

		/* LCD timer start */
		if((iTimerFlag == LCD_TIMER_START) && (iMixDSPFlg_tmp == MIX_DSP_ON)){
			lcd_timer.expires = jiffies + (MIX_DSP_TIMEOUT * 10 * HZ / 1000);
			add_timer(&lcd_timer);
		}

		list_for_each(list_tmp, &list_lcd_var){
			p_lcd_var = list_entry(list_tmp, struct mp200_lcd_var, list);
			switch (p_lcd_var->ctrl_func){
			case 2:
				callback_func[i] = lcd_callback_imc_refresh;
				i++;
				/* FALL THROUGH */
			case 0:
			default:
				break;
			}
		}

		for (i = 0; i < 4; i++){
			if (callback_func[i] == NULL) break;
			else                          callback_func[i]();
		}

		DPM_suspend_flg = 0;
	}
	return 0;
}
#endif /* CONFIG_PM || CONFIG_DPM */


MODULE_AUTHOR("NEC Electronics Corporation.");
MODULE_DESCRIPTION("EM1 LCD control driver");
MODULE_LICENSE("GPL");


/* module alias */
#ifdef MODULE
module_init(mp200_lcd_init_module);
module_exit(mp200_lcd_exit_module);
#else
#ifndef CONFIG_FB_MP200
__initcall(mp200_lcd_init_module);
#endif
#endif

/* EXPORT_SYMBOL */
EXPORT_SYMBOL(mp200_lcd_set_v4l2_image);
#if defined(CONFIG_PGPF_FB_PND_EXT) || defined(CONFIG_PGPF_FB_UX_EXT)
EXPORT_SYMBOL(mp200_lcd_set_fb_image_ex2);
#else /* CONFIG_PGPF_FB_*_EXT */
EXPORT_SYMBOL(mp200_lcd_set_fb_image);
#endif /* CONFIG_PGPF_FB_*_EXT */

