/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/mp200_lcdhw.c
 * Function        : LCD Driver corresponding to V4L2 (H/W Control)
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


/********************************************************
 *  Definitions                                         *
 *******************************************************/
#define _DEBUG_LCDHW  0x00 /* 00008421(bit) */
                           /* 0x01: debug function in
                            * 0x02: debug function out
                            * 0x04: debug IMC
                            * 0x08: debug frame change
                            */


/********************************************************
 *  Definitions                                         *
 *******************************************************/
#define DEV_NAME "mp200_lcdhw"


/********************************************************
 *  Macros                                              *
 *******************************************************/
#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_LCDHW
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif


/********************************************************
 *  Include Files                                       *
 *******************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/platform_device.h>

#include <asm/io.h>
#include <asm/delay.h>
#include <asm/arch/irqs.h>
#include <asm/arch/spi.h>
#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/arch/em1_mem.h>

#include <asm/arch/gpio.h>

#include "lcdc.h"
#include "imc.h"
#include "mp200_common.h"
#include "mp200_lcd_common.h"
#include "mp200_lcd.h"
#include "mp200_imc.h"


/********************************************************
 *  IMC request structure                               *
 *******************************************************/
struct mp200_imc_param
{
	unsigned long			imc_control;
	struct mp200_imc_preset		imc_preset;
	struct mp200_imc_update_vsync	imc_vsync;
	struct mp200_imc_update_reserve	imc_reserve;

	struct imc_gamma_param		param_gamma;
	struct imc_wb_param		param_wb;
	struct imc_yuv_param		param_yuv;
	struct imc_keycolor_param	param_keycolor;
	struct imc_bytelane_param	param_bytelane;
	struct l01_param		param_l0;
	struct l01_param		param_l1a;
	struct l01_param		param_l1b;
	struct l01_param		param_l1c;
	struct l2_param			param_l2a;
	struct l2_param			param_l2b;
	struct bg_param			param_bg;
#ifdef CONFIG_PGPF_FB_PND_EXT
	unsigned long imc_argb4444_mode;
#endif /* CONFIG_PGPF_FB_PND_EXT */

	imc_callback_func_refresh	callback_refresh;
	imc_callback_func_wb		callback_wb;
};

static struct mp200_imc_param          *imc_param   = NULL;
static struct mp200_imc_preset         *imc_preset  = NULL;
static struct mp200_imc_update_vsync   *imc_vsync   = NULL;
static struct mp200_imc_update_reserve *imc_reserve = NULL;

       struct l01_param			fb_layer;
#ifdef CONFIG_PGPF_FB_PND_EXT
       struct l01_param         console_layer;
       struct bg_param          bg_layer;
#endif /* CONFIG_PGPF_FB_PND_EXT */
#ifdef CONFIG_PGPF_FB_LAYER2
       struct l2_param          l2_layer;
#endif /* CONFIG_PGPF_FB_LAYER2 */
       struct l2_param			v4l2_layer;
       struct mp200_imc_update_vsync	ImcNxtVsync;
       struct imc_keycolor_param	ImcNxtKeycol;
       struct imc_bytelane_param	ImcNxtBytelane;


/********************************************************
 * IMC register initialize                              *
 *******************************************************/
#define IMC_CPUBUFSEL_INIT		0x00000000
#define IMC_GAMMA_ADR_INIT		0x00000000
#define IMC_GAMMA_DATA_INIT		0x00000000

#define IMC_L0_KEYCOLOR_INIT		0x00000000
#define IMC_L1x_KEYCOLOR_INIT		0x00000000
#define IMC_L2A_BYTELANE_INIT		0x0000E4E4
#define IMC_L2B_BYTELANE_INIT		0x0000E4E4

#define IMC_YGAIN_OFFSET_INIT		0x00000080
#define IMC_UGAIN_OFFSET_INIT		0x00000080
#define IMC_VGAIN_OFFSET_INIT		0x00000080

#define IMC_COEF_R0_INIT		0x00000000
#define IMC_COEF_R1_INIT		0x00000000
#define IMC_COEF_R2_INIT		0x00000000
#define IMC_COEF_R3_INIT		0x00000000
#define IMC_COEF_G0_INIT		0x00000000
#define IMC_COEF_G1_INIT		0x00000000
#define IMC_COEF_G2_INIT		0x00000000
#define IMC_COEF_G3_INIT		0x00000000
#define IMC_COEF_B0_INIT		0x00000000
#define IMC_COEF_B1_INIT		0x00000000
#define IMC_COEF_B2_INIT		0x00000000
#define IMC_COEF_B3_INIT		0x00000000

#define IMC_L0_CONTROL_INIT		0x00000000
#define IMC_L0_FORMAT_INIT		0x00000000
#define IMC_L0_KEYENABLE_INIT		0x00000000
#define IMC_L0_ALPHA_INIT		0x00000000
#define IMC_L0_RESIZE_INIT		0x00000000
#define IMC_L0_OFFSET_INIT		0x00000000
#define IMC_L0_FRAMEADR_INIT		0x00000000
#define IMC_L0_POSITION_INIT		0x00000000
#define IMC_L0_SIZE_INIT		0x00000000

#define IMC_L1A_CONTROL_INIT		0x00000000
#define IMC_L1A_FORMAT_INIT		0x00000000
#define IMC_L1A_KEYENABLE_INIT		0x00000000
#define IMC_L1A_ALPHA_INIT		0x00000000
#define IMC_L1A_RESIZE_INIT		0x00000000
#define IMC_L1A_OFFSET_INIT		0x00000000
#define IMC_L1A_FRAMEADR_INIT		0x00000000
#define IMC_L1A_POSITION_INIT		0x00000000
#define IMC_L1A_SIZE_INIT		0x00000000

#define IMC_L1B_CONTROL_INIT		0x00000000
#define IMC_L1B_KEYENABLE_INIT		0x00000000
#define IMC_L1B_ALPHA_INIT		0x00000000
#define IMC_L1B_FRAMEADR_INIT		0x00000000
#define IMC_L1B_POSITION_INIT		0x00000000
#define IMC_L1B_SIZE_INIT		0x00000000

#define IMC_L1C_CONTROL_INIT		0x00000000
#define IMC_L1C_KEYENABLE_INIT		0x00000000
#define IMC_L1C_ALPHA_INIT		0x00000000
#define IMC_L1C_FRAMEADR_INIT		0x00000000
#define IMC_L1C_POSITION_INIT		0x00000000
#define IMC_L1C_SIZE_INIT		0x00000000

#define IMC_L2A_CONTROL_INIT		0x00000000
#define IMC_L2A_FORMAT_INIT		0x00000000
#define IMC_L2A_BUFSEL_INIT		0x00000000
#define IMC_L2A_RESIZE_INIT		0x00000000
#define IMC_L2A_MIRROR_INIT		0x00000000
#define IMC_L2A_OFFSET_INIT		0x00000000
#define IMC_L2A_FRAMEADR_YP_INIT	0x00000000
#define IMC_L2A_FRAMEADR_UP_INIT	0x00000000
#define IMC_L2A_FRAMEADR_YQ_INIT	0x00000000
#define IMC_L2A_FRAMEADR_UQ_INIT	0x00000000
#define IMC_L2A_POSITION_INIT		0x00000000
#define IMC_L2A_SIZE_INIT		0x00000000

#define IMC_L2B_CONTROL_INIT		0x00000000
#define IMC_L2B_FORMAT_INIT		0x00000000
#define IMC_L2B_BUFSEL_INIT		0x00000000
#define IMC_L2B_RESIZE_INIT		0x00000000
#define IMC_L2B_MIRROR_INIT		0x00000000
#define IMC_L2B_OFFSET_INIT		0x00000000
#define IMC_L2B_FRAMEADR_YP_INIT	0x00000000
#define IMC_L2B_FRAMEADR_UP_INIT	0x00000000
#define IMC_L2B_FRAMEADR_YQ_INIT	0x00000000
#define IMC_L2B_FRAMEADR_UQ_INIT	0x00000000
#define IMC_L2B_POSITION_INIT		0x00000000
#define IMC_L2B_SIZE_INIT		0x00000000

#define IMC_BG_FORMAT_INIT		0x00000000
#define IMC_BG_RESIZE_INIT		0x00000000
#define IMC_BG_OFFSET_INIT		0x00000000
#define IMC_BG_FRAMEADR_INIT		0x00000000


/********************************************************
 * LCD Module timing                                    *
 *******************************************************/
#define LCD_HAREA_VAL		FRONT_WIDTH
#define LCD_HPULSE		1
#define LCD_HFRONTP		6
#define LCD_HBACKP		4
#define LCD_HBLANK_VAL		(LCD_HFRONTP + LCD_HPULSE + LCD_HBACKP)
#define LCD_VAREA_VAL		FRONT_HEIGHT
#define LCD_VPULSE		1
#define LCD_VFRONTP		3
#define LCD_VBACKP		6
#define LCD_VBLANK_VAL		(LCD_VFRONTP + LCD_VPULSE + LCD_VBACKP)


/********************************************************
 * LCDC register initialize                             *
 *******************************************************/
#if defined(CONFIG_MP200_LCD_RGB666)
#define LCD_CONTROL_INIT	LCD_OFORMAT_RGB666			/* Control for LCDC      : RGB666 output */
#define LCD_IFORMAT_INIT	LCD_IFORMAT_RGB666			/* Input Format select   : RGB666 input  */
#else
#define LCD_CONTROL_INIT	LCD_OFORMAT_RGB565			/* Control for LCDC      : RGB565 output */
#define LCD_IFORMAT_INIT	LCD_IFORMAT_RGB565			/* Input Format select   : RGB565 input  */
#endif /* CONFIG_MP200_LCD_RGB666 */

#define LCD_HTOTAL_INIT		(LCD_HAREA_VAL + LCD_HBLANK_VAL)	/* Horizontal Synchronizing Signal Total Width : 811 */
#define LCD_HAREA_INIT		(LCD_HAREA_VAL)				/* Horizontal Display Area Width               : 800 */
#define LCD_HEDGE1_INIT		(LCD_HFRONTP)				/* Horizontal Synchronizing Signal Edge1       :   6 */
#define LCD_HEDGE2_INIT		(LCD_HFRONTP + LCD_HPULSE)		/* Horizontal Synchronizing Signal Edge2       :   7 */
#define LCD_VTOTAL_INIT		(LCD_VAREA_VAL + LCD_VBLANK_VAL)	/* Vertical Synchronizing Signal Total Width   : 488 */
#define LCD_VAREA_INIT		(LCD_VAREA_VAL)				/* Vertical Display Area Width                 : 480 */
#define LCD_VEDGE1_INIT		(LCD_VFRONTP)				/* Vertical Synchronizing Signal Edge1         :   3 */
#define LCD_VEDGE2_INIT		(LCD_VFRONTP + LCD_VPULSE)		/* Vertical Synchronizing Signal Edge2         :   4 */

#define LCD_QOS_INIT		0x00000180
#define LCD_BUSSEL_UPDATE	LCD_BUSSEL_LOCAL			/* Access Bus Select             : Local Bus         */
#define LCD_BUSSEL_INIT		LCD_BUSSEL_BLACK			/* Access Bus Select             : Black Only        */
#define LCD_AREAADR_INIT	0x00					/* AREA Address for Frame Buffer :                   */
#define LCD_BACKCOL_INIT	(0x00 << LCD_BGRED_SFT | 0x00 << LCD_BGGREEN_SFT | 0x00 << LCD_BGBLUE_SFT)


/********************************************************
 * LCD Module initialize command                        *
 *******************************************************/
#define LCDM_SPI_CMD_POWERON	81
#define LCDM_SPI_CMD_START	1
#define LCDM_SPI_CMD_POWEROFF	4
#define LCDM_SPI_CMD_STANDBY	1
#define LCDM_SPI_CMD_WAKEUP	1

const unsigned int lcdm_spi_cmd_poweron[LCDM_SPI_CMD_POWERON][2] = {
	{0x03, 0x01}, {0x00, 0x00}, {0x01, 0x01}, {0x04, 0x00}, {0x05, 0x14},
	{0x06, 0x24}, {0x10, 0xd7}, {0x11, 0x00}, {0x12, 0x00}, {0x13, 0x55},
	{0x14, 0x01}, {0x15, 0x70}, {0x16, 0x1e}, {0x17, 0x25}, {0x18, 0x25},
	{0x19, 0x02}, {0x1a, 0x02}, {0x1b, 0xa0}, {0x20, 0x2f}, {0x21, 0x0f},
	{0x22, 0x0f}, {0x23, 0x0f}, {0x24, 0x0f}, {0x25, 0x0f}, {0x26, 0x0f},
	{0x27, 0x00}, {0x28, 0x02}, {0x29, 0x02}, {0x2a, 0x02}, {0x2b, 0x0f},
	{0x2c, 0x0f}, {0x2d, 0x0f}, {0x2e, 0x0f}, {0x2f, 0x0f}, {0x30, 0x0f},
	{0x31, 0x0f}, {0x32, 0x00}, {0x33, 0x02}, {0x34, 0x02}, {0x35, 0x02},
	{0x50, 0x0c}, {0x53, 0x42}, {0x54, 0x42}, {0x55, 0x41}, {0x56, 0x14},
	{0x59, 0x88}, {0x5a, 0x01}, {0x5b, 0x00}, {0x5c, 0x02}, {0x5d, 0x0c},
	{0x5e, 0x1c}, {0x5f, 0x27}, {0x62, 0x49}, {0x63, 0x27}, {0x66, 0x76},
	{0x67, 0x27}, {0x70, 0x01}, {0x71, 0x0e}, {0x72, 0x02}, {0x73, 0x0c},
	{0x76, 0x0c}, {0x79, 0x30}, {0x82, 0x00}, {0x83, 0x00}, {0x84, 0xfc},
	{0x86, 0x00}, {0x88, 0x00}, {0x8a, 0x00}, {0x8b, 0x00}, {0x8c, 0x00},
	{0x8d, 0xfc}, {0x8f, 0x00}, {0x91, 0x00}, {0x93, 0x00}, {0x94, 0x00},
	{0x95, 0x00}, {0x96, 0xfc}, {0x98, 0x00}, {0x9a, 0x00}, {0x9c, 0x00},
	{0x9d, 0x00}
};

const unsigned int lcdm_spi_cmd_start[LCDM_SPI_CMD_START][2] = {
	{0x02, 0x00}
};

const unsigned int lcdm_spi_cmd_poweroff[LCDM_SPI_CMD_POWEROFF][2] = {
	{0x10, 0x05}, {0x10, 0x01}, {0x10, 0x00}, {0x03, 0x01}
};

#if 0
const unsigned int lcdm_spi_cmd_standby[LCDM_SPI_CMD_STANDBY][2] = {
	{0x02, 0x01}
};

const unsigned int lcdm_spi_cmd_wakeup[LCDM_SPI_CMD_WAKEUP][2] = {
	{0x02, 0x00}
};
#endif


/********************************************************
 * LCDC register initialize                             *
 *******************************************************/
#define CHG_PINSEL_G48_LCD_BIT  0xFFFFFFF0
#define CHG_PINSEL_G48_LCD_INIT 0x55555550
#define CHG_PINSEL_G64_LCD_BIT  0x0000FFFF
#define CHG_PINSEL_G64_LCD_INIT 0x00005555


/********************************************************
 *  Variables                                           *
 *******************************************************/
       int init_is_first   = 1;


/********************************************************
 *  Prototype declarations of function                  *
 *******************************************************/
/*
 * LCD Driver (H/W)
 */

/* ------------------ LCD control function -------------------------------------- */
       int           change_frame(void);
       int           init_lcdhw(void);
       void          exit_lcdhw(void);

/* ------------------ LCD H/W control function ---------------------------------- */
static void          lcd_hw_save_reg(void);
static void          lcd_hw_restore_reg(void);
#ifdef CONFIG_MACH_PGPF
void   lcd_hw_start(void);
#else /* CONFIG_MACH_PGPF */
static inline void   lcd_hw_start(void);
#endif /* CONFIG_MACH_PGPF */
       void          lcd_hw_stop(void);
       void          lcd_hw_reset(void);
       void          lcd_hw_unreset(void);
       void          lcd_hw_power_off(void);
       void          lcd_hw_power_on(void);
       void          lcd_hw_power_standby(void);
       void          lcd_hw_power_wakeup(void);
       void          lcd_hw_backlight_off(void);
       void          lcd_hw_backlight_on(void);

/* ------------------ LCDC rgister check function ------------------------------ */
       unsigned long lcd_hw_chk_status(void);
       unsigned long lcd_hw_chk_bussel(void);
       unsigned long lcd_hw_chk_int_status(void);
       unsigned long lcd_hw_chk_int_rawstatus(void);

/* ------------------ LCDC interruput control function -------------------------- */
       void          lcd_hw_int_enable(void);
       void          lcd_hw_int_disable(void);
       void          lcd_hw_int_factor_clr(void);

/* ------------------ LCD initialize function ----------------------------------- */
static void          lcd_hw_init(void);
#ifdef CONFIG_MACH_PGPF
       void          lcd_controller_hw_init(void);
#else /* CONFIG_MACH_PGPF */
static void          lcd_controller_hw_init(void);
#endif /* CONFIG_MACH_PGPF */
static inline void   lcd_controller_hw_init_lcdsize(void);

/* ------------------ LCD module initialize function ---------------------------- */
static int           lcd_module_hw_init(void);
static void          lcd_module_hw_unreset(void);
static int           lcd_module_hw_power_on(void);
static int           lcd_module_hw_power_off(void);
static int           lcd_module_hw_start(void);
#if 0
static int           lcd_module_hw_standby(void);
static int           lcd_module_hw_wakeup(void);
#endif
static int           lcd_module_hw_spiw(unsigned char cmd, unsigned char param);

/* ------------------ IMC initialize function ----------------------------------- */
#ifdef CONFIG_MACH_PGPF
       int           lcd_hw_imc_hw_init(void);
#else /* CONFIG_MACH_PGPF */
static int           imc_hw_init(void);
#endif /* CONFIG_MACH_PGPF */
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
static int           imc_hw_restore_reg(void);
#endif /* CONFIG_PM || CONFIG_DPM */

static void         *imc_hw_init_preset_gamma(struct imc_gamma_param *param_gamma);
static void         *imc_hw_init_preset_wb(struct imc_wb_param *param_wb);
static void         *imc_hw_init_preset_yuv(struct imc_yuv_param *param_yuv);
static void         *imc_hw_init_preset_keycolor(struct imc_keycolor_param *param_keycolor);
static void         *imc_hw_init_preset_bytelane(struct imc_bytelane_param *param_bytelane);
static void         *imc_hw_init_reserve_l0(struct l01_param *param_l0);
static void         *imc_hw_init_reserve_l1a(struct l01_param *param_l1a);
static void         *imc_hw_init_reserve_l1b(struct l01_param *param_l1b);
static void         *imc_hw_init_reserve_l1c(struct l01_param *param_l1c);
static void         *imc_hw_init_reserve_l2a(struct l2_param *param_l2a);
static void         *imc_hw_init_reserve_l2b(struct l2_param *param_l2b);
static void         *imc_hw_init_reserve_bg(struct bg_param *param_bg);

/* ------------------ IMC control function -------------------------------------- */
       int           imc_hw_set_update_vsync(struct mp200_imc_update_vsync *vsync);
       int           imc_hw_set_update_keycolor(struct imc_keycolor_param *keycolor);
       int           imc_hw_set_update_bytelane(struct imc_bytelane_param *bytelane);
       int           imc_hw_set_update_reserve(int type, int mixdsp, int update);




/* ------------------ inline function ------------------------------------------- */
inline void chk_errno(int errno, char **cnum)
{
	switch(errno){
	case ENODEV: *cnum = "ENODEV"; break;
	case EINVAL: *cnum = "EINVAL"; break;
	case EPERM:  *cnum = "EPERM";  break;
	case EIO:    *cnum = "EIO";    break;
	case EAGAIN: *cnum = "EAGAIN"; break;
	default:     *cnum = "****";   break;
	}
}


/* ------------------ LCD control function -------------------------------------- */
/******************************************************************************
* MODULE   : change_frame
* FUNCTION : change LCD display page
* RETURN   : 0 : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
int change_frame(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	writel(LCD_BUSSEL_UPDATE,  LCDCMmioV + LCD_BUSSEL);

	return 0;
}


/******************************************************************************
* MODULE   : init_lcdhw
* FUNCTION : LCD(H/W) initialized
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
int init_lcdhw(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* LCD Module initialize */
	if(lcd_module_hw_init()){
		printk_err("fail in LCD module initialize\n");
		return LCD_INIT_ERROR__LCDM_INIT;
	}

	/* LCDC initialize */
	lcd_hw_init();

	/* LCDC register initialize */
	lcd_controller_hw_init();

	/* lcd display size set */
	lcd_controller_hw_init_lcdsize();

	/* disable LCDC interrupts */
	lcd_hw_int_factor_clr();
	lcd_hw_int_disable();

	/* IMC initialize */
#ifdef CONFIG_MACH_PGPF
	if (lcd_hw_imc_hw_init()) {
#else /* CONFIG_MACH_PGPF */
	if(imc_hw_init()){
#endif /* CONFIG_MACH_PGPF */
		printk_err("fail in IMC initialize\n");
		return LCD_INIT_ERROR__IMC_INIT;
	}

	/* LCDC start */
	lcd_hw_start();

	/* LCD Module start */
	if(lcd_module_hw_start()){
		printk_err("fail in LCD module start\n");
		return LCD_INIT_ERROR__LCDM_START;
	}

	/* Power & BackLight */
	lcd_hw_power_on();
	lcd_hw_backlight_on();

	return 0;
}


/******************************************************************************
* MODULE   : exit_lcdhw
* FUNCTION : LCD(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.05
* UPDATE   :
******************************************************************************/
void exit_lcdhw(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	lcd_hw_backlight_off();
	lcd_hw_power_off();

	if(imc_param){
		kfree(imc_param);
	}
}


/* ------------------ LCD H/W control function ---------------------------------- */
/********************************************************
 *  save/restore Function Definitions                   *
 *******************************************************/
/******************************************************************************
* MODULE   : lcd_hw_save_reg
* FUNCTION : save LCD(H/W) register data
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2006.08.17 H.Nagashima
* UPDATE   :
******************************************************************************/
static void lcd_hw_save_reg(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
}


/******************************************************************************
* MODULE   : lcd_hw_restore_reg
* FUNCTION : restore LCD(H/W) register data
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2006.08.17 H.Nagashima
* UPDATE   :
******************************************************************************/
static void lcd_hw_restore_reg(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* restore LCD framebuffer address */
	lcd_controller_hw_init();

	/* init LCD size */
	lcd_controller_hw_init_lcdsize();
}


/******************************************************************************
* MODULE   : lcd_hw_start
* FUNCTION : start LCD(H/W)
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
#ifdef CONFIG_MACH_PGPF
void   lcd_hw_start(void)
#else /* CONFIG_MACH_PGPF */
static inline void lcd_hw_start(void)
#endif /* CONFIG_MACH_PGPF */
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* enable auto frequency control */
	writel((readl(SMU_DFS_FIFO_REQMASK) & ~(1 << 0)),  SMU_DFS_FIFO_REQMASK); /* unmask LCD (bit0=0) */
	writel((readl(SMU_AUTO_FRQ_MASK0)   |  (1 << 12)), SMU_AUTO_FRQ_MASK0);   /* OFF control of IMG (bit12=1) */

	writel(LCD_LCDOUT_ENABLE, LCDCMmioV + LCD_LCDOUT);
}


/******************************************************************************
* MODULE   : lcd_hw_stop
* FUNCTION : stop LCD(H/W)
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
void lcd_hw_stop(void)
{
	unsigned long ulRegVal32;
	int wait_max = 16;
	int wait_cnt = 0;

	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
	writel(LCD_LCDOUT_DISABLE, LCDCMmioV + LCD_LCDOUT);

	/* wait for LCD disp off (max 16ms) */
	do{
		ulRegVal32 = lcd_hw_chk_int_rawstatus();
		if(ulRegVal32 & LCD_LCDSTOPRAW_BIT){
			printk_dbg((_DEBUG_LCDHW & 0x01), "wait(%dms)\n", wait_cnt);
			wait_cnt = wait_max;
		}
		else{
			if(wait_cnt < wait_max){
				mdelay(1);
			}
		}
		wait_cnt++;
	}while(wait_cnt <= wait_max);

	/* disable auto frequency control */
	writel((readl(SMU_DFS_FIFO_REQMASK) |  (1 << 0)),  SMU_DFS_FIFO_REQMASK); /* mask LCD (bit0=1) */
	writel((readl(SMU_AUTO_FRQ_MASK0)   & ~(1 << 12)), SMU_AUTO_FRQ_MASK0);   /* ON control of IMG (bit12=0) */

}


/******************************************************************************
* MODULE   : lcd_hw_reset
* FUNCTION : LCD(H/W) reset
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.11.29 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_reset(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* Reset LCD */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
	mp200_pmu_reset_device(MP200_RESETDEVICE_LCD);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_LCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CLK);
}


/******************************************************************************
* MODULE   : lcd_hw_unreset
* FUNCTION : LCD(H/W) unreset
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.11.29 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_unreset(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* UnReset LCD */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_LCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_LCD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDPCLK);
}


/******************************************************************************
* MODULE   : lcd_hw_power_off
* FUNCTION : LCD(H/W) power control(off)
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.09 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_power_off(void)
{
	/* LCD Module power off */
	lcd_module_hw_power_off();
}


/******************************************************************************
* MODULE   : lcd_hw_power_on
* FUNCTION : LCD(H/W) power control(on)
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.09 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_power_on(void)
{
	;
}


/******************************************************************************
* MODULE   : lcd_hw_power_standby
* FUNCTION : LCD(H/W) power control(standby)
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_power_standby(void)
{
	/* Save LCD H/W register data */
	lcd_hw_save_reg();

	/* LCD display off */
	lcd_hw_stop();

#if 0
	/* LCD Module Standby */
	lcd_module_hw_standby();
#endif
}


/******************************************************************************
* MODULE   : lcd_hw_power_wakeup
* FUNCTION : LCD(H/W) power control(wakeup)
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_power_wakeup(void)
{
#if defined(CONFIG_PM) || defined(CONFIG_DPM)
	/* Restore IMC H/W register data */
	if (IMC_reset_flg) imc_hw_restore_reg();
#endif /* CONFIG_PM || CONFIG_DPM */

	/* Restore LCD H/W register data */
	lcd_hw_restore_reg();

#if 0
	/* LCD Module Waleup */
	lcd_module_hw_wakeup();
#endif

	/* LCD display on */
	lcd_hw_start();
}


/******************************************************************************
* MODULE   : lcd_hw_backlight_off
* FUNCTION : LCD(H/W) backlight control(off)
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.05 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_backlight_off(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
#ifdef CONFIG_MACH_MP200
	gpio_direction_output(GPIO_LCD_PD, 0x0); /* GIO_P2 output Low */
#endif
}


/******************************************************************************
* MODULE   : lcd_hw_backlight_on
* FUNCTION : LCD(H/W) backlight control(on)
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.07.05 M.Haguro
* UPDATE   :
******************************************************************************/
void lcd_hw_backlight_on(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
#ifdef CONFIG_MACH_MP200
	mdelay(110);
	gpio_direction_output(GPIO_LCD_PD, 0x1); /* GIO_P2 output High */
#endif
}


/* ------------------ LCDC rgister check function ------------------------------ */
/******************************************************************************
* MODULE   : lcd_hw_chk_status
* FUNCTION : check LCDC status
* RETURN   :  0 : success
*            -1 : failed
* NOTE     : none
* CREATE   : 2007.11.19 M.Haguro
* UPDATE   :
******************************************************************************/
unsigned long lcd_hw_chk_status(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
	/* check lcd status */
	return (unsigned long)readl(LCDCMmioV + LCD_STATUS);
}


/*******************************************************************************
* MODULE   : lcd_hw_chk_int_status
* FUNCTION : check LCDC bussel
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
********************************************************************************/
unsigned long lcd_hw_chk_bussel(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
	/*  check bussel */
	return (unsigned long)readl(LCDCMmioV + LCD_BUSSEL);
}


/*******************************************************************************
* MODULE   : lcd_hw_chk_int_status
* FUNCTION : check LCDC interrupt status
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
********************************************************************************/
unsigned long lcd_hw_chk_int_status(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
	/*  check Interuppt Status */
	return (unsigned long)readl(LCDCMmioV + LCD_INTSTATUS);
}


/*******************************************************************************
* MODULE   : lcd_hw_chk_int_rawstatus
* FUNCTION : check LCDC interrupt RAW status
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
********************************************************************************/
unsigned long lcd_hw_chk_int_rawstatus(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
	/*  check Interuppt RAW Status */
	return (unsigned long)readl(LCDCMmioV + LCD_INTRAWSTATUS);
}


/* ------------------ LCDC interruput control function -------------------------- */
/*******************************************************************************
* MODULE   : lcd_hw_int_enable
* FUNCTION : enable LCDC VSYNC interrupt
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.07.14 Y.Utsuhara
* UPDATE   :
********************************************************************************/
void lcd_hw_int_enable(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/*  Interuppt Enable Set */
	writel(LCD_LCDVSEN_BIT, LCDCMmioV + LCD_INTENSET);
}


/*******************************************************************************
* MODULE   : lcd_hw_int_disable
* FUNCTION : disable LCDC interrupts
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.07.14 Y.Utsuhara
* UPDATE   :
********************************************************************************/
void lcd_hw_int_disable(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* Interrupt Enable Clear */
	writel(LCD_INTENCLR_ALL_BIT, LCDCMmioV + LCD_INTENCLR);
}


/*******************************************************************************
* MODULE   : lcd_hw_int_factor_clr
* FUNCTION : disable LCDC interrupts
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.07.14 Y.Utsuhara
* UPDATE   :
********************************************************************************/
void lcd_hw_int_factor_clr(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/* Interrupt Status Clear */
	writel(LCD_INTFFCLR_ALL_BIT, LCDCMmioV + LCD_INTFFCLR);
}


/* ------------------ LCD initialize function ----------------------------------- */
/******************************************************************************
* MODULE   : lcd_hw_init
* FUNCTION : LCD(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.06.25 M.Haguro
* UPDATE   :
******************************************************************************/
static void lcd_hw_init(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/********************************/
	/* port / terminal switching    */
	/********************************/
	writel((readl(CHG_PINSEL_G48) & ~CHG_PINSEL_G48_LCD_BIT) | CHG_PINSEL_G48_LCD_INIT, CHG_PINSEL_G48);
	writel((readl(CHG_PINSEL_G64) & ~CHG_PINSEL_G64_LCD_BIT) | CHG_PINSEL_G64_LCD_INIT, CHG_PINSEL_G64);

	/********************************/
	/* clock supply                 */
	/********************************/
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CCLK);        /* LCD_CCLK       -> ON */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_LCLK);        /* LCD_LCLK       -> ON */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_PCLK);        /* LCD_PCLK       -> ON */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CLK);         /* LCD_CLK        -> ON */

	/********************************/
	/* clock auto control -> OFF    */
	/********************************/
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);

	/********************************/
	/* LCDC unreset                 */
	/********************************/
	mp200_pmu_unreset_device(MP200_RESETDEVICE_LCD);

	/********************************/
	/* clock auto control -> ON     */
	/********************************/
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDPCLK);
}


/******************************************************************************
* MODULE   : lcd_controller_hw_init
* FUNCTION : LCD(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.06.25 M.Haguro
* UPDATE   :
******************************************************************************/
#ifdef CONFIG_MACH_PGPF
void lcd_controller_hw_init(void)
#else /* CONFIG_MACH_PGPF */
static void lcd_controller_hw_init(void)
#endif /* CONFIG_MACH_PGPF */
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
#ifndef CONFIG_MACH_PGPF
	writel(LCD_CONTROL_INIT, LCDCMmioV + LCD_CONTROL);         /* Control for LCDC  */
#endif /* !CONFIG_MACH_PGPF */
	writel(LCD_QOS_INIT,     LCDCMmioV + LCD_QOS);             /* QoS for LCDC      */
	writel(LCD_IFORMAT_INIT, LCDCMmioV + LCD_IFORMAT);         /* Input Format      */
	writel(LCD_BACKCOL_INIT, LCDCMmioV + LCD_BACKCOLOR);       /* Background color  */
	writel(LCD_BUSSEL_INIT,  LCDCMmioV + LCD_BUSSEL);          /* Access Bus Select */
	init_is_first = 0;
}


/******************************************************************************
* MODULE   : lcd_controller_hw_init_lcdsize
* FUNCTION : LCD(H/W) size initialize
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
static inline void lcd_controller_hw_init_lcdsize(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	writel(LCD_HTOTAL_INIT,  LCDCMmioV + LCD_HTOTAL);   /* Horizontal Synchronizing Signal Total Width  */
	writel(LCD_HAREA_INIT,   LCDCMmioV + LCD_HAREA);    /* Horizontal Display Area Width                */
	writel(LCD_HEDGE1_INIT,  LCDCMmioV + LCD_HEDGE1);   /* Horizontal Synchronizing Signal Edge Point 1 */
	writel(LCD_HEDGE2_INIT,  LCDCMmioV + LCD_HEDGE2);   /* Horizontal Synchronizing Signal Edge Point 2 */
	writel(LCD_VTOTAL_INIT,  LCDCMmioV + LCD_VTOTAL);   /* Vertical Synchronizing Signal Total Width    */
	writel(LCD_VAREA_INIT,   LCDCMmioV + LCD_VAREA);    /* Vertical Display Area Width                  */
	writel(LCD_VEDGE1_INIT,  LCDCMmioV + LCD_VEDGE1);   /* Vertical Synchronizing Signal Edge Point 1   */
	writel(LCD_VEDGE2_INIT,  LCDCMmioV + LCD_VEDGE2);   /* Vertical Synchronizing Signal Edge Point 2   */
}


/* ------------------ LCD module initialize function ---------------------------- */
/******************************************************************************
* MODULE   : lcd_module_hw_init
* FUNCTION : LCD(H/W) initialized
* RETURN   :    0 : success
*          : other: failed
* NOTE     : none
* CREATE   : 2007.07.10 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_init(void)
{
	int iRet = 0;
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	/********************************/
	/* LCD Module unreset           */
	/********************************/
	lcd_module_hw_unreset();
	mdelay(1);

	/********************************/
	/* LCD Module Initialize        */
	/********************************/
	iRet = lcd_module_hw_power_on();
	if(!iRet){
		udelay(20);
	}

	return iRet;
}


/*****************************************************************************
* MODULE   : lcd_module_hw_unreset
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2007.06.25 M.Haguro
* UPDATE   :
******************************************************************************/
static void lcd_module_hw_unreset(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");
#ifdef CONFIG_MACH_MP200
	gpio_direction_output(GPIO_LCD_RST, 0x1); /* GIO_P3 output High */
#endif
}


/*****************************************************************************
* MODULE   : lcd_module_hw_power_on
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_power_on(void)
{
	int iRet = 0, i;
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	for(i = 0; i < LCDM_SPI_CMD_POWERON; i++){
		iRet = lcd_module_hw_spiw(lcdm_spi_cmd_poweron[i][0], lcdm_spi_cmd_poweron[i][1]);
		if(iRet < 0){
			printk_err("LCD Module power on failed!\n");
			return iRet;
		}
	}
	return iRet;
}


/*****************************************************************************
* MODULE   : lcd_module_hw_power_off
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_power_off(void)
{
	int iRet = 0, i;
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	for(i = 0; i < LCDM_SPI_CMD_POWEROFF; i++){
		iRet = lcd_module_hw_spiw(lcdm_spi_cmd_poweroff[i][0], lcdm_spi_cmd_poweroff[i][1]);
		if(iRet < 0){
			printk_err("LCD Module power off failed!\n");
			return iRet;
		}
	}
	return iRet;
}


/*****************************************************************************
* MODULE   : lcd_module_hw_start
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2009.3.24 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_start(void)
{
	int iRet = 0, i;
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	for(i = 0; i < LCDM_SPI_CMD_START; i++){
		iRet = lcd_module_hw_spiw(lcdm_spi_cmd_start[i][0], lcdm_spi_cmd_start[i][1]);
		if(iRet < 0){
			printk_err("LCD Module power on failed!\n");
			return iRet;
		}
	}
	return iRet;
}


#if 0
/*****************************************************************************
* MODULE   : lcd_module_hw_standby
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_standby(void)
{
	int iRet = 0, i;
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	for(i = 0; i < LCDM_SPI_CMD_STANDBY; i++){
		iRet = lcd_module_hw_spiw(lcdm_spi_cmd_standby[i][0], lcdm_spi_cmd_standby[i][1]);
		if(iRet < 0){
			printk_err("LCD Module initialize failed!\n");
			return iRet;
		}
	}
	mdelay(40);
	return iRet;
}


/*****************************************************************************
* MODULE   : lcd_module_hw_wakeup
* FUNCTION : 
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_wakeup(void)
{
	int iRet = 0, i;
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	for(i = 0; i < LCDM_SPI_CMD_WAKEUP; i++){
		iRet = lcd_module_hw_spiw(lcdm_spi_cmd_wakeup[i][0], lcdm_spi_cmd_wakeup[i][1]);
		if(iRet < 0){
			printk_err("LCD Module initialize failed!\n");
			return iRet;
		}
	}
	return iRet;
}
#endif


/* ------------------ spi control function -------------------------------------- */
/*****************************************************************************
* MODULE   : lcd_module_hw_spiw
* FUNCTION : 
* RETURN   :    0 : success
*          : other: failed
* NOTE     : none
* CREATE   : 2007.06.25 M.Haguro
* UPDATE   :
******************************************************************************/
static int lcd_module_hw_spiw(unsigned char cmd, unsigned char param)
{
	char *cRet;
	int   iRet;
	SPI_CONFIG spi_conf;
	unsigned int tx_data = (0 << 24) | (cmd << 16) | (1 << 8) | (param << 0);

	spi_conf.dev    = SPI_DEV_SP0;
	spi_conf.nbw    = SPI_NB_32BIT;
	spi_conf.nbr    = 0;
	spi_conf.cs_sel = SPI_CS_SEL_CS1;
	spi_conf.m_s    = SPI_M_S_MASTER;
	spi_conf.dma    = SPI_DMA_OFF;
	spi_conf.pol    = (SPI_CSW_8CLK | SPI_CK_DLY_ON | SPI_CK_POL_POS | SPI_CS_POL_NEG); //SPI_POL_SP0_CS1
	spi_conf.sclk   = SPI_SCLK_3MHZ;
	spi_conf.tiecs  = SPI_TIECS_NORMAL;

	iRet = spi_write(&spi_conf, (char *)&tx_data, 0, 4, 0);
	if(iRet < 0){
		chk_errno(iRet, &cRet);
		printk_err("spi_write(%s) error\n", cRet);
		return iRet;
	}
	printk_dbg((_DEBUG_LCDHW & 0x01), "->cmd(0x%02x) ->param(0x%02x)\n", cmd, param);
	return 0;
}


/* ------------------ IMC initialize function ----------------------------------- */
/******************************************************************************
* MODULE   : imc_hw_init
* FUNCTION : IMC(H/W) initialized
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2007.06.28 M.Haguro
* UPDATE   :
******************************************************************************/
#ifdef CONFIG_MACH_PGPF
int lcd_hw_imc_hw_init(void)
#else /* CONFIG_MACH_PGPF */
static int imc_hw_init(void)
#endif /* CONFIG_MACH_PGPF */
{
	unsigned long sync, format;
	int iRet;

	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!imc_param){
		imc_param = kmalloc(sizeof(struct mp200_imc_param), GFP_KERNEL);
		if(!imc_param){
			printk_err("could not allocate memory for \"IMC info\"\n");
			return -ENOMEM;
		}
		memset(imc_param, 0, sizeof(struct mp200_imc_param));
		
		imc_preset  = &imc_param->imc_preset;
		imc_vsync   = &imc_param->imc_vsync;
		imc_reserve = &imc_param->imc_reserve;

		/* set callback function */
		imc_param->callback_refresh = lcd_callback_imc_refresh;
		imc_param->callback_wb      = lcd_callback_imc_wb;
		printk_dbg((_DEBUG_LCDHW & 0x04), "kmalloc()\n");
	}

	/**********************
	 * set_callback
	 **********************/
	mp200_imc_set_callback(imc_param->callback_refresh, imc_param->callback_wb);
	printk_dbg((_DEBUG_LCDHW & 0x04), "set_callback()\n");

	/**********************
	 * set_preset
	 **********************/
	sync   = IMC_START_MODE_SYNCHRO;	/* VSYNC syncro mode */
#if defined(CONFIG_MP200_LCD_RGB666)
	format = IMC_FORMAT_RGB666;
#else
	format = IMC_FORMAT_RGB565;
#endif /* CONFIG_MP200_LCD_RGB666 */

	imc_param->imc_control  = (format | sync | IMC_CLKCNT_ALL_ON );
	imc_preset->imc_control = &imc_param->imc_control;
#ifdef CONFIG_PGPF_FB_PND_EXT
	/* L1B is always ARGB4444 mode */
	imc_param->imc_argb4444_mode = IMC_L1B_ARGB_BIT;
	imc_preset->imc_argb4444_mode = &imc_param->imc_argb4444_mode;
#endif /* CONFIG_PGPF_FB_PND_EXT */
	imc_preset->gamma       = imc_hw_init_preset_gamma(&imc_param->param_gamma);
	imc_preset->wb          = imc_hw_init_preset_wb(&imc_param->param_wb);
	imc_preset->yuv         = imc_hw_init_preset_yuv(&imc_param->param_yuv);
	imc_preset->keycolor    = imc_hw_init_preset_keycolor(&imc_param->param_keycolor);
	imc_preset->bytelane    = imc_hw_init_preset_bytelane(&imc_param->param_bytelane);

	iRet = mp200_imc_set_preset(imc_preset, SET_INIT_ATTR);
	if(iRet){
		return -1;
	}
	printk_dbg((_DEBUG_LCDHW & 0x04), "set_preset()\n");

	/**********************
	 * set_update_vsync
	 **********************/
	ImcNxtVsync.cpubufsel = IMC_CPUBUFSEL_INIT;
	ImcNxtVsync.mirror    = uiInverseFlag_tmp = IMC_MIRROR_OFF; 
	iRet = imc_hw_set_update_vsync(&ImcNxtVsync);
	if(iRet){
		return -1;
	}
	printk_dbg((_DEBUG_LCDHW & 0x04), "update_vsync()\n");

	/**********************
	 * set_update_reserve
	 **********************/
	imc_reserve->lo  = imc_hw_init_reserve_l0(&imc_param->param_l0);
	imc_reserve->l1a = imc_hw_init_reserve_l1a(&imc_param->param_l1a);
	imc_reserve->l1b = imc_hw_init_reserve_l1b(&imc_param->param_l1b);
	imc_reserve->l1c = imc_hw_init_reserve_l1c(&imc_param->param_l1c);
	imc_reserve->l2a = imc_hw_init_reserve_l2a(&imc_param->param_l2a);
	imc_reserve->l2b = imc_hw_init_reserve_l2b(&imc_param->param_l2b);
	imc_reserve->bg  = imc_hw_init_reserve_bg(&imc_param->param_bg);

#ifdef CONFIG_PGPF_FB_PND_EXT
	console_layer = *imc_reserve->l1a;
	fb_layer = *imc_reserve->l1b;
	bg_layer = *imc_reserve->bg;
#endif /* CONFIG_PGPF_FB_PND_EXT */
#ifdef CONFIG_PGPF_FB_LAYER2
	l2_layer = *imc_reserve->l2a;
#endif /* CONFIG_PGPF_FB_LAYER2 */

	iRet = mp200_imc_set_update_reserve(imc_reserve, imc_param->callback_refresh);
	if(iRet){
		return -1;
	}
	printk_dbg((_DEBUG_LCDHW & 0x04), "update_reserve()\n");

	return 0;
}


#if defined(CONFIG_PM) || defined(CONFIG_DPM)
/******************************************************************************
* MODULE   : imc_hw_restore_reg
* FUNCTION : restore IMC(H/W) register
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2007.11.19 M.Haguro
* UPDATE   :
******************************************************************************/
static int imc_hw_restore_reg(void)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!imc_param){
		printk_err("not allocate memory for \"IMC info\"\n");
		return -ENOMEM;
	}

	/**********************
	 * set_preset
	 **********************/
	if(mp200_imc_set_preset(imc_preset, SET_INIT_ATTR)){
		printk_err("set_preset() failed!\n");
		return -1;
	}
	printk_dbg((_DEBUG_LCDHW & 0x04), "set_preset()\n");

	/**********************
	 * set_update_vsync
	 **********************/
	if(imc_hw_set_update_vsync(&ImcNxtVsync)){
		printk_err("set_update_vsync() failed!\n");
		return -1;
	}
	printk_dbg((_DEBUG_LCDHW & 0x04), "set_update_vsync()\n");

	IMC_reset_flg = 0;
	return 0;
}
#endif /* CONFIG_PM || CONFIG_DPM */


/*******************************************************************************
* MODULE   : imc_hw_init_preset_gamma
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_preset_gamma(struct imc_gamma_param *param_gamma)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_gamma) return NULL;
	memset(param_gamma, 0, sizeof(struct imc_gamma_param));

	param_gamma->en   = IMC_GAMMAEN_OFF;
	param_gamma->adr  = IMC_GAMMA_ADR_INIT;
	param_gamma->data = IMC_GAMMA_DATA_INIT;

	return param_gamma;
}


/*******************************************************************************
* MODULE   : imc_hw_init_preset_wb
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_preset_wb(struct imc_wb_param *param_wb)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_wb) return NULL;
	memset(param_wb, 0, sizeof(struct imc_wb_param));

#if defined(CONFIG_MP200_LCD_RGB666)
	param_wb->format  = IMC_WB_FORMAT_RGB666;
#else
	param_wb->format  = IMC_WB_FORMAT_RGB565;
#endif /* CONFIG_MP200_LCD_RGB666 */
	param_wb->areaadr = LCD_AREAADR_INIT;
	param_wb->hsize   = FRONT_WIDTH;
	param_wb->vsize   = FRONT_HEIGHT;
	param_wb->hoffset = param_wb->hsize * BITS_PER_PIXEL / 8;

	return param_wb;
}


/*******************************************************************************
* MODULE   : imc_hw_init_preset_yuv
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_preset_yuv(struct imc_yuv_param *param_yuv)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_yuv) return NULL;
	memset(param_yuv, 0, sizeof(struct imc_yuv_param));

	param_yuv->ygain   = IMC_YGAIN_OFFSET_INIT;
	param_yuv->ugain   = IMC_UGAIN_OFFSET_INIT;
	param_yuv->vgain   = IMC_VGAIN_OFFSET_INIT;
	param_yuv->yuv2rgb = (IMC_DITHER_OFF | IMC_TRANS_MODE_BT601);
	if(param_yuv->yuv2rgb == IMC_TRANS_MODE_CUSTOM1 ||
	   param_yuv->yuv2rgb == IMC_TRANS_MODE_CUSTOM2)
	{
		param_yuv->coef_r[0] = IMC_COEF_R0_INIT;
		param_yuv->coef_r[1] = IMC_COEF_R1_INIT;
		param_yuv->coef_r[2] = IMC_COEF_R2_INIT;
		param_yuv->coef_r[3] = IMC_COEF_R3_INIT;
		param_yuv->coef_g[0] = IMC_COEF_G0_INIT;
		param_yuv->coef_g[1] = IMC_COEF_G1_INIT;
		param_yuv->coef_g[2] = IMC_COEF_G2_INIT;
		param_yuv->coef_g[3] = IMC_COEF_G3_INIT;
		param_yuv->coef_b[0] = IMC_COEF_B0_INIT;
		param_yuv->coef_b[1] = IMC_COEF_B1_INIT;
		param_yuv->coef_b[2] = IMC_COEF_B2_INIT;
		param_yuv->coef_b[3] = IMC_COEF_B3_INIT;
	}

	return param_yuv;
}


/*******************************************************************************
* MODULE   : imc_hw_init_preset_keycolor
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.12.17 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_preset_keycolor(struct imc_keycolor_param *param_keycolor)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_keycolor) return NULL;
	memset(param_keycolor, 0, sizeof(struct imc_keycolor_param));

	param_keycolor->l0_keycolor  = IMC_L0_KEYCOLOR_INIT;
	param_keycolor->l1x_keycolor = IMC_L1x_KEYCOLOR_INIT;

	ImcNxtKeycol.l0_keycolor  = param_keycolor->l0_keycolor;
	ImcNxtKeycol.l1x_keycolor = param_keycolor->l1x_keycolor;

	return param_keycolor;
}


/*******************************************************************************
* MODULE   : imc_hw_init_preset_bytelane
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.12.17 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_preset_bytelane(struct imc_bytelane_param *param_bytelane)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_bytelane) return NULL;
	memset(param_bytelane, 0, sizeof(struct imc_bytelane_param));

	param_bytelane->l2a_bytelane = IMC_L2A_BYTELANE_INIT;
	param_bytelane->l2b_bytelane = IMC_L2B_BYTELANE_INIT;

	return param_bytelane;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_l0
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_l0(struct l01_param *param_l0)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_l0) return NULL;
	memset(param_l0, 0, sizeof(struct l01_param));

	param_l0->control   = IMC_L0_CONTROL_INIT;
	param_l0->format    = IMC_L0_FORMAT_INIT;
	param_l0->keyenable = IMC_L0_KEYENABLE_INIT;
	param_l0->alpha     = IMC_L0_ALPHA_INIT;
	param_l0->resize    = IMC_L0_RESIZE_INIT;
	param_l0->offset    = IMC_L0_OFFSET_INIT;
	param_l0->frameadr  = IMC_L0_FRAMEADR_INIT;
	param_l0->position  = IMC_L0_POSITION_INIT;
	param_l0->size      = IMC_L0_SIZE_INIT;

	return param_l0;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_l1a
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_l1a(struct l01_param *param_l1a)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_l1a) return NULL;
	memset(param_l1a, 0, sizeof(struct l01_param));

	/* Layer1x public setting */
#ifdef CONFIG_PGPF_FB_PND_EXT
	/* L1A is always RGB565 format (independent of LCD format) */
	param_l1a->format    = IMC_Lx_FORMAT_RGB565;
#else /* CONFIG_PGPF_FB_PND_EXT */
	/* Layer1x public setting */
#if defined(CONFIG_MP200_LCD_RGB666)
	param_l1a->format    = IMC_Lx_FORMAT_RGB666;
#else
	param_l1a->format    = IMC_Lx_FORMAT_RGB565;
#endif /* CONFIG_MP200_LCD_RGB666 */
#endif /* CONFIG_PGPF_FB_PND_EXT */
	param_l1a->resize    = IMC_Lx_RESIZE_DISABLE;

#ifdef CONFIG_PGPF_FB_PND_EXT
	param_l1a->offset    = LCD_CONSOLE_MAX_XRES * FB_BPP / 8;
#else /* CONFIG_PGPF_FB_PND_EXT */
	param_l1a->offset    = FRONT_WIDTH * BITS_PER_PIXEL / 8;
#endif /* CONFIG_PGPF_FB_PND_EXT */

	/* Layer1A private setting */
	param_l1a->control   = IMC_L1A_CONTROL_INIT;
	param_l1a->keyenable = IMC_L1A_KEYENABLE_INIT;
	param_l1a->alpha     = IMC_L1A_ALPHA_INIT;
	param_l1a->frameadr  = IMC_L1A_FRAMEADR_INIT;
	param_l1a->position  = IMC_L1A_POSITION_INIT;
	param_l1a->size      = IMC_L1A_SIZE_INIT;

	return param_l1a;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_l1b
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_l1b(struct l01_param *param_l1b)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_l1b) return NULL;
	memset(param_l1b, 0, sizeof(struct l01_param));

	param_l1b->control   = IMC_L1B_CONTROL_INIT;
	param_l1b->keyenable = IMC_L1B_KEYENABLE_INIT;
	param_l1b->alpha     = IMC_L1B_ALPHA_INIT;
	param_l1b->frameadr  = IMC_L1B_FRAMEADR_INIT;
	param_l1b->position  = IMC_L1B_POSITION_INIT;
	param_l1b->size      = IMC_L1B_SIZE_INIT;

	return param_l1b;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_l1c
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.06.29 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_l1c(struct l01_param *param_l1c)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_l1c) return NULL;
	memset(param_l1c, 0, sizeof(struct l01_param));

	param_l1c->control   = IMC_L1C_CONTROL_INIT;
	param_l1c->keyenable = IMC_L1C_KEYENABLE_INIT;
	param_l1c->alpha     = IMC_L1C_ALPHA_INIT;
	param_l1c->frameadr  = IMC_L1C_FRAMEADR_INIT;
	param_l1c->position  = IMC_L1C_POSITION_INIT;
	param_l1c->size      = IMC_L1C_SIZE_INIT;

	return param_l1c;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_l2a
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.07.02 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_l2a(struct l2_param *param_l2a)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_l2a) return NULL;
	memset(param_l2a, 0, sizeof(struct l2_param));

#ifdef CONFIG_PGPF_FB_LAYER2
	param_l2a->format      = IMC_LBG_FORMAT_RGB565;
	param_l2a->bufsel      = IMC_L2A_BUFSEL_INIT;
	param_l2a->resize      = IMC_L2A_RESIZE_INIT;
	param_l2a->mirror      = IMC_L2A_MIRROR_INIT;
	param_l2a->offset      = LCD_SUB_MAX_XRES * SUB_BPP / 8;
	param_l2a->frameadr_yp = SMEM_START + SUB_A_OFFSET;
	param_l2a->frameadr_up = IMC_L2A_FRAMEADR_UP_INIT;
	param_l2a->frameadr_yq = SMEM_START + SUB_B_OFFSET;
	param_l2a->frameadr_uq = IMC_L2A_FRAMEADR_UQ_INIT;
	param_l2a->position    = IMC_L2A_POSITION_INIT;
	param_l2a->size        = IMC_L2A_SIZE_INIT;
#else /* CONFIG_PGPF_FB_LAYER2 */
	param_l2a->format      = IMC_L2A_FORMAT_INIT;
	param_l2a->bufsel      = IMC_L2A_BUFSEL_INIT;
	param_l2a->resize      = IMC_L2A_RESIZE_INIT;
	param_l2a->mirror      = IMC_L2A_MIRROR_INIT;
	param_l2a->offset      = IMC_L2A_OFFSET_INIT;
	param_l2a->frameadr_yp = IMC_L2A_FRAMEADR_YP_INIT;
	param_l2a->frameadr_up = IMC_L2A_FRAMEADR_UP_INIT;
	param_l2a->frameadr_yq = IMC_L2A_FRAMEADR_YQ_INIT;
	param_l2a->frameadr_uq = IMC_L2A_FRAMEADR_UQ_INIT;
	param_l2a->position    = IMC_L2A_POSITION_INIT;
	param_l2a->size        = IMC_L2A_SIZE_INIT;
#endif /* CONFIG_PGPF_FB_LAYER2 */

	param_l2a->control     = IMC_L2A_CONTROL_INIT;
	param_l2a->bufsel      = IMC_L2A_BUFSEL_INIT;
	param_l2a->resize      = IMC_L2A_RESIZE_INIT;
	param_l2a->mirror      = IMC_L2A_MIRROR_INIT;
	param_l2a->offset      = IMC_L2A_OFFSET_INIT;
	param_l2a->frameadr_yp = IMC_L2A_FRAMEADR_YP_INIT;
	param_l2a->frameadr_up = IMC_L2A_FRAMEADR_UP_INIT;
	param_l2a->frameadr_yq = IMC_L2A_FRAMEADR_YQ_INIT;
	param_l2a->frameadr_uq = IMC_L2A_FRAMEADR_UQ_INIT;
	param_l2a->position    = IMC_L2A_POSITION_INIT;
	param_l2a->size        = IMC_L2A_SIZE_INIT;

	return param_l2a;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_l2b
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.07.02 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_l2b(struct l2_param *param_l2b)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_l2b) return NULL;
	memset(param_l2b, 0, sizeof(struct l2_param));

	param_l2b->control     = IMC_L2B_CONTROL_INIT;
	param_l2b->format      = IMC_L2B_FORMAT_INIT;
	param_l2b->bufsel      = IMC_L2B_BUFSEL_INIT;
	param_l2b->resize      = IMC_L2B_RESIZE_INIT;
	param_l2b->mirror      = IMC_L2B_MIRROR_INIT;
	param_l2b->offset      = IMC_L2B_OFFSET_INIT;
	param_l2b->frameadr_yp = IMC_L2B_FRAMEADR_YP_INIT;
	param_l2b->frameadr_up = IMC_L2B_FRAMEADR_UP_INIT;
	param_l2b->frameadr_yq = IMC_L2B_FRAMEADR_YQ_INIT;
	param_l2b->frameadr_uq = IMC_L2B_FRAMEADR_UQ_INIT;
	param_l2b->position    = IMC_L2B_POSITION_INIT;
	param_l2b->size        = IMC_L2B_SIZE_INIT;

	return param_l2b;
}


/*******************************************************************************
* MODULE   : imc_hw_init_reserve_bg
* FUNCTION : 
* RETURN   : char *
* NOTE     : none
* CREATE   : 2007.07.02 M.Haguro
* UPDATE   :
********************************************************************************/
static void *imc_hw_init_reserve_bg(struct bg_param *param_bg)
{
	printk_dbg((_DEBUG_LCDHW & 0x01), "\n");

	if(!param_bg) return NULL;
	memset(param_bg, 0, sizeof(struct bg_param));

#if 0
	param_bg->format   = IMC_BG_FORMAT_INIT;
	param_bg->resize   = IMC_BG_RESIZE_INIT;
	param_bg->offset   = IMC_BG_OFFSET_INIT;
	param_bg->frameadr = IMC_BG_FRAMEADR_INIT;
#else
#ifdef CONFIG_PGPF_FB_PND_EXT
	param_bg->format   = IMC_LBG_FORMAT_RGB565;
	param_bg->offset   =  LCD_BASE_MAX_XRES * BASE_BPP / 8;
	param_bg->frameadr = SMEM_START + BASE_A_OFFSET;
#else /* CONFIG_PGPF_FB_PND_EXT */
/*	param_bg->format   = IMC_LBG_FORMAT_BLACK; */
	param_bg->format   = IMC_LBG_FORMAT_OPTIONAL;
	param_bg->offset   = FRONT_WIDTH * BITS_PER_PIXEL / 8;
	param_bg->frameadr = 0;
#endif /* CONFIG_PGPF_FB_PND_EXT */
	param_bg->resize   = IMC_Lx_RESIZE_DISABLE;
#endif

	return param_bg;
}


/*******************************************************************************
* MODULE   : imc_hw_set_update_vsync
* FUNCTION : 
* RETURN   :  0: success
*          : -1: failed
* NOTE     : none
* CREATE   : 2007.08.01 M.Haguro
* UPDATE   :
********************************************************************************/
int imc_hw_set_update_vsync(struct mp200_imc_update_vsync *vsync)
{
	static int chg_flg = 1;
	printk_dbg((_DEBUG_LCDHW & 0x01), "in\n");

	if(imc_vsync->cpubufsel != vsync->cpubufsel){
		chg_flg = 1;
	}

	if(imc_vsync->mirror == IMC_MIRROR_OFF && vsync->mirror != NO_INVERSE){
		vsync->mirror = (IMC_MIRROR_UD | IMC_MIRROR_LR);
		chg_flg = 1;
	}
	else if(imc_vsync->mirror != IMC_MIRROR_OFF && vsync->mirror == NO_INVERSE){
		vsync->mirror = IMC_MIRROR_OFF;
		chg_flg = 1;
	}

	if(chg_flg){
		chg_flg = 0;
		if(mp200_imc_set_update_vsync(vsync)){
			printk_dbg((_DEBUG_LCDHW & 0x04), "mp200_imc_set_update_vsync() failed\n");
			return -1;
		}
		printk_dbg((_DEBUG_LCDHW & 0x04), "mp200_imc_set_update_vsync() success\n");
		imc_vsync->cpubufsel = vsync->cpubufsel;
		imc_vsync->mirror    = vsync->mirror;
	}

	printk_dbg((_DEBUG_LCDHW & 0x02), "out\n");
	return 0;
}


/*******************************************************************************
* MODULE   : imc_hw_set_update_keycolor
* FUNCTION : 
* RETURN   :  0: success
*          : -1: failed
* NOTE     : none
* CREATE   : 2007.12.17 M.Haguro
* UPDATE   :
********************************************************************************/
int imc_hw_set_update_keycolor(struct imc_keycolor_param *keycolor)
{
	int chg_flg = 0;
	struct mp200_imc_preset   preset;
	struct imc_keycolor_param save_keycol;
	printk_dbg((_DEBUG_LCDHW & 0x01), "in\n");

	memset(&preset, 0, sizeof(struct mp200_imc_preset));
	preset.keycolor = &imc_param->param_keycolor;
	memcpy(&save_keycol, preset.keycolor, sizeof(struct imc_keycolor_param));

	if(preset.keycolor->l0_keycolor != keycolor->l0_keycolor){
		/* change Layer0 colorkey */
		preset.keycolor->l0_keycolor = keycolor->l0_keycolor;
		printk_dbg((_DEBUG_LCDHW & 0x02), "l0_keycolor(%06lx)\n", keycolor->l0_keycolor);
		chg_flg = 1;
	}
	if(preset.keycolor->l1x_keycolor != keycolor->l1x_keycolor){
		/* change Layer1 colorkey */
		preset.keycolor->l1x_keycolor = keycolor->l1x_keycolor;
		printk_dbg((_DEBUG_LCDHW & 0x02), "l1x_keycolor(%06lx)\n", keycolor->l1x_keycolor);
		chg_flg = 1;
	}

	if(chg_flg){
		if(mp200_imc_set_preset(&preset, SET_INIT_ATTR)){
			printk_dbg((_DEBUG_LCDHW & 0x04), "mp200_imc_set_prest() failed\n");
			memcpy(preset.keycolor, &save_keycol, sizeof(struct imc_keycolor_param));
			return -1;
		}
		printk_dbg((_DEBUG_LCDHW & 0x04), "mp200_imc_set_prest() success\n");
		imc_preset->keycolor->l0_keycolor  = keycolor->l0_keycolor;  /* save state */
		imc_preset->keycolor->l1x_keycolor = keycolor->l1x_keycolor; /* save state */
	}

	printk_dbg((_DEBUG_LCDHW & 0x02), "out\n");
	return 0;
}


/*******************************************************************************
* MODULE   : imc_hw_set_update_bytelane
* FUNCTION : 
* RETURN   :  0: success
*          : -1: failed
* NOTE     : none
* CREATE   : 2008.10.24 M.Haguro
* UPDATE   :
********************************************************************************/
int imc_hw_set_update_bytelane(struct imc_bytelane_param *bytelane)
{
	int chg_flg = 0;
	struct mp200_imc_preset   preset;
	struct imc_bytelane_param save_bytelane;
	printk_dbg((_DEBUG_LCDHW & 0x01), "in\n");

	memset(&preset, 0, sizeof(struct mp200_imc_preset));
	preset.bytelane = &imc_param->param_bytelane;
	memcpy(&save_bytelane, preset.bytelane, sizeof(struct imc_bytelane_param));

	if(preset.bytelane->l2a_bytelane != bytelane->l2a_bytelane){
		/* change Layer2a bytealne */
		preset.bytelane->l2a_bytelane = bytelane->l2a_bytelane;
		printk_dbg((_DEBUG_LCDHW & 0x02), "l2a_bytelane(%04lx)\n", bytelane->l2a_bytelane);
		chg_flg = 1;
	}
	if(preset.bytelane->l2b_bytelane != bytelane->l2b_bytelane){
		/* change Layer2b bytealne */
		preset.bytelane->l2b_bytelane = bytelane->l2b_bytelane;
		printk_dbg((_DEBUG_LCDHW & 0x02), "l2b_bytelane(%04lx)\n", bytelane->l2b_bytelane);
		chg_flg = 1;
	}

	if(chg_flg){
		if(mp200_imc_set_preset(&preset, SET_INIT_ATTR)){
			printk_dbg((_DEBUG_LCDHW & 0x04), "mp200_imc_set_prest() failed\n");
			memcpy(preset.bytelane, &save_bytelane, sizeof(struct imc_bytelane_param));
			return -1;
		}
		printk_dbg((_DEBUG_LCDHW & 0x04), "mp200_imc_set_prest() success\n");
		imc_preset->bytelane->l2a_bytelane = bytelane->l2a_bytelane; /* save state */
		imc_preset->bytelane->l2b_bytelane = bytelane->l2b_bytelane; /* save state */
	}

	printk_dbg((_DEBUG_LCDHW & 0x02), "out\n");
	return 0;
}


/*******************************************************************************
* MODULE   : imc_hw_set_update_reserve
* FUNCTION : 
* RETURN   :  0: success
*          : -1: failed
* NOTE     : none
* CREATE   : 2007.07.27 M.Haguro
* UPDATE   :
********************************************************************************/
int imc_hw_set_update_reserve(int type, int mixdsp, int update)
{
	static struct l01_param tmp_fb_layer;
	int iRet;
	printk_dbg((_DEBUG_LCDHW & 0x01), "in\n");

	/**********************
	 * set_update_reserve
	 **********************/
#ifdef CONFIG_PGPF_FB_PND_EXT
	/* force setting console and bg layer */
	imc_reserve->l1a = &console_layer;
	imc_reserve->bg = &bg_layer;
#endif /* CONFIG_PGPF_FB_PND_EXT */
#ifdef CONFIG_PGPF_FB_LAYER2
	imc_reserve->l2a = &l2_layer;
#endif /* CONFIG_PGPF_FB_LAYER2 */
	if(type == SET_LAYER_2D){
		memcpy(&tmp_fb_layer, &fb_layer, sizeof(struct l01_param));
#ifndef CONFIG_MACH_PGPF
		if(mixdsp != MIX_DSP_ON) tmp_fb_layer.keyenable = IMC_Lx_KEYEN_DISABLE;
#endif /* !CONFIG_MACH_PGPF */

		/* set 2D(DispBuffer) layer */
		printk_dbg((_DEBUG_LCDHW & 0x04), "   set2D\n");
#ifdef CONFIG_PGPF_FB_PND_EXT
		imc_reserve->l1b = &tmp_fb_layer;
#else /* CONFIG_PGPF_FB_PND_EXT */
		imc_reserve->l1a = &tmp_fb_layer;
#endif /* CONFIG_PGPF_FB_PND_EXT */
	}
	else if(type == SET_LAYER_V4L2){
		/* set v4l2 layer */
		printk_dbg((_DEBUG_LCDHW & 0x04), "   setV4L2\n");
#ifdef CONFIG_PGPF_FB_PND_EXT
		imc_reserve->l1b = &fb_layer;
#else /* CONFIG_PGPF_FB_PND_EXT */
		imc_reserve->l1a = &fb_layer;
#endif /* CONFIG_PGPF_FB_PND_EXT */
		imc_reserve->l2b = &v4l2_layer;
	}
	else if(type == CLR_LAYER_V4L2){
		memcpy(&tmp_fb_layer, &fb_layer, sizeof(struct l01_param));
#ifndef CONFIG_MACH_PGPF
		if(mixdsp != MIX_DSP_ON) tmp_fb_layer.keyenable = IMC_Lx_KEYEN_DISABLE;
#endif /* !CONFIG_MACH_PGPF */

		/* clr v4l2 layer */
		printk_dbg((_DEBUG_LCDHW & 0x04), "   clrV4L2\n");
#ifdef CONFIG_PGPF_FB_PND_EXT
		imc_reserve->l1b = &fb_layer;
#else /* CONFIG_PGPF_FB_PND_EXT */
		imc_reserve->l1a = &tmp_fb_layer;
#endif /* CONFIG_PGPF_FB_PND_EXT */
		imc_reserve->l2b = imc_hw_init_reserve_l2b(&imc_param->param_l2b);
	}
	else{
		printk_dbg((_DEBUG_LCDHW & 0x04), "   set----\n");
		return -1;
	}

	if(update ==  UPDATE_ON){
		iRet = mp200_imc_set_update_reserve(imc_reserve, imc_param->callback_refresh);
		if(iRet){
			return -1;
		}
	}
	printk_dbg((_DEBUG_LCDHW & 0x02), "out\n");

	return 0;
}



