/* 2010-08-25: File added by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/nts/mp200_nts_common.h
 * Function        : NTSC Driver corresponding to V4L2
 * Release Version : Ver 1.00
 * Release Date    : 2008/10/29
 *
 * Copyright (C) NEC Electronics Corporation 2008
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#ifndef _MP200_NTS_COMMON_H_
#define _MP200_NTS_COMMON_H_


/********************************************************
 *  Definitions                                         *
 *******************************************************/
/* for iOutmodeFlg */
#define FB_OFF			0x01
#define FB_ON			0x02
#define V4L2_OFF		0x04
#define V4L2_ON			0x08

#define FB_V4L2_OFF		0x05
#define FB_ONLY			0x06
#define V4L2_ONLY		0x09
#define FB_V4L2_ON		0x0A

#define FB_BIT			0x03
#define V4L2_BIT		0x0C

/* display frame no. for iFrameNoNow/iFrameNoNext */
#define NTS_DISP_INIT		NTS_AREASEL_BUFDISABLE
#define NTS_DISP_FRAME_A	NTS_AREASEL_BUFA
#define NTS_DISP_FRAME_B	NTS_AREASEL_BUFB
#define NTS_DISP_FRAME_C	NTS_AREASEL_BUFC


/********************************************************
 *  Structure                                           *
 *******************************************************/
/* structure video endcoder */
typedef struct encoder_reg {
	char   name[32];
	int    (*hw_init)(void *);
	void   (*hw_shutdown)(void *);
	int    (*hw_command)(unsigned int cmd, void* arg);
	void  *private_data;
} ENCODER_REG;

/* structure nts device status */
struct mp200_nts_dev {
	/* semafore */
	struct semaphore         sem_img_data;	/* semafore v4l2/fb/img local data */
	struct semaphore         sem_img_flag;	/* semafore v4l2/fb/img local flag */
	/* spinlock */
	spinlock_t               lock_tmr;		/* spin lock for NTS timer         */
	spinlock_t               lock_callback;		/* spin lock for callback          */
	/* work queue */
	struct workqueue_struct *mp200_nts_workqueue;
	struct work_struct       wk_timeout_bottom_half;

	/* waite timer. movie is pause state */
	struct timer_list        nts_timer;
	int                      iTimerFlag;		/* timer operation status          */

	/* wait queue */
	wait_queue_head_t        wait_img;
	int                      img_complete;

	/* flags */
	int                      iNtsActive;
	unsigned int             uiMixFrameBufferPage;	/* next frame buffer (2D)          */

	int                      iMixDSPFlg;		/* movie & UI mix ON/OFF           */
	int                      iMixDSPFlg_toFb;	/* movie & UI mix ON/OFF (for FB)  */

	int                      iV4L2PauseFlg;		/* V4L2 pause status flag          */
	int                      iV4L2CallbackFlg;	/* need to callback to V4L2        */
	struct _FRAME_DATA       v4l2_callback_data;

	int                      iFrameNoNow;		/* now display frame no  */
	int                      iFrameNoNext;		/* set change frame no   */

	int                      iOutmodeFlg;		/* output i/f                      */
	int                      iMixImageMode;		/*  */

	unsigned int             uiAbsolutelyUpFlag;	/* absolutely update flag    */

	FB_IMAGE_INFO            from_fb;
	V4L2_IMAGE_INFO          from_v4l2;

	/* NTSC Encoder */
	ENCODER_REG             *encoder;
};




#endif /* _MP200_NTS_COMMON_H_ */
