/* 2010-08-25: File added by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/nts/mp200_nts_debug.h
 * Function        : NTSC Driver corresponding to V4L2
 * Release Version : Ver 1.00
 * Release Date    : 2008/10/29
 *
 * Copyright (C) NEC Electronics Corporation 2008
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#ifndef _MP200_NTS_DEBUG_H_
#define _MP200_NTS_DEBUG_H_

/*=============================================================================
   Include files
 *=============================================================================*/
#include <linux/module.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/processor.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/timer.h>
#include <linux/sched.h>

#include <asm/ioctl.h>
#include <asm/types.h>
#include <asm/io.h>

#include <asm/arch/img.h>


/*=============================================================================
   Systemcall Function
 *=============================================================================*/
inline int load_image(unsigned long paddr, int format, int load_w, int load_h, int load_p, char *load_num);
inline int save_image(unsigned long paddr, int format, int save_w, int save_h, int save_p, char *file_num);
static int read_file(unsigned long paddr, int format, int read_w, int read_h, int read_p, char *read_num);
static int write_file(unsigned long paddr, int format, int write_w, int write_h, int write_p, char *write_num);


/*=============================================================================
   Internal Function
 *=============================================================================*/
/*****************************************************************************
* MODULE   : load_image
* FUNCTION : 
* IMPORT   : none
* RETURN   :  0
*          : -1
* NOTE     : none
* CREATE   : 2008.12.18 M.Haguro
* UPDATE   : 
******************************************************************************/
inline int load_image(unsigned long paddr, int format, int load_w, int load_h, int load_p, char *load_num)
{
	return read_file(paddr, format, load_w, load_h, load_p, load_num);
}


/*****************************************************************************
* MODULE   : save_image
* FUNCTION : 
* IMPORT   : none
* RETURN   :  0
*          : -1
* NOTE     : none
* CREATE   : 2008.12.18 M.Haguro
* UPDATE   : 
******************************************************************************/
inline int save_image(unsigned long paddr, int format, int save_w, int save_h, int save_p, char *file_num)
{
	return write_file(paddr, format, save_w, save_h, save_p, file_num);
}


/*****************************************************************************
* MODULE   : read_file
* FUNCTION : 
* IMPORT   : none
* RETURN   :  0
*          : -1
* NOTE     : none
* CREATE   : 2008.12.18 M.Haguro
* UPDATE   : 
******************************************************************************/
static int read_file(unsigned long paddr, int format, int read_w, int read_h, int read_p, char *read_num)
{
	static int read_cnt = 0;
	int    nr_read, read_size, pitch, i;
	struct file *filp;
	mm_segment_t fs;
	char *vram, *vram_tmp, *file_num = NULL;

	switch(format){
	case IMG_FORMAT_YUV420_SEMI_PLANAR: /* YUV420 Semi-Planar */
	case IMG_FORMAT_YUV420_PLANAR:      /* YUV420 Planar      */
		read_size = read_p * read_h * 3/2;
		pitch = read_p - read_w *1;
		break;
	case IMG_FORMAT_YUV422_SEMI_PLANAR: /* YUV422 Semi-Planar */
	case IMG_FORMAT_YUV422_PLANAR:      /* YUV422 Planar      */
	case IMG_FORMAT_YUV422_INTERLEAVE:  /* YUV422 Interleave  */
		read_size = read_p * read_h * 2;
		pitch = read_p - read_w *1;
		break;
	case IMG_FORMAT_RGB565:             /* RGB565             */
		read_size = read_p * read_h * 2;
		pitch = read_p - read_w *2;
		break;
	case IMG_FORMAT_RGB666:             /* RGB565             */
		read_size = read_p * read_h * 18/8;
		pitch = read_p - read_w *18/8;
		break;
	case IMG_FORMAT_RGB888:             /* RGB888             */
		read_size = read_p * read_h * 3;
		pitch = read_p - read_w *3;
		break;
	default:
		return -1;
	}

	vram = ioremap(paddr, read_size); 
	if(vram){
		sprintf(file_num, "%s_%02d", read_num, read_cnt++);
		filp = filp_open(file_num, O_RDONLY | O_LARGEFILE, 0);
		if(!filp){
			printk("%s: \"%s\" --can't open\n", __func__, file_num);
			read_cnt--;
			return -1;
		}
		fs = get_fs();
		set_fs(get_ds());

		if (pitch == 0) {
			nr_read = filp->f_op->read(filp, vram, read_size, &filp->f_pos);
			printk("%s: --load-- %d byte <<< \"%s\"\n", __func__, nr_read, file_num);
		}
		else { /* pitch */
			printk("@@@@@ read_w(%d)  read_h(%d)  read_p(%d)\n", read_w, read_h, read_p);
			for (i = 0;  i < read_h; i++) {
				vram_tmp = vram + (pitch * i);
				printk("@@@@@ vram(0x%p)  vram_tmp(0x%p)  filp(0x%p)  read_size(%d)\n", vram, vram_tmp, filp, read_size);
				nr_read = filp->f_op->read(filp, vram_tmp, read_size, &filp->f_pos);
			}
			printk("%s: --load-- %d byte <<< \"%s\"\n", __func__, read_size, file_num);
		}
		set_fs(fs);
		filp_close(filp, NULL);
		schedule();
	}
	else{
		printk("%s: \"%s\" --cant't allocate memory\n", __func__, file_num);
		return -1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : write_file
* FUNCTION : 
* IMPORT   : none
* RETURN   :  0
*          : -1
* NOTE     : none
* CREATE   : 2008.12.18 M.Haguro
* UPDATE   : 
******************************************************************************/
static int write_file(unsigned long paddr, int format, int write_w, int write_h, int write_p, char *write_num)
{
	static int save_cnt = 0;
	int    nr_write, write_size;
	struct file *filp;
	mm_segment_t fs;
	char *vram; 
	char file_num[50];
	memset(file_num, 0, sizeof(file_num));

	switch(format){
	case IMG_FORMAT_YUV420_SEMI_PLANAR: /* YUV420 Semi-Planar */
	case IMG_FORMAT_YUV420_PLANAR:      /* YUV420 Planar      */
		write_size = write_p * write_h * 3/2;
		break;
	case IMG_FORMAT_YUV422_SEMI_PLANAR: /* YUV422 Semi-Planar */
	case IMG_FORMAT_YUV422_PLANAR:      /* YUV422 Planar      */
	case IMG_FORMAT_YUV422_INTERLEAVE:  /* YUV422 Interleave  */
	case IMG_FORMAT_RGB565:             /* RGB565             */
		write_size = write_p * write_h * 2;
		break;
	case IMG_FORMAT_RGB666:             /* RGB565             */
		write_size = write_p * write_h * 18/8;
		break;
	case IMG_FORMAT_RGB888:             /* RGB888             */
		write_size = write_p * write_h * 3;
		break;
	default:
		return -1;
	}

	vram = ioremap(SMEM_START, SMEM_LENGTH);
	vram = vram + (paddr - SMEM_START);
	if(vram){
		sprintf(file_num, "%s_%d_%d_%02d", write_num, write_w, write_h, save_cnt++);
		filp = filp_open(file_num, O_CREAT | O_RDWR | O_LARGEFILE, 0);
		if(!filp){
			printk("%s: \"%s\" --can't open\n", __func__, file_num);
			save_cnt--;
			iounmap(vram);
			return -1;
		}
		fs = get_fs();
		set_fs(get_ds());
		nr_write = filp->f_op->write(filp, vram, write_size-1, &filp->f_pos);
		printk("%s: --save-- %d byte >>> \"%s\"\n", __func__, nr_write, file_num);
		set_fs(fs);
		filp_close(filp, NULL);
		iounmap(vram);
		schedule();
	}
	else{
		printk("%s: \"%s\" --cant't allocate memory\n", __func__, file_num);
		return -1;
	}
	return 0;
}


#endif /* _MP200_NTS_DEBUG_H_ */

