/* 2010-08-25: File added and changed by Sony Corporation */
/*
 * File Name       : /drivers/video/mp200/nts/mp200_ntshw.c
 * Function        : NTSC Driver corresponding to V4L2 (H/W Control)
 * Release Version : Ver 1.00
 * Release Date    : 2008/10/29
 *
 * Copyright (C) NEC Electronics Corporation 2008
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


/********************************************************
 *  Include Files                                       *
 *******************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/platform_device.h>

#include <asm/io.h>
#include <asm/delay.h>
#include <asm/arch/irqs.h>
#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/arch/em1_mem.h>
#include <asm/arch/gdma.h>

#include <asm/arch/gpio.h>

#include "../mp200_common.h"
#include "ntsc.h"
#include "mp200_nts_common.h"
#include "mp200_ntsipu.h"
#include "mp200_ntshw.h"


/********************************************************
 *  Definitions                                         *
 *******************************************************/
#define DEV_NAME "mp200_ntshw"


/********************************************************
 *  debug parameters                                    *
 *******************************************************/
#define _DEBUG_NTSHW  0x00 /* 00008421(bit) */
                           /* 0x01: debug function in
                            * 0x02: debug function out
                            * 0x04: debug IMC
                            * 0x08: debug frame change
                            */


#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_NTSHW
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif


/********************************************************
 *  Macros                                              *
 *******************************************************/


/********************************************************
 * NTSC register initialize                             *
 *******************************************************/
#if 0
#define CHG_PINSEL_G00_NTS_BIT  0x00C00000
#define CHG_PINSEL_G00_NTS_INIT 0x00400000
#define CHG_PINSEL_G16_NTS_BIT  0x3F000000
#define CHG_PINSEL_G16_NTS_INIT 0x15000000
#define CHG_PINSEL_G32_NTS_BIT  0x0FFFC000
#define CHG_PINSEL_G32_NTS_INIT 0x05554000
#else
#define CHG_PINSEL_G64_NTS_BIT  0xFFFF0000
#define CHG_PINSEL_G64_NTS_INIT 0x55550000
#define CHG_PINSEL_G80_NTS_BIT  0x0000003F
#define CHG_PINSEL_G80_NTS_INIT 0x00000015
#endif

#define CHG_PULL_G72_NTS_BIT    0xFFFFFFFF
#define CHG_PULL_G72_NTS_INIT   0x00000004
#define CHG_PULL_G80_NTS_BIT    0xFFFFFFFF
#define CHG_PULL_G80_NTS_INIT   0x00000000


/********************************************************
 *  Variables                                           *
 *******************************************************/


/********************************************************
 *  Structure                                           *
 *******************************************************/
struct mp200_ntshw {
	/* output mode */
	int           Outmode;

	/* NTSC MMIO */
	unsigned long NTSCMmio;
	unsigned long NTSCMmioLength;
	char         *NTSCMmioV;

	/* NTSC framebuffer */
	unsigned long NtscFrameA_Y;
	unsigned long NtscFrameA_UV;
	unsigned long NtscFrameB_Y;
	unsigned long NtscFrameB_UV;
	unsigned long NtscFrameC_Y;
	unsigned long NtscFrameC_UV;
	unsigned long NtscFrameLength;
	char         *NtscFrameAV;
	char         *NtscFrameBV;

	/* Smem (common memory to LCD) */
	unsigned long Smem;
	unsigned long SmemLength;
	char         *SmemV;
};
static struct mp200_ntshw *ntshw;


/********************************************************
 *  Prototype declarations of function                  *
 *******************************************************/
/* ------------------ NTSC H/W control function ---------------------------------- */
       void          ntshw_start(int iStartFlg);
       void          ntshw_reset(int iResetFlg);
       void          ntshw_module_reset(int iResetFlg);
       void          ntshw_standby(int iStandbyFlg);
/* ------------------ NTSC rgister set function --------------------------------- */
       void          ntshw_set_interruput(int iIntFlg);
       int           ntshw_set_framesel(int iSetFrameNo);
       int           ntshw_set_ntsout(int iSetNtsoutFlg);
#ifdef NOT_USE_FUNC
static int           ntshw_set_control_upscale(int iSetUpscaleFlg);
#endif /* NOT_USE_FUNC */
static int           ntshw_set_control_outmode(int iSetOutmodeFlg);
static int           ntshw_set_areaad(int iSetAreaFlg, unsigned long ulSetAddrY, unsigned long ulSetAddrUV);
static int           ntshw_set_hoffset(unsigned long ulSetOffset);
static int           ntshw_set_register(int iSetOutmode);
/* ------------------ NTSC rgister check function ------------------------------- */
       unsigned long ntshw_chk_control(void);
       unsigned long ntshw_chk_status(void);
       unsigned long ntshw_chk_framesel(void);
       unsigned long ntshw_chk_intstatus(void);
#ifdef NOT_USE_FUNC
static unsigned long ntshw_chk_intrawstatus(void);
#endif /* NOT_USE_FUNC */
/* ------------------ NTSC rgister backup function ------------------------------ */
static void          ntshw_save_reg(void);
static void          ntshw_restore_reg(void);
/* ------------------ NTSC initialize function ----------------------------------- */
       int           ntshw_reserve(int iSetOutmode);
       void          ntshw_release(void);
       void          ntshw_clr_framebuf(int iClrFrameNo);
static int           ntshw_init_framebuf(void);
static void          ntshw_exit_framebuf(void);
static void         *ntshw_init_val(void);
       int           ntshw_init(void);
       void          ntshw_exit(void);





/* ------------------ NTSC H/W control function ---------------------------------- */
/******************************************************************************
* MODULE   : ntshw_start
* FUNCTION : start NTS(H/W)
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_start(int iStartFlg)
{
	if(iStartFlg == NTSHW_START){
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_START\n");
		ntshw_set_ntsout(NTS_NTSOUT_ENABLE_BLACK);
	}
	else{
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_STOP\n");
		ntshw_set_ntsout(NTS_NTSOUT_DISABLE);
		while((ntshw_chk_status() & NTS_STATUS_BIT) != NTS_STATUS_DISABLE){
			schedule();
		}
	}
}


/******************************************************************************
* MODULE   : ntshw_reset
* FUNCTION : NTS(H/W) reset
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_reset(int iResetFlg)
{
	if(iResetFlg == NTSHW_RESET){
#if 0	/* Do not reset NTSC because it's unnecessary 
	   and because reseting before waiting NTS out is disabled 
	   causes BUS hung up.             (2009.05.07 Y.Utsuhara) */
		/* Reset NTSC */
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_RESET\n");

		/* clock auto control -> OFF    */
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_NTS);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_NTSPCLK);
		/* NTSC reset                   */
		mp200_pmu_reset_device(MP200_RESETDEVICE_NTS);
		/* clock demand                 */
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_NTS_PCLK);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_NTS_CLK);
#endif
	}
	else{
		/* UnReset NTSC */
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_UNRESET\n");

		/* clock supply                 */
		mp200_pmu_open_clockgate(MP200_CLOCKGATE_NTS_PCLK);
		mp200_pmu_open_clockgate(MP200_CLOCKGATE_NTS_CLK);
		/* clock auto control -> OFF    */
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_NTS);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_NTSPCLK);
		/* NTSC unreset                 */
		mp200_pmu_unreset_device(MP200_RESETDEVICE_NTS);
		/* clock auto control -> ON     */
		mp200_pmu_clkctrl_on(MP200_CLKCTRL_NTS);
		mp200_pmu_clkctrl_on(MP200_CLKCTRL_NTSPCLK);
	}
}


/******************************************************************************
* MODULE   : ntshw_module_reset
* FUNCTION : NTS(H/W) reset
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.12.15 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_module_reset(int iResetFlg)
{
	if(iResetFlg == NTSHW_RESET){
		/* Reset NTSC Module */
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_RESET\n");
		gpio_direction_output(GPIO_NTSC_RESET, 0);
	}
	else{
		/* UnReset NTSC Module */
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_UNRESET\n");
		gpio_direction_output(GPIO_NTSC_RESET, 1);
	}
}


/******************************************************************************
* MODULE   : ntshw_standby
* FUNCTION : NTS(H/W) power control(standby)
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_standby(int iStandbyFlg)
{
	if(iStandbyFlg == NTSHW_STANDBY){
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_STANDBY\n");

		/* Save NTSC H/W register data */
		ntshw_save_reg();
		ntshw_start(NTSHW_STOP);
	}
	else{
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_WAKEUP\n");

		/* Restore NTSC H/W register data */
		ntshw_restore_reg();
		ntshw_start(NTSHW_START);
	}
}


/* ------------------ NTSC rgister set function -------------------------------- */
/*******************************************************************************
* MODULE   : ntshw_set_interruput
* FUNCTION : enable NTSC VSYNC interrupt
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
 UPDATE   :
********************************************************************************/
void ntshw_set_interruput(int iIntFlg)
{
	if(iIntFlg == NTSHW_INTENSET){
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_INTENSET\n");

		/* Interuppt Enable Set */
		writel(NTS_NTSVSEN_BIT, ntshw->NTSCMmioV + NTS_INTENSET);
	}
	else if(iIntFlg == NTSHW_INTENCLR){
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_INTENCLR\n");

		/* Interrupt Enable Clear */
		writel(NTS_INTENCLR_ALL_BIT, ntshw->NTSCMmioV + NTS_INTENCLR);
	}
	else{
		printk_dbg((_DEBUG_NTSHW & 0x01), "NTSHW_INTFFCLR\n");

		/* Interrupt Status Clear */
		writel(NTS_INTFFCLR_ALL_BIT, ntshw->NTSCMmioV + NTS_INTFFCLR);
	}
}


/******************************************************************************
* MODULE   : ntshw_set_framesel
* FUNCTION : change NTSC display buffer
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
int ntshw_set_framesel(int iSetFrameNo)
{
	int iRet;
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	switch (iSetFrameNo) {
	case NTS_AREASEL_BUFA:		/* frame change -> A */ /* FALL THROUGH */
	case NTS_AREASEL_BUFB:		/* frame change -> B */ /* FALL THROUGH */
	case NTS_AREASEL_BUFC:		/* frame change -> C */
		writel(iSetFrameNo, ntshw->NTSCMmioV + NTS_FRAMESEL);
		iRet = 0;
		break;
	case NTS_AREASEL_BUFDISABLE:	/* FALL THROUGH */
	default:
		iRet = -EINVAL;
		break;
	}
	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_set_ntsout
* FUNCTION : change NTSC output mode
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
int ntshw_set_ntsout(int iSetNtsoutFlg)
{
	int iRet;
	printk_dbg((_DEBUG_NTSHW & 0x01), "(%d)\n", iSetNtsoutFlg);

	switch (iSetNtsoutFlg) {
	case NTS_NTSOUT_ENABLE:		/* ntsout -> external screen */ /* FALL THROUGH */
	case NTS_NTSOUT_ENABLE_BLUE:	/* ntsout -> blue screen     */ /* FALL THROUGH */
	case NTS_NTSOUT_ENABLE_BLACK:	/* ntsout -> black screen    */ /* FALL THROUGH */
	case NTS_NTSOUT_DISABLE:	/* ntsout -> stop            */
		writel(iSetNtsoutFlg, ntshw->NTSCMmioV + NTS_OUT);
		iRet = 0;
		break;
	default:
		iRet = -EINVAL;
		break;
	}
	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_set_control_upscale
* FUNCTION : change NTS_CONTROL upscale register
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.11.13 M.Haguro
* UPDATE   :
******************************************************************************/
#ifdef NOT_USE_FUNC
static int ntshw_set_control_upscale(int iSetUpscaleFlg)
{
	int iRet;
	unsigned long ulRegVal32;

	printk_dbg((_DEBUG_NTSHW & 0x01), "(%d)\n", iSetUpscaleFlg);

	switch (iSetUpscaleFlg) {
	case NTS_UPSCALE_ON:	/* upscale -> ON  */ /* FALL THROUGH */
	case NTS_UPSCALE_OFF:	/* upscale -> OFF */
		ulRegVal32 = readl(ntshw->NTSCMmioV + NTS_CONTROL);
		ulRegVal32 = (ulRegVal32 & ~NTS_UPSCALE_BIT) | (iSetUpscaleFlg);
		writel(ulRegVal32, ntshw->NTSCMmioV + NTS_CONTROL);
		iRet = 0;
		break;
	default:
		iRet = -EINVAL;
		break;
	}
	return iRet;
}
#endif /* NOT_USE_FUNC */


/******************************************************************************
* MODULE   : ntshw_set_control_outmode
* FUNCTION : change NTS_CONTROL outmode register
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
static int ntshw_set_control_outmode(int iSetOutmodeFlg)
{
	int iRet;
	unsigned long ulRegVal32;

	printk_dbg((_DEBUG_NTSHW & 0x01), "(%d)\n", iSetOutmodeFlg);

	switch (iSetOutmodeFlg) {
	case NTS_OUTMODE_PAL:	/* outmode -> PAL  */ /* FALL THROUGH */
	case NTS_OUTMODE_NTSC:	/* outmode -> NTSC */
		ulRegVal32 = readl(ntshw->NTSCMmioV + NTS_CONTROL);
		ulRegVal32 = (ulRegVal32 & ~NTS_OUTMODE_BIT) | (iSetOutmodeFlg);
		writel(ulRegVal32, ntshw->NTSCMmioV + NTS_CONTROL);
		ntshw->Outmode = iSetOutmodeFlg ? NTS_OUTPUT_PAL : NTS_OUTPUT_NTSC;
		iRet = 0;
		break;
	default:
		ntshw->Outmode = NTS_OUTPUT_DISABLE;
		iRet = -EINVAL;
		break;
	}
	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_set_areaad
* FUNCTION : change NTS_AREAAD register
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.11.13 M.Haguro
* UPDATE   :
******************************************************************************/
static int ntshw_set_areaad(int iSetAreaFlg, unsigned long ulSetAddrY, unsigned long ulSetAddrUV)
{
	int iRet;
	printk_dbg((_DEBUG_NTSHW & 0x01), "(%d) Y(%08lx) UV(%08lx)\n", iSetAreaFlg, ulSetAddrY, ulSetAddrUV);

	switch (iSetAreaFlg) {
	case NTS_AREAAD_A:	/* framebuffer area -> A */ /* FALL THROUGH */
	case NTS_AREAAD_B:	/* framebuffer area -> B */ /* FALL THROUGH */
	case NTS_AREAAD_C:	/* framebuffer area -> C */
		writel(ulSetAddrY,  ntshw->NTSCMmioV + NTS_YAREAAD_A  + iSetAreaFlg);
		writel(ulSetAddrUV, ntshw->NTSCMmioV + NTS_UVAREAAD_A + iSetAreaFlg);
		iRet = 0;
		break;
	default:
		iRet = -EINVAL;
		break;
	}
	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_set_hoffset
* FUNCTION : change NTS_HOFFSET register
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2008.11.18 M.Haguro
* UPDATE   :
******************************************************************************/
static int ntshw_set_hoffset(unsigned long ulSetOffset)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "(%lx)\n", ulSetOffset);

	writel(ulSetOffset,  ntshw->NTSCMmioV + NTS_HOFFSET);
	return 0;
}


/******************************************************************************
* MODULE   : ntshw_set_register
* FUNCTION : register initialize & restore
* RETURN   : 
* NOTE     : none
* CREATE   : 2009.3.11 M.Haguro
* UPDATE   :
******************************************************************************/
static int ntshw_set_register(int iSetOutmode)
{
	int iRet;

	/* disable NTSC interrupts      */
	ntshw_set_interruput(NTSHW_INTFFCLR);
	ntshw_set_interruput(NTSHW_INTENCLR);

	/* set AreaAddress              */
	if(ntshw_set_areaad(NTS_AREAAD_A, ntshw->NtscFrameA_Y, ntshw->NtscFrameA_UV)){
		iRet = -EINVAL;
		goto fail_areaad_a;
	}
	if(ntshw_set_areaad(NTS_AREAAD_B, ntshw->NtscFrameB_Y, ntshw->NtscFrameB_UV)){
		iRet = -EINVAL;
		goto fail_areaad_b;
	}
	if(ntshw_set_areaad(NTS_AREAAD_C, ntshw->NtscFrameC_Y, ntshw->NtscFrameC_UV)){
		iRet = -EINVAL;
		goto fail_areaad_c;
	}

	/* set HOFFSET                  */
	if(ntshw_set_hoffset(NTS_WIDTH)){
		iRet = -EINVAL;
		goto fail_hoffset;
	}

	/* set output mode              */
	switch(iSetOutmode){
	case NTS_OUTPUT_NTSC:
		iRet = ntshw_set_control_outmode(NTS_OUTMODE_NTSC);
		break;
	case NTS_OUTPUT_PAL:
		iRet = ntshw_set_control_outmode(NTS_OUTMODE_PAL);
		break;
	default:
		iRet = -EINVAL;
		goto fail_outputmode;
	}
	goto success;

fail_outputmode:
	ntshw_set_hoffset(0);
fail_hoffset:
	ntshw_set_areaad(NTS_AREAAD_C, 0, 0);
fail_areaad_c:
	ntshw_set_areaad(NTS_AREAAD_B, 0, 0);
fail_areaad_b:
	ntshw_set_areaad(NTS_AREAAD_A, 0, 0);
fail_areaad_a:
success:
	return iRet;
}


/* ------------------ NTSC rgister check function ------------------------------ */
/******************************************************************************
* MODULE   : ntshw_chk_control
* FUNCTION : check NTSC control
* RETURN   : 
* NOTE     : none
* CREATE   : 2009.3.26 M.Haguro
* UPDATE   :
******************************************************************************/
unsigned long ntshw_chk_control(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
	/* check nts control */
	return (unsigned long)readl(ntshw->NTSCMmioV + NTS_CONTROL);
}


/******************************************************************************
* MODULE   : ntshw_chk_status
* FUNCTION : check NTSC status
* RETURN   : 
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
unsigned long ntshw_chk_status(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
	/* check nts status */
	return (unsigned long)readl(ntshw->NTSCMmioV + NTS_STATUS);
}


/*******************************************************************************
* MODULE   : ntshw_chk_framesel
* FUNCTION : check NTSC framesel
* RETURN   : 
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
********************************************************************************/
unsigned long ntshw_chk_framesel(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
	/*  check framesel */
	return (unsigned long)readl(ntshw->NTSCMmioV + NTS_FRAMESEL);
}


/*******************************************************************************
* MODULE   : ntshw_chk_intstatus
* FUNCTION : check NTSC interrupt status
* RETURN   : 
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
********************************************************************************/
unsigned long ntshw_chk_intstatus(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
	/*  check Interuppt Status */
	return (unsigned long)readl(ntshw->NTSCMmioV + NTS_INTSTATUS);
}


/*******************************************************************************
* MODULE   : ntshw_chk_intrawstatus
* FUNCTION : check NTSC interrupt RAW status
* RETURN   : 
* NOTE     : none
* CREATE   : 2008.10.09 M.Haguro
* UPDATE   :
********************************************************************************/
#ifdef NOT_USE_FUNC
static unsigned long ntshw_chk_intrawstatus(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
	/*  check Interuppt RAW Status */
	return (unsigned long)readl(ntshw->NTSCMmioV + NTS_INTRAWSTATUS);
}
#endif /* NOT_USE_FUNC */


/******************************************************************************
* MODULE   : ntshw_save_reg
* FUNCTION : save NTS(H/W) register data
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
static void ntshw_save_reg(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
}


/******************************************************************************
* MODULE   : ntshw_restore_reg
* FUNCTION : restore NTS(H/W) register data
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
static void ntshw_restore_reg(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");
	ntshw_set_register(ntshw->Outmode);
}


/* ------------------ NTSC initialize function ----------------------------------- */
/******************************************************************************
* MODULE   : ntshw_reserve
* FUNCTION : NTS(H/W) initialized
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
int ntshw_reserve(int iSetOutmode)
{
	int iRet = -1;
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	/********************************/
	/* port / terminal switching    */
	/********************************/
#if 0
//@	writel((readl(CHG_PINSEL_G00) & ~CHG_PINSEL_G00_NTS_BIT) | CHG_PINSEL_G00_NTS_INIT, CHG_PINSEL_G00);
//@	writel((readl(CHG_PINSEL_G16) & ~CHG_PINSEL_G16_NTS_BIT) | CHG_PINSEL_G16_NTS_INIT, CHG_PINSEL_G16);
//@	writel((readl(CHG_PINSEL_G32) & ~CHG_PINSEL_G32_NTS_BIT) | CHG_PINSEL_G32_NTS_INIT, CHG_PINSEL_G32);
#else
	writel((readl(CHG_PINSEL_G64) & ~CHG_PINSEL_G64_NTS_BIT) | CHG_PINSEL_G64_NTS_INIT, CHG_PINSEL_G64);
	writel((readl(CHG_PINSEL_G80) & ~CHG_PINSEL_G80_NTS_BIT) | CHG_PINSEL_G80_NTS_INIT, CHG_PINSEL_G80);
#endif
#if 0
	writel((readl(CHG_PULL_G72)   & ~CHG_PULL_G72_NTS_BIT)   | CHG_PULL_G72_NTS_INIT,   CHG_PULL_G72);
	writel((readl(CHG_PULL_G80)   & ~CHG_PULL_G80_NTS_BIT)   | CHG_PULL_G80_NTS_INIT,   CHG_PULL_G80);
#endif

	/********************************/
	/* UnReset Module               */
	/********************************/
	ntshw_module_reset(NTSHW_UNRESET);

	/********************************/
	/* UnReset Device               */
	/********************************/
	ntshw_reset(NTSHW_UNRESET);

	/********************************/
	/* Set NTSC register             */
	/********************************/
	iRet = ntshw_set_register(iSetOutmode);

	/********************************/
	/* clear frame buffer           */
	/********************************/
	ntshw_clr_framebuf(NTS_DISP_FRAME_A);
	ntshw_clr_framebuf(NTS_DISP_FRAME_B);

	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_release
* FUNCTION : NTS(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_release(void)
{
	/********************************/
	/* Set NTSC register             */
	/********************************/
	ntshw_set_hoffset(0);
	ntshw_set_areaad(NTS_AREAAD_A, 0, 0);
	ntshw_set_areaad(NTS_AREAAD_B, 0, 0);
	ntshw_set_areaad(NTS_AREAAD_C, 0, 0);

	/********************************/
	/* Reset Module/Device          */
	/********************************/
	ntshw_reset(NTSHW_RESET);
	ntshw_module_reset(NTSHW_RESET);
}


/******************************************************************************
* MODULE   : ntshw_clr_framebuf
* FUNCTION : NTS(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2009.1.13 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_clr_framebuf(int iClrFrameNo)
{
	struct image_data buffdata;
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	buffdata.uiFormat      = DMA_FORMAT_YUV422_SEMI_PLANAR;
	buffdata.uiX           = 0;
	buffdata.uiY           = 0;
	buffdata.uiWidth       = NTS_WIDTH;
	buffdata.uiHeight      = NTS_HEIGHT;
	buffdata.uiScreenWidth = NTS_WIDTH;
	buffdata.uiEndian      = IMG_PEL_ENDIAN_OUT_ENDI_LITTLE;

	if(iClrFrameNo == NTS_DISP_FRAME_A){
		buffdata.ulPhysAddrYRGB = ntshw->NtscFrameA_Y;
		buffdata.ulPhysAddrUV   = ntshw->NtscFrameA_UV;
	}
	else{ /* NTS_DISP_FRAME_B */
		buffdata.ulPhysAddrYRGB = ntshw->NtscFrameB_Y;
		buffdata.ulPhysAddrUV   = ntshw->NtscFrameB_UV;
	}
	buffdata.ulPhysAddrV = 0;

	/* memory clear. use IPU-GDMA */
	ntsipu_clr_framebuf(&buffdata);
}


/******************************************************************************
* MODULE   : ntshw_init_framebuf
* FUNCTION : NTS(H/W) initialized
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.11.17 M.Haguro
* UPDATE   :
******************************************************************************/
static int ntshw_init_framebuf(void)
{
	int iRet = 0;

	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	ntshw->SmemV = ioremap_nocache(ntshw->Smem, ntshw->SmemLength);
	if(!ntshw->SmemV){
		printk_wrn("cannot ioremap_nocache smem\n");
		iRet = -ENOMEM;
		goto fail_ioremap;
	}
	ntshw->NtscFrameAV = ntshw->SmemV + NTSCFRAME_A_OFFSET;
	ntshw->NtscFrameBV = ntshw->SmemV + NTSCFRAME_B_OFFSET;
	printk_dbg((_DEBUG_NTSHW), "NtscFrameA(0x%lx)  NtscFrameAV(0x%p)  NtscFrameLength (0x%lx)\n", ntshw->NtscFrameA_Y, ntshw->NtscFrameAV, ntshw->NtscFrameLength);
	printk_dbg((_DEBUG_NTSHW), "NtscFrameB(0x%lx)  NtscFrameBV(0x%p)  NtscFrameLength (0x%lx)\n", ntshw->NtscFrameB_Y, ntshw->NtscFrameBV, ntshw->NtscFrameLength);

	iRet = 0;
	goto success;

fail_ioremap:
success:
	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_exit_framebuf
* FUNCTION : NTS(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.11.17 M.Haguro
* UPDATE   :
******************************************************************************/
static void ntshw_exit_framebuf(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	ntshw_set_areaad(NTS_AREAAD_A, 0, 0);
	ntshw_set_areaad(NTS_AREAAD_B, 0, 0);
	ntshw_set_areaad(NTS_AREAAD_C, 0, 0);
	ntshw_set_hoffset(0);

	if(ntshw->NtscFrameAV) iounmap(ntshw->NtscFrameAV);
	if(ntshw->NtscFrameBV) iounmap(ntshw->NtscFrameBV);
	if(ntshw->SmemV)       iounmap(ntshw->SmemV);

	ntshw->NtscFrameAV = NULL;
	ntshw->NtscFrameBV = NULL;
	ntshw->SmemV     = NULL;
}


/******************************************************************************
* MODULE   : ntshw_init_val
* FUNCTION : NTS(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.11.17 M.Haguro
* UPDATE   :
******************************************************************************/
static void *ntshw_init_val(void)
{
	void *alloc_val;

	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	alloc_val = kmalloc(sizeof(struct mp200_ntshw), GFP_KERNEL);
	if(!alloc_val){
		;
	}
	else{
		memset(alloc_val, 0, sizeof(struct mp200_ntshw));

		/* NTSC MMIO */
		((struct mp200_ntshw *)alloc_val)->NTSCMmio        = NTSC_BASE;
		((struct mp200_ntshw *)alloc_val)->NTSCMmioLength  = NTSC_SIZE;
		((struct mp200_ntshw *)alloc_val)->NTSCMmioV       = NULL;

		/* NTSC framebuffer */
		((struct mp200_ntshw *)alloc_val)->NtscFrameA_Y    = SMEM_START + NTSCFRAME_A_OFFSET;
		((struct mp200_ntshw *)alloc_val)->NtscFrameA_UV   = ((struct mp200_ntshw *)alloc_val)->NtscFrameA_Y + NTS_WIDTH * NTS_HEIGHT;
		((struct mp200_ntshw *)alloc_val)->NtscFrameB_Y    = SMEM_START + NTSCFRAME_B_OFFSET;
		((struct mp200_ntshw *)alloc_val)->NtscFrameB_UV   = ((struct mp200_ntshw *)alloc_val)->NtscFrameB_Y + NTS_WIDTH * NTS_HEIGHT;
		((struct mp200_ntshw *)alloc_val)->NtscFrameC_Y    = 0l;
		((struct mp200_ntshw *)alloc_val)->NtscFrameC_UV   = 0l;
		((struct mp200_ntshw *)alloc_val)->NtscFrameLength = NTSCFRAME_LENGTH;
		((struct mp200_ntshw *)alloc_val)->NtscFrameBV     = NULL;
		((struct mp200_ntshw *)alloc_val)->NtscFrameAV     = NULL;

		/* Smem (common memory to LCD) */
		((struct mp200_ntshw *)alloc_val)->Smem            = SMEM_START;
		((struct mp200_ntshw *)alloc_val)->SmemLength      = SMEM_LENGTH;
		((struct mp200_ntshw *)alloc_val)->SmemV           = NULL;
	}
	return alloc_val;
}


/******************************************************************************
* MODULE   : ntshw_init
* FUNCTION : NTS(H/W) initialized
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2008.10.29 M.Haguro
* UPDATE   :
******************************************************************************/
int ntshw_init(void)
{
	int iRet = 0;

	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	ntshw = (struct mp200_ntshw *)ntshw_init_val();
	if(!ntshw){
		printk_wrn("failed ntshw_init_val\n");
		goto fail_alloc_val;
	}

	/* reserve NTSMmio */
#ifdef IO_ADDRESS
	ntshw->NTSCMmioV = (char *)IO_ADDRESS(ntshw->NTSCMmio);
#else
	if (!request_mem_region(NTSCMmio, NTSCMmioLength, DEV_NAME)) {
		printk_wrn("cannnot request NTSMmio\n");
		goto fail_mmio_reserve;
	}
	ntshw->NTSCMmioV = ioremap_nocache(NTSCMmio, NTSCMmioLength);
	if (!ntshw->NTSCMmioV) {
		printk_wrn("cannot ioremap_nocache NTSMmio\n");
		goto fail_mmio_ioremap;
	}
#endif /* IO_ADDRESS */
	printk_dbg((_DEBUG_NTSHW),  "NTSCMmio(0x%08lx)  NTSCMmioV(0x%p)  NTSCMmioLength(0x%lx)\n", ntshw->NTSCMmio, ntshw->NTSCMmioV, ntshw->NTSCMmioLength);

	/* init framebuffer */
	if(ntshw_init_framebuf()){
		printk_wrn("failed ntshw_init_framebuf\n");
		goto fail_init_framebuf;
	}
	goto success;

fail_init_framebuf:
#ifndef IO_ADDRESS
	iounmap(ntshw->NTSCMmioV);
fail_mmio_ioremap:
	release_mem_region(ntshw->NTSCMmio, ntshw->NTSCMmioLength);
fail_mmio_reserve:
#endif /* IO_ADDRESS */
	if(ntshw) kfree(ntshw);
fail_alloc_val:
	iRet = -ENOMEM;

success:
	return iRet;
}


/******************************************************************************
* MODULE   : ntshw_exit
* FUNCTION : NTS(H/W) initialized
* RETURN   : none
* NOTE     : none
* CREATE   : 2008.11.13 M.Haguro
* UPDATE   :
******************************************************************************/
void ntshw_exit(void)
{
	printk_dbg((_DEBUG_NTSHW & 0x01), "\n");

	/* disable NTS_OUT */
	ntshw_start(NTSHW_STOP);

	/* disable NTSC interrupts */
	ntshw_set_interruput(NTSHW_INTFFCLR);
	ntshw_set_interruput(NTSHW_INTENCLR);

	/* Reset Module/Device */
	ntshw_reset(NTSHW_RESET);
	ntshw_module_reset(NTSHW_RESET);

	if(ntshw){
		ntshw_exit_framebuf();

#ifndef IO_ADDRESS
		iounmap(ntshw->NTSCMmioV);
		release_mem_region(ntshw->NTSCMmio, ntshw->NTSCMmioLength);
#endif /* IO_ADDRESS */

		memset(ntshw, 0, sizeof(struct mp200_ntshw));
		kfree(ntshw);
	}
	ntshw = NULL;
}


