/* 2009-08-24: File added by Sony Corporation */
/*
 *  Backlight Driver Support for PGPF
 *
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/backlight.h>
#include <linux/fb.h>
#include <asm/arch/pgpf/blhw.h>
#ifdef CONFIG_PGPF_NAVU2
#include <asm/arch/pmic_light.h>
#include <asm/arch/navufb.h>
#endif /* CONFIG_PGPF_NAVU2 */

#define DEV_NAME "pgpf_blhw"

#define _DEBUG_BL  0x00

#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_BL
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif

#ifdef CONFIG_PGPF_NAVU2
#define TPLED_DUTYCYCLE 0x03
#endif /* CONFIG_PGPF_NAVU2 */

struct pgpf_bl_device* pgpf_bl_dev = NULL;
EXPORT_SYMBOL(pgpf_bl_dev);

int pgpf_bl_init(struct fb_info* info)
{
	int ret = -1;

	pgpf_bl_dev->suspend_flag = 1;
	printk_dbg(_DEBUG_BL, "pgpf_bl_init() called\n");
	if (pgpf_bl_dev && pgpf_bl_dev->init) {
		ret = pgpf_bl_dev->init(info);
	}
#ifdef CONFIG_PGPF_NAVU2
	pgpf_bl_dev->tpled_mode = SONY_NAVU_TPLED_AUTO;
#endif

	return ret;
}
EXPORT_SYMBOL(pgpf_bl_init);

void pgpf_bl_exit(struct fb_info* info)
{
	printk_dbg(_DEBUG_BL, "pgpf_bl_exit() called\n");

	if (pgpf_bl_dev && pgpf_bl_dev->exit) {
		pgpf_bl_dev->exit(info);
	}
}
EXPORT_SYMBOL(pgpf_bl_exit);

static int pgpf_bl_probe(struct platform_device *pdev)
{
	printk_dbg(_DEBUG_BL, "pgpf_bl_probe() called\n");
	pgpf_bl_register(&pgpf_bl_dev, pdev);
	return 0;
}

static struct platform_driver pgpf_bl_driver = {
	.probe		= pgpf_bl_probe,
	.driver		= {
		.name	= "pgpf_bl",
	},
};

static int __init pgpf_bl_module_init(void)
{
	printk_dbg(_DEBUG_BL, "pgpf_bl_module_init() called\n");
	return platform_driver_register(&pgpf_bl_driver);
}

static void __exit pgpf_bl_module_exit(void)
{
	printk_dbg(_DEBUG_BL, "pgpf_bl_module_exit() called\n");
	platform_driver_unregister(&pgpf_bl_driver);
	pgpf_bl_unregister();
}

int __weak pgpf_bl_register(struct pgpf_bl_device** bl_dev, struct platform_device *pf_dev)
{
	printk_err("pgpf_bl_register() called, but must not call base method!\n");
	return -1;
}
EXPORT_SYMBOL(pgpf_bl_register);

int __weak pgpf_bl_unregister(void)
{
	printk_dbg(_DEBUG_BL, "pgpf_bl_unregister() called\n");
	return 0;
}
EXPORT_SYMBOL(pgpf_bl_unregister);

int pgpf_bl_get_brightness(struct backlight_device* bd)
{
	int ret = -1;
	printk_dbg(_DEBUG_BL, "pgpf_bl_get_brightness() called\n");
	if (pgpf_bl_dev && pgpf_bl_dev->get_brightness) {
		ret = pgpf_bl_dev->get_brightness(bd);
	}
	return ret;
}
EXPORT_SYMBOL(pgpf_bl_get_brightness);

#ifdef CONFIG_PGPF_NAVU2
inline void pgpf_bl_set_tpled(int enable) {
	int dutycycle;
	printk_dbg(_DEBUG_BL, "pgpf_bl_set_tpled() called\n");
	if (enable) {
		dutycycle = TPLED_DUTYCYCLE;
	} else {
		dutycycle = 0x00;
	}
	pmic_tcled_fun_set_dutycycle(TCLED_FUN_BANK1, TCLED_FUN_CHANNEL2, dutycycle);
	pmic_tcled_fun_set_dutycycle(TCLED_FUN_BANK1, TCLED_FUN_CHANNEL3, dutycycle);

}
#endif /* CONFIG_PGPF_NAVU2 */

int pgpf_bl_update_status(struct backlight_device* bd)
{
	int ret = -1;
	printk_dbg(_DEBUG_BL, "pgpf_bl_update_status() called\n");
	printk_dbg(_DEBUG_BL, "suspend_flag = %d\n", pgpf_bl_dev->suspend_flag);
	if (pgpf_bl_dev && pgpf_bl_dev->update_status && !pgpf_bl_dev->suspend_flag) {
		ret = pgpf_bl_dev->update_status(bd);
#ifdef CONFIG_PGPF_NAVU2
		if (ret == 0) {
			pmic_bklit_tcled_master_enable();
			switch (pgpf_bl_dev->tpled_mode) {
			case SONY_NAVU_TPLED_OFF:
				pgpf_bl_set_tpled(0);
				break;
			case SONY_NAVU_TPLED_AUTO:
				if (bd->props.brightness == 0 ||
					bd->props.power != FB_BLANK_UNBLANK ||
					bd->props.fb_blank != FB_BLANK_UNBLANK) {
					pgpf_bl_set_tpled(0);
				} else {
					pgpf_bl_set_tpled(1);
				}
				break;
			case SONY_NAVU_TPLED_ON:
				pgpf_bl_set_tpled(1);
				break;
			}
		}
#endif /* CONFIG_PGPF_NAVU2 */
	}
	return ret;
}
EXPORT_SYMBOL(pgpf_bl_update_status);

#ifdef CONFIG_PGPF_NAVU2
void pgpf_bl_set_tpled_mode(int mode)
{
	pgpf_bl_dev->tpled_mode = mode;
	pgpf_bl_update_status(pgpf_bl_dev->bd);
}
EXPORT_SYMBOL(pgpf_bl_set_tpled_mode);
#endif /* CONFIG_PGPF_NAVU2 */

struct backlight_ops pgpf_bl_ops = {
	.get_brightness = pgpf_bl_get_brightness,
	.update_status  = pgpf_bl_update_status,
};
EXPORT_SYMBOL(pgpf_bl_ops);

subsys_initcall(pgpf_bl_module_init);
module_exit(pgpf_bl_module_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("Backlight Driver Support for PGPF");

