/* 2009-08-24: File added by Sony Corporation */
/*
 *  A050FW02 LCD Support for PGPF
 *
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>

#include <asm/arch/hardware.h>
#include <asm/arch/smu.h>
#include <asm/arch/spi.h>

#include "lcdc.h"
#include "mp200_lcdhw.h"
#include <asm/arch/pgpf/lcdhw.h>

#if defined(CONFIG_PLL3_238MHZ)
#define LCD_LCLK_DIV 24
#define LCD_A050FW02_DIVLCDLCLK (SMU_PLLSEL_PLL3 | SMU_DIV24)
#elif defined(CONFIG_PLL3_230MHZ)
#define LCD_LCLK_DIV 24
#define LCD_A050FW02_DIVLCDLCLK (SMU_PLLSEL_PLL3 | SMU_DIV24)
#endif /* CONFIG_PLL3_* */
#define LCD_A050FW02_PIXCLOCK (1.0 / (PGPF_PLL3_CLK / LCD_LCLK_DIV) * 1000000 * 1000000)

#define LCD_A050FW02_CONTROL_BASE (LCD_CLKPOL_NEG | LCD_HPOL_NEG | LCD_VPOL_NEG | LCD_ENPOL_HI)
#if defined(CONFIG_MP200_LCD_RGB666)
#define LCD_A050FW02_CONTROL (LCD_OFORMAT_RGB666 | LCD_A050FW02_CONTROL_BASE)
#else /* CONFIG_MP200_LCD_RGB666 */
#define LCD_A050FW02_CONTROL (LCD_OFORMAT_RGB565 | LCD_A050FW02_CONTROL_BASE)
#endif /* CONFIG_MP200_LCD_RGB666 */

#define LCD_A050FW02_HTOTAL    (525)
#define LCD_A050FW02_HAREA_ACT (480)
#define LCD_A050FW02_HAREA     (LCD_A050FW02_HAREA_ACT)
#define LCD_A050FW02_HEDGE1    (0)
#define LCD_A050FW02_HEDGE2    (LCD_A050FW02_HTOTAL - LCD_A050FW02_HAREA)

#define LCD_A050FW02_VTOTAL    (288)
#define LCD_A050FW02_VAREA_ACT (272)
#define LCD_A050FW02_VAREA     (LCD_A050FW02_VAREA_ACT)
#define LCD_A050FW02_VEDGE1    (0)
#define LCD_A050FW02_VEDGE2    (LCD_A050FW02_VTOTAL - LCD_A050FW02_VAREA)

#define _DEBUG_LCD  0x00

#define DEV_NAME "pgpf_lcd_a050fw02"

#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_LCD
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif

static int pgpf_lcdhw_a050fw02_spiw(uint8_t address, uint8_t param)
{
	int ret = -1;
	SPI_CONFIG spi_conf;
	uint16_t tx_data = ((uint16_t)address << 10) | param;

	/* @todo confirm parameters */
	spi_conf.dev = SPI_DEV_SPI1;
	spi_conf.pol = (SPI_POL_CSW_16SCLK | SPI_POL_CK0_DLY_ON | SPI_POL_CK0_POL_NEG | SPI_POL_CS0_POL_NEG);
	spi_conf.sclk = SPI_SCLK_3MHZ;
	spi_conf.mode = SPI_MODE_NB_16BIT | SPI_MODE_CS_SEL_CS0 | SPI_MODE_DMA_OFF | SPI_MODE_M_S_MASTER;
	spi_conf.size = 2;

	ret = spi_write_without_intr(&spi_conf, (const char*)&tx_data, SPI_RESET_OFF);
	if (ret) {
		printk_err("spi write error!\n");
	}

	return ret;
}

static int pgpf_lcdhw_a050fw02_resume(int restore, struct pgpf_bl_device* bl_dev)
{
	/* set to pwm */
	if (bl_dev && bl_dev->prep) {
		bl_dev->prep();
	}

	/* LCD_PWRON on */
	pgpf_set_lcd_pwron(1);

	/* wait 40ms(min) */
	mdelay(40);

	/* STB high by serial command */
	pgpf_lcdhw_a050fw02_spiw(0x01, 0x1f);
	
	/* wait 0ms(min) */
	mdelay(10);

	if (restore) {
		lcd_hw_imc_hw_init();
		lcd_controller_hw_init();
		pgpf_lcdhw_controller_hw_init();
	}

	pgpf_lcdhw_change_gio_settings(1);

	/* LCD start (PXCLK, DATA, ...) */
	lcd_hw_start();

	/* wait 10 frame */
	mdelay(153);

	/* Backlight on */
	if (bl_dev && bl_dev->resume) {
		bl_dev->resume();
	}

	return 0;
}

static int pgpf_lcdhw_a050fw02_suspend(int save, struct pgpf_bl_device* bl_dev)
{
	/* STB low by serial command */
	pgpf_lcdhw_a050fw02_spiw(0x01, 0x17);

	/* wait 1 frame */
	mdelay(16);

	/* Backlight off */
	if (bl_dev && bl_dev->suspend) {
		bl_dev->suspend();
	}

	/* wait 8 frame(min) */
	mdelay(130);

	/* LCD stop (PXCLK, DATA, ...) */
	lcd_hw_stop();

	pgpf_lcdhw_change_gio_settings(0);

	/* wait 10ms(min) */
	mdelay(10);

	/* LCD_PWRON off */
	pgpf_set_lcd_pwron(0);

	/* set to gpio */
	if (bl_dev && bl_dev->post) {
		bl_dev->post();
	}

	return 0;
}


static int pgpf_lcdhw_a050fw02_init(struct pgpf_bl_device* bl_dev)
{
	if (!(lcd_hw_chk_status() & LCD_LCDOUT_ENABLE)) {
		/* LCD Module initialize */
		pgpf_lcdhw_change_gio_settings(0);
		pgpf_lcdhw_setup_clock();
		pgpf_lcdhw_a050fw02_resume(1, bl_dev);
	} else {
		lcd_hw_imc_hw_init();
		lcd_controller_hw_init();
		pgpf_lcdhw_controller_hw_init();
	}

	return 0;
}

static void pgpf_lcdhw_a050fw02_exit(struct pgpf_bl_device* bl_dev)
{
	pgpf_lcdhw_a050fw02_suspend(0, bl_dev);
}

static struct pgpf_lcdhw_device pgpf_lcdhw_a050fw02_dev = {
	.init = pgpf_lcdhw_a050fw02_init,
	.exit = pgpf_lcdhw_a050fw02_exit,
	.suspend = pgpf_lcdhw_a050fw02_suspend,
	.resume = pgpf_lcdhw_a050fw02_resume,

	.config.xres = LCD_A050FW02_HAREA_ACT,
	.config.yres = LCD_A050FW02_VAREA_ACT,

	.config.pixclock = LCD_A050FW02_PIXCLOCK,
	.config.divlcdlclk = LCD_A050FW02_DIVLCDLCLK,

	.config.control = LCD_A050FW02_CONTROL,

	.config.htotal = LCD_A050FW02_HTOTAL,
	.config.harea  = LCD_A050FW02_HAREA,
	.config.hedge1 = LCD_A050FW02_HEDGE1,
	.config.hedge2 = LCD_A050FW02_HEDGE2,
	.config.vtotal = LCD_A050FW02_VTOTAL,
	.config.varea  = LCD_A050FW02_VAREA,
	.config.vedge1 = LCD_A050FW02_VEDGE1,
	.config.vedge2 = LCD_A050FW02_VEDGE2,
};

int pgpf_lcdhw_register(struct pgpf_lcdhw_device** lcd_dev, struct platform_device* pf_dev)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_register() called\n");
	*lcd_dev = &pgpf_lcdhw_a050fw02_dev;
	return 0;
}

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("A050FW02 LCD Support for PGPF");
