/* 2009-08-24: File added and changed by Sony Corporation */
/*
 *  LB070WV6 LCD Support for PGPF
 *
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>

#include <asm/arch/hardware.h>
#include <asm/arch/smu.h>

#include "lcdc.h"
#include "mp200_lcdhw.h"
#include <asm/arch/pgpf/lcdhw.h>

#ifdef CONFIG_PGPF_LCD_LB070WV6_PIXCLOCK_WORKAROUND
/* use PLL2 for pixclock */
#define LCD_LCLK_DIV 14
#define SMU_DIV14_PLL2 0x61
#define LCD_LB070WV6_DIVLCDLCLK (SMU_PLLSEL_PLL2 | SMU_DIV14_PLL2)
#define PLL2_N_VAL (0x70)
#define PGPF_PLL2_CLK (32.768 * 1000 * 125 * (PLL2_N_VAL + 1))
#define LCD_LB070WV6_PIXCLOCK (1.0 / (PGPF_PLL2_CLK / LCD_LCLK_DIV) * 1000000 * 1000000)
#else /* CONFIG_PGPF_LCD_LB070WV6_PIXCLOCK_WORKAROUND */
/* Notice: the following clock configuration is a little out of specification! */
#if defined(CONFIG_PLL3_238MHZ)
#define LCD_LCLK_DIV 8
#define LCD_LB070WV6_DIVLCDLCLK (SMU_PLLSEL_PLL3 | SMU_DIV8)
#elif defined(CONFIG_PLL3_230MHZ)
#define LCD_LCLK_DIV 8
#define LCD_LB070WV6_DIVLCDLCLK (SMU_PLLSEL_PLL3 | SMU_DIV8)
#endif /* CONFIG_PLL3_* */
#define LCD_LB070WV6_PIXCLOCK (1.0 / (PGPF_PLL3_CLK / LCD_LCLK_DIV) * 1000000 * 1000000)
#endif /* CONFIG_PGPF_LCD_LB070WV6_PIXCLOCK_WORKAROUND */


#define LCD_LB070WV6_CONTROL_BASE (LCD_CLKPOL_NEG | LCD_HPOL_POS | LCD_VPOL_POS | LCD_ENPOL_HI)
#if defined(CONFIG_MP200_LCD_RGB666)
#define LCD_LB070WV6_CONTROL (LCD_OFORMAT_RGB666 | LCD_LB070WV6_CONTROL_BASE)
#else /* CONFIG_MP200_LCD_RGB666 */
#define LCD_LB070WV6_CONTROL (LCD_OFORMAT_RGB565 | LCD_LB070WV6_CONTROL_BASE)
#endif /* CONFIG_MP200_LCD_RGB666 */

#define LCD_LB070WV6_HPULSE      64
#define LCD_LB070WV6_HFRONTP     64
#define LCD_LB070WV6_HBACKP      128
#define LCD_LB070WV6_VPULSE      3
#define LCD_LB070WV6_VFRONTP     20
#define LCD_LB070WV6_VBACKP      22
#define LCD_LB070WV6_HAREA_VAL   800
#define LCD_LB070WV6_VAREA_VAL   480

#define LCD_LB070WV6_HBLANK_VAL (LCD_LB070WV6_HFRONTP + LCD_LB070WV6_HPULSE + LCD_LB070WV6_HBACKP)
#define LCD_LB070WV6_VBLANK_VAL (LCD_LB070WV6_VFRONTP + LCD_LB070WV6_VPULSE + LCD_LB070WV6_VBACKP)

#define LCD_LB070WV6_HTOTAL (LCD_LB070WV6_HAREA_VAL + LCD_LB070WV6_HBLANK_VAL)
#define LCD_LB070WV6_HAREA  (LCD_LB070WV6_HAREA_VAL)
#define LCD_LB070WV6_HEDGE1 (LCD_LB070WV6_HFRONTP)
#define LCD_LB070WV6_HEDGE2 (LCD_LB070WV6_HFRONTP + LCD_LB070WV6_HPULSE)
#define LCD_LB070WV6_VTOTAL (LCD_LB070WV6_VAREA_VAL + LCD_LB070WV6_VBLANK_VAL)
#define LCD_LB070WV6_VAREA  (LCD_LB070WV6_VAREA_VAL)
#define LCD_LB070WV6_VEDGE1 (LCD_LB070WV6_VFRONTP)
#define LCD_LB070WV6_VEDGE2 (LCD_LB070WV6_VFRONTP + LCD_LB070WV6_VPULSE)

#define _DEBUG_LCD  0x00

#define DEV_NAME "pgpf_lcd_lb070wv6"

#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_LCD
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif

static int pgpf_lcdhw_lb070wv6_resume(int restore, struct pgpf_bl_device* bl_dev)
{
	/* set to pwm */
	if (bl_dev && bl_dev->prep) {
		bl_dev->prep();
	}

	/* LCD_PWRON on */
	pgpf_set_lcd_pwron(1);

	/* wait Vcc on */
	mdelay(10);

	if (restore) {
		lcd_hw_imc_hw_init();
		lcd_controller_hw_init();
		pgpf_lcdhw_controller_hw_init();
	}

	pgpf_lcdhw_change_gio_settings(1);

	/* LCD start (PXCLK, DATA, ...) */
	lcd_hw_start();

	/* wait 20ms(min) */
	mdelay(20);

	/* LCD_PON on */
	pgpf_set_lcd_pon(1);

	/* wait 200ms(min) from LCD IF Signal */
	mdelay(180);

	/* Backlight on */
	if (bl_dev && bl_dev->resume) {
		bl_dev->resume();
	}

	return 0;
}

static int pgpf_lcdhw_lb070wv6_suspend(int save, struct pgpf_bl_device* bl_dev)
{
	/* Backlight off */
	if (bl_dev && bl_dev->suspend) {
		bl_dev->suspend();
	}

	/* wait 200ms(min) for IF Signale off */
	mdelay(180);

	/* LCD_PON off */
	pgpf_set_lcd_pon(0);

	/* wait 20ms(min) */
	mdelay(20);

	/* LCD stop (PXCLK, DATA, ...) */
	lcd_hw_stop();

	pgpf_lcdhw_change_gio_settings(0);

	/* wait 50ms(max) */

	/* LCD_PWRON off */
	pgpf_set_lcd_pwron(0);

	/* set to gpio */
	if (bl_dev && bl_dev->post) {
		bl_dev->post();
	}

	/* for next resume */
	mdelay(400);

	return 0;
}


static int pgpf_lcdhw_lb070wv6_init(struct pgpf_bl_device* bl_dev)
{
	if (!(lcd_hw_chk_status() & LCD_LCDOUT_ENABLE)) {
		/* LCD Module initialize */
		pgpf_lcdhw_change_gio_settings(0);
#ifdef CONFIG_PGPF_LCD_LB070WV6_PIXCLOCK_WORKAROUND
		writel(PLL2_N_VAL, SMU_PLL2CTRL0);
		writel(0x00, SMU_PLL2CTRL1);
#endif /* CONFIG_PGPF_LCD_LB070WV6_PIXCLOCK_WORKAROUND */
		pgpf_lcdhw_setup_clock();
		pgpf_lcdhw_lb070wv6_resume(1, bl_dev);
	} else {
		lcd_hw_imc_hw_init();
		lcd_controller_hw_init();
		pgpf_lcdhw_controller_hw_init();
	}

	return 0;
}

static void pgpf_lcdhw_lb070wv6_exit(struct pgpf_bl_device* bl_dev)
{
	pgpf_lcdhw_lb070wv6_suspend(0, bl_dev);
}

static struct pgpf_lcdhw_device pgpf_lcdhw_lb070wv6_dev = {
	.init = pgpf_lcdhw_lb070wv6_init,
	.exit = pgpf_lcdhw_lb070wv6_exit,
	.suspend = pgpf_lcdhw_lb070wv6_suspend,
	.resume = pgpf_lcdhw_lb070wv6_resume,

	.config.xres = LCD_LB070WV6_HAREA_VAL,
	.config.yres = LCD_LB070WV6_VAREA_VAL,

	.config.pixclock = LCD_LB070WV6_PIXCLOCK,
	.config.divlcdlclk = LCD_LB070WV6_DIVLCDLCLK,

	.config.control = LCD_LB070WV6_CONTROL,

	.config.htotal = LCD_LB070WV6_HTOTAL,
	.config.harea  = LCD_LB070WV6_HAREA,
	.config.hedge1 = LCD_LB070WV6_HEDGE1,
	.config.hedge2 = LCD_LB070WV6_HEDGE2,
	.config.vtotal = LCD_LB070WV6_VTOTAL,
	.config.varea  = LCD_LB070WV6_VAREA,
	.config.vedge1 = LCD_LB070WV6_VEDGE1,
	.config.vedge2 = LCD_LB070WV6_VEDGE2,
};

int pgpf_lcdhw_register(struct pgpf_lcdhw_device** lcd_dev, struct platform_device* pf_dev)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_register() called\n");
	*lcd_dev = &pgpf_lcdhw_lb070wv6_dev;
	return 0;
}

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("LB070WV6 LCD Support for PGPF");
