/* 2009-08-24: File added by Sony Corporation */
/*
 *  LCD Support for PGPF
 *
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/platform_device.h>
#include <linux/interrupt.h>

#include <asm/arch/pmu.h>
#include <asm/arch/smu.h>
#include <asm/arch/gpio.h>

#include <asm/arch/pgpf/lcdhw.h>
#include <asm/arch/pgpf/blhw.h>
#include "mp200_common.h"
#include "mp200_lcd_common.h"
#include "mp200_lcd.h"
#include "mp200_lcdhw.h"
#include "lcdc.h"

#define DEV_NAME "pgpf_lcdhw"

#define G48_LCD_PINSEL_MASK (0xfffffff0)
#define G48_LCD_PINSEL_LCD  (0x55555550)
#define G48_LCD_PINSEL_GIO  (0x00000000)
#define G64_LCD_PINSEL_MASK (0x0000ffff)
#define G64_LCD_PINSEL_LCD  (0x00005555)
#define G64_LCD_PINSEL_GIO  (0x00000000)

#define _DEBUG_LCD  0x00

#define printk_err(fmt, arg...) \
	do{                     \
		printk(KERN_ERR DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_wrn(fmt, arg...) \
	do{                     \
		printk(KERN_WARNING DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)

#define printk_info(fmt, arg...) \
	do{                      \
		printk(KERN_INFO DEV_NAME ": " fmt, ## arg); \
	}while(0)

#if _DEBUG_LCD
#define printk_dbg(level, fmt, arg...) \
	do{                            \
		if(level > 0) printk(KERN_DEBUG DEV_NAME ": %s: " fmt, __func__, ## arg); \
	}while(0)
#else
#define printk_dbg(level, fmt, arg...) \
	;
#endif

struct pgpf_lcdhw_device* pgpf_lcdhw_dev = NULL;
EXPORT_SYMBOL(pgpf_lcdhw_dev);

#ifdef CONFIG_PGPF_BACKLIGHT
extern struct pgpf_bl_device* pgpf_bl_dev;
#else
struct pgpf_bl_device* pgpf_bl_dev;
#endif /* CONFIG_PGPF_BACKLIGHT */

int pgpf_lcdhw_init(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_init() called\n");
	if (pgpf_lcdhw_dev && pgpf_lcdhw_dev->init) {
		pgpf_lcdhw_dev->status = PGPF_LCD_STATUS_INIT;
		if (pgpf_lcdhw_dev->init(pgpf_bl_dev)) {
			return LCD_INIT_ERROR__LCDM_INIT;
		}
	}
	pgpf_lcdhw_dev->status = PGPF_LCD_STATUS_ON;
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_init);

int pgpf_lcdhw_exit(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_exit() called\n");
	if (pgpf_lcdhw_dev && pgpf_lcdhw_dev->exit) {
		pgpf_lcdhw_dev->exit(pgpf_bl_dev);
	}
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_exit);

int pgpf_lcdhw_suspend(int save)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_suspend() called\n");
	if (pgpf_lcdhw_dev && pgpf_lcdhw_dev->suspend) {
		pgpf_lcdhw_dev->suspend(save, pgpf_bl_dev);
	}
	pgpf_lcdhw_dev->status = PGPF_LCD_STATUS_OFF;
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_suspend);

int pgpf_lcdhw_resume(int restore)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_resume() called\n");
	if (pgpf_lcdhw_dev && pgpf_lcdhw_dev->resume) {
		pgpf_lcdhw_dev->resume(restore, pgpf_bl_dev);
	}
	pgpf_lcdhw_dev->status = PGPF_LCD_STATUS_ON;
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_resume);

int pgpf_lcdhw_get_status(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_get_status() called\n");
	if (pgpf_lcdhw_dev) {
		return pgpf_lcdhw_dev->status;
	} else {
		return PGPF_LCD_STATUS_INIT;
	}
}
EXPORT_SYMBOL(pgpf_lcdhw_get_status);

int pgpf_lcdhw_change_gio_settings(int lcdmode)
{
	int port;
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_change_gio_settings() called\n");

	if (lcdmode) {
		outl((inl(CHG_PINSEL_G48) & ~G48_LCD_PINSEL_MASK) | G48_LCD_PINSEL_LCD, CHG_PINSEL_G48);
		outl((inl(CHG_PINSEL_G64) & ~G64_LCD_PINSEL_MASK) | G64_LCD_PINSEL_LCD, CHG_PINSEL_G64);
	} else {
		for (port = GPIO_P50; port <= GPIO_P71; port++) {
			gpio_direction_output(port, 0);
		}
		outl((inl(CHG_PINSEL_G48) & ~G48_LCD_PINSEL_MASK) | G48_LCD_PINSEL_GIO, CHG_PINSEL_G48);
		outl((inl(CHG_PINSEL_G64) & ~G64_LCD_PINSEL_MASK) | G64_LCD_PINSEL_GIO, CHG_PINSEL_G64);
	}
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_change_gio_settings);

int pgpf_lcdhw_setup_clock(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_setup_clock() called\n");
	/********************************/
	/* port / terminal switching    */
	/********************************/
	/* move to pgpf_lcdhw_change_gio_settings */
/*
	writel((readl(CHG_PINSEL_G48) & ~CHG_PINSEL_G48_LCD_BIT) | CHG_PINSEL_G48_LCD_INIT, CHG_PINSEL_G48);
	writel((readl(CHG_PINSEL_G64) & ~CHG_PINSEL_G64_LCD_BIT) | CHG_PINSEL_G64_LCD_INIT, CHG_PINSEL_G64);
*/

	/********************************/
	/* clock supply                 */
	/********************************/
	if (pgpf_lcdhw_dev) {
		writel(pgpf_lcdhw_dev->config.divlcdlclk, SMU_DIVLCDLCLK);
	}

	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CCLK);        /* LCD_CCLK       -> ON */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_LCLK);        /* LCD_LCLK       -> ON */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_PCLK);        /* LCD_PCLK       -> ON */
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CLK);         /* LCD_CLK        -> ON */

	/********************************/
	/* clock auto control -> OFF    */
	/********************************/
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);

	/********************************/
	/* LCDC unreset                 */
	/********************************/
	mp200_pmu_unreset_device(MP200_RESETDEVICE_LCD);

	/********************************/
	/* clock auto control -> ON     */
	/********************************/
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDCCK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDPCLK);

	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_setup_clock);

int pgpf_lcdhw_controller_hw_init(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_controller_hw_init() called\n");
	if (pgpf_lcdhw_dev) {
		writel(pgpf_lcdhw_dev->config.control, LCDCMmioV + LCD_CONTROL);  /* Control for LCDC  */
		
		writel(pgpf_lcdhw_dev->config.htotal,  LCDCMmioV + LCD_HTOTAL);   /* Horizontal Synchronizing Signal Total Width  */
		writel(pgpf_lcdhw_dev->config.harea,   LCDCMmioV + LCD_HAREA);    /* Horizontal Display Area Width                */
		writel(pgpf_lcdhw_dev->config.hedge1,  LCDCMmioV + LCD_HEDGE1);   /* Horizontal Synchronizing Signal Edge Point 1 */
		writel(pgpf_lcdhw_dev->config.hedge2,  LCDCMmioV + LCD_HEDGE2);   /* Horizontal Synchronizing Signal Edge Point 2 */
		writel(pgpf_lcdhw_dev->config.vtotal,  LCDCMmioV + LCD_VTOTAL);   /* Vertical Synchronizing Signal Total Width    */
		writel(pgpf_lcdhw_dev->config.varea,   LCDCMmioV + LCD_VAREA);    /* Vertical Display Area Width                  */
		writel(pgpf_lcdhw_dev->config.vedge1,  LCDCMmioV + LCD_VEDGE1);   /* Vertical Synchronizing Signal Edge Point 1   */
		writel(pgpf_lcdhw_dev->config.vedge2,  LCDCMmioV + LCD_VEDGE2);   /* Vertical Synchronizing Signal Edge Point 2   */
	}
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_controller_hw_init);

int pgpf_lcdhw_set_vscreeninfo(struct fb_var_screeninfo* vinfo)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_set_vscreeninfo() called\n");
	if (pgpf_lcdhw_dev) {
		vinfo->xres = pgpf_lcdhw_dev->config.xres;
		vinfo->yres = pgpf_lcdhw_dev->config.yres;
		vinfo->pixclock = pgpf_lcdhw_dev->config.pixclock;
		vinfo->left_margin = pgpf_lcdhw_dev->config.htotal - pgpf_lcdhw_dev->config.harea - pgpf_lcdhw_dev->config.hedge2;
		vinfo->hsync_len = pgpf_lcdhw_dev->config.hedge2 - pgpf_lcdhw_dev->config.hedge1;
		vinfo->right_margin = pgpf_lcdhw_dev->config.hedge1;
		vinfo->upper_margin = pgpf_lcdhw_dev->config.vtotal - pgpf_lcdhw_dev->config.varea - pgpf_lcdhw_dev->config.vedge2;
		vinfo->vsync_len = pgpf_lcdhw_dev->config.vedge2 - pgpf_lcdhw_dev->config.vedge1;
		vinfo->lower_margin = pgpf_lcdhw_dev->config.vedge1;
	}
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_set_vscreeninfo);

int __weak pgpf_lcdhw_register(struct pgpf_lcdhw_device** lcd_dev, struct platform_device* pf_dev) {
	printk_err("pgpf_lcdhw_register() called, but must not call base method!\n");
	return -1;
}
EXPORT_SYMBOL(pgpf_lcdhw_register);

int __weak pgpf_lcdhw_unregister(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcdhw_unregister() called\n");
	return 0;
}
EXPORT_SYMBOL(pgpf_lcdhw_unregister);

static int pgpf_lcd_probe(struct platform_device* pf_dev)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcd_probe() called\n");
	pgpf_lcdhw_register(&pgpf_lcdhw_dev, pf_dev);
	return 0;
}

static struct platform_driver pgpf_lcd_driver = {
	.probe		= pgpf_lcd_probe,
	.driver		= {
		.name	= "pgpf_lcd",
	},
};

static int __init pgpf_lcd_module_init(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcd_module_init() called\n");
	return platform_driver_register(&pgpf_lcd_driver);
}

static void __exit pgpf_lcd_module_exit(void)
{
	printk_dbg(_DEBUG_LCD, "pgpf_lcd_module_exit() called\n");
	pgpf_lcdhw_unregister();
	platform_driver_unregister(&pgpf_lcd_driver);
}

subsys_initcall(pgpf_lcd_module_init);
module_exit(pgpf_lcd_module_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("LCD Support for PGPF");
