/* 2009-08-24: File added by Sony Corporation */
/**
 ** @file  elan_tp.h
 ** @brief 
 ** @author Hiroyuki.Motousu@jp.sony.com
 **
 ** $Id:$
 **
 **/
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_ELAN_TP_H
#define __ASM_ARCH_ELAN_TP_H

typedef struct {
    int min;
    int max;
} elan_finger_level_t;

typedef struct {
    int x;
    int y;
    int button;
} elan_finger_sens_t;

typedef struct {
    int major_ver;
    int minor_ver;
    int id_high;
    int id_low;
} elan_fw_info_t;

typedef enum {
    ELAN_POWER_STATE_DEEP_SLEEP = 0,
    ELAN_POWER_STATE_NORMAL,
} elan_power_state_t;

typedef enum {
    ELAN_COORDINATE_TYPE_ABSOLUTE = 0,
    ELAN_COORDINATE_TYPE_INDEX,
} elan_coordinate_t;

typedef enum {
    ELAN_Y_AXIS_REPORT_DEVICE_FORWARD_DIRECTION = 0, /* device default */
    ELAN_Y_AXIS_REPORT_DEVICE_BACKWARD_DIRECTION,    /* driver default */
} elan_y_direction_t;

/* --------- ioctl code --------- */    
#define ELAN_MAGIC 'T'
#define XYZ5_ELAN_SET_SENSITIVITY             (_IOW(ELAN_MAGIC, 1, elan_finger_sens_t))
#define XYZ5_ELAN_GET_SENSITIVITY             (_IOR(ELAN_MAGIC, 2, elan_finger_sens_t))
#define XYZ5_ELAN_SET_FINGER_LEVEL_THRESHOLD  (_IOW(ELAN_MAGIC, 3, elan_finger_level_t))
#define XYZ5_ELAN_GET_FINGER_LEVEL_THRESHOLD  (_IOR(ELAN_MAGIC, 4, elan_finger_level_t))
#define XYZ5_ELAN_GET_FINGER_LEVEL            (_IOR(ELAN_MAGIC, 5, int))
#define XYZ5_ELAN_SET_POWER_STATE             (_IOW(ELAN_MAGIC, 6, elan_power_state_t))
#define XYZ5_ELAN_GET_POWER_STATE             (_IOR(ELAN_MAGIC, 7, elan_power_state_t))
#define XYZ5_ELAN_GET_FW_INFO                 (_IOR(ELAN_MAGIC, 8, elan_fw_info_t))
#define XYZ5_ELAN_GET_COORDINATE_TYPE         (_IOR(ELAN_MAGIC, 9, elan_coordinate_t))
#define XYZ5_ELAN_SET_COORDINATE_TYPE         (_IOW(ELAN_MAGIC,10, elan_coordinate_t))
#define XYZ5_ELAN_GET_Y_AXIS_REPORT_DIRECTION (_IOR(ELAN_MAGIC,11, elan_y_direction_t))
#define XYZ5_ELAN_SET_Y_AXIS_REPORT_DIRECTION (_IOW(ELAN_MAGIC,12, elan_y_direction_t))

/*=====================================================================
 * end of elan_tp.h
 */
#endif /* __ASM_ARCH_ELAN_TP_H */
