/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /include/asm-arm/arch-mp200/GDMA.h
*  Function        : DMA MMIO definitions, and Driver - Driver I/F of IPU driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/

#ifndef _DMA_H_
#define _DMA_H_


/*===============================================================*/
/* GDMA MMIO definitions   for uPD77601                          */
/*===============================================================*/
#include <asm/arch/hardware.h>
#define GDMA_OFFSET    MP200_IPUDMA_OFFSET
#define GDMA_BASE      MP200_IPUDMA_BASE
#define GDMA_SIZE      MP200_IPUDMA_SIZE


/*===============================================================*/
/* DMA register definitions   for uPD77601                       */
/*===============================================================*/
//                                                           R/W   Bit  Reset
// Mode for DMA                                              R/W   7:0  0x00
#define DMA_MODE                 (0x00)

// DMA Request                                                W    0:0  0x0
#define DMA_REQ                  (0x04)

// Current Processing Status                                  R    0:0  0x0
#define DMA_ACK                  (0x08)

// Status for DMA                                             R    8:0  0x0
#define DMA_STATUS               (0x0C)

// Interuppt Raw Status                                       R    8:0  0x0
#define DMA_RAWSTATUS            (0x10)

// Interuppt Enable Set                                      R/W   8:0  0x0
#define DMA_ENSET                (0x14)

// Interuppt Enable Clear                                     W    8:0  0x0
#define DMA_ENCLR                (0x18)

// Interuppt FF Clear                                         W    8:0  0x0
#define DMA_FFCLR                (0x1C)

// Address increment value for the source1 image             R/W  14:0  0x000
#define DMA_SRCSIZE_1            (0x20)

// Address increment value for the source2 image             R/W  14:0  0x000
#define DMA_SRCSIZE_2            (0x24)

// Address increment value for the destination image         R/W  14:0  0x000
#define DMA_DSTSIZE              (0x28)

// Y/RGB Plane Address for the source1 image                 R/W  31:0  0x0000_0000
#define DMA_SRCYRGBADR_1         (0x30)

// Y/RGB Plane Address for the source2 image                 R/W  31:0  0x0000_0000
#define DMA_SRCYRGBADR_2         (0x34)

// Y/RGB Plane Address for the destination image             R/W  31:0  0x0000_0000
#define DMA_DSTYRGBADR           (0x38)

// UV Plane Address for the source1 image                    R/W  31:0  0x0000_0000
#define DMA_SRCUVADR_1           (0x40)

// UV Plane Address for the source2 image                    R/W  31:0  0x0000_0000
#define DMA_SRCUVADR_2           (0x44)

// UV Plane Address for the destination image                R/W  31:0  0x0000_0000
#define DMA_DSTUVADR             (0x48)

// V Plane Address for the source1 image                     R/W  31:0  0x0000_0000
#define DMA_SRCVADR_1            (0x50)

// V Plane Address for the source2 image                     R/W  31:0  0x0000_0000
#define DMA_SRCVADR_2            (0x54)

// V Plane Address for the destination image                 R/W  31:0  0x0000_0000
#define DMA_DSTVADR              (0x58)

// Horizontal size for output image                          R/W  12:0  0x008
#define DMA_HSIZE                (0x60)

// Vartical size for output image                            R/W  12:0  0x008
#define DMA_VSIZE                (0x64)

// Mask Color                                                R/W  23  0x0000
#define DMA_MASKCOLR             (0x68)

// Painting out data                                         R/W  31:0  0x0000
#define DMA_FILLDATA             (0x6C)

// Image format                                              R/W  11:0  0x0000
#define DMA_FORMAT               (0x70)

// BYTE lane for the source1 image                           R/W   7:0  0xE4
#define DMA_SRCBYTE_1            (0x74)

// BYTE lane for the source2 image                           R/W   7:0  0xE4
#define DMA_SRCBYTE_2            (0x78)

// BYTE lane for the destination image                       R/W   7:0  0xE4
#define DMA_DSTBYTE              (0x7C)

// Register update reservation setting                       R/W   0:0  0x0000
#define DMA_DUAL_FF              (0x80)

// BYTE lane for the source1 image                           R/W  15:0  0xE4E4
#define DMA_SRCBYTE_1_CMP        (0x90)

// BYTE lane for the source2 image                           R/W  15:0  0xE4E4
#define DMA_SRCBYTE_2_CMP        (0x94)

// BYTE lane for the destination image                       R/W  15:0  0xE4E4
#define DMA_DSTBYTE_CMP          (0x98)

// Auto Scan                                                 R/W  25:0  0x0
#define DMA_AUTO_SCAN            (0xA0)


/*===============================================================*/
/* DMA register bit assigns                                      */
/*===============================================================*/
//
/*------------------------------*/
/* DMA_MODE                     */
/*------------------------------*/
#define DMA_MASKOBJ_BIT         0x00000080
#define DMA_MASKOBJ_SFT         0x7
#define DMA_MASKOBJ_SRC1        0x00000000
#define DMA_MASKOBJ_SRC2        0x00000080

#define DMA_KEY_BIT             0x00000040
#define DMA_KEY_SFT             0x6
#define DMA_KEY_MASK_OFF        0x00000000
#define DMA_KEY_MASK_ON         0x00000040

#define DMA_OP_BIT              0x00000030
#define DMA_OP_SFT              0x4
#define DMA_OP_FILL             0x00000000
#define DMA_OP_ROP              0x00000020

#define DMA_ROP_BIT             0x0000000F
#define DMA_ROP_SFT             0x0
#define DMA_ROP_ALL0            0x00000000
#define DMA_ROP_SRC1_NOR_SRC2   0x00000001
#define DMA_ROP_NSRC1_AND_SRC2  0x00000002
#define DMA_ROP_NSRC1           0x00000003
#define DMA_ROP_SRC1_AND_NSRC2  0x00000004
#define DMA_ROP_NSRC2           0x00000005
#define DMA_ROP_SRC1_XOR_SRC2   0x00000006
#define DMA_ROP_SRC1_NAND_SRC2  0x00000007
#define DMA_ROP_SRC1_AND_SRC2   0x00000008
#define DMA_ROP_SRC1_XNOR_SRC2  0x00000009
#define DMA_ROP_SRC2            0x0000000A
#define DMA_ROP_NSRC1_OR_SRC2   0x0000000B
#define DMA_ROP_SRC1            0x0000000C
#define DMA_ROP_SRC1_OR_NSRC2   0x0000000D
#define DMA_ROP_SRC1_OR_SRC2    0x0000000E
#define DMA_ROP_ALL1            0x0000000F


/*------------------------------*/
/* DMA_REQ                      */
/*------------------------------*/
#define DMA_REQ_BIT             0x00000001
#define DMA_REQ_SFT             0x0


/*------------------------------*/
/* DMA_ACK                      */
/*------------------------------*/
#define DMA_ACK_BIT             0x00000001
#define DMA_ACK_SFT             0x0


/*------------------------------*/
/* DMA_STATUS                   */
/*------------------------------*/
#define DMA_AHBERR_BIT          0x00000100
#define DMA_AHBERR_SFT          0x8

#define DMA_ACTIVE_BIT          0x00000004
#define DMA_ACTIVE_SFT          0x2

#define DMA_DFF_RSV_BIT         0x00000002
#define DMA_DFF_RSV_SFT         0x1

#define DMA_STAT_BIT            0x00000001
#define DMA_STAT_SFT            0x0


/*------------------------------*/
/* DMA_RAWSTATUS                */
/*------------------------------*/
#define DMA_RAWAHBERR_BIT       0x00000100
#define DMA_RAWAHBERR_SFT       0x8

#define DMA_RAWSTAT_BIT         0x00000001
#define DMA_RAWSTAT_SFT         0x0


/*------------------------------*/
/* DMA_ENSET                    */
/*------------------------------*/
#define DMA_ENSETAHBERR_BIT     0x00000100
#define DMA_ENSETAHBERR_SFT     0x8

#define DMA_ENSETSTAT_BIT       0x00000001
#define DMA_ENSETSTAT_SFT       0x0


/*------------------------------*/
/* DMA_ENCLR                    */
/*------------------------------*/
#define DMA_ENCLRAHBERR_BIT     0x00000100
#define DMA_ENCLRAHBERR_SFT     0x8

#define DMA_ENCLRSTAT_BIT       0x00000001
#define DMA_ENCLRSTAT_SFT       0x0


/*------------------------------*/
/* DMA_FFCLR                    */
/*------------------------------*/
#define DMA_FFCLRAHBERR_BIT     0x00000100
#define DMA_FFCLRAHBERR_SFT     0x8

#define DMA_FFCLRSTAT_BIT       0x00000001
#define DMA_FFCLRSTAT_SFT       0x0


/*------------------------------*/
/* DMA_SRCSIZE_1                */
/*------------------------------*/
#define DMA_SRCSIZE_1_BIT       0x00007FFF
#define DMA_SRCSIZE_1_SFT       0x00


/*------------------------------*/
/* DMA_SRCSIZE_2                */
/*------------------------------*/
#define DMA_SRCSIZE_2_BIT       0x00007FFF
#define DMA_SRCSIZE_2_SFT       0x00


/*------------------------------*/
/* DMA_DSTSIZE                  */
/*------------------------------*/
#define DMA_DSTSIZE_BIT         0x00007FFF
#define DMA_DSTSIZE_SFT         0x00


/*------------------------------*/
/* DMA_SRCYRGBADR_1             */
/*------------------------------*/
#define DMA_SRCYRGBADR_1_BIT    0xFFFFFFFF
#define DMA_SRCYRGBADR_1_SFT    0x00


/*------------------------------*/
/* DMA_SRCYRGBADR_2             */
/*------------------------------*/
#define DMA_SRCYRGBADR_2_BIT    0xFFFFFFFF
#define DMA_SRCYRGBADR_2_SFT    0x00


/*------------------------------*/
/* DMA_DSTYRGBADR               */
/*------------------------------*/
#define DMA_DSTYRGBADR_BIT      0xFFFFFFFF
#define DMA_DSTYRGBADR_SFT      0x00


/*------------------------------*/
/* DMA_SRCUVADR_1               */
/*------------------------------*/
#define DMA_SRCUVADR_1_BIT      0xFFFFFFFF
#define DMA_SRCUVADR_1_SFT      0x00


/*------------------------------*/
/* DMA_SRCUVADR_2               */
/*------------------------------*/
#define DMA_SRCUVADR_2_BIT      0xFFFFFFFF
#define DMA_SRCUVADR_2_SFT      0x00


/*------------------------------*/
/* DMA_DSTUVADR                 */
/*------------------------------*/
#define DMA_DSTUVADR_BIT        0xFFFFFFFF
#define DMA_DSTUVADR_SFT        0x00


/*------------------------------*/
/* DMA_SRCVADR_1                */
/*------------------------------*/
#define DMA_SRCVBADR_1_BIT      0xFFFFFFFF
#define DMA_SRCVADR_1_SFT       0x00


/*------------------------------*/
/* DMA_SRCVADR_2                */
/*------------------------------*/
#define DMA_SRCVADR_2_BIT       0xFFFFFFFF
#define DMA_SRCVADR_2_SFT       0x00


/*------------------------------*/
/* DMA_DSTVRGBADR               */
/*------------------------------*/
#define DMA_DSTVADR_BIT         0xFFFFFFFF
#define DMA_DSTVADR_SFT         0x00


/*------------------------------*/
/* DMA_HSIZE                    */
/*------------------------------*/
#define DMA_HSIZE_BIT           0x00001FFF
#define DMA_HSIZE_SFT           0x00


/*------------------------------*/
/* DMA_VSIZE                    */
/*------------------------------*/
#define DMA_VSIZE_BIT           0x00001FFF
#define DMA_VSIZE_SFT           0x00


/*------------------------------*/
/* DMA_MASKCOLR                 */
/*------------------------------*/
#define DMA_MASKCOLR_BIT        0x00FFFFFF
#define DMA_MASKCOLR_SFT        0x00


/*------------------------------*/
/* DMA_FORMAT                   */
/*------------------------------*/
#define DMA_FORMAT_BIT                  0x0000000F
#define DMA_FORMAT_SFT                  0x0
#define DMA_FORMAT_RGB888               0x00000000
#define DMA_FORMAT_RGB666               0x00000001
#define DMA_FORMAT_RGB565               0x00000002
#define DMA_FORMAT_YUV420_PLANAR        0x00000004
#define DMA_FORMAT_YUV420_SEMI_PLANAR   0x00000005
#define DMA_FORMAT_YUV422_PLANAR        0x00000008
#define DMA_FORMAT_YUV422_SEMI_PLANAR   0x00000009
#define DMA_FORMAT_YUV422_INTERLEAVE    0x0000000A


/*------------------------------*/
/* BYTELANE                     */
/* (temporary declaration)      */
/*------------------------------*/
#define DMA_BYTELANE_DATAx_BYTE0   0x0
#define DMA_BYTELANE_DATAx_BYTE1   0x1
#define DMA_BYTELANE_DATAx_BYTE2   0x2
#define DMA_BYTELANE_DATAx_BYTE3   0x3

/*------------------------------*/
/* DMA_SRCBYTE_1                */
/*------------------------------*/
#define DMA_SRCBYTE_1_DATA3_BIT    0x000000C0
#define DMA_SRCBYTE_1_DATA3_SFT    0x6
#define DMA_SRCBYTE_1_DATA3_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_1_DATA3_SFT)
#define DMA_SRCBYTE_1_DATA3_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_1_DATA3_SFT)
#define DMA_SRCBYTE_1_DATA3_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_1_DATA3_SFT)
#define DMA_SRCBYTE_1_DATA3_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_1_DATA3_SFT)

#define DMA_SRCBYTE_1_DATA2_BIT    0x00000030
#define DMA_SRCBYTE_1_DATA2_SFT    0x4
#define DMA_SRCBYTE_1_DATA2_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_1_DATA2_SFT)
#define DMA_SRCBYTE_1_DATA2_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_1_DATA2_SFT)
#define DMA_SRCBYTE_1_DATA2_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_1_DATA2_SFT)
#define DMA_SRCBYTE_1_DATA2_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_1_DATA2_SFT)

#define DMA_SRCBYTE_1_DATA1_BIT    0x0000000C
#define DMA_SRCBYTE_1_DATA1_SFT    0x2
#define DMA_SRCBYTE_1_DATA1_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_1_DATA1_SFT)
#define DMA_SRCBYTE_1_DATA1_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_1_DATA1_SFT)
#define DMA_SRCBYTE_1_DATA1_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_1_DATA1_SFT)
#define DMA_SRCBYTE_1_DATA1_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_1_DATA1_SFT)

#define DMA_SRCBYTE_1_DATA0_BIT    0x00000003
#define DMA_SRCBYTE_1_DATA0_SFT    0x0
#define DMA_SRCBYTE_1_DATA0_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_1_DATA0_SFT)
#define DMA_SRCBYTE_1_DATA0_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_1_DATA0_SFT)
#define DMA_SRCBYTE_1_DATA0_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_1_DATA0_SFT)
#define DMA_SRCBYTE_1_DATA0_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_1_DATA0_SFT)


/*------------------------------*/
/* DMA_SRCBYTE_2                */
/*------------------------------*/
#define DMA_SRCBYTE_2_DATA3_BIT    0x000000C0
#define DMA_SRCBYTE_2_DATA3_SFT    0x6
#define DMA_SRCBYTE_2_DATA3_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_2_DATA3_SFT)
#define DMA_SRCBYTE_2_DATA3_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_2_DATA3_SFT)
#define DMA_SRCBYTE_2_DATA3_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_2_DATA3_SFT)
#define DMA_SRCBYTE_2_DATA3_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_2_DATA3_SFT)

#define DMA_SRCBYTE_2_DATA2_BIT    0x00000030
#define DMA_SRCBYTE_2_DATA2_SFT    0x4
#define DMA_SRCBYTE_2_DATA2_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_2_DATA2_SFT)
#define DMA_SRCBYTE_2_DATA2_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_2_DATA2_SFT)
#define DMA_SRCBYTE_2_DATA2_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_2_DATA2_SFT)
#define DMA_SRCBYTE_2_DATA2_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_2_DATA2_SFT)

#define DMA_SRCBYTE_2_DATA1_BIT    0x0000000C
#define DMA_SRCBYTE_2_DATA1_SFT    0x2
#define DMA_SRCBYTE_2_DATA1_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_2_DATA1_SFT)
#define DMA_SRCBYTE_2_DATA1_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_2_DATA1_SFT)
#define DMA_SRCBYTE_2_DATA1_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_2_DATA1_SFT)
#define DMA_SRCBYTE_2_DATA1_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_2_DATA1_SFT)

#define DMA_SRCBYTE_2_DATA0_BIT    0x00000003
#define DMA_SRCBYTE_2_DATA0_SFT    0x0
#define DMA_SRCBYTE_2_DATA0_BYTE0  (DMA_BYTELANE_DATAx_BYTE0 << DMA_SRCBYTE_2_DATA0_SFT)
#define DMA_SRCBYTE_2_DATA0_BYTE1  (DMA_BYTELANE_DATAx_BYTE1 << DMA_SRCBYTE_2_DATA0_SFT)
#define DMA_SRCBYTE_2_DATA0_BYTE2  (DMA_BYTELANE_DATAx_BYTE2 << DMA_SRCBYTE_2_DATA0_SFT)
#define DMA_SRCBYTE_2_DATA0_BYTE3  (DMA_BYTELANE_DATAx_BYTE3 << DMA_SRCBYTE_2_DATA0_SFT)


/*------------------------------*/
/* DMA_DSTBYTE                  */
/*------------------------------*/
#define DMA_DSTBYTE_DATA3_BIT      0x000000C0
#define DMA_DSTBYTE_DATA3_SFT      0x6
#define DMA_DSTBYTE_DATA3_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_DSTBYTE_DATA3_SFT)
#define DMA_DSTBYTE_DATA3_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_DSTBYTE_DATA3_SFT)
#define DMA_DSTBYTE_DATA3_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_DSTBYTE_DATA3_SFT)
#define DMA_DSTBYTE_DATA3_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_DSTBYTE_DATA3_SFT)

#define DMA_DSTBYTE_DATA2_BIT      0x00000030
#define DMA_DSTBYTE_DATA2_SFT      0x4
#define DMA_DSTBYTE_DATA2_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_DSTBYTE_DATA2_SFT)
#define DMA_DSTBYTE_DATA2_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_DSTBYTE_DATA2_SFT)
#define DMA_DSTBYTE_DATA2_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_DSTBYTE_DATA2_SFT)
#define DMA_DSTBYTE_DATA2_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_DSTBYTE_DATA2_SFT)

#define DMA_DSTBYTE_DATA1_BIT      0x0000000C
#define DMA_DSTBYTE_DATA1_SFT      0x2
#define DMA_DSTBYTE_DATA1_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_DSTBYTE_DATA1_SFT)
#define DMA_DSTBYTE_DATA1_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_DSTBYTE_DATA1_SFT)
#define DMA_DSTBYTE_DATA1_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_DSTBYTE_DATA1_SFT)
#define DMA_DSTBYTE_DATA1_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_DSTBYTE_DATA1_SFT)

#define DMA_DSTBYTE_DATA0_BIT      0x00000003
#define DMA_DSTBYTE_DATA0_SFT      0x0
#define DMA_DSTBYTE_DATA0_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_DSTBYTE_DATA0_SFT)
#define DMA_DSTBYTE_DATA0_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_DSTBYTE_DATA0_SFT)
#define DMA_DSTBYTE_DATA0_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_DSTBYTE_DATA0_SFT)
#define DMA_DSTBYTE_DATA0_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_DSTBYTE_DATA0_SFT)


/*------------------------------*/
/* DMA_SRCBYTE_1_CMP            */
/*------------------------------*/
/* DMA_SRCBYTE_2_CMP            */
/*------------------------------*/
/* DMA_DSTBYTE_CMP              */
/*------------------------------*/
#define DMA_BYTE_DATA3_UV_BIT      0x0000C000
#define DMA_BYTE_DATA3_UV_SFT      0xE
#define DMA_BYTE_DATA3_UV_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA3_UV_SFT)
#define DMA_BYTE_DATA3_UV_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA3_UV_SFT)
#define DMA_BYTE_DATA3_UV_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA3_UV_SFT)
#define DMA_BYTE_DATA3_UV_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA3_UV_SFT)

#define DMA_BYTE_DATA2_UV_BIT      0x00003000
#define DMA_BYTE_DATA2_UV_SFT      0xC
#define DMA_BYTE_DATA2_UV_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA2_UV_SFT)
#define DMA_BYTE_DATA2_UV_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA2_UV_SFT)
#define DMA_BYTE_DATA2_UV_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA2_UV_SFT)
#define DMA_BYTE_DATA2_UV_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA2_UV_SFT)

#define DMA_BYTE_DATA1_UV_BIT      0x00000C00
#define DMA_BYTE_DATA1_UV_SFT      0xA
#define DMA_BYTE_DATA1_UV_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA1_UV_SFT)
#define DMA_BYTE_DATA1_UV_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA1_UV_SFT)
#define DMA_BYTE_DATA1_UV_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA1_UV_SFT)
#define DMA_BYTE_DATA1_UV_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA1_UV_SFT)

#define DMA_BYTE_DATA0_UV_BIT      0x00000300
#define DMA_BYTE_DATA0_UV_SFT      0x8
#define DMA_BYTE_DATA0_UV_BYTE0    (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA0_UV_SFT)
#define DMA_BYTE_DATA0_UV_BYTE1    (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA0_UV_SFT)
#define DMA_BYTE_DATA0_UV_BYTE2    (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA0_UV_SFT)
#define DMA_BYTE_DATA0_UV_BYTE3    (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA0_UV_SFT)

#define DMA_BYTE_DATA3_Y_BIT       0x000000C0
#define DMA_BYTE_DATA3_Y_SFT       0x6
#define DMA_BYTE_DATA3_Y_BYTE0     (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA3_Y_SFT)
#define DMA_BYTE_DATA3_Y_BYTE1     (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA3_Y_SFT)
#define DMA_BYTE_DATA3_Y_BYTE2     (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA3_Y_SFT)
#define DMA_BYTE_DATA3_Y_BYTE3     (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA3_Y_SFT)

#define DMA_BYTE_DATA2_Y_BIT       0x00000030
#define DMA_BYTE_DATA2_Y_SFT       0x4
#define DMA_BYTE_DATA2_Y_BYTE0     (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA2_Y_SFT)
#define DMA_BYTE_DATA2_Y_BYTE1     (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA2_Y_SFT)
#define DMA_BYTE_DATA2_Y_BYTE2     (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA2_Y_SFT)
#define DMA_BYTE_DATA2_Y_BYTE3     (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA2_Y_SFT)

#define DMA_BYTE_DATA1_Y_BIT       0x0000000C
#define DMA_BYTE_DATA1_Y_SFT       0x2
#define DMA_BYTE_DATA1_Y_BYTE0     (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA1_Y_SFT)
#define DMA_BYTE_DATA1_Y_BYTE1     (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA1_Y_SFT)
#define DMA_BYTE_DATA1_Y_BYTE2     (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA1_Y_SFT)
#define DMA_BYTE_DATA1_Y_BYTE3     (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA1_Y_SFT)

#define DMA_BYTE_DATA0_Y_BIT       0x00000003
#define DMA_BYTE_DATA0_Y_SFT       0x0
#define DMA_BYTE_DATA0_Y_BYTE0     (DMA_BYTELANE_DATAx_BYTE0 << DMA_BYTE_DATA0_Y_SFT)
#define DMA_BYTE_DATA0_Y_BYTE1     (DMA_BYTELANE_DATAx_BYTE1 << DMA_BYTE_DATA0_Y_SFT)
#define DMA_BYTE_DATA0_Y_BYTE2     (DMA_BYTELANE_DATAx_BYTE2 << DMA_BYTE_DATA0_Y_SFT)
#define DMA_BYTE_DATA0_Y_BYTE3     (DMA_BYTELANE_DATAx_BYTE3 << DMA_BYTE_DATA0_Y_SFT)


/*------------------------------*/
/* DMA_FILLDATA                 */
/*------------------------------*/
#define DMA_FILL_DATA_BIT       0xFFFFFFFF
#define DMA_FILL_DATA_SFT       0x00


/*------------------------------*/
/* DMA_DUAL_FF                  */
/*------------------------------*/
#define DMA_REG_EN_BIT          0x00000001
#define DMA_REG_EN_SFT          0x00


/*------------------------------*/
/* DMA_AUTO_SCAN                */
/*------------------------------*/
#define DMA_AUTO_SCAN_SRC1_Y_BIT  0x03000000
#define DMA_AUTO_SCAN_SRC1_Y_SFT  0x18
#define DMA_AUTO_SCAN_SRC1_Y_0    0x00000000
#define DMA_AUTO_SCAN_SRC1_Y_1    0x01000000
#define DMA_AUTO_SCAN_SRC1_Y_2    0x02000000
#define DMA_AUTO_SCAN_SRC1_Y_3    0x03000000

#define DMA_AUTO_SCAN_SRC1_U_BIT  0x00300000
#define DMA_AUTO_SCAN_SRC1_U_SFT  0x14
#define DMA_AUTO_SCAN_SRC1_U_0    0x00000000
#define DMA_AUTO_SCAN_SRC1_U_1    0x00100000
#define DMA_AUTO_SCAN_SRC1_U_2    0x00200000
#define DMA_AUTO_SCAN_SRC1_U_3    0x00300000

#define DMA_AUTO_SCAN_SRC1_V_BIT  0x00030000
#define DMA_AUTO_SCAN_SRC1_V_SFT  0x10
#define DMA_AUTO_SCAN_SRC1_V_0    0x00000000
#define DMA_AUTO_SCAN_SRC1_V_1    0x00010000
#define DMA_AUTO_SCAN_SRC1_V_2    0x00020000
#define DMA_AUTO_SCAN_SRC1_V_3    0x00030000

#define DMA_AUTO_SCAN_SRC2_Y_BIT  0x00000300
#define DMA_AUTO_SCAN_SRC2_Y_SFT  0x8
#define DMA_AUTO_SCAN_SRC2_Y_0    0x00000000
#define DMA_AUTO_SCAN_SRC2_Y_1    0x00000100
#define DMA_AUTO_SCAN_SRC2_Y_2    0x00000200
#define DMA_AUTO_SCAN_SRC2_Y_3    0x00000300

#define DMA_AUTO_SCAN_SRC2_U_BIT  0x00000030
#define DMA_AUTO_SCAN_SRC2_U_SFT  0x4
#define DMA_AUTO_SCAN_SRC2_U_0    0x00000000
#define DMA_AUTO_SCAN_SRC2_U_1    0x00000010
#define DMA_AUTO_SCAN_SRC2_U_2    0x00000020
#define DMA_AUTO_SCAN_SRC2_U_3    0x00000030

#define DMA_AUTO_SCAN_SRC2_V_BIT  0x00000003
#define DMA_AUTO_SCAN_SRC2_V_SFT  0x0
#define DMA_AUTO_SCAN_SRC2_V_0    0x00000000
#define DMA_AUTO_SCAN_SRC2_V_1    0x00000001
#define DMA_AUTO_SCAN_SRC2_V_2    0x00000002
#define DMA_AUTO_SCAN_SRC2_V_3    0x00000003


/*===============================================================*/
/* DMA register reset                                            */
/*===============================================================*/
#define DMA_MODE_RESET          0x00000000
#define DMA_SRCBYTE_1_RESET     0x000000E4
#define DMA_SRCBYTE_2_RESET     0x000000E4
#define DMA_DSTBYTE_RESET       0x000000E4
#define DMA_DUAL_FF_RESET       0x00000000
#define DMA_AUTO_SCAN_RESET     0x02220222

#endif /* _DMA_H_ */
