/* 2010-08-25: File added and changed by Sony Corporation */
/*
*  File Name       : /include/asm-arm/arch-mp200/IMG.h
*  Function        : IMG MMIO definitions, and Driver - Driver I/F of IPU driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/

#ifndef _IMG_H_
#define _IMG_H_

/*===============================================================*/
/* IMG MMIO definitions   for uPD77601                           */
/*===============================================================*/
#include <asm/arch/hardware.h>

#define IMAGEPROC_OFFSET    MP200_IMAGEPROC_OFFSET
#define IMAGEPROC_BASE      MP200_IMAGEPROC_BASE
#define IMAGEPROC_SIZE      MP200_IMAGEPROC_SIZE


/*===============================================================*/
/* IMG register definitions   for uPD77601                       */
/*===============================================================*/
//                                                           R/W  Bit   Reset
// Functions                                                 R/W  21:0  0x0
#define IMG_MODE            (0x00)

// Request for Processing                                     W    0:0  0x0
#define IMG_REQ             (0x04)

// Current Processing Status                                  R    0:0  0x0
#define IMG_ACK             (0x08)

// Interuppt Status                                           R    5:0  0x0
#define IMG_STATUS          (0x0C)

// Interuppt Raw Status                                       R    3:0  0x0
#define IMG_RAWSTATUS       (0x10)

// Interuppt Enable Set                                      R/W   3:0  0x0
#define IMG_ENSET           (0x14)

// Interuppt Enable Clear                                     W    3:0  0x0
#define IMG_ENCLR           (0x18)

// Interuppt FF Clear                                         W    3:0  0x0
#define IMG_FFCLR           (0x1C)

// Address increment value for the source back image         R/W  14:2  0x000
#define IMG_SRCSIZE_B       (0x20)

// Address increment value for the source front image        R/W  14:2  0x000
#define IMG_SRCSIZE_F       (0x24)

// Address increment value for the destination image         R/W  14:2  0x000
#define IMG_DSTSIZE         (0x28)

// Y/RGB Plane Address for the source image                  R/W  31:2  0x0000_0000
#define IMG_SRCYRGBADR_B    (0x30)

// Y/RGB Plane Address for the source image                  R/W  31:2  0x0000_0000
#define IMG_SRCYRGBADR_F    (0x34)

// Y/RGB Plane Address for the destination image             R/W  31:2  0x0000_0000
#define IMG_DSTYRGBADR      (0x38)

// UV Plane Address for the source image                     R/W  31:2  0x0000_0000
#define IMG_SRCUVADR_B      (0x40)

// UV Plane Address for the source image                     R/W  31:2  0x0000_0000
#define IMG_SRCUVADR_F      (0x44)

// UV Plane Address for the destination image                R/W  31:2  0x0000_0000
#define IMG_DSTUVADR        (0x48)

// Horizontal size for the source image                      R/W  12:0  0x008
#define IMG_SRCHSIZE_B      (0x50)

// Horizontal size for the source image                      R/W  12:0  0x008
#define IMG_SRCHSIZE_F      (0x54)

// Vartical size of the source image                         R/W  12:0  0x008
#define IMG_SRCVSIZE_B      (0x58)

// Vartical size of the source image                         R/W  12:0  0x008
#define IMG_SRCVSIZE_F      (0x5C)

// Horizontal location offset                                R/W  12:0  0x000
#define IMG_OFFSETX         (0x60)

// Vertical location offset                                  R/W  12:0  0x000
#define IMG_OFFSETY         (0x64)

// Mask Color                                                R/W  15:0  0x0000
#define IMG_MASKCOLR        (0x68)

// Transpearency                                              R/W  7:0  0xFF
#define IMG_ALPHA           (0x6C)

// Horizontal size for output image                          R/W  12:0  0x008
#define IMG_DSTHSIZE        (0x70)

// Vartical size for output image                            R/W  12:0  0x008
#define IMG_DSTVSIZE        (0x74)

// Horizontal Enlargement and Reduction steps for resizing   R/W  13:0  0x100
#define IMG_HSTEP           (0x78)

// Vertical Enlargement and Reducation steps for resizing    R/W  13:0  0x100
#define IMG_VSTEP           (0x7C)

// Horizontal Enlargement and Reduction rate for resizing    R/W   8:0  0x40
#define IMG_HFOLD           (0x80)

// Vertical Enlargement and Reducation rate for resizing     R/W   8:0  0x40
#define IMG_VFOLD           (0x84)

// Pixel Endian Component                                    R/W   1:0  0x0
#define IMG_PEL_ENDIAN      (0xAC)

// HADDR for debuging                                        R/W  31:0  0x0
#define IMG_DBG_HADDR       (0xB0)

// Color Conversion Matrix Configuratin (RGB->YUV)           R/W   0:0  0x0
#define IMG_RGBYUV_CONF     (0xC0)

// Color Conversion Matrix (RGB->YUV) 00 Component           R/W  11:0  0x04D
#define IMG_RGBYUV00        (0xC4)

// Color Conversion Matrix (RGB->YUV) 01 Component           R/W  11:0  0x096
#define IMG_RGBYUV01        (0xC8)

// Color Conversion Matrix (RGB->YUV) 02 Component           R/W  11:0  0x01D
#define IMG_RGBYUV02        (0xCC)

// Color Conversion Matrix (RGB->YUV) 10 Component           R/W  11:0  0x82C
#define IMG_RGBYUV10        (0xD0)

// Color Conversion Matrix (RGB->YUV) 11 Component           R/W  11:0  0x857
#define IMG_RGBYUV11        (0xD4)

// Color Conversion Matrix (RGB->YUV) 12 Component           R/W  11:0  0x083
#define IMG_RGBYUV12        (0xD8)

// Color Conversion Matrix (RGB->YUV) 20 Component           R/W  11:0  0x083
#define IMG_RGBYUV20        (0xDC)

// Color Conversion Matrix (RGB->YUV) 21 Component           R/W  11:0  0x86E
#define IMG_RGBYUV21        (0xE0)

// Color Conversion Matrix (RGB->YUV) 22 Component           R/W  11:0  0x815
#define IMG_RGBYUV22        (0xE4)

// Color Conversion Matrix Configuratin (YUV->RGB)           R/W   0:0  0x0
#define IMG_YUVRGB_CONF     (0xE8)

// Color Conversion Matrix (YUV->RGB) 00 Component           R/W  11:0  0x100
#define IMG_YUVRGB00        (0xEC)

// Color Conversion Matrix (YUV->RGB) 01 Component           R/W  11:0  0x0
#define IMG_YUVRGB01        (0xF0)

// Color Conversion Matrix (YUV->RGB) 02 Component           R/W  11:0  0x15F
#define IMG_YUVRGB02        (0xF4)

// Color Conversion Matrix (YUV->RGB) 10 Component           R/W  11:0  0x100
#define IMG_YUVRGB10        (0xF8)

// Color Conversion Matrix (YUV->RGB) 11 Component           R/W  11:0  0x856
#define IMG_YUVRGB11        (0xFC)

// Color Conversion Matrix (YUV->RGB) 12 Component           R/W  11:0  0x8B3
#define IMG_YUVRGB12        (0x100)

// Color Conversion Matrix (YUV->RGB) 20 Component           R/W  11:0  0x100
#define IMG_YUVRGB20        (0x104)

// Color Conversion Matrix (YUV->RGB) 21 Component           R/W  11:0  0x1BB
#define IMG_YUVRGB21        (0x108)

// Color Conversion Matrix (YUV->RGB) 22 Component           R/W  11:0  0x000
#define IMG_YUVRGB22        (0x10C)

// Image format                                              R/W  11:0  0x0000
#define IMG_FORMAT          (0x110)

// IN BYTE lane for the source image                         R/W   7:0  0xE4
#define IMG_INDATABYTE      (0x114)

// OUT BYTE lane for the destination image                   R/W   7:0  0xE4
#define IMG_OUTDATABYTE     (0x118)

// V Plane Address for the source image                      R/W  31:2  0x0000_0000
#define IMG_SRCVADR_B       (0x11C)

// V Plane Address for the source image                      R/W  31:2  0x0000_0000
#define IMG_SRCVADR_F       (0x120)

// V Plane Address for the destination image                 R/W  31:2  0x0000_0000
#define IMG_DSTVADR         (0x124)

// Register update reservation setting                       R/W   0:0  0x0000
#define IMG_DUAL_FF         (0x128)

// IN BYTE lane for the source back image                    R/W  15:0  0xE4E4
#define IMG_INDATABYTE_B_CMP (0x130)

// IN BYTE lane for the source front image                   R/W  15:0  0xE4E4
#define IMG_INDATABYTE_F_CMP (0x134)

// OUT BYTE lane for the destination image                   R/W  15:0  0xE4E4
#define IMG_OUTDATABYTE_CMP  (0x138)

// R BRITNESS set                                            R/W  10:0  0x00
#define IMG_R_BRITNESS       (0x13C)

// G BRITNESS set                                            R/W  10:0  0x00
#define IMG_G_BRITNESS       (0x140)

// B BRITNESS set                                            R/W  10:0  0x00
#define IMG_B_BRITNESS       (0x144)



/*===============================================================*/
/* IMG register bit assigns                                      */
/*===============================================================*/

/*------------------------------*/
/* IMG_MODE                     */
/*------------------------------*/
#define IMG_NEWEN_BIT           0x00100000
#define IMG_NEWEN_SFT           20

#define IMG_MASK_COL_BIT        0x00080000
#define IMG_MASK_COL_SFT        19

#define IMG_RLINV_BIT           0x00040000
#define IMG_RLINV_SFT           18

#define IMG_UDINV_BIT           0x00020000
#define IMG_UDINV_SFT           17

#define IMG_OP_MODE_00_BIT      0x00000000
#define IMG_OP_MODE_01_BIT      0x00001000
#define IMG_OP_MODE_02_BIT      0x00002000
#define IMG_OP_MODE_03_BIT      0x00003000
#define IMG_OP_MODE_04_BIT      0x00004000
#define IMG_OP_MODE_05_BIT      0x00005000
#define IMG_OP_MODE_06_BIT      0x00006000
#define IMG_OP_MODE_07_BIT      0x00007000
#define IMG_OP_MODE_08_BIT      0x00010200
#define IMG_OP_MODE_09_BIT      0x00010500
#define IMG_OP_MODE_10_BIT      0x00011300
#define IMG_OP_MODE_11_BIT      0x00011400
#define IMG_OP_MODE_12_BIT      0x00012000
#define IMG_OP_MODE_13_BIT      0x00012700
#define IMG_OP_MODE_14_BIT      0x00013100
#define IMG_OP_MODE_15_BIT      0x00013600
#define IMG_OP_MODE_16_BIT      0x00014100
#define IMG_OP_MODE_17_BIT      0x00014600
#define IMG_OP_MODE_18_BIT      0x00015000
#define IMG_OP_MODE_19_BIT      0x00015700
#define IMG_OP_MODE_20_BIT      0x00016300
#define IMG_OP_MODE_21_BIT      0x00016400
#define IMG_OP_MODE_22_BIT      0x00017200
#define IMG_OP_MODE_23_BIT      0x00017500
#define IMG_OP_MODE_24_BIT      0x00008000
#define IMG_OP_MODE_25_BIT      0x00009000
#define IMG_OP_MODE_26_BIT      0x0000a000

#define IMG_OVERLAY_BIT         0x00010000
#define IMG_OVERLAY_SFT         0x10

#define IMG_OP_MODE_BIT         0x0001FF00
#define IMG_OP_MODE_SFT         0x8

#define IMG_OPB_BIT             0x0000F000
#define IMG_OPB_SFT             0xC

#define IMG_OPF_BIT             0x00000F00
#define IMG_OPF_SFT             0x8

#define IMG_RFILTER_B_BIT       0x00000080
#define IMG_RFILTER_B_SFT       0x7

#define IMG_RFILTER_F_BIT       0x00000040
#define IMG_RFILTER_F_SFT       0x6

#define IMG_PELPACK_BIT         0x00000020
#define IMG_PELPACK_SFT         0x5

#define IMG_DITHER_B_BIT        0x00000010
#define IMG_DITHER_B_SFT        0x4

#define IMG_DITHER_F_BIT        0x00000008
#define IMG_DITHER_F_SFT        0x3

#define IMG_DSTFMT_BIT          0x00000004
#define IMG_DSTFMT_SFT          0x2

#define IMG_SRCFMT_B_BIT        0x00000002
#define IMG_SRCFMT_B_SFT        0x1

#define IMG_SRCFMT_F_BIT        0x00000001
#define IMG_SRCFMT_F_SFT        0x0


/*------------------------------*/
/* IMG_REQ                      */
/*------------------------------*/
#define IMG_REQ_BIT             0x00000001
#define IMG_REQ_SFT             0x0


/*------------------------------*/
/* IMG_ACK                      */
/*------------------------------*/
#define IMG_ACK_BIT             0x00000001
#define IMG_ACK_SFT             0x0


/*------------------------------*/
/* IMG_STATUS                   */
/*------------------------------*/
#define IMG_ACTIVE_BIT          0x00000020
#define IMG_ACTIVE_SFT          0x5

#define IMG_DFF_RSV_BIT         0x00000010
#define IMG_DFF_RSV_SFT         0x4

#define IMG_FLT_BIT             0x00000008
#define IMG_FLT_SFT             0x3

#define IMG_STOP_BIT            0x00000004
#define IMG_STOP_SFT            0x2

#define IMG_ERROR_BIT           0x00000002
#define IMG_ERROR_SFT           0x1

#define IMG_STAT_BIT            0x00000001
#define IMG_STAT_SFT            0x0


/*------------------------------*/
/* IMG_RAWSTATUS                */
/*------------------------------*/
#define IMG_RAWFLT_BIT          0x00000008
#define IMG_RAWFLT_SFT          0x3

#define IMG_RAWSTOP_BIT         0x00000004
#define IMG_RAWSTOP_SFT         0x2

#define IMG_RAWERROR_BIT        0x00000002
#define IMG_RAWERROR_SFT        0x1

#define IMG_RAWSTAT_BIT         0x00000001
#define IMG_RAWSTAT_SFT         0x0


/*------------------------------*/
/* IMG_ENSET                    */
/*------------------------------*/
#define IMG_ENSETFLT_BIT        0x00000008
#define IMG_ENSETFLT_SFT        0x3

#define IMG_ENSETSTOP_BIT       0x00000004
#define IMG_ENSETSTOP_SFT       0x2

#define IMG_ENSETERROR_BIT      0x00000002
#define IMG_ENSETERROR_SFT      0x1

#define IMG_ENSETSTAT_BIT       0x00000001
#define IMG_ENSETSTAT_SFT       0x0


/*------------------------------*/
/* IMG_ENCLR                    */
/*------------------------------*/
#define IMG_ENCLRFLT_BIT        0x00000008
#define IMG_ENCLRFLT_SFT        0x3

#define IMG_ENCLRSTOP_BIT       0x00000004
#define IMG_ENCLRSTOP_SFT       0x2

#define IMG_ENCLRERROR_BIT      0x00000002
#define IMG_ENCLRERROR_SFT      0x1

#define IMG_ENCLRSTAT_BIT       0x00000001
#define IMG_ENCLRSTAT_SFT       0x0


/*------------------------------*/
/* IMG_FFCLR                    */
/*------------------------------*/
#define IMG_FFCLRFLT_BIT        0x00000008
#define IMG_FFCLRFLT_SFT        0x3

#define IMG_FFCLRSTOP_BIT       0x00000004
#define IMG_FFCLRSTOP_SFT       0x2

#define IMG_FFCLRERROR_BIT      0x00000002
#define IMG_FFCLRERROR_SFT      0x1

#define IMG_FFCLRSTAT_BIT       0x00000001
#define IMG_FFCLRSTAT_SFT       0x0


/*------------------------------*/
/* IMG_SRCSIZE_B                */
/*------------------------------*/
#define IMG_SRCSIZE_B_BIT       0x00007FFF
#define IMG_SRCSIZE_B_SFT       0x0


/*------------------------------*/
/* IMG_SRCSIZE_F                */
/*------------------------------*/
#define IMG_SRCSIZE_F_BIT       0x00007FFF
#define IMG_SRCSIZE_F_SFT       0x0


/*------------------------------*/
/* IMG_DSTSIZE                  */
/*------------------------------*/
#define IMG_DSTSIZE_BIT         0x00007FFF
#define IMG_DSTSIZE_SFT         0x0


/*------------------------------*/
/* IMG_SRCYRGBADR_B             */
/*------------------------------*/
#define IMG_SRCYRGBADR_B_BIT    0xFFFFFFFF
#define IMG_SRCYRGBADR_B_SFT    0x0


/*------------------------------*/
/* IMG_SRCYRGBADR_F             */
/*------------------------------*/
#define IMG_SRCYRGBADR_F_BIT    0xFFFFFFFF
#define IMG_SRCYRGBADR_F_SFT    0x0


/*------------------------------*/
/* IMG_DSTYRGBADR               */
/*------------------------------*/
#define IMG_DSTYRGBADR_BIT      0xFFFFFFFF
#define IMG_DSTYRGBADR_SFT      0x0


/*------------------------------*/
/* IMG_SRCUVADR_B               */
/*------------------------------*/
#define IMG_SRCUVADR_B_BIT      0xFFFFFFFF
#define IMG_SRCUVADR_B_SFT      0x0


/*------------------------------*/
/* IMG_SRCUVADR_F               */
/*------------------------------*/
#define IMG_SRCUVADR_F_BIT      0xFFFFFFFF
#define IMG_SRCUVADR_F_SFT      0x0


/*------------------------------*/
/* IMG_DSTUVADR                 */
/*------------------------------*/
#define IMG_DSTUVADR_BIT        0xFFFFFFFF
#define IMG_DSTUVADR_SFT        0x0

/*------------------------------*/
/* IMG_SRCVADR_B               */
/*------------------------------*/
#define IMG_SRCVADR_B_BIT       0xFFFFFFFF
#define IMG_SRCVADR_B_SFT       0x0


/*------------------------------*/
/* IMG_SRCVADR_F               */
/*------------------------------*/
#define IMG_SRCVADR_F_BIT       0xFFFFFFFF
#define IMG_SRCVADR_F_SFT       0x0


/*------------------------------*/
/* IMG_DSTVADR                 */
/*------------------------------*/
#define IMG_DSTVADR_BIT         0xFFFFFFFF
#define IMG_DSTVADR_SFT         0x0


/*------------------------------*/
/* IMG_SRCHSIZE_B               */
/*------------------------------*/
#define IMG_SRCHSIZE_B_BIT      0x00001FFF
#define IMG_SRCHSIZE_B_SFT      0x0


/*------------------------------*/
/* IMG_SRCHSIZE_F               */
/*------------------------------*/
#define IMG_SRCHSIZE_F_BIT      0x00001FFF
#define IMG_SRCHSIZE_F_SFT      0x0


/*------------------------------*/
/* IMG_SRCVSIZE_B               */
/*------------------------------*/
#define IMG_SRCVSIZE_B_BIT      0x00001FFF
#define IMG_SRCVSIZE_B_SFT      0x0


/*------------------------------*/
/* IMG_SRCVSIZE_F               */
/*------------------------------*/
#define IMG_SRCVSIZE_F_BIT      0x00001FFF
#define IMG_SRCVSIZE_F_SFT      0x0


/*------------------------------*/
/* IMG_OFFSETX                  */
/*------------------------------*/
#define IMG_OFFSETX_BIT         0x00001FFF
#define IMG_OFFSETX_SFT         0x0

/*------------------------------*/
/* IMG_OFFSETY                  */
/*------------------------------*/
#define IMG_OFFSETY_BIT         0x00001FFF
#define IMG_OFFSETY_SFT         0x0

/*------------------------------*/
/* IMG_MASKCOLR                 */
/*------------------------------*/
#define IMG_MASKCOLR_BIT        0x0000FFFF
#define IMG_MASKCOLR_SFT        0x0

/*------------------------------*/
/* IMG_ALPHA                    */
/*------------------------------*/
#define IMG_ALPHA_BIT           0x000000FF
#define IMG_ALPHA_SFT           0x0


/*------------------------------*/
/* IMG_DSTHSIZE                 */
/*------------------------------*/
#define IMG_DSTHSIZE_BIT        0x00001FFF
#define IMG_DSTHSIZE_SFT        0x0


/*------------------------------*/
/* IMG_DSTVSIZE                 */
/*------------------------------*/
#define IMG_DSTVSIZE_BIT        0x00001FFF
#define IMG_DSTVSIZE_SFT        0x0


/*------------------------------*/
/* IMG_HSTEP                    */
/*------------------------------*/
#define IMG_HSTEP_BIT           0x00003FFF
#define IMG_HSTEP_SFT           0x0


/*------------------------------*/
/* IMG_VSTEP                    */
/*------------------------------*/
#define IMG_VSTEP_BIT           0x00003FFF
#define IMG_VSTEP_SFT           0x0


/*------------------------------*/
/* IMG_HFOLD                    */
/*------------------------------*/
#define IMG_HFOLD_BIT           0x000001FF
#define IMG_HFOLD_SFT           0x0


/*------------------------------*/
/* IMG_VFOLD                    */
/*------------------------------*/
#define IMG_VFOLD_BIT           0x000001FF
#define IMG_VFOLD_SFT           0x0


/*------------------------------*/
/* IMG_PEL_ENDIAN               */
/*------------------------------*/
#define IMG_PEL_ENDIAN_BIT             0x00000003
#define IMG_PEL_ENDIAN_IN_ENDI_LITTLE  0x1
#define IMG_PEL_ENDIAN_IN_ENDI_BIG     0x0
#define IMG_PEL_ENDIAN_OUT_ENDI_LITTLE 0x2
#define IMG_PEL_ENDIAN_OUT_ENDI_BIG    0x0
#define IMG_PEL_ENDIAN_IN_ENDI_SFT     0x0
#define IMG_PEL_ENDIAN_OUT_ENDI_SFT    0x1



/*------------------------------*/
/* IMG_DBG_HADDR                */
/*------------------------------*/
#define IMG_DBG_HADDR_BIT       0xFFFFFFFF
#define IMG_DBG_HADDR_SFT       0x0


/*------------------------------*/
/* IMG_RGBYUV00                 */
/*------------------------------*/
#define IMG_RGBYUV00_BIT        0x00000FFF
#define IMG_RGBYUV00_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV01                 */
/*------------------------------*/
#define IMG_RGBYUV01_BIT        0x00000FFF
#define IMG_RGBYUV01_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV02                 */
/*------------------------------*/
#define IMG_RGBYUV02_BIT        0x00000FFF
#define IMG_RGBYUV02_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV10                 */
/*------------------------------*/
#define IMG_RGBYUV10_BIT        0x00000FFF
#define IMG_RGBYUV10_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV11                 */
/*------------------------------*/
#define IMG_RGBYUV11_BIT        0x00000FFF
#define IMG_RGBYUV11_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV12                 */
/*------------------------------*/
#define IMG_RGBYUV12_BIT        0x00000FFF
#define IMG_RGBYUV12_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV20                 */
/*------------------------------*/
#define IMG_RGBYUV20_BIT        0x00000FFF
#define IMG_RGBYUV20_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV21                 */
/*------------------------------*/
#define IMG_RGBYUV21_BIT        0x00000FFF
#define IMG_RGBYUV21_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV22                 */
/*------------------------------*/
#define IMG_RGBYUV22_BIT        0x00000FFF
#define IMG_RGBYUV22_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB00                 */
/*------------------------------*/
#define IMG_YUVRGB00_BIT        0x00000FFF
#define IMG_YUVRGB00_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB01                 */
/*------------------------------*/
#define IMG_YUVRGB01_BIT        0x00000FFF
#define IMG_YUVRGB01_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB02                 */
/*------------------------------*/
#define IMG_YUVRGB02_BIT        0x00000FFF
#define IMG_YURGBV02_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB10                 */
/*------------------------------*/
#define IMG_YUVRGB10_BIT        0x00000FFF
#define IMG_YUVRGB10_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB11                 */
/*------------------------------*/
#define IMG_YUVRGB11_BIT        0x00000FFF
#define IMG_YUVRGB11_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB12                 */
/*------------------------------*/
#define IMG_YUVRGB12_BIT        0x00000FFF
#define IMG_YUVRGB12_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB20                 */
/*------------------------------*/
#define IMG_YUVRGB20_BIT        0x00000FFF
#define IMG_YUVRGB20_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB21                 */
/*------------------------------*/
#define IMG_YUVRGB21_BIT        0x00000FFF
#define IMG_YUVRGB21_SFT        0x0


/*------------------------------*/
/* IMG_YUVRGB22                 */
/*------------------------------*/
#define IMG_YUVRGB22_BIT        0x00000FFF
#define IMG_YUVRGB22_SFT        0x0


/*------------------------------*/
/* IMG_RGBYUV_CONF              */
/*------------------------------*/
#define IMG_RGBYUV_YOFFSET_VALID_BIT    0x00000001
#define IMG_RGBYUV_YOFFSET_VALID_SFT    0x0


/*------------------------------*/
/* IMG_YUVRGB_CONF              */
/*------------------------------*/
#define IMG_YUVRGB_YOFFSET_VALID_BIT    0x00000001
#define IMG_YUVRGB_YOFFSET_VALID_SFT    0x0


/*------------------------------*/
/* IMG_FORMAT                   */
/* (temporary declaration)      */
/*------------------------------*/
#define IMG_FORMAT_RGB888                      0x0
#define IMG_FORMAT_RGB666                      0x1
#define IMG_FORMAT_RGB565                      0x2
#define IMG_FORMAT_YUV420_PLANAR               0x4
#define IMG_FORMAT_YUV420_SEMI_PLANAR          0x5
#define IMG_FORMAT_YUV422_PLANAR               0x8
#define IMG_FORMAT_YUV422_SEMI_PLANAR          0x9
#define IMG_FORMAT_YUV422_INTERLEAVE           0xA

/*------------------------------*/
/* IMG_FORMAT                   */
/*------------------------------*/
#define IMG_FORMAT_DSTFMT_BIT                  0x00000F00
#define IMG_FORMAT_DSTFMT_SFT                  0x8
#define IMG_FORMAT_DSTFMT_RGB888               (IMG_FORMAT_RGB888             << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_RGB666               (IMG_FORMAT_RGB666             << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_RGB565               (IMG_FORMAT_RGB565             << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_YUV420_PLANAR        (IMG_FORMAT_YUV420_PLANAR      << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_YUV420_SEMI_PLANAR   (IMG_FORMAT_YUV420_SEMI_PLANAR << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_YUV422_PLANAR        (IMG_FORMAT_YUV422_PLANAR      << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_YUV422_SEMI_PLANAR   (IMG_FORMAT_YUV422_SEMI_PLANAR << IMG_FORMAT_DSTFMT_SFT)
#define IMG_FORMAT_DSTFMT_YUV422_INTERLEAVE    (IMG_FORMAT_YUV422_INTERLEAVE  << IMG_FORMAT_DSTFMT_SFT)

#define IMG_FORMAT_SRCFMT_B_BIT                0x000000F0
#define IMG_FORMAT_SRCFMT_B_SFT                0x4
#define IMG_FORMAT_SRCFMT_B_RGB888             (IMG_FORMAT_RGB888             << IMG_FORMAT_SRCFMT_B_SFT)
#define IMG_FORMAT_SRCFMT_B_RGB565             (IMG_FORMAT_RGB565             << IMG_FORMAT_SRCFMT_B_SFT)
#define IMG_FORMAT_SRCFMT_B_YUV420_PLANAR      (IMG_FORMAT_YUV420_PLANAR      << IMG_FORMAT_SRCFMT_B_SFT)
#define IMG_FORMAT_SRCFMT_B_YUV420_SEMI_PLANAR (IMG_FORMAT_YUV420_SEMI_PLANAR << IMG_FORMAT_SRCFMT_B_SFT)
#define IMG_FORMAT_SRCFMT_B_YUV422_PLANAR      (IMG_FORMAT_YUV422_PLANAR      << IMG_FORMAT_SRCFMT_B_SFT)
#define IMG_FORMAT_SRCFMT_B_YUV422_SEMI_PLANAR (IMG_FORMAT_YUV422_SEMI_PLANAR << IMG_FORMAT_SRCFMT_B_SFT)
#define IMG_FORMAT_SRCFMT_B_YUV422_INTERLEAVE  (IMG_FORMAT_YUV422_INTERLEAVE  << IMG_FORMAT_SRCFMT_B_SFT)

#define IMG_FORMAT_SRCFMT_F_BIT                0x0000000F
#define IMG_FORMAT_SRCFMT_F_SFT                0x0
#define IMG_FORMAT_SRCFMT_F_RGB888             (IMG_FORMAT_RGB888             << IMG_FORMAT_SRCFMT_F_SFT)
#define IMG_FORMAT_SRCFMT_F_RGB565             (IMG_FORMAT_RGB565             << IMG_FORMAT_SRCFMT_F_SFT)
#define IMG_FORMAT_SRCFMT_F_YUV420_PLANAR      (IMG_FORMAT_YUV420_PLANAR      << IMG_FORMAT_SRCFMT_F_SFT)
#define IMG_FORMAT_SRCFMT_F_YUV420_SEMI_PLANAR (IMG_FORMAT_YUV420_SEMI_PLANAR << IMG_FORMAT_SRCFMT_F_SFT)
#define IMG_FORMAT_SRCFMT_F_YUV422_PLANAR      (IMG_FORMAT_YUV422_PLANAR      << IMG_FORMAT_SRCFMT_F_SFT)
#define IMG_FORMAT_SRCFMT_F_YUV422_SEMI_PLANAR (IMG_FORMAT_YUV422_SEMI_PLANAR << IMG_FORMAT_SRCFMT_F_SFT)
#define IMG_FORMAT_SRCFMT_F_YUV422_INTERLEAVE  (IMG_FORMAT_YUV422_INTERLEAVE  << IMG_FORMAT_SRCFMT_F_SFT)


/*------------------------------*/
/* BYTELANE                     */
/* (temporary declaration)      */
/*------------------------------*/
#define IMG_BYTELANE_DATAx_BYTE0    0x0
#define IMG_BYTELANE_DATAx_BYTE1    0x1
#define IMG_BYTELANE_DATAx_BYTE2    0x2
#define IMG_BYTELANE_DATAx_BYTE3    0x3

/*------------------------------*/
/* IMG_INDATABYTE               */
/*------------------------------*/
#define IMG_INDATABYTE_DATA3_BIT    0x000000C0
#define IMG_INDATABYTE_DATA3_SFT    0x6
#define IMG_INDATABYTE_DATA3_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_DATA3_SFT)
#define IMG_INDATABYTE_DATA3_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_DATA3_SFT)
#define IMG_INDATABYTE_DATA3_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_DATA3_SFT)
#define IMG_INDATABYTE_DATA3_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_DATA3_SFT)

#define IMG_INDATABYTE_DATA2_BIT    0x00000030
#define IMG_INDATABYTE_DATA2_SFT    0x4
#define IMG_INDATABYTE_DATA2_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_DATA2_SFT)
#define IMG_INDATABYTE_DATA2_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_DATA2_SFT)
#define IMG_INDATABYTE_DATA2_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_DATA2_SFT)
#define IMG_INDATABYTE_DATA2_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_DATA2_SFT)

#define IMG_INDATABYTE_DATA1_BIT    0x0000000C
#define IMG_INDATABYTE_DATA1_SFT    0x2
#define IMG_INDATABYTE_DATA1_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_DATA1_SFT)
#define IMG_INDATABYTE_DATA1_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_DATA1_SFT)
#define IMG_INDATABYTE_DATA1_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_DATA1_SFT)
#define IMG_INDATABYTE_DATA1_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_DATA1_SFT)

#define IMG_INDATABYTE_DATA0_BIT    0x00000003
#define IMG_INDATABYTE_DATA0_SFT    0x0
#define IMG_INDATABYTE_DATA0_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_DATA0_SFT)
#define IMG_INDATABYTE_DATA0_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_DATA0_SFT)
#define IMG_INDATABYTE_DATA0_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_DATA0_SFT)
#define IMG_INDATABYTE_DATA0_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_DATA0_SFT)


/*------------------------------*/
/* IMG_OUTDATABYTE              */
/*------------------------------*/
#define IMG_OUTDATABYTE_DATA3_BIT   0x000000C0
#define IMG_OUTDATABYTE_DATA3_SFT   0x6
#define IMG_OUTDATABYTE_DATA3_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_OUTDATABYTE_DATA3_SFT)
#define IMG_OUTDATABYTE_DATA3_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_OUTDATABYTE_DATA3_SFT)
#define IMG_OUTDATABYTE_DATA3_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_OUTDATABYTE_DATA3_SFT)
#define IMG_OUTDATABYTE_DATA3_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_OUTDATABYTE_DATA3_SFT)

#define IMG_OUTDATABYTE_DATA2_BIT   0x00000030
#define IMG_OUTDATABYTE_DATA2_SFT   0x4
#define IMG_OUTDATABYTE_DATA2_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_OUTDATABYTE_DATA2_SFT)
#define IMG_OUTDATABYTE_DATA2_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_OUTDATABYTE_DATA2_SFT)
#define IMG_OUTDATABYTE_DATA2_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_OUTDATABYTE_DATA2_SFT)
#define IMG_OUTDATABYTE_DATA2_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_OUTDATABYTE_DATA2_SFT)

#define IMG_OUTDATABYTE_DATA1_BIT   0x0000000C
#define IMG_OUTDATABYTE_DATA1_SFT   0x2
#define IMG_OUTDATABYTE_DATA1_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_OUTDATABYTE_DATA1_SFT)
#define IMG_OUTDATABYTE_DATA1_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_OUTDATABYTE_DATA1_SFT)
#define IMG_OUTDATABYTE_DATA1_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_OUTDATABYTE_DATA1_SFT)
#define IMG_OUTDATABYTE_DATA1_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_OUTDATABYTE_DATA1_SFT)

#define IMG_OUTDATABYTE_DATA0_BIT   0x00000003
#define IMG_OUTDATABYTE_DATA0_SFT   0x0
#define IMG_OUTDATABYTE_DATA0_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_OUTDATABYTE_DATA0_SFT)
#define IMG_OUTDATABYTE_DATA0_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_OUTDATABYTE_DATA0_SFT)
#define IMG_OUTDATABYTE_DATA0_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_OUTDATABYTE_DATA0_SFT)
#define IMG_OUTDATABYTE_DATA0_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_OUTDATABYTE_DATA0_SFT)


/*------------------------------*/
/* IMG_INDATABYTE_B_CMP         */
/*------------------------------*/
/* IMG_INDATABYTE_F_CMP         */
/*------------------------------*/
/* IMG_OUTDATABYTE_CMP          */
/*------------------------------*/
#define IMG_DATABYTE_DATA3_UV_BIT   0x0000C000
#define IMG_DATABYTE_DATA3_UV_SFT   0xE
#define IMG_DATABYTE_DATA3_UV_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA3_UV_SFT)
#define IMG_DATABYTE_DATA3_UV_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA3_UV_SFT)
#define IMG_DATABYTE_DATA3_UV_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA3_UV_SFT)
#define IMG_DATABYTE_DATA3_UV_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA3_UV_SFT)

#define IMG_DATABYTE_DATA2_UV_BIT   0x00003000
#define IMG_DATABYTE_DATA2_UV_SFT   0xC
#define IMG_DATABYTE_DATA2_UV_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA2_UV_SFT)
#define IMG_DATABYTE_DATA2_UV_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA2_UV_SFT)
#define IMG_DATABYTE_DATA2_UV_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA2_UV_SFT)
#define IMG_DATABYTE_DATA2_UV_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA2_UV_SFT)

#define IMG_DATABYTE_DATA1_UV_BIT   0x00000C00
#define IMG_DATABYTE_DATA1_UV_SFT   0xA
#define IMG_DATABYTE_DATA1_UV_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA1_UV_SFT)
#define IMG_DATABYTE_DATA1_UV_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA1_UV_SFT)
#define IMG_DATABYTE_DATA1_UV_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA1_UV_SFT)
#define IMG_DATABYTE_DATA1_UV_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA1_UV_SFT)

#define IMG_DATABYTE_DATA0_UV_BIT   0x00000300
#define IMG_DATABYTE_DATA0_UV_SFT   0x8
#define IMG_DATABYTE_DATA0_UV_BYTE0 (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA0_UV_SFT)
#define IMG_DATABYTE_DATA0_UV_BYTE1 (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA0_UV_SFT)
#define IMG_DATABYTE_DATA0_UV_BYTE2 (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA0_UV_SFT)
#define IMG_DATABYTE_DATA0_UV_BYTE3 (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA0_UV_SFT)

#define IMG_DATABYTE_DATA3_Y_BIT    0x000000C0
#define IMG_DATABYTE_DATA3_Y_SFT    0x6
#define IMG_DATABYTE_DATA3_Y_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA3_Y_SFT)
#define IMG_DATABYTE_DATA3_Y_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA3_Y_SFT)
#define IMG_DATABYTE_DATA3_Y_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA3_Y_SFT)
#define IMG_DATABYTE_DATA3_Y_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA3_Y_SFT)

#define IMG_DATABYTE_DATA2_Y_BIT    0x00000030
#define IMG_DATABYTE_DATA2_Y_SFT    0x4
#define IMG_DATABYTE_DATA2_Y_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA2_Y_SFT)
#define IMG_DATABYTE_DATA2_Y_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA2_Y_SFT)
#define IMG_DATABYTE_DATA2_Y_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA2_Y_SFT)
#define IMG_DATABYTE_DATA2_Y_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA2_Y_SFT)

#define IMG_DATABYTE_DATA1_Y_BIT    0x0000000C
#define IMG_DATABYTE_DATA1_Y_SFT    0x2
#define IMG_DATABYTE_DATA1_Y_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA1_Y_SFT)
#define IMG_DATABYTE_DATA1_Y_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA1_Y_SFT)
#define IMG_DATABYTE_DATA1_Y_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA1_Y_SFT)
#define IMG_DATABYTE_DATA1_Y_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA1_Y_SFT)

#define IMG_DATABYTE_DATA0_Y_BIT    0x00000003
#define IMG_DATABYTE_DATA0_Y_SFT    0x0
#define IMG_DATABYTE_DATA0_Y_BYTE0  (IMG_BYTELANE_DATAx_BYTE0 << IMG_INDATABYTE_B_DATA0_Y_SFT)
#define IMG_DATABYTE_DATA0_Y_BYTE1  (IMG_BYTELANE_DATAx_BYTE1 << IMG_INDATABYTE_B_DATA0_Y_SFT)
#define IMG_DATABYTE_DATA0_Y_BYTE2  (IMG_BYTELANE_DATAx_BYTE2 << IMG_INDATABYTE_B_DATA0_Y_SFT)
#define IMG_DATABYTE_DATA0_Y_BYTE3  (IMG_BYTELANE_DATAx_BYTE3 << IMG_INDATABYTE_B_DATA0_Y_SFT)


/*------------------------------*/
/* IMG_DUAL_FF                  */
/*------------------------------*/
#define IMG_REG_EN_BIT          0x00000001
#define IMG_REG_EN_SFT          0x0


/*------------------------------*/
/* IMG_R_BRITNESS               */
/*------------------------------*/
#define IMG_R_BRITNESS_BIT      0x000007FF
#define IMG_R_BRITNESS_SFT      0x0


/*------------------------------*/
/* IMG_G_BRITNESS               */
/*------------------------------*/
#define IMG_G_BRITNESS_BIT      0x000007FF
#define IMG_G_BRITNESS_SFT      0x0


/*------------------------------*/
/* IMG_B_BRITNESS               */
/*------------------------------*/
#define IMG_B_BRITNESS_BIT      0x000007FF
#define IMG_B_BRITNESS_SFT      0x0


/*===============================================================*/
/* IMG register reset                                            */
/*===============================================================*/
#define IMG_MODE_RESET              0x00000000
#define IMG_HSTEP_RESET             0x00000100
#define IMG_VSTEP_RESET             0x00000100
#define IMG_HFOLD_RESET             0x00000040
#define IMG_VFOLD_RESET             0x00000040
#define IMG_INDATABYTE_RESET        0x000000E4
#define IMG_OUTDATABYTE_RESET       0x000000E4
#define IMG_DUAL_FF_RESET           0x00000000

#define IMG_RGBYUV_CONF_RESET       0x00000000
#define IMG_RGBYUV00_RESET          0x0000004D
#define IMG_RGBYUV01_RESET          0x00000096
#define IMG_RGBYUV02_RESET          0x0000001D
#define IMG_RGBYUV10_RESET          0x0000082C
#define IMG_RGBYUV11_RESET          0x00000857
#define IMG_RGBYUV12_RESET          0x00000083
#define IMG_RGBYUV20_RESET          0x00000083
#define IMG_RGBYUV21_RESET          0x0000086E
#define IMG_RGBYUV22_RESET          0x00000815
#define IMG_YUVRGB_CONF_RESET       0x00000000
#define IMG_YUVRGB00_RESET          0x00000100
#define IMG_YUVRGB01_RESET          0x00000000
#define IMG_YUVRGB02_RESET          0x0000015F
#define IMG_YUVRGB10_RESET          0x00000100
#define IMG_YUVRGB11_RESET          0x00000856
#define IMG_YUVRGB12_RESET          0x000008B3
#define IMG_YUVRGB20_RESET          0x00000100
#define IMG_YUVRGB21_RESET          0x000001BB
#define IMG_YUVRGB22_RESET          0x00000000
#define IMG_R_BRITNESS_RESET        0x00000000
#define IMG_G_BRITNESS_RESET        0x00000000
#define IMG_B_BRITNESS_RESET        0x00000000

#endif /* _IMG_H_ */
