/* 2010-08-25: File added and changed by Sony Corporation */
/* -*- mode: c; c-basic-offset: 8; comment-column: 32; -*- */

/*
 * InterDSP Driver
 *
 * Copyright (C) NEC Electronics Corporation 2004-2009
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef INTER_DSP_IOCTL_H
#define INTER_DSP_IOCTL_H

#ifdef __KERNEL__
#include <linux/types.h>
#include <linux/ioctl.h>
#else
#include <inttypes.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#endif

/* for dac hangup */
#define DSP_DACHANGUP 1

/*
 * IOCTL struct
 */

struct interdsp_chinfo {
	uint32_t header_offset;
	uint32_t buffer_offset;
	uint32_t buffer_blknum;
	uint32_t buffer_blksize;
};

/* INTERDSP_READ/INTERDSP_WRITE */

/* struct dsp_cmd_rw flags */
#define INTERDSP_WRITEBUF_OK	0x0001
#define INTERDSP_SENDHDR_OK	0x0002

struct dsp_cmd_rw {
	uint32_t fdata[2];
	uint8_t *bdata;
	size_t bsize;
	size_t offset;
	int intq_num;
	uint32_t flags;
};

/* ch info */
struct dsp_cmd_ch {
	uint32_t dsparm;
	uint32_t chnum;
	struct interdsp_chinfo chinfo;
};

/* TaskID Channel */
struct dsp_cmd_task_ch {
	uint32_t taskid;
	int32_t chnum;
};

/* Interval Get/Set */
struct dsp_cmd_ch_interval {
	int32_t chnum;
	int32_t intq_num;
	int32_t interval;
	int32_t repeat;
	int32_t missfire;
};

struct dsp_cmd_ch_status_v0 {
	uint32_t armchmask;	/* Ch. No. Mask (1<<0,1<<1,1<<2) */
	uint32_t dspchmask;	/* Ch. No. Mask (1<<0,1<<1,1<<2) */
};

/* Channel Status 2 */
struct dsp_cmd_ch_status {
	uint32_t armchmask;	/* Ch. No. Mask (1<<0,1<<1,1<<2) */
	uint32_t dspchmask;	/* Ch. No. Mask (1<<0,1<<1,1<<2) */

	unsigned long armch_blockbitmap[3];
};


/*
 * Program DownLoad
 */
/* control flags */
#define INTERDSP_DL_PROG	0x01
				     /* 0x02 unused (INTERDSP_DL_POWER) */
#define INTERDSP_DL_RESET	0x04
#define INTERDSP_DL_CLOCK	0x08
#define INTERDSP_DL_CLKCTRL	0x10

/*
 * Download `offset' examples
 *
 *   memtype  target-addr     offset
 *   0        0xb0001234  =>  0x00001234    [target - 0xb0000000]
 *   1        0xb0108000  =>  0x00008000    [target - 0xb0100000]
 *   2        0xa0000100  =>  0x00000100    [target - 0xa0000000]
 *   3        0x33a04000  =>  0x03a04000    [target - 0x30000000]
 */

/* DSP Control */
struct dsp_cmd_download {
	uint32_t control;	/* control flags */

	/* INTERDSP_DL_PROG */
	void *data;		/* Program/Data Address */
	uint32_t memtype;	/* 0:InstSRAM  1:DataSRAM  2:ExtSRAM 3:SDRAM */
	uint32_t offset;	/* Write Offset in SRAM/SDRAM */
	uint32_t size;		/* Copy Size */

	/* INTERDSP_DL_RESET */
	uint32_t reset;		/* 0:clear 1:set 2:(first set, last clear) */

	uint32_t power;		/* unused (backward compatibility) */

	/* INTERDSP_DL_CLOCK */
	uint32_t clock;		/* 1:start, 0:stop */

	/* INTERDSP_DL_CLKCTRL */
	uint32_t clkctrl;	/* 1:on, 0:off (auto clock control) */
};

/*
 * Errot Bit Number (INTERDSP_GET_ERROR)
 */

#define INTERDSP_ERROR_ARMCH_ACK_TIMEOUT(CH)	(CH)

#define INTERDSP_ERROR_WRAPPER_NR		8

/*
 * DCV
 */

struct dcv_reg {
	uint16_t bank_no;
	union {
		struct {
			unsigned bank_offset:12; /* DCV_BANKn_OFFSET */
			unsigned bank_set:4; /* DCV_BANKn_SET */
		} bank_reg;
		uint16_t bank_addr;
	} bank_val;
};

struct dsp_cmd_dcv_set {
	unsigned int count;	/* set/get number */
	struct dcv_reg *dcv_regs; /* pointer to `struct dcv_reg' */
};

/* ADD-S 20070521 [DSP DIVPMxSCLK] */
/*
 * SMU_DIVPMxSCLK
 */

/* reg_num */
#define DSPDEV_DIVPM0SCLK	(0)
#define DSPDEV_DIVPM1SCLK	(1)

/* reg_val macros */
#define DSPDEV_DIVPMxSCLK_MLD_NFSIN(N)	(((N)&0xfU)<<12)
#define DSPDEV_DIVPMxSCLK_SEL_PLL2	(0U<<8)
#define DSPDEV_DIVPMxSCLK_SEL_PLL3	(1U<<8)
#define DSPDEV_DIVPMxSCLK_SEL_PLLB	(2U<<8)
#define DSPDEV_DIVPMxSCLK_SEL_PLL26MHz	(3U<<8)
#define DSPDEV_DIVPMxSCLK_SEL_MLD_NFSIN	(4U<<8)
#define DSPDEV_DIVPMxSCLK_DIV1(N)	(((N)&0xfU)<<4)
#define DSPDEV_DIVPMxSCLK_DIV0(N)	(((N)&0x7U)<<0)

struct dsp_cmd_divpmxsclk {
	unsigned int reg_num;	/* one of DSPDEV_DIVPM[01]SCLK */
	unsigned int reg_val;	/* DIVPM[01]SCLK values */
};
/* ADD-E 20070521 [DSP DIVPMxSCLK] */

/*
 *	ioctl command
 */

#define INTERDSP_READ			_IOR('D', 0x01, struct dsp_cmd_rw)
#define INTERDSP_WRITE			_IOW('D', 0x02, struct dsp_cmd_rw)

#define INTERDSP_WRITEBUF		_IOWR('D', 0x08, struct dsp_cmd_rw)
#define INTERDSP_SENDHDR		_IOWR('D', 0x09, struct dsp_cmd_rw)

#define INTERDSP_GET_CHINFO	_IOR('D', 0x10, struct dsp_cmd_ch)
#define INTERDSP_SET_CHINFO	_IOW('D', 0x11, struct dsp_cmd_ch)
#define INTERDSP_GET_TASKCH	_IOR('D', 0x12, struct dsp_cmd_task_ch)
#define INTERDSP_SET_TASKCH	_IOWR('D', 0x13, struct dsp_cmd_task_ch)
#define INTERDSP_SHMEM_INIT	_IO('D', 0x18)

#define INTERDSP_CHECKCH	_IOWR('D', 0x22, struct dsp_cmd_ch_status)
#define INTERDSP_WAITCH		_IOWR('D', 0x23, struct dsp_cmd_ch_status)

#define INTERDSP_DOWNLOAD	_IOWR('D', 0x30, struct dsp_cmd_download)

#define INTERDSP_GET_ERROR	_IOR('D', 0x40, unsigned long)

#define INTERDSP_SET_DCV_REGS	_IOW('D', 0x50, struct dsp_cmd_dcv_set)
#define INTERDSP_GET_DCV_REGS	_IOW('D', 0x51, struct dsp_cmd_dcv_set)


/* Start/Stop Peripherals */
#define DSPDEV_START_PERIPHERALS	_IOW('P', 1, unsigned)
#define DSPDEV_STOP_PERIPHERALS		_IOW('P', 2, unsigned)

#define DSPDEV_PERIP_PM0		(1U<<0)
#define DSPDEV_PERIP_PM1		(1U<<1)
#define DSPDEV_PERIP_TI3		(1U<<4)
#define DSPDEV_PERIP_DCV		(1U<<8)

#define DSPDEV_PERIP_PM0_STOP_SCLK	(1U<<16)
#define DSPDEV_PERIP_PM1_STOP_SCLK	(1U<<17)


/* Set Timer clock */
#define DSPDEV_TIN_SEL			_IO('P', 3)

#define DSPDEV_TIN_TI3_BIT		(1U<<8)
#define DSPDEV_TIN_TI3_VAL		(1U<<0)
#define DSPDEV_TIN_TI3_0		DSPDEV_TIN_TI3_BIT
#define DSPDEV_TIN_TI3_1		(DSPDEV_TIN_TI3_BIT|DSPDEV_TIN_TI3_VAL)


/* Set DMA INT_SEL DSP */
#define DSPDEV_DMA_INT_SEL		_IO('P', 7)

#define DSPDEV_DMA_INT_SEL_PCH_SHIFT	16
#define DSPDEV_DMA_INT_SEL_M2M		(1U << DSPDEV_DMA_INT_SEL_PCH_SHIFT)
#define DSPDEV_DMA_INT_SEL_P2M		(2U << DSPDEV_DMA_INT_SEL_PCH_SHIFT)
#define DSPDEV_DMA_INT_SEL_M2P		(3U << DSPDEV_DMA_INT_SEL_PCH_SHIFT)
#define DSPDEV_DMA_INT_SEL_PCH_MASK	(3U << DSPDEV_DMA_INT_SEL_PCH_SHIFT)

#define DSPDEV_DMA_INT_SEL_LCH0		(1U<<0)
#define DSPDEV_DMA_INT_SEL_LCH1		(1U<<1)
#define DSPDEV_DMA_INT_SEL_LCH2		(1U<<2)
#define DSPDEV_DMA_INT_SEL_LCH3		(1U<<3)
#define DSPDEV_DMA_INT_SEL_LCH4		(1U<<4)
#define DSPDEV_DMA_INT_SEL_LCH5		(1U<<5)
#define DSPDEV_DMA_INT_SEL_LCH6		(1U<<6)
#define DSPDEV_DMA_INT_SEL_LCH7		(1U<<7)
#define DSPDEV_DMA_INT_SEL_LCH8		(1U<<8)
#define DSPDEV_DMA_INT_SEL_LCH9		(1U<<9)
#define DSPDEV_DMA_INT_SEL_LCH10	(1U<<10)
#define DSPDEV_DMA_INT_SEL_LCH11	(1U<<11)
#define DSPDEV_DMA_INT_SEL_LCH12	(1U<<12)
#define DSPDEV_DMA_INT_SEL_LCH13	(1U<<13)
#define DSPDEV_DMA_INT_SEL_LCH14	(1U<<14)
#define DSPDEV_DMA_INT_SEL_LCH_MASK	0x7fff



/* Set DMA INT_SEL CPU */
#define CPUDEV_DMA_INT_SEL		_IO('P', 8)

#define CPUDEV_DMA_INT_SEL_PCH_SHIFT	16
#define CPUDEV_DMA_INT_SEL_M2M		(1U << CPUDEV_DMA_INT_SEL_PCH_SHIFT)
#define CPUDEV_DMA_INT_SEL_P2M		(2U << CPUDEV_DMA_INT_SEL_PCH_SHIFT)
#define CPUDEV_DMA_INT_SEL_M2P		(3U << CPUDEV_DMA_INT_SEL_PCH_SHIFT)
#define CPUDEV_DMA_INT_SEL_PCH_MASK	(3U << CPUDEV_DMA_INT_SEL_PCH_SHIFT)

#define CPUDEV_DMA_INT_SEL_LCH0		(1U<<0)
#define CPUDEV_DMA_INT_SEL_LCH1		(1U<<1)
#define CPUDEV_DMA_INT_SEL_LCH2		(1U<<2)
#define CPUDEV_DMA_INT_SEL_LCH3		(1U<<3)
#define CPUDEV_DMA_INT_SEL_LCH4		(1U<<4)
#define CPUDEV_DMA_INT_SEL_LCH5		(1U<<5)
#define CPUDEV_DMA_INT_SEL_LCH6		(1U<<6)
#define CPUDEV_DMA_INT_SEL_LCH7		(1U<<7)
#define CPUDEV_DMA_INT_SEL_LCH8		(1U<<8)
#define CPUDEV_DMA_INT_SEL_LCH9		(1U<<9)
#define CPUDEV_DMA_INT_SEL_LCH10	(1U<<10)
#define CPUDEV_DMA_INT_SEL_LCH11	(1U<<11)
#define CPUDEV_DMA_INT_SEL_LCH12	(1U<<12)
#define CPUDEV_DMA_INT_SEL_LCH13	(1U<<13)
#define CPUDEV_DMA_INT_SEL_LCH14	(1U<<14)
#define CPUDEV_DMA_INT_SEL_LCH_MASK	0x7fff

/* for DAC hangup */
#define DSPDEV_DACTIMER_START _IO('P', 9)
#define DSPDEV_DACTIMER_STOP  _IO('P', 10)

/* ADD-S 20070521 [DSP DIVPMxSCLK] */
/* SMU_DIVPMxSCLK accessor */
#define DSPDEV_SET_DIVPMxSCLK		_IOW('P', 9, struct dsp_cmd_divpmxsclk)
/* ADD-E 20070521 [DSP DIVPMxSCLK] */


#define DSPDEV_SRAMCTRL	_IOW('P', 10, unsigned)


#endif /* INTER_DSP_IOCTL_H */
