/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /include/asm-arm/arch-mp200/IPU.h
*  Function        : IPU MMIO definitions, and Driver - Driver I/F of IPU driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/

#ifndef _IPU_H_
#define _IPU_H_

#include <asm/arch/img.h>
#include <asm/arch/rot.h>
#include <asm/arch/gdma.h>


/*===============================================================*/
/* IPU Driver Definitions                                        */
/*===============================================================*/

/* H/W Function of mp200_ipu_req(h/w function) */
#define IPU_HW_FUNC_IMG             0x01    /* Image Processor Function                   */
#define IPU_HW_FUNC_ROT             0x02    /* Image Rotator Function                     */
#define IPU_HW_FUNC_GDMA            0x04    /* GraphicsDMA Function                       */
#define IPU_HW_FUNC_ALL             (IPU_HW_FUNC_IMG | IPU_HW_FUNC_ROT | IPU_HW_FUNC_GDMA)
                                            /* IPU Function                               */

/* Statuses of IPU request */
#define IPU_REQ_STATUS_REGISTERED   1    /* registered                                 */
#define IPU_REQ_STATUS_RUNNING      2    /* running                                    */
#define IPU_REQ_STATUS_WAIT_CANCEL  3    /* waiting for cancelled request to be done   */
#define IPU_REQ_STATUS_CANCELLED    4    /* cancelled                                  */
#define IPU_REQ_STATUS_DONE         5    /* done                                       */
#define IPU_REQ_STATUS_ERROR        6    /* failed                                     */
#define IPU_REQ_STATUS_IDLE         8    /* unregistration                             */
#define IPU_REQ_STATUS_CANCEL       9    /* cancel start                               */

/* Parameters of mp200_ipu_regist (regist request) */
#define IPU_REQ_REGISTERED          0    /* the request is registed                    */
#define IPU_REQ_BUSY           -EBUSY    /* fail to register the request busy          */
#define IPU_REQ_PARAMERR      -EINVAL    /* fail to register the request parameter     */

/* Parameters of mp200_ipu_cancel (cancel request) */
#define IPU_CANCEL_SYNC             1    /* SYNC flag for cancel request               */

/* Returns of mp200_ipu_cancel (cancel request) */
#define IPU_CANCEL_DONE             0    /* cancelled                                  */
#define IPU_CANCEL_WAIT             1    /* waiting for cancelled request to be done   */
#define IPU_CANCEL_NO_REQ           2    /* target request does not exist              */

/* Parameters of callback function */
#define IPU_CALLBACK_SUCCESS        0    /* IPU process is completed (Success)         */
#define IPU_CALLBACK_SET_PARAM      1    /* set values to IPU registers (Success)      */
#define IPU_CALLBACK_CANCEL         2    /* cancelled request is done (Success)        */
#define IPU_CALLBACK_FAIL          -1    /* can't start IPU process (Error)            */
#define IPU_CALLBACK_IPU_ERROR     -2    /* IPU ERROR interrupt (Error)                */
#define IPU_CALLBACK_IPU_STOP      -3    /* IPU RETRY/SPLIT interrupt (Error)          */


struct mp200_ipu_req;                    /* Declaration of structure tag (incomplete type) */
struct mp200_ipu_queue;

/*===============================================================*/
/* Callback function type                                        */
/*===============================================================*/
typedef void (*ipu_callback_func)(unsigned long sequence, int flag);


/*===============================================================*/
/* IPU register setting information structure                    */
/*===============================================================*/
typedef struct mp200_ipu_reg_info {
    unsigned long reg;                   /* offset to IPU register               */
    unsigned long data;                  /* data to set                          */
} MP200_IPU_REG_INFO;
#define SIZE_MP200_IPU_REG_INFO 8

#define IPU_REG_INFO_END -1

/*===============================================================*/
/* IPU request information structure                             */
/*===============================================================*/
#define IPU_REG_INFO_MAX  230            /* max number of IPU register setting informations   */

typedef struct mp200_ipu_req {
    unsigned long       sequence;        /* sequence No                                       */
    int                 status;          /* Status of IPU request (used by IPU driver)        */
    ipu_callback_func   callback;        /* callback function                                 */
    unsigned long       data;            /* optional data (used by caller)                    */
    int                 ipu_hw_func;     /* H/W Function (processing demand)                  */
                                         /*          IMG Function   : IPU_HW_FUNC_IMG         */
                                         /*          ROT Function   : IPU_HW_FUNC_ROT         */
                                         /*          GDMA Function  : IPU_HW_FUNC_GDMA        */
    struct mp200_ipu_reg_info ipu_reg_info[];
                                         /* IPU register setting informations                 */
                                         /* repeat until end condition (max 230 pairs)        */
                                         /*  end condition :                                  */
                                         /*     set 0x00000001 to IMG_REQ/ROT_REQ/GDMA_REQ    */
                                         /*                    (request to start IPU process) */
                                         /*     reg=IMG_REQ/ROT_REQ/GDMA_REQ data=0x00000001, */
                                         /*                             or reg=-1(0xFFFFFFFF) */
} MP200_IPU_REQ;
#define MAX_SIZE_MP200_IPU_REQ (20 + (SIZE_MP200_IPU_REG_INFO * IPU_REG_INFO_MAX))


#ifndef _IPU_USER_H_
/*===============================================================*/
/* extern declarations of public functions                       */
/*===============================================================*/
extern int mp200_ipu_regist( struct mp200_ipu_req *p );
extern int mp200_ipu_cancel( unsigned long sequence, int flag );
#endif /* _IPU_USER_H_ */

#endif /* _IPU_H_ */
