/* 2010-08-25: File added by Sony Corporation */
/*
*  File Name       : /include/asm-arm/arch-mp200/IPU_user.h
*  Function        : User - Driver I/F of IPU driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/

#ifndef _IPU_USER_H_
#define _IPU_USER_H_

#include <asm/arch/ipu.h>


/*===============================================================*/
/* IOCTLs to IPU driver. 0x4E is 'N' for NEC.                    */
/*===============================================================*/
#define IMGIO_CONVERT             0x4E01  /* composite & forward image with IMG */
#define ROTIO_ROTATION            0x4E02  /* rotate image with ROT              */

#define IPUIO_MP200_IPU_REGIST    0x4E12  /* regist  IPU request                */
#define IPUIO_MP200_IPU_CALLBACK  0x4E13  /* waiting IPU request to finish      */
#define IPUIO_MP200_IPU_CANCEL    0x4E14  /* cancel  IPU request                */


/*===================================================================================
* IOCTL    : IPUIO_MP200_IPU_REGIST
* FUNCTION : register IPU request in Normal Request's queue.
*            can't register a new request before registered request is done.
*            open() without O_NONBLOCK :
*              register the IPU request, wait for the request to be done (BLOCK), and return.
*            open() with O_NONBLOCK :
*              register the IPU request, and return immediately (non BLOCK). not wait for the request to be done.
* INPUT    : pointer to IPU request information
* OUTPUT   : Returns
*              IPU_REQ_REGISTERED(0) : the request is registered
*              IPU_REQ_COMPLETE  (1) : the request is registered and done
*              IPU_REQ_CANCELLED (2) : the request is registered and cancelled
*              -1                    : fail to register the request
*                errno = EBUSY       :   busy
*                errno = EINVAL      :   parameter error
*            result_data(Status of IPU process)
*              <without O_NONBLOCK and return=1(the request is registered and done)>
*              IPU_CALLBACK_SUCCESS       : IPU process is completed (Success)
*              IPU_CALLBACK_SET_PARAM     : set values to IPU registers (Success)
*              IPU_CALLBACK_CANCEL        : cancelled request is done (Success)
*              IPU_CALLBACK_FAIL          : can't start IPU process (Error)
*              IPU_CALLBACK_IPU_ERROR     : IPU ERROR interrupt (Error)
*              IPU_CALLBACK_IPU_STOP      : IPU RETRY/SPLIT interrupt (Error)
*====================================================================================*/
typedef struct tagIPUIO_MP200_IPU_REGIST{
    size_t        request_length;   /* size of data pointed by request_arg */
    MP200_IPU_REQ *request_arg;     /* pointer to IPU request information  */
    long          result_data;      /* processing result                   */
}_IPUIO_MP200_IPU_REGIST;
#define SIZE_IPUIO_MP200_IPU_REGIST 12


#ifndef IPU_REQ_REGISTERED
#define IPU_REQ_REGISTERED 0        /* the request is registered               */
#endif
#define IPU_REQ_COMPLETE   1        /* the request is registered and done      */
#define IPU_REQ_CANCELLED  2        /* the request is registered and cancelled */
#ifndef IPU_REQ_BUSY
#define IPU_REQ_BUSY       -EBUSY   /* fail to register the request busy          */
#endif
#ifndef IPU_REQ_PARAMERR
#define IPU_REQ_PARAMERR   -EINVAL  /* fail to register the request parameter     */
#endif


/*===================================================================================
* IOCTL    : IPUIO_MP200_IPU_CALLBACK
* FUNCTION : wait for the registered IPU request to be done, or get status of the request.
*            open() without O_NONBLOCK :
*              wait for the request to be done (BLOCK), and return.
*            open() with O_NONBLOCK :
*              return immediately (non BLOCK). not wait for the request to be done.
* INPUT    : pointer to IPU request information
* OUTPUT   : Returns
*              IPU_STATUS_DONE(0)         : IPU process is completed
*              IPU_STATUS_WAIT(1)         : IPU process is running (already registered)
*              IPU_STATUS_NO_REQ(2)       : target request does not exist
*              IPU_STATUS_CANCELLED(3)    : target request is cancelled
*            result_data(status of IPU process)
*              <return=0(IPU request had been done)>
*              IPU_CALLBACK_SUCCESS       : IPU process is completed (Success)
*              IPU_CALLBACK_SET_PARAM     : set values to IPU registers (Success)
*              IPU_CALLBACK_CANCEL        : cancelled request is done (Success)
*              IPU_CALLBACK_FAIL          : can't start IPU process (Error)
*              IPU_CALLBACK_IPU_ERROR     : IPU ERROR interrupt (Error)
*              IPU_CALLBACK_IPU_STOP      : IPU RETRY/SPLIT interrupt (Error)
*              <return=1(IPU request is running)>
*              IPU_REQ_STATUS_REGISTERED  : registered (before running)
*              IPU_REQ_STATUS_RUNNING     : running (running)
*              IPU_REQ_STATUS_WAIT_CANCEL : waiting for cancelled request to be done (running)
*====================================================================================*/
typedef struct tagIPUIO_MP200_IPU_CALLBACK{
    unsigned long sequence;         /* sequence number of ipu processing demand */
    long          result_data;      /* processing result                        */
}_IPUIO_MP200_IPU_CALLBACK;
#define SIZE_IPUIO_MP200_IPU_CALLBACK 8

#define IPU_STATUS_DONE      0      /* IPU process is completed                    */
#define IPU_STATUS_WAIT      1      /* IPU process is running (already registered) */
#define IPU_STATUS_NO_REQ    2      /* target request does not exist               */
#define IPU_STATUS_CANCELLED 3      /* target request is cancelled                 */



/*===================================================================================
* IOCTL    : IPUIO_MP200_IPU_CANCEL
* FUNCTION : cancel registered IPU request
*            target IPU request has not been started :
*                remove the target request from the queue.
*            target IPU request has been started (running) :
*              without IPU_CANCEL_SYNC :
*                set IPU_CANCEL_WAIT to the status of the request,
*                and wait for the request to be done.
*                Then set IPU_CALLBACK_CANCEL to result_data
*                as return of IPUIO_MP200_IPU_CALLBACK.
*              with IPU_CANCEL_SYNC    :
*                remove the target request from the queue immediately.
*                and skip waiting for the request to be done (IPUIO_MP200_IPU_CALLBACK)
* INPUT    : pointer to IPU request information
*            SYNC flag
* OUTPUT   : Returns
*              IPU_CANCEL_DONE(0)   : cancelled
*              IPU_CANCEL_WAIT(1)   : waiting for cancelled request to be done
*              IPU_CANCEL_NO_REQ(2) : target request does not exist
*====================================================================================*/
typedef struct tagIPUIO_MP200_IPU_CANCEL{
    unsigned long sequence;         /* sequence number of ipu processing demand */
    int           flag;
    long          result_data;      /* processing result                        */
}_IPUIO_MP200_IPU_CANCEL;
#define SIZE_IPUIO_MP200_IPU_CANCEL 8


/*===================================================================================
* IOCTL    : IMGIO_CONVERT
* FUNCTION : composite & forward image with IMG
* INPUT    : input data address, width, height, format, endian, rotation angle,
*            output data address, endian
* OUTPUT   :
*            Return
*              IMG_COMPLETE(0) : Success
*              IMG_ERROR(-1)   : Fail
*====================================================================================*/
typedef struct tagIMG_MODE{
    unsigned char srcfmt_f;     /* input  image format(FRONT)                    */
    unsigned char srcfmt_b;     /* input  image format(BACK)                     */
    unsigned char dstfmt;       /* output image format                           */
    unsigned char dither_f;     /* enable dither (in front color depth downsize) */
    unsigned char dither_b;     /* enable dither (in back color depth downsize)  */
    unsigned char rfilter_f;    /* enable smooth filter (in front image zoom)    */
    unsigned char rfilter_b;    /* enable smooth filter (in back image zoom)     */
    unsigned int  op_mode;      /* op_mode                                       */
    unsigned char udinv;        /* enable UP/DOWN mirroring                      */
    unsigned char rlinv;        /* enable RIGHT/LEFT mirroring                   */
    unsigned char msk_col;      /* enable mask color                             */
}_IMG_MODE;
#define SIZE_IMG_MODE 15

typedef struct tagIMAGE_DATA{
    unsigned int  size;         /* image data pitch                              */
    unsigned long yrgbadr;      /* Y/RGB data start address (physical)           */
    unsigned long uvadr;        /* UV data start address    (physical)           */
    unsigned long vadr;         /* V data start address     (physical)           */
    unsigned int  x;            /* image data x position                         */
    unsigned int  y;            /* image data y position                         */
    unsigned int  hsize;        /* image data height                             */
    unsigned int  vsize;        /* image data width                              */
}_IMAGE_DATA;
#define SIZE_IMAGE_DATA 32

typedef struct tagIMGIO_CONVERT{
    _IMG_MODE     img_mode;     /* img_mode                                      */
    _IMAGE_DATA   src_b_image;  /* input  image data(BACK)                       */
    _IMAGE_DATA   src_f_image;  /* input  image data(FRONT)                      */
    _IMAGE_DATA   dest_image;   /* output image data                             */
    unsigned int  offsetx;      /* image offset x (in front & back composite)    */
    unsigned int  offsety;      /* image offset y (in front & back composite)    */
    unsigned int  maskcolr;     /* mask color                                    */
    unsigned int  alpha;        /* alpha blending (on maskcolor)                 */
    unsigned long in_byte;      /* input byte                                    */
    unsigned long out_byte;     /* output bute                                   */
    unsigned int  set_matrix;   /* flag if set color-space conversion matrix     */
                                /*   TRUE  : set matrix_conf and matrix[9]       */
                                /*   FALSE : use IMG initial value               */
    unsigned long matrix_conf;  /* color-space conversion matrix configuration   */
    unsigned long matrix[9];    /* color-space conversion matrix                 */
}_IMGIO_CONVERT;
#define SIZE_IMGIO_CONVERT (SIZE_IMG_MODE+(SIZE_IMAGE_DATA*3)+24+44)


/*======================================================================*/
/* _IMG_MODE  definitions                                               */
/*======================================================================*/
/* input front data format & input back data format & output data format*/
#define IPU_FORM_RGB888             0x0 /* 0000 */
#define IPU_FORM_RGB666             0x1 /* 0001 */
#define IPU_FORM_RGB565             0x2 /* 0010 */
#define IPU_FORM_YUV420_PLANAR      0x4 /* 0100 */
#define IPU_FORM_YUV420_SEMI_PLANAR 0x5 /* 0101 */
#define IPU_FORM_YUV422_PLANAR      0x8 /* 1000 */
#define IPU_FORM_YUV422_SEMI_PLANAR 0x9 /* 1001 */
#define IPU_FORM_YUV422_INTERLEAVE  0xA /* 1010 */

#define IMG_FORM_RGB888             IPU_FORM_RGB888
#define IMG_FORM_RGB666             IPU_FORM_RGB666
#define IMG_FORM_RGB565             IPU_FORM_RGB565
#define IMG_FORM_YUV420_PLANAR      IPU_FORM_YUV420_PLANAR
#define IMG_FORM_YUV420_SEMI_PLANAR IPU_FORM_YUV420_SEMI_PLANAR
#define IMG_FORM_YUV422_PLANAR      IPU_FORM_YUV422_PLANAR
#define IMG_FORM_YUV422_SEMI_PLANAR IPU_FORM_YUV422_SEMI_PLANAR
#define IMG_FORM_YUV422_INTERLEAVE  IPU_FORM_YUV422_INTERLEAVE


/* input front deta dithering & input back data dithering */
#define IMG_SET_DITHER_OFF         0x0
#define IMG_SET_DITHER_ON          0x1

/* filter  */
#define IMG_SET_RADICAL_FILTER     0x0
#define IMG_SET_LINEAR_FILTER      0x0
#define IMG_SET_SMOOTH_FILTER      0x1

/* img mode */
#define IMG_SET_OP_MODE_00         0x000
#define IMG_SET_OP_MODE_01         0x010
#define IMG_SET_OP_MODE_02         0x020
#define IMG_SET_OP_MODE_03         0x030
#define IMG_SET_OP_MODE_04         0x040
#define IMG_SET_OP_MODE_05         0x050
#define IMG_SET_OP_MODE_06         0x060
#define IMG_SET_OP_MODE_07         0x070
#define IMG_SET_OP_MODE_08         0x102
#define IMG_SET_OP_MODE_09         0x105
#define IMG_SET_OP_MODE_10         0x113
#define IMG_SET_OP_MODE_11         0x114
#define IMG_SET_OP_MODE_12         0x120
#define IMG_SET_OP_MODE_13         0x127
#define IMG_SET_OP_MODE_14         0x131
#define IMG_SET_OP_MODE_15         0x136
#define IMG_SET_OP_MODE_16         0x141
#define IMG_SET_OP_MODE_17         0x146
#define IMG_SET_OP_MODE_18         0x150
#define IMG_SET_OP_MODE_19         0x157
#define IMG_SET_OP_MODE_20         0x163
#define IMG_SET_OP_MODE_21         0x164
#define IMG_SET_OP_MODE_22         0x172
#define IMG_SET_OP_MODE_23         0x175

/* output data up-down inverse & output data right-left inverse */
#define IMG_SET_MIRROR_OFF         0x0
#define IMG_SET_MIRROR_ON          0x1

/* input front data color no-mask */
#define IMG_SET_MASKCOLOR_ON       0x0
#define IMG_SET_MASKCOLOR_OFF      0x1


/*======================================================================*/
/* _IMGIO_CONVERT  definitions                                          */
/*======================================================================*/

/* input data endian & output data endian  */
#define IMG_SET_ENDI_BIG           0xB1
#define IMG_SET_ENDI_LITTLE        0xE4

/*======================================================================*/
/* IMGIO_CONVERT return definitions                                     */
/*======================================================================*/
#define IMG_COMPLETE  0
#define IMG_ERROR    -1


/*===================================================================================
* IOCTL    : ROTIO_ROTATION
* FUNCTION : rotate image with IPU
* INPUT    : input data address, width, height, format, endian, rotation angle,
*            output data address, endian
* OUTPUT   :
*            Return
*              ROT_COMPLETE(0) : Success
*              ROT_ERROR(-1)   : Fail (IPU busy or parameter error)
*====================================================================================*/
typedef struct tagROTIO_ROTATION{
    unsigned long input_data_addr_y;    /* input Y data  start address (physical)            */
    unsigned long input_data_addr_uv;   /* input UV data start address (physical)            */
    unsigned long input_data_addr_v;    /* input V data  start address (physical)            */
    unsigned int  input_data_size;      /* input data pitch                                  */
    unsigned int  input_data_width;     /* input data width                                  */
    unsigned int  input_data_height;    /* input data height                                 */
    unsigned int  input_data_format;    /* input data format                                 */
                                        /*   ROT_FORM_RGB888 : RGB888                        */
                                        /*   ROT_FORM_RGB565 : RGB565                        */
                                        /*   ROT_FORM_YUV420_PLANAR      : YUV420PLANAR      */
                                        /*   ROT_FORM_YUV420_SEMI_PLANAR : YUV420SEMI_PLANAR */
                                        /*   ROT_FORM_YUV422_SEMI_PLANAR : YUV422SEMI_PLANAR */
                                        /*   ROT_FORM_YUV422_INTERLEAVE  : YUV422INTERLEAVE  */
    unsigned long input_byte;           /* input bytelane                                    */
                                        /*   ROT_ENDIAN_BIG   :big endian                    */
                                        /*   ROT_ENDIAN_LITTLE:little endian                 */
    unsigned int  rotation_angle;       /* rotation angle                                    */
                                        /*   ROT_ANGLE_0   :  0 degrees                      */
                                        /*   ROT_ANGLE_90  : 90 degrees                      */
                                        /*   ROT_ANGLE_180 :180 degrees                      */
                                        /*   ROT_ANGLE_270 :270 degrees                      */
    unsigned long output_data_addr_y;   /* output Y data start address  (physical)           */
    unsigned long output_data_addr_uv;  /* output UV data start address (physical)           */
    unsigned long output_data_addr_v;   /* output V data start address  (physical)           */
    unsigned int  output_data_size;     /* output data pitch                                 */
    unsigned int  output_data_format;   /* output data format                                */
                                        /*   ROT_FORM_RGB888 : RGB888                        */
                                        /*   ROT_FORM_RGB565 : RGB565                        */
                                        /*   ROT_FORM_YUV420_PLANAR      : YUV420PLANAR      */
                                        /*   ROT_FORM_YUV420_SEMI_PLANAR : YUV420SEMI_PLANAR */
                                        /*   ROT_FORM_YUV422_SEMI_PLANAR : YUV422SEMI_PLANAR */
                                        /*   ROT_FORM_YUV422_INTERLEAVE  : YUV422INTERLEAVE  */
    unsigned long output_byte;          /* output bytelane                                   */
                                        /*   ROT_ENDIAN_BIG   :big endian                    */
                                        /*   ROT_ENDIAN_LITTLE:little endian                 */
}_ROTIO_ROTATION;
#define SIZE_ROTIO_ROTATION 52


#define ROT_FORM_RGB888             IPU_FORM_RGB888
#define ROT_FORM_RGB565             IPU_FORM_RGB565
#define ROT_FORM_YUV420_PLANAR      IPU_FORM_YUV420_PLANAR
#define ROT_FORM_YUV420_SEMI_PLANAR IPU_FORM_YUV420_SEMI_PLANAR
#define ROT_FORM_YUV422_SEMI_PLANAR IPU_FORM_YUV422_SEMI_PLANAR
#define ROT_FORM_YUV422_INTERLEAVE  IPU_FORM_YUV422_INTERLEAVE

#define ROT_ENDIAN_BIG              0xB1
#define ROT_ENDIAN_LITTLE           0xE4

#define ROT_ANGLE_0                 0
#define ROT_ANGLE_90                1
#define ROT_ANGLE_180               2
#define ROT_ANGLE_270               3

#define ROT_COMPLETE  0
#define ROT_ERROR    -1


#define MAX_SIZE_IPUIOCTL SIZE_IMGIO_CONVERT

#endif /* _IPU_USER_H_ */
