/* 2010-08-25: File added and changed by Sony Corporation */
/*
*  File Name       : /include/asm-arm/arch-mp200/IPU.h
*  Function        : ROT MMIO definitions, and Driver - Driver I/F of IPU driver
*  Release Version : Ver 1.00
*  Release Date    : 2006/11/20
*
*  Copyright (C) NEC Electronics Corporation 2006
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of
*  the GNU General Public License as published by Free Softwere Foundation; either version 2
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program;
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/

/* History */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* Date      |Rev.No|Name              |KeyWord            | Reason                                         */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2006/11/20| 1.00 | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/
/* 2008/03/13|      | M.Sasaki         |                   |                                                */
/*-----------+------+------------------+-------------------+------------------------------------------------*/

#ifndef _ROT_H_
#define _ROT_H_

/*===============================================================*/
/* ROT MMIO definitions   for uPD77601                           */
/*===============================================================*/
#include <asm/arch/hardware.h>
#define ROTATOR_OFFSET    MP200_ROTATOR_OFFSET
#define ROTATOR_BASE      MP200_ROTATOR_BASE
#define ROTATOR_SIZE      MP200_ROTATOR_SIZE


/*===============================================================*/
/* ROT register definitions   for uPD77601                       */
/*===============================================================*/
//                                                           R/W   Bit  Reset
// Mode for ROTATOR                                          R/W   5:0  0x00
#define ROT_MODE                 (0x00)

// Frame Select                                              R/W   3:0  0x5
#define ROT_FRAME                (0x04)

// Rotate Request                                             W    0:0  0x0
#define ROT_REQ                  (0x08)

// Status for ROTATOR                                         R    0:0  0x0
#define ROT_STATUS               (0x0C)

// Original Address Size                                     R/W  10:2  0x000
#define ROT_SRCSIZE              (0x20)

// Rotate Address Size                                       R/W  10:2  0x000
#define ROT_DSTSIZE              (0x24)

// Original Y Address for A Frame                            R/W  30:2  0x0000_0000
#define ROT_SRCYADR_A            (0x30)

// Original Y Address for B Frame                            R/W  30:2  0x0000_0000
#define ROT_SRCYADR_B            (0x34)

// Original Y Address for C Frame                            R/W  30:2  0x0000_0000
#define ROT_SRCYADR_C            (0x38)

// Rotate Y Address for A Frame                              R/W  30:2  0x0000_0000
#define ROT_DSTYADR_A            (0x40)

// Rotate Y Address for B Frame                              R/W  30:2  0x0000_0000
#define ROT_DSTYADR_B            (0x44)

// Rotate Y Address for C Frame                              R/W  30:2  0x0000_0000
#define ROT_DSTYADR_C            (0x48)

// Original UV Address for A Frame                           R/W  30:2  0x0000_0000
#define ROT_SRCUVADR_A           (0x60)

// Original UV Address for B Frame                           R/W  30:2  0x0000_0000
#define ROT_SRCUVADR_B           (0x64)

// Original UV Address for C Frame                           R/W  30:2  0x0000_0000
#define ROT_SRCUVADR_C           (0x68)

// Rotate UV Address for A Frame                             R/W  30:2  0x0000_0000
#define ROT_DSTUVADR_A           (0x70)

// Rotate UV Address for B Frame                             R/W  30:2  0x0000_0000
#define ROT_DSTUVADR_B           (0x74)

// Rotate UV Address for C Frame                             R/W  30:2  0x0000_0000
#define ROT_DSTUVADR_C           (0x78)

// Horizontal Size                                           R/W  10:2  0x008
#define ROT_SRCHSIZE             (0x90)

// Vertical Size                                             R/W  10:2  0x008
#define ROT_SRCVSIZE             (0xA0)

// Interrupt Status                                           R    4:0  0x0
#define ROT_INTSTATUS            (0xB0)

// Interrupt Raw Status                                       R    2:0  0x0
#define ROT_INTRAWSTATUS         (0xB4)

// Interrupt Enable Set                                      R/W   2:0  0x0
#define ROT_INTENSET             (0xB8)

// Interrupt Enable Clear                                     W    2:0  0x0
#define ROT_INTENCLR             (0xBC)

// Interrupt Status Clear                                     W    2:0  0x0
#define ROT_INTFFCLR             (0xC0)

// DMA Error Respons Address                                 R/W  31:0  0x0000_0000
#define ROT_ERRORADR             (0xC4)

// Image format                                              R/W   7:0  0x0
#define ROT_FORMAT               (0xC8)

// BYTE lane for the source image                            R/W   7:0  0xE4
#define ROT_SRCBYTE              (0xCC)

// BYTE lane for the destination image                       R/W   7:0  0xE4
#define ROT_DSTBYTE              (0xD0)

// V Plane Address for the source image                      R/W  30:2  0x0000_0000
#define ROT_SRCVADR_A            (0xD4)

// V Plane Address for the source image                      R/W  30:2  0x0000_0000
#define ROT_SRCVADR_B            (0xD8)

// V Plane Address for the source image                      R/W  30:2  0x0000_0000
#define ROT_SRCVADR_C            (0xDC)

// V Plane Address for the rotator image                     R/W  30:2  0x0000_0000
#define ROT_DSTVADR_A            (0xE0)

// V Plane Address for the rotator image                     R/W  30:2  0x0000_0000
#define ROT_DSTVADR_B            (0xE4)

// V Plane Address for the rotator image                     R/W  30:2  0x0000_0000
#define ROT_DSTVADR_C            (0xE8)

// Register update reservation setting                       R/W   0:0  0x0
#define ROT_DUAL_FF              (0xEC)

// BYTE lane for the source image                            R/W  15:0  0xE4E4
#define ROT_SRCBYTE_CMP          (0xF0)

// BYTE lane for the destination image                       R/W  15:0  0xE4E4
#define ROT_DSTBYTE_CMP          (0xF4)



/*===============================================================*/
/* ROT register bit assigns                                      */
/*===============================================================*/

/*------------------------------*/
/* ROT_MODE                     */
/*------------------------------*/
#define ROT_NEWEN_BIT           0x00000020
#define ROT_NEWEN_SFT           0x5

#define ROT_SRCENDIAN_BIT       0x00000010
#define ROT_SRCENDIAN_SFT       0x4
#define ROT_SRCENDIAN_BIG       0x00000000
#define ROT_SRCENDIAN_LITTLE    0x00000010

#define ROT_DSTENDIAN_BIT       0x00000008
#define ROT_DSTENDIAN_SFT       0x3
#define ROT_DSTENDIAN_BIG       0x00000000
#define ROT_DSTENDIAN_LITTLE    0x00000008

#define ROT_SRCYUV_BIT          0x00000004
#define ROT_SRCYUV_SFT          0x2
#define ROT_SRCYUV_YUV422       0x00000000
#define ROT_SRCYUV_YUV420       0x00000004

#define ROT_MOD_BIT             0x00000003
#define ROT_MOD_SFT             0x0
#define ROT_MOD_0               0x00000000
#define ROT_MOD_90              0x00000001
#define ROT_MOD_180             0x00000002
#define ROT_MOD_270             0x00000003


/*------------------------------*/
/* ROT_FRAME                    */
/*------------------------------*/
#define ROT_DSTSEL_BIT          0x0000000C
#define ROT_DSTSEL_SFT          0x2
#define ROT_DSTSEL_FRAME_A      0x00000004
#define ROT_DSTSEL_FRAME_B      0x00000008
#define ROT_DSTSEL_FRAME_C      0x0000000C

#define ROT_SRCSEL_BIT          0x00000003
#define ROT_SRCSEL_SFT          0x0
#define ROT_SRCSEL_FRAME_A      0x00000001
#define ROT_SRCSEL_FRAME_B      0x00000002
#define ROT_SRCSEL_FRAME_C      0x00000003


/*------------------------------*/
/* ROT_REQ                      */
/*------------------------------*/
#define ROT_REQ_BIT             0x00000001
#define ROT_REQ_SFT             0x0


/*------------------------------*/
/* ROT_STATUS                   */
/*------------------------------*/
#define ROT_STATUS_BIT          0x00000001
#define ROT_STATUS_SFT          0x0


/*------------------------------*/
/* ROT_SRCSIZE                  */
/*------------------------------*/
#define ROT_SRCSIZE_BIT         0x00000FFF
#define ROT_SRCSIZE_SFT         0x2


/*------------------------------*/
/* ROT_DSTSIZE                  */
/*------------------------------*/
#define ROT_DSTSIZE_BIT         0x00000FFF
#define ROT_DSTSIZE_SFT         0x2


/*------------------------------*/
/* ROT_SRCYADR_A                */
/*------------------------------*/
#define ROT_SRCYADR_A_BIT       0xFFFFFFFFF
#define ROT_SRCYADR_A_SFT       0x2


/*------------------------------*/
/* ROT_DSTYADR_A                */
/*------------------------------*/
#define ROT_DSTYADR_A_BIT       0xFFFFFFFFF
#define ROT_DSTYADR_A_SFT       0x2


/*------------------------------*/
/* ROT_SRCYADR_B                */
/*------------------------------*/
#define ROT_SRCYADR_B_BIT       0xFFFFFFFFF
#define ROT_SRCYADR_B_SFT       0x2


/*------------------------------*/
/* ROT_DSTYADR_B                */
/*------------------------------*/
#define ROT_DSTYADR_B_BIT       0xFFFFFFFFF
#define ROT_DSTYADR_B_SFT       0x2


/*------------------------------*/
/* ROT_SRCYADR_C                */
/*------------------------------*/
#define ROT_SRCYADR_C_BIT       0xFFFFFFFFF
#define ROT_SRCYADR_C_SFT       0x2


/*------------------------------*/
/* ROT_DSTYADR_C                */
/*------------------------------*/
#define ROT_DSTYADR_C_BIT       0xFFFFFFFFF
#define ROT_DSTYADR_C_SFT       0x2


/*------------------------------*/
/* ROT_SRCUVADR_A                */
/*------------------------------*/
#define ROT_SRCUVADR_A_BIT      0xFFFFFFFFF
#define ROT_SRCUVADR_A_SFT      0x2


/*------------------------------*/
/* ROT_DSTUVADR_A                */
/*------------------------------*/
#define ROT_DSTUVADR_A_BIT      0xFFFFFFFFF
#define ROT_DSTUVADR_A_SFT      0x2


/*------------------------------*/
/* ROT_SRCUVADR_B                */
/*------------------------------*/
#define ROT_SRCUVADR_B_BIT      0xFFFFFFFFF
#define ROT_SRCUVADR_B_SFT      0x2


/*------------------------------*/
/* ROT_DSTUVADR_B                */
/*------------------------------*/
#define ROT_DSTUVADR_B_BIT      0xFFFFFFFFF
#define ROT_DSTUVADR_B_SFT      0x2


/*------------------------------*/
/* ROT_SRCUVADR_C                */
/*------------------------------*/
#define ROT_SRCUVADR_C_BIT      0xFFFFFFFFF
#define ROT_SRCUVADR_C_SFT      0x2


/*------------------------------*/
/* ROT_DSTUVADR_C                */
/*------------------------------*/
#define ROT_DSTUVADR_C_BIT      0xFFFFFFFFF
#define ROT_DSTUVADR_C_SFT      0x2


/*------------------------------*/
/* ROT_SRCVADR_A                */
/*------------------------------*/
#define ROT_SRCVADR_A_BIT       0xFFFFFFFFF
#define ROT_SRCVADR_A_SFT       0x2


/*------------------------------*/
/* ROT_DSTVADR_A                */
/*------------------------------*/
#define ROT_DSTVADR_A_BIT       0xFFFFFFFFF
#define ROT_DSTVADR_A_SFT       0x2


/*------------------------------*/
/* ROT_SRCVADR_B                */
/*------------------------------*/
#define ROT_SRCVADR_B_BIT       0xFFFFFFFFF
#define ROT_SRCVADR_B_SFT       0x2


/*------------------------------*/
/* ROT_DSTVADR_B                */
/*------------------------------*/
#define ROT_DSTVADR_B_BIT       0xFFFFFFFFF
#define ROT_DSTVADR_B_SFT       0x2


/*------------------------------*/
/* ROT_SRCVADR_C                */
/*------------------------------*/
#define ROT_SRCVADR_C_BIT       0xFFFFFFFFF
#define ROT_SRCVADR_C_SFT       0x2


/*------------------------------*/
/* ROT_DSTVADR_C                */
/*------------------------------*/
#define ROT_DSTVADR_C_BIT       0xFFFFFFFFF
#define ROT_DSTVADR_C_SFT       0x2


/*------------------------------*/
/* ROT_SRCHSIZE                 */
/*------------------------------*/
#define ROT_SRCHSIZE_BIT        0x00000FFF
#define ROT_SRCHSIZE_SFT        0x2


/*------------------------------*/
/* ROT_SRCVSIZE                 */
/*------------------------------*/
#define ROT_SRCVSIZE_BIT        0x00000FFF
#define ROT_SRCVSIZE_SFT        0x2


/*------------------------------*/
/* ROT_INTSTATUS                */
/*------------------------------*/
#define ROT_ACTIVE_BIT          0x00000010
#define ROT_ACTIVE_SFT          0x04

#define ROT_DFF_RSV_BIT         0x00000008
#define ROT_DFF_RSV_SFT         0x03

#define ROT_DMASTOP_BIT         0x00000004
#define ROT_DMASTOP_SFT         0x02

#define ROT_DMAERR_BIT          0x00000002
#define ROT_DMAERR_SFT          0x01

#define ROT_ROTEND_BIT          0x00000001
#define ROT_ROTEND_SFT          0x00


/*------------------------------*/
/* ROT_INTRAWSTATUS             */
/*------------------------------*/
#define ROT_DMASTOPRAW_BIT      0x00000004
#define ROT_DMASTOPRAW_SFT      0x02

#define ROT_DMAERRRAW_BIT       0x00000002
#define ROT_DMAERRRAW_SFT       0x01

#define ROT_ROTENDRAW_BIT       0x00000001
#define ROT_ROTENDRAW_SFT       0x00


/*------------------------------*/
/* ROT_INTENSET                 */
/*------------------------------*/
#define ROT_DMASTOPEN_BIT       0x00000004
#define ROT_DMASTOPEN_SFT       0x02

#define ROT_DMAERREN_BIT        0x00000002
#define ROT_DMAERREN_SFT        0x01

#define ROT_ROTENDEN_BIT        0x00000001
#define ROT_ROTENDEN_SFT        0x00


/*------------------------------*/
/* ROT_INTENCLR                 */
/*------------------------------*/
#define ROT_DMASTOPMASK_BIT     0x00000004
#define ROT_DMASTOPMASK_SFT     0x02

#define ROT_DMAERRMASK_BIT      0x00000002
#define ROT_DMAERRMASK_SFT      0x01

#define ROT_ROTENDMASK_BIT      0x00000001
#define ROT_ROTENDMASK_SFT      0x00


/*------------------------------*/
/* ROT_INTFFCLR                 */
/*------------------------------*/
#define ROT_DMASTOPCLR_BIT      0x00000004
#define ROT_DMASTOPCLR_SFT      0x02

#define ROT_DMAERRCLR_BIT       0x00000002
#define ROT_DMAERRCLR_SFT       0x01

#define ROT_ROTENDCLR_BIT       0x00000001
#define ROT_ROTENDCLR_SFT       0x00


/*------------------------------*/
/* ROT_ERRORADR                 */
/*------------------------------*/
#define ROT_ERRADR_BIT          0xFFFFFFFC
#define ROT_ERRADR_SFT          0x2

#define ROT_LOCK_BIT            0x00000001
#define ROT_LOCK_SFT            0x0


/*------------------------------*/
/* ROT_FORMAT                   */
/* (temporary declaration)      */
/*------------------------------*/
#define ROT_FORMAT_RGB888             0x0
#define ROT_FORMAT_RGB565             0x2
#define ROT_FORMAT_YUV420_PLANAR      0x4
#define ROT_FORMAT_YUV420_SEMI_PLANAR 0x5
#define ROT_FORMAT_YUV422_SEMI_PLANAR 0x9
#define ROT_FORMAT_YUV422_INTERLEAVE  0xA

/*------------------------------*/
/* ROT_FORMAT                   */
/*------------------------------*/
#define ROT_DSTFMT_BIT                0x000000F0
#define ROT_DSTFMT_SFT                0x4
#define ROT_DSTFMT_RGB888             (ROT_FORMAT_RGB888             << ROT_DSTFMT_SFT)
#define ROT_DSTFMT_RGB565             (ROT_FORMAT_RGB565             << ROT_DSTFMT_SFT)
#define ROT_DSTFMT_YUV420_PLANAR      (ROT_FORMAT_YUV420_PLANAR      << ROT_DSTFMT_SFT)
#define ROT_DSTFMT_YUV420_SEMI_PLANAR (ROT_FORMAT_YUV420_SEMI_PLANAR << ROT_DSTFMT_SFT)
#define ROT_DSTFMT_YUV422_SEMI_PLANAR (ROT_FORMAT_YUV422_SEMI_PLANAR << ROT_DSTFMT_SFT)
#define ROT_DSTFMT_YUV422_INTERLEAVE  (ROT_FORMAT_YUV422_INTERLEAVE  << ROT_DSTFMT_SFT)

#define ROT_SRCFMT_BIT                0x0000000F
#define ROT_SRCFMT_SFT                0x0
#define ROT_SRCFMT_RGB888             (ROT_FORMAT_RGB888             << ROT_SRCFMT_SFT)
#define ROT_SRCFMT_RGB565             (ROT_FORMAT_RGB565             << ROT_SRCFMT_SFT)
#define ROT_SRCFMT_YUV420_PLANAR      (ROT_FORMAT_YUV420_PLANAR      << ROT_SRCFMT_SFT)
#define ROT_SRCFMT_YUV420_SEMI_PLANAR (ROT_FORMAT_YUV420_SEMI_PLANAR << ROT_SRCFMT_SFT)
#define ROT_SRCFMT_YUV422_SEMI_PLANAR (ROT_FORMAT_YUV422_SEMI_PLANAR << ROT_SRCFMT_SFT)
#define ROT_SRCFMT_YUV422_INTERLEAVE  (ROT_FORMAT_YUV422_INTERLEAVE  << ROT_SRCFMT_SFT)


/*------------------------------*/
/* BYTELANE                     */
/* (temporary declaration)      */
/*------------------------------*/
#define ROT_BYTELANE_DATAx_BYTE0  0x0
#define ROT_BYTELANE_DATAx_BYTE1  0x1
#define ROT_BYTELANE_DATAx_BYTE2  0x2
#define ROT_BYTELANE_DATAx_BYTE3  0x3

/*------------------------------*/
/* ROT_SRCBYTE                  */
/*------------------------------*/
#define ROT_SRCBYTE_DATA3_BIT     0x000000C0
#define ROT_SRCBYTE_DATA3_SFT     0x6
#define ROT_SRCBYTE_DATA3_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_SRCBYTE_DATA3_SFT)
#define ROT_SRCBYTE_DATA3_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_SRCBYTE_DATA3_SFT)
#define ROT_SRCBYTE_DATA3_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_SRCBYTE_DATA3_SFT)
#define ROT_SRCBYTE_DATA3_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_SRCBYTE_DATA3_SFT)

#define ROT_SRCBYTE_DATA2_BIT     0x00000030
#define ROT_SRCBYTE_DATA2_SFT     0x4
#define ROT_SRCBYTE_DATA2_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_SRCBYTE_DATA2_SFT)
#define ROT_SRCBYTE_DATA2_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_SRCBYTE_DATA2_SFT)
#define ROT_SRCBYTE_DATA2_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_SRCBYTE_DATA2_SFT)
#define ROT_SRCBYTE_DATA2_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_SRCBYTE_DATA2_SFT)

#define ROT_SRCBYTE_DATA1_BIT     0x0000000C
#define ROT_SRCBYTE_DATA1_SFT     0x2
#define ROT_SRCBYTE_DATA1_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_SRCBYTE_DATA1_SFT)
#define ROT_SRCBYTE_DATA1_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_SRCBYTE_DATA1_SFT)
#define ROT_SRCBYTE_DATA1_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_SRCBYTE_DATA1_SFT)
#define ROT_SRCBYTE_DATA1_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_SRCBYTE_DATA1_SFT)

#define ROT_SRCBYTE_DATA0_BIT     0x00000003
#define ROT_SRCBYTE_DATA0_SFT     0x0
#define ROT_SRCBYTE_DATA0_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_SRCBYTE_DATA0_SFT)
#define ROT_SRCBYTE_DATA0_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_SRCBYTE_DATA0_SFT)
#define ROT_SRCBYTE_DATA0_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_SRCBYTE_DATA0_SFT)
#define ROT_SRCBYTE_DATA0_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_SRCBYTE_DATA0_SFT)


/*------------------------------*/
/* ROT_DSTBYTE                  */
/*------------------------------*/
#define ROT_DSTBYTE_DATA3_BIT     0x000000C0
#define ROT_DSTBYTE_DATA3_SFT     0x6
#define ROT_DSTBYTE_DATA3_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_DSTBYTE_DATA3_SFT)
#define ROT_DSTBYTE_DATA3_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_DSTBYTE_DATA3_SFT)
#define ROT_DSTBYTE_DATA3_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_DSTBYTE_DATA3_SFT)
#define ROT_DSTBYTE_DATA3_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_DSTBYTE_DATA3_SFT)

#define ROT_DSTBYTE_DATA2_BIT     0x00000030
#define ROT_DSTBYTE_DATA2_SFT     0x4
#define ROT_DSTBYTE_DATA2_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_DSTBYTE_DATA2_SFT)
#define ROT_DSTBYTE_DATA2_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_DSTBYTE_DATA2_SFT)
#define ROT_DSTBYTE_DATA2_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_DSTBYTE_DATA2_SFT)
#define ROT_DSTBYTE_DATA2_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_DSTBYTE_DATA2_SFT)

#define ROT_DSTBYTE_DATA1_BIT     0x0000000C
#define ROT_DSTBYTE_DATA1_SFT     0x2
#define ROT_DSTBYTE_DATA1_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_DSTBYTE_DATA1_SFT)
#define ROT_DSTBYTE_DATA1_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_DSTBYTE_DATA1_SFT)
#define ROT_DSTBYTE_DATA1_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_DSTBYTE_DATA1_SFT)
#define ROT_DSTBYTE_DATA1_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_DSTBYTE_DATA1_SFT)

#define ROT_DSTBYTE_DATA0_BIT     0x00000003
#define ROT_DSTBYTE_DATA0_SFT     0x0
#define ROT_DSTBYTE_DATA0_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_DSTBYTE_DATA0_SFT)
#define ROT_DSTBYTE_DATA0_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_DSTBYTE_DATA0_SFT)
#define ROT_DSTBYTE_DATA0_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_DSTBYTE_DATA0_SFT)
#define ROT_DSTBYTE_DATA0_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_DSTBYTE_DATA0_SFT)


/*------------------------------*/
/* ROT_SRCBYTE_CMP              */
/*------------------------------*/
/* ROT_DSTBYTE_CMP              */
/*------------------------------*/
#define ROT_BYTE_DATA3_UV_BIT     0x0000C000
#define ROT_BYTE_DATA3_UV_SFT     0xE
#define ROT_BYTE_DATA3_UV_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA3_UV_SFT)
#define ROT_BYTE_DATA3_UV_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA3_UV_SFT)
#define ROT_BYTE_DATA3_UV_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA3_UV_SFT)
#define ROT_BYTE_DATA3_UV_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA3_UV_SFT)

#define ROT_BYTE_DATA2_UV_BIT     0x00003000
#define ROT_BYTE_DATA2_UV_SFT     0xC
#define ROT_BYTE_DATA2_UV_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA2_UV_SFT)
#define ROT_BYTE_DATA2_UV_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA2_UV_SFT)
#define ROT_BYTE_DATA2_UV_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA2_UV_SFT)
#define ROT_BYTE_DATA2_UV_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA2_UV_SFT)

#define ROT_BYTE_DATA1_UV_BIT     0x00000C00
#define ROT_BYTE_DATA1_UV_SFT     0xA
#define ROT_BYTE_DATA1_UV_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA1_UV_SFT)
#define ROT_BYTE_DATA1_UV_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA1_UV_SFT)
#define ROT_BYTE_DATA1_UV_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA1_UV_SFT)
#define ROT_BYTE_DATA1_UV_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA1_UV_SFT)

#define ROT_BYTE_DATA0_UV_BIT     0x00000300
#define ROT_BYTE_DATA0_UV_SFT     0x8
#define ROT_BYTE_DATA0_UV_BYTE0   (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA0_UV_SFT)
#define ROT_BYTE_DATA0_UV_BYTE1   (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA0_UV_SFT)
#define ROT_BYTE_DATA0_UV_BYTE2   (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA0_UV_SFT)
#define ROT_BYTE_DATA0_UV_BYTE3   (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA0_UV_SFT)

#define ROT_BYTE_DATA3_Y_BIT      0x000000C0
#define ROT_BYTE_DATA3_Y_SFT      0x6
#define ROT_BYTE_DATA3_Y_BYTE0    (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA3_Y_SFT)
#define ROT_BYTE_DATA3_Y_BYTE1    (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA3_Y_SFT)
#define ROT_BYTE_DATA3_Y_BYTE2    (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA3_Y_SFT)
#define ROT_BYTE_DATA3_Y_BYTE3    (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA3_Y_SFT)

#define ROT_BYTE_DATA2_Y_BIT      0x00000030
#define ROT_BYTE_DATA2_Y_SFT      0x4
#define ROT_BYTE_DATA2_Y_BYTE0    (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA2_Y_SFT)
#define ROT_BYTE_DATA2_Y_BYTE1    (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA2_Y_SFT)
#define ROT_BYTE_DATA2_Y_BYTE2    (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA2_Y_SFT)
#define ROT_BYTE_DATA2_Y_BYTE3    (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA2_Y_SFT)

#define ROT_BYTE_DATA1_Y_BIT      0x0000000C
#define ROT_BYTE_DATA1_Y_SFT      0x2
#define ROT_BYTE_DATA1_Y_BYTE0    (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA1_Y_SFT)
#define ROT_BYTE_DATA1_Y_BYTE1    (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA1_Y_SFT)
#define ROT_BYTE_DATA1_Y_BYTE2    (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA1_Y_SFT)
#define ROT_BYTE_DATA1_Y_BYTE3    (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA1_Y_SFT)

#define ROT_BYTE_DATA0_Y_BIT      0x00000003
#define ROT_BYTE_DATA0_Y_SFT      0x0
#define ROT_BYTE_DATA0_Y_BYTE0    (ROT_BYTELANE_DATAx_BYTE0 << ROT_BYTE_DATA0_Y_SFT)
#define ROT_BYTE_DATA0_Y_BYTE1    (ROT_BYTELANE_DATAx_BYTE1 << ROT_BYTE_DATA0_Y_SFT)
#define ROT_BYTE_DATA0_Y_BYTE2    (ROT_BYTELANE_DATAx_BYTE2 << ROT_BYTE_DATA0_Y_SFT)
#define ROT_BYTE_DATA0_Y_BYTE3    (ROT_BYTELANE_DATAx_BYTE3 << ROT_BYTE_DATA0_Y_SFT)


/*------------------------------*/
/* ROT_DUAL_FF                  */
/*------------------------------*/
#define ROT_DUAL_FF_MODE_BIT    0x00000002
#define ROT_DUAL_FF_MODE_SFT    0x1

#define ROT_REG_EN_BIT          0x00000001
#define ROT_REG_EN_SFT          0x0


/*===============================================================*/
/* ROT register reset                                            */
/*===============================================================*/
#define ROT_MODE_RESET         0x00000000
#define ROT_SRCBYTE_RESET      0x000000E4
#define ROT_DSTBYTE_RESET      0x000000E4
#define ROT_DUAL_FF_RESET      0x00000000

#endif /* _ROT_H_ */
