/* 2009-08-24: File added and changed by Sony Corporation */
/*
 *  File Name	    : linux/include/asm-arm/arch-mp200/spi.h
 *  Function	    : MP200 SPI interface
 *  Release Version : Ver 0.01
 *  Release Date    : 2007/07/11
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __ASM_ARM_ARCH_MP200_SPI_H
#define __ASM_ARM_ARCH_MP200_SPI_H


#if defined(CONFIG_MP200_SPI)||defined(CONFIG_MP200_SPI0)||defined(CONFIG_MP200_SPI_KERN)

/* SPI_CONFIG */
typedef struct {
	unsigned char dev;	/* device */
	unsigned int  pol;	/* poll */
	unsigned int  sclk;	/* clock */

/* spi_write_without_intr only */
	unsigned int  size;	/* byte size */
	unsigned int  mode;	/* mode */

/* spi_read/spi_write/spi_cmd_read only */
	unsigned int nbr;	/* bit length (read) */
	unsigned int nbw;	/* bit length (write) */
	unsigned int cs_sel;	/* chip select */
	unsigned int m_s;	/* master/slave */
	unsigned int dma;	/* dma on/off */
	unsigned int tiecs;	/* tiecs */
} SPI_CONFIG;

/* SPI_CONFIG.nbr/nbw (8 to 32) */
/* spi_mode_t.nb */
#define SPI_NB_8BIT		 8
#define SPI_NB_16BIT		16
#define SPI_NB_24BIT		24
#define SPI_NB_32BIT		32

/* SPI_CONFIG.cs_sel */
/* spi_mode_t.cs_sel */
#define SPI_CS_SEL_CS0		0
#define SPI_CS_SEL_CS1		1
#define SPI_CS_SEL_CS2		2
#define SPI_CS_SEL_CS3		3
#define SPI_CS_SEL_CS4		4
#define SPI_CS_SEL_CS5		5

/* SPI_CONFIG.m_s */
/* spi_mode_t.m_s */
#define SPI_M_S_MASTER		0
#define SPI_M_S_SLAVE		1

/* SPI_CONFIG.dma */
/* spi_mode_t.dma */
#define SPI_DMA_OFF		0
#define SPI_DMA_ON		1

/* SPI_CONFIG.sclk */
/* spi_mode_t.sclk */
#define SPI_SCLK_1500KHZ	0
#define SPI_SCLK_3MHZ		1
#define SPI_SCLK_6MHZ		2
#define SPI_SCLK_12MHZ		3
#define SPI_SCLK_24MHZ		4
#define SPI_SCLK_48MHZ		5

/* SPI_CONFIG.tiecs */
#define SPI_TIECS_NORMAL	0
#define SPI_TIECS_FIXED		1

/* flags */
#define SPI_BLOCK		0
#define SPI_NONBLOCK		1
#define SPI_RW_2CYCLE		2

#endif /* defined(CONFIG_MP200_SPI)||defined(CONFIG_MP200_SPI0)||defined(CONFIG_MP200_SPI_KERN) */

#if defined(CONFIG_MP200_SPI)||defined(CONFIG_MP200_SPI_KERN)

/* SPI_CONFIG.dev */
#define SPI_DEV_SPI0		0
#define SPI_DEV_SPI1		1
#define SPI_DEV_SPI2		2

/* SPI_CONFIG.pol */
#define SPI_POL_CSW_16SCLK	(0x0F << 12)
#define SPI_POL_CSW_8SCLK	(0x07 << 12)
#define SPI_POL_CSW_4SCLK	(0x03 << 12)
#define SPI_POL_CSW_2SCLK	(0x01 << 12)

#define SPI_POL_CK5_DLY_ON	(0x01 << 21)
#define SPI_POL_CK5_DLY_OFF	(0x00 << 21)
#define SPI_POL_CK5_POL_NEG	(0x01 << 20)
#define SPI_POL_CK5_POL_POS	(0x00 << 20)
#define SPI_POL_CS5_POL_NEG	(0x01 << 19)
#define SPI_POL_CS5_POL_POS	(0x00 << 19)

#define SPI_POL_CK4_DLY_ON	(0x01 << 18)
#define SPI_POL_CK4_DLY_OFF	(0x00 << 18)
#define SPI_POL_CK4_POL_NEG	(0x01 << 17)
#define SPI_POL_CK4_POL_POS	(0x00 << 17)
#define SPI_POL_CS4_POL_NEG	(0x01 << 16)
#define SPI_POL_CS4_POL_POS	(0x00 << 16)

#define SPI_POL_CK3_DLY_ON	(0x01 << 11)
#define SPI_POL_CK3_DLY_OFF	(0x00 << 11)
#define SPI_POL_CK3_POL_NEG	(0x01 << 10)
#define SPI_POL_CK3_POL_POS	(0x00 << 10)
#define SPI_POL_CS3_POL_NEG	(0x01 << 9)
#define SPI_POL_CS3_POL_POS	(0x00 << 9)

#define SPI_POL_CK2_DLY_ON	(0x01 << 8)
#define SPI_POL_CK2_DLY_OFF	(0x00 << 8)
#define SPI_POL_CK2_POL_NEG	(0x01 << 7)
#define SPI_POL_CK2_POL_POS	(0x00 << 7)
#define SPI_POL_CS2_POL_NEG	(0x01 << 6)
#define SPI_POL_CS2_POL_POS	(0x00 << 6)

#define SPI_POL_CK1_DLY_ON	(0x01 << 5)
#define SPI_POL_CK1_DLY_OFF	(0x00 << 5)
#define SPI_POL_CK1_POL_NEG	(0x01 << 4)
#define SPI_POL_CK1_POL_POS	(0x00 << 4)
#define SPI_POL_CS1_POL_NEG	(0x01 << 3)
#define SPI_POL_CS1_POL_POS	(0x00 << 3)

#define SPI_POL_CK0_DLY_ON	(0x01 << 2)
#define SPI_POL_CK0_DLY_OFF	(0x00 << 2)
#define SPI_POL_CK0_POL_NEG	(0x01 << 1)
#define SPI_POL_CK0_POL_POS	(0x00 << 1)
#define SPI_POL_CS0_POL_NEG	(0x01 << 0)
#define SPI_POL_CS0_POL_POS	(0x00 << 0)

/* SPI_POL: default value */
#define SPI_POL_VAL_SPI0	(SPI_POL_CSW_8SCLK | \
	SPI_POL_CK5_DLY_OFF | SPI_POL_CK5_POL_NEG | SPI_POL_CS5_POL_NEG | \
	SPI_POL_CK4_DLY_OFF | SPI_POL_CK4_POL_NEG | SPI_POL_CS4_POL_NEG | \
	SPI_POL_CK3_DLY_OFF | SPI_POL_CK3_POL_NEG | SPI_POL_CS3_POL_NEG | \
	SPI_POL_CK2_DLY_OFF | SPI_POL_CK2_POL_POS | SPI_POL_CS2_POL_NEG | \
	SPI_POL_CK1_DLY_ON  | SPI_POL_CK1_POL_POS | SPI_POL_CS1_POL_NEG | \
	SPI_POL_CK0_DLY_ON  | SPI_POL_CK0_POL_POS | SPI_POL_CS0_POL_POS)

#define SPI_POL_VAL_SPI1	(SPI_POL_CSW_8SCLK | \
	SPI_POL_CK5_DLY_OFF | SPI_POL_CK5_POL_NEG | SPI_POL_CS5_POL_NEG | \
	SPI_POL_CK4_DLY_OFF | SPI_POL_CK4_POL_NEG | SPI_POL_CS4_POL_NEG | \
	SPI_POL_CK3_DLY_OFF | SPI_POL_CK3_POL_NEG | SPI_POL_CS3_POL_NEG | \
	SPI_POL_CK2_DLY_OFF | SPI_POL_CK2_POL_NEG | SPI_POL_CS2_POL_NEG | \
	SPI_POL_CK1_DLY_OFF | SPI_POL_CK1_POL_NEG | SPI_POL_CS1_POL_NEG | \
	SPI_POL_CK0_DLY_OFF | SPI_POL_CK0_POL_NEG | SPI_POL_CS0_POL_NEG)

#define SPI_POL_VAL_SPI2	(SPI_POL_CSW_8SCLK | \
	SPI_POL_CK5_DLY_OFF | SPI_POL_CK5_POL_NEG | SPI_POL_CS5_POL_NEG | \
	SPI_POL_CK4_DLY_OFF | SPI_POL_CK4_POL_NEG | SPI_POL_CS4_POL_NEG | \
	SPI_POL_CK3_DLY_OFF | SPI_POL_CK3_POL_NEG | SPI_POL_CS3_POL_NEG | \
	SPI_POL_CK2_DLY_OFF | SPI_POL_CK2_POL_NEG | SPI_POL_CS2_POL_NEG | \
	SPI_POL_CK1_DLY_OFF | SPI_POL_CK1_POL_NEG | SPI_POL_CS1_POL_NEG | \
	SPI_POL_CK0_DLY_OFF | SPI_POL_CK0_POL_NEG | SPI_POL_CS0_POL_NEG)

/* rst_flag */
#define SPI_RESET_OFF		0
#define SPI_RESET_ON		1

extern int
spi_write_without_intr(const SPI_CONFIG *config, const char *buf, int rst_flag);

/*
 * spi mode info
 */
typedef struct {
	unsigned int nb;	/* bit length */
	unsigned int cs_sel;	/* chip select */
	unsigned int m_s;	/* master/slave */
	unsigned int dma;	/* dma on/off */
	unsigned int blksize;	/* block size */
	unsigned int sclk;	/* clock */
} spi_mode_t;

/* SPI_CONFIG.mode */
#define SPI_MODE_NB_8BIT	(0x07 << 8)
#define SPI_MODE_NB_16BIT	(0x0F << 8)
#define SPI_MODE_NB_24BIT	(0x17 << 8)
#define SPI_MODE_NB_32BIT	(0x1F << 8)

#define SPI_MODE_CS_SEL_CS0	(0x00 << 4)
#define SPI_MODE_CS_SEL_CS1	(0x01 << 4)
#define SPI_MODE_CS_SEL_CS2	(0x02 << 4)
#define SPI_MODE_CS_SEL_CS3	(0x03 << 4)
#define SPI_MODE_CS_SEL_CS4	(0x04 << 4)
#define SPI_MODE_CS_SEL_CS5	(0x05 << 4)

#define SPI_MODE_M_S_MASTER	(0x00 << 1)
#define SPI_MODE_M_S_SLAVE	(0x01 << 1)

#define SPI_MODE_DMA_OFF	(0x00 << 0)
#define SPI_MODE_DMA_ON		(0x01 << 0)

/* read/write mode*/
#define SPI_READ_MODE      1
#define SPI_WRITE_MODE     2

/*
 * operation command
 */
#define SPI_CMD_RX_START	_IO('s', 0x01)
#define SPI_CMD_RX_STOP		_IO('s', 0x02)

/*
 * setting command
 */
#define SPI_CMD_GET_MODE	_IOR('s', 0x11, spi_mode_t)
#define SPI_CMD_SET_MODE	_IOW('s', 0x12, spi_mode_t)

#endif /* defined(CONFIG_MP200_SPI)||defined(CONFIG_MP200_SPI_KERN) */

#ifdef CONFIG_MP200_SPI
extern int mp201_spi_read(unsigned char dev,char *buf,
			  unsigned int count, unsigned char block_mode);

extern int mp201_spi_write(unsigned char dev, const char *buf,
			   unsigned int count, unsigned char block_mode);

extern int mp201_spi_init(unsigned char dev, unsigned char rw_mode);

extern int mp201_spi_end(unsigned char dev);

extern int mp201_spi_setmode(unsigned char dev, spi_mode_t *mode);

extern int mp201_spi_getmode(unsigned char dev, spi_mode_t *mode);
#endif /* CONFIG_MP200_SPI */


#ifdef CONFIG_MP200_SPI0

/* SPI_CONFIG.dev */
#define SPI_DEV_SP0		0
#define SPI_DEV_SP1		1
#define SPI_DEV_SP2		2

/* SPI_CONFIG.pol */
#define SPI_CSW_1CLK		(0 << 12)
#define SPI_CSW_2CLK		(1 << 12)
#define SPI_CSW_4CLK		(3 << 12)
#define SPI_CSW_8CLK		(7 << 12)
#define SPI_CSW_16CLK		(15 << 12)

#define SPI_CK_DLY_OFF		(0 << 2)
#define SPI_CK_DLY_ON		(1 << 2)

#define SPI_CK_POL_POS		(0 << 1)
#define SPI_CK_POL_NEG		(1 << 1)

#define SPI_CS_POL_POS		(0 << 0)
#define SPI_CS_POL_NEG		(1 << 0)

/* SPI_CONFIG.pol : default value */
#if defined(CONFIG_PGPF_NAVU2)
#include <asm/arch/pgpf/navu2/spi.h>
#else

#define SPI_POL_SP0_CS0		\
	(SPI_CSW_8CLK | SPI_CK_DLY_ON | SPI_CK_POL_POS | SPI_CS_POL_POS)
#define SPI_POL_SP0_CS1		\
	(SPI_CSW_8CLK | SPI_CK_DLY_ON | SPI_CK_POL_POS | SPI_CS_POL_NEG)
#define SPI_POL_SP0_CS2		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_POS | SPI_CS_POL_NEG)
#define SPI_POL_SP0_CS3		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP0_CS4		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP0_CS5		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)

#define SPI_POL_SP1_CS0		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP1_CS1		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP1_CS2		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP1_CS3		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP1_CS4		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP1_CS5		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)

#define SPI_POL_SP2_CS0		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP2_CS1		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP2_CS2		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP2_CS3		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP2_CS4		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)
#define SPI_POL_SP2_CS5		\
	(SPI_CSW_8CLK | SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG)

#endif

extern int spi_read(SPI_CONFIG * config, char *buf, unsigned long phys,
		    unsigned int count, unsigned int flags);
extern int spi_write(SPI_CONFIG * config, char *buf, unsigned long phys,
		     unsigned int count, unsigned int flags);
extern int spi_cmd_read(SPI_CONFIG * config, char *cmd, char *buf,
			unsigned int flags);

#endif /* CONFIG_MP200_SPI0 */


#endif				/* __ASM_ARM_ARCH_MP200_SPI_H */
