/* 2010-08-25: File added by Sony Corporation */
/*
 * omap35x_dc.h - header for OMAP35XDC I2C devices and GPMC configuration
 *
 * Copyright (C) 2008-2009 Texas Instruments, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef __OMAP35X_DC_H_
#define __OMAP35X_DC_H_

/* I2C device ID present in the OMAP35X daughter card */
/* TVP5146 ID */
#define OMAP35XDC_MODULE_TVP5146	0x00
/* AIC23 ID */
#define OMAP35XDC_MODULE_AIC23		0x01

/*
 * OMAP35X daughter card mux id to enable/disable signal routing to different
 * peripherals present
 */
enum omap35xdc_mux {
	MUX_TVP5146 = 0,
	MUX_AIC23_MCBSP2,
	MUX_SPDIF_MCBSP2,
	MUX_HDMI_MCBSP2,
	MUX_BT_MCBSP3,
	MUX_JAC_MCSPI2,
	MUX_BT_UART1,
	MUX_MLC_NAND,		/* not connected */
	MUX_JAC_MCBSP1,
	MUX_IMAGE_SENSOR,
	MUX_JAC_MCBSP3,
	MUX_SPDIF_MCSPI2,
	NUM_MUX
};

/* enum to enable or disable mux */
enum config_mux {
	DISABLE_MUX,
	ENABLE_MUX
};

/* Functions to read and write from OMAP35XDC I2C devices */

/*
 * IMP NOTE:
 * It is the caller's responsibility to ensure sane values
 */
int omap35xdc_i2c_write_u8(u8 mod_no, u8 reg, u8 value);
int omap35xdc_i2c_read_u8(u8 mod_no, u8 reg, u8 *value);
/*
 * i2c_write: IMPORTANT - Allocate value num_bytes+1 and valid data starts at
 *		Offset 1.
 */
int omap35xdc_i2c_write(u8 mod_no, u8 reg, u8 *value, u8 num_bytes);
int omap35xdc_i2c_read(u8 mod_no, u8 reg, u8 *value, u8 num_bytes);

/*
 * Sets OMAP35X daughter card mux to enable/disable signal routing to different
 * peripherals present. This function will take care of writing appropriate
 * values for active low signals as well
 */
int omap35xdc_set_mux(enum omap35xdc_mux mux_id, enum config_mux value);

#endif /* End of __OMAP35X_DC_H */
