/* 2010-08-25: File added by Sony Corporation */
/*
 * include/asm-arm/arch-omap/twl4030-madc.h
 *
 * TWL4030 MADC interface for Linux
 *
 * Copyright (C) 2008 Texas Instruments, Inc.
 *
 * Author: Texas Instruments, Inc.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef __TWL4030_MADC_H__
#define __TWL4030_MADC_H__

/**
 * Madc Channels. Some channels are reserved for
 * internal use by BCI and USB.Some are available as
 * general purpose inputs.
 */
enum madc_channel {
	ADCIN0 = 0,	/* BCI : Battery Type */
	ADCIN1,		/* BCI : Battery Temperature */
	ADCIN2,		/* General-purpose analog input */
	ADCIN3,		/* General-purpose analog input */
	ADCIN4,		/* General-purpose analog input */
	ADCIN5,		/* General-purpose analog input */
	ADCIN6,		/* General-purpose analog input */
	ADCIN7,		/* General-purpose analog input */
	ADCIN8,		/* BCI : VBUS voltage */
	ADCIN9,		/* BCI : Backup Battery Voltage */
	ADCIN10,	/* BCI : Battery Charger Current */
	ADCIN11,	/* BCI : Battery Charger Voltage */
	ADCIN12,	/* BCI : Main Battery Volatge */
	ADCIN13,	/* Reserved */
	ADCIN14,	/* Reserved */
	ADCIN15,	/* VRBUS supply or speaker right or
			 * speaker left polarization level
			 */
};

/**
 * This structure contains the channel information.
 * The number of the channel to be converted , the converted
 * data to be populated by madc driver after conversion
 *  is done and whether averaging has to be done during
 * conversion.One structure instance has to be populated
 * per channel to be converted.
 */
struct madc_chrequest {
	enum madc_channel channel;
	unsigned int data;
	int is_average;
};

/**
 * This is the structure to be passed from user
 * side to the ioctl for SW1 conversion on madc
 * channels.It contains an array of madc_chrequest
 * structures containing the channel information and
 * the number of elements in the array
 */
struct madc_swconvrequest {
	struct madc_chrequest *request;
	unsigned int no_request;
};

/*
 *	I O C T L   C O D E S   F O R  TWL4030MADC
 */
#define	MADC_SW1CONVERT		_IOWR ('G', 32, struct madc_swconvrequest)

#endif	/* __TWL4030_MADC_H__ */
