/* 2009-08-24: File added and changed by Sony Corporation */
/*
 *  Copyright 2009 Sony Corporation.
 *
 *  This program is based on sound/arm/mp200-mixer.c
 *    and sound/arm/mxc-alsa-pmic.c
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* here is the "mp200-mixer.c" original copyright. */
/*
 * linux/sound/mp200-mixer.c -- sound codec support for mp200 chip.
 *
 * Created:	Oct 09, 2006
 *
 */

/* here is the "mxc-alsa-pmic.c" original copyright. */
/*
 * Copyright 2004-2007 Freescale Semiconductor, Inc. All Rights Reserved.
 */

#include <linux/errno.h>
#include <linux/workqueue.h>
#include <linux/timer.h>
#include <asm/arch/pwc.h>
#include <asm/io.h>
#include <sound/driver.h>
#include <sound/core.h>
#include <sound/control.h>
#include <sound/pcm.h>
#include <sound/initval.h>

#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include "mp200-pcm.h"

#include <asm/arch/pmic_external.h>
#include <asm/arch/pmic_audio.h>
#include <asm/arch/pmic_power.h>
#include <linux/soundcard.h>
#include "mxc-alsa-pmic.h"
#include "mxc-alsa-common.h"

#ifdef AUDIO_MAKING_DEBUG
#include <linux/moduleparam.h>
#endif


/*** debug code by the making ->*/
#ifdef AUDIO_MAKING_DEBUG
#define FNC_ENTRY	\
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "entry:%s\n", __FUNCTION__); \
	}
#define FNC_EXIT	\
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "exit:%s:%d\n", __FUNCTION__ , __LINE__); \
	}
#define d0b(fmt, args...)	\
		printk(KERN_DEBUG "%s:%d: " fmt , __FUNCTION__ , __LINE__ , ## args);
#define d1b(fmt, args...)	\
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "%s:%d: " fmt , __FUNCTION__ , __LINE__ , ## args); \
	}
#define d7b(fmt, args...)	\
	if (debug == 7 || debug >= 9) {	\
		printk(KERN_DEBUG "%s:%d: " fmt , __FUNCTION__ , __LINE__ , ## args); \
	}
#define d8b(fmt, args...)	\
	if (debug == 8 || debug >= 9) {	\
		printk(KERN_DEBUG "%s:%d: " fmt , __FUNCTION__ , __LINE__ , ## args); \
	}
#define d9b(fmt, args...)	\
	if (debug == 9 || debug >= 9) {	\
		printk(KERN_DEBUG fmt , ## args); \
	}
extern int debug;
#else
#define FNC_ENTRY
#define FNC_EXIT
#define d0b(fmt, args...)
#define d1b(fmt, args...)
#define d7b(fmt, args...)
#define d8b(fmt, args...)
#define d9b(fmt, args...)
#endif
/*<- debug code by the making ***/

#include <linux/delay.h>

#define STDAC_CLI_FREQ	STDAC_CLI_15_36MHZ
#define VCODEC_CLI_FREQ	VCODEC_CLI_15_36MHZ


/* codec intefaces */
static int navu2_codec_startup(struct snd_pcm_substream *substream);
static void navu2_codec_shutdown(struct snd_pcm_substream *substream);
static int navu2_codec_prepare(struct snd_pcm_substream *substream);

/* callback headphone detect interrupt */
static void navu2_set_output_setting(void);
static void navu2_hsdet_callback_work(struct work_struct *work);

/*!
  * This structure is the global configuration of the soundcard
  * that are accessed by the mixer as well as by the playback/recording
  * stream. This contains various volume, balance, mono adder settings
  *
  */
typedef struct audio_mixer_control {

	/*!
	 * This variable holds the current active output device(s)
	 */
	int output_device;

	/*!
	 * This variable holds the current active input device.
	 */
	int input_device;

	/* Used only for playback/recording on codec .. Use 1 for playback
	 * and 0 for recording*/
	int direction;

	/*!
	 * This variable holds the current source for active ouput device(s)
	 */
	OUTPUT_SOURCE source_for_output[OP_MAXDEV];

	/*!
	 * This variable says if a given output device is part of an ongoing
	 * playback. This variable will be set and reset by the playback stream
	 * when stream is activated and when stream is closed. This shall also
	 * be set and reset my mixer functions for enabling/disabling output devs
	 */
	int output_active[OP_MAXDEV];

	/*!
	 * This variable holds the current volume for active input device.
	 * This maps to the input gain of recording device
	 */
	int input_volume;

	/*!
	 * This variable holds the current volume for playback devices.
	 */
	//int output_volume[OP_MAXDEV];
	int master_volume_out;
	/*!
	 * Stereo DAC playback volume
	 */
	int stdac_volume_out;
	/*!
	 * Voice Codec playback volume
	 */
	int vcodec_volume_out;

	/*!
	 * This variable holds the balance setting for the mixer out.
	 * The range is 0 to 100. 50 means both L and R equal.
	 * < 50 attenuates left side and > 50 attenualtes right side
	 */
	int mixer_balance;

	/*!
	 * This variable holds the current mono adder config.
	 */
	PMIC_AUDIO_MONO_ADDER_MODE mixer_mono_adder;

	/*!
	 * Semaphore used to control the access to this structure.
	 */
	struct semaphore sem;

	/*!
	 * These variables are set by PCM stream and mixer when the voice codec's / ST dac's outputs are
	 * connected to the analog mixer of PMIC audio chip
	 */
	int codec_out_to_mixer;
	int stdac_out_to_mixer;

	int codec_playback_active;
	int codec_capture_active;
	int stdac_playback_active;

	int stdac_active;
	int vcodec_active;

	int stdac_unmute;
	int vcodec_unmute;

	int stdac_forceon;
	int vcodec_forceon;

	PMIC_AUDIO_HANDLE stdac_handle;
	PMIC_AUDIO_HANDLE voice_codec_handle;

	struct delayed_work late_init_wq; /* used for initialization/resuming */
	struct workqueue_struct *callback_wq;
	struct work_struct callback_work;
} audio_mixer_control_t;

/*!
 * pmic audio chip parameters for IP/OP and volume controls
 */
audio_mixer_control_t audio_mixer_control;




/*!
  * This function sets the input device in PMIC. It takes an
  * ALSA value and modifies registers using pmic-specific values.
  *
  * @param       handle  Handle to the PMIC device opened
  * @param       val     ALSA value. This value defines the input device that
  *                      PMIC should activate to get audio signal (recording)
  * @param       enable  Whether to enable or diable the input
  */
int set_mixer_input_device(PMIC_AUDIO_HANDLE handle, INPUT_DEVICES dev,
			   bool enable)
{

	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	if (handle != NULL) {
		if (audio_mixer_control.input_device & SOUND_MASK_PHONEIN) {
			pmic_audio_vcodec_set_mic(handle, MIC1_LEFT,
						  MIC1_RIGHT_MIC_MONO);
			pmic_audio_vcodec_enable_micbias(handle, MIC_BIAS1);
		} else {
			pmic_audio_vcodec_set_mic_on_off(handle,
							 MIC1_LEFT,
							 MIC1_RIGHT_MIC_MONO);
			pmic_audio_vcodec_disable_micbias(handle, MIC_BIAS1);
		}
		if (audio_mixer_control.input_device & SOUND_MASK_MIC) {
			pmic_audio_vcodec_set_mic(handle, NO_MIC, MIC2_AUX);
			pmic_audio_vcodec_enable_micbias(handle, MIC_BIAS2);
		} else {
			pmic_audio_vcodec_set_mic_on_off(handle, NO_MIC,
							 MIC2_AUX);
			pmic_audio_vcodec_disable_micbias(handle, MIC_BIAS2);
		}
		if (audio_mixer_control.input_device & SOUND_MASK_LINE) {
			pmic_audio_vcodec_set_mic(handle, NO_MIC, TXIN_EXT);
		} else {
			pmic_audio_vcodec_set_mic_on_off(handle, NO_MIC,
							 TXIN_EXT);
		}
		up(&audio_mixer_control.sem);
		return 0;

	}
	switch (dev) {
	case IP_HANDSET:
		pr_debug("Input: SOUND_MASK_PHONEIN \n");
		if (handle == NULL) {
			if (enable) {
				if (audio_mixer_control.codec_capture_active) {
					handle =
					    audio_mixer_control.
					    voice_codec_handle;
					pmic_audio_vcodec_set_mic(handle,
								  MIC1_LEFT,
								  MIC1_RIGHT_MIC_MONO);
					pmic_audio_vcodec_enable_micbias(handle,
									 MIC_BIAS1);
				}
				audio_mixer_control.input_device |=
				    SOUND_MASK_PHONEIN;
			} else {
				if (audio_mixer_control.codec_capture_active) {
					handle =
					    audio_mixer_control.
					    voice_codec_handle;
					pmic_audio_vcodec_set_mic_on_off(handle,
									 MIC1_LEFT,
									 MIC1_RIGHT_MIC_MONO);
					pmic_audio_vcodec_disable_micbias
					    (handle, MIC_BIAS1);
				}
				audio_mixer_control.input_device &=
				    ~SOUND_MASK_PHONEIN;
			}
		}
		break;

	case IP_HEADSET:
		if (handle == NULL) {
			if (enable) {
				if (audio_mixer_control.codec_capture_active) {
					handle =
					    audio_mixer_control.
					    voice_codec_handle;
					pmic_audio_vcodec_set_mic(handle,
								  NO_MIC,
								  MIC2_AUX);
					pmic_audio_vcodec_enable_micbias(handle,
									 MIC_BIAS2);
				}
				audio_mixer_control.input_device |=
				    SOUND_MASK_MIC;
			} else {
				if (audio_mixer_control.codec_capture_active) {
					handle =
					    audio_mixer_control.
					    voice_codec_handle;
					pmic_audio_vcodec_set_mic_on_off(handle,
									 NO_MIC,
									 MIC2_AUX);
					pmic_audio_vcodec_disable_micbias
					    (handle, MIC_BIAS2);
				}
				audio_mixer_control.input_device &=
				    ~SOUND_MASK_MIC;
			}
			// Enable Mic with MIC2_AUX
		}
		break;

	case IP_LINEIN:
		if (handle == NULL) {
			if (enable) {
				if (audio_mixer_control.codec_capture_active) {
					handle =
					    audio_mixer_control.
					    voice_codec_handle;
					pmic_audio_vcodec_set_mic(handle,
								  NO_MIC,
								  TXIN_EXT);
				}
				audio_mixer_control.input_device |=
				    SOUND_MASK_LINE;
			} else {
				if (audio_mixer_control.codec_capture_active) {
					handle =
					    audio_mixer_control.
					    voice_codec_handle;
					pmic_audio_vcodec_set_mic_on_off(handle,
									 NO_MIC,
									 TXIN_EXT);
				}
				audio_mixer_control.input_device &=
				    ~SOUND_MASK_LINE;
			}
		}
		break;

	default:
		up(&audio_mixer_control.sem);
		return -1;
		break;
	}
	up(&audio_mixer_control.sem);
	return 0;
}

int get_mixer_input_device()
{
	int val;
	val = audio_mixer_control.input_device;
	return val;
}

/*!
  * This function sets the PMIC input device's gain.
  * Note that the gain is the input volume
  *
  * @param       handle  Handle to the opened PMIC device
  * @param       val     gain to be applied. This value can go
  *                      from 0 (mute) to 100 (max gain)
  */
int set_mixer_input_gain(PMIC_AUDIO_HANDLE handle, int val)
{
	int leftdb, rightdb;
	int left, right;

	left = (val & 0x00ff);
	right = ((val & 0xff00) >> 8);
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	leftdb = (left * PMIC_INPUT_VOLUME_MAX) / INPUT_VOLUME_MAX;
	rightdb = (right * PMIC_INPUT_VOLUME_MAX) / INPUT_VOLUME_MAX;
	audio_mixer_control.input_volume = val;
	if (audio_mixer_control.voice_codec_handle == handle) {
		pmic_audio_vcodec_set_record_gain(handle, VOLTAGE_TO_VOLTAGE,
						  leftdb, VOLTAGE_TO_VOLTAGE,
						  rightdb);
	} else if ((handle == NULL)
		   && (audio_mixer_control.codec_capture_active)) {
		pmic_audio_vcodec_set_record_gain(audio_mixer_control.
						  voice_codec_handle,
						  VOLTAGE_TO_VOLTAGE, leftdb,
						  VOLTAGE_TO_VOLTAGE, rightdb);
	}
	up(&audio_mixer_control.sem);
	return 0;
}

int get_mixer_input_gain()
{
	int val;
	val = audio_mixer_control.input_volume;
	return val;
}

/*!
  * This function sets the PMIC output device's volume.
  *
  * @param       handle  Handle to the PMIC device opened
  * @param       volume  ALSA value. This value defines the playback volume
  * @param       dev     which output device gets affected by this volume
  * @param       is_hp   if  1, set headphone gain setting
  * @param               if  0, set speaker gain setting
  * @param               if -1, detect headpone in this function and set gain prolerly
  *
  */

static int navu2_set_mixer_output_volume(PMIC_AUDIO_HANDLE handle, int volume,
					 OUTPUT_DEVICES dev, int is_hp)
{
	int left, right;
	int s_right, s_left, v_right, v_left;
	int s_leftdb, s_rightdb, v_leftdb, v_rightdb;
	t_sensor_bits s;

	left = (volume & 0x00ff);
	right = ((volume & 0xff00) >> 8);

	s_left = (left * (audio_mixer_control.stdac_volume_out & 0x00ff)) / 100;
	s_right = (right * ((audio_mixer_control.stdac_volume_out & 0xff00) >> 8)) / 100;
	s_leftdb = (s_left * PMIC_OUTPUT_VOLUME_MAX) / OUTPUT_VOLUME_MAX;
	s_rightdb = (s_right * PMIC_OUTPUT_VOLUME_MAX) / OUTPUT_VOLUME_MAX;

	v_left = (left * (audio_mixer_control.vcodec_volume_out & 0x00ff)) / 100;
	v_right = (right * ((audio_mixer_control.vcodec_volume_out & 0xff00) >> 8)) / 100;
	v_leftdb = (v_left * PMIC_OUTPUT_VOLUME_MAX) / OUTPUT_VOLUME_MAX;
	v_rightdb = (v_right * PMIC_OUTPUT_VOLUME_MAX) / OUTPUT_VOLUME_MAX;
	
	/* Invoked by mixer */
	audio_mixer_control.master_volume_out = volume;
	if (is_hp < 0) {
		pmic_get_sensors(&s);
		if (s.sense_hsdets) {
			is_hp = 1;
		} else {
			is_hp = 0;
		}
	}
	if (is_hp) {
		v_rightdb = v_rightdb + 2;
		s_rightdb = s_rightdb + 2;
		if (v_rightdb > PMIC_OUTPUT_VOLUME_MAX) {
			v_rightdb = PMIC_OUTPUT_VOLUME_MAX;
		}
		if (s_rightdb > PMIC_OUTPUT_VOLUME_MAX) {
			s_rightdb = PMIC_OUTPUT_VOLUME_MAX;
		}
	}
	if ((right != 0) && ((audio_mixer_control.vcodec_volume_out & 0xff00) != 0)
	    && (audio_mixer_control.vcodec_unmute)) {
		pmic_audio_output_enable_mixer(audio_mixer_control.voice_codec_handle);
		pmic_audio_output_set_pgaGain(audio_mixer_control.
					      voice_codec_handle,
					      v_rightdb);
	} else {
		/* muting */
		pmic_audio_output_set_pgaGain(audio_mixer_control.voice_codec_handle, 0);
		pmic_audio_output_disable_mixer(audio_mixer_control.voice_codec_handle);
	}
	if ((right != 0) && ((audio_mixer_control.stdac_volume_out & 0xff00) != 0)
	    && (audio_mixer_control.stdac_unmute)) {
		pmic_audio_output_enable_mixer(audio_mixer_control.stdac_handle);
		pmic_audio_output_set_pgaGain(audio_mixer_control.
					      stdac_handle, s_rightdb);
	} else {
		/* muting */
		pmic_audio_output_set_pgaGain(audio_mixer_control.stdac_handle, 0);
		pmic_audio_output_disable_mixer(audio_mixer_control.stdac_handle);
	}

	return 0;
}

int set_mixer_output_volume(PMIC_AUDIO_HANDLE handle, int volume,
			    OUTPUT_DEVICES dev)
{
	int ret;

	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	ret = navu2_set_mixer_output_volume(handle, volume, dev, -1);
	up(&audio_mixer_control.sem);

	return ret;
}


static int navu2_set_mixer_stdac_output_volume(PMIC_AUDIO_HANDLE handle, int volume,
			    OUTPUT_DEVICES dev)
{
	audio_mixer_control.stdac_volume_out = volume;
	return navu2_set_mixer_output_volume(audio_mixer_control.stdac_handle, 
				      audio_mixer_control.master_volume_out, 0, -1);
}

int set_mixer_stdac_output_volume(PMIC_AUDIO_HANDLE handle, int volume,
			    OUTPUT_DEVICES dev)
{
	int ret;

	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	ret = navu2_set_mixer_stdac_output_volume(handle, volume, dev);
	up(&audio_mixer_control.sem);
	return ret;
}

static int navu2_set_mixer_vcodec_output_volume(PMIC_AUDIO_HANDLE handle, int volume,
			    OUTPUT_DEVICES dev)
{
	audio_mixer_control.vcodec_volume_out = volume;
	return navu2_set_mixer_output_volume(audio_mixer_control.stdac_handle, 
				      audio_mixer_control.master_volume_out, 0, -1);
}

int set_mixer_vcodec_output_volume(PMIC_AUDIO_HANDLE handle, int volume,
			    OUTPUT_DEVICES dev)
{
	int ret;

	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	ret = navu2_set_mixer_vcodec_output_volume(handle, volume, dev);
	up(&audio_mixer_control.sem);
	return ret;
}

int get_mixer_output_volume()
{
	int val;
	val = audio_mixer_control.master_volume_out;
	return val;
}

int get_mixer_stdac_output_volume()
{
	int val;
	val = audio_mixer_control.stdac_volume_out;
	return val;
}

int get_mixer_vcodec_output_volume()
{
	int val;
	val = audio_mixer_control.vcodec_volume_out;
	return val;
}

/*!
  * This function sets the PMIC output device's balance.
  *
  * @param       bal     Balance to be applied. This value can go
  *                      from 0 (Left atten) to 100 (Right atten)
  *                      50 is both equal
  */
int set_mixer_output_balance(int bal)
{
	int channel = 0;
	PMIC_AUDIO_OUTPUT_BALANCE_GAIN b_gain;
	PMIC_AUDIO_HANDLE handle;
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	// Convert ALSA value to PMIC value i.e. atten and channel value
	if (bal < 0)
		bal = 0;
	if (bal > 100)
		bal = 100;
	if (bal < 50) {
		channel = 1;
	} else {
		bal = 100 - bal;
		channel = 0;
	}

	b_gain = bal / 8;

	audio_mixer_control.mixer_balance = bal;
	if (audio_mixer_control.codec_playback_active) {
		handle = audio_mixer_control.voice_codec_handle;
		// Use codec's handle to set balance
	} else if (audio_mixer_control.stdac_playback_active) {
		handle = audio_mixer_control.stdac_handle;
		// Use STDac's handle to set balance
	} else {
		up(&audio_mixer_control.sem);
		return 0;
	}
	if (channel == 0)
		pmic_audio_output_set_balance(handle, BAL_GAIN_0DB, b_gain);
	else
		pmic_audio_output_set_balance(handle, b_gain, BAL_GAIN_0DB);
	up(&audio_mixer_control.sem);
	return 0;
}

int get_mixer_output_balance()
{
	int val;
	val = audio_mixer_control.mixer_balance;
	return val;
}

/*!
  * This function sets the PMIC output device's mono adder config.
  *
  * @param       mode    Mono adder mode to be set
  */
int set_mixer_output_mono_adder(PMIC_AUDIO_MONO_ADDER_MODE mode)
{
	PMIC_AUDIO_HANDLE handle;
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	audio_mixer_control.mixer_mono_adder = mode;
	if (audio_mixer_control.codec_playback_active) {
		handle = audio_mixer_control.voice_codec_handle;
		// Use codec's handle to set balance
		pmic_audio_output_enable_mono_adder(audio_mixer_control.
						    voice_codec_handle, mode);
	} else if (audio_mixer_control.stdac_playback_active) {
		handle = audio_mixer_control.stdac_handle;
		pmic_audio_output_enable_mono_adder(audio_mixer_control.
						    stdac_handle, mode);
		// Use STDac's handle to set balance
	}
	up(&audio_mixer_control.sem);
	return 0;
}

int get_mixer_output_mono_adder()
{
	int val;
	val = audio_mixer_control.mixer_mono_adder;
	return val;
}

/*!
  * This function sets the output device(s) in PMIC. It takes an
  * ALSA value and modifies registers using PMIC-specific values.
  *
  * @param       handle  handle to the device already opened
  * @param       src     Source connected to o/p device
  * @param       dev     Output device to be enabled
  * @param       enable  Enable or disable the device
  *
  */
static int navu2_set_mixer_output_device(PMIC_AUDIO_HANDLE handle, OUTPUT_SOURCE src,
			    OUTPUT_DEVICES dev, bool enable)
{
	PMIC_AUDIO_OUTPUT_PORT port;
	if (!((src == CODEC_DIR_OUT) || (src == MIXER_OUT))) {
		return -1;
	}
	if (handle != (PMIC_AUDIO_HANDLE) NULL) {
		/* Invoked by playback stream */
		if (audio_mixer_control.output_device & SOUND_MASK_PHONEOUT) {
			audio_mixer_control.output_active[OP_EARPIECE] = 1;
			pmic_audio_output_set_port(handle, MONO_SPEAKER);
		}
		if (audio_mixer_control.output_device & SOUND_MASK_SPEAKER) {
			audio_mixer_control.output_active[OP_HANDSFREE] = 1;
			pmic_audio_output_set_port(handle, MONO_LOUDSPEAKER);
		}
		if (audio_mixer_control.output_device & SOUND_MASK_VOLUME) {
			audio_mixer_control.output_active[OP_HEADSET] = 1;
			pmic_audio_output_set_port(handle,
						   STEREO_HEADSET_LEFT |
						   STEREO_HEADSET_RIGHT);
		}
		if (audio_mixer_control.output_device & SOUND_MASK_PCM) {
			audio_mixer_control.output_active[OP_LINEOUT] = 1;
			pmic_audio_output_set_port(handle,
						   STEREO_OUT_LEFT |
						   STEREO_OUT_RIGHT);
		}
	} else {
		switch (dev) {
		case OP_EARPIECE:
			if (enable) {
				audio_mixer_control.output_device |=
				    SOUND_MASK_PHONEOUT;
				audio_mixer_control.source_for_output[dev] =
				    src;
			} else {
				audio_mixer_control.output_device &=
				    ~SOUND_MASK_PHONEOUT;
			}
			port = MONO_SPEAKER;
			break;
		case OP_HANDSFREE:
			if (enable) {
				audio_mixer_control.output_device |=
				    SOUND_MASK_SPEAKER;
				audio_mixer_control.source_for_output[dev] =
				    src;
			} else {
				audio_mixer_control.output_device &=
				    ~SOUND_MASK_SPEAKER;
			}
			port = MONO_LOUDSPEAKER;
			break;
		case OP_HEADSET:
			if (enable) {
				audio_mixer_control.output_device |=
				    SOUND_MASK_VOLUME;
				audio_mixer_control.source_for_output[dev] =
				    src;
			} else {
				audio_mixer_control.output_device &=
				    ~SOUND_MASK_VOLUME;
			}
			port = STEREO_HEADSET_LEFT | STEREO_HEADSET_RIGHT;
			break;
		case OP_LINEOUT:
			if (enable) {
				audio_mixer_control.output_device |=
				    SOUND_MASK_PCM;
				audio_mixer_control.source_for_output[dev] =
				    src;
			} else {
				audio_mixer_control.output_device &=
				    ~SOUND_MASK_PCM;
			}
			port = STEREO_OUT_LEFT | STEREO_OUT_RIGHT;
			break;
		default:
			return -1;
			break;
		}
		/* Invoked by mixer .. little tricky to handle over here */
		if (audio_mixer_control.codec_playback_active) {
			if (enable) {
				audio_mixer_control.output_active[dev] = 1;
				pmic_audio_output_set_port(audio_mixer_control.
							   voice_codec_handle,
							   port);
			} else {
				audio_mixer_control.output_active[dev] = 0;
				pmic_audio_output_clear_port
				    (audio_mixer_control.voice_codec_handle,
				     port);
			}
		}
		if (audio_mixer_control.stdac_playback_active) {
			if (enable) {
				audio_mixer_control.output_active[dev] = 1;
				pmic_audio_output_set_port(audio_mixer_control.
							   stdac_handle, port);
			} else {
				audio_mixer_control.output_active[dev] = 0;
				pmic_audio_output_clear_port
				    (audio_mixer_control.stdac_handle, port);
			}
		}

	}
	return 0;
	// Set O/P device with handle and port

}

int set_mixer_output_device(PMIC_AUDIO_HANDLE handle, OUTPUT_SOURCE src,
			    OUTPUT_DEVICES dev, bool enable)
{
	int ret;

	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;
	ret = navu2_set_mixer_output_device(handle, src, dev, enable);
	up(&audio_mixer_control.sem);

	return ret;
}


int get_mixer_output_device()
{
	int val;
	val = audio_mixer_control.output_device;
	return val;
}

/*
 * DAC muting controls
 */
int set_mixer_stdac_unmute(PMIC_AUDIO_HANDLE handle, int unmute)
{
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;

	if (audio_mixer_control.stdac_unmute == unmute) {
		up(&audio_mixer_control.sem);
		return 0;
	}

	audio_mixer_control.stdac_unmute = unmute;
	navu2_set_output_setting();

	up(&audio_mixer_control.sem);
	return 0;
}
int get_mixer_stdac_unmute()
{
	return audio_mixer_control.stdac_unmute;
}
int set_mixer_vcodec_unmute(PMIC_AUDIO_HANDLE handle, int unmute)
{
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;

	if (audio_mixer_control.vcodec_unmute == unmute) {
		up(&audio_mixer_control.sem);
		return 0;
	}

	audio_mixer_control.vcodec_unmute = unmute;
	navu2_set_output_setting();

	up(&audio_mixer_control.sem);
	return 0;
}
int get_mixer_vcodec_unmute()
{
	return audio_mixer_control.vcodec_unmute;
}

/*
 * DAC force ON
 */
int set_mixer_stdac_forceon(PMIC_AUDIO_HANDLE handle, int forceon)
{
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;

	if (audio_mixer_control.stdac_forceon == forceon) {
		up(&audio_mixer_control.sem);
		return 0;
	}

	audio_mixer_control.stdac_forceon = forceon;
	if (forceon) {
		navu2_set_mixer_stdac_output_volume(handle,
					      audio_mixer_control.stdac_volume_out,
					      OP_NODEV);
	} else {
		if (!audio_mixer_control.stdac_active) {
			pmic_audio_output_set_pgaGain(audio_mixer_control.stdac_handle, 0);
		}
	}
	navu2_set_output_setting();
	up(&audio_mixer_control.sem);

	return 0;
}
int get_mixer_stdac_forceon()
{
	return audio_mixer_control.stdac_forceon;
}
int set_mixer_vcodec_forceon(PMIC_AUDIO_HANDLE handle, int forceon)
{
	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;

	if (audio_mixer_control.vcodec_forceon == forceon) {
		up(&audio_mixer_control.sem);
		return 0;
	}

	audio_mixer_control.vcodec_forceon = forceon;
	if (forceon) {
		navu2_set_mixer_vcodec_output_volume(handle,
					       audio_mixer_control.vcodec_volume_out,
					       OP_NODEV);
	} else {
		if (!audio_mixer_control.vcodec_active) {
			pmic_audio_output_set_pgaGain(audio_mixer_control.voice_codec_handle, 0);
		}
	}
	navu2_set_output_setting();
	up(&audio_mixer_control.sem);

	return 0;
}
int get_mixer_vcodec_forceon()
{
	return audio_mixer_control.vcodec_forceon;
}


/*!
  * These are the callback functions for mixer controls
  *
  */
/* Output device control*/
static int pmic_mixer_output_info(struct snd_kcontrol *kcontrol,
				  struct snd_ctl_elem_info *uinfo)
{
	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 15;
	uinfo->value.integer.step = 1;
	return 0;
}
static int pmic_mixer_output_put(struct snd_kcontrol *kcontrol,
				 struct snd_ctl_elem_value *uvalue)
{
	int dev, i;
	dev = uvalue->value.integer.value[0];
	for (i = OP_EARPIECE; i < OP_MAXDEV; i++) {
		if (dev & (1 << i)) {
			set_mixer_output_device(NULL, MIXER_OUT, i, 1);
		} else {
			set_mixer_output_device(NULL, MIXER_OUT, i, 0);
		}
	}
	return 0;
}
static int pmic_mixer_output_get(struct snd_kcontrol *kcontrol,
				 struct snd_ctl_elem_value *uvalue)
{
	int val, ret = 0, i = 0;
	for (i = OP_EARPIECE; i < OP_MAXDEV; i++) {
		val = get_mixer_output_device();
		if (val & SOUND_MASK_PHONEOUT)
			ret = ret | 1;
		if (val & SOUND_MASK_SPEAKER)
			ret = ret | 2;
		if (val & SOUND_MASK_VOLUME)
			ret = ret | 4;
		if (val & SOUND_MASK_PCM)
			ret = ret | 8;
		uvalue->value.integer.value[0] = ret;
	}
	return 0;

}

/* Input gain control*/
static int pmic_cap_volume_info(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 100;
	uinfo->value.integer.step = 1;
	return 0;
}
static int pmic_cap_volume_get(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_value *uvalue)
{
	int val;
	val = get_mixer_input_gain();
	val = val & 0xFF;
	uvalue->value.integer.value[0] = val;
	return 0;
}

static int pmic_cap_volume_put(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_value *uvalue)
{

	int vol;
	vol = uvalue->value.integer.value[0];
	vol = vol | (vol << 8);
	set_mixer_input_gain(NULL, vol);
	return 0;
}

/* Mono adder control*/
static int pmic_pb_monoconfig_info(struct snd_kcontrol *kcontrol,
				   struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 3;
	uinfo->value.integer.step = 1;
	return 0;
}
static int pmic_pb_monoconfig_put(struct snd_kcontrol *kcontrol,
				  struct snd_ctl_elem_value *uvalue)
{
	int mono;
	mono = uvalue->value.integer.value[0];
	set_mixer_output_mono_adder(mono);
	return 0;
}
static int pmic_pb_monoconfig_get(struct snd_kcontrol *kcontrol,
				  struct snd_ctl_elem_value *uvalue)
{
	uvalue->value.integer.value[0] = get_mixer_output_mono_adder();
	return 0;
}

/*!
  * These are the ALSA control structures with init values
  *
  */

/* Input device control*/
static int pmic_cap_input_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 7;
	uinfo->value.integer.step = 1;
	return 0;
}
static int pmic_cap_input_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int dev, i;
	dev = uvalue->value.integer.value[0];
	for (i = IP_HANDSET; i < IP_MAXDEV; i++) {
		if (dev & (1 << i)) {
			set_mixer_input_device(NULL, i, 1);
		} else {
			set_mixer_input_device(NULL, i, 0);
		}
	}
	return 0;
}
static int pmic_cap_input_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int val, ret = 0, i = 0;
	for (i = IP_HANDSET; i < IP_MAXDEV; i++) {
		val = get_mixer_input_device();
		if (val & SOUND_MASK_PHONEIN)
			ret = ret | 1;
		if (val & SOUND_MASK_MIC)
			ret = ret | 2;
		if (val & SOUND_MASK_LINE)
			ret = ret | 4;
		uvalue->value.integer.value[0] = ret;
	}
	return 0;
}

/* Volume control*/
static int pmic_pb_volume_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int volume;
	volume = uvalue->value.integer.value[0];
	volume = volume | (volume << 8);
	set_mixer_output_volume(NULL, volume, OP_NODEV);
	return 0;
}
static int pmic_pb_volume_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 100;
	uinfo->value.integer.step = 1;
	return 0;
}

static int pmic_pb_volume_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int val;
	val = get_mixer_output_volume();
	val = val & 0xFF;
	uvalue->value.integer.value[0] = val;
	return 0;
}

/* Balance control start */
static int pmic_pb_balance_info(struct snd_kcontrol *kcontrol,
				struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 100;
	uinfo->value.integer.step = 1;
	return 0;
}

static int pmic_pb_balance_get(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_value *uvalue)
{
	uvalue->value.integer.value[0] = get_mixer_output_balance();
	return 0;

}
static int pmic_pb_balance_put(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_value *uvalue)
{
	int bal;
	bal = uvalue->value.integer.value[0];
	set_mixer_output_balance(bal);
	return 0;
}

/* Balance control end */

/* Stere DAC Volume control*/
static int pmic_stdac_volume_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int volume;
	volume = uvalue->value.integer.value[0];
	volume = volume | (volume << 8);
	set_mixer_stdac_output_volume(NULL, volume, OP_NODEV);
	return 0;
}
static int pmic_stdac_volume_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 100;
	uinfo->value.integer.step = 1;
	return 0;
}
static int pmic_stdac_volume_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int val;
	val = get_mixer_stdac_output_volume();
	val = val & 0xFF;
	uvalue->value.integer.value[0] = val;
	return 0;
}

/* Voice Codec Volume control*/
static int pmic_vcodec_volume_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int volume;
	volume = uvalue->value.integer.value[0];
	volume = volume | (volume << 8);
	set_mixer_vcodec_output_volume(NULL, volume, OP_NODEV);
	return 0;
}
static int pmic_vcodec_volume_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 100;
	uinfo->value.integer.step = 1;
	return 0;
}
static int pmic_vcodec_volume_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	int val;
	val = get_mixer_vcodec_output_volume();
	val = val & 0xFF;
	uvalue->value.integer.value[0] = val;
	return 0;
}

/*
 * DAC muting controls
 */
static int pmic_stdac_unmute_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_BOOLEAN;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 1;
	return 0;
}
static int pmic_stdac_unmute_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	uvalue->value.integer.value[0] = get_mixer_stdac_unmute();
	return 0;
}
static int pmic_stdac_unmute_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	set_mixer_stdac_unmute(NULL, uvalue->value.integer.value[0] & 1);
	return 0;
}
static int pmic_vcodec_unmute_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_BOOLEAN;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 1;
	return 0;
}
static int pmic_vcodec_unmute_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	uvalue->value.integer.value[0] = get_mixer_vcodec_unmute();
	return 0;
}
static int pmic_vcodec_unmute_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	set_mixer_vcodec_unmute(NULL, uvalue->value.integer.value[0] & 1);
	return 0;
}

/*
 * DAC force ON controls
 */
static int pmic_stdac_forceon_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_BOOLEAN;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 1;
	return 0;
}
static int pmic_stdac_forceon_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	uvalue->value.integer.value[0] = get_mixer_stdac_forceon();
	return 0;
}
static int pmic_stdac_forceon_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	set_mixer_stdac_forceon(NULL, uvalue->value.integer.value[0] & 1);
	return 0;
}
static int pmic_vcodec_forceon_info(struct snd_kcontrol *kcontrol,
			       struct snd_ctl_elem_info *uinfo)
{

	uinfo->type = SNDRV_CTL_ELEM_TYPE_BOOLEAN;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 1;
	return 0;
}
static int pmic_vcodec_forceon_get(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	uvalue->value.integer.value[0] = get_mixer_vcodec_forceon();
	return 0;
}
static int pmic_vcodec_forceon_put(struct snd_kcontrol *kcontrol,
			      struct snd_ctl_elem_value *uvalue)
{
	set_mixer_vcodec_forceon(NULL, uvalue->value.integer.value[0] & 1);
	return 0;
}


/* Kcontrol structure definitions */
struct snd_kcontrol_new pmic_control_pb_vol __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "Master Playback Volume",
	.index = 0x00,
	.info = pmic_pb_volume_info,
	.get = pmic_pb_volume_get,
	.put = pmic_pb_volume_put,
	.private_value = 0xffab1,
};

struct snd_kcontrol_new pmic_control_pb_bal __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "Master Balance Playback Volume",
	.index = 0x00,
	.info = pmic_pb_balance_info,
	.get = pmic_pb_balance_get,
	.put = pmic_pb_balance_put,
	.private_value = 0xffab2,
};
struct snd_kcontrol_new pmic_control_pb_monoconfig __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "Master Monoconfig Playback Volume",
	.index = 0x00,
	.info = pmic_pb_monoconfig_info,
	.get = pmic_pb_monoconfig_get,
	.put = pmic_pb_monoconfig_put,
	.private_value = 0xffab2,
};
struct snd_kcontrol_new pmic_control_op_sw __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "Master Output Playback Volume",
	.index = 0x00,
	.info = pmic_mixer_output_info,
	.get = pmic_mixer_output_get,
	.put = pmic_mixer_output_put,
	.private_value = 0xffab4,
};

struct snd_kcontrol_new pmic_control_cap_vol __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "Mic Capture Volume",
	.index = 0x00,
	.info = pmic_cap_volume_info,
	.get = pmic_cap_volume_get,
	.put = pmic_cap_volume_put,
	.private_value = 0xffab5,
};
struct snd_kcontrol_new pmic_control_ip_sw __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "Master Input Capture Volume",
	.index = 0x00,
	.info = pmic_cap_input_info,
	.get = pmic_cap_input_get,
	.put = pmic_cap_input_put,
	.private_value = 0xffab5,
};

struct snd_kcontrol_new pmic_control_stdac_pb_vol __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "PCM Playback Volume",
	.index = 0x00,
	.info = pmic_stdac_volume_info,
	.get = pmic_stdac_volume_get,
	.put = pmic_stdac_volume_put,
	.private_value = 0xffab6,
};
struct snd_kcontrol_new pmic_control_vcodec_pb_vol __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "PCM Playback Volume",
	.index = 0x01,
	.info = pmic_vcodec_volume_info,
	.get = pmic_vcodec_volume_get,
	.put = pmic_vcodec_volume_put,
	.private_value = 0xffab7,
};

struct snd_kcontrol_new pmic_control_stdac_unmute_sw __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "PCM Playback Switch",
	.index = 0x00,
	.info = pmic_stdac_unmute_info,
	.get = pmic_stdac_unmute_get,
	.put = pmic_stdac_unmute_put,
	.private_value = 0xffab8,
};
struct snd_kcontrol_new pmic_control_vcodec_unmute_sw __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "PCM Playback Switch",
	.index = 0x01,
	.info = pmic_vcodec_unmute_info,
	.get = pmic_vcodec_unmute_get,
	.put = pmic_vcodec_unmute_put,
	.private_value = 0xffab8,
};

struct snd_kcontrol_new pmic_control_stdac_forceon_sw __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "PCM Force ON Switch",
	.index = 0x00,
	.info = pmic_stdac_forceon_info,
	.get = pmic_stdac_forceon_get,
	.put = pmic_stdac_forceon_put,
	.private_value = 0xffab9,
};
struct snd_kcontrol_new pmic_control_vcodec_forceon_sw __devinitdata = {
	.iface = SNDRV_CTL_ELEM_IFACE_MIXER,
	.name = "PCM Force ON Switch",
	.index = 0x01,
	.info = pmic_vcodec_forceon_info,
	.get = pmic_vcodec_forceon_get,
	.put = pmic_vcodec_forceon_put,
	.private_value = 0xffaba,
};

/*!
  * This function registers the control components of ALSA Mixer
  * It is called by ALSA PCM init.
  *
  * @param	card pointer to the ALSA sound card structure.
  *
  * @return              0 on success, -ve otherwise.
  */
int mxc_alsa_create_ctl(struct snd_card *card, void *p_value)
{
	int err = 0;

	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_op_sw, p_value))) < 0)
		return err;

	if ((err =
	     snd_ctl_add(card,
			 snd_ctl_new1(&pmic_control_pb_vol, p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card,
			 snd_ctl_new1(&pmic_control_pb_monoconfig,
				      p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card,
			 snd_ctl_new1(&pmic_control_pb_bal, p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card,
			 snd_ctl_new1(&pmic_control_cap_vol, p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_ip_sw, p_value))) < 0)
		return err;

	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_stdac_pb_vol, p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_vcodec_pb_vol, p_value))) < 0)
		return err;

	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_stdac_unmute_sw, p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_vcodec_unmute_sw, p_value))) < 0)
		return err;

	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_stdac_forceon_sw, p_value))) < 0)
		return err;
	if ((err =
	     snd_ctl_add(card, snd_ctl_new1(&pmic_control_vcodec_forceon_sw, p_value))) < 0)
		return err;

	return 0;
}

/*!
  * This function normalizes speed given by the user
  * if speed is not supported, the function will
  * calculate the nearest one.
  *
  * @param       speed   speed requested by the user.
  *
  * @return      The normalized speed.
  */
int adapt_speed(int speed)
{

	/* speeds from 8k to 96k */
	if (speed >= (64000 + 96000) / 2) {
		speed = 96000;
	} else if (speed >= (48000 + 64000) / 2) {
		speed = 64000;
	} else if (speed >= (44100 + 48000) / 2) {
		speed = 48000;
	} else if (speed >= (32000 + 44100) / 2) {
		speed = 44100;
	} else if (speed >= (24000 + 32000) / 2) {
		speed = 32000;
	} else if (speed >= (22050 + 24000) / 2) {
		speed = 24000;
	} else if (speed >= (16000 + 22050) / 2) {
		speed = 22050;
	} else if (speed >= (12000 + 16000) / 2) {
		speed = 16000;
	} else if (speed >= (11025 + 12000) / 2) {
		speed = 12000;
	} else if (speed >= (8000 + 11025) / 2) {
		speed = 11025;
	} else {
		speed = 8000;
	}
	return speed;
}

static int navu2_init_stdac(void)
{
	PMIC_AUDIO_HANDLE handle;

	pmic_audio_open(&handle, STEREO_DAC);
	audio_mixer_control.stdac_handle = handle;

	audio_mixer_control.mixer_mono_adder = MONO_ADDER_OFF;

	pmic_audio_stdac_set_rxtx_timeslot(handle, USE_TS0_TS1);
	pmic_audio_stdac_enable_mixer(handle, USE_TS2_TS3, STDAC_NO_MIX,
				      STDAC_MIX_OUT_0DB);
	pmic_audio_set_protocol(handle, AUDIO_DATA_BUS_2, NORMAL_MSB_JUSTIFIED_MODE,
				BUS_MASTER_MODE, USE_2_TIMESLOTS);
	pmic_audio_stdac_set_clock(handle, CLOCK_IN_CLIA, STDAC_CLI_FREQ,
				   STDAC_RATE_44_1_KHZ, INVERT_FRAMESYNC|INVERT_BITCLOCK);

	pmic_audio_stdac_set_config(handle, STDAC_MASTER_CLOCK_OUTPUTS);
	audio_mixer_control.stdac_out_to_mixer = 1;
	pmic_audio_digital_filter_reset(handle);
	msleep(10);
	pmic_audio_output_set_pgaGain(handle, 0);
	pmic_audio_output_enable_mono_adder(handle,
					    audio_mixer_control.
					    mixer_mono_adder);
	set_mixer_output_device(handle, MIXER_OUT, OP_NODEV, 1);
	pmic_audio_enable(audio_mixer_control.stdac_handle);
	pmic_audio_output_enable_mixer(audio_mixer_control.stdac_handle);

	return 0;
}

static int navu2_init_vcodec(void)
{
	PMIC_AUDIO_HANDLE handle;

	pmic_audio_open(&handle, VOICE_CODEC);
	audio_mixer_control.voice_codec_handle = handle;

	pmic_audio_vcodec_set_rxtx_timeslot(handle, USE_TS0);
	pmic_audio_vcodec_enable_mixer(handle, USE_TS1, VCODEC_MIX_IN_0DB,
				       VCODEC_MIX_OUT_0DB);
	pmic_audio_set_protocol(handle, AUDIO_DATA_BUS_1, NETWORK_MODE, BUS_MASTER_MODE,
				USE_4_TIMESLOTS);

	msleep(20);
	pmic_audio_vcodec_set_clock(handle, CLOCK_IN_CLIA, VCODEC_CLI_FREQ,
				    VCODEC_RATE_8_KHZ, NO_INVERT);
	msleep(20);
	pmic_audio_vcodec_set_config(handle, VCODEC_MASTER_CLOCK_OUTPUTS);
	pmic_audio_digital_filter_reset(handle);
	msleep(15);
	set_mixer_output_device(handle, MIXER_OUT, OP_NODEV, 1);
	pmic_audio_output_set_pgaGain(handle, 0);
	set_mixer_input_device(handle, IP_NODEV, 1);
	set_mixer_input_gain(handle, audio_mixer_control.input_volume);
	pmic_audio_enable(audio_mixer_control.voice_codec_handle);
	pmic_audio_output_enable_mixer(audio_mixer_control.voice_codec_handle);

	return 0;
}

/* set speaker output */
static int navu2_set_speaker_output(void)
{
	u32 reg = 0;

	pmic_power_regulator_off(REGU_GPO1);
	if ((audio_mixer_control.stdac_active && audio_mixer_control.stdac_unmute) ||
	    (audio_mixer_control.vcodec_active && audio_mixer_control.vcodec_unmute) ||
	     audio_mixer_control.stdac_forceon || audio_mixer_control.vcodec_forceon) {
		pmic_power_regulator_on(REGU_GPO2);
	} else {
		pmic_read_reg(REG_POWER_MISCELLANEOUS, &reg, PMIC_ALL_BITS);
		pmic_power_regulator_off(REGU_GPO2);
		if (reg & (1<<8)) { /* on->off ? */
			msleep(100);
		}
	}
	navu2_set_mixer_output_device(NULL, MIXER_OUT, OP_HANDSFREE, 1);
	navu2_set_mixer_output_device(NULL, MIXER_OUT, OP_HEADSET, 0);
	pmic_audio_output_enable_mono_adder(audio_mixer_control.stdac_handle,
					    MONO_ADD_LEFT_RIGHT);

	return 0;
}

/* set headphone output */
static int navu2_set_headphone_output(void)
{
	if ((audio_mixer_control.stdac_active && audio_mixer_control.stdac_unmute) ||
	    (audio_mixer_control.vcodec_active && audio_mixer_control.vcodec_unmute) ||
	     audio_mixer_control.stdac_forceon || audio_mixer_control.vcodec_forceon) {
		pmic_power_regulator_on(REGU_GPO1);
	} else {
		pmic_power_regulator_off(REGU_GPO1);
	}
	pmic_power_regulator_off(REGU_GPO2);
	navu2_set_mixer_output_device(NULL, MIXER_OUT, OP_HANDSFREE, 0);
	navu2_set_mixer_output_device(NULL, MIXER_OUT, OP_HEADSET, 1);
	pmic_audio_output_enable_mono_adder(audio_mixer_control.stdac_handle,
					    MONO_ADDER_OFF);

	return 0;
}

static void navu2_set_output_setting(void)
{
	t_sensor_bits s;

	pmic_get_sensors(&s);
	if (s.sense_hsdets) {
		navu2_set_headphone_output();
		navu2_set_mixer_output_volume(0, audio_mixer_control.master_volume_out, 0, 1);
	} else {
		navu2_set_speaker_output();
		navu2_set_mixer_output_volume(0, audio_mixer_control.master_volume_out, 0, 0);
	}
}

/* headphone detect callback work queue tqask */
static void navu2_hsdet_callback_work(struct work_struct *work)
{
	if (down_interruptible(&audio_mixer_control.sem))
		return;
	navu2_set_output_setting();
	up(&audio_mixer_control.sem);
}

/* headphone detect handler */
static void navu2_hsdet_callback(void *arg)
{
	queue_work(audio_mixer_control.callback_wq, &audio_mixer_control.callback_work);
}


static pmic_event_callback_t navu2_hsdet_callback_param = {
	.func = navu2_hsdet_callback,
	.param = 0,
};

/* delayed initializer */
static void navu2_mixer_late_init(struct work_struct *work)
{
	pmic_audio_digital_filter_reset(audio_mixer_control.stdac_handle);
	pmic_audio_digital_filter_reset(audio_mixer_control.voice_codec_handle);
	msleep(10);
	pmic_event_subscribe(EVENT_HSDETI, navu2_hsdet_callback_param);
	navu2_hsdet_callback(0);
}


static pcm_ctrl_t pcm_sett[] = {
	[0] = {
		.func = {
			.mode_sel	= PCM_MODE_3,
			.m_s		= PCM_SLAVE_MODE,
			.tx_tim		= PCM_TX_04_WORD,
		},
		.cyc = {
			.cyc_val	= 15,
			.sib		= 15,
			.rx_pd		= PCM_PADDING_ON,
			.sob		= 15,
			.tx_pd		= PCM_PADDING_ON,
		},
		.cyc2 = {
			.cyc_val2	= 0,
			.sib2		= 0,
			.sob2		= 0,
		},
	},
	[1] = {
		.func = {
			.mode_sel	= PCM_MODE_0,
			.m_s		= PCM_SLAVE_MODE,
			.tx_tim		= PCM_TX_04_WORD,
		},
		.cyc = {
			.cyc_val	= 63,
			.sib		= 15,
			.rx_pd		= PCM_PADDING_ON,
			.sob		= 15,
			.tx_pd		= PCM_PADDING_ON,
		},
		.cyc2 = {
			.cyc_val2	= 0,
			.sib2		= 0,
			.sob2		= 0,
		},
	},
};


static int __init navu2_codec_mixer_new(struct snd_card *card)
{
	audio_mixer_control_t *audio_control;
	int i = 0;

	pmic_audio_antipop_enable(ANTI_POP_RAMP_SLOW);

	audio_control = &audio_mixer_control;
	memset(audio_control, 0, sizeof(audio_mixer_control_t));
	sema_init(&audio_control->sem, 1);

	audio_control->input_device = SOUND_MASK_MIC;
	audio_control->output_device = SOUND_MASK_VOLUME | SOUND_MASK_SPEAKER;

	/* PMIC has to internal sources that can be routed to output
	   One is codec direct out and the other is mixer out
	   Initially we configure all outputs to have no source and
	   will be later configured either by PCM stream handler or mixer */
	for (i = 0; i < OP_MAXDEV && i != OP_HEADSET; i++) {
		audio_control->source_for_output[i] = MIXER_OUT;
	}

	/* These bits are initially reset and set when playback begins */
	audio_control->codec_out_to_mixer = 1;
	audio_control->stdac_out_to_mixer = 1;

	audio_control->mixer_balance = 50;
	audio_control->mixer_mono_adder = MONO_ADDER_OFF;

	/* Default values for input and output */
	audio_control->input_volume = ((40 << 8) & 0xff00) | (40 & 0x00ff);
	audio_control->master_volume_out = ((50 << 8) & 0xff00) | (50 & 0x00ff);
	audio_control->stdac_volume_out = ((100 << 8) & 0xff00) | (100 & 0x00ff);
	audio_control->vcodec_volume_out = ((100 << 8) & 0xff00) | (100 & 0x00ff);

	audio_control->codec_playback_active = 1;
	audio_control->codec_capture_active = 1;
	audio_control->stdac_playback_active = 1;

	audio_control->stdac_active = 0;
	audio_control->vcodec_active = 0;

	audio_control->stdac_unmute = 1;
	audio_control->vcodec_unmute = 1;
	audio_control->stdac_forceon = 0;
	audio_control->vcodec_forceon = 0;

	navu2_init_stdac();
	navu2_init_vcodec();

	pmic_audio_output_set_config(audio_mixer_control.stdac_handle,
				     HEADSET_DETECT_ENABLE);
	pmic_audio_output_set_config(audio_mixer_control.stdac_handle,
				     STEREO_HEADSET_AMP_AUTO_DISABLE);

	audio_control->callback_wq = create_singlethread_workqueue("navu2-mixer");
	INIT_WORK(&audio_control->callback_work, navu2_hsdet_callback_work);

	/* set headphone/speaker muting */
	INIT_DELAYED_WORK(&audio_control->late_init_wq, navu2_mixer_late_init);
	schedule_delayed_work(&audio_control->late_init_wq, msecs_to_jiffies(250));

	snprintf(card->mixername, sizeof(card->mixername), "atlas mixer");
	return mxc_alsa_create_ctl(card, &audio_control);
}


static struct snd_pcm *navu2_codec[2];

static unsigned int rates0[] = {
	7350, 8000, 11025, 12000, 14700, 16000,
	22050, 24000, 29400, 32000, 44100, 48000,
};
static unsigned int rates1[] = {
	8000, 16000,
};

static struct snd_pcm_hw_constraint_list hw_constraints_rates[] = {
	[0] = {
		.count	= ARRAY_SIZE(rates0),
		.list	= rates0,
		.mask	= 0,
	},
	[1] = {
		.count	= ARRAY_SIZE(rates1),
		.list	= rates1,
		.mask	= 0,
	},
};

static struct audio_stream navu2_codec_out[] = {
	[0] = {
		.id			= "voice out dma m2p lch9",
		.dma_ch			= MP200_DMAC_M2P_LCH9,
	},
	[1] = {
		.id			= "voice out dma m2p lch10",
		.dma_ch			= MP200_DMAC_M2P_LCH10,
	},
};

static struct audio_stream navu2_codec_in[] = {
	[0] = {
	},
	[1] = {
		.id			= "voice in dma m2p lch10",
		.dma_ch			= MP200_DMAC_P2M_LCH10,
	}
};

static struct mp200_pcm_client navu2_codec_client[] = {
	[0] = {
		.pcm_ch			= MP200_PCM_CH0,
		.startup		= navu2_codec_startup,
		.shutdown		= navu2_codec_shutdown,
		.prepare		= navu2_codec_prepare,
	},
	[1] = {
		.pcm_ch			= MP200_PCM_CH1,
		.startup		= navu2_codec_startup,
		.shutdown		= navu2_codec_shutdown,
		.prepare		= navu2_codec_prepare,
	},
};

static int navu2_codec_startup(struct snd_pcm_substream *substream)
{
	struct snd_pcm_runtime *runtime = substream->runtime;
	int channel = substream->pcm->device;
	int err;
	FNC_ENTRY

	runtime->hw.formats = SNDRV_PCM_FMTBIT_S16_LE;
	if (channel == MP200_PCM_CH0) {
		runtime->hw.rates = (SNDRV_PCM_RATE_8000 |
				     SNDRV_PCM_RATE_11025 |
				     SNDRV_PCM_RATE_16000 |
				     SNDRV_PCM_RATE_22050 |
				     SNDRV_PCM_RATE_32000 |
				     SNDRV_PCM_RATE_44100 |
				     SNDRV_PCM_RATE_48000 |
				     SNDRV_PCM_RATE_KNOT);
		runtime->hw.rate_min = 8000;
		runtime->hw.rate_max = 48000;
		runtime->hw.channels_min = 2;
		runtime->hw.channels_max = 2;
	} else {
		runtime->hw.rates = (SNDRV_PCM_RATE_8000 |
				     SNDRV_PCM_RATE_16000);
		runtime->hw.rate_min = 8000;
		runtime->hw.rate_max = 16000;
		runtime->hw.channels_min = 1;
		runtime->hw.channels_max = 1;
	}

	if ((err = snd_pcm_hw_constraint_integer(
			runtime, SNDRV_PCM_HW_PARAM_PERIODS)) < 0) {
		FNC_EXIT return err;
	}
	if ((err = snd_pcm_hw_constraint_list(
			runtime, 0, SNDRV_PCM_HW_PARAM_RATE,
			&hw_constraints_rates[channel])) < 0) {
		FNC_EXIT return err;
	}

	FNC_EXIT return 0;
}

static void navu2_codec_shutdown(struct snd_pcm_substream *substream)
{
	int channel = substream->pcm->device;
	PMIC_AUDIO_HANDLE handle;
	int forceon;
	FNC_ENTRY

	if (down_interruptible(&audio_mixer_control.sem))
		return;

	if (channel == MP200_PCM_CH0) {
		audio_mixer_control.stdac_active = 0;
	} else {
		audio_mixer_control.vcodec_active = 0;
	}

	navu2_set_output_setting();

	handle = (channel == MP200_PCM_CH0) ?
		audio_mixer_control.stdac_handle : audio_mixer_control.voice_codec_handle;
	forceon = (channel == MP200_PCM_CH0) ?
		audio_mixer_control.stdac_forceon : audio_mixer_control.vcodec_forceon;

	if (!forceon) {
		pmic_audio_output_set_pgaGain(handle, 0);
		pmic_audio_output_disable_mixer(handle);
	}

	if (channel == MP200_PCM_CH1) {
		pmic_audio_vcodec_set_clock(audio_mixer_control.voice_codec_handle, 
					   CLOCK_IN_CLIA, VCODEC_CLI_FREQ,
					   VCODEC_RATE_8_KHZ, NO_INVERT);
		pmic_audio_digital_filter_reset(handle);
	}

	up(&audio_mixer_control.sem);
	FNC_EXIT return;
}

static int navu2_codec_prepare(struct snd_pcm_substream *substream)
{
	int ret = 0;
	struct snd_pcm_runtime *runtime = substream->runtime;
	int channel = substream->pcm->device;
	int sample_rate;
	int volume, mvolume, db, unmute;
	PMIC_AUDIO_HANDLE handle;
	FNC_ENTRY

	if (down_interruptible(&audio_mixer_control.sem))
		return -EINTR;

	if (channel == MP200_PCM_CH0) {
		audio_mixer_control.stdac_active = 1;
	} else {
		audio_mixer_control.vcodec_active = 1;
	}
	runtime->rate = adapt_speed(runtime->rate);
	handle = (channel == MP200_PCM_CH0) ?
		audio_mixer_control.stdac_handle : audio_mixer_control.voice_codec_handle;
	mvolume = (audio_mixer_control.master_volume_out & 0xff00) >> 8;
	volume = (channel == MP200_PCM_CH0) ?
		(mvolume * ((audio_mixer_control.stdac_volume_out & 0xff00) >> 8)) / 100 :
		(mvolume * ((audio_mixer_control.vcodec_volume_out & 0xff00) >> 8)) / 100;
	db = (volume * PMIC_OUTPUT_VOLUME_MAX) / OUTPUT_VOLUME_MAX;
	unmute = (channel == MP200_PCM_CH0) ?
		audio_mixer_control.stdac_unmute : audio_mixer_control.vcodec_unmute;

	if (channel == MP200_PCM_CH0) {
		switch (runtime->rate) {
		case 8000:
			sample_rate = STDAC_RATE_8_KHZ;
			break;

		case 11025:
			sample_rate = STDAC_RATE_11_025_KHZ;
			break;

		case 12000:
			sample_rate = STDAC_RATE_12_KHZ;
			break;

		case 16000:
			sample_rate = STDAC_RATE_16_KHZ;
			break;

		case 22050:
			sample_rate = STDAC_RATE_22_050_KHZ;
			break;

		case 24000:
			sample_rate = STDAC_RATE_24_KHZ;
			break;

		case 32000:
			sample_rate = STDAC_RATE_32_KHZ;
			break;

		case 44100:
			sample_rate = STDAC_RATE_44_1_KHZ;
			break;

		case 48000:
			sample_rate = STDAC_RATE_48_KHZ;
			break;

		case 64000:
			sample_rate = STDAC_RATE_64_KHZ;
			break;

		case 96000:
			sample_rate = STDAC_RATE_96_KHZ;
			break;

		default:
			sample_rate = STDAC_RATE_8_KHZ;
		}
		ret = pmic_audio_stdac_set_clock(audio_mixer_control.stdac_handle, 
					   CLOCK_IN_CLIA, STDAC_CLI_FREQ,
					   sample_rate, INVERT_FRAMESYNC|INVERT_BITCLOCK);
		if (ret != PMIC_SUCCESS)
			goto out;
	} else {
		switch (runtime->rate) {
		case 8000:
			sample_rate = VCODEC_RATE_8_KHZ;
			break;
		case 16000:
			sample_rate = VCODEC_RATE_16_KHZ;
			break;
		default:
			sample_rate = VCODEC_RATE_8_KHZ;
			break;
		}
		ret = pmic_audio_vcodec_set_clock(audio_mixer_control.voice_codec_handle, 
					   CLOCK_IN_CLIA, VCODEC_CLI_FREQ,
					   sample_rate, NO_INVERT);
		if (ret != PMIC_SUCCESS)
			goto out;
	}

	pmic_audio_digital_filter_reset(handle);
	msleep(1);
	navu2_set_output_setting();

 out:
	up(&audio_mixer_control.sem);
	FNC_EXIT return ret;
}


/* DAC hang up callbackk */
int mp200_codec_reset(void)
{
	printk(KERN_WARNING "DAC hangup detected.\n");
	return 0;
}


/* codec sound card */

static char *id = NULL;	/* ID for this card */
module_param(id, charp, 0444);

#ifdef CONFIG_PM
static int navu2_codec_suspend(struct platform_device *dev, pm_message_t state)
{
	struct snd_card *card = platform_get_drvdata(dev);
	int i;

	pmic_event_unsubscribe(EVENT_HSDETI, navu2_hsdet_callback_param);
	pmic_power_regulator_off(REGU_GPO1);
	pmic_power_regulator_off(REGU_GPO2);

	pmic_audio_antipop_disable();
	pmic_power_regulator_off(REGU_VAUDIO);
	msleep(1200);

	snd_power_change_state(card, SNDRV_CTL_POWER_D3hot);
	for (i = 0; i < 2; i++) {
		snd_pcm_suspend_all(navu2_codec[i]);
	}

	return 0;
}

static int navu2_codec_resume(struct platform_device *dev)
{
	struct snd_card *card = platform_get_drvdata(dev);
	int i;

	pmic_power_regulator_on(REGU_VAUDIO);
	for (i = 0; i < 2; i++) {
		mp200_pcm_set_ctrl(navu2_codec_client[i].pcm_regs, &pcm_sett[i]);
	}
	snd_power_change_state(card, SNDRV_CTL_POWER_D0);
	pmic_audio_antipop_enable(ANTI_POP_RAMP_SLOW);
	schedule_delayed_work(&audio_mixer_control.late_init_wq, msecs_to_jiffies(250));

	return 0;
}
#else
#define navu2_codec_suspend	NULL
#define navu2_codec_resume	NULL
#endif

static int navu2_codec_probe(struct platform_device *devptr)
{
	struct snd_card *card = NULL;
	t_regulator_config regconfig;
	int ret;
	int i;
	FNC_ENTRY

	card = snd_card_new(SNDRV_DEFAULT_IDX1, id, THIS_MODULE, 0);
	if (card == NULL) {
		FNC_EXIT return -ENOMEM;
	}

	snd_card_set_dev(card, &devptr->dev);

	for (i = 0; i < 2; i++) {
		navu2_codec_client[i].s[SNDRV_PCM_STREAM_PLAYBACK] = &navu2_codec_out[i];
		if (i == 0) /* PM0 (stereo DAC) doesn't support capture function */
			navu2_codec_client[i].s[SNDRV_PCM_STREAM_CAPTURE] = NULL;
		else
			navu2_codec_client[i].s[SNDRV_PCM_STREAM_CAPTURE] = &navu2_codec_in[i];

		navu2_codec_client[i].sett = &pcm_sett[i];

		ret = mp200_pcm_new(card, &navu2_codec_client[i], &navu2_codec[i]);
		if (ret)
			goto err;

		ret = mp200_pcm_set_ctrl(navu2_codec_client[i].pcm_regs, &pcm_sett[i]);
		if (ret )
			goto err;
	}

	/* setup VAUDIO */
	regconfig.lp_mode = LOW_POWER_CTRL_BY_PIN;
	pmic_power_regulator_set_config(REGU_VAUDIO, &regconfig);

	ret = navu2_codec_mixer_new(card);
	if (ret)
		goto err;

	snprintf(card->shortname, sizeof(card->shortname),
		 "%s", "navu2-codec");
	snprintf(card->longname, sizeof(card->longname),
		 "%s (%s)", "sound codec", card->mixername);

	if ((ret = snd_card_register(card)) == 0) {
#ifdef AUDIO_MAKING_DEBUG
		printk(KERN_INFO "Starting sound codec. with debug : M%d\n",debug);
#else
		printk(KERN_INFO "Starting sound codec.\n");
#endif
		platform_set_drvdata(devptr, card);
		FNC_EXIT return 0;
	}

 err:
	if (card)
		snd_card_free(card);

	FNC_EXIT return ret;
}

static int navu2_codec_remove(struct platform_device *devptr)
{
	int i;
	FNC_ENTRY

	pmic_event_unsubscribe(EVENT_HSDETI, navu2_hsdet_callback_param);
	for (i = 0; i < 2; i++) {
		mp200_pcm_free(&navu2_codec_client[i]);
	}
	snd_card_free(platform_get_drvdata(devptr));
	platform_set_drvdata(devptr, NULL);

	FNC_EXIT return 0;
}

#define NAVU2_PCM_DRIVER	"navu2_pcm"

static struct platform_device *device;
static struct platform_driver navu2_codec_driver = {
	.probe		= navu2_codec_probe,
	.remove		= __devexit_p(navu2_codec_remove),
#ifdef CONFIG_PM
	.suspend	= navu2_codec_suspend,
	.resume		= navu2_codec_resume,
#endif
	.driver		= {
		.name	= NAVU2_PCM_DRIVER,
	},
};

static int __init navu2_codec_init(void)
{
	int err;

	if ((err = platform_driver_register(&navu2_codec_driver)) < 0)
		return err;

	device = platform_device_register_simple(NAVU2_PCM_DRIVER, -1, NULL, 0);
	if (!IS_ERR(device)) {
		if (platform_get_drvdata(device))
			return 0;
		platform_device_unregister(device);
		err = -ENODEV;
	} else
		err = PTR_ERR(device);
	platform_driver_unregister(&navu2_codec_driver);
	return err;
}

static void __exit navu2_codec_exit(void)
{
	platform_device_unregister(device);
	platform_driver_unregister(&navu2_codec_driver);

}

module_init(navu2_codec_init);
module_exit(navu2_codec_exit);

MODULE_DESCRIPTION("sound codec driver for NAVU2");
MODULE_LICENSE("GPL");

