/* 2010-08-25: File added by Sony Corporation */
/*
 * sound/arm/omap/omap2-audio-aic23.h
 *
 * The Audio Specific Definitions for TLV320 AIC23B Codec
 *
 * Copyright (C) 2007 Texas Instruments, Inc.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * History:
 *  -------
 *  2008-04-03 Anuj Aggarwal - Created
 */

#ifndef __OMAP2_AUDIO_AIC23_H__
#define __OMAP2_AUDIO_AIC23_H__

/****************************************
 *  AUDIO_VOICE
 ****************************************/
/* Codec TLV320AIC23 */
#define AIC23_REG_LEFT_INPUT_VOLUME		0x00
#define AIC23_REG_RIGHT_INPUT_VOLUME		0x01
#define AIC23_REG_LEFT_OUTPUT_VOLUME		0x02
#define AIC23_REG_RIGHT_OUTPUT_VOLUME		0x03
#define AIC23_REG_ANALOG_PATH			0x04
#define AIC23_REG_DIGITAL_PATH			0x05
#define AIC23_REG_POWER_DOWN			0x06
#define AIC23_REG_DIGITAL_INTERFACE_FORMAT	0x07
#define AIC23_REG_SAMPLE_RATE			0x08
#define AIC23_REG_DIGITAL_INTERFACE_ENABLE	0x09
#define AIC23_REG_RESET				0x0f

#define AIC23_NUM_REGS				11

#define AIC23_CHANNEL_LEFT			0x01
#define AIC23_CHANNEL_RIGHT			0x02
#define AIC23_CHANNEL_BOTH			0x03

#define AIC23_SOURCE_LINEIN 			0x00
#define AIC23_SOURCE_MIC			0x01

#define AIC23_MAX_INPUT_VOLUME			0x001f
#define AIC23_DEFAULT_INPUT_VOLUME		0x0017

#define AIC23_MAX_OUTPUT_VOLUME			0x007f
#define AIC23_DEFAULT_OUTPUT_VOLUME		0x0079

/**** BitField Definitions */

/* Left Line Input volume control register */
#define AIC23_REG_INVOL_LRS_ENABLED		0x0100
#define AIC23_REG_INVOL_LIM_MUTED		0x0080
#define AIC23_REG_INVOL_LIM_UNMUTED		0x0000
#define AIC23_REG_INVOL_LIV_DEFAULT		0x0017
#define AIC23_REG_INVOL_LIV_MAX			0x001f
#define AIC23_REG_INVOL_LIV_MIN			0x0000

/* Right Line Input volume control register */
#define AIC23_REG_INVOL_RLS_ENABLED		0x0100
#define AIC23_REG_INVOL_RIM_MUTED		0x0080
#define AIC23_REG_INVOL_RIM_UNMUTED		0x0000
#define AIC23_REG_INVOL_RIV_DEFAULT		0x0017
#define AIC23_REG_INVOL_RIV_MAX			0x001f
#define AIC23_REG_INVOL_RIV_MIN			0x0000

/* Left channel headphone volume control register */
#define AIC23_REG_HPVOL_LRS_ENABLED		0x0100
#define AIC23_REG_HPVOL_LZC_DETECT		0x0080
#define AIC23_REG_HPVOL_LHV_DEFAULT		0x0079
#define AIC23_REG_HPVOL_LHV_MAX			0x007f
#define AIC23_REG_HPVOL_LHV_MIN			0x0000

/* Right channel headphone volume control register */
#define AIC23_REG_HPVOL_RLS_ENABLED		0x0100
#define AIC23_REG_HPVOL_RZC_DETECT		0x0080
#define AIC23_REG_HPVOL_RHV_DEFAULT		0x0079
#define AIC23_REG_HPVOL_RHV_MAX			0x007f
#define AIC23_REG_HPVOL_RHV_MIN			0x0000

/* Analog audio path control register */
#define AIC23_REG_ANAP_SIDETONE_0DB		0x0100
#define AIC23_REG_ANAP_SIDETONE_M6DB		0x0000
#define AIC23_REG_ANAP_SIDETONE_M9DB		0x0040
#define AIC23_REG_ANAP_SIDETONE_M12DB		0x0080
#define AIC23_REG_ANAP_SIDETONE_M18DB		0x00C0
#define AIC23_REG_ANAP_SIDETONE_COUNT		0x0005
#define AIC23_REG_ANAP_STE_DISABLED		0x0000
#define AIC23_REG_ANAP_STE_ENABLED		0x0020
#define AIC23_REG_ANAP_SIDETONE_MASK		0x01c0
#define AIC23_REG_ANAP_DAC_ENABLE		0x0010
#define AIC23_REG_ANAP_BYPASS			0x0008
#define AIC23_REG_ANAP_INPUT_USE_LINEIN		0x0000
#define AIC23_REG_ANAP_INPUT_USE_MIC		0x0004
#define AIC23_REG_ANAP_MIC_MUTE			0x0002
#define AIC23_REG_ANAP_MIC_BOOST_20DB		0x0001

/* Digital audio path control register */
#define AIC23_REG_DIGP_DAC_SOFT_MUTE		0x0008
#define AIC23_REG_DIGP_DEEMPHASIS_32KHZ		0x0002
#define AIC23_REG_DIGP_DEEMPHASIS_44KHZ		0x0004
#define AIC23_REG_DIGP_DEEMPHASIS_48KHZ		0x0006
#define AIC23_REG_DIGP_ADC_HIGH_PASS		0x0001

/* Power Down Control Register */
#define AIC23_REG_PWRDN_NONE			0x0000
#define AIC23_REG_PWRDN_ALL			0x00ff
#define AIC23_REG_PWRDN_DEVICE			0x0080
#define AIC23_REG_PWRDN_CLOCK			0x0040
#define AIC23_REG_PWRDN_OSCILLATORS		0x0020
#define AIC23_REG_PWRDN_OUTPUT			0x0010
#define AIC23_REG_PWRDN_DAC			0x0008
#define AIC23_REG_PWRDN_ADC			0x0004
#define AIC23_REG_PWRDN_MIC			0x0002
#define AIC23_REG_PWRDN_LINEIN			0x0001

/* Digital audio interface register */
#define AIC23_REG_DIGFMT_MASTER			0x0040
#define AIC23_REG_DIGFMT_SLAVE			0x0000
#define AIC23_REG_DIGFMT_DAC_LR_SWAP		0x0020
#define AIC23_REG_DIGFMT_DAC_PHASE_0		0x0000
#define AIC23_REG_DIGFMT_DAC_PHASE_1		0x0010
#define AIC23_REG_DIGFMT_INPUT_16BIT		0x0000
#define AIC23_REG_DIGFMT_INPUT_20BIT		0x0004
#define AIC23_REG_DIGFMT_INPUT_24BIT		0x0008
#define AIC23_REG_DIGFMT_INPUT_32BIT		0x000c
#define AIC23_REG_DIGFMT_FORMAT_DSP		0x0003
#define AIC23_REG_DIGFMT_FORMAT_I2S		0x0002
#define AIC23_REG_DIGFMT_FORMAT_MSB_FIRST_LEFT_ALIGN	0x0001
#define AIC23_REG_DIGFMT_FORMAT_MSB_FIRST_RIGHT_ALIGN	0x0000

/* Sample rate control register */
#define AIC23_REG_SRATE_HALF_OUTPUT_CLOCK	0x0080
#define AIC23_REG_SRATE_HALF_INPUT_CLOCK	0x0040
#define AIC23_REG_SRATE_CLKIN_SHIFT		0x0006
#define AIC23_REG_SRATE_SAMPLE_RATE_0		0x0000
#define AIC23_REG_SRATE_SAMPLE_RATE_1		0x0004
#define AIC23_REG_SRATE_SAMPLE_RATE_2		0x0008
#define AIC23_REG_SRATE_SAMPLE_RATE_3		0x000c
#define AIC23_REG_SRATE_SAMPLE_RATE_4		0x0010
#define AIC23_REG_SRATE_SAMPLE_RATE_5		0x0014
#define AIC23_REG_SRATE_SAMPLE_RATE_6		0x0018
#define AIC23_REG_SRATE_SAMPLE_RATE_7		0x001c
#define AIC23_REG_SRATE_SAMPLE_RATE_8		0x0020
#define AIC23_REG_SRATE_SAMPLE_RATE_9		0x0024
#define AIC23_REG_SRATE_SAMPLE_RATE_10		0x0028
#define AIC23_REG_SRATE_SAMPLE_RATE_11		0x002c
#define AIC23_REG_SRATE_SAMPLE_RATE_12		0x0030
#define AIC23_REG_SRATE_SAMPLE_RATE_13		0x0034
#define AIC23_REG_SRATE_SAMPLE_RATE_14		0x0038
#define AIC23_REG_SRATE_SAMPLE_RATE_15		0x003c
#define AIC23_REG_SRATE_OVERSAMPLING_0		0x0000
#define AIC23_REG_SRATE_OVERSAMPLING_1		0x0002
#define AIC23_REG_SRATE_CLOCK_MODE_NORMAL	0x0000
#define AIC23_REG_SRATE_CLOCK_MODE_USB		0x0001
#define AIC23_REG_SRATE_BOSR_SHIFT		0x0001

/* Digital interface register */
#define AIC23_REG_DIGINT_ENABLE			0x0001

/* Reset Register */
#define AIC23_REG_RST_RESET			0x0000


/************** INTERNAL DETAILS ********************/
#define STEREO_MODE				(0x1)
#define MONO_MODE				(0x2)

#define AUDIO_SAMPLE_DATA_WIDTH_16		16
#define AUDIO_SAMPLE_DATA_WIDTH_24		24

#define CODEC_NAME			"TLV320AIC23B"
#define MIXER_NAME			"TLV320AIC23B Mixer"

#define DIR_OUT				(0)
#define DIR_IN				(1<<7)

#define OUTPUT_VOLUME			(DIR_OUT | (1 << 1))
#define OUTPUT_HEADPHONE		(DIR_OUT | (1 << 2))
#define OUTPUT_LINEOUT			(DIR_OUT | (1 << 3))
#define OUTPUT_SIDETONE			(DIR_OUT | (1 << 4))

#define INPUT_VOLUME			(DIR_IN | (1 << 1))
#define INPUT_LINEIN			(DIR_IN | (1 << 2))
#define INPUT_MIC			(DIR_IN | (1 << 3))

#define DEFAULT_INPUT_DEVICE		INPUT_LINEIN
#define DEFAULT_OUTPUT_DEVICE		OUTPUT_HEADPHONE

#if defined(CONFIG_ARCH_OMAP3430)
#define PLATFORM_NAME			"OMAP3430"
#else
#error "UnSupported platform"
#endif

/* Select the McBSP For Audio */
#define AUDIO_MCBSP			OMAP2_MCBSP_INTERFACE2
#define CODEC_CLOCK			12000000

struct aic23_clk_info {
	struct clk *parent_clk;
	struct clk *aic23_clk;
};

struct aic23_clk_info aic23_clocks = {NULL, NULL};

#define DEFAULT_ANALOG_AUDIO_CONTROL	(AIC23_REG_ANAP_DAC_ENABLE \
					| AIC23_REG_ANAP_MIC_MUTE)

/* To Store the default sample rate */
#define AUDIO_RATE_DEFAULT		8000
static long audio_samplerate = AUDIO_RATE_DEFAULT;

static u8 current_bitspersample = AUDIO_SAMPLE_DATA_WIDTH_16;

static u8 current_stereomode = STEREO_MODE;

static unsigned int aic23_sample_rates[] = {
	8000, 16000, 22050,
	24000, 32000, 44100,
	48000,
};

#define NUMBER_OF_RATES_SUPPORTED 	(sizeof (aic23_sample_rates) / \
					sizeof (unsigned int))

struct aic23_samplerate_reg_info {
	u32 sample_rate;
	u8 control;		/* SR3, SR2, SR1, SR0 and BOSR */
	u8 divider;		/* if 0 CLKIN = MCLK, if 1 CLKIN = MCLK/2 */
};

static const struct aic23_samplerate_reg_info
	 aic23_rate_reg_info[NUMBER_OF_RATES_SUPPORTED] = {
		{8000, 0x06, 0},		/*  8000 */
		{16000, 0x0C, 1},		/* 16000 */
		{22050, 0x11, 1},               /* 22050 */
		{24000, 0x00, 1},		/* 24000 */
		{32000, 0x0C, 0},		/* 32000 */
		{44100, 0x11, 0},		/* 44100 */
		{48000, 0x00, 0},		/* 48000 */
};

static struct snd_pcm_hw_constraint_list aic23_pcm_hw_constraint_list = {
	.count = ARRAY_SIZE(aic23_sample_rates),
	.list = aic23_sample_rates,
	.mask = 0,
};

static struct snd_pcm_hardware aic23_pcm_hardware_playback = {
	.info = (SNDRV_PCM_INFO_INTERLEAVED | SNDRV_PCM_INFO_BLOCK_TRANSFER |
		 SNDRV_PCM_INFO_PAUSE | SNDRV_PCM_INFO_RESUME),
	.formats = (SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S24_LE),
	.rates = (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_16000 |
		  SNDRV_PCM_RATE_22050 | SNDRV_PCM_RATE_32000 |
		  SNDRV_PCM_RATE_44100 | SNDRV_PCM_RATE_48000 |
		  SNDRV_PCM_RATE_KNOT),
	.rate_min = 8000,
	.rate_max = 48000,
	.channels_min = 1,
	.channels_max = 2,
	.buffer_bytes_max = 128 * 1024,
	.period_bytes_min = 32,
	.period_bytes_max = 8 * 1024,
	.periods_min = 16,
	.periods_max = 255,
	.fifo_size = 0,
};

static struct snd_pcm_hardware aic23_pcm_hardware_capture = {
	.info = (SNDRV_PCM_INFO_INTERLEAVED | SNDRV_PCM_INFO_BLOCK_TRANSFER |
		 SNDRV_PCM_INFO_PAUSE | SNDRV_PCM_INFO_RESUME),
	.formats = (SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S24_LE),
	.rates = (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_16000 |
		  SNDRV_PCM_RATE_22050 | SNDRV_PCM_RATE_32000 |
		  SNDRV_PCM_RATE_44100 | SNDRV_PCM_RATE_48000 |
		  SNDRV_PCM_RATE_KNOT),
	.rate_min = 8000,
	.rate_max = 48000,
	.channels_min = 1,
	.channels_max = 2,
	.buffer_bytes_max = 128 * 1024,
	.period_bytes_min = 32,
	.period_bytes_max = 8 * 1024,
	.periods_min = 16,
	.periods_max = 255,
	.fifo_size = 0,
};

#define READ_LEFT_VOLUME(ARG)                    ( (ARG) & 0xFF )
#define READ_RIGHT_VOLUME(ARG)                   ( ((ARG) & 0xFF00)>>8 )
#define WRITE_LEFT_VOLUME(ARG)                   ( (ARG) & 0xFF )
#define WRITE_RIGHT_VOLUME(ARG)                  ( ((ARG) & 0xFF)<<8 )
#define WRITE_LR_VOLUME(ARG)                     ( WRITE_RIGHT_VOLUME(ARG) | \
						   WRITE_LEFT_VOLUME(ARG) )
struct aic23_local_info {
	/* Volume Control */
	u8 headphone_volume_l;	/* Store the headphone volume */
	u8 headphone_volume_r;	/* Store the headphone volume */

	u8 linein_volume_l; 	/* Store the linein record volume */
	u8 linein_volume_r; 	/* Store the linein record volume */

	u16 sidetone_volume; 	/* Store sidetone volume */
	u8 sidetone_enabled; 	/* Sidetone enabled */
	u8 lineout_mute; 		/* Line output muted */

	u8 mic_boost;		/* mic boost enabled */
	u8 mic_muted;		/* mic muted */

	u8 bypass_enabled;	/* Analog bypass enabled/disabled. */
	u8 pcm_enabled;	/* pcm playback enabled/disabled. */
	u8 linein_enabled;	/* line capture enabled/disabled */

	int rec_src; /* current active record source: LineIn / Mic */
};

static struct aic23_local_info aic23_local = {
	/* Volume Control */
	.headphone_volume_l =  AIC23_DEFAULT_OUTPUT_VOLUME,
	.headphone_volume_r =  AIC23_DEFAULT_OUTPUT_VOLUME,

	.linein_volume_l = AIC23_DEFAULT_INPUT_VOLUME,
	.linein_volume_r = AIC23_DEFAULT_INPUT_VOLUME,

	.sidetone_volume = 5,//AIC23_REG_ANAP_SIDETONE_0DB,
	.sidetone_enabled = 0, /* 1: enable, 0: disable */
	.lineout_mute = 1,	//0: unmuted, 1: muted

	.mic_boost = 0,	//0: no boost, 1: yes boost
	.mic_muted = 1,	//0: unmuted, 1: muted

	.bypass_enabled = 0, /* 1: enable, 0: disable */
	.pcm_enabled = 1, /* 1: enable, 0: disable */
	.linein_enabled = 1,  /* 1: enable, 0: disable */

	.rec_src = DEFAULT_INPUT_DEVICE,
};

/* Default input/output source */
static int current_input = DEFAULT_INPUT_DEVICE;
static int current_output = DEFAULT_OUTPUT_DEVICE;

/* Sidetone volumes */
u16 aic23_snd_sidetone[6] = {
	0,	/* Mute */
	AIC23_REG_ANAP_SIDETONE_M18DB,
	AIC23_REG_ANAP_SIDETONE_M12DB,
	AIC23_REG_ANAP_SIDETONE_M9DB,
	AIC23_REG_ANAP_SIDETONE_M6DB,
	AIC23_REG_ANAP_SIDETONE_0DB
};



/* Configured count -
 * Codec is configured when this reaches 1
 * Codec is shutdown when this reaches 0
 */
static char aic23_configured = 0;

struct mcbsp_config {
	u8 mcbsp_clk_src;	/* source it from prcm? */
	u8 srg_clk_src;		/* clks/fclk/clkr/clkx */
	u8 srg_clk_sync;	/* free_running or just running */
	u8 srg_clk_pol;		/* polarity of srg clk */
	u8 tx_clk_pol;          /* TX clock polarity */
	u8 tx_clk_src;		/* internal/external (master based) */
	u8 rx_clk_pol;          /* TX clock polarity */
	u8 rx_clk_src;		/* internal/external (master based) */
	u8 fs_clk_pol;		/* Frame sync polarity */
	u8 tx_polarity;		/* TX polarity */
	u8 rx_polarity;		/* RX polarity */
	u8 rx_ip_clk;           /* Rx input clock */
	u8 tx_ip_clk;           /* Tx input clock */
	omap2_mcbsp_transfer_params tx_params; /* Transmit parameters */
	omap2_mcbsp_transfer_params rx_params; /* Recieve parameters */
};

/* To handle transfer errors.. */
static int tx_err, rx_err;

/******************** MODULES SPECIFIC FUNCTION PROTOTYPES ********************/
static inline int audio_aic23_write(u8 address, u16 data);
static inline u16 audio_aic23_read(u8 address);
static inline int aic23_codec_on(void);
static inline int aic23_codec_off(void);
static int aic23_conf_data_interface(void);
static int aic23_set_samplerate(long sample_rate);
static int aic23_stereomode_set(int mode, int dsp);

static void aic23_unconfigure(void);
static int aic23_configure(void);
static void aic23_mcbsp_dma_cb(u16 ch_status, void *arg);

static int omap_aic23_transfer(int mode, void *buffer_phy,u32 size, void *arg);
static int omap_aic23_transfer_stop(int mode);
static int omap_aic23_transfer_posn(int mode);
static int omap_aic23_transfer_init(int mode);
static int omap_aic23_initialize(void);
static void omap_aic23_shutdown(void);
static int omap_aic23_sidle(u32 level);
static int omap_aic23_probe(void);
static int omap_aic23_mixer_init(struct snd_card *card);
static int omap_aic23_mixer_shutdown(struct snd_card *card);

#ifdef TONE_GEN
void toneGen(void);
#endif


/******************** DATA STRUCTURES USING FUNCTION POINTERS *****************/
static struct mcbsp_config plat_mcbsp_config = {
#if defined(CONFIG_ARCH_OMAP3430)
	.mcbsp_clk_src = OMAP2_MCBSP_FCLKSRC_PRCM,
	/* Driven by AIC23 */
	.srg_clk_src = OMAP2_MCBSP_SRGCLKSRC_CLKX,
	.tx_clk_src = OMAP2_MCBSP_TXFSYNC_EXTERNAL,
	.rx_clk_src = OMAP2_MCBSP_RXFSYNC_EXTERNAL,
	.rx_ip_clk = OMAP2_MCBSP_CLKRXSRC_EXTERNAL,
	.tx_ip_clk = OMAP2_MCBSP_CLKTXSRC_EXTERNAL,

	.srg_clk_sync = OMAP2_MCBSP_SRG_FREERUNNING,

	.tx_polarity = OMAP2_MCBSP_FS_ACTIVE_HIGH,
	.rx_polarity = OMAP2_MCBSP_FS_ACTIVE_HIGH,
	.srg_clk_pol = OMAP2_MCBSP_CLKX_POLARITY_FALLING,
	.tx_clk_pol = OMAP2_MCBSP_CLKX_POLARITY_FALLING,
	.rx_clk_pol = OMAP2_MCBSP_CLKR_POLARITY_FALLING,
	/* we will start with right channel and transmit the MSB of DXR */
	.fs_clk_pol = OMAP2_MCBSP_FS_ACTIVE_HIGH,

	/* DSP */
	.tx_params = {
		       .data_type = OMAP2_MCBSP_WORDLEN_NONE,
		       .skip_alt = OMAP2_MCBSP_SKIP_NONE,
		       .auto_reset = OMAP2_MCBSP_AUTO_XRST,
		       .phase = OMAP2_MCBSP_FRAME_SINGLEPHASE,
		       .data_delay = OMAP2_MCBSP_DATADELAY1,	/* 1 bit delay expected */
		       .reverse_compand = OMAP2_MCBSP_MSBFIRST,	/* Set msb first */
		       .word_length1 = OMAP2_MCBSP_WORDLEN_16,	/* RWDLEN1 */
		       .word_length2 = OMAP2_MCBSP_WORDLEN_16,	/* RWDLEN2 -dnt care*/
		       .frame_length1 = OMAP2_MCBSP_FRAMELEN_2,	/* RFRLEN1 */
		       .frame_length2 = OMAP2_MCBSP_FRAMELEN_2,	/* RFRLEN2 -dnt care*/
		       .justification = OMAP2_MCBSP_RJUST_ZEROMSB,	/* RJUST and fill 0s */
		       .dxena = 0,
		       .dxendly = 0,
		       .callback = aic23_mcbsp_dma_cb,
		       },
	.rx_params = {
		       .data_type = OMAP2_MCBSP_WORDLEN_NONE,
		       .skip_alt = OMAP2_MCBSP_SKIP_NONE,
		       .auto_reset = OMAP2_MCBSP_AUTO_RRST,
		       .phase = OMAP2_MCBSP_FRAME_SINGLEPHASE,
		       .data_delay = OMAP2_MCBSP_DATADELAY1,	/* 1 bit delay expected */
		       .reverse_compand = OMAP2_MCBSP_MSBFIRST,	/* Set msb first */
		       .word_length1 = OMAP2_MCBSP_WORDLEN_16,	/* RWDLEN1 */
		       .word_length2 = OMAP2_MCBSP_WORDLEN_16,	/* RWDLEN2 -dntcare*/
		       .frame_length1 = OMAP2_MCBSP_FRAMELEN_2,	/* RFRLEN1 */
		       .frame_length2 = OMAP2_MCBSP_FRAMELEN_2,	/* RFRLEN2 */
		       .justification = OMAP2_MCBSP_RJUST_ZEROMSB,	/* RJUST and fill 0s */
		       .dxena = 0,
		       .dxendly = 0,
		       .callback = aic23_mcbsp_dma_cb,
		       },
#else
#error "Unsupported Platform"
#endif
};


static int aic23_default_samplerate(void)
{
	return AUDIO_RATE_DEFAULT;
}


/* To store characteristic info regarding the codec for the audio driver */
static struct omap_alsa_codec omap_aic23_codec = {
	.name = CODEC_NAME,

	.pcm_hw_constraint_list = &aic23_pcm_hw_constraint_list,
	.pcm_hardware_playback = &aic23_pcm_hardware_playback,
	.pcm_hardware_capture = &aic23_pcm_hardware_capture,

	.codec_probe = omap_aic23_probe,
	.codec_init = omap_aic23_initialize,
	.codec_shutdown = omap_aic23_shutdown,
	.codec_sidle = omap_aic23_sidle,

	.codec_set_samplerate = aic23_set_samplerate,
#if defined (CONFIG_SND_OMAP3EVM)
	.codec_set_stereomode = aic23_stereomode_set,
#endif
	.codec_default_samplerate = aic23_default_samplerate,

	.codec_transfer_init = omap_aic23_transfer_init,
	.codec_transfer_start = omap_aic23_transfer,
	.codec_transfer_stop = omap_aic23_transfer_stop,
	.codec_transfer_posn = omap_aic23_transfer_posn,

	.mixer_init = omap_aic23_mixer_init,
	.mixer_shutdown = omap_aic23_mixer_shutdown,
};

#endif	/* End of __OMAP2_AUDIO_AIC23_H__ */
